@@ -1,30 +1,36 @@
 """Advantage Air climate integration."""
-
 from datetime import timedelta
 import logging
 
 from advantage_air import ApiError, advantage_air
 
+from homeassistant.config_entries import ConfigEntry
 from homeassistant.const import CONF_IP_ADDRESS, CONF_PORT, Platform
+from homeassistant.core import HomeAssistant
 from homeassistant.helpers.aiohttp_client import async_get_clientsession
+from homeassistant.helpers.debounce import Debouncer
 from homeassistant.helpers.update_coordinator import DataUpdateCoordinator, UpdateFailed
 
 from .const import ADVANTAGE_AIR_RETRY, DOMAIN
+from .models import AdvantageAirData
 
 ADVANTAGE_AIR_SYNC_INTERVAL = 15
 PLATFORMS = [
     Platform.BINARY_SENSOR,
     Platform.CLIMATE,
     Platform.COVER,
+    Platform.LIGHT,
     Platform.SELECT,
     Platform.SENSOR,
     Platform.SWITCH,
+    Platform.UPDATE,
 ]
 
 _LOGGER = logging.getLogger(__name__)
+REQUEST_REFRESH_DELAY = 0.5
 
 
-async def async_setup_entry(hass, entry):
+async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
     """Set up Advantage Air config."""
     ip_address = entry.data[CONF_IP_ADDRESS]
     port = entry.data[CONF_PORT]
@@ -47,29 +53,22 @@ async def async_setup_entry(hass, entry):
         name="Advantage Air",
         update_method=async_get,
         update_interval=timedelta(seconds=ADVANTAGE_AIR_SYNC_INTERVAL),
+        request_refresh_debouncer=Debouncer(
+            hass, _LOGGER, cooldown=REQUEST_REFRESH_DELAY, immediate=False
+        ),
     )
 
-    async def async_change(change):
-        try:
-            if await api.async_change(change):
-                await coordinator.async_refresh()
-        except ApiError as err:
-            _LOGGER.warning(err)
-
     await coordinator.async_config_entry_first_refresh()
 
     hass.data.setdefault(DOMAIN, {})
-    hass.data[DOMAIN][entry.entry_id] = {
-        "coordinator": coordinator,
-        "async_change": async_change,
-    }
+    hass.data[DOMAIN][entry.entry_id] = AdvantageAirData(coordinator, api)
 
-    hass.config_entries.async_setup_platforms(entry, PLATFORMS)
+    await hass.config_entries.async_forward_entry_setups(entry, PLATFORMS)
 
     return True
 
 
-async def async_unload_entry(hass, entry):
+async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
     """Unload Advantage Air Config."""
     unload_ok = await hass.config_entries.async_unload_platforms(entry, PLATFORMS)
 
