@@ -2,63 +2,25 @@
 from __future__ import annotations
 
 from datetime import timedelta
-import logging
-from typing import Any, Final
+from typing import Any
 
-from AIOAladdinConnect import AladdinConnectClient
-import voluptuous as vol
+from AIOAladdinConnect import AladdinConnectClient, session_manager
 
-from homeassistant.components.cover import (
-    PLATFORM_SCHEMA as BASE_PLATFORM_SCHEMA,
-    CoverDeviceClass,
-    CoverEntity,
-)
-from homeassistant.config_entries import SOURCE_IMPORT, ConfigEntry
-from homeassistant.const import (
-    CONF_PASSWORD,
-    CONF_USERNAME,
-    STATE_CLOSED,
-    STATE_CLOSING,
-    STATE_OPENING,
-)
+from homeassistant.components.cover import CoverDeviceClass, CoverEntity
+from homeassistant.config_entries import ConfigEntry
+from homeassistant.const import STATE_CLOSED, STATE_CLOSING, STATE_OPENING
 from homeassistant.core import HomeAssistant
-from homeassistant.exceptions import PlatformNotReady
-import homeassistant.helpers.config_validation as cv
-from homeassistant.helpers.entity import DeviceInfo
+from homeassistant.exceptions import HomeAssistantError, PlatformNotReady
+import homeassistant.helpers.device_registry as dr
+from homeassistant.helpers.device_registry import DeviceInfo
 from homeassistant.helpers.entity_platform import AddEntitiesCallback
-from homeassistant.helpers.typing import ConfigType, DiscoveryInfoType
 
 from .const import DOMAIN, STATES_MAP, SUPPORTED_FEATURES
 from .model import DoorDevice
 
-_LOGGER: Final = logging.getLogger(__name__)
-
-PLATFORM_SCHEMA: Final = BASE_PLATFORM_SCHEMA.extend(
-    {vol.Required(CONF_USERNAME): cv.string, vol.Required(CONF_PASSWORD): cv.string}
-)
 SCAN_INTERVAL = timedelta(seconds=300)
 
 
-async def async_setup_platform(
-    hass: HomeAssistant,
-    config: ConfigType,
-    async_add_entities: AddEntitiesCallback,
-    discovery_info: DiscoveryInfoType | None = None,
-) -> None:
-    """Set up Aladdin Connect devices yaml depreciated."""
-    _LOGGER.warning(
-        "Configuring Aladdin Connect through yaml is deprecated"
-        "Please remove it from your configuration as it has already been imported to a config entry"
-    )
-    await hass.async_create_task(
-        hass.config_entries.flow.async_init(
-            DOMAIN,
-            context={"source": SOURCE_IMPORT},
-            data=config,
-        )
-    )
-
-
 async def async_setup_entry(
     hass: HomeAssistant,
     config_entry: ConfigEntry,
@@ -72,6 +34,34 @@ async def async_setup_entry(
     async_add_entities(
         (AladdinDevice(acc, door, config_entry) for door in doors),
     )
+    remove_stale_devices(hass, config_entry, doors)
+
+
+def remove_stale_devices(
+    hass: HomeAssistant, config_entry: ConfigEntry, devices: list[dict]
+) -> None:
+    """Remove stale devices from device registry."""
+    device_registry = dr.async_get(hass)
+    device_entries = dr.async_entries_for_config_entry(
+        device_registry, config_entry.entry_id
+    )
+    all_device_ids = {f"{door['device_id']}-{door['door_number']}" for door in devices}
+
+    for device_entry in device_entries:
+        device_id: str | None = None
+
+        for identifier in device_entry.identifiers:
+            if identifier[0] == DOMAIN:
+                device_id = identifier[1]
+                break
+
+        if device_id is None or device_id not in all_device_ids:
+            # If device_id is None an invalid device entry was found for this config entry.
+            # If the device_id is not in existing device ids it's a stale device entry.
+            # Remove config entry from this device entry in either case.
+            device_registry.async_update_device(
+                device_entry.id, remove_config_entry_id=config_entry.entry_id
+            )
 
 
 class AladdinDevice(CoverEntity):
@@ -79,56 +69,57 @@ class AladdinDevice(CoverEntity):
 
     _attr_device_class = CoverDeviceClass.GARAGE
     _attr_supported_features = SUPPORTED_FEATURES
+    _attr_has_entity_name = True
+    _attr_name = None
 
     def __init__(
         self, acc: AladdinConnectClient, device: DoorDevice, entry: ConfigEntry
     ) -> None:
         """Initialize the Aladdin Connect cover."""
         self._acc = acc
-
         self._device_id = device["device_id"]
         self._number = device["door_number"]
-        self._name = device["name"]
         self._serial = device["serial"]
-        self._model = device["model"]
-        self._attr_unique_id = f"{self._device_id}-{self._number}"
-        self._attr_has_entity_name = True
 
-    @property
-    def device_info(self) -> DeviceInfo | None:
-        """Device information for Aladdin Connect cover."""
-        return DeviceInfo(
+        self._attr_device_info = DeviceInfo(
             identifiers={(DOMAIN, f"{self._device_id}-{self._number}")},
-            name=self._name,
+            name=device["name"],
             manufacturer="Overhead Door",
-            model=self._model,
+            model=device["model"],
         )
+        self._attr_unique_id = f"{self._device_id}-{self._number}"
 
     async def async_added_to_hass(self) -> None:
         """Connect Aladdin Connect to the cloud."""
 
-        async def update_callback() -> None:
-            """Schedule a state update."""
-            self.async_write_ha_state()
-
-        self._acc.register_callback(update_callback, self._serial, self._number)
+        self._acc.register_callback(
+            self.async_write_ha_state, self._serial, self._number
+        )
         await self._acc.get_doors(self._serial)
 
     async def async_will_remove_from_hass(self) -> None:
         """Close Aladdin Connect before removing."""
+        self._acc.unregister_callback(self._serial, self._number)
         await self._acc.close()
 
     async def async_close_cover(self, **kwargs: Any) -> None:
         """Issue close command to cover."""
-        await self._acc.close_door(self._device_id, self._number)
+        if not await self._acc.close_door(self._device_id, self._number):
+            raise HomeAssistantError("Aladdin Connect API failed to close the cover")
 
     async def async_open_cover(self, **kwargs: Any) -> None:
         """Issue open command to cover."""
-        await self._acc.open_door(self._device_id, self._number)
+        if not await self._acc.open_door(self._device_id, self._number):
+            raise HomeAssistantError("Aladdin Connect API failed to open the cover")
 
     async def async_update(self) -> None:
         """Update status of cover."""
-        await self._acc.get_doors(self._serial)
+        try:
+            await self._acc.get_doors(self._serial)
+            self._attr_available = True
+
+        except (session_manager.ConnectionError, session_manager.InvalidPasswordError):
+            self._attr_available = False
 
     @property
     def is_closed(self) -> bool | None:
