@@ -1,4 +1,4 @@
-"""Support for functionality to interact with Android TV/Fire TV devices."""
+"""Support for functionality to interact with Android/Fire TV devices."""
 from __future__ import annotations
 
 from collections.abc import Mapping
@@ -6,6 +6,13 @@ import os
 from typing import Any
 
 from adb_shell.auth.keygen import keygen
+from adb_shell.exceptions import (
+    AdbTimeoutError,
+    InvalidChecksumError,
+    InvalidCommandError,
+    InvalidResponseError,
+    TcpTimeoutException,
+)
 from androidtv.adb_manager.adb_manager_sync import ADBPythonSync, PythonRSASigner
 from androidtv.setup_async import (
     AndroidTVAsync,
@@ -21,7 +28,7 @@ from homeassistant.const import (
     EVENT_HOMEASSISTANT_STOP,
     Platform,
 )
-from homeassistant.core import HomeAssistant
+from homeassistant.core import Event, HomeAssistant
 from homeassistant.exceptions import ConfigEntryNotReady
 from homeassistant.helpers.device_registry import format_mac
 from homeassistant.helpers.dispatcher import async_dispatcher_send
@@ -43,6 +50,18 @@ from .const import (
     SIGNAL_CONFIG_ENTITY,
 )
 
+ADB_PYTHON_EXCEPTIONS: tuple = (
+    AdbTimeoutError,
+    BrokenPipeError,
+    ConnectionResetError,
+    ValueError,
+    InvalidChecksumError,
+    InvalidCommandError,
+    InvalidResponseError,
+    TcpTimeoutException,
+)
+ADB_TCP_EXCEPTIONS: tuple = (ConnectionResetError, RuntimeError)
+
 PLATFORMS = [Platform.MEDIA_PLAYER]
 RELOAD_OPTIONS = [CONF_STATE_DETECTION_RULES]
 
@@ -116,11 +135,11 @@ async def async_connect_androidtv(
     if not aftv.available:
         # Determine the name that will be used for the device in the log
         if config[CONF_DEVICE_CLASS] == DEVICE_ANDROIDTV:
-            device_name = "Android TV device"
+            device_name = "Android device"
         elif config[CONF_DEVICE_CLASS] == DEVICE_FIRETV:
             device_name = "Fire TV device"
         else:
-            device_name = "Android TV / Fire TV device"
+            device_name = "Android / Fire TV device"
 
         error_message = f"Could not connect to {device_name} at {address} {adb_log}"
         return None, error_message
@@ -129,17 +148,26 @@ async def async_connect_androidtv(
 
 
 async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
-    """Set up Android TV platform."""
+    """Set up Android Debug Bridge platform."""
 
     state_det_rules = entry.options.get(CONF_STATE_DETECTION_RULES)
-    aftv, error_message = await async_connect_androidtv(
-        hass, entry.data, state_detection_rules=state_det_rules
-    )
+    if CONF_ADB_SERVER_IP not in entry.data:
+        exceptions = ADB_PYTHON_EXCEPTIONS
+    else:
+        exceptions = ADB_TCP_EXCEPTIONS
+
+    try:
+        aftv, error_message = await async_connect_androidtv(
+            hass, entry.data, state_detection_rules=state_det_rules
+        )
+    except exceptions as exc:
+        raise ConfigEntryNotReady(exc) from exc
+
     if not aftv:
         raise ConfigEntryNotReady(error_message)
 
-    async def async_close_connection(event):
-        """Close Android TV connection on HA Stop."""
+    async def async_close_connection(event: Event) -> None:
+        """Close Android Debug Bridge connection on HA Stop."""
         await aftv.adb_close()
 
     entry.async_on_unload(
