@@ -1,24 +1,33 @@
 """Rest API for Home Assistant."""
 import asyncio
+from asyncio import shield, timeout
+from functools import lru_cache
 from http import HTTPStatus
-import json
 import logging
+from typing import Any
 
 from aiohttp import web
 from aiohttp.web_exceptions import HTTPBadRequest
-import async_timeout
 import voluptuous as vol
 
+from homeassistant.auth.models import User
 from homeassistant.auth.permissions.const import POLICY_READ
 from homeassistant.bootstrap import DATA_LOGGING
-from homeassistant.components.http import HomeAssistantView
+from homeassistant.components.http import (
+    KEY_HASS,
+    KEY_HASS_USER,
+    HomeAssistantView,
+    require_admin,
+)
 from homeassistant.const import (
+    CONTENT_TYPE_JSON,
     EVENT_HOMEASSISTANT_STOP,
-    EVENT_TIME_CHANGED,
+    EVENT_STATE_CHANGED,
     MATCH_ALL,
     URL_API,
     URL_API_COMPONENTS,
     URL_API_CONFIG,
+    URL_API_CORE_STATE,
     URL_API_ERROR_LOG,
     URL_API_EVENTS,
     URL_API_SERVICES,
@@ -27,12 +36,20 @@ from homeassistant.const import (
     URL_API_TEMPLATE,
 )
 import homeassistant.core as ha
-from homeassistant.core import HomeAssistant
-from homeassistant.exceptions import ServiceNotFound, TemplateError, Unauthorized
-from homeassistant.helpers import template
-from homeassistant.helpers.json import JSONEncoder
+from homeassistant.core import Event, HomeAssistant
+from homeassistant.exceptions import (
+    InvalidEntityFormatError,
+    InvalidStateError,
+    ServiceNotFound,
+    TemplateError,
+    Unauthorized,
+)
+from homeassistant.helpers import config_validation as cv, template
+from homeassistant.helpers.event import EventStateChangedData
+from homeassistant.helpers.json import json_dumps, json_fragment
 from homeassistant.helpers.service import async_get_all_descriptions
-from homeassistant.helpers.typing import ConfigType
+from homeassistant.helpers.typing import ConfigType, EventType
+from homeassistant.util.json import json_loads
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -48,11 +65,15 @@ ATTR_VERSION = "version"
 DOMAIN = "api"
 STREAM_PING_PAYLOAD = "ping"
 STREAM_PING_INTERVAL = 50  # seconds
+SERVICE_WAIT_TIMEOUT = 10
+
+CONFIG_SCHEMA = cv.empty_config_schema(DOMAIN)
 
 
 async def async_setup(hass: HomeAssistant, config: ConfigType) -> bool:
     """Register the API with the HTTP interface."""
     hass.http.register_view(APIStatusView)
+    hass.http.register_view(APICoreStateView)
     hass.http.register_view(APIEventStream)
     hass.http.register_view(APIConfigView)
     hass.http.register_view(APIStatesView)
@@ -77,34 +98,48 @@ class APIStatusView(HomeAssistantView):
     name = "api:status"
 
     @ha.callback
-    def get(self, request):
+    def get(self, request: web.Request) -> web.Response:
         """Retrieve if API is running."""
         return self.json_message("API running.")
 
 
+class APICoreStateView(HomeAssistantView):
+    """View to handle core state requests."""
+
+    url = URL_API_CORE_STATE
+    name = "api:core:state"
+
+    @ha.callback
+    def get(self, request: web.Request) -> web.Response:
+        """Retrieve the current core state.
+
+        This API is intended to be a fast and lightweight way to check if the
+        Home Assistant core is running. Its primary use case is for supervisor
+        to check if Home Assistant is running.
+        """
+        hass: HomeAssistant = request.app[KEY_HASS]
+        return self.json({"state": hass.state.value})
+
+
 class APIEventStream(HomeAssistantView):
     """View to handle EventStream requests."""
 
     url = URL_API_STREAM
     name = "api:stream"
 
-    async def get(self, request):
+    @require_admin
+    async def get(self, request: web.Request) -> web.StreamResponse:
         """Provide a streaming interface for the event bus."""
-        # pylint: disable=no-self-use
-        if not request["hass_user"].is_admin:
-            raise Unauthorized()
-        hass = request.app["hass"]
+        hass: HomeAssistant = request.app[KEY_HASS]
         stop_obj = object()
-        to_write = asyncio.Queue()
+        to_write: asyncio.Queue[object | str] = asyncio.Queue()
 
-        if restrict := request.query.get("restrict"):
-            restrict = restrict.split(",") + [EVENT_HOMEASSISTANT_STOP]
+        restrict: list[str] | None = None
+        if restrict_str := request.query.get("restrict"):
+            restrict = restrict_str.split(",") + [EVENT_HOMEASSISTANT_STOP]
 
-        async def forward_events(event):
+        async def forward_events(event: Event) -> None:
             """Forward events to the open request."""
-            if event.event_type == EVENT_TIME_CHANGED:
-                return
-
             if restrict and event.event_type not in restrict:
                 return
 
@@ -113,7 +148,7 @@ class APIEventStream(HomeAssistantView):
             if event.event_type == EVENT_HOMEASSISTANT_STOP:
                 data = stop_obj
             else:
-                data = json.dumps(event, cls=JSONEncoder)
+                data = json_dumps(event)
 
             await to_write.put(data)
 
@@ -131,7 +166,7 @@ class APIEventStream(HomeAssistantView):
 
             while True:
                 try:
-                    async with async_timeout.timeout(STREAM_PING_INTERVAL):
+                    async with timeout(STREAM_PING_INTERVAL):
                         payload = await to_write.get()
 
                     if payload is stop_obj:
@@ -160,9 +195,10 @@ class APIConfigView(HomeAssistantView):
     name = "api:config"
 
     @ha.callback
-    def get(self, request):
+    def get(self, request: web.Request) -> web.Response:
         """Get current configuration."""
-        return self.json(request.app["hass"].config.as_dict())
+        hass: HomeAssistant = request.app[KEY_HASS]
+        return self.json(hass.config.as_dict())
 
 
 class APIStatesView(HomeAssistantView):
@@ -172,16 +208,26 @@ class APIStatesView(HomeAssistantView):
     name = "api:states"
 
     @ha.callback
-    def get(self, request):
+    def get(self, request: web.Request) -> web.Response:
         """Get current states."""
-        user = request["hass_user"]
-        entity_perm = user.permissions.check_entity
-        states = [
-            state
-            for state in request.app["hass"].states.async_all()
-            if entity_perm(state.entity_id, "read")
-        ]
-        return self.json(states)
+        user: User = request[KEY_HASS_USER]
+        hass: HomeAssistant = request.app[KEY_HASS]
+        if user.is_admin:
+            states = (state.as_dict_json for state in hass.states.async_all())
+        else:
+            entity_perm = user.permissions.check_entity
+            states = (
+                state.as_dict_json
+                for state in hass.states.async_all()
+                if entity_perm(state.entity_id, "read")
+            )
+        response = web.Response(
+            body=b"[" + b",".join(states) + b"]",
+            content_type=CONTENT_TYPE_JSON,
+            zlib_executor_size=32768,
+        )
+        response.enable_compression()
+        return response
 
 
 class APIEntityStateView(HomeAssistantView):
@@ -191,21 +237,26 @@ class APIEntityStateView(HomeAssistantView):
     name = "api:entity-state"
 
     @ha.callback
-    def get(self, request, entity_id):
+    def get(self, request: web.Request, entity_id: str) -> web.Response:
         """Retrieve state of entity."""
-        user = request["hass_user"]
+        user: User = request[KEY_HASS_USER]
+        hass: HomeAssistant = request.app[KEY_HASS]
         if not user.permissions.check_entity(entity_id, POLICY_READ):
             raise Unauthorized(entity_id=entity_id)
 
-        if state := request.app["hass"].states.get(entity_id):
-            return self.json(state)
+        if state := hass.states.get(entity_id):
+            return web.Response(
+                body=state.as_dict_json,
+                content_type=CONTENT_TYPE_JSON,
+            )
         return self.json_message("Entity not found.", HTTPStatus.NOT_FOUND)
 
-    async def post(self, request, entity_id):
+    async def post(self, request: web.Request, entity_id: str) -> web.Response:
         """Update state of entity."""
-        if not request["hass_user"].is_admin:
+        user: User = request[KEY_HASS_USER]
+        if not user.is_admin:
             raise Unauthorized(entity_id=entity_id)
-        hass = request.app["hass"]
+        hass: HomeAssistant = request.app[KEY_HASS]
         try:
             data = await request.json()
         except ValueError:
@@ -220,24 +271,33 @@ class APIEntityStateView(HomeAssistantView):
         is_new_state = hass.states.get(entity_id) is None
 
         # Write state
-        hass.states.async_set(
-            entity_id, new_state, attributes, force_update, self.context(request)
-        )
+        try:
+            hass.states.async_set(
+                entity_id, new_state, attributes, force_update, self.context(request)
+            )
+        except InvalidEntityFormatError:
+            return self.json_message(
+                "Invalid entity ID specified.", HTTPStatus.BAD_REQUEST
+            )
+        except InvalidStateError:
+            return self.json_message("Invalid state specified.", HTTPStatus.BAD_REQUEST)
 
         # Read the state back for our response
         status_code = HTTPStatus.CREATED if is_new_state else HTTPStatus.OK
-        resp = self.json(hass.states.get(entity_id), status_code)
+        assert (state := hass.states.get(entity_id))
+        resp = self.json(state.as_dict(), status_code)
 
         resp.headers.add("Location", f"/api/states/{entity_id}")
 
         return resp
 
     @ha.callback
-    def delete(self, request, entity_id):
+    def delete(self, request: web.Request, entity_id: str) -> web.Response:
         """Remove entity."""
-        if not request["hass_user"].is_admin:
+        if not request[KEY_HASS_USER].is_admin:
             raise Unauthorized(entity_id=entity_id)
-        if request.app["hass"].states.async_remove(entity_id):
+        hass: HomeAssistant = request.app[KEY_HASS]
+        if hass.states.async_remove(entity_id):
             return self.json_message("Entity removed.")
         return self.json_message("Entity not found.", HTTPStatus.NOT_FOUND)
 
@@ -249,9 +309,10 @@ class APIEventListenersView(HomeAssistantView):
     name = "api:event-listeners"
 
     @ha.callback
-    def get(self, request):
+    def get(self, request: web.Request) -> web.Response:
         """Get event listeners."""
-        return self.json(async_events_json(request.app["hass"]))
+        hass: HomeAssistant = request.app[KEY_HASS]
+        return self.json(async_events_json(hass))
 
 
 class APIEventView(HomeAssistantView):
@@ -260,13 +321,12 @@ class APIEventView(HomeAssistantView):
     url = "/api/events/{event_type}"
     name = "api:event"
 
-    async def post(self, request, event_type):
+    @require_admin
+    async def post(self, request: web.Request, event_type: str) -> web.Response:
         """Fire events."""
-        if not request["hass_user"].is_admin:
-            raise Unauthorized()
         body = await request.text()
         try:
-            event_data = json.loads(body) if body else None
+            event_data: Any = json_loads(body) if body else None
         except ValueError:
             return self.json_message(
                 "Event data should be valid JSON.", HTTPStatus.BAD_REQUEST
@@ -279,14 +339,15 @@ class APIEventView(HomeAssistantView):
 
         # Special case handling for event STATE_CHANGED
         # We will try to convert state dicts back to State objects
-        if event_type == ha.EVENT_STATE_CHANGED and event_data:
+        if event_type == EVENT_STATE_CHANGED and event_data:
             for key in ("old_state", "new_state"):
-                state = ha.State.from_dict(event_data.get(key))
+                state = ha.State.from_dict(event_data[key])
 
                 if state:
                     event_data[key] = state
 
-        request.app["hass"].bus.async_fire(
+        hass: HomeAssistant = request.app[KEY_HASS]
+        hass.bus.async_fire(
             event_type, event_data, ha.EventOrigin.remote, self.context(request)
         )
 
@@ -299,9 +360,10 @@ class APIServicesView(HomeAssistantView):
     url = URL_API_SERVICES
     name = "api:services"
 
-    async def get(self, request):
+    async def get(self, request: web.Request) -> web.Response:
         """Get registered services."""
-        services = await async_services_json(request.app["hass"])
+        hass: HomeAssistant = request.app[KEY_HASS]
+        services = await async_services_json(hass)
         return self.json(services)
 
 
@@ -311,34 +373,53 @@ class APIDomainServicesView(HomeAssistantView):
     url = "/api/services/{domain}/{service}"
     name = "api:domain-services"
 
-    async def post(self, request, domain, service):
+    async def post(
+        self, request: web.Request, domain: str, service: str
+    ) -> web.Response:
         """Call a service.
 
         Returns a list of changed states.
         """
-        hass: ha.HomeAssistant = request.app["hass"]
+        hass: HomeAssistant = request.app[KEY_HASS]
         body = await request.text()
         try:
-            data = json.loads(body) if body else None
+            data = json_loads(body) if body else None
         except ValueError:
             return self.json_message(
                 "Data should be valid JSON.", HTTPStatus.BAD_REQUEST
             )
 
         context = self.context(request)
+        changed_states: list[json_fragment] = []
+
+        @ha.callback
+        def _async_save_changed_entities(
+            event: EventType[EventStateChangedData],
+        ) -> None:
+            if event.context == context and (state := event.data["new_state"]):
+                changed_states.append(state.json_fragment)
+
+        cancel_listen = hass.bus.async_listen(
+            EVENT_STATE_CHANGED,
+            _async_save_changed_entities,  # type: ignore[arg-type]
+            run_immediately=True,
+        )
 
         try:
-            await hass.services.async_call(
-                domain, service, data, blocking=True, context=context
+            # shield the service call from cancellation on connection drop
+            await shield(
+                hass.services.async_call(
+                    domain,
+                    service,
+                    data,  # type: ignore[arg-type]
+                    blocking=True,
+                    context=context,
+                )
             )
         except (vol.Invalid, ServiceNotFound) as ex:
             raise HTTPBadRequest() from ex
-
-        changed_states = []
-
-        for state in hass.states.async_all():
-            if state.context is context:
-                changed_states.append(state)
+        finally:
+            cancel_listen()
 
         return self.json(changed_states)
 
@@ -350,9 +431,16 @@ class APIComponentsView(HomeAssistantView):
     name = "api:components"
 
     @ha.callback
-    def get(self, request):
+    def get(self, request: web.Request) -> web.Response:
         """Get current loaded components."""
-        return self.json(request.app["hass"].config.components)
+        hass: HomeAssistant = request.app[KEY_HASS]
+        return self.json(list(hass.config.components))
+
+
+@lru_cache
+def _cached_template(template_str: str, hass: HomeAssistant) -> template.Template:
+    """Return a cached template."""
+    return template.Template(template_str, hass)
 
 
 class APITemplateView(HomeAssistantView):
@@ -361,14 +449,13 @@ class APITemplateView(HomeAssistantView):
     url = URL_API_TEMPLATE
     name = "api:template"
 
-    async def post(self, request):
+    @require_admin
+    async def post(self, request: web.Request) -> web.Response:
         """Render a template."""
-        if not request["hass_user"].is_admin:
-            raise Unauthorized()
         try:
             data = await request.json()
-            tpl = template.Template(data["template"], request.app["hass"])
-            return tpl.async_render(variables=data.get("variables"), parse_result=False)
+            tpl = _cached_template(data["template"], request.app[KEY_HASS])
+            return tpl.async_render(variables=data.get("variables"), parse_result=False)  # type: ignore[no-any-return]
         except (ValueError, TemplateError) as ex:
             return self.json_message(
                 f"Error rendering template: {ex}", HTTPStatus.BAD_REQUEST
@@ -381,22 +468,21 @@ class APIErrorLog(HomeAssistantView):
     url = URL_API_ERROR_LOG
     name = "api:error_log"
 
-    async def get(self, request):
+    @require_admin
+    async def get(self, request: web.Request) -> web.FileResponse:
         """Retrieve API error log."""
-        # pylint: disable=no-self-use
-        if not request["hass_user"].is_admin:
-            raise Unauthorized()
-        return web.FileResponse(request.app["hass"].data[DATA_LOGGING])
+        hass: HomeAssistant = request.app[KEY_HASS]
+        return web.FileResponse(hass.data[DATA_LOGGING])
 
 
-async def async_services_json(hass):
+async def async_services_json(hass: HomeAssistant) -> list[dict[str, Any]]:
     """Generate services data to JSONify."""
     descriptions = await async_get_all_descriptions(hass)
     return [{"domain": key, "services": value} for key, value in descriptions.items()]
 
 
 @ha.callback
-def async_events_json(hass):
+def async_events_json(hass: HomeAssistant) -> list[dict[str, Any]]:
     """Generate event data to JSONify."""
     return [
         {"event": key, "listener_count": value}
