@@ -13,7 +13,7 @@ from homeassistant.const import (
     CONF_TYPE,
 )
 from homeassistant.core import CALLBACK_TYPE, Event, HassJob, HomeAssistant, callback
-from homeassistant.helpers import config_validation as cv, entity_registry
+from homeassistant.helpers import config_validation as cv, entity_registry as er
 from homeassistant.helpers.trigger import TriggerActionType, TriggerInfo
 from homeassistant.helpers.typing import ConfigType
 
@@ -22,7 +22,7 @@ from .const import DOMAIN, EVENT_TURN_ON
 TRIGGER_TYPES = {"turn_on"}
 TRIGGER_SCHEMA = DEVICE_TRIGGER_BASE_SCHEMA.extend(
     {
-        vol.Required(CONF_ENTITY_ID): cv.entity_id,
+        vol.Required(CONF_ENTITY_ID): cv.entity_id_or_uuid,
         vol.Required(CONF_TYPE): vol.In(TRIGGER_TYPES),
     }
 )
@@ -32,18 +32,18 @@ async def async_get_triggers(
     hass: HomeAssistant, device_id: str
 ) -> list[dict[str, str]]:
     """List device triggers for Arcam FMJ Receiver control devices."""
-    registry = entity_registry.async_get(hass)
+    registry = er.async_get(hass)
     triggers = []
 
     # Get all the integrations entities for this device
-    for entry in entity_registry.async_entries_for_device(registry, device_id):
+    for entry in er.async_entries_for_device(registry, device_id):
         if entry.domain == "media_player":
             triggers.append(
                 {
                     CONF_PLATFORM: "device",
                     CONF_DEVICE_ID: device_id,
                     CONF_DOMAIN: DOMAIN,
-                    CONF_ENTITY_ID: entry.entity_id,
+                    CONF_ENTITY_ID: entry.id,
                     CONF_TYPE: "turn_on",
                 }
             )
@@ -62,7 +62,8 @@ async def async_attach_trigger(
     job = HassJob(action)
 
     if config[CONF_TYPE] == "turn_on":
-        entity_id = config[CONF_ENTITY_ID]
+        registry = er.async_get(hass)
+        entity_id = er.async_resolve_entity_id(registry, config[ATTR_ENTITY_ID])
 
         @callback
         def _handle_event(event: Event) -> None:
@@ -71,9 +72,10 @@ async def async_attach_trigger(
                     job,
                     {
                         "trigger": {
-                            **trigger_data,  # type: ignore[arg-type]  # https://github.com/python/mypy/issues/9117
+                            **trigger_data,
                             **config,
                             "description": f"{DOMAIN} - {entity_id}",
+                            "entity_id": entity_id,
                         }
                     },
                     event.context,
