@@ -2,7 +2,6 @@
 from __future__ import annotations
 
 from dataclasses import dataclass
-from numbers import Real
 
 from homeassistant.components.sensor import (
     SensorDeviceClass,
@@ -12,152 +11,168 @@ from homeassistant.components.sensor import (
 )
 from homeassistant.config_entries import ConfigEntry
 from homeassistant.const import (
-    DATA_GIGABYTES,
-    DATA_RATE_MEGABITS_PER_SECOND,
-    TEMP_CELSIUS,
+    EntityCategory,
+    UnitOfDataRate,
+    UnitOfInformation,
+    UnitOfTemperature,
 )
 from homeassistant.core import HomeAssistant
-from homeassistant.helpers.entity import EntityCategory
+from homeassistant.helpers.entity_platform import AddEntitiesCallback
 from homeassistant.helpers.update_coordinator import (
     CoordinatorEntity,
     DataUpdateCoordinator,
 )
+from homeassistant.util import slugify
 
 from .const import (
     DATA_ASUSWRT,
     DOMAIN,
+    KEY_COORDINATOR,
+    KEY_SENSORS,
     SENSORS_BYTES,
     SENSORS_CONNECTED_DEVICE,
     SENSORS_LOAD_AVG,
     SENSORS_RATES,
     SENSORS_TEMPERATURES,
 )
-from .router import KEY_COORDINATOR, KEY_SENSORS, AsusWrtRouter
+from .router import AsusWrtRouter
 
 
-@dataclass
+@dataclass(frozen=True)
 class AsusWrtSensorEntityDescription(SensorEntityDescription):
     """A class that describes AsusWrt sensor entities."""
 
     factor: int | None = None
-    precision: int = 2
 
 
-DEFAULT_PREFIX = "Asuswrt"
 UNIT_DEVICES = "Devices"
 
 CONNECTION_SENSORS: tuple[AsusWrtSensorEntityDescription, ...] = (
     AsusWrtSensorEntityDescription(
         key=SENSORS_CONNECTED_DEVICE[0],
-        name="Devices Connected",
-        icon="mdi:router-network",
+        translation_key="devices_connected",
         state_class=SensorStateClass.MEASUREMENT,
         native_unit_of_measurement=UNIT_DEVICES,
     ),
     AsusWrtSensorEntityDescription(
         key=SENSORS_RATES[0],
-        name="Download Speed",
-        icon="mdi:download-network",
+        translation_key="download_speed",
+        device_class=SensorDeviceClass.DATA_RATE,
         state_class=SensorStateClass.MEASUREMENT,
-        native_unit_of_measurement=DATA_RATE_MEGABITS_PER_SECOND,
+        native_unit_of_measurement=UnitOfDataRate.MEGABITS_PER_SECOND,
         entity_registry_enabled_default=False,
+        suggested_display_precision=2,
         factor=125000,
     ),
     AsusWrtSensorEntityDescription(
         key=SENSORS_RATES[1],
-        name="Upload Speed",
-        icon="mdi:upload-network",
+        translation_key="upload_speed",
+        device_class=SensorDeviceClass.DATA_RATE,
         state_class=SensorStateClass.MEASUREMENT,
-        native_unit_of_measurement=DATA_RATE_MEGABITS_PER_SECOND,
+        native_unit_of_measurement=UnitOfDataRate.MEGABITS_PER_SECOND,
         entity_registry_enabled_default=False,
+        suggested_display_precision=2,
         factor=125000,
     ),
     AsusWrtSensorEntityDescription(
         key=SENSORS_BYTES[0],
-        name="Download",
-        icon="mdi:download",
+        translation_key="download",
         state_class=SensorStateClass.TOTAL_INCREASING,
-        native_unit_of_measurement=DATA_GIGABYTES,
+        native_unit_of_measurement=UnitOfInformation.GIGABYTES,
+        device_class=SensorDeviceClass.DATA_SIZE,
         entity_registry_enabled_default=False,
+        suggested_display_precision=2,
         factor=1000000000,
     ),
     AsusWrtSensorEntityDescription(
         key=SENSORS_BYTES[1],
-        name="Upload",
-        icon="mdi:upload",
+        translation_key="upload",
         state_class=SensorStateClass.TOTAL_INCREASING,
-        native_unit_of_measurement=DATA_GIGABYTES,
+        native_unit_of_measurement=UnitOfInformation.GIGABYTES,
+        device_class=SensorDeviceClass.DATA_SIZE,
         entity_registry_enabled_default=False,
+        suggested_display_precision=2,
         factor=1000000000,
     ),
     AsusWrtSensorEntityDescription(
         key=SENSORS_LOAD_AVG[0],
-        name="Load Avg (1m)",
-        icon="mdi:cpu-32-bit",
+        translation_key="load_avg_1m",
         state_class=SensorStateClass.MEASUREMENT,
         entity_category=EntityCategory.DIAGNOSTIC,
         entity_registry_enabled_default=False,
-        factor=1,
-        precision=1,
+        suggested_display_precision=1,
     ),
     AsusWrtSensorEntityDescription(
         key=SENSORS_LOAD_AVG[1],
-        name="Load Avg (5m)",
-        icon="mdi:cpu-32-bit",
+        translation_key="load_avg_5m",
         state_class=SensorStateClass.MEASUREMENT,
         entity_category=EntityCategory.DIAGNOSTIC,
         entity_registry_enabled_default=False,
-        factor=1,
-        precision=1,
+        suggested_display_precision=1,
     ),
     AsusWrtSensorEntityDescription(
         key=SENSORS_LOAD_AVG[2],
-        name="Load Avg (15m)",
-        icon="mdi:cpu-32-bit",
+        translation_key="load_avg_15m",
         state_class=SensorStateClass.MEASUREMENT,
         entity_category=EntityCategory.DIAGNOSTIC,
         entity_registry_enabled_default=False,
-        factor=1,
-        precision=1,
+        suggested_display_precision=1,
     ),
     AsusWrtSensorEntityDescription(
         key=SENSORS_TEMPERATURES[0],
-        name="2.4GHz Temperature",
+        translation_key="24ghz_temperature",
         state_class=SensorStateClass.MEASUREMENT,
         device_class=SensorDeviceClass.TEMPERATURE,
-        native_unit_of_measurement=TEMP_CELSIUS,
+        native_unit_of_measurement=UnitOfTemperature.CELSIUS,
         entity_category=EntityCategory.DIAGNOSTIC,
         entity_registry_enabled_default=False,
-        factor=1,
-        precision=1,
+        suggested_display_precision=1,
     ),
     AsusWrtSensorEntityDescription(
         key=SENSORS_TEMPERATURES[1],
-        name="5GHz Temperature",
+        translation_key="5ghz_temperature",
         state_class=SensorStateClass.MEASUREMENT,
         device_class=SensorDeviceClass.TEMPERATURE,
-        native_unit_of_measurement=TEMP_CELSIUS,
+        native_unit_of_measurement=UnitOfTemperature.CELSIUS,
         entity_category=EntityCategory.DIAGNOSTIC,
         entity_registry_enabled_default=False,
-        factor=1,
-        precision=1,
+        suggested_display_precision=1,
     ),
     AsusWrtSensorEntityDescription(
         key=SENSORS_TEMPERATURES[2],
-        name="CPU Temperature",
+        translation_key="cpu_temperature",
         state_class=SensorStateClass.MEASUREMENT,
         device_class=SensorDeviceClass.TEMPERATURE,
-        native_unit_of_measurement=TEMP_CELSIUS,
+        native_unit_of_measurement=UnitOfTemperature.CELSIUS,
         entity_category=EntityCategory.DIAGNOSTIC,
         entity_registry_enabled_default=False,
-        factor=1,
-        precision=1,
+        suggested_display_precision=1,
+    ),
+    AsusWrtSensorEntityDescription(
+        key=SENSORS_TEMPERATURES[3],
+        translation_key="5ghz_2_temperature",
+        state_class=SensorStateClass.MEASUREMENT,
+        device_class=SensorDeviceClass.TEMPERATURE,
+        native_unit_of_measurement=UnitOfTemperature.CELSIUS,
+        entity_category=EntityCategory.DIAGNOSTIC,
+        entity_registry_enabled_default=False,
+        suggested_display_precision=1,
+    ),
+    AsusWrtSensorEntityDescription(
+        key=SENSORS_TEMPERATURES[4],
+        translation_key="6ghz_temperature",
+        state_class=SensorStateClass.MEASUREMENT,
+        device_class=SensorDeviceClass.TEMPERATURE,
+        native_unit_of_measurement=UnitOfTemperature.CELSIUS,
+        entity_category=EntityCategory.DIAGNOSTIC,
+        entity_registry_enabled_default=False,
+        suggested_display_precision=1,
     ),
 )
 
 
 async def async_setup_entry(
-    hass: HomeAssistant, entry: ConfigEntry, async_add_entities
+    hass: HomeAssistant, entry: ConfigEntry, async_add_entities: AddEntitiesCallback
 ) -> None:
     """Set up the sensors."""
     router: AsusWrtRouter = hass.data[DOMAIN][entry.entry_id][DATA_ASUSWRT]
@@ -180,6 +195,9 @@ async def async_setup_entry(
 class AsusWrtSensor(CoordinatorEntity, SensorEntity):
     """Representation of a AsusWrt sensor."""
 
+    entity_description: AsusWrtSensorEntityDescription
+    _attr_has_entity_name = True
+
     def __init__(
         self,
         coordinator: DataUpdateCoordinator,
@@ -188,18 +206,17 @@ class AsusWrtSensor(CoordinatorEntity, SensorEntity):
     ) -> None:
         """Initialize a AsusWrt sensor."""
         super().__init__(coordinator)
-        self.entity_description: AsusWrtSensorEntityDescription = description
+        self.entity_description = description
 
-        self._attr_name = f"{DEFAULT_PREFIX} {description.name}"
-        self._attr_unique_id = f"{DOMAIN} {self.name}"
+        self._attr_unique_id = slugify(f"{router.unique_id}_{description.key}")
         self._attr_device_info = router.device_info
         self._attr_extra_state_attributes = {"hostname": router.host}
 
     @property
-    def native_value(self) -> float | str | None:
+    def native_value(self) -> float | int | str | None:
         """Return current state."""
         descr = self.entity_description
-        state = self.coordinator.data.get(descr.key)
-        if state is not None and descr.factor and isinstance(state, Real):
-            return round(state / descr.factor, descr.precision)
+        state: float | int | str | None = self.coordinator.data.get(descr.key)
+        if state is not None and descr.factor and isinstance(state, (float, int)):
+            return state / descr.factor
         return state
