@@ -42,7 +42,7 @@ Device = namedtuple("Device", ["mac", "name", "ip", "last_update"])
 
 
 class BboxDeviceScanner(DeviceScanner):
-    """This class scans for devices connected to the bbox."""
+    """Scanner for devices connected to the bbox."""
 
     def __init__(self, config):
         """Get host from config."""
