@@ -70,62 +70,6 @@ CONF_OPENED = "opened"
 CONF_NOT_OPENED = "not_opened"
 
 
-TURNED_ON = [
-    CONF_BAT_LOW,
-    CONF_CO,
-    CONF_COLD,
-    CONF_CONNECTED,
-    CONF_GAS,
-    CONF_HOT,
-    CONF_LIGHT,
-    CONF_NOT_LOCKED,
-    CONF_MOIST,
-    CONF_MOTION,
-    CONF_MOVING,
-    CONF_OCCUPIED,
-    CONF_OPENED,
-    CONF_PLUGGED_IN,
-    CONF_POWERED,
-    CONF_PRESENT,
-    CONF_PROBLEM,
-    CONF_RUNNING,
-    CONF_SMOKE,
-    CONF_SOUND,
-    CONF_UNSAFE,
-    CONF_UPDATE,
-    CONF_VIBRATION,
-    CONF_TAMPERED,
-    CONF_TURNED_ON,
-]
-
-TURNED_OFF = [
-    CONF_NOT_BAT_LOW,
-    CONF_NOT_COLD,
-    CONF_NOT_CONNECTED,
-    CONF_NOT_HOT,
-    CONF_LOCKED,
-    CONF_NOT_MOIST,
-    CONF_NOT_MOVING,
-    CONF_NOT_OCCUPIED,
-    CONF_NOT_OPENED,
-    CONF_NOT_PLUGGED_IN,
-    CONF_NOT_POWERED,
-    CONF_NOT_PRESENT,
-    CONF_NOT_TAMPERED,
-    CONF_NOT_UNSAFE,
-    CONF_NO_CO,
-    CONF_NO_GAS,
-    CONF_NO_LIGHT,
-    CONF_NO_MOTION,
-    CONF_NO_PROBLEM,
-    CONF_NOT_RUNNING,
-    CONF_NO_SMOKE,
-    CONF_NO_SOUND,
-    CONF_NO_VIBRATION,
-    CONF_TURNED_OFF,
-]
-
-
 ENTITY_TRIGGERS = {
     BinarySensorDeviceClass.BATTERY: [
         {CONF_TYPE: CONF_BAT_LOW},
@@ -168,8 +112,8 @@ ENTITY_TRIGGERS = {
         {CONF_TYPE: CONF_NO_LIGHT},
     ],
     BinarySensorDeviceClass.LOCK: [
-        {CONF_TYPE: CONF_LOCKED},
         {CONF_TYPE: CONF_NOT_LOCKED},
+        {CONF_TYPE: CONF_LOCKED},
     ],
     BinarySensorDeviceClass.MOISTURE: [
         {CONF_TYPE: CONF_MOIST},
@@ -245,10 +189,13 @@ ENTITY_TRIGGERS = {
     ],
 }
 
+TURNED_ON = [trigger[0][CONF_TYPE] for trigger in ENTITY_TRIGGERS.values()]
+TURNED_OFF = [trigger[1][CONF_TYPE] for trigger in ENTITY_TRIGGERS.values()]
+
 
 TRIGGER_SCHEMA = DEVICE_TRIGGER_BASE_SCHEMA.extend(
     {
-        vol.Required(CONF_ENTITY_ID): cv.entity_id,
+        vol.Required(CONF_ENTITY_ID): cv.entity_id_or_uuid,
         vol.Required(CONF_TYPE): vol.In(TURNED_OFF + TURNED_ON),
         vol.Optional(CONF_FOR): cv.positive_time_period_dict,
     }
@@ -307,7 +254,7 @@ async def async_get_triggers(
                 **automation,
                 "platform": "device",
                 "device_id": device_id,
-                "entity_id": entry.entity_id,
+                "entity_id": entry.id,
                 "domain": DOMAIN,
             }
             for automation in templates
