@@ -30,6 +30,9 @@ _LOGGER = logging.getLogger(__name__)
 
 ALLOWED_CONDITION_BASED_SERVICE_KEYS = {
     "BRAKE_FLUID",
+    "BRAKE_PADS_FRONT",
+    "BRAKE_PADS_REAR",
+    "EMISSION_CHECK",
     "ENGINE_OIL",
     "OIL",
     "TIRE_WEAR_FRONT",
@@ -37,10 +40,14 @@ ALLOWED_CONDITION_BASED_SERVICE_KEYS = {
     "VEHICLE_CHECK",
     "VEHICLE_TUV",
 }
-LOGGED_CONDITION_BASED_SERVICE_WARNINGS = set()
+LOGGED_CONDITION_BASED_SERVICE_WARNINGS: set[str] = set()
 
-ALLOWED_CHECK_CONTROL_MESSAGE_KEYS = {"ENGINE_OIL", "TIRE_PRESSURE"}
-LOGGED_CHECK_CONTROL_MESSAGE_WARNINGS = set()
+ALLOWED_CHECK_CONTROL_MESSAGE_KEYS = {
+    "ENGINE_OIL",
+    "TIRE_PRESSURE",
+    "WASHING_FLUID",
+}
+LOGGED_CHECK_CONTROL_MESSAGE_WARNINGS: set[str] = set()
 
 
 def _condition_based_services(
@@ -102,14 +109,14 @@ def _format_cbs_report(
     return result
 
 
-@dataclass
+@dataclass(frozen=True)
 class BMWRequiredKeysMixin:
     """Mixin for required keys."""
 
     value_fn: Callable[[MyBMWVehicle], bool]
 
 
-@dataclass
+@dataclass(frozen=True)
 class BMWBinarySensorEntityDescription(
     BinarySensorEntityDescription, BMWRequiredKeysMixin
 ):
@@ -121,7 +128,7 @@ class BMWBinarySensorEntityDescription(
 SENSOR_TYPES: tuple[BMWBinarySensorEntityDescription, ...] = (
     BMWBinarySensorEntityDescription(
         key="lids",
-        name="Lids",
+        translation_key="lids",
         device_class=BinarySensorDeviceClass.OPENING,
         icon="mdi:car-door-lock",
         # device class opening: On means open, Off means closed
@@ -132,7 +139,7 @@ SENSOR_TYPES: tuple[BMWBinarySensorEntityDescription, ...] = (
     ),
     BMWBinarySensorEntityDescription(
         key="windows",
-        name="Windows",
+        translation_key="windows",
         device_class=BinarySensorDeviceClass.OPENING,
         icon="mdi:car-door",
         # device class opening: On means open, Off means closed
@@ -143,7 +150,7 @@ SENSOR_TYPES: tuple[BMWBinarySensorEntityDescription, ...] = (
     ),
     BMWBinarySensorEntityDescription(
         key="door_lock_state",
-        name="Door lock state",
+        translation_key="door_lock_state",
         device_class=BinarySensorDeviceClass.LOCK,
         icon="mdi:car-key",
         # device class lock: On means unlocked, Off means locked
@@ -156,7 +163,7 @@ SENSOR_TYPES: tuple[BMWBinarySensorEntityDescription, ...] = (
     ),
     BMWBinarySensorEntityDescription(
         key="condition_based_services",
-        name="Condition based services",
+        translation_key="condition_based_services",
         device_class=BinarySensorDeviceClass.PROBLEM,
         icon="mdi:wrench",
         # device class problem: On means problem detected, Off means no problem
@@ -165,7 +172,7 @@ SENSOR_TYPES: tuple[BMWBinarySensorEntityDescription, ...] = (
     ),
     BMWBinarySensorEntityDescription(
         key="check_control_messages",
-        name="Check control messages",
+        translation_key="check_control_messages",
         device_class=BinarySensorDeviceClass.PROBLEM,
         icon="mdi:car-tire-alert",
         # device class problem: On means problem detected, Off means no problem
@@ -175,7 +182,7 @@ SENSOR_TYPES: tuple[BMWBinarySensorEntityDescription, ...] = (
     # electric
     BMWBinarySensorEntityDescription(
         key="charging_status",
-        name="Charging status",
+        translation_key="charging_status",
         device_class=BinarySensorDeviceClass.BATTERY_CHARGING,
         icon="mdi:ev-station",
         # device class power: On means power detected, Off means no power
@@ -183,11 +190,19 @@ SENSOR_TYPES: tuple[BMWBinarySensorEntityDescription, ...] = (
     ),
     BMWBinarySensorEntityDescription(
         key="connection_status",
-        name="Connection status",
+        translation_key="connection_status",
         device_class=BinarySensorDeviceClass.PLUG,
         icon="mdi:car-electric",
         value_fn=lambda v: v.fuel_and_battery.is_charger_connected,
     ),
+    BMWBinarySensorEntityDescription(
+        key="is_pre_entry_climatization_enabled",
+        translation_key="is_pre_entry_climatization_enabled",
+        icon="mdi:car-seat-heater",
+        value_fn=lambda v: v.charging_profile.is_pre_entry_climatization_enabled
+        if v.charging_profile
+        else False,
+    ),
 )
 
 
