@@ -6,27 +6,22 @@ from typing import Any
 from urllib.parse import urlparse
 
 from aiohttp import CookieJar
-from pybravia import BraviaTV, BraviaTVAuthError, BraviaTVError, BraviaTVNotSupported
+from pybravia import BraviaAuthError, BraviaClient, BraviaError, BraviaNotSupported
 import voluptuous as vol
 
 from homeassistant import config_entries
 from homeassistant.components import ssdp
 from homeassistant.config_entries import ConfigEntry
-from homeassistant.const import CONF_HOST, CONF_MAC, CONF_NAME, CONF_PIN
-from homeassistant.core import callback
+from homeassistant.const import CONF_CLIENT_ID, CONF_HOST, CONF_MAC, CONF_NAME, CONF_PIN
 from homeassistant.data_entry_flow import FlowResult
 from homeassistant.helpers import instance_id
 from homeassistant.helpers.aiohttp_client import async_create_clientsession
-import homeassistant.helpers.config_validation as cv
 from homeassistant.util.network import is_host_valid
 
-from . import BraviaTVCoordinator
 from .const import (
     ATTR_CID,
     ATTR_MAC,
     ATTR_MODEL,
-    CONF_CLIENT_ID,
-    CONF_IGNORED_SOURCES,
     CONF_NICKNAME,
     CONF_USE_PSK,
     DOMAIN,
@@ -41,16 +36,10 @@ class BraviaTVConfigFlow(config_entries.ConfigFlow, domain=DOMAIN):
 
     def __init__(self) -> None:
         """Initialize config flow."""
-        self.client: BraviaTV | None = None
+        self.client: BraviaClient | None = None
         self.device_config: dict[str, Any] = {}
         self.entry: ConfigEntry | None = None
 
-    @staticmethod
-    @callback
-    def async_get_options_flow(config_entry: ConfigEntry) -> BraviaTVOptionsFlowHandler:
-        """Bravia TV options callback."""
-        return BraviaTVOptionsFlowHandler(config_entry)
-
     def create_client(self) -> None:
         """Create Bravia TV client from config."""
         host = self.device_config[CONF_HOST]
@@ -58,7 +47,7 @@ class BraviaTVConfigFlow(config_entries.ConfigFlow, domain=DOMAIN):
             self.hass,
             cookie_jar=CookieJar(unsafe=True, quote_cookie=False),
         )
-        self.client = BraviaTV(host=host, session=session)
+        self.client = BraviaClient(host=host, session=session)
 
     async def gen_instance_ids(self) -> tuple[str, str]:
         """Generate client_id and nickname."""
@@ -162,18 +151,18 @@ class BraviaTVConfigFlow(config_entries.ConfigFlow, domain=DOMAIN):
                 if self.entry:
                     return await self.async_reauth_device()
                 return await self.async_create_device()
-            except BraviaTVAuthError:
+            except BraviaAuthError:
                 errors["base"] = "invalid_auth"
-            except BraviaTVNotSupported:
+            except BraviaNotSupported:
                 errors["base"] = "unsupported_model"
-            except BraviaTVError:
+            except BraviaError:
                 errors["base"] = "cannot_connect"
 
         assert self.client
 
         try:
             await self.client.pair(client_id, nickname)
-        except BraviaTVError:
+        except BraviaError:
             return self.async_abort(reason="no_ip_control")
 
         return self.async_show_form(
@@ -198,11 +187,11 @@ class BraviaTVConfigFlow(config_entries.ConfigFlow, domain=DOMAIN):
                 if self.entry:
                     return await self.async_reauth_device()
                 return await self.async_create_device()
-            except BraviaTVAuthError:
+            except BraviaAuthError:
                 errors["base"] = "invalid_auth"
-            except BraviaTVNotSupported:
+            except BraviaNotSupported:
                 errors["base"] = "unsupported_model"
-            except BraviaTVError:
+            except BraviaError:
                 errors["base"] = "cannot_connect"
 
         return self.async_show_form(
@@ -257,51 +246,3 @@ class BraviaTVConfigFlow(config_entries.ConfigFlow, domain=DOMAIN):
         self.entry = self.hass.config_entries.async_get_entry(self.context["entry_id"])
         self.device_config = {**entry_data}
         return await self.async_step_authorize()
-
-
-class BraviaTVOptionsFlowHandler(config_entries.OptionsFlowWithConfigEntry):
-    """Config flow options for Bravia TV."""
-
-    data_schema: vol.Schema
-
-    async def async_step_init(
-        self, user_input: dict[str, Any] | None = None
-    ) -> FlowResult:
-        """Manage the options."""
-        coordinator: BraviaTVCoordinator
-        coordinator = self.hass.data[DOMAIN][self.config_entry.entry_id]
-
-        try:
-            await coordinator.async_update_sources()
-        except BraviaTVError:
-            return self.async_abort(reason="failed_update")
-
-        sources = coordinator.source_map.values()
-        source_list = [item["title"] for item in sources]
-        ignored_sources = self.options.get(CONF_IGNORED_SOURCES, [])
-
-        for item in ignored_sources:
-            if item not in source_list:
-                source_list.append(item)
-
-        self.data_schema = vol.Schema(
-            {
-                vol.Optional(CONF_IGNORED_SOURCES): cv.multi_select(source_list),
-            }
-        )
-
-        return await self.async_step_user()
-
-    async def async_step_user(
-        self, user_input: dict[str, Any] | None = None
-    ) -> FlowResult:
-        """Handle a flow initialized by the user."""
-        if user_input is not None:
-            return self.async_create_entry(title="", data=user_input)
-
-        return self.async_show_form(
-            step_id="user",
-            data_schema=self.add_suggested_values_to_schema(
-                self.data_schema, self.options
-            ),
-        )
