@@ -1,13 +1,17 @@
 """Alexa configuration for Home Assistant Cloud."""
+from __future__ import annotations
+
 import asyncio
+from collections.abc import Callable
 from contextlib import suppress
-from datetime import timedelta
+from datetime import datetime, timedelta
 from http import HTTPStatus
 import logging
+from typing import TYPE_CHECKING, Any
 
 import aiohttp
-import async_timeout
 from hass_nabucasa import Cloud, cloud_api
+from yarl import URL
 
 from homeassistant.components import persistent_notification
 from homeassistant.components.alexa import (
@@ -17,23 +21,113 @@ from homeassistant.components.alexa import (
     errors as alexa_errors,
     state_report as alexa_state_report,
 )
+from homeassistant.components.binary_sensor import BinarySensorDeviceClass
+from homeassistant.components.homeassistant.exposed_entities import (
+    async_expose_entity,
+    async_get_assistant_settings,
+    async_listen_entity_updates,
+    async_should_expose,
+)
+from homeassistant.components.sensor import SensorDeviceClass
 from homeassistant.const import CLOUD_NEVER_EXPOSED_ENTITIES
-from homeassistant.core import HomeAssistant, callback, split_entity_id
+from homeassistant.core import Event, HomeAssistant, callback, split_entity_id
+from homeassistant.exceptions import HomeAssistantError
 from homeassistant.helpers import entity_registry as er, start
+from homeassistant.helpers.entity import get_device_class
+from homeassistant.helpers.entityfilter import EntityFilter
 from homeassistant.helpers.event import async_call_later
 from homeassistant.setup import async_setup_component
 from homeassistant.util.dt import utcnow
 
-from .const import CONF_ENTITY_CONFIG, CONF_FILTER, PREF_SHOULD_EXPOSE
-from .prefs import CloudPreferences
+from .const import (
+    CONF_ENTITY_CONFIG,
+    CONF_FILTER,
+    DOMAIN as CLOUD_DOMAIN,
+    PREF_ALEXA_REPORT_STATE,
+    PREF_ENABLE_ALEXA,
+    PREF_SHOULD_EXPOSE,
+)
+from .prefs import ALEXA_SETTINGS_VERSION, CloudPreferences
+
+if TYPE_CHECKING:
+    from .client import CloudClient
 
 _LOGGER = logging.getLogger(__name__)
 
+CLOUD_ALEXA = f"{CLOUD_DOMAIN}.{ALEXA_DOMAIN}"
+
 # Time to wait when entity preferences have changed before syncing it to
 # the cloud.
 SYNC_DELAY = 1
 
 
+SUPPORTED_DOMAINS = {
+    "alarm_control_panel",
+    "alert",
+    "automation",
+    "button",
+    "camera",
+    "climate",
+    "cover",
+    "fan",
+    "group",
+    "humidifier",
+    "image_processing",
+    "input_boolean",
+    "input_button",
+    "input_number",
+    "light",
+    "lock",
+    "media_player",
+    "number",
+    "scene",
+    "script",
+    "switch",
+    "timer",
+    "vacuum",
+}
+
+SUPPORTED_BINARY_SENSOR_DEVICE_CLASSES = {
+    BinarySensorDeviceClass.DOOR,
+    BinarySensorDeviceClass.GARAGE_DOOR,
+    BinarySensorDeviceClass.MOTION,
+    BinarySensorDeviceClass.OPENING,
+    BinarySensorDeviceClass.PRESENCE,
+    BinarySensorDeviceClass.WINDOW,
+}
+
+SUPPORTED_SENSOR_DEVICE_CLASSES = {
+    SensorDeviceClass.TEMPERATURE,
+}
+
+
+def entity_supported(hass: HomeAssistant, entity_id: str) -> bool:
+    """Return if the entity is supported.
+
+    This is called when migrating from legacy config format to avoid exposing
+    all binary sensors and sensors.
+    """
+    domain = split_entity_id(entity_id)[0]
+    if domain in SUPPORTED_DOMAINS:
+        return True
+
+    try:
+        device_class = get_device_class(hass, entity_id)
+    except HomeAssistantError:
+        # The entity no longer exists
+        return False
+    if (
+        domain == "binary_sensor"
+        and device_class in SUPPORTED_BINARY_SENSOR_DEVICE_CLASSES
+    ):
+        return True
+
+    if domain == "sensor" and device_class in SUPPORTED_SENSOR_DEVICE_CLASSES:
+        return True
+
+    return False
+
+
 class CloudAlexaConfig(alexa_config.AbstractConfig):
     """Alexa Configuration."""
 
@@ -43,7 +137,7 @@ class CloudAlexaConfig(alexa_config.AbstractConfig):
         config: dict,
         cloud_user: str,
         prefs: CloudPreferences,
-        cloud: Cloud,
+        cloud: Cloud[CloudClient],
     ) -> None:
         """Initialize the Alexa config."""
         super().__init__(hass)
@@ -52,14 +146,13 @@ class CloudAlexaConfig(alexa_config.AbstractConfig):
         self._prefs = prefs
         self._cloud = cloud
         self._token = None
-        self._token_valid = None
-        self._cur_entity_prefs = prefs.alexa_entity_configs
-        self._cur_default_expose = prefs.alexa_default_expose
-        self._alexa_sync_unsub = None
-        self._endpoint = None
+        self._token_valid: datetime | None = None
+        self._cur_entity_prefs = async_get_assistant_settings(hass, CLOUD_ALEXA)
+        self._alexa_sync_unsub: Callable[[], None] | None = None
+        self._endpoint: str | URL | None = None
 
     @property
-    def enabled(self):
+    def enabled(self) -> bool:
         """Return if Alexa is enabled."""
         return (
             self._cloud.is_logged_in
@@ -68,17 +161,21 @@ class CloudAlexaConfig(alexa_config.AbstractConfig):
         )
 
     @property
-    def supports_auth(self):
+    def supports_auth(self) -> bool:
         """Return if config supports auth."""
         return True
 
     @property
-    def should_report_state(self):
+    def should_report_state(self) -> bool:
         """Return if states should be proactively reported."""
-        return self._prefs.alexa_report_state and self.authorized
+        return (
+            self._prefs.alexa_enabled
+            and self._prefs.alexa_report_state
+            and self.authorized
+        )
 
     @property
-    def endpoint(self):
+    def endpoint(self) -> str | URL | None:
         """Endpoint for report state."""
         if self._endpoint is None:
             raise ValueError("No endpoint available. Fetch access token first")
@@ -86,30 +183,79 @@ class CloudAlexaConfig(alexa_config.AbstractConfig):
         return self._endpoint
 
     @property
-    def locale(self):
+    def locale(self) -> str:
         """Return config locale."""
         # Not clear how to determine locale atm.
         return "en-US"
 
     @property
-    def entity_config(self):
+    def entity_config(self) -> dict[str, Any]:
         """Return entity config."""
         return self._config.get(CONF_ENTITY_CONFIG) or {}
 
     @callback
-    def user_identifier(self):
+    def user_identifier(self) -> str:
         """Return an identifier for the user that represents this config."""
         return self._cloud_user
 
-    async def async_initialize(self):
+    def _migrate_alexa_entity_settings_v1(self) -> None:
+        """Migrate alexa entity settings to entity registry options."""
+        if not self._config[CONF_FILTER].empty_filter:
+            # Don't migrate if there's a YAML config
+            return
+
+        for entity_id in {
+            *self.hass.states.async_entity_ids(),
+            *self._prefs.alexa_entity_configs,
+        }:
+            async_expose_entity(
+                self.hass,
+                CLOUD_ALEXA,
+                entity_id,
+                self._should_expose_legacy(entity_id),
+            )
+
+    async def async_initialize(self) -> None:
         """Initialize the Alexa config."""
         await super().async_initialize()
 
-        async def hass_started(hass):
+        async def on_hass_started(hass: HomeAssistant) -> None:
+            if self._prefs.alexa_settings_version != ALEXA_SETTINGS_VERSION:
+                _LOGGER.info(
+                    "Start migration of Alexa settings from v%s to v%s",
+                    self._prefs.alexa_settings_version,
+                    ALEXA_SETTINGS_VERSION,
+                )
+                if self._prefs.alexa_settings_version < 2 or (
+                    # Recover from a bug we had in 2023.5.0 where entities didn't get exposed
+                    self._prefs.alexa_settings_version < 3
+                    and not any(
+                        settings.get("should_expose", False)
+                        for settings in async_get_assistant_settings(
+                            hass, CLOUD_ALEXA
+                        ).values()
+                    )
+                ):
+                    self._migrate_alexa_entity_settings_v1()
+
+                _LOGGER.info(
+                    "Finished migration of Alexa settings from v%s to v%s",
+                    self._prefs.alexa_settings_version,
+                    ALEXA_SETTINGS_VERSION,
+                )
+                await self._prefs.async_update(
+                    alexa_settings_version=ALEXA_SETTINGS_VERSION
+                )
+            async_listen_entity_updates(
+                self.hass, CLOUD_ALEXA, self._async_exposed_entities_updated
+            )
+
+        async def on_hass_start(hass: HomeAssistant) -> None:
             if self.enabled and ALEXA_DOMAIN not in self.hass.config.components:
                 await async_setup_component(self.hass, ALEXA_DOMAIN, {})
 
-        start.async_at_start(self.hass, hass_started)
+        start.async_at_start(self.hass, on_hass_start)
+        start.async_at_started(self.hass, on_hass_started)
 
         self._prefs.async_listen_updates(self._async_prefs_updated)
         self.hass.bus.async_listen(
@@ -117,38 +263,53 @@ class CloudAlexaConfig(alexa_config.AbstractConfig):
             self._handle_entity_registry_updated,
         )
 
-    def should_expose(self, entity_id):
+    def _should_expose_legacy(self, entity_id: str) -> bool:
         """If an entity should be exposed."""
         if entity_id in CLOUD_NEVER_EXPOSED_ENTITIES:
             return False
 
-        if not self._config[CONF_FILTER].empty_filter:
-            return self._config[CONF_FILTER](entity_id)
-
         entity_configs = self._prefs.alexa_entity_configs
         entity_config = entity_configs.get(entity_id, {})
-        entity_expose = entity_config.get(PREF_SHOULD_EXPOSE)
+        entity_expose: bool | None = entity_config.get(PREF_SHOULD_EXPOSE)
         if entity_expose is not None:
             return entity_expose
 
         entity_registry = er.async_get(self.hass)
         if registry_entry := entity_registry.async_get(entity_id):
-            auxiliary_entity = registry_entry.entity_category is not None
+            auxiliary_entity = (
+                registry_entry.entity_category is not None
+                or registry_entry.hidden_by is not None
+            )
         else:
             auxiliary_entity = False
 
         # Backwards compat
         if (default_expose := self._prefs.alexa_default_expose) is None:
-            return not auxiliary_entity
+            return not auxiliary_entity and entity_supported(self.hass, entity_id)
 
-        return not auxiliary_entity and split_entity_id(entity_id)[0] in default_expose
+        return (
+            not auxiliary_entity
+            and split_entity_id(entity_id)[0] in default_expose
+            and entity_supported(self.hass, entity_id)
+        )
 
     @callback
-    def async_invalidate_access_token(self):
+    def should_expose(self, entity_id: str) -> bool:
+        """If an entity should be exposed."""
+        entity_filter: EntityFilter = self._config[CONF_FILTER]
+        if not entity_filter.empty_filter:
+            if entity_id in CLOUD_NEVER_EXPOSED_ENTITIES:
+                return False
+            return entity_filter(entity_id)
+
+        return async_should_expose(self.hass, CLOUD_ALEXA, entity_id)
+
+    @callback
+    def async_invalidate_access_token(self) -> None:
         """Invalidate access token."""
         self._token_valid = None
 
-    async def async_get_access_token(self):
+    async def async_get_access_token(self) -> str | None:
         """Get an access token."""
         if self._token_valid is not None and self._token_valid > utcnow():
             return self._token
@@ -161,9 +322,11 @@ class CloudAlexaConfig(alexa_config.AbstractConfig):
                 if self.should_report_state:
                     persistent_notification.async_create(
                         self.hass,
-                        f"There was an error reporting state to Alexa ({body['reason']}). "
-                        "Please re-link your Alexa skill via the Alexa app to "
-                        "continue using it.",
+                        (
+                            "There was an error reporting state to Alexa"
+                            f" ({body['reason']}). Please re-link your Alexa skill via"
+                            " the Alexa app to continue using it."
+                        ),
                         "Alexa state reporting disabled",
                         "cloud_alexa_report",
                     )
@@ -176,7 +339,7 @@ class CloudAlexaConfig(alexa_config.AbstractConfig):
         self._token_valid = utcnow() + timedelta(seconds=body["expires_in"])
         return self._token
 
-    async def _async_prefs_updated(self, prefs):
+    async def _async_prefs_updated(self, prefs: CloudPreferences) -> None:
         """Handle updated preferences."""
         if not self._cloud.is_logged_in:
             if self.is_reporting_states:
@@ -187,6 +350,8 @@ class CloudAlexaConfig(alexa_config.AbstractConfig):
                 self._alexa_sync_unsub = None
             return
 
+        updated_prefs = prefs.last_updated
+
         if (
             ALEXA_DOMAIN not in self.hass.config.components
             and self.enabled
@@ -208,41 +373,46 @@ class CloudAlexaConfig(alexa_config.AbstractConfig):
             await self.async_sync_entities()
             return
 
-        # If user has filter in config.yaml, don't sync.
-        if not self._config[CONF_FILTER].empty_filter:
-            return
-
-        # If entity prefs are the same, don't sync.
-        if (
-            self._cur_entity_prefs is prefs.alexa_entity_configs
-            and self._cur_default_expose is prefs.alexa_default_expose
+        # Nothing to do if no Alexa related things have changed
+        if not any(
+            key in updated_prefs
+            for key in (
+                PREF_ALEXA_REPORT_STATE,
+                PREF_ENABLE_ALEXA,
+            )
         ):
             return
 
+        await self.async_sync_entities()
+
+    @callback
+    def _async_exposed_entities_updated(self) -> None:
+        """Handle updated preferences."""
+        # Delay updating as we might update more
         if self._alexa_sync_unsub:
             self._alexa_sync_unsub()
-            self._alexa_sync_unsub = None
-
-        if self._cur_default_expose is not prefs.alexa_default_expose:
-            await self.async_sync_entities()
-            return
 
         self._alexa_sync_unsub = async_call_later(
             self.hass, SYNC_DELAY, self._sync_prefs
         )
 
-    async def _sync_prefs(self, _now):
+    async def _sync_prefs(self, _now: datetime) -> None:
         """Sync the updated preferences to Alexa."""
         self._alexa_sync_unsub = None
         old_prefs = self._cur_entity_prefs
-        new_prefs = self._prefs.alexa_entity_configs
+        new_prefs = async_get_assistant_settings(self.hass, CLOUD_ALEXA)
 
         seen = set()
         to_update = []
         to_remove = []
+        is_enabled = self.enabled
 
         for entity_id, info in old_prefs.items():
             seen.add(entity_id)
+
+            if not is_enabled:
+                to_remove.append(entity_id)
+
             old_expose = info.get(PREF_SHOULD_EXPOSE)
 
             if entity_id in new_prefs:
@@ -278,7 +448,7 @@ class CloudAlexaConfig(alexa_config.AbstractConfig):
         if await self._sync_helper(to_update, to_remove):
             self._cur_entity_prefs = new_prefs
 
-    async def async_sync_entities(self):
+    async def async_sync_entities(self) -> bool:
         """Sync all entities to Alexa."""
         # Remove any pending sync
         if self._alexa_sync_unsub:
@@ -288,15 +458,17 @@ class CloudAlexaConfig(alexa_config.AbstractConfig):
         to_update = []
         to_remove = []
 
+        is_enabled = self.enabled
+
         for entity in alexa_entities.async_get_entities(self.hass, self):
-            if self.should_expose(entity.entity_id):
+            if is_enabled and self.should_expose(entity.entity_id):
                 to_update.append(entity.entity_id)
             else:
                 to_remove.append(entity.entity_id)
 
         return await self._sync_helper(to_update, to_remove)
 
-    async def _sync_helper(self, to_update, to_remove) -> bool:
+    async def _sync_helper(self, to_update: list[str], to_remove: list[str]) -> bool:
         """Sync entities to Alexa.
 
         Return boolean if it was successful.
@@ -311,18 +483,24 @@ class CloudAlexaConfig(alexa_config.AbstractConfig):
 
         if to_update:
             tasks.append(
-                alexa_state_report.async_send_add_or_update_message(
-                    self.hass, self, to_update
+                asyncio.create_task(
+                    alexa_state_report.async_send_add_or_update_message(
+                        self.hass, self, to_update
+                    )
                 )
             )
 
         if to_remove:
             tasks.append(
-                alexa_state_report.async_send_delete_message(self.hass, self, to_remove)
+                asyncio.create_task(
+                    alexa_state_report.async_send_delete_message(
+                        self.hass, self, to_remove
+                    )
+                )
             )
 
         try:
-            async with async_timeout.timeout(10):
+            async with asyncio.timeout(10):
                 await asyncio.wait(tasks, return_when=asyncio.ALL_COMPLETED)
 
             return True
@@ -335,7 +513,7 @@ class CloudAlexaConfig(alexa_config.AbstractConfig):
             _LOGGER.warning("Error trying to sync entities to Alexa: %s", err)
             return False
 
-    async def _handle_entity_registry_updated(self, event):
+    async def _handle_entity_registry_updated(self, event: Event) -> None:
         """Handle when entity registry updated."""
         if not self.enabled or not self._cloud.is_logged_in:
             return
