@@ -2,6 +2,7 @@
 from __future__ import annotations
 
 import logging
+from typing import Any, Self, TypeVar
 
 import voluptuous as vol
 
@@ -21,6 +22,8 @@ from homeassistant.helpers.restore_state import RestoreEntity
 from homeassistant.helpers.storage import Store
 from homeassistant.helpers.typing import ConfigType
 
+_T = TypeVar("_T")
+
 _LOGGER = logging.getLogger(__name__)
 
 ATTR_INITIAL = "initial"
@@ -42,12 +45,12 @@ ENTITY_ID_FORMAT = DOMAIN + ".{}"
 SERVICE_DECREMENT = "decrement"
 SERVICE_INCREMENT = "increment"
 SERVICE_RESET = "reset"
-SERVICE_CONFIGURE = "configure"
+SERVICE_SET_VALUE = "set_value"
 
 STORAGE_KEY = DOMAIN
 STORAGE_VERSION = 1
 
-CREATE_FIELDS = {
+STORAGE_FIELDS = {
     vol.Optional(CONF_ICON): cv.icon,
     vol.Optional(CONF_INITIAL, default=DEFAULT_INITIAL): cv.positive_int,
     vol.Required(CONF_NAME): vol.All(cv.string, vol.Length(min=1)),
@@ -57,18 +60,8 @@ CREATE_FIELDS = {
     vol.Optional(CONF_STEP, default=DEFAULT_STEP): cv.positive_int,
 }
 
-UPDATE_FIELDS = {
-    vol.Optional(CONF_ICON): cv.icon,
-    vol.Optional(CONF_INITIAL): cv.positive_int,
-    vol.Optional(CONF_NAME): cv.string,
-    vol.Optional(CONF_MAXIMUM): vol.Any(None, vol.Coerce(int)),
-    vol.Optional(CONF_MINIMUM): vol.Any(None, vol.Coerce(int)),
-    vol.Optional(CONF_RESTORE): cv.boolean,
-    vol.Optional(CONF_STEP): cv.positive_int,
-}
 
-
-def _none_to_empty_dict(value):
+def _none_to_empty_dict(value: _T | None) -> _T | dict[str, Any]:
     if value is None:
         return {}
     return value
@@ -103,19 +96,18 @@ CONFIG_SCHEMA = vol.Schema(
 
 async def async_setup(hass: HomeAssistant, config: ConfigType) -> bool:
     """Set up the counters."""
-    component = EntityComponent(_LOGGER, DOMAIN, hass)
+    component = EntityComponent[Counter](_LOGGER, DOMAIN, hass)
     id_manager = collection.IDManager()
 
     yaml_collection = collection.YamlCollection(
         logging.getLogger(f"{__name__}.yaml_collection"), id_manager
     )
     collection.sync_entity_lifecycle(
-        hass, DOMAIN, DOMAIN, component, yaml_collection, Counter.from_yaml
+        hass, DOMAIN, DOMAIN, component, yaml_collection, Counter
     )
 
     storage_collection = CounterStorageCollection(
         Store(hass, STORAGE_VERSION, STORAGE_KEY),
-        logging.getLogger(f"{__name__}.storage_collection"),
         id_manager,
     )
     collection.sync_entity_lifecycle(
@@ -127,62 +119,62 @@ async def async_setup(hass: HomeAssistant, config: ConfigType) -> bool:
     )
     await storage_collection.async_load()
 
-    collection.StorageCollectionWebsocket(
-        storage_collection, DOMAIN, DOMAIN, CREATE_FIELDS, UPDATE_FIELDS
+    collection.DictStorageCollectionWebsocket(
+        storage_collection, DOMAIN, DOMAIN, STORAGE_FIELDS, STORAGE_FIELDS
     ).async_setup(hass)
 
     component.async_register_entity_service(SERVICE_INCREMENT, {}, "async_increment")
     component.async_register_entity_service(SERVICE_DECREMENT, {}, "async_decrement")
     component.async_register_entity_service(SERVICE_RESET, {}, "async_reset")
     component.async_register_entity_service(
-        SERVICE_CONFIGURE,
-        {
-            vol.Optional(ATTR_MINIMUM): vol.Any(None, vol.Coerce(int)),
-            vol.Optional(ATTR_MAXIMUM): vol.Any(None, vol.Coerce(int)),
-            vol.Optional(ATTR_STEP): cv.positive_int,
-            vol.Optional(ATTR_INITIAL): cv.positive_int,
-            vol.Optional(VALUE): cv.positive_int,
-        },
-        "async_configure",
+        SERVICE_SET_VALUE,
+        {vol.Required(VALUE): cv.positive_int},
+        "async_set_value",
     )
 
     return True
 
 
-class CounterStorageCollection(collection.StorageCollection):
+class CounterStorageCollection(collection.DictStorageCollection):
     """Input storage based collection."""
 
-    CREATE_SCHEMA = vol.Schema(CREATE_FIELDS)
-    UPDATE_SCHEMA = vol.Schema(UPDATE_FIELDS)
+    CREATE_UPDATE_SCHEMA = vol.Schema(STORAGE_FIELDS)
 
     async def _process_create_data(self, data: dict) -> dict:
         """Validate the config is valid."""
-        return self.CREATE_SCHEMA(data)
+        return self.CREATE_UPDATE_SCHEMA(data)  # type: ignore[no-any-return]
 
     @callback
     def _get_suggested_id(self, info: dict) -> str:
         """Suggest an ID based on the config."""
-        return info[CONF_NAME]
+        return info[CONF_NAME]  # type: ignore[no-any-return]
 
-    async def _update_data(self, data: dict, update_data: dict) -> dict:
+    async def _update_data(self, item: dict, update_data: dict) -> dict:
         """Return a new updated data object."""
-        update_data = self.UPDATE_SCHEMA(update_data)
-        return {**data, **update_data}
+        update_data = self.CREATE_UPDATE_SCHEMA(update_data)
+        return {CONF_ID: item[CONF_ID]} | update_data
 
 
-class Counter(RestoreEntity):
+class Counter(collection.CollectionEntity, RestoreEntity):
     """Representation of a counter."""
 
     _attr_should_poll: bool = False
+    editable: bool
 
-    def __init__(self, config: dict) -> None:
+    def __init__(self, config: ConfigType) -> None:
         """Initialize a counter."""
-        self._config: dict = config
+        self._config: ConfigType = config
         self._state: int | None = config[CONF_INITIAL]
-        self.editable: bool = True
 
     @classmethod
-    def from_yaml(cls, config: dict) -> Counter:
+    def from_storage(cls, config: ConfigType) -> Self:
+        """Create counter instance from storage."""
+        counter = cls(config)
+        counter.editable = True
+        return counter
+
+    @classmethod
+    def from_yaml(cls, config: ConfigType) -> Self:
         """Create counter instance from yaml config."""
         counter = cls(config)
         counter.editable = False
@@ -221,9 +213,9 @@ class Counter(RestoreEntity):
     @property
     def unique_id(self) -> str | None:
         """Return unique id of the entity."""
-        return self._config[CONF_ID]
+        return self._config[CONF_ID]  # type: ignore[no-any-return]
 
-    def compute_next_state(self, state) -> int:
+    def compute_next_state(self, state: int | None) -> int | None:
         """Keep the state within the range of min/max values."""
         if self._config[CONF_MINIMUM] is not None:
             state = max(self._config[CONF_MINIMUM], state)
@@ -242,10 +234,6 @@ class Counter(RestoreEntity):
             and (state := await self.async_get_last_state()) is not None
         ):
             self._state = self.compute_next_state(int(state.state))
-            self._config[CONF_INITIAL] = state.attributes.get(ATTR_INITIAL)
-            self._config[CONF_MAXIMUM] = state.attributes.get(ATTR_MAXIMUM)
-            self._config[CONF_MINIMUM] = state.attributes.get(ATTR_MINIMUM)
-            self._config[CONF_STEP] = state.attributes.get(ATTR_STEP)
 
     @callback
     def async_decrement(self) -> None:
@@ -266,14 +254,27 @@ class Counter(RestoreEntity):
         self.async_write_ha_state()
 
     @callback
-    def async_configure(self, **kwargs) -> None:
-        """Change the counter's settings with a service."""
-        new_state = kwargs.pop(VALUE, self._state)
-        self._config = {**self._config, **kwargs}
-        self._state = self.compute_next_state(new_state)
+    def async_set_value(self, value: int) -> None:
+        """Set counter to value."""
+        if (maximum := self._config.get(CONF_MAXIMUM)) is not None and value > maximum:
+            raise ValueError(
+                f"Value {value} for {self.entity_id} exceeding the maximum value of {maximum}"
+            )
+
+        if (minimum := self._config.get(CONF_MINIMUM)) is not None and value < minimum:
+            raise ValueError(
+                f"Value {value} for {self.entity_id} exceeding the minimum value of {minimum}"
+            )
+
+        if (step := self._config.get(CONF_STEP)) is not None and value % step != 0:
+            raise ValueError(
+                f"Value {value} for {self.entity_id} is not a multiple of the step size {step}"
+            )
+
+        self._state = value
         self.async_write_ha_state()
 
-    async def async_update_config(self, config: dict) -> None:
+    async def async_update_config(self, config: ConfigType) -> None:
         """Change the counter's settings WS CRUD."""
         self._config = config
         self._state = self.compute_next_state(self._state)
