@@ -2,17 +2,14 @@
 from __future__ import annotations
 
 from collections.abc import Callable
-from dataclasses import dataclass
 from datetime import timedelta
-from enum import IntEnum
+from enum import IntFlag, StrEnum
 import functools as ft
 import logging
-from typing import Any, TypeVar, final
+from typing import TYPE_CHECKING, Any, ParamSpec, TypeVar, final
 
-from typing_extensions import ParamSpec
 import voluptuous as vol
 
-from homeassistant.backports.enum import StrEnum
 from homeassistant.config_entries import ConfigEntry
 from homeassistant.const import (
     SERVICE_CLOSE_COVER,
@@ -35,11 +32,22 @@ from homeassistant.helpers.config_validation import (  # noqa: F401
     PLATFORM_SCHEMA,
     PLATFORM_SCHEMA_BASE,
 )
+from homeassistant.helpers.deprecation import (
+    DeprecatedConstantEnum,
+    all_with_deprecated_constants,
+    check_if_deprecated_constant,
+    dir_with_deprecated_constants,
+)
 from homeassistant.helpers.entity import Entity, EntityDescription
 from homeassistant.helpers.entity_component import EntityComponent
 from homeassistant.helpers.typing import ConfigType
 from homeassistant.loader import bind_hass
 
+if TYPE_CHECKING:
+    from functools import cached_property
+else:
+    from homeassistant.backports.functools import cached_property
+
 _LOGGER = logging.getLogger(__name__)
 
 DOMAIN = "cover"
@@ -72,21 +80,37 @@ DEVICE_CLASSES_SCHEMA = vol.All(vol.Lower, vol.Coerce(CoverDeviceClass))
 # DEVICE_CLASS* below are deprecated as of 2021.12
 # use the CoverDeviceClass enum instead.
 DEVICE_CLASSES = [cls.value for cls in CoverDeviceClass]
-DEVICE_CLASS_AWNING = CoverDeviceClass.AWNING.value
-DEVICE_CLASS_BLIND = CoverDeviceClass.BLIND.value
-DEVICE_CLASS_CURTAIN = CoverDeviceClass.CURTAIN.value
-DEVICE_CLASS_DAMPER = CoverDeviceClass.DAMPER.value
-DEVICE_CLASS_DOOR = CoverDeviceClass.DOOR.value
-DEVICE_CLASS_GARAGE = CoverDeviceClass.GARAGE.value
-DEVICE_CLASS_GATE = CoverDeviceClass.GATE.value
-DEVICE_CLASS_SHADE = CoverDeviceClass.SHADE.value
-DEVICE_CLASS_SHUTTER = CoverDeviceClass.SHUTTER.value
-DEVICE_CLASS_WINDOW = CoverDeviceClass.WINDOW.value
+_DEPRECATED_DEVICE_CLASS_AWNING = DeprecatedConstantEnum(
+    CoverDeviceClass.AWNING, "2025.1"
+)
+_DEPRECATED_DEVICE_CLASS_BLIND = DeprecatedConstantEnum(
+    CoverDeviceClass.BLIND, "2025.1"
+)
+_DEPRECATED_DEVICE_CLASS_CURTAIN = DeprecatedConstantEnum(
+    CoverDeviceClass.CURTAIN, "2025.1"
+)
+_DEPRECATED_DEVICE_CLASS_DAMPER = DeprecatedConstantEnum(
+    CoverDeviceClass.DAMPER, "2025.1"
+)
+_DEPRECATED_DEVICE_CLASS_DOOR = DeprecatedConstantEnum(CoverDeviceClass.DOOR, "2025.1")
+_DEPRECATED_DEVICE_CLASS_GARAGE = DeprecatedConstantEnum(
+    CoverDeviceClass.GARAGE, "2025.1"
+)
+_DEPRECATED_DEVICE_CLASS_GATE = DeprecatedConstantEnum(CoverDeviceClass.GATE, "2025.1")
+_DEPRECATED_DEVICE_CLASS_SHADE = DeprecatedConstantEnum(
+    CoverDeviceClass.SHADE, "2025.1"
+)
+_DEPRECATED_DEVICE_CLASS_SHUTTER = DeprecatedConstantEnum(
+    CoverDeviceClass.SHUTTER, "2025.1"
+)
+_DEPRECATED_DEVICE_CLASS_WINDOW = DeprecatedConstantEnum(
+    CoverDeviceClass.WINDOW, "2025.1"
+)
 
 # mypy: disallow-any-generics
 
 
-class CoverEntityFeature(IntEnum):
+class CoverEntityFeature(IntFlag):
     """Supported features of the cover entity."""
 
     OPEN = 1
@@ -101,14 +125,24 @@ class CoverEntityFeature(IntEnum):
 
 # These SUPPORT_* constants are deprecated as of Home Assistant 2022.5.
 # Please use the CoverEntityFeature enum instead.
-SUPPORT_OPEN = 1
-SUPPORT_CLOSE = 2
-SUPPORT_SET_POSITION = 4
-SUPPORT_STOP = 8
-SUPPORT_OPEN_TILT = 16
-SUPPORT_CLOSE_TILT = 32
-SUPPORT_STOP_TILT = 64
-SUPPORT_SET_TILT_POSITION = 128
+_DEPRECATED_SUPPORT_OPEN = DeprecatedConstantEnum(CoverEntityFeature.OPEN, "2025.1")
+_DEPRECATED_SUPPORT_CLOSE = DeprecatedConstantEnum(CoverEntityFeature.CLOSE, "2025.1")
+_DEPRECATED_SUPPORT_SET_POSITION = DeprecatedConstantEnum(
+    CoverEntityFeature.SET_POSITION, "2025.1"
+)
+_DEPRECATED_SUPPORT_STOP = DeprecatedConstantEnum(CoverEntityFeature.STOP, "2025.1")
+_DEPRECATED_SUPPORT_OPEN_TILT = DeprecatedConstantEnum(
+    CoverEntityFeature.OPEN_TILT, "2025.1"
+)
+_DEPRECATED_SUPPORT_CLOSE_TILT = DeprecatedConstantEnum(
+    CoverEntityFeature.CLOSE_TILT, "2025.1"
+)
+_DEPRECATED_SUPPORT_STOP_TILT = DeprecatedConstantEnum(
+    CoverEntityFeature.STOP_TILT, "2025.1"
+)
+_DEPRECATED_SUPPORT_SET_TILT_POSITION = DeprecatedConstantEnum(
+    CoverEntityFeature.SET_TILT_POSITION, "2025.1"
+)
 
 ATTR_CURRENT_POSITION = "current_position"
 ATTR_CURRENT_TILT_POSITION = "current_tilt_position"
@@ -214,28 +248,38 @@ async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
     return await component.async_unload_entry(entry)
 
 
-@dataclass
-class CoverEntityDescription(EntityDescription):
+class CoverEntityDescription(EntityDescription, frozen_or_thawed=True):
     """A class that describes cover entities."""
 
-    device_class: CoverDeviceClass | str | None = None
+    device_class: CoverDeviceClass | None = None
 
 
-class CoverEntity(Entity):
+CACHED_PROPERTIES_WITH_ATTR_ = {
+    "current_cover_position",
+    "current_cover_tilt_position",
+    "device_class",
+    "is_opening",
+    "is_closing",
+    "is_closed",
+}
+
+
+class CoverEntity(Entity, cached_properties=CACHED_PROPERTIES_WITH_ATTR_):
     """Base class for cover entities."""
 
     entity_description: CoverEntityDescription
     _attr_current_cover_position: int | None = None
     _attr_current_cover_tilt_position: int | None = None
-    _attr_device_class: CoverDeviceClass | str | None
+    _attr_device_class: CoverDeviceClass | None
     _attr_is_closed: bool | None
     _attr_is_closing: bool | None = None
     _attr_is_opening: bool | None = None
     _attr_state: None = None
+    _attr_supported_features: CoverEntityFeature | None
 
     _cover_is_last_toggle_direction_open = True
 
-    @property
+    @cached_property
     def current_cover_position(self) -> int | None:
         """Return current position of cover.
 
@@ -243,7 +287,7 @@ class CoverEntity(Entity):
         """
         return self._attr_current_cover_position
 
-    @property
+    @cached_property
     def current_cover_tilt_position(self) -> int | None:
         """Return current position of cover tilt.
 
@@ -251,8 +295,8 @@ class CoverEntity(Entity):
         """
         return self._attr_current_cover_tilt_position
 
-    @property
-    def device_class(self) -> CoverDeviceClass | str | None:
+    @cached_property
+    def device_class(self) -> CoverDeviceClass | None:
         """Return the class of this entity."""
         if hasattr(self, "_attr_device_class"):
             return self._attr_device_class
@@ -291,10 +335,14 @@ class CoverEntity(Entity):
         return data
 
     @property
-    def supported_features(self) -> int:
+    def supported_features(self) -> CoverEntityFeature:
         """Flag supported features."""
-        if self._attr_supported_features is not None:
-            return self._attr_supported_features
+        if (features := self._attr_supported_features) is not None:
+            if type(features) is int:  # noqa: E721
+                new_features = CoverEntityFeature(features)
+                self._report_deprecated_supported_features_values(new_features)
+                return new_features
+            return features
 
         supported_features = (
             CoverEntityFeature.OPEN | CoverEntityFeature.CLOSE | CoverEntityFeature.STOP
@@ -313,17 +361,17 @@ class CoverEntity(Entity):
 
         return supported_features
 
-    @property
+    @cached_property
     def is_opening(self) -> bool | None:
         """Return if the cover is opening or not."""
         return self._attr_is_opening
 
-    @property
+    @cached_property
     def is_closing(self) -> bool | None:
         """Return if the cover is closing or not."""
         return self._attr_is_closing
 
-    @property
+    @cached_property
     def is_closed(self) -> bool | None:
         """Return if the cover is closed or not."""
         return self._attr_is_closed
@@ -433,7 +481,7 @@ class CoverEntity(Entity):
     def _get_toggle_function(
         self, fns: dict[str, Callable[_P, _R]]
     ) -> Callable[_P, _R]:
-        if CoverEntityFeature.STOP | self.supported_features and (
+        if self.supported_features & CoverEntityFeature.STOP and (
             self.is_closing or self.is_opening
         ):
             return fns["stop"]
@@ -442,3 +490,11 @@ class CoverEntity(Entity):
         if self._cover_is_last_toggle_direction_open:
             return fns["close"]
         return fns["open"]
+
+
+# These can be removed if no deprecated constant are in this module anymore
+__getattr__ = ft.partial(check_if_deprecated_constant, module_globals=globals())
+__dir__ = ft.partial(
+    dir_with_deprecated_constants, module_globals_keys=[*globals().keys()]
+)
+__all__ = all_with_deprecated_constants(globals())
