@@ -1,4 +1,6 @@
 """Support for DD-WRT routers."""
+from __future__ import annotations
+
 from http import HTTPStatus
 import logging
 import re
@@ -18,7 +20,9 @@ from homeassistant.const import (
     CONF_USERNAME,
     CONF_VERIFY_SSL,
 )
+from homeassistant.core import HomeAssistant
 import homeassistant.helpers.config_validation as cv
+from homeassistant.helpers.typing import ConfigType
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -42,7 +46,7 @@ PLATFORM_SCHEMA = PARENT_PLATFORM_SCHEMA.extend(
 )
 
 
-def get_scanner(hass, config):
+def get_scanner(hass: HomeAssistant, config: ConfigType) -> DdWrtDeviceScanner | None:
     """Validate the configuration and return a DD-WRT scanner."""
     try:
         return DdWrtDeviceScanner(config[DOMAIN])
@@ -51,7 +55,7 @@ def get_scanner(hass, config):
 
 
 class DdWrtDeviceScanner(DeviceScanner):
-    """This class queries a wireless router running DD-WRT firmware."""
+    """Class which queries a wireless router running DD-WRT firmware."""
 
     def __init__(self, config):
         """Initialize the DD-WRT scanner."""
@@ -67,8 +71,7 @@ class DdWrtDeviceScanner(DeviceScanner):
 
         # Test the router is accessible
         url = f"{self.protocol}://{self.host}/Status_Wireless.live.asp"
-        data = self.get_ddwrt_data(url)
-        if not data:
+        if not self.get_ddwrt_data(url):
             raise ConnectionError("Cannot connect to DD-Wrt router")
 
     def scan_devices(self):
@@ -82,9 +85,8 @@ class DdWrtDeviceScanner(DeviceScanner):
         # If not initialised and not already scanned and not found.
         if device not in self.mac2name:
             url = f"{self.protocol}://{self.host}/Status_Lan.live.asp"
-            data = self.get_ddwrt_data(url)
 
-            if not data:
+            if not (data := self.get_ddwrt_data(url)):
                 return None
 
             if not (dhcp_leases := data.get("dhcp_leases")):
@@ -115,9 +117,8 @@ class DdWrtDeviceScanner(DeviceScanner):
 
         endpoint = "Wireless" if self.wireless_only else "Lan"
         url = f"{self.protocol}://{self.host}/Status_{endpoint}.live.asp"
-        data = self.get_ddwrt_data(url)
 
-        if not data:
+        if not (data := self.get_ddwrt_data(url)):
             return False
 
         self.last_results = []
