@@ -6,33 +6,30 @@ from dataclasses import dataclass
 
 from pydeconz.models.event import EventType
 from pydeconz.models.scene import Scene as PydeconzScene
+from pydeconz.models.sensor.presence import Presence
 
 from homeassistant.components.button import (
     DOMAIN,
+    ButtonDeviceClass,
     ButtonEntity,
     ButtonEntityDescription,
 )
 from homeassistant.config_entries import ConfigEntry
+from homeassistant.const import EntityCategory
 from homeassistant.core import HomeAssistant, callback
-from homeassistant.helpers.entity import EntityCategory
 from homeassistant.helpers.entity_platform import AddEntitiesCallback
 
-from .deconz_device import DeconzSceneMixin
+from .deconz_device import DeconzDevice, DeconzSceneMixin
 from .gateway import DeconzGateway, get_gateway_from_config_entry
 
 
-@dataclass
-class DeconzButtonDescriptionMixin:
-    """Required values when describing deCONZ button entities."""
-
-    suffix: str
-    button_fn: str
-
-
-@dataclass
-class DeconzButtonDescription(ButtonEntityDescription, DeconzButtonDescriptionMixin):
+@dataclass(frozen=True, kw_only=True)
+class DeconzButtonDescription(ButtonEntityDescription):
     """Class describing deCONZ button entities."""
 
+    button_fn: str
+    suffix: str
+
 
 ENTITY_DESCRIPTIONS = {
     PydeconzScene: [
@@ -61,7 +58,7 @@ async def async_setup_entry(
         """Add scene button from deCONZ."""
         scene = gateway.api.scenes[scene_id]
         async_add_entities(
-            DeconzButton(scene, gateway, description)
+            DeconzSceneButton(scene, gateway, description)
             for description in ENTITY_DESCRIPTIONS.get(PydeconzScene, [])
         )
 
@@ -70,8 +67,20 @@ async def async_setup_entry(
         gateway.api.scenes,
     )
 
+    @callback
+    def async_add_presence_sensor(_: EventType, sensor_id: str) -> None:
+        """Add presence sensor reset button from deCONZ."""
+        sensor = gateway.api.sensors.presence[sensor_id]
+        if sensor.presence_event is not None:
+            async_add_entities([DeconzPresenceResetButton(sensor, gateway)])
 
-class DeconzButton(DeconzSceneMixin, ButtonEntity):
+    gateway.register_platform_add_device_callback(
+        async_add_presence_sensor,
+        gateway.api.sensors.presence,
+    )
+
+
+class DeconzSceneButton(DeconzSceneMixin, ButtonEntity):
     """Representation of a deCONZ button entity."""
 
     TYPE = DOMAIN
@@ -99,3 +108,22 @@ class DeconzButton(DeconzSceneMixin, ButtonEntity):
     def get_device_identifier(self) -> str:
         """Return a unique identifier for this scene."""
         return f"{super().get_device_identifier()}-{self.entity_description.key}"
+
+
+class DeconzPresenceResetButton(DeconzDevice[Presence], ButtonEntity):
+    """Representation of a deCONZ presence reset button entity."""
+
+    _name_suffix = "Reset Presence"
+    unique_id_suffix = "reset_presence"
+
+    _attr_entity_category = EntityCategory.CONFIG
+    _attr_device_class = ButtonDeviceClass.RESTART
+
+    TYPE = DOMAIN
+
+    async def async_press(self) -> None:
+        """Store reset presence state."""
+        await self.gateway.api.sensors.presence.set_config(
+            id=self._device.resource_id,
+            reset_presence=True,
+        )
