@@ -28,6 +28,7 @@ from .deconz_event import (
     DeconzAlarmEvent,
     DeconzEvent,
     DeconzPresenceEvent,
+    DeconzRelativeRotaryEvent,
 )
 from .gateway import DeconzGateway
 
@@ -127,6 +128,18 @@ HUE_TAP_REMOTE = {
     (CONF_SHORT_PRESS, CONF_BUTTON_4): {CONF_EVENT: 18},
 }
 
+HUE_WALL_REMOTE_MODEL = "RDM001"  # Hue wall switch
+HUE_WALL_REMOTE = {
+    (CONF_SHORT_PRESS, CONF_BUTTON_1): {CONF_EVENT: 1000},
+    (CONF_SHORT_RELEASE, CONF_BUTTON_1): {CONF_EVENT: 1002},
+    (CONF_LONG_PRESS, CONF_BUTTON_1): {CONF_EVENT: 1001},
+    (CONF_LONG_RELEASE, CONF_BUTTON_1): {CONF_EVENT: 1003},
+    (CONF_SHORT_PRESS, CONF_BUTTON_2): {CONF_EVENT: 2000},
+    (CONF_SHORT_RELEASE, CONF_BUTTON_2): {CONF_EVENT: 2002},
+    (CONF_LONG_PRESS, CONF_BUTTON_2): {CONF_EVENT: 2001},
+    (CONF_LONG_RELEASE, CONF_BUTTON_2): {CONF_EVENT: 2003},
+}
+
 FRIENDS_OF_HUE_SWITCH_MODEL = "FOHSWITCH"
 FRIENDS_OF_HUE_SWITCH = {
     (CONF_SHORT_PRESS, CONF_BUTTON_1): {CONF_EVENT: 1000},
@@ -157,18 +170,18 @@ FRIENDS_OF_HUE_SWITCH = {
 
 STYRBAR_REMOTE_MODEL = "Remote Control N2"
 STYRBAR_REMOTE = {
-    (CONF_SHORT_RELEASE, CONF_TURN_ON): {CONF_EVENT: 1002},
-    (CONF_LONG_PRESS, CONF_TURN_ON): {CONF_EVENT: 1001},
-    (CONF_LONG_RELEASE, CONF_TURN_ON): {CONF_EVENT: 1003},
-    (CONF_SHORT_RELEASE, CONF_DIM_UP): {CONF_EVENT: 2002},
-    (CONF_LONG_PRESS, CONF_DIM_UP): {CONF_EVENT: 2001},
-    (CONF_LONG_RELEASE, CONF_DIM_UP): {CONF_EVENT: 2003},
-    (CONF_SHORT_RELEASE, CONF_DIM_DOWN): {CONF_EVENT: 3002},
-    (CONF_LONG_PRESS, CONF_DIM_DOWN): {CONF_EVENT: 3001},
-    (CONF_LONG_RELEASE, CONF_DIM_DOWN): {CONF_EVENT: 3003},
-    (CONF_SHORT_RELEASE, CONF_TURN_OFF): {CONF_EVENT: 4002},
-    (CONF_LONG_PRESS, CONF_TURN_OFF): {CONF_EVENT: 4001},
-    (CONF_LONG_RELEASE, CONF_TURN_OFF): {CONF_EVENT: 4003},
+    (CONF_SHORT_RELEASE, CONF_DIM_UP): {CONF_EVENT: 1002},
+    (CONF_LONG_PRESS, CONF_DIM_UP): {CONF_EVENT: 1001},
+    (CONF_LONG_RELEASE, CONF_DIM_UP): {CONF_EVENT: 1003},
+    (CONF_SHORT_RELEASE, CONF_DIM_DOWN): {CONF_EVENT: 2002},
+    (CONF_LONG_PRESS, CONF_DIM_DOWN): {CONF_EVENT: 2001},
+    (CONF_LONG_RELEASE, CONF_DIM_DOWN): {CONF_EVENT: 2003},
+    (CONF_SHORT_RELEASE, CONF_LEFT): {CONF_EVENT: 3002},
+    (CONF_LONG_PRESS, CONF_LEFT): {CONF_EVENT: 3001},
+    (CONF_LONG_RELEASE, CONF_LEFT): {CONF_EVENT: 3003},
+    (CONF_SHORT_RELEASE, CONF_RIGHT): {CONF_EVENT: 4002},
+    (CONF_LONG_PRESS, CONF_RIGHT): {CONF_EVENT: 4001},
+    (CONF_LONG_RELEASE, CONF_RIGHT): {CONF_EVENT: 4003},
 }
 
 SYMFONISK_SOUND_CONTROLLER_MODEL = "SYMFONISK Sound Controller"
@@ -478,6 +491,7 @@ LEGRAND_ZGP_SCENE_SWITCH = {
 }
 
 LIDL_SILVERCREST_DOORBELL_MODEL = "HG06668"
+LIDL_SILVERCREST_DOORBELL_MODEL_2 = "TS0211"
 LIDL_SILVERCREST_DOORBELL = {
     (CONF_SHORT_PRESS, ""): {CONF_EVENT: 1002},
 }
@@ -537,6 +551,7 @@ BUSCH_JAEGER_REMOTE = {
 
 SONOFF_SNZB_01_1_MODEL = "WB01"
 SONOFF_SNZB_01_2_MODEL = "WB-01"
+SONOFF_SNZB_01P_MODEL = "SNZB-01P"
 SONOFF_SNZB_01_SWITCH = {
     (CONF_SHORT_RELEASE, CONF_BUTTON_1): {CONF_EVENT: 1002},
     (CONF_LONG_RELEASE, CONF_BUTTON_1): {CONF_EVENT: 1003},
@@ -581,6 +596,7 @@ REMOTES = {
     HUE_DIMMER_REMOTE_MODEL_GEN3: HUE_DIMMER_REMOTE,
     HUE_BUTTON_REMOTE_MODEL: HUE_BUTTON_REMOTE,
     HUE_TAP_REMOTE_MODEL: HUE_TAP_REMOTE,
+    HUE_WALL_REMOTE_MODEL: HUE_WALL_REMOTE,
     FRIENDS_OF_HUE_SWITCH_MODEL: FRIENDS_OF_HUE_SWITCH,
     STYRBAR_REMOTE_MODEL: STYRBAR_REMOTE,
     SYMFONISK_SOUND_CONTROLLER_MODEL: SYMFONISK_SOUND_CONTROLLER,
@@ -613,6 +629,7 @@ REMOTES = {
     LEGRAND_ZGP_TOGGLE_SWITCH_MODEL: LEGRAND_ZGP_TOGGLE_SWITCH,
     LEGRAND_ZGP_SCENE_SWITCH_MODEL: LEGRAND_ZGP_SCENE_SWITCH,
     LIDL_SILVERCREST_DOORBELL_MODEL: LIDL_SILVERCREST_DOORBELL,
+    LIDL_SILVERCREST_DOORBELL_MODEL_2: LIDL_SILVERCREST_DOORBELL,
     LIDL_SILVERCREST_BUTTON_REMOTE_MODEL: LIDL_SILVERCREST_BUTTON_REMOTE,
     LIGHTIFIY_FOUR_BUTTON_REMOTE_MODEL: LIGHTIFIY_FOUR_BUTTON_REMOTE,
     LIGHTIFIY_FOUR_BUTTON_REMOTE_4X_MODEL: LIGHTIFIY_FOUR_BUTTON_REMOTE,
@@ -625,6 +642,7 @@ REMOTES = {
     UBISYS_CONTROL_UNIT_C4_MODEL: UBISYS_CONTROL_UNIT_C4,
     SONOFF_SNZB_01_1_MODEL: SONOFF_SNZB_01_SWITCH,
     SONOFF_SNZB_01_2_MODEL: SONOFF_SNZB_01_SWITCH,
+    SONOFF_SNZB_01P_MODEL: SONOFF_SNZB_01_SWITCH,
 }
 
 TRIGGER_SCHEMA = DEVICE_TRIGGER_BASE_SCHEMA.extend(
@@ -635,7 +653,7 @@ TRIGGER_SCHEMA = DEVICE_TRIGGER_BASE_SCHEMA.extend(
 def _get_deconz_event_from_device(
     hass: HomeAssistant,
     device: dr.DeviceEntry,
-) -> DeconzAlarmEvent | DeconzEvent | DeconzPresenceEvent:
+) -> DeconzAlarmEvent | DeconzEvent | DeconzPresenceEvent | DeconzRelativeRotaryEvent:
     """Resolve deconz event from device."""
     gateways: dict[str, DeconzGateway] = hass.data.get(DOMAIN, {})
     for gateway in gateways.values():
@@ -724,7 +742,7 @@ async def async_get_triggers(
         return []
 
     triggers = []
-    for trigger, subtype in REMOTES[device.model].keys():
+    for trigger, subtype in REMOTES[device.model]:
         triggers.append(
             {
                 CONF_DEVICE_ID: device_id,
