@@ -7,10 +7,10 @@ from collections.abc import Callable
 from types import MappingProxyType
 from typing import TYPE_CHECKING, Any, cast
 
-import async_timeout
 from pydeconz import DeconzSession, errors
-from pydeconz.interfaces.api import APIItems, GroupedAPIItems
-from pydeconz.interfaces.groups import Groups
+from pydeconz.interfaces import sensors
+from pydeconz.interfaces.api_handlers import APIHandler, GroupedAPIHandler
+from pydeconz.interfaces.groups import GroupHandler
 from pydeconz.models.event import EventType
 
 from homeassistant.config_entries import SOURCE_HASSIO, ConfigEntry
@@ -40,7 +40,41 @@ from .const import (
 from .errors import AuthenticationRequired, CannotConnect
 
 if TYPE_CHECKING:
-    from .deconz_event import DeconzAlarmEvent, DeconzEvent
+    from .deconz_event import (
+        DeconzAlarmEvent,
+        DeconzEvent,
+        DeconzPresenceEvent,
+        DeconzRelativeRotaryEvent,
+    )
+
+SENSORS = (
+    sensors.SensorResourceManager,
+    sensors.AirPurifierHandler,
+    sensors.AirQualityHandler,
+    sensors.AlarmHandler,
+    sensors.AncillaryControlHandler,
+    sensors.BatteryHandler,
+    sensors.CarbonMonoxideHandler,
+    sensors.ConsumptionHandler,
+    sensors.DaylightHandler,
+    sensors.DoorLockHandler,
+    sensors.FireHandler,
+    sensors.GenericFlagHandler,
+    sensors.GenericStatusHandler,
+    sensors.HumidityHandler,
+    sensors.LightLevelHandler,
+    sensors.OpenCloseHandler,
+    sensors.PowerHandler,
+    sensors.PresenceHandler,
+    sensors.PressureHandler,
+    sensors.RelativeRotaryHandler,
+    sensors.SwitchHandler,
+    sensors.TemperatureHandler,
+    sensors.ThermostatHandler,
+    sensors.TimeHandler,
+    sensors.VibrationHandler,
+    sensors.WaterHandler,
+)
 
 
 class DeconzGateway:
@@ -60,14 +94,22 @@ class DeconzGateway:
         self.ignore_state_updates = False
 
         self.signal_reachable = f"deconz-reachable-{config_entry.entry_id}"
-        self.signal_reload_clip_sensors = f"deconz_reload_clip_{config_entry.entry_id}"
 
         self.deconz_ids: dict[str, str] = {}
         self.entities: dict[str, set[str]] = {}
-        self.events: list[DeconzAlarmEvent | DeconzEvent] = []
-        self.ignored_devices: set[tuple[Callable[[EventType, str], None], str]] = set()
+        self.events: list[
+            DeconzAlarmEvent
+            | DeconzEvent
+            | DeconzPresenceEvent
+            | DeconzRelativeRotaryEvent
+        ] = []
+        self.clip_sensors: set[tuple[Callable[[EventType, str], None], str]] = set()
         self.deconz_groups: set[tuple[Callable[[EventType, str], None], str]] = set()
+        self.ignored_devices: set[tuple[Callable[[EventType, str], None], str]] = set()
 
+        self.option_allow_clip_sensor = self.config_entry.options.get(
+            CONF_ALLOW_CLIP_SENSOR, DEFAULT_ALLOW_CLIP_SENSOR
+        )
         self.option_allow_deconz_groups = config_entry.options.get(
             CONF_ALLOW_DECONZ_GROUPS, DEFAULT_ALLOW_DECONZ_GROUPS
         )
@@ -90,20 +132,12 @@ class DeconzGateway:
         """Gateway which is used with deCONZ services without defining id."""
         return cast(bool, self.config_entry.options[CONF_MASTER_GATEWAY])
 
-    # Options
-
-    @property
-    def option_allow_clip_sensor(self) -> bool:
-        """Allow loading clip sensor from gateway."""
-        return self.config_entry.options.get(
-            CONF_ALLOW_CLIP_SENSOR, DEFAULT_ALLOW_CLIP_SENSOR
-        )
-
     @callback
     def register_platform_add_device_callback(
         self,
         add_device_callback: Callable[[EventType, str], None],
-        deconz_device_interface: APIItems | GroupedAPIItems,
+        deconz_device_interface: APIHandler | GroupedAPIHandler,
+        always_ignore_clip_sensors: bool = False,
     ) -> None:
         """Wrap add_device_callback to check allow_new_devices option."""
 
@@ -123,11 +157,18 @@ class DeconzGateway:
                 self.ignored_devices.add((async_add_device, device_id))
                 return
 
-            if isinstance(deconz_device_interface, Groups):
+            if isinstance(deconz_device_interface, GroupHandler):
                 self.deconz_groups.add((async_add_device, device_id))
                 if not self.option_allow_deconz_groups:
                     return
 
+            if isinstance(deconz_device_interface, SENSORS):
+                device = deconz_device_interface[device_id]
+                if device.type.startswith("CLIP") and not always_ignore_clip_sensors:
+                    self.clip_sensors.add((async_add_device, device_id))
+                    if not self.option_allow_clip_sensor:
+                        return
+
             add_device_callback(EventType.ADDED, device_id)
 
         self.config_entry.async_on_unload(
@@ -137,7 +178,7 @@ class DeconzGateway:
             )
         )
 
-        for device_id in deconz_device_interface:
+        for device_id in sorted(deconz_device_interface, key=int):
             async_add_device(EventType.ADDED, device_id)
 
         initializing = False
@@ -193,9 +234,15 @@ class DeconzGateway:
     ) -> None:
         """Handle signals of config entry being updated.
 
-        This is a static method because a class method (bound method), can not be used with weak references.
-        Causes for this is either discovery updating host address or config entry options changing.
+        This is a static method because a class method (bound method),
+        cannot be used with weak references.
+        Causes for this is either discovery updating host address or
+        config entry options changing.
         """
+        if entry.entry_id not in hass.data[DECONZ_DOMAIN]:
+            # A race condition can occur if multiple config entries are
+            # unloaded in parallel
+            return
         gateway = get_gateway_from_config_entry(hass, entry)
 
         if gateway.api.host != gateway.host:
@@ -212,15 +259,20 @@ class DeconzGateway:
 
         # Allow CLIP sensors
 
-        if self.option_allow_clip_sensor:
-            async_dispatcher_send(self.hass, self.signal_reload_clip_sensors)
-
-        else:
-            deconz_ids += [
-                sensor.deconz_id
-                for sensor in self.api.sensors.values()
-                if sensor.type.startswith("CLIP")
-            ]
+        option_allow_clip_sensor = self.config_entry.options.get(
+            CONF_ALLOW_CLIP_SENSOR, DEFAULT_ALLOW_CLIP_SENSOR
+        )
+        if option_allow_clip_sensor != self.option_allow_clip_sensor:
+            self.option_allow_clip_sensor = option_allow_clip_sensor
+            if option_allow_clip_sensor:
+                for add_device, device_id in self.clip_sensors:
+                    add_device(EventType.ADDED, device_id)
+            else:
+                deconz_ids += [
+                    sensor.deconz_id
+                    for sensor in self.api.sensors.values()
+                    if sensor.type.startswith("CLIP")
+                ]
 
         # Allow Groups
 
@@ -300,7 +352,7 @@ async def get_deconz_session(
         config[CONF_API_KEY],
     )
     try:
-        async with async_timeout.timeout(10):
+        async with asyncio.timeout(10):
             await deconz_session.refresh_state()
         return deconz_session
 
