@@ -18,21 +18,22 @@ from homeassistant.components.number import (
     NumberEntityDescription,
 )
 from homeassistant.config_entries import ConfigEntry
+from homeassistant.const import EntityCategory
 from homeassistant.core import HomeAssistant, callback
-from homeassistant.helpers.entity import EntityCategory
 from homeassistant.helpers.entity_platform import AddEntitiesCallback
 import homeassistant.helpers.entity_registry as er
 
 from .const import DOMAIN as DECONZ_DOMAIN
 from .deconz_device import DeconzDevice
 from .gateway import DeconzGateway, get_gateway_from_config_entry
+from .util import serial_from_unique_id
 
 T = TypeVar("T", Presence, PydeconzSensorBase)
 
 
-@dataclass
-class DeconzNumberDescriptionMixin(Generic[T]):
-    """Required values when describing deCONZ number entities."""
+@dataclass(frozen=True, kw_only=True)
+class DeconzNumberDescription(Generic[T], NumberEntityDescription):
+    """Class describing deCONZ number entities."""
 
     instance_check: type[T]
     name_suffix: str
@@ -41,11 +42,6 @@ class DeconzNumberDescriptionMixin(Generic[T]):
     value_fn: Callable[[T], float | None]
 
 
-@dataclass
-class DeconzNumberDescription(NumberEntityDescription, DeconzNumberDescriptionMixin[T]):
-    """Class describing deCONZ number entities."""
-
-
 ENTITY_DESCRIPTIONS: tuple[DeconzNumberDescription, ...] = (
     DeconzNumberDescription[Presence](
         key="delay",
@@ -88,7 +84,7 @@ def async_update_unique_id(
     if ent_reg.async_get_entity_id(DOMAIN, DECONZ_DOMAIN, new_unique_id):
         return
 
-    unique_id = f'{unique_id.split("-", 1)[0]}-{description.key}'
+    unique_id = f"{serial_from_unique_id(unique_id)}-{description.key}"
     if entity_id := ent_reg.async_get_entity_id(DOMAIN, DECONZ_DOMAIN, unique_id):
         ent_reg.async_update_entity(entity_id, new_unique_id=new_unique_id)
 
