@@ -7,20 +7,18 @@ from homeassistant.components.binary_sensor import (
 )
 from homeassistant.config_entries import ConfigEntry
 from homeassistant.core import HomeAssistant
-from homeassistant.helpers.entity import DeviceInfo
+from homeassistant.helpers.device_registry import DeviceInfo
 from homeassistant.helpers.entity_platform import AddEntitiesCallback
-from homeassistant.helpers.typing import ConfigType, DiscoveryInfoType
 
 from . import DOMAIN
 
 
-async def async_setup_platform(
+async def async_setup_entry(
     hass: HomeAssistant,
-    config: ConfigType,
+    config_entry: ConfigEntry,
     async_add_entities: AddEntitiesCallback,
-    discovery_info: DiscoveryInfoType | None = None,
 ) -> None:
-    """Set up the Demo binary sensor platform."""
+    """Set up the demo binary sensor platform."""
     async_add_entities(
         [
             DemoBinarySensor(
@@ -36,42 +34,30 @@ async def async_setup_platform(
     )
 
 
-async def async_setup_entry(
-    hass: HomeAssistant,
-    config_entry: ConfigEntry,
-    async_add_entities: AddEntitiesCallback,
-) -> None:
-    """Set up the Demo config entry."""
-    await async_setup_platform(hass, {}, async_add_entities)
-
-
 class DemoBinarySensor(BinarySensorEntity):
     """representation of a Demo binary sensor."""
 
+    _attr_has_entity_name = True
+    _attr_name = None
     _attr_should_poll = False
 
     def __init__(
         self,
         unique_id: str,
-        name: str,
+        device_name: str,
         state: bool,
         device_class: BinarySensorDeviceClass,
     ) -> None:
         """Initialize the demo sensor."""
         self._unique_id = unique_id
-        self._attr_name = name
         self._state = state
-        self._sensor_type = device_class
-
-    @property
-    def device_info(self) -> DeviceInfo:
-        """Return device info."""
-        return DeviceInfo(
+        self._attr_device_class = device_class
+        self._attr_device_info = DeviceInfo(
             identifiers={
                 # Serial numbers are unique identifiers within a specific domain
                 (DOMAIN, self.unique_id)
             },
-            name=self.name,
+            name=device_name,
         )
 
     @property
@@ -79,11 +65,6 @@ class DemoBinarySensor(BinarySensorEntity):
         """Return the unique id."""
         return self._unique_id
 
-    @property
-    def device_class(self) -> BinarySensorDeviceClass:
-        """Return the class of this sensor."""
-        return self._sensor_type
-
     @property
     def is_on(self) -> bool:
         """Return true if the binary sensor is on."""
