@@ -6,7 +6,7 @@ from collections.abc import Awaitable, Callable, Coroutine, Sequence
 import contextlib
 from datetime import datetime, timedelta
 import functools
-from typing import Any, TypeVar
+from typing import Any, Concatenate, ParamSpec, TypeVar
 
 from async_upnp_client.client import UpnpService, UpnpStateVariable
 from async_upnp_client.const import NotificationSubType
@@ -14,7 +14,6 @@ from async_upnp_client.exceptions import UpnpError, UpnpResponseError
 from async_upnp_client.profiles.dlna import DmrDevice, PlayMode, TransportState
 from async_upnp_client.utils import async_get_local_ip
 from didl_lite import didl_lite
-from typing_extensions import Concatenate, ParamSpec
 
 from homeassistant import config_entries
 from homeassistant.components import media_source, ssdp
@@ -28,9 +27,9 @@ from homeassistant.components.media_player import (
     RepeatMode,
     async_process_play_media_url,
 )
-from homeassistant.const import CONF_DEVICE_ID, CONF_TYPE, CONF_URL
+from homeassistant.const import CONF_DEVICE_ID, CONF_MAC, CONF_TYPE, CONF_URL
 from homeassistant.core import HomeAssistant
-from homeassistant.helpers import device_registry, entity_registry
+from homeassistant.helpers import device_registry as dr, entity_registry as er
 from homeassistant.helpers.entity_platform import AddEntitiesCallback
 
 from .const import (
@@ -38,7 +37,6 @@ from .const import (
     CONF_CALLBACK_URL_OVERRIDE,
     CONF_LISTEN_PORT,
     CONF_POLL_AVAILABILITY,
-    DOMAIN,
     LOGGER as _LOGGER,
     MEDIA_METADATA_DIDL,
     MEDIA_TYPE_MAP,
@@ -57,7 +55,7 @@ _P = ParamSpec("_P")
 
 
 def catch_request_errors(
-    func: Callable[Concatenate[_DlnaDmrEntityT, _P], Awaitable[_R]]
+    func: Callable[Concatenate[_DlnaDmrEntityT, _P], Awaitable[_R]],
 ) -> Callable[Concatenate[_DlnaDmrEntityT, _P], Coroutine[Any, Any, _R | None]]:
     """Catch UpnpError errors."""
 
@@ -98,6 +96,7 @@ async def async_setup_entry(
         event_callback_url=entry.options.get(CONF_CALLBACK_URL_OVERRIDE),
         poll_availability=entry.options.get(CONF_POLL_AVAILABILITY, False),
         location=entry.data[CONF_URL],
+        mac_address=entry.data.get(CONF_MAC),
         browse_unfiltered=entry.options.get(CONF_BROWSE_UNFILTERED, False),
     )
 
@@ -130,6 +129,9 @@ class DlnaDmrEntity(MediaPlayerEntity):
     # determine whether further device polling is required.
     _attr_should_poll = True
 
+    # Name of the current sound mode, not supported by DLNA
+    _attr_sound_mode = None
+
     def __init__(
         self,
         udn: str,
@@ -139,6 +141,7 @@ class DlnaDmrEntity(MediaPlayerEntity):
         event_callback_url: str | None,
         poll_availability: bool,
         location: str,
+        mac_address: str | None,
         browse_unfiltered: bool,
     ) -> None:
         """Initialize DLNA DMR entity."""
@@ -148,6 +151,7 @@ class DlnaDmrEntity(MediaPlayerEntity):
         self._event_addr = EventListenAddr(None, event_port, event_callback_url)
         self.poll_availability = poll_availability
         self.location = location
+        self.mac_address = mac_address
         self.browse_unfiltered = browse_unfiltered
         self._device_lock = asyncio.Lock()
 
@@ -272,6 +276,11 @@ class DlnaDmrEntity(MediaPlayerEntity):
         self.poll_availability = entry.options.get(CONF_POLL_AVAILABILITY, False)
         self.browse_unfiltered = entry.options.get(CONF_BROWSE_UNFILTERED, False)
 
+        new_mac_address = entry.data.get(CONF_MAC)
+        if new_mac_address != self.mac_address:
+            self.mac_address = new_mac_address
+            self._update_device_registry(set_mac=True)
+
         new_port = entry.options.get(CONF_LISTEN_PORT) or 0
         new_callback_url = entry.options.get(CONF_CALLBACK_URL_OVERRIDE)
 
@@ -338,35 +347,49 @@ class DlnaDmrEntity(MediaPlayerEntity):
                 _LOGGER.debug("Error while subscribing during device connect: %r", err)
                 raise
 
-        if (
-            not self.registry_entry
-            or not self.registry_entry.config_entry_id
-            or self.registry_entry.device_id
-        ):
-            return
+        self._update_device_registry()
+
+    def _update_device_registry(self, set_mac: bool = False) -> None:
+        """Update the device registry with new information about the DMR."""
+        if not self._device:
+            return  # Can't get all the required information without a connection
+
+        if not self.registry_entry or not self.registry_entry.config_entry_id:
+            return  # No config registry entry to link to
+
+        if self.registry_entry.device_id and not set_mac:
+            return  # No new information
+
+        connections = set()
+        # Connections based on the root device's UDN, and the DMR embedded
+        # device's UDN. They may be the same, if the DMR is the root device.
+        connections.add(
+            (
+                dr.CONNECTION_UPNP,
+                self._device.profile_device.root_device.udn,
+            )
+        )
+        connections.add((dr.CONNECTION_UPNP, self._device.udn))
+
+        if self.mac_address:
+            # Connection based on MAC address, if known
+            connections.add(
+                # Device MAC is obtained from the config entry, which uses getmac
+                (dr.CONNECTION_NETWORK_MAC, self.mac_address)
+            )
 
         # Create linked HA DeviceEntry now the information is known.
-        dev_reg = device_registry.async_get(self.hass)
+        dev_reg = dr.async_get(self.hass)
         device_entry = dev_reg.async_get_or_create(
             config_entry_id=self.registry_entry.config_entry_id,
-            # Connections are based on the root device's UDN, and the DMR
-            # embedded device's UDN. They may be the same, if the DMR is the
-            # root device.
-            connections={
-                (
-                    device_registry.CONNECTION_UPNP,
-                    self._device.profile_device.root_device.udn,
-                ),
-                (device_registry.CONNECTION_UPNP, self._device.udn),
-            },
-            identifiers={(DOMAIN, self.unique_id)},
+            connections=connections,
             default_manufacturer=self._device.manufacturer,
             default_model=self._device.model_name,
             default_name=self._device.name,
         )
 
         # Update entity registry to link to the device
-        ent_reg = entity_registry.async_get(self.hass)
+        ent_reg = er.async_get(self.hass)
         ent_reg.async_get_or_create(
             self.registry_entry.domain,
             self.registry_entry.platform,
@@ -430,10 +453,9 @@ class DlnaDmrEntity(MediaPlayerEntity):
             for state_variable in state_variables:
                 # Force a state refresh when player begins or pauses playback
                 # to update the position info.
-                if (
-                    state_variable.name == "TransportState"
-                    and state_variable.value
-                    in (TransportState.PLAYING, TransportState.PAUSED_PLAYBACK)
+                if state_variable.name == "TransportState" and state_variable.value in (
+                    TransportState.PLAYING,
+                    TransportState.PAUSED_PLAYBACK,
                 ):
                     force_refresh = True
 
@@ -478,15 +500,15 @@ class DlnaDmrEntity(MediaPlayerEntity):
         return MediaPlayerState.IDLE
 
     @property
-    def supported_features(self) -> int:
+    def supported_features(self) -> MediaPlayerEntityFeature:
         """Flag media player features that are supported at this moment.
 
         Supported features may change as the device enters different states.
         """
         if not self._device:
-            return 0
+            return MediaPlayerEntityFeature(0)
 
-        supported_features = 0
+        supported_features = MediaPlayerEntityFeature(0)
 
         if self._device.has_volume_level:
             supported_features |= MediaPlayerEntityFeature.VOLUME_SET
@@ -725,11 +747,6 @@ class DlnaDmrEntity(MediaPlayerEntity):
             "Couldn't find a suitable mode for shuffle=%s, repeat=%s", shuffle, repeat
         )
 
-    @property
-    def sound_mode(self) -> str | None:
-        """Name of the current sound mode, not supported by DLNA."""
-        return None
-
     @property
     def sound_mode_list(self) -> list[str] | None:
         """List of available sound modes."""
@@ -745,7 +762,7 @@ class DlnaDmrEntity(MediaPlayerEntity):
 
     async def async_browse_media(
         self,
-        media_content_type: str | None = None,
+        media_content_type: MediaType | str | None = None,
         media_content_id: str | None = None,
     ) -> BrowseMedia:
         """Implement the websocket media browsing helper.
