@@ -4,29 +4,30 @@ from __future__ import annotations
 import asyncio
 from collections.abc import Callable, Coroutine
 from dataclasses import dataclass
+from enum import StrEnum
 import functools
 from typing import Any, TypeVar, cast
 
-from async_upnp_client import UpnpEventHandler, UpnpFactory, UpnpRequester
 from async_upnp_client.aiohttp import AiohttpSessionRequester
+from async_upnp_client.client import UpnpRequester
+from async_upnp_client.client_factory import UpnpFactory
 from async_upnp_client.const import NotificationSubType
 from async_upnp_client.exceptions import UpnpActionError, UpnpConnectionError, UpnpError
 from async_upnp_client.profiles.dlna import ContentDirectoryErrorCode, DmsDevice
 from didl_lite import didl_lite
 
-from homeassistant.backports.enum import StrEnum
+from homeassistant.backports.functools import cached_property
 from homeassistant.components import ssdp
-from homeassistant.components.media_player.const import MEDIA_CLASS_DIRECTORY
-from homeassistant.components.media_player.errors import BrowseError
+from homeassistant.components.media_player import BrowseError, MediaClass
 from homeassistant.components.media_source.error import Unresolvable
 from homeassistant.components.media_source.models import BrowseMediaSource, PlayMedia
 from homeassistant.config_entries import ConfigEntry
 from homeassistant.const import CONF_DEVICE_ID, CONF_URL
 from homeassistant.core import HomeAssistant, callback
 from homeassistant.helpers import aiohttp_client
-from homeassistant.util import slugify
 
 from .const import (
+    CONF_SOURCE_ID,
     DLNA_BROWSE_FILTER,
     DLNA_PATH_FILTER,
     DLNA_RESOLVE_FILTER,
@@ -42,17 +43,15 @@ from .const import (
 )
 
 _DlnaDmsDeviceMethod = TypeVar("_DlnaDmsDeviceMethod", bound="DmsDeviceSource")
-_RetType = TypeVar("_RetType")
+_R = TypeVar("_R")
 
 
 class DlnaDmsData:
     """Storage class for domain global data."""
 
     hass: HomeAssistant
-    lock: asyncio.Lock
     requester: UpnpRequester
     upnp_factory: UpnpFactory
-    event_handler: UpnpEventHandler
     devices: dict[str, DmsDeviceSource]  # Indexed by config_entry.unique_id
     sources: dict[str, DmsDeviceSource]  # Indexed by source_id
 
@@ -62,69 +61,32 @@ class DlnaDmsData:
     ) -> None:
         """Initialize global data."""
         self.hass = hass
-        self.lock = asyncio.Lock()
         session = aiohttp_client.async_get_clientsession(hass, verify_ssl=False)
         self.requester = AiohttpSessionRequester(session, with_sleep=True)
         self.upnp_factory = UpnpFactory(self.requester, non_strict=True)
-        # NOTE: event_handler is not actually used, and is only created to
-        # satisfy the DmsDevice.__init__ signature
-        self.event_handler = UpnpEventHandler("", self.requester)
         self.devices = {}
         self.sources = {}
 
     async def async_setup_entry(self, config_entry: ConfigEntry) -> bool:
         """Create a DMS device connection from a config entry."""
         assert config_entry.unique_id
-        async with self.lock:
-            source_id = self._generate_source_id(config_entry.title)
-            device = DmsDeviceSource(self.hass, config_entry, source_id)
-            self.devices[config_entry.unique_id] = device
-            self.sources[device.source_id] = device
-
-        # Update the device when the associated config entry is modified
-        config_entry.async_on_unload(
-            config_entry.add_update_listener(self.async_update_entry)
-        )
-
+        device = DmsDeviceSource(self.hass, config_entry)
+        self.devices[config_entry.unique_id] = device
+        # source_id must be unique, which generate_source_id should guarantee.
+        # Ensure this is the case, for debugging purposes.
+        assert device.source_id not in self.sources
+        self.sources[device.source_id] = device
         await device.async_added_to_hass()
         return True
 
     async def async_unload_entry(self, config_entry: ConfigEntry) -> bool:
         """Unload a config entry and disconnect the corresponding DMS device."""
         assert config_entry.unique_id
-        async with self.lock:
-            device = self.devices.pop(config_entry.unique_id)
-            del self.sources[device.source_id]
+        device = self.devices.pop(config_entry.unique_id)
+        del self.sources[device.source_id]
         await device.async_will_remove_from_hass()
         return True
 
-    async def async_update_entry(
-        self, hass: HomeAssistant, config_entry: ConfigEntry
-    ) -> None:
-        """Update a DMS device when the config entry changes."""
-        assert config_entry.unique_id
-        async with self.lock:
-            device = self.devices[config_entry.unique_id]
-            # Update the source_id to match the new name
-            del self.sources[device.source_id]
-            device.source_id = self._generate_source_id(config_entry.title)
-            self.sources[device.source_id] = device
-
-    def _generate_source_id(self, name: str) -> str:
-        """Generate a unique source ID.
-
-        Caller should hold self.lock when calling this method.
-        """
-        source_id_base = slugify(name)
-        if source_id_base not in self.sources:
-            return source_id_base
-
-        tries = 1
-        while (suggested_source_id := f"{source_id_base}_{tries}") in self.sources:
-            tries += 1
-
-        return suggested_source_id
-
 
 @callback
 def get_domain_data(hass: HomeAssistant) -> DlnaDmsData:
@@ -162,12 +124,12 @@ class ActionError(DlnaDmsDeviceError):
 
 
 def catch_request_errors(
-    func: Callable[[_DlnaDmsDeviceMethod, str], Coroutine[Any, Any, _RetType]]
-) -> Callable[[_DlnaDmsDeviceMethod, str], Coroutine[Any, Any, _RetType]]:
+    func: Callable[[_DlnaDmsDeviceMethod, str], Coroutine[Any, Any, _R]],
+) -> Callable[[_DlnaDmsDeviceMethod, str], Coroutine[Any, Any, _R]]:
     """Catch UpnpError errors."""
 
     @functools.wraps(func)
-    async def wrapper(self: _DlnaDmsDeviceMethod, req_param: str) -> _RetType:
+    async def wrapper(self: _DlnaDmsDeviceMethod, req_param: str) -> _R:
         """Catch UpnpError errors and check availability before and after request."""
         if not self.available:
             LOGGER.warning("Device disappeared when trying to call %s", func.__name__)
@@ -200,12 +162,6 @@ def catch_request_errors(
 class DmsDeviceSource:
     """DMS Device wrapper, providing media files as a media_source."""
 
-    hass: HomeAssistant
-    config_entry: ConfigEntry
-
-    # Unique slug used for media-source URIs
-    source_id: str
-
     # Last known URL for the device, used when adding this wrapper to hass to
     # try to connect before SSDP has rediscovered it, or when SSDP discovery
     # fails.
@@ -220,13 +176,10 @@ class DmsDeviceSource:
     # Track BOOTID in SSDP advertisements for device changes
     _bootid: int | None = None
 
-    def __init__(
-        self, hass: HomeAssistant, config_entry: ConfigEntry, source_id: str
-    ) -> None:
+    def __init__(self, hass: HomeAssistant, config_entry: ConfigEntry) -> None:
         """Initialize a DMS Source."""
         self.hass = hass
         self.config_entry = config_entry
-        self.source_id = source_id
         self.location = self.config_entry.data[CONF_URL]
         self._device_lock = asyncio.Lock()
 
@@ -334,16 +287,13 @@ class DmsDeviceSource:
     async def device_connect(self) -> None:
         """Connect to the device now that it's available."""
         LOGGER.debug("Connecting to device at %s", self.location)
+        assert self.location
 
         async with self._device_lock:
             if self._device:
                 LOGGER.debug("Trying to connect when device already connected")
                 return
 
-            if not self.location:
-                LOGGER.debug("Not connecting because location is not known")
-                return
-
             domain_data = get_domain_data(self.hass)
 
             # Connect to the base UPNP device
@@ -352,7 +302,7 @@ class DmsDeviceSource:
             )
 
             # Create profile wrapper
-            self._device = DmsDevice(upnp_device, domain_data.event_handler)
+            self._device = DmsDevice(upnp_device, event_handler=None)
 
             # Update state variables. We don't care if they change, so this is
             # only done once, here.
@@ -382,7 +332,7 @@ class DmsDeviceSource:
     @property
     def usn(self) -> str:
         """Get the USN (Unique Service Name) for the wrapped UPnP device end-point."""
-        return self.config_entry.data[CONF_DEVICE_ID]
+        return self.config_entry.data[CONF_DEVICE_ID]  # type: ignore[no-any-return]
 
     @property
     def udn(self) -> str:
@@ -394,13 +344,15 @@ class DmsDeviceSource:
         """Return a name for the media server."""
         return self.config_entry.title
 
+    @property
+    def source_id(self) -> str:
+        """Return a unique ID (slug) for this source for people to use in URLs."""
+        return self.config_entry.data[CONF_SOURCE_ID]  # type: ignore[no-any-return]
+
     @property
     def icon(self) -> str | None:
         """Return an URL to an icon for the media server."""
-        if not self._device:
-            return None
-
-        return self._device.icon
+        return self._device.icon if self._device else None
 
     # MediaSource methods
 
@@ -409,6 +361,8 @@ class DmsDeviceSource:
         LOGGER.debug("async_resolve_media(%s)", identifier)
         action, parameters = _parse_identifier(identifier)
 
+        assert action is not None, f"Invalid identifier: {identifier}"
+
         if action is Action.OBJECT:
             return await self.async_resolve_object(parameters)
 
@@ -416,11 +370,8 @@ class DmsDeviceSource:
             object_id = await self.async_resolve_path(parameters)
             return await self.async_resolve_object(object_id)
 
-        if action is Action.SEARCH:
-            return await self.async_resolve_search(parameters)
-
-        LOGGER.debug("Invalid identifier %s", identifier)
-        raise Unresolvable(f"Invalid identifier {identifier}")
+        assert action is Action.SEARCH
+        return await self.async_resolve_search(parameters)
 
     async def async_browse_media(self, identifier: str | None) -> BrowseMediaSource:
         """Browse media."""
@@ -567,7 +518,7 @@ class DmsDeviceSource:
         media_source = BrowseMediaSource(
             domain=DOMAIN,
             identifier=self._make_identifier(Action.SEARCH, query),
-            media_class=MEDIA_CLASS_DIRECTORY,
+            media_class=MediaClass.DIRECTORY,
             media_content_type="",
             title="Search results",
             can_play=False,
@@ -575,9 +526,6 @@ class DmsDeviceSource:
             children=children,
         )
 
-        if media_source.children:
-            media_source.calculate_children_class()
-
         return media_source
 
     def _didl_to_play_media(self, item: didl_lite.DidlObject) -> DidlPlayMedia:
@@ -646,9 +594,6 @@ class DmsDeviceSource:
             thumbnail=self._didl_thumbnail_url(item),
         )
 
-        if media_source.children:
-            media_source.calculate_children_class()
-
         return media_source
 
     def _didl_thumbnail_url(self, item: didl_lite.DidlObject) -> str | None:
@@ -675,8 +620,7 @@ class DmsDeviceSource:
         """Make an identifier for BrowseMediaSource."""
         return f"{self.source_id}/{action}{object_id}"
 
-    @property  # type: ignore
-    @functools.cache
+    @cached_property
     def _sort_criteria(self) -> list[str]:
         """Return criteria to be used for sorting results.
 
