@@ -1,4 +1,6 @@
 """Support for the DOODS service."""
+from __future__ import annotations
+
 import io
 import logging
 import os
@@ -10,16 +12,22 @@ import voluptuous as vol
 
 from homeassistant.components.image_processing import (
     CONF_CONFIDENCE,
-    CONF_ENTITY_ID,
-    CONF_NAME,
-    CONF_SOURCE,
     PLATFORM_SCHEMA,
     ImageProcessingEntity,
 )
-from homeassistant.const import CONF_COVERS, CONF_TIMEOUT, CONF_URL
-from homeassistant.core import split_entity_id
+from homeassistant.const import (
+    CONF_COVERS,
+    CONF_ENTITY_ID,
+    CONF_NAME,
+    CONF_SOURCE,
+    CONF_TIMEOUT,
+    CONF_URL,
+)
+from homeassistant.core import HomeAssistant, split_entity_id
 from homeassistant.helpers import template
 import homeassistant.helpers.config_validation as cv
+from homeassistant.helpers.entity_platform import AddEntitiesCallback
+from homeassistant.helpers.typing import ConfigType, DiscoveryInfoType
 from homeassistant.util.pil import draw_box
 
 _LOGGER = logging.getLogger(__name__)
@@ -73,7 +81,12 @@ PLATFORM_SCHEMA = PLATFORM_SCHEMA.extend(
 )
 
 
-def setup_platform(hass, config, add_entities, discovery_info=None):
+def setup_platform(
+    hass: HomeAssistant,
+    config: ConfigType,
+    add_entities: AddEntitiesCallback,
+    discovery_info: DiscoveryInfoType | None = None,
+) -> None:
     """Set up the Doods client."""
     url = config[CONF_URL]
     auth_key = config[CONF_AUTH_KEY]
@@ -154,8 +167,7 @@ class Doods(ImageProcessingEntity):
                     continue
 
                 # If label confidence is not specified, use global confidence
-                label_confidence = label.get(CONF_CONFIDENCE)
-                if not label_confidence:
+                if not (label_confidence := label.get(CONF_CONFIDENCE)):
                     label_confidence = confidence
                 if label_name not in dconfig or dconfig[label_name] > label_confidence:
                     dconfig[label_name] = label_confidence
@@ -187,8 +199,7 @@ class Doods(ImageProcessingEntity):
         # Handle global detection area
         self._area = [0, 0, 1, 1]
         self._covers = True
-        area_config = config.get(CONF_AREA)
-        if area_config:
+        if area_config := config.get(CONF_AREA):
             self._area = [
                 area_config[CONF_TOP],
                 area_config[CONF_LEFT],
@@ -244,7 +255,6 @@ class Doods(ImageProcessingEntity):
             )
 
         for label, values in matches.items():
-
             # Draw custom label regions/areas
             if label in self._label_areas and self._label_areas[label] != [0, 0, 1, 1]:
                 box_label = f"{label.capitalize()} Detection Area"
@@ -272,8 +282,7 @@ class Doods(ImageProcessingEntity):
 
         for path in paths:
             _LOGGER.info("Saving results image to %s", path)
-            if not os.path.exists(os.path.dirname(path)):
-                os.makedirs(os.path.dirname(path), exist_ok=True)
+            os.makedirs(os.path.dirname(path), exist_ok=True)
             img.save(path)
 
     def process_image(self, image):
@@ -287,7 +296,10 @@ class Doods(ImageProcessingEntity):
 
         if self._aspect and abs((img_width / img_height) - self._aspect) > 0.1:
             _LOGGER.debug(
-                "The image aspect: %s and the detector aspect: %s differ by more than 0.1",
+                (
+                    "The image aspect: %s and the detector aspect: %s differ by more"
+                    " than 0.1"
+                ),
                 (img_width / img_height),
                 self._aspect,
             )
@@ -338,14 +350,13 @@ class Doods(ImageProcessingEntity):
                     or boxes[3] > self._area[3]
                 ):
                     continue
-            else:
-                if (
-                    boxes[0] > self._area[2]
-                    or boxes[1] > self._area[3]
-                    or boxes[2] < self._area[0]
-                    or boxes[3] < self._area[1]
-                ):
-                    continue
+            elif (
+                boxes[0] > self._area[2]
+                or boxes[1] > self._area[3]
+                or boxes[2] < self._area[0]
+                or boxes[3] < self._area[1]
+            ):
+                continue
 
             # Exclude matches outside label specific area definition
             if self._label_areas.get(label):
@@ -357,14 +368,13 @@ class Doods(ImageProcessingEntity):
                         or boxes[3] > self._label_areas[label][3]
                     ):
                         continue
-                else:
-                    if (
-                        boxes[0] > self._label_areas[label][2]
-                        or boxes[1] > self._label_areas[label][3]
-                        or boxes[2] < self._label_areas[label][0]
-                        or boxes[3] < self._label_areas[label][1]
-                    ):
-                        continue
+                elif (
+                    boxes[0] > self._label_areas[label][2]
+                    or boxes[1] > self._label_areas[label][3]
+                    or boxes[2] < self._label_areas[label][0]
+                    or boxes[3] < self._label_areas[label][1]
+                ):
+                    continue
 
             if label not in matches:
                 matches[label] = []
@@ -382,6 +392,10 @@ class Doods(ImageProcessingEntity):
                 else:
                     paths.append(path_template)
             self._save_image(image, matches, paths)
+        else:
+            _LOGGER.debug(
+                "Not saving image(s), no detections found or no output file configured"
+            )
 
         self._matches = matches
         self._total_matches = total_matches
