@@ -6,9 +6,12 @@ from functools import partial
 import os
 from typing import Any
 
-from async_timeout import timeout
 from dsmr_parser import obis_references as obis_ref
 from dsmr_parser.clients.protocol import create_dsmr_reader, create_tcp_dsmr_reader
+from dsmr_parser.clients.rfxtrx_protocol import (
+    create_rfxtrx_dsmr_reader,
+    create_rfxtrx_tcp_dsmr_reader,
+)
 from dsmr_parser.objects import DSMRObject
 import serial
 import serial.tools.list_ports
@@ -16,7 +19,7 @@ import voluptuous as vol
 
 from homeassistant import config_entries, core, exceptions
 from homeassistant.config_entries import ConfigEntry
-from homeassistant.const import CONF_HOST, CONF_PORT, CONF_TYPE
+from homeassistant.const import CONF_HOST, CONF_PORT, CONF_PROTOCOL, CONF_TYPE
 from homeassistant.core import callback
 from homeassistant.data_entry_flow import FlowResult
 
@@ -27,8 +30,10 @@ from .const import (
     CONF_TIME_BETWEEN_UPDATE,
     DEFAULT_TIME_BETWEEN_UPDATE,
     DOMAIN,
+    DSMR_PROTOCOL,
     DSMR_VERSIONS,
     LOGGER,
+    RFXTRX_DSMR_PROTOCOL,
 )
 
 CONF_MANUAL_PATH = "Enter Manually"
@@ -37,13 +42,18 @@ CONF_MANUAL_PATH = "Enter Manually"
 class DSMRConnection:
     """Test the connection to DSMR and receive telegram to read serial ids."""
 
-    def __init__(self, host: str | None, port: int, dsmr_version: str) -> None:
+    def __init__(
+        self, host: str | None, port: int, dsmr_version: str, protocol: str
+    ) -> None:
         """Initialize."""
         self._host = host
         self._port = port
         self._dsmr_version = dsmr_version
+        self._protocol = protocol
         self._telegram: dict[str, DSMRObject] = {}
         self._equipment_identifier = obis_ref.EQUIPMENT_IDENTIFIER
+        if dsmr_version == "5B":
+            self._equipment_identifier = obis_ref.BELGIUM_EQUIPMENT_IDENTIFIER
         if dsmr_version == "5L":
             self._equipment_identifier = obis_ref.LUXEMBOURG_EQUIPMENT_IDENTIFIER
         if dsmr_version == "Q3D":
@@ -78,16 +88,24 @@ class DSMRConnection:
                 transport.close()
 
         if self._host is None:
+            if self._protocol == DSMR_PROTOCOL:
+                create_reader = create_dsmr_reader
+            else:
+                create_reader = create_rfxtrx_dsmr_reader
             reader_factory = partial(
-                create_dsmr_reader,
+                create_reader,
                 self._port,
                 self._dsmr_version,
                 update_telegram,
                 loop=hass.loop,
             )
         else:
+            if self._protocol == DSMR_PROTOCOL:
+                create_reader = create_tcp_dsmr_reader
+            else:
+                create_reader = create_rfxtrx_tcp_dsmr_reader
             reader_factory = partial(
-                create_tcp_dsmr_reader,
+                create_reader,
                 self._host,
                 self._port,
                 self._dsmr_version,
@@ -97,13 +115,13 @@ class DSMRConnection:
 
         try:
             transport, protocol = await asyncio.create_task(reader_factory())
-        except (serial.serialutil.SerialException, OSError):
+        except (serial.SerialException, OSError):
             LOGGER.exception("Error connecting to DSMR")
             return False
 
         if transport:
             try:
-                async with timeout(30):
+                async with asyncio.timeout(30):
                     await protocol.wait_closed()
             except asyncio.TimeoutError:
                 # Timeout (no data received), close transport and return True (if telegram is empty, will result in CannotCommunicate error)
@@ -113,10 +131,15 @@ class DSMRConnection:
 
 
 async def _validate_dsmr_connection(
-    hass: core.HomeAssistant, data: dict[str, Any]
+    hass: core.HomeAssistant, data: dict[str, Any], protocol: str
 ) -> dict[str, str | None]:
     """Validate the user input allows us to connect."""
-    conn = DSMRConnection(data.get(CONF_HOST), data[CONF_PORT], data[CONF_DSMR_VERSION])
+    conn = DSMRConnection(
+        data.get(CONF_HOST),
+        data[CONF_PORT],
+        data[CONF_DSMR_VERSION],
+        protocol,
+    )
 
     if not await conn.validate_connect(hass):
         raise CannotConnect
@@ -260,9 +283,14 @@ class DSMRFlowHandler(config_entries.ConfigFlow, domain=DOMAIN):
         data = input_data
 
         try:
-            info = await _validate_dsmr_connection(self.hass, data)
+            try:
+                protocol = DSMR_PROTOCOL
+                info = await _validate_dsmr_connection(self.hass, data, protocol)
+            except CannotCommunicate:
+                protocol = RFXTRX_DSMR_PROTOCOL
+                info = await _validate_dsmr_connection(self.hass, data, protocol)
 
-            data = {**data, **info}
+            data = {**data, **info, CONF_PROTOCOL: protocol}
 
             if info[CONF_SERIAL_ID]:
                 await self.async_set_unique_id(info[CONF_SERIAL_ID])
