@@ -1,9 +1,10 @@
 """Support for Fibaro lights."""
 from __future__ import annotations
 
-import asyncio
 from contextlib import suppress
-from functools import partial
+from typing import Any
+
+from pyfibaro.fibaro_device import DeviceModel
 
 from homeassistant.components.light import (
     ATTR_BRIGHTNESS,
@@ -20,7 +21,7 @@ from homeassistant.const import Platform
 from homeassistant.core import HomeAssistant
 from homeassistant.helpers.entity_platform import AddEntitiesCallback
 
-from . import FIBARO_DEVICES, FibaroDevice
+from . import FibaroController, FibaroDevice
 from .const import DOMAIN
 
 PARALLEL_UPDATES = 2
@@ -53,13 +54,9 @@ async def async_setup_entry(
     async_add_entities: AddEntitiesCallback,
 ) -> None:
     """Perform the setup for Fibaro controller devices."""
+    controller: FibaroController = hass.data[DOMAIN][entry.entry_id]
     async_add_entities(
-        [
-            FibaroLight(device)
-            for device in hass.data[DOMAIN][entry.entry_id][FIBARO_DEVICES][
-                Platform.LIGHT
-            ]
-        ],
+        [FibaroLight(device) for device in controller.fibaro_devices[Platform.LIGHT]],
         True,
     )
 
@@ -67,16 +64,14 @@ async def async_setup_entry(
 class FibaroLight(FibaroDevice, LightEntity):
     """Representation of a Fibaro Light, including dimmable."""
 
-    def __init__(self, fibaro_device):
+    def __init__(self, fibaro_device: DeviceModel) -> None:
         """Initialize the light."""
-        self._update_lock = asyncio.Lock()
-
         supports_color = (
             "color" in fibaro_device.properties
             or "colorComponents" in fibaro_device.properties
             or "RGB" in fibaro_device.type
             or "rgb" in fibaro_device.type
-            or "color" in fibaro_device.baseType
+            or "color" in fibaro_device.base_type
         ) and (
             "setColor" in fibaro_device.actions
             or "setColorComponents" in fibaro_device.actions
@@ -87,7 +82,7 @@ class FibaroLight(FibaroDevice, LightEntity):
             or "rgbw" in fibaro_device.type
         )
         supports_dimming = (
-            "levelChange" in fibaro_device.interfaces
+            fibaro_device.has_interface("levelChange")
             and "setValue" in fibaro_device.actions
         )
 
@@ -107,13 +102,8 @@ class FibaroLight(FibaroDevice, LightEntity):
         super().__init__(fibaro_device)
         self.entity_id = ENTITY_ID_FORMAT.format(self.ha_id)
 
-    async def async_turn_on(self, **kwargs):
+    def turn_on(self, **kwargs: Any) -> None:
         """Turn the light on."""
-        async with self._update_lock:
-            await self.hass.async_add_executor_job(partial(self._turn_on, **kwargs))
-
-    def _turn_on(self, **kwargs):
-        """Really turn the light on."""
         if ATTR_BRIGHTNESS in kwargs:
             self._attr_brightness = kwargs[ATTR_BRIGHTNESS]
             self.set_level(scaleto99(self._attr_brightness))
@@ -121,26 +111,23 @@ class FibaroLight(FibaroDevice, LightEntity):
 
         if ATTR_RGB_COLOR in kwargs:
             # Update based on parameters
-            self._attr_rgb_color = kwargs[ATTR_RGB_COLOR]
-            self.call_set_color(*self._attr_rgb_color, 0)
+            rgb = kwargs[ATTR_RGB_COLOR]
+            self._attr_rgb_color = rgb
+            self.call_set_color(int(rgb[0]), int(rgb[1]), int(rgb[2]), 0)
             return
 
         if ATTR_RGBW_COLOR in kwargs:
             # Update based on parameters
-            self._attr_rgbw_color = kwargs[ATTR_RGBW_COLOR]
-            self.call_set_color(*self._attr_rgbw_color)
+            rgbw = kwargs[ATTR_RGBW_COLOR]
+            self._attr_rgbw_color = rgbw
+            self.call_set_color(int(rgbw[0]), int(rgbw[1]), int(rgbw[2]), int(rgbw[3]))
             return
 
         # The simplest case is left for last. No dimming, just switch on
         self.call_turn_on()
 
-    async def async_turn_off(self, **kwargs):
+    def turn_off(self, **kwargs: Any) -> None:
         """Turn the light off."""
-        async with self._update_lock:
-            await self.hass.async_add_executor_job(partial(self._turn_off, **kwargs))
-
-    def _turn_off(self, **kwargs):
-        """Really turn the light off."""
         self.call_turn_off()
 
     @property
@@ -152,45 +139,38 @@ class FibaroLight(FibaroDevice, LightEntity):
 
         JSON for HC2 uses always string, HC3 uses int for integers.
         """
-        props = self.fibaro_device.properties
         if self.current_binary_state:
             return True
-        with suppress(ValueError, TypeError):
-            if "brightness" in props and int(props.brightness) != 0:
+        with suppress(TypeError):
+            if self.fibaro_device.brightness != 0:
                 return True
-        with suppress(ValueError, TypeError):
-            if "currentProgram" in props and int(props.currentProgram) != 0:
+        with suppress(TypeError):
+            if self.fibaro_device.current_program != 0:
                 return True
-        with suppress(ValueError, TypeError):
-            if "currentProgramID" in props and int(props.currentProgramID) != 0:
+        with suppress(TypeError):
+            if self.fibaro_device.current_program_id != 0:
                 return True
 
         return False
 
-    async def async_update(self):
+    def update(self) -> None:
         """Update the state."""
-        async with self._update_lock:
-            await self.hass.async_add_executor_job(self._update)
-
-    def _update(self):
-        """Really update the state."""
+        super().update()
         # Brightness handling
         if brightness_supported(self.supported_color_modes):
-            self._attr_brightness = scaleto255(int(self.fibaro_device.properties.value))
+            self._attr_brightness = scaleto255(self.fibaro_device.value.int_value())
 
         # Color handling
         if (
             color_supported(self.supported_color_modes)
-            and "color" in self.fibaro_device.properties
-            and "," in self.fibaro_device.properties.color
+            and self.fibaro_device.color.has_color
         ):
             # Fibaro communicates the color as an 'R, G, B, W' string
-            rgbw_s = self.fibaro_device.properties.color
-            if rgbw_s == "0,0,0,0" and "lastColorSet" in self.fibaro_device.properties:
-                rgbw_s = self.fibaro_device.properties.lastColorSet
-            rgbw_list = [int(i) for i in rgbw_s.split(",")][:4]
+            rgbw = self.fibaro_device.color.rgbw_color
+            if rgbw == (0, 0, 0, 0) and self.fibaro_device.last_color_set.has_color:
+                rgbw = self.fibaro_device.last_color_set.rgbw_color
 
             if self._attr_color_mode == ColorMode.RGB:
-                self._attr_rgb_color = tuple(rgbw_list[:3])
+                self._attr_rgb_color = rgbw[:3]
             else:
-                self._attr_rgbw_color = tuple(rgbw_list)
+                self._attr_rgbw_color = rgbw
