@@ -1,10 +1,12 @@
-"""
-Flux for Home-Assistant.
+"""Flux for Home-Assistant.
 
 The idea was taken from https://github.com/KpaBap/hue-flux/
 """
+from __future__ import annotations
+
 import datetime
 import logging
+from typing import Any
 
 import voluptuous as vol
 
@@ -13,7 +15,6 @@ from homeassistant.components.light import (
     ATTR_COLOR_TEMP,
     ATTR_RGB_COLOR,
     ATTR_TRANSITION,
-    ATTR_WHITE_VALUE,
     ATTR_XY_COLOR,
     DOMAIN as LIGHT_DOMAIN,
     VALID_TRANSITION,
@@ -32,9 +33,12 @@ from homeassistant.const import (
     SUN_EVENT_SUNRISE,
     SUN_EVENT_SUNSET,
 )
+from homeassistant.core import HomeAssistant, ServiceCall
 from homeassistant.helpers import config_validation as cv, event
+from homeassistant.helpers.entity_platform import AddEntitiesCallback
 from homeassistant.helpers.restore_state import RestoreEntity
 from homeassistant.helpers.sun import get_astral_event_date
+from homeassistant.helpers.typing import ConfigType, DiscoveryInfoType
 from homeassistant.util import slugify
 from homeassistant.util.color import (
     color_RGB_to_xy_brightness,
@@ -96,7 +100,6 @@ async def async_set_lights_xy(hass, lights, x_val, y_val, brightness, transition
                 service_data[ATTR_XY_COLOR] = [x_val, y_val]
             if brightness is not None:
                 service_data[ATTR_BRIGHTNESS] = brightness
-                service_data[ATTR_WHITE_VALUE] = brightness
             if transition is not None:
                 service_data[ATTR_TRANSITION] = transition
             await hass.services.async_call(LIGHT_DOMAIN, SERVICE_TURN_ON, service_data)
@@ -128,7 +131,12 @@ async def async_set_lights_rgb(hass, lights, rgb, transition):
             await hass.services.async_call(LIGHT_DOMAIN, SERVICE_TURN_ON, service_data)
 
 
-async def async_setup_platform(hass, config, async_add_entities, discovery_info=None):
+async def async_setup_platform(
+    hass: HomeAssistant,
+    config: ConfigType,
+    async_add_entities: AddEntitiesCallback,
+    discovery_info: DiscoveryInfoType | None = None,
+) -> None:
     """Set up the Flux switches."""
     name = config.get(CONF_NAME)
     lights = config.get(CONF_LIGHTS)
@@ -159,7 +167,7 @@ async def async_setup_platform(hass, config, async_add_entities, discovery_info=
     )
     async_add_entities([flux])
 
-    async def async_update(call=None):
+    async def async_update(call: ServiceCall | None = None) -> None:
         """Update lights."""
         await flux.async_flux_update()
 
@@ -212,13 +220,19 @@ class FluxSwitch(SwitchEntity, RestoreEntity):
         """Return true if switch is on."""
         return self.unsub_tracker is not None
 
-    async def async_added_to_hass(self):
+    async def async_added_to_hass(self) -> None:
         """Call when entity about to be added to hass."""
         last_state = await self.async_get_last_state()
         if last_state and last_state.state == STATE_ON:
             await self.async_turn_on()
 
-    async def async_turn_on(self, **kwargs):
+    async def async_will_remove_from_hass(self) -> None:
+        """Run when entity will be removed from hass."""
+        if self.unsub_tracker:
+            self.unsub_tracker()
+        return await super().async_will_remove_from_hass()
+
+    async def async_turn_on(self, **kwargs: Any) -> None:
         """Turn on flux."""
         if self.is_on:
             return
@@ -234,7 +248,7 @@ class FluxSwitch(SwitchEntity, RestoreEntity):
 
         self.async_write_ha_state()
 
-    async def async_turn_off(self, **kwargs):
+    async def async_turn_off(self, **kwargs: Any) -> None:
         """Turn off flux."""
         if self.is_on:
             self.unsub_tracker()
@@ -307,8 +321,10 @@ class FluxSwitch(SwitchEntity, RestoreEntity):
                 self.hass, self._lights, x_val, y_val, brightness, self._transition
             )
             _LOGGER.debug(
-                "Lights updated to x:%s y:%s brightness:%s, %s%% "
-                "of %s cycle complete at %s",
+                (
+                    "Lights updated to x:%s y:%s brightness:%s, %s%% "
+                    "of %s cycle complete at %s"
+                ),
                 x_val,
                 y_val,
                 brightness,
@@ -332,8 +348,10 @@ class FluxSwitch(SwitchEntity, RestoreEntity):
                 self.hass, self._lights, mired, brightness, self._transition
             )
             _LOGGER.debug(
-                "Lights updated to mired:%s brightness:%s, %s%% "
-                "of %s cycle complete at %s",
+                (
+                    "Lights updated to mired:%s brightness:%s, %s%% "
+                    "of %s cycle complete at %s"
+                ),
                 mired,
                 brightness,
                 round(percentage_complete * 100),
