@@ -6,6 +6,7 @@ from dataclasses import dataclass, field
 from datetime import datetime, timedelta
 from functools import partial
 import logging
+import re
 from types import MappingProxyType
 from typing import Any, TypedDict, cast
 
@@ -19,6 +20,7 @@ from fritzconnection.core.exceptions import (
 from fritzconnection.lib.fritzhosts import FritzHosts
 from fritzconnection.lib.fritzstatus import FritzStatus
 from fritzconnection.lib.fritzwlan import DEFAULT_PASSWORD_LENGTH, FritzGuestWLAN
+import xmltodict
 
 from homeassistant.components.device_tracker import (
     CONF_CONSIDER_HOME,
@@ -34,8 +36,10 @@ from homeassistant.helpers import (
     entity_registry as er,
     update_coordinator,
 )
-from homeassistant.helpers.dispatcher import dispatcher_send
-from homeassistant.helpers.entity import DeviceInfo
+from homeassistant.helpers.device_registry import DeviceInfo
+from homeassistant.helpers.dispatcher import async_dispatcher_send
+from homeassistant.helpers.entity import EntityDescription
+from homeassistant.helpers.typing import StateType
 from homeassistant.util import dt as dt_util
 
 from .const import (
@@ -127,6 +131,36 @@ class Interface(TypedDict):
     type: str
 
 
+HostAttributes = TypedDict(
+    "HostAttributes",
+    {
+        "Index": int,
+        "IPAddress": str,
+        "MACAddress": str,
+        "Active": bool,
+        "HostName": str,
+        "InterfaceType": str,
+        "X_AVM-DE_Port": int,
+        "X_AVM-DE_Speed": int,
+        "X_AVM-DE_UpdateAvailable": bool,
+        "X_AVM-DE_UpdateSuccessful": str,
+        "X_AVM-DE_InfoURL": str | None,
+        "X_AVM-DE_MACAddressList": str | None,
+        "X_AVM-DE_Model": str | None,
+        "X_AVM-DE_URL": str | None,
+        "X_AVM-DE_Guest": bool,
+        "X_AVM-DE_RequestClient": str,
+        "X_AVM-DE_VPN": bool,
+        "X_AVM-DE_WANAccess": str,
+        "X_AVM-DE_Disallow": bool,
+        "X_AVM-DE_IsMeshable": str,
+        "X_AVM-DE_Priority": str,
+        "X_AVM-DE_FriendlyName": str,
+        "X_AVM-DE_FriendlyNameIsWriteable": str,
+    },
+)
+
+
 class HostInfo(TypedDict):
     """FRITZ!Box host info class."""
 
@@ -136,7 +170,16 @@ class HostInfo(TypedDict):
     status: bool
 
 
-class FritzBoxTools(update_coordinator.DataUpdateCoordinator):
+class UpdateCoordinatorDataType(TypedDict):
+    """Update coordinator data type."""
+
+    call_deflections: dict[int, dict]
+    entity_states: dict[str, StateType | bool]
+
+
+class FritzBoxTools(
+    update_coordinator.DataUpdateCoordinator[UpdateCoordinatorDataType]
+):  # pylint: disable=hass-enforce-coordinator-module
     """FritzBoxTools class."""
 
     def __init__(
@@ -170,11 +213,15 @@ class FritzBoxTools(update_coordinator.DataUpdateCoordinator):
         self.password = password
         self.port = port
         self.username = username
+        self.has_call_deflections: bool = False
         self._model: str | None = None
         self._current_firmware: str | None = None
         self._latest_firmware: str | None = None
         self._update_available: bool = False
         self._release_url: str | None = None
+        self._entity_update_functions: dict[
+            str, Callable[[FritzStatus, StateType], Any]
+        ] = {}
 
     async def async_setup(
         self, options: MappingProxyType[str, Any] | None = None
@@ -223,7 +270,12 @@ class FritzBoxTools(update_coordinator.DataUpdateCoordinator):
             self._unique_id = info.serial_number
 
         self._model = info.model_name
-        self._current_firmware = info.software_version
+        if (
+            version_normalized := re.search(r"^\d+\.[0]?(.*)", info.software_version)
+        ) is not None:
+            self._current_firmware = version_normalized.group(1)
+        else:
+            self._current_firmware = info.software_version
 
         (
             self._update_available,
@@ -237,13 +289,51 @@ class FritzBoxTools(update_coordinator.DataUpdateCoordinator):
             )
             self.device_is_router = self.fritz_status.has_wan_enabled
 
-    async def _async_update_data(self) -> None:
+        self.has_call_deflections = "X_AVM-DE_OnTel1" in self.connection.services
+
+    def register_entity_updates(
+        self, key: str, update_fn: Callable[[FritzStatus, StateType], Any]
+    ) -> Callable[[], None]:
+        """Register an entity to be updated by coordinator."""
+
+        def unregister_entity_updates() -> None:
+            """Unregister an entity to be updated by coordinator."""
+            if key in self._entity_update_functions:
+                _LOGGER.debug("unregister entity %s from updates", key)
+                self._entity_update_functions.pop(key)
+
+        if key not in self._entity_update_functions:
+            _LOGGER.debug("register entity %s for updates", key)
+            self._entity_update_functions[key] = update_fn
+        return unregister_entity_updates
+
+    async def _async_update_data(self) -> UpdateCoordinatorDataType:
         """Update FritzboxTools data."""
+        entity_data: UpdateCoordinatorDataType = {
+            "call_deflections": {},
+            "entity_states": {},
+        }
         try:
             await self.async_scan_devices()
+            for key in list(self._entity_update_functions):
+                _LOGGER.debug("update entity %s", key)
+                entity_data["entity_states"][
+                    key
+                ] = await self.hass.async_add_executor_job(
+                    self._entity_update_functions[key],
+                    self.fritz_status,
+                    self.data["entity_states"].get(key),
+                )
+            if self.has_call_deflections:
+                entity_data[
+                    "call_deflections"
+                ] = await self.async_update_call_deflections()
         except FRITZ_EXCEPTIONS as ex:
             raise update_coordinator.UpdateFailed(ex) from ex
 
+        _LOGGER.debug("enity_data: %s", entity_data)
+        return entity_data
+
     @property
     def unique_id(self) -> str:
         """Return unique id."""
@@ -302,14 +392,86 @@ class FritzBoxTools(update_coordinator.DataUpdateCoordinator):
         """Event specific per FRITZ!Box entry to signal updates in devices."""
         return f"{DOMAIN}-device-update-{self._unique_id}"
 
-    def _update_hosts_info(self) -> list[HostInfo]:
-        """Retrieve latest hosts information from the FRITZ!Box."""
+    async def _async_get_wan_access(self, ip_address: str) -> bool | None:
+        """Get WAN access rule for given IP address."""
         try:
-            return self.fritz_hosts.get_hosts_info()  # type: ignore [no-any-return]
+            wan_access = await self.hass.async_add_executor_job(
+                partial(
+                    self.connection.call_action,
+                    "X_AVM-DE_HostFilter:1",
+                    "GetWANAccessByIP",
+                    NewIPv4Address=ip_address,
+                )
+            )
+            return not wan_access.get("NewDisallow")
+        except FRITZ_EXCEPTIONS as ex:
+            _LOGGER.debug(
+                (
+                    "could not get WAN access rule for client device with IP '%s',"
+                    " error: %s"
+                ),
+                ip_address,
+                ex,
+            )
+            return None
+
+    async def _async_update_hosts_info(self) -> dict[str, Device]:
+        """Retrieve latest hosts information from the FRITZ!Box."""
+        hosts_attributes: list[HostAttributes] = []
+        hosts_info: list[HostInfo] = []
+        try:
+            try:
+                hosts_attributes = await self.hass.async_add_executor_job(
+                    self.fritz_hosts.get_hosts_attributes
+                )
+            except FritzActionError:
+                hosts_info = await self.hass.async_add_executor_job(
+                    self.fritz_hosts.get_hosts_info
+                )
         except Exception as ex:  # pylint: disable=[broad-except]
             if not self.hass.is_stopping:
                 raise HomeAssistantError("Error refreshing hosts info") from ex
-        return []
+
+        hosts: dict[str, Device] = {}
+        if hosts_attributes:
+            for attributes in hosts_attributes:
+                if not attributes.get("MACAddress"):
+                    continue
+
+                if (wan_access := attributes.get("X_AVM-DE_WANAccess")) is not None:
+                    wan_access_result = "granted" in wan_access
+                else:
+                    wan_access_result = None
+
+                hosts[attributes["MACAddress"]] = Device(
+                    name=attributes["HostName"],
+                    connected=attributes["Active"],
+                    connected_to="",
+                    connection_type="",
+                    ip_address=attributes["IPAddress"],
+                    ssid=None,
+                    wan_access=wan_access_result,
+                )
+        else:
+            for info in hosts_info:
+                if not info.get("mac"):
+                    continue
+
+                if info["ip"]:
+                    wan_access_result = await self._async_get_wan_access(info["ip"])
+                else:
+                    wan_access_result = None
+
+                hosts[info["mac"]] = Device(
+                    name=info["name"],
+                    connected=info["status"],
+                    connected_to="",
+                    connection_type="",
+                    ip_address=info["ip"],
+                    ssid=None,
+                    wan_access=wan_access_result,
+                )
+        return hosts
 
     def _update_device_info(self) -> tuple[bool, str | None, str | None]:
         """Retrieve latest device information from the FRITZ!Box."""
@@ -318,25 +480,26 @@ class FritzBoxTools(update_coordinator.DataUpdateCoordinator):
         release_url = info.get("NewX_AVM-DE_InfoURL")
         return bool(version), version, release_url
 
-    def _get_wan_access(self, ip_address: str) -> bool | None:
-        """Get WAN access rule for given IP address."""
-        try:
-            return not self.connection.call_action(
-                "X_AVM-DE_HostFilter:1",
-                "GetWANAccessByIP",
-                NewIPv4Address=ip_address,
-            ).get("NewDisallow")
-        except FRITZ_EXCEPTIONS as ex:
-            _LOGGER.debug(
-                "could not get WAN access rule for client device with IP '%s', error: %s",
-                ip_address,
-                ex,
-            )
-            return None
+    async def _async_update_device_info(self) -> tuple[bool, str | None, str | None]:
+        """Retrieve latest device information from the FRITZ!Box."""
+        return await self.hass.async_add_executor_job(self._update_device_info)
 
-    async def async_scan_devices(self, now: datetime | None = None) -> None:
-        """Wrap up FritzboxTools class scan."""
-        await self.hass.async_add_executor_job(self.scan_devices, now)
+    async def async_update_call_deflections(
+        self,
+    ) -> dict[int, dict[str, Any]]:
+        """Call GetDeflections action from X_AVM-DE_OnTel service."""
+        raw_data = await self.hass.async_add_executor_job(
+            partial(self.connection.call_action, "X_AVM-DE_OnTel1", "GetDeflections")
+        )
+        if not raw_data:
+            return {}
+
+        xml_data = xmltodict.parse(raw_data["NewDeflectionList"])
+        if xml_data.get("List") and (items := xml_data["List"].get("Item")) is not None:
+            if not isinstance(items, list):
+                items = [items]
+            return {int(item["DeflectionId"]): item for item in items}
+        return {}
 
     def manage_device_info(
         self, dev_info: Device, dev_mac: str, consider_home: bool
@@ -353,13 +516,13 @@ class FritzBoxTools(update_coordinator.DataUpdateCoordinator):
         self._devices[dev_mac] = device
         return True
 
-    def send_signal_device_update(self, new_device: bool) -> None:
+    async def async_send_signal_device_update(self, new_device: bool) -> None:
         """Signal device data updated."""
-        dispatcher_send(self.hass, self.signal_device_update)
+        async_dispatcher_send(self.hass, self.signal_device_update)
         if new_device:
-            dispatcher_send(self.hass, self.signal_device_new)
+            async_dispatcher_send(self.hass, self.signal_device_new)
 
-    def scan_devices(self, now: datetime | None = None) -> None:
+    async def async_scan_devices(self, now: datetime | None = None) -> None:
         """Scan for new devices and return a list of found device ids."""
 
         if self.hass.is_stopping:
@@ -371,7 +534,7 @@ class FritzBoxTools(update_coordinator.DataUpdateCoordinator):
             self._update_available,
             self._latest_firmware,
             self._release_url,
-        ) = self._update_device_info()
+        ) = await self._async_update_device_info()
 
         _LOGGER.debug("Checking devices for FRITZ!Box device %s", self.host)
         _default_consider_home = DEFAULT_CONSIDER_HOME.total_seconds()
@@ -383,20 +546,7 @@ class FritzBoxTools(update_coordinator.DataUpdateCoordinator):
             consider_home = _default_consider_home
 
         new_device = False
-        hosts = {}
-        for host in self._update_hosts_info():
-            if not host.get("mac"):
-                continue
-
-            hosts[host["mac"]] = Device(
-                name=host["name"],
-                connected=host["status"],
-                connected_to="",
-                connection_type="",
-                ip_address=host["ip"],
-                ssid=None,
-                wan_access=None,
-            )
+        hosts = await self._async_update_hosts_info()
 
         if not self.fritz_status.device_has_mesh_support or (
             self._options
@@ -407,15 +557,18 @@ class FritzBoxTools(update_coordinator.DataUpdateCoordinator):
             )
             self.mesh_role = MeshRoles.NONE
             for mac, info in hosts.items():
-                if info.ip_address:
-                    info.wan_access = self._get_wan_access(info.ip_address)
                 if self.manage_device_info(info, mac, consider_home):
                     new_device = True
-            self.send_signal_device_update(new_device)
+            await self.async_send_signal_device_update(new_device)
             return
 
         try:
-            if not (topology := self.fritz_hosts.get_mesh_topology()):
+            if not (
+                topology := await self.hass.async_add_executor_job(
+                    self.fritz_hosts.get_mesh_topology
+                )
+            ):
+                # pylint: disable-next=broad-exception-raised
                 raise Exception("Mesh supported but empty topology reported")
         except FritzActionError:
             self.mesh_role = MeshRoles.SLAVE
@@ -453,9 +606,6 @@ class FritzBoxTools(update_coordinator.DataUpdateCoordinator):
 
                 dev_info: Device = hosts[dev_mac]
 
-                if dev_info.ip_address:
-                    dev_info.wan_access = self._get_wan_access(dev_info.ip_address)
-
                 for link in interf["node_links"]:
                     intf = mesh_intf.get(link["node_interface_1_uid"])
                     if intf is not None:
@@ -469,7 +619,7 @@ class FritzBoxTools(update_coordinator.DataUpdateCoordinator):
                 if self.manage_device_info(dev_info, dev_mac, consider_home):
                     new_device = True
 
-        self.send_signal_device_update(new_device)
+        await self.async_send_signal_device_update(new_device)
 
     async def async_trigger_firmware_update(self) -> bool:
         """Trigger firmware update."""
@@ -498,9 +648,7 @@ class FritzBoxTools(update_coordinator.DataUpdateCoordinator):
         self, config_entry: ConfigEntry | None = None
     ) -> None:
         """Trigger device trackers cleanup."""
-        device_hosts_list = await self.hass.async_add_executor_job(
-            self.fritz_hosts.get_hosts_info
-        )
+        device_hosts = await self._async_update_hosts_info()
         entity_reg: er.EntityRegistry = er.async_get(self.hass)
 
         if config_entry is None:
@@ -515,9 +663,9 @@ class FritzBoxTools(update_coordinator.DataUpdateCoordinator):
 
         device_hosts_macs = set()
         device_hosts_names = set()
-        for device in device_hosts_list:
-            device_hosts_macs.add(device["mac"])
-            device_hosts_names.add(device["name"])
+        for mac, device in device_hosts.items():
+            device_hosts_macs.add(mac)
+            device_hosts_names.add(device.name)
 
         for entry in ha_entity_reg_list:
             if entry.original_name is None:
@@ -574,21 +722,24 @@ class FritzBoxTools(update_coordinator.DataUpdateCoordinator):
         try:
             if service_call.service == SERVICE_REBOOT:
                 _LOGGER.warning(
-                    'Service "fritz.reboot" is deprecated, please use the corresponding button entity instead'
+                    'Service "fritz.reboot" is deprecated, please use the corresponding'
+                    " button entity instead"
                 )
                 await self.async_trigger_reboot()
                 return
 
             if service_call.service == SERVICE_RECONNECT:
                 _LOGGER.warning(
-                    'Service "fritz.reconnect" is deprecated, please use the corresponding button entity instead'
+                    'Service "fritz.reconnect" is deprecated, please use the'
+                    " corresponding button entity instead"
                 )
                 await self.async_trigger_reconnect()
                 return
 
             if service_call.service == SERVICE_CLEANUP:
                 _LOGGER.warning(
-                    'Service "fritz.cleanup" is deprecated, please use the corresponding button entity instead'
+                    'Service "fritz.cleanup" is deprecated, please use the'
+                    " corresponding button entity instead"
                 )
                 await self.async_trigger_cleanup(config_entry)
                 return
@@ -606,10 +757,10 @@ class FritzBoxTools(update_coordinator.DataUpdateCoordinator):
             raise HomeAssistantError("Service not supported") from ex
 
 
-class AvmWrapper(FritzBoxTools):
+class AvmWrapper(FritzBoxTools):  # pylint: disable=hass-enforce-coordinator-module
     """Setup AVM wrapper for API calls."""
 
-    def _service_call_action(
+    async def _async_service_call(
         self,
         service_name: str,
         service_suffix: str,
@@ -626,41 +777,45 @@ class AvmWrapper(FritzBoxTools):
             return {}
 
         try:
-            result: dict = self.connection.call_action(
-                f"{service_name}:{service_suffix}",
-                action_name,
-                **kwargs,
+            result: dict = await self.hass.async_add_executor_job(
+                partial(
+                    self.connection.call_action,
+                    f"{service_name}:{service_suffix}",
+                    action_name,
+                    **kwargs,
+                )
             )
             return result
         except FritzSecurityError:
-            _LOGGER.error(
-                "Authorization Error: Please check the provided credentials and verify that you can log into the web interface",
-                exc_info=True,
+            _LOGGER.exception(
+                "Authorization Error: Please check the provided credentials and"
+                " verify that you can log into the web interface"
             )
         except FRITZ_EXCEPTIONS:
-            _LOGGER.error(
+            _LOGGER.exception(
                 "Service/Action Error: cannot execute service %s with action %s",
                 service_name,
                 action_name,
-                exc_info=True,
             )
         except FritzConnectionException:
-            _LOGGER.error(
-                "Connection Error: Please check the device is properly configured for remote login",
-                exc_info=True,
+            _LOGGER.exception(
+                "Connection Error: Please check the device is properly configured"
+                " for remote login"
             )
         return {}
 
     async def async_get_upnp_configuration(self) -> dict[str, Any]:
         """Call X_AVM-DE_UPnP service."""
 
-        return await self.hass.async_add_executor_job(self.get_upnp_configuration)
+        return await self._async_service_call("X_AVM-DE_UPnP", "1", "GetInfo")
 
     async def async_get_wan_link_properties(self) -> dict[str, Any]:
         """Call WANCommonInterfaceConfig service."""
 
-        return await self.hass.async_add_executor_job(
-            partial(self.get_wan_link_properties)
+        return await self._async_service_call(
+            "WANCommonInterfaceConfig",
+            "1",
+            "GetCommonLinkProperties",
         )
 
     async def async_ipv6_active(self) -> bool:
@@ -691,25 +846,25 @@ class AvmWrapper(FritzBoxTools):
         )
         return connection_info
 
+    async def async_get_num_port_mapping(self, con_type: str) -> dict[str, Any]:
+        """Call GetPortMappingNumberOfEntries action."""
+
+        return await self._async_service_call(
+            con_type, "1", "GetPortMappingNumberOfEntries"
+        )
+
     async def async_get_port_mapping(self, con_type: str, index: int) -> dict[str, Any]:
         """Call GetGenericPortMappingEntry action."""
 
-        return await self.hass.async_add_executor_job(
-            partial(self.get_port_mapping, con_type, index)
+        return await self._async_service_call(
+            con_type, "1", "GetGenericPortMappingEntry", NewPortMappingIndex=index
         )
 
     async def async_get_wlan_configuration(self, index: int) -> dict[str, Any]:
         """Call WLANConfiguration service."""
 
-        return await self.hass.async_add_executor_job(
-            partial(self.get_wlan_configuration, index)
-        )
-
-    async def async_get_ontel_deflections(self) -> dict[str, Any]:
-        """Call GetDeflections action from X_AVM-DE_OnTel service."""
-
-        return await self.hass.async_add_executor_job(
-            partial(self.get_ontel_deflections)
+        return await self._async_service_call(
+            "WLANConfiguration", str(index), "GetInfo"
         )
 
     async def async_set_wlan_configuration(
@@ -717,8 +872,11 @@ class AvmWrapper(FritzBoxTools):
     ) -> dict[str, Any]:
         """Call SetEnable action from WLANConfiguration service."""
 
-        return await self.hass.async_add_executor_job(
-            partial(self.set_wlan_configuration, index, turn_on)
+        return await self._async_service_call(
+            "WLANConfiguration",
+            str(index),
+            "SetEnable",
+            NewEnable="1" if turn_on else "0",
         )
 
     async def async_set_deflection_enable(
@@ -726,94 +884,7 @@ class AvmWrapper(FritzBoxTools):
     ) -> dict[str, Any]:
         """Call SetDeflectionEnable service."""
 
-        return await self.hass.async_add_executor_job(
-            partial(self.set_deflection_enable, index, turn_on)
-        )
-
-    async def async_add_port_mapping(
-        self, con_type: str, port_mapping: Any
-    ) -> dict[str, Any]:
-        """Call AddPortMapping service."""
-
-        return await self.hass.async_add_executor_job(
-            partial(
-                self.add_port_mapping,
-                con_type,
-                port_mapping,
-            )
-        )
-
-    async def async_set_allow_wan_access(
-        self, ip_address: str, turn_on: bool
-    ) -> dict[str, Any]:
-        """Call X_AVM-DE_HostFilter service."""
-
-        return await self.hass.async_add_executor_job(
-            partial(self.set_allow_wan_access, ip_address, turn_on)
-        )
-
-    def get_upnp_configuration(self) -> dict[str, Any]:
-        """Call X_AVM-DE_UPnP service."""
-
-        return self._service_call_action("X_AVM-DE_UPnP", "1", "GetInfo")
-
-    def get_ontel_num_deflections(self) -> dict[str, Any]:
-        """Call GetNumberOfDeflections action from X_AVM-DE_OnTel service."""
-
-        return self._service_call_action(
-            "X_AVM-DE_OnTel", "1", "GetNumberOfDeflections"
-        )
-
-    def get_ontel_deflections(self) -> dict[str, Any]:
-        """Call GetDeflections action from X_AVM-DE_OnTel service."""
-
-        return self._service_call_action("X_AVM-DE_OnTel", "1", "GetDeflections")
-
-    def get_default_connection(self) -> dict[str, Any]:
-        """Call Layer3Forwarding service."""
-
-        return self._service_call_action(
-            "Layer3Forwarding", "1", "GetDefaultConnectionService"
-        )
-
-    def get_num_port_mapping(self, con_type: str) -> dict[str, Any]:
-        """Call GetPortMappingNumberOfEntries action."""
-
-        return self._service_call_action(con_type, "1", "GetPortMappingNumberOfEntries")
-
-    def get_port_mapping(self, con_type: str, index: int) -> dict[str, Any]:
-        """Call GetGenericPortMappingEntry action."""
-
-        return self._service_call_action(
-            con_type, "1", "GetGenericPortMappingEntry", NewPortMappingIndex=index
-        )
-
-    def get_wlan_configuration(self, index: int) -> dict[str, Any]:
-        """Call WLANConfiguration service."""
-
-        return self._service_call_action("WLANConfiguration", str(index), "GetInfo")
-
-    def get_wan_link_properties(self) -> dict[str, Any]:
-        """Call WANCommonInterfaceConfig service."""
-
-        return self._service_call_action(
-            "WANCommonInterfaceConfig", "1", "GetCommonLinkProperties"
-        )
-
-    def set_wlan_configuration(self, index: int, turn_on: bool) -> dict[str, Any]:
-        """Call SetEnable action from WLANConfiguration service."""
-
-        return self._service_call_action(
-            "WLANConfiguration",
-            str(index),
-            "SetEnable",
-            NewEnable="1" if turn_on else "0",
-        )
-
-    def set_deflection_enable(self, index: int, turn_on: bool) -> dict[str, Any]:
-        """Call SetDeflectionEnable service."""
-
-        return self._service_call_action(
+        return await self._async_service_call(
             "X_AVM-DE_OnTel",
             "1",
             "SetDeflectionEnable",
@@ -821,17 +892,24 @@ class AvmWrapper(FritzBoxTools):
             NewEnable="1" if turn_on else "0",
         )
 
-    def add_port_mapping(self, con_type: str, port_mapping: Any) -> dict[str, Any]:
+    async def async_add_port_mapping(
+        self, con_type: str, port_mapping: Any
+    ) -> dict[str, Any]:
         """Call AddPortMapping service."""
 
-        return self._service_call_action(
-            con_type, "1", "AddPortMapping", **port_mapping
+        return await self._async_service_call(
+            con_type,
+            "1",
+            "AddPortMapping",
+            **port_mapping,
         )
 
-    def set_allow_wan_access(self, ip_address: str, turn_on: bool) -> dict[str, Any]:
+    async def async_set_allow_wan_access(
+        self, ip_address: str, turn_on: bool
+    ) -> dict[str, Any]:
         """Call X_AVM-DE_HostFilter service."""
 
-        return self._service_call_action(
+        return await self._async_service_call(
             "X_AVM-DE_HostFilter",
             "1",
             "DisallowWANAccessByIP",
@@ -987,6 +1065,7 @@ class SwitchInfo(TypedDict):
     type: str
     callback_update: Callable
     callback_switch: Callable
+    init_state: bool
 
 
 class FritzBoxBaseEntity:
@@ -1016,6 +1095,56 @@ class FritzBoxBaseEntity:
         )
 
 
+@dataclass(frozen=True)
+class FritzRequireKeysMixin:
+    """Fritz entity description mix in."""
+
+    value_fn: Callable[[FritzStatus, Any], Any] | None
+
+
+@dataclass(frozen=True)
+class FritzEntityDescription(EntityDescription, FritzRequireKeysMixin):
+    """Fritz entity base description."""
+
+
+class FritzBoxBaseCoordinatorEntity(update_coordinator.CoordinatorEntity[AvmWrapper]):
+    """Fritz host coordinator entity base class."""
+
+    entity_description: FritzEntityDescription
+    _attr_has_entity_name = True
+
+    def __init__(
+        self,
+        avm_wrapper: AvmWrapper,
+        device_name: str,
+        description: FritzEntityDescription,
+    ) -> None:
+        """Init device info class."""
+        super().__init__(avm_wrapper)
+        if description.value_fn is not None:
+            self.async_on_remove(
+                avm_wrapper.register_entity_updates(
+                    description.key, description.value_fn
+                )
+            )
+        self.entity_description = description
+        self._device_name = device_name
+        self._attr_unique_id = f"{avm_wrapper.unique_id}-{description.key}"
+
+    @property
+    def device_info(self) -> DeviceInfo:
+        """Return the device information."""
+        return DeviceInfo(
+            configuration_url=f"http://{self.coordinator.host}",
+            connections={(dr.CONNECTION_NETWORK_MAC, self.coordinator.mac)},
+            identifiers={(DOMAIN, self.coordinator.unique_id)},
+            manufacturer="AVM",
+            model=self.coordinator.model,
+            name=self._device_name,
+            sw_version=self.coordinator.current_firmware,
+        )
+
+
 @dataclass
 class ConnectionInfo:
     """Fritz sensor connection information class."""
