@@ -8,28 +8,28 @@ from typing import Final
 from pyfritzhome.fritzhomedevice import FritzhomeDevice
 
 from homeassistant.components.binary_sensor import (
-    DEVICE_CLASS_WINDOW,
+    BinarySensorDeviceClass,
     BinarySensorEntity,
     BinarySensorEntityDescription,
 )
 from homeassistant.config_entries import ConfigEntry
-from homeassistant.core import HomeAssistant
+from homeassistant.const import EntityCategory
+from homeassistant.core import HomeAssistant, callback
 from homeassistant.helpers.entity_platform import AddEntitiesCallback
 
-from . import FritzBoxEntity
-from .const import CONF_COORDINATOR, DOMAIN as FRITZBOX_DOMAIN
-from .coordinator import FritzboxDataUpdateCoordinator
+from . import FritzBoxDeviceEntity
+from .common import get_coordinator
 from .model import FritzEntityDescriptionMixinBase
 
 
-@dataclass
+@dataclass(frozen=True)
 class FritzEntityDescriptionMixinBinarySensor(FritzEntityDescriptionMixinBase):
     """BinarySensor description mixin for Fritz!Smarthome entities."""
 
     is_on: Callable[[FritzhomeDevice], bool | None]
 
 
-@dataclass
+@dataclass(frozen=True)
 class FritzBinarySensorEntityDescription(
     BinarySensorEntityDescription, FritzEntityDescriptionMixinBinarySensor
 ):
@@ -39,10 +39,26 @@ class FritzBinarySensorEntityDescription(
 BINARY_SENSOR_TYPES: Final[tuple[FritzBinarySensorEntityDescription, ...]] = (
     FritzBinarySensorEntityDescription(
         key="alarm",
-        name="Alarm",
-        device_class=DEVICE_CLASS_WINDOW,
-        suitable=lambda device: device.has_alarm,  # type: ignore[no-any-return]
-        is_on=lambda device: device.alert_state,  # type: ignore[no-any-return]
+        translation_key="alarm",
+        device_class=BinarySensorDeviceClass.WINDOW,
+        suitable=lambda device: device.has_alarm,
+        is_on=lambda device: device.alert_state,
+    ),
+    FritzBinarySensorEntityDescription(
+        key="lock",
+        translation_key="lock",
+        device_class=BinarySensorDeviceClass.LOCK,
+        entity_category=EntityCategory.DIAGNOSTIC,
+        suitable=lambda device: device.lock is not None,
+        is_on=lambda device: not device.lock,
+    ),
+    FritzBinarySensorEntityDescription(
+        key="device_lock",
+        translation_key="device_lock",
+        device_class=BinarySensorDeviceClass.LOCK,
+        entity_category=EntityCategory.DIAGNOSTIC,
+        suitable=lambda device: device.device_lock is not None,
+        is_on=lambda device: not device.device_lock,
     ),
 )
 
@@ -51,35 +67,33 @@ async def async_setup_entry(
     hass: HomeAssistant, entry: ConfigEntry, async_add_entities: AddEntitiesCallback
 ) -> None:
     """Set up the FRITZ!SmartHome binary sensor from ConfigEntry."""
-    coordinator = hass.data[FRITZBOX_DOMAIN][entry.entry_id][CONF_COORDINATOR]
+    coordinator = get_coordinator(hass, entry.entry_id)
 
-    async_add_entities(
-        [
+    @callback
+    def _add_entities(devices: set[str] | None = None) -> None:
+        """Add devices."""
+        if devices is None:
+            devices = coordinator.new_devices
+        if not devices:
+            return
+        async_add_entities(
             FritzboxBinarySensor(coordinator, ain, description)
-            for ain, device in coordinator.data.items()
+            for ain in devices
             for description in BINARY_SENSOR_TYPES
-            if description.suitable(device)
-        ]
-    )
+            if description.suitable(coordinator.data.devices[ain])
+        )
+
+    entry.async_on_unload(coordinator.async_add_listener(_add_entities))
+
+    _add_entities(set(coordinator.data.devices))
 
 
-class FritzboxBinarySensor(FritzBoxEntity, BinarySensorEntity):
+class FritzboxBinarySensor(FritzBoxDeviceEntity, BinarySensorEntity):
     """Representation of a binary FRITZ!SmartHome device."""
 
     entity_description: FritzBinarySensorEntityDescription
 
-    def __init__(
-        self,
-        coordinator: FritzboxDataUpdateCoordinator,
-        ain: str,
-        entity_description: FritzBinarySensorEntityDescription,
-    ) -> None:
-        """Initialize the FritzBox entity."""
-        super().__init__(coordinator, ain, entity_description)
-        self._attr_name = self.device.name
-        self._attr_unique_id = ain
-
     @property
     def is_on(self) -> bool | None:
         """Return true if sensor is on."""
-        return self.entity_description.is_on(self.device)
+        return self.entity_description.is_on(self.data)
