@@ -3,7 +3,7 @@ from __future__ import annotations
 
 from collections.abc import Callable
 from dataclasses import dataclass
-from typing import Any, cast
+from typing import Any
 
 from greeclimate.device import Device
 
@@ -17,23 +17,18 @@ from homeassistant.core import HomeAssistant, callback
 from homeassistant.helpers.dispatcher import async_dispatcher_connect
 from homeassistant.helpers.entity_platform import AddEntitiesCallback
 
-from .const import COORDINATORS, DISPATCH_DEVICE_DISCOVERED, DISPATCHERS, DOMAIN
+from .const import COORDINATORS, DISPATCH_DEVICE_DISCOVERED, DOMAIN
 from .entity import GreeEntity
 
 
-@dataclass
-class GreeRequiredKeysMixin:
-    """Mixin for required keys."""
+@dataclass(kw_only=True, frozen=True)
+class GreeSwitchEntityDescription(SwitchEntityDescription):
+    """Describes a Gree switch entity."""
 
     get_value_fn: Callable[[Device], bool]
     set_value_fn: Callable[[Device, bool], None]
 
 
-@dataclass
-class GreeSwitchEntityDescription(SwitchEntityDescription, GreeRequiredKeysMixin):
-    """Describes Gree switch entity."""
-
-
 def _set_light(device: Device, value: bool) -> None:
     """Typed helper to set device light property."""
     device.light = value
@@ -62,33 +57,33 @@ def _set_anion(device: Device, value: bool) -> None:
 GREE_SWITCHES: tuple[GreeSwitchEntityDescription, ...] = (
     GreeSwitchEntityDescription(
         icon="mdi:lightbulb",
-        name="Panel Light",
-        key="light",
+        key="Panel Light",
+        translation_key="light",
         get_value_fn=lambda d: d.light,
         set_value_fn=_set_light,
     ),
     GreeSwitchEntityDescription(
-        name="Quiet",
-        key="quiet",
+        key="Quiet",
+        translation_key="quiet",
         get_value_fn=lambda d: d.quiet,
         set_value_fn=_set_quiet,
     ),
     GreeSwitchEntityDescription(
-        name="Fresh Air",
-        key="fresh_air",
+        key="Fresh Air",
+        translation_key="fresh_air",
         get_value_fn=lambda d: d.fresh_air,
         set_value_fn=_set_fresh_air,
     ),
     GreeSwitchEntityDescription(
-        name="XFan",
-        key="xfan",
+        key="XFan",
+        translation_key="xfan",
         get_value_fn=lambda d: d.xfan,
         set_value_fn=_set_xfan,
     ),
     GreeSwitchEntityDescription(
         icon="mdi:pine-tree",
-        name="Health mode",
-        key="anion",
+        key="Health mode",
+        translation_key="health_mode",
         get_value_fn=lambda d: d.anion,
         set_value_fn=_set_anion,
         entity_registry_enabled_default=False,
@@ -98,7 +93,7 @@ GREE_SWITCHES: tuple[GreeSwitchEntityDescription, ...] = (
 
 async def async_setup_entry(
     hass: HomeAssistant,
-    config_entry: ConfigEntry,
+    entry: ConfigEntry,
     async_add_entities: AddEntitiesCallback,
 ) -> None:
     """Set up the Gree HVAC device from a config entry."""
@@ -115,7 +110,7 @@ async def async_setup_entry(
     for coordinator in hass.data[DOMAIN][COORDINATORS]:
         init_device(coordinator)
 
-    hass.data[DOMAIN][DISPATCHERS].append(
+    entry.async_on_unload(
         async_dispatcher_connect(hass, DISPATCH_DEVICE_DISCOVERED, init_device)
     )
 
@@ -130,7 +125,7 @@ class GreeSwitch(GreeEntity, SwitchEntity):
         """Initialize the Gree device."""
         self.entity_description = description
 
-        super().__init__(coordinator, cast(str, description.name))
+        super().__init__(coordinator, description.key)
 
     @property
     def is_on(self) -> bool:
