@@ -8,9 +8,16 @@ DEFAULT_PLANT_ID = "0"
 DEFAULT_NAME = "Growatt"
 
 SERVER_URLS = [
+    "https://openapi.growatt.com/",  # Other regional server
+    "https://openapi-cn.growatt.com/",  # Chinese server
+    "https://openapi-us.growatt.com/",  # North American server
+    "http://server.smten.com/",  # smten server
+]
+
+DEPRECATED_URLS = [
     "https://server.growatt.com/",
+    "https://server-api.growatt.com/",
     "https://server-us.growatt.com/",
-    "http://server.smten.com/",
 ]
 
 DEFAULT_URL = SERVER_URLS[0]
