@@ -11,9 +11,9 @@ from homeassistant.components.sensor import SensorEntity
 from homeassistant.config_entries import ConfigEntry
 from homeassistant.const import CONF_NAME, CONF_PASSWORD, CONF_URL, CONF_USERNAME
 from homeassistant.core import HomeAssistant
-from homeassistant.helpers.entity import DeviceInfo
+from homeassistant.helpers.device_registry import DeviceInfo
 from homeassistant.helpers.entity_platform import AddEntitiesCallback
-from homeassistant.util import Throttle, dt
+from homeassistant.util import Throttle, dt as dt_util
 
 from .const import (
     CONF_PLANT_ID,
@@ -80,12 +80,8 @@ async def async_setup_entry(
         config[CONF_URL] = url
         hass.config_entries.async_update_entry(config_entry, data=config)
 
-    # Initialise the library with a random user id each time it is started,
-    # also extend the library's default identifier to include 'home-assistant'
-    api = growattServer.GrowattApi(
-        add_random_user_id=True,
-        agent_identifier=f"{growattServer.GrowattApi.agent_identifier} - home-assistant",
-    )
+    # Initialise the library with the username & a random id each time it is started
+    api = growattServer.GrowattApi(add_random_user_id=True, agent_identifier=username)
     api.server_url = url
 
     devices, plant_id = await hass.async_add_executor_job(get_device_list, api, config)
@@ -142,16 +138,17 @@ async def async_setup_entry(
 class GrowattInverter(SensorEntity):
     """Representation of a Growatt Sensor."""
 
+    _attr_has_entity_name = True
+
     entity_description: GrowattSensorEntityDescription
 
     def __init__(
         self, probe, name, unique_id, description: GrowattSensorEntityDescription
-    ):
+    ) -> None:
         """Initialize a PVOutput sensor."""
         self.probe = probe
         self.entity_description = description
 
-        self._attr_name = f"{name} {description.name}"
         self._attr_unique_id = unique_id
         self._attr_icon = "mdi:solar-power"
 
@@ -232,23 +229,27 @@ class GrowattData:
                 )
 
                 mix_detail = self.api.mix_detail(self.device_id, self.plant_id)
-                # Get the chart data and work out the time of the last entry, use this as the last time data was published to the Growatt Server
+                # Get the chart data and work out the time of the last entry, use this
+                # as the last time data was published to the Growatt Server
                 mix_chart_entries = mix_detail["chartData"]
                 sorted_keys = sorted(mix_chart_entries)
 
                 # Create datetime from the latest entry
-                date_now = dt.now().date()
-                last_updated_time = dt.parse_time(str(sorted_keys[-1]))
+                date_now = dt_util.now().date()
+                last_updated_time = dt_util.parse_time(str(sorted_keys[-1]))
                 mix_detail["lastdataupdate"] = datetime.datetime.combine(
-                    date_now, last_updated_time, dt.DEFAULT_TIME_ZONE
+                    date_now, last_updated_time, dt_util.DEFAULT_TIME_ZONE
                 )
 
-                # Dashboard data is largely inaccurate for mix system but it is the only call with the ability to return the combined
-                # imported from grid value that is the combination of charging AND load consumption
+                # Dashboard data is largely inaccurate for mix system but it is the only
+                # call with the ability to return the combined imported from grid value
+                # that is the combination of charging AND load consumption
                 dashboard_data = self.api.dashboard_data(self.plant_id)
-                # Dashboard values have units e.g. "kWh" as part of their returned string, so we remove it
+                # Dashboard values have units e.g. "kWh" as part of their returned
+                # string, so we remove it
                 dashboard_values_for_mix = {
-                    # etouser is already used by the results from 'mix_detail' so we rebrand it as 'etouser_combined'
+                    # etouser is already used by the results from 'mix_detail' so we
+                    # rebrand it as 'etouser_combined'
                     "etouser_combined": float(
                         dashboard_data["etouser"].replace("kWh", "")
                     )
@@ -290,7 +291,10 @@ class GrowattData:
             and api_value is not None
         ):
             _LOGGER.debug(
-                "%s - Drop threshold specified (%s), checking for drop... API Value: %s, Previous Value: %s",
+                (
+                    "%s - Drop threshold specified (%s), checking for drop... API"
+                    " Value: %s, Previous Value: %s"
+                ),
                 entity_description.name,
                 entity_description.previous_value_drop_threshold,
                 api_value,
@@ -298,14 +302,18 @@ class GrowattData:
             )
             diff = float(api_value) - float(previous_value)
 
-            # Check if the value has dropped (negative value i.e. < 0) and it has only dropped by a
-            # small amount, if so, use the previous value.
-            # Note - The energy dashboard takes care of drops within 10% of the current value,
-            # however if the value is low e.g. 0.2 and drops by 0.1 it classes as a reset.
+            # Check if the value has dropped (negative value i.e. < 0) and it has only
+            # dropped by a small amount, if so, use the previous value.
+            # Note - The energy dashboard takes care of drops within 10%
+            # of the current value, however if the value is low e.g. 0.2
+            # and drops by 0.1 it classes as a reset.
             if -(entity_description.previous_value_drop_threshold) <= diff < 0:
                 _LOGGER.debug(
-                    "Diff is negative, but only by a small amount therefore not a nightly reset, "
-                    "using previous value (%s) instead of api value (%s)",
+                    (
+                        "Diff is negative, but only by a small amount therefore not a"
+                        " nightly reset, using previous value (%s) instead of api value"
+                        " (%s)"
+                    ),
                     previous_value,
                     api_value,
                 )
@@ -315,21 +323,25 @@ class GrowattData:
                     "%s - No drop detected, using API value", entity_description.name
                 )
 
-        # Lifetime total values should always be increasing, they will never reset, however
-        # the API sometimes returns 0 values when the clock turns to 00:00 local time
-        # in that scenario we should just return the previous value
+        # Lifetime total values should always be increasing, they will never reset,
+        # however the API sometimes returns 0 values when the clock turns to 00:00
+        # local time in that scenario we should just return the previous value
         # Scenarios:
         # 1 - System has a genuine 0 value when it it first commissioned:
         #        - will return 0 until a non-zero value is registered
-        # 2 - System has been running fine but temporarily resets to 0 briefly at midnight:
+        # 2 - System has been running fine but temporarily resets to 0 briefly
+        #     at midnight:
         #        - will return the previous value
         # 3 - HA is restarted during the midnight 'outage' - Not handled:
         #        - Previous value will not exist meaning 0 will be returned
-        #        - This is an edge case that would be better handled by looking up the previous
-        #          value of the entity from the recorder
+        #        - This is an edge case that would be better handled by looking
+        #          up the previous value of the entity from the recorder
         if entity_description.never_resets and api_value == 0 and previous_value:
             _LOGGER.debug(
-                "API value is 0, but this value should never reset, returning previous value (%s) instead",
+                (
+                    "API value is 0, but this value should never reset, returning"
+                    " previous value (%s) instead"
+                ),
                 previous_value,
             )
             return_value = previous_value
