@@ -1,7 +1,9 @@
 """Sensors for the Elexa Guardian integration."""
 from __future__ import annotations
 
+from collections.abc import Callable
 from dataclasses import dataclass
+from typing import Any
 
 from homeassistant.components.sensor import (
     SensorDeviceClass,
@@ -10,11 +12,17 @@ from homeassistant.components.sensor import (
     SensorStateClass,
 )
 from homeassistant.config_entries import ConfigEntry
-from homeassistant.const import ELECTRIC_POTENTIAL_VOLT, TEMP_FAHRENHEIT, TIME_MINUTES
+from homeassistant.const import (
+    EntityCategory,
+    UnitOfElectricCurrent,
+    UnitOfElectricPotential,
+    UnitOfTemperature,
+    UnitOfTime,
+)
 from homeassistant.core import HomeAssistant, callback
 from homeassistant.helpers.dispatcher import async_dispatcher_connect
-from homeassistant.helpers.entity import EntityCategory
 from homeassistant.helpers.entity_platform import AddEntitiesCallback
+from homeassistant.helpers.typing import StateType
 
 from . import (
     GuardianData,
@@ -25,55 +33,106 @@ from . import (
 from .const import (
     API_SYSTEM_DIAGNOSTICS,
     API_SYSTEM_ONBOARD_SENSOR_STATUS,
+    API_VALVE_STATUS,
     CONF_UID,
     DOMAIN,
     SIGNAL_PAIRED_SENSOR_COORDINATOR_ADDED,
 )
 
+SENSOR_KIND_AVG_CURRENT = "average_current"
 SENSOR_KIND_BATTERY = "battery"
+SENSOR_KIND_INST_CURRENT = "instantaneous_current"
+SENSOR_KIND_INST_CURRENT_DDT = "instantaneous_current_ddt"
 SENSOR_KIND_TEMPERATURE = "temperature"
+SENSOR_KIND_TRAVEL_COUNT = "travel_count"
 SENSOR_KIND_UPTIME = "uptime"
 
 
-@dataclass
+@dataclass(frozen=True, kw_only=True)
+class PairedSensorDescription(SensorEntityDescription):
+    """Describe a Guardian paired sensor."""
+
+    value_fn: Callable[[dict[str, Any]], StateType]
+
+
+@dataclass(frozen=True, kw_only=True)
 class ValveControllerSensorDescription(
     SensorEntityDescription, ValveControllerEntityDescription
 ):
     """Describe a Guardian valve controller sensor."""
 
+    value_fn: Callable[[dict[str, Any]], StateType]
+
 
 PAIRED_SENSOR_DESCRIPTIONS = (
-    SensorEntityDescription(
+    PairedSensorDescription(
         key=SENSOR_KIND_BATTERY,
-        name="Battery",
         device_class=SensorDeviceClass.VOLTAGE,
         entity_category=EntityCategory.DIAGNOSTIC,
-        native_unit_of_measurement=ELECTRIC_POTENTIAL_VOLT,
+        native_unit_of_measurement=UnitOfElectricPotential.VOLT,
+        value_fn=lambda data: data["battery"],
     ),
-    SensorEntityDescription(
+    PairedSensorDescription(
         key=SENSOR_KIND_TEMPERATURE,
-        name="Temperature",
         device_class=SensorDeviceClass.TEMPERATURE,
-        native_unit_of_measurement=TEMP_FAHRENHEIT,
+        native_unit_of_measurement=UnitOfTemperature.FAHRENHEIT,
         state_class=SensorStateClass.MEASUREMENT,
+        value_fn=lambda data: data["temperature"],
     ),
 )
 VALVE_CONTROLLER_DESCRIPTIONS = (
+    ValveControllerSensorDescription(
+        key=SENSOR_KIND_AVG_CURRENT,
+        translation_key="current",
+        device_class=SensorDeviceClass.CURRENT,
+        entity_category=EntityCategory.DIAGNOSTIC,
+        native_unit_of_measurement=UnitOfElectricCurrent.MILLIAMPERE,
+        api_category=API_VALVE_STATUS,
+        value_fn=lambda data: data["average_current"],
+    ),
+    ValveControllerSensorDescription(
+        key=SENSOR_KIND_INST_CURRENT,
+        translation_key="instantaneous_current",
+        device_class=SensorDeviceClass.CURRENT,
+        entity_category=EntityCategory.DIAGNOSTIC,
+        native_unit_of_measurement=UnitOfElectricCurrent.MILLIAMPERE,
+        api_category=API_VALVE_STATUS,
+        value_fn=lambda data: data["instantaneous_current"],
+    ),
+    ValveControllerSensorDescription(
+        key=SENSOR_KIND_INST_CURRENT_DDT,
+        translation_key="instantaneous_current_ddt",
+        device_class=SensorDeviceClass.CURRENT,
+        entity_category=EntityCategory.DIAGNOSTIC,
+        native_unit_of_measurement=UnitOfElectricCurrent.MILLIAMPERE,
+        api_category=API_VALVE_STATUS,
+        value_fn=lambda data: data["instantaneous_current_ddt"],
+    ),
     ValveControllerSensorDescription(
         key=SENSOR_KIND_TEMPERATURE,
-        name="Temperature",
         device_class=SensorDeviceClass.TEMPERATURE,
-        native_unit_of_measurement=TEMP_FAHRENHEIT,
+        native_unit_of_measurement=UnitOfTemperature.FAHRENHEIT,
         state_class=SensorStateClass.MEASUREMENT,
         api_category=API_SYSTEM_ONBOARD_SENSOR_STATUS,
+        value_fn=lambda data: data["temperature"],
     ),
     ValveControllerSensorDescription(
         key=SENSOR_KIND_UPTIME,
-        name="Uptime",
+        translation_key="uptime",
         icon="mdi:timer",
         entity_category=EntityCategory.DIAGNOSTIC,
-        native_unit_of_measurement=TIME_MINUTES,
+        native_unit_of_measurement=UnitOfTime.MINUTES,
         api_category=API_SYSTEM_DIAGNOSTICS,
+        value_fn=lambda data: data["uptime"],
+    ),
+    ValveControllerSensorDescription(
+        key=SENSOR_KIND_TRAVEL_COUNT,
+        translation_key="travel_count",
+        icon="mdi:counter",
+        entity_category=EntityCategory.DIAGNOSTIC,
+        native_unit_of_measurement="revolutions",
+        api_category=API_VALVE_STATUS,
+        value_fn=lambda data: data["travel_count"],
     ),
 )
 
@@ -124,15 +183,12 @@ async def async_setup_entry(
 class PairedSensorSensor(PairedSensorEntity, SensorEntity):
     """Define a binary sensor related to a Guardian valve controller."""
 
-    entity_description: SensorEntityDescription
+    entity_description: PairedSensorDescription
 
-    @callback
-    def _async_update_from_latest_data(self) -> None:
-        """Update the entity's underlying data."""
-        if self.entity_description.key == SENSOR_KIND_BATTERY:
-            self._attr_native_value = self.coordinator.data["battery"]
-        elif self.entity_description.key == SENSOR_KIND_TEMPERATURE:
-            self._attr_native_value = self.coordinator.data["temperature"]
+    @property
+    def native_value(self) -> StateType:
+        """Return the value reported by the sensor."""
+        return self.entity_description.value_fn(self.coordinator.data)
 
 
 class ValveControllerSensor(ValveControllerEntity, SensorEntity):
@@ -140,10 +196,7 @@ class ValveControllerSensor(ValveControllerEntity, SensorEntity):
 
     entity_description: ValveControllerSensorDescription
 
-    @callback
-    def _async_update_from_latest_data(self) -> None:
-        """Update the entity's underlying data."""
-        if self.entity_description.key == SENSOR_KIND_TEMPERATURE:
-            self._attr_native_value = self.coordinator.data["temperature"]
-        elif self.entity_description.key == SENSOR_KIND_UPTIME:
-            self._attr_native_value = self.coordinator.data["uptime"]
+    @property
+    def native_value(self) -> StateType:
+        """Return the value reported by the sensor."""
+        return self.entity_description.value_fn(self.coordinator.data)
