@@ -1,10 +1,13 @@
 """Support for Harmony Hub devices."""
+from __future__ import annotations
+
+from collections.abc import Iterable
 import json
 import logging
+from typing import Any
 
 import voluptuous as vol
 
-from homeassistant.components import remote
 from homeassistant.components.remote import (
     ATTR_ACTIVITY,
     ATTR_DELAY_SECS,
@@ -12,16 +15,17 @@ from homeassistant.components.remote import (
     ATTR_HOLD_SECS,
     ATTR_NUM_REPEATS,
     DEFAULT_DELAY_SECS,
-    SUPPORT_ACTIVITY,
+    RemoteEntity,
+    RemoteEntityFeature,
 )
 from homeassistant.config_entries import ConfigEntry
 from homeassistant.core import HomeAssistant, callback
 from homeassistant.helpers import entity_platform
 import homeassistant.helpers.config_validation as cv
 from homeassistant.helpers.dispatcher import async_dispatcher_connect
+from homeassistant.helpers.entity_platform import AddEntitiesCallback
 from homeassistant.helpers.restore_state import RestoreEntity
 
-from .connection_state import ConnectionStateMixin
 from .const import (
     ACTIVITY_POWER_OFF,
     ATTR_ACTIVITY_STARTING,
@@ -34,6 +38,8 @@ from .const import (
     SERVICE_CHANGE_CHANNEL,
     SERVICE_SYNC,
 )
+from .data import HarmonyData
+from .entity import HarmonyEntity
 from .subscriber import HarmonyCallback
 
 _LOGGER = logging.getLogger(__name__)
@@ -49,22 +55,22 @@ HARMONY_CHANGE_CHANNEL_SCHEMA = {
 
 
 async def async_setup_entry(
-    hass: HomeAssistant, entry: ConfigEntry, async_add_entities
-):
+    hass: HomeAssistant, entry: ConfigEntry, async_add_entities: AddEntitiesCallback
+) -> None:
     """Set up the Harmony config entry."""
 
-    data = hass.data[DOMAIN][entry.entry_id][HARMONY_DATA]
+    data: HarmonyData = hass.data[DOMAIN][entry.entry_id][HARMONY_DATA]
 
     _LOGGER.debug("HarmonyData : %s", data)
 
-    default_activity = entry.options.get(ATTR_ACTIVITY)
-    delay_secs = entry.options.get(ATTR_DELAY_SECS, DEFAULT_DELAY_SECS)
+    default_activity: str | None = entry.options.get(ATTR_ACTIVITY)
+    delay_secs: float = entry.options.get(ATTR_DELAY_SECS, DEFAULT_DELAY_SECS)
 
     harmony_conf_file = hass.config.path(f"harmony_{entry.unique_id}.conf")
     device = HarmonyRemote(data, default_activity, delay_secs, harmony_conf_file)
     async_add_entities([device])
 
-    platform = entity_platform.current_platform.get()
+    platform = entity_platform.async_get_current_platform()
 
     platform.async_register_entity_service(
         SERVICE_SYNC,
@@ -76,30 +82,29 @@ async def async_setup_entry(
     )
 
 
-class HarmonyRemote(ConnectionStateMixin, remote.RemoteEntity, RestoreEntity):
+class HarmonyRemote(HarmonyEntity, RemoteEntity, RestoreEntity):
     """Remote representation used to control a Harmony device."""
 
-    def __init__(self, data, activity, delay_secs, out_path):
+    _attr_supported_features = RemoteEntityFeature.ACTIVITY
+
+    def __init__(
+        self, data: HarmonyData, activity: str | None, delay_secs: float, out_path: str
+    ) -> None:
         """Initialize HarmonyRemote class."""
-        super().__init__()
-        self._data = data
-        self._name = data.name
-        self._state = None
+        super().__init__(data=data)
+        self._state: bool | None = None
         self._current_activity = ACTIVITY_POWER_OFF
         self.default_activity = activity
         self._activity_starting = None
         self._is_initial_update = True
         self.delay_secs = delay_secs
-        self._unique_id = data.unique_id
         self._last_activity = None
         self._config_path = out_path
+        self._attr_unique_id = data.unique_id
+        self._attr_device_info = self._data.device_info(DOMAIN)
+        self._attr_name = data.name
 
-    @property
-    def supported_features(self):
-        """Supported features for the remote."""
-        return SUPPORT_ACTIVITY
-
-    async def _async_update_options(self, data):
+    async def _async_update_options(self, data: dict[str, Any]) -> None:
         """Change options when the options flow does."""
         if ATTR_DELAY_SECS in data:
             self.delay_secs = data[ATTR_DELAY_SECS]
@@ -107,16 +112,18 @@ class HarmonyRemote(ConnectionStateMixin, remote.RemoteEntity, RestoreEntity):
         if ATTR_ACTIVITY in data:
             self.default_activity = data[ATTR_ACTIVITY]
 
-    def _setup_callbacks(self):
-        callbacks = {
-            "connected": self.async_got_connected,
-            "disconnected": self.async_got_disconnected,
-            "config_updated": self.async_new_config,
-            "activity_starting": self.async_new_activity,
-            "activity_started": self.async_new_activity_finished,
-        }
-
-        self.async_on_remove(self._data.async_subscribe(HarmonyCallback(**callbacks)))
+    def _setup_callbacks(self) -> None:
+        self.async_on_remove(
+            self._data.async_subscribe(
+                HarmonyCallback(
+                    connected=self.async_got_connected,
+                    disconnected=self.async_got_disconnected,
+                    config_updated=self.async_new_config,
+                    activity_starting=self.async_new_activity,
+                    activity_started=self.async_new_activity_finished,
+                )
+            )
+        )
 
     @callback
     def async_new_activity_finished(self, activity_info: tuple) -> None:
@@ -124,11 +131,11 @@ class HarmonyRemote(ConnectionStateMixin, remote.RemoteEntity, RestoreEntity):
         self._activity_starting = None
         self.async_write_ha_state()
 
-    async def async_added_to_hass(self):
+    async def async_added_to_hass(self) -> None:
         """Complete the initialization."""
         await super().async_added_to_hass()
 
-        _LOGGER.debug("%s: Harmony Hub added", self._name)
+        _LOGGER.debug("%s: Harmony Hub added", self.name)
 
         self.async_on_remove(self._clear_disconnection_delay)
         self._setup_callbacks()
@@ -148,8 +155,7 @@ class HarmonyRemote(ConnectionStateMixin, remote.RemoteEntity, RestoreEntity):
         # Restore the last activity so we know
         # how what to turn on if nothing
         # is specified
-        last_state = await self.async_get_last_state()
-        if not last_state:
+        if not (last_state := await self.async_get_last_state()):
             return
         if ATTR_LAST_ACTIVITY not in last_state.attributes:
             return
@@ -158,26 +164,6 @@ class HarmonyRemote(ConnectionStateMixin, remote.RemoteEntity, RestoreEntity):
 
         self._last_activity = last_state.attributes[ATTR_LAST_ACTIVITY]
 
-    @property
-    def device_info(self):
-        """Return device info."""
-        return self._data.device_info(DOMAIN)
-
-    @property
-    def unique_id(self):
-        """Return the unique id."""
-        return self._unique_id
-
-    @property
-    def name(self):
-        """Return the Harmony device's name."""
-        return self._name
-
-    @property
-    def should_poll(self):
-        """Return the fact that we should not be polled."""
-        return False
-
     @property
     def current_activity(self):
         """Return the current activity."""
@@ -189,7 +175,7 @@ class HarmonyRemote(ConnectionStateMixin, remote.RemoteEntity, RestoreEntity):
         return self._data.activity_names
 
     @property
-    def extra_state_attributes(self):
+    def extra_state_attributes(self) -> dict[str, Any]:
         """Add platform specific attributes."""
         return {
             ATTR_ACTIVITY_STARTING: self._activity_starting,
@@ -198,20 +184,15 @@ class HarmonyRemote(ConnectionStateMixin, remote.RemoteEntity, RestoreEntity):
         }
 
     @property
-    def is_on(self):
+    def is_on(self) -> bool:
         """Return False if PowerOff is the current activity, otherwise True."""
         return self._current_activity not in [None, "PowerOff"]
 
-    @property
-    def available(self):
-        """Return True if connected to Hub, otherwise False."""
-        return self._data.available
-
     @callback
     def async_new_activity(self, activity_info: tuple) -> None:
         """Call for updating the current activity."""
         activity_id, activity_name = activity_info
-        _LOGGER.debug("%s: activity reported as: %s", self._name, activity_name)
+        _LOGGER.debug("%s: activity reported as: %s", self.name, activity_name)
         self._current_activity = activity_name
         if self._is_initial_update:
             self._is_initial_update = False
@@ -225,13 +206,13 @@ class HarmonyRemote(ConnectionStateMixin, remote.RemoteEntity, RestoreEntity):
         self._state = bool(activity_id != -1)
         self.async_write_ha_state()
 
-    async def async_new_config(self, _=None):
+    async def async_new_config(self, _: dict | None = None) -> None:
         """Call for updating the current activity."""
-        _LOGGER.debug("%s: configuration has been updated", self._name)
+        _LOGGER.debug("%s: configuration has been updated", self.name)
         self.async_new_activity(self._data.current_activity)
         await self.hass.async_add_executor_job(self.write_config_file)
 
-    async def async_turn_on(self, **kwargs):
+    async def async_turn_on(self, **kwargs: Any) -> None:
         """Start an activity from the Harmony device."""
         _LOGGER.debug("%s: Turn On", self.name)
 
@@ -240,25 +221,22 @@ class HarmonyRemote(ConnectionStateMixin, remote.RemoteEntity, RestoreEntity):
         if not activity or activity == PREVIOUS_ACTIVE_ACTIVITY:
             if self._last_activity:
                 activity = self._last_activity
-            else:
-                all_activities = self._data.activity_names
-                if all_activities:
-                    activity = all_activities[0]
+            elif all_activities := self._data.activity_names:
+                activity = all_activities[0]
 
         if activity:
             await self._data.async_start_activity(activity)
         else:
             _LOGGER.error("%s: No activity specified with turn_on service", self.name)
 
-    async def async_turn_off(self, **kwargs):
+    async def async_turn_off(self, **kwargs: Any) -> None:
         """Start the PowerOff activity."""
         await self._data.async_power_off()
 
-    async def async_send_command(self, command, **kwargs):
+    async def async_send_command(self, command: Iterable[str], **kwargs: Any) -> None:
         """Send a list of commands to one device."""
         _LOGGER.debug("%s: Send Command", self.name)
-        device = kwargs.get(ATTR_DEVICE)
-        if device is None:
+        if (device := kwargs.get(ATTR_DEVICE)) is None:
             _LOGGER.error("%s: Missing required argument: device", self.name)
             return
 
@@ -269,16 +247,16 @@ class HarmonyRemote(ConnectionStateMixin, remote.RemoteEntity, RestoreEntity):
             command, device, num_repeats, delay_secs, hold_secs
         )
 
-    async def change_channel(self, channel):
+    async def change_channel(self, channel: int) -> None:
         """Change the channel using Harmony remote."""
         await self._data.change_channel(channel)
 
-    async def sync(self):
+    async def sync(self) -> None:
         """Sync the Harmony device with the web service."""
         if await self._data.sync():
             await self.hass.async_add_executor_job(self.write_config_file)
 
-    def write_config_file(self):
+    def write_config_file(self) -> None:
         """Write Harmony configuration file.
 
         This is a handy way for users to figure out the available commands for automations.
@@ -286,8 +264,7 @@ class HarmonyRemote(ConnectionStateMixin, remote.RemoteEntity, RestoreEntity):
         _LOGGER.debug(
             "%s: Writing hub configuration to file: %s", self.name, self._config_path
         )
-        json_config = self._data.json_config
-        if json_config is None:
+        if (json_config := self._data.json_config) is None:
             _LOGGER.warning("%s: No configuration received from hub", self.name)
             return
 
