@@ -2,18 +2,15 @@
 import aioharmony.exceptions as harmony_exceptions
 from aioharmony.harmonyapi import HarmonyAPI
 
-from homeassistant.const import CONF_HOST, CONF_NAME
-
-from .const import DOMAIN
+from homeassistant.const import CONF_NAME
 
 
 def find_unique_id_for_remote(harmony: HarmonyAPI):
     """Find the unique id for both websocket and xmpp clients."""
-    websocket_unique_id = harmony.hub_config.info.get("activeRemoteId")
-    if websocket_unique_id is not None:
-        return str(websocket_unique_id)
+    if harmony.hub_id is not None:
+        return str(harmony.hub_id)
 
-    # fallback to the xmpp unique id if websocket is not available
+    # fallback timeStampHash if Hub ID is not available
     return harmony.config["global"]["timeStampHash"].split(";")[-1]
 
 
@@ -28,7 +25,7 @@ def find_best_name_for_remote(data: dict, harmony: HarmonyAPI):
     return data[CONF_NAME]
 
 
-async def get_harmony_client_if_available(ip_address: str):
+async def get_harmony_client_if_available(ip_address: str) -> HarmonyAPI | None:
     """Connect to a harmony hub and fetch info."""
     harmony = HarmonyAPI(ip_address=ip_address)
 
@@ -42,11 +39,3 @@ async def get_harmony_client_if_available(ip_address: str):
     await harmony.close()
 
     return harmony
-
-
-def find_matching_config_entries_for_host(hass, host):
-    """Search existing config entries for one matching the host."""
-    for entry in hass.config_entries.async_entries(DOMAIN):
-        if entry.data[CONF_HOST] == host:
-            return entry
-    return None
