@@ -2,12 +2,16 @@
 from __future__ import annotations
 
 import asyncio
+from collections.abc import Coroutine
 from http import HTTPStatus
 import logging
 import os
+from typing import Any
 
 import aiohttp
+from yarl import URL
 
+from homeassistant.auth.models import RefreshToken
 from homeassistant.components.http import (
     CONF_SERVER_HOST,
     CONF_SERVER_PORT,
@@ -60,7 +64,7 @@ async def async_get_addon_info(hass: HomeAssistant, slug: str) -> dict:
     The add-on must be installed.
     The caller of the function should handle HassioAPIError.
     """
-    hassio = hass.data[DOMAIN]
+    hassio: HassIO = hass.data[DOMAIN]
     return await hassio.get_addon_info(slug)
 
 
@@ -81,7 +85,7 @@ async def async_update_diagnostics(hass: HomeAssistant, diagnostics: bool) -> di
 
     The caller of the function should handle HassioAPIError.
     """
-    hassio = hass.data[DOMAIN]
+    hassio: HassIO = hass.data[DOMAIN]
     return await hassio.update_diagnostics(diagnostics)
 
 
@@ -92,7 +96,7 @@ async def async_install_addon(hass: HomeAssistant, slug: str) -> dict:
 
     The caller of the function should handle HassioAPIError.
     """
-    hassio = hass.data[DOMAIN]
+    hassio: HassIO = hass.data[DOMAIN]
     command = f"/addons/{slug}/install"
     return await hassio.send_command(command, timeout=None)
 
@@ -104,7 +108,7 @@ async def async_uninstall_addon(hass: HomeAssistant, slug: str) -> dict:
 
     The caller of the function should handle HassioAPIError.
     """
-    hassio = hass.data[DOMAIN]
+    hassio: HassIO = hass.data[DOMAIN]
     command = f"/addons/{slug}/uninstall"
     return await hassio.send_command(command, timeout=60)
 
@@ -120,7 +124,7 @@ async def async_update_addon(
 
     The caller of the function should handle HassioAPIError.
     """
-    hassio = hass.data[DOMAIN]
+    hassio: HassIO = hass.data[DOMAIN]
     command = f"/addons/{slug}/update"
     return await hassio.send_command(
         command,
@@ -136,7 +140,7 @@ async def async_start_addon(hass: HomeAssistant, slug: str) -> dict:
 
     The caller of the function should handle HassioAPIError.
     """
-    hassio = hass.data[DOMAIN]
+    hassio: HassIO = hass.data[DOMAIN]
     command = f"/addons/{slug}/start"
     return await hassio.send_command(command, timeout=60)
 
@@ -148,7 +152,7 @@ async def async_restart_addon(hass: HomeAssistant, slug: str) -> dict:
 
     The caller of the function should handle HassioAPIError.
     """
-    hassio = hass.data[DOMAIN]
+    hassio: HassIO = hass.data[DOMAIN]
     command = f"/addons/{slug}/restart"
     return await hassio.send_command(command, timeout=None)
 
@@ -160,7 +164,7 @@ async def async_stop_addon(hass: HomeAssistant, slug: str) -> dict:
 
     The caller of the function should handle HassioAPIError.
     """
-    hassio = hass.data[DOMAIN]
+    hassio: HassIO = hass.data[DOMAIN]
     command = f"/addons/{slug}/stop"
     return await hassio.send_command(command, timeout=60)
 
@@ -174,7 +178,7 @@ async def async_set_addon_options(
 
     The caller of the function should handle HassioAPIError.
     """
-    hassio = hass.data[DOMAIN]
+    hassio: HassIO = hass.data[DOMAIN]
     command = f"/addons/{slug}/options"
     return await hassio.send_command(command, payload=options)
 
@@ -182,7 +186,7 @@ async def async_set_addon_options(
 @bind_hass
 async def async_get_addon_discovery_info(hass: HomeAssistant, slug: str) -> dict | None:
     """Return discovery data for an add-on."""
-    hassio = hass.data[DOMAIN]
+    hassio: HassIO = hass.data[DOMAIN]
     data = await hassio.retrieve_discovery_messages()
     discovered_addons = data[ATTR_DISCOVERY]
     return next((addon for addon in discovered_addons if addon["addon"] == slug), None)
@@ -197,7 +201,7 @@ async def async_create_backup(
 
     The caller of the function should handle HassioAPIError.
     """
-    hassio = hass.data[DOMAIN]
+    hassio: HassIO = hass.data[DOMAIN]
     backup_type = "partial" if partial else "full"
     command = f"/backups/new/{backup_type}"
     return await hassio.send_command(command, payload=payload, timeout=None)
@@ -210,7 +214,7 @@ async def async_update_os(hass: HomeAssistant, version: str | None = None) -> di
 
     The caller of the function should handle HassioAPIError.
     """
-    hassio = hass.data[DOMAIN]
+    hassio: HassIO = hass.data[DOMAIN]
     command = "/os/update"
     return await hassio.send_command(
         command,
@@ -226,7 +230,7 @@ async def async_update_supervisor(hass: HomeAssistant) -> dict:
 
     The caller of the function should handle HassioAPIError.
     """
-    hassio = hass.data[DOMAIN]
+    hassio: HassIO = hass.data[DOMAIN]
     command = "/supervisor/update"
     return await hassio.send_command(command, timeout=None)
 
@@ -240,7 +244,7 @@ async def async_update_core(
 
     The caller of the function should handle HassioAPIError.
     """
-    hassio = hass.data[DOMAIN]
+    hassio: HassIO = hass.data[DOMAIN]
     command = "/core/update"
     return await hassio.send_command(
         command,
@@ -249,6 +253,70 @@ async def async_update_core(
     )
 
 
+@bind_hass
+@_api_bool
+async def async_apply_suggestion(hass: HomeAssistant, suggestion_uuid: str) -> bool:
+    """Apply a suggestion from supervisor's resolution center.
+
+    The caller of the function should handle HassioAPIError.
+    """
+    hassio: HassIO = hass.data[DOMAIN]
+    command = f"/resolution/suggestion/{suggestion_uuid}"
+    return await hassio.send_command(command, timeout=None)
+
+
+@api_data
+async def async_get_green_settings(hass: HomeAssistant) -> dict[str, bool]:
+    """Return settings specific to Home Assistant Green."""
+    hassio: HassIO = hass.data[DOMAIN]
+    return await hassio.send_command("/os/boards/green", method="get")
+
+
+@api_data
+async def async_set_green_settings(
+    hass: HomeAssistant, settings: dict[str, bool]
+) -> dict:
+    """Set settings specific to Home Assistant Green.
+
+    Returns an empty dict.
+    """
+    hassio: HassIO = hass.data[DOMAIN]
+    return await hassio.send_command(
+        "/os/boards/green", method="post", payload=settings
+    )
+
+
+@api_data
+async def async_get_yellow_settings(hass: HomeAssistant) -> dict[str, bool]:
+    """Return settings specific to Home Assistant Yellow."""
+    hassio: HassIO = hass.data[DOMAIN]
+    return await hassio.send_command("/os/boards/yellow", method="get")
+
+
+@api_data
+async def async_set_yellow_settings(
+    hass: HomeAssistant, settings: dict[str, bool]
+) -> dict:
+    """Set settings specific to Home Assistant Yellow.
+
+    Returns an empty dict.
+    """
+    hassio: HassIO = hass.data[DOMAIN]
+    return await hassio.send_command(
+        "/os/boards/yellow", method="post", payload=settings
+    )
+
+
+@api_data
+async def async_reboot_host(hass: HomeAssistant) -> dict:
+    """Reboot the host.
+
+    Returns an empty dict.
+    """
+    hassio: HassIO = hass.data[DOMAIN]
+    return await hassio.send_command("/host/reboot", method="post", timeout=60)
+
+
 class HassIO:
     """Small API wrapper for Hass.io."""
 
@@ -262,9 +330,10 @@ class HassIO:
         self.loop = loop
         self.websession = websession
         self._ip = ip
+        self._base_url = URL(f"http://{ip}")
 
     @_api_bool
-    def is_connected(self):
+    def is_connected(self) -> Coroutine:
         """Return true if it connected to Hass.io supervisor.
 
         This method returns a coroutine.
@@ -272,7 +341,7 @@ class HassIO:
         return self.send_command("/supervisor/ping", method="get", timeout=15)
 
     @api_data
-    def get_info(self):
+    def get_info(self) -> Coroutine:
         """Return generic Supervisor information.
 
         This method returns a coroutine.
@@ -280,7 +349,7 @@ class HassIO:
         return self.send_command("/info", method="get")
 
     @api_data
-    def get_host_info(self):
+    def get_host_info(self) -> Coroutine:
         """Return data for Host.
 
         This method returns a coroutine.
@@ -288,7 +357,7 @@ class HassIO:
         return self.send_command("/host/info", method="get")
 
     @api_data
-    def get_os_info(self):
+    def get_os_info(self) -> Coroutine:
         """Return data for the OS.
 
         This method returns a coroutine.
@@ -296,7 +365,7 @@ class HassIO:
         return self.send_command("/os/info", method="get")
 
     @api_data
-    def get_core_info(self):
+    def get_core_info(self) -> Coroutine:
         """Return data for Home Asssistant Core.
 
         This method returns a coroutine.
@@ -304,7 +373,7 @@ class HassIO:
         return self.send_command("/core/info", method="get")
 
     @api_data
-    def get_supervisor_info(self):
+    def get_supervisor_info(self) -> Coroutine:
         """Return data for the Supervisor.
 
         This method returns a coroutine.
@@ -312,7 +381,7 @@ class HassIO:
         return self.send_command("/supervisor/info", method="get")
 
     @api_data
-    def get_addon_info(self, addon):
+    def get_addon_info(self, addon: str) -> Coroutine:
         """Return data for a Add-on.
 
         This method returns a coroutine.
@@ -320,7 +389,7 @@ class HassIO:
         return self.send_command(f"/addons/{addon}/info", method="get")
 
     @api_data
-    def get_core_stats(self):
+    def get_core_stats(self) -> Coroutine:
         """Return stats for the core.
 
         This method returns a coroutine.
@@ -328,7 +397,7 @@ class HassIO:
         return self.send_command("/core/stats", method="get")
 
     @api_data
-    def get_addon_stats(self, addon):
+    def get_addon_stats(self, addon: str) -> Coroutine:
         """Return stats for an Add-on.
 
         This method returns a coroutine.
@@ -336,14 +405,14 @@ class HassIO:
         return self.send_command(f"/addons/{addon}/stats", method="get")
 
     @api_data
-    def get_supervisor_stats(self):
+    def get_supervisor_stats(self) -> Coroutine:
         """Return stats for the supervisor.
 
         This method returns a coroutine.
         """
         return self.send_command("/supervisor/stats", method="get")
 
-    def get_addon_changelog(self, addon):
+    def get_addon_changelog(self, addon: str) -> Coroutine:
         """Return changelog for an Add-on.
 
         This method returns a coroutine.
@@ -353,7 +422,7 @@ class HassIO:
         )
 
     @api_data
-    def get_store(self):
+    def get_store(self) -> Coroutine:
         """Return data from the store.
 
         This method returns a coroutine.
@@ -361,7 +430,7 @@ class HassIO:
         return self.send_command("/store", method="get")
 
     @api_data
-    def get_ingress_panels(self):
+    def get_ingress_panels(self) -> Coroutine:
         """Return data for Add-on ingress panels.
 
         This method returns a coroutine.
@@ -369,7 +438,7 @@ class HassIO:
         return self.send_command("/ingress/panels", method="get")
 
     @_api_bool
-    def restart_homeassistant(self):
+    def restart_homeassistant(self) -> Coroutine:
         """Restart Home-Assistant container.
 
         This method returns a coroutine.
@@ -377,7 +446,7 @@ class HassIO:
         return self.send_command("/homeassistant/restart")
 
     @_api_bool
-    def stop_homeassistant(self):
+    def stop_homeassistant(self) -> Coroutine:
         """Stop Home-Assistant container.
 
         This method returns a coroutine.
@@ -385,15 +454,15 @@ class HassIO:
         return self.send_command("/homeassistant/stop")
 
     @_api_bool
-    def refresh_updates(self):
+    def refresh_updates(self) -> Coroutine:
         """Refresh available updates.
 
         This method returns a coroutine.
         """
-        return self.send_command("/refresh_updates", timeout=None)
+        return self.send_command("/refresh_updates", timeout=300)
 
     @api_data
-    def retrieve_discovery_messages(self):
+    def retrieve_discovery_messages(self) -> Coroutine:
         """Return all discovery data from Hass.io API.
 
         This method returns a coroutine.
@@ -401,7 +470,7 @@ class HassIO:
         return self.send_command("/discovery", method="get", timeout=60)
 
     @api_data
-    def get_discovery_message(self, uuid):
+    def get_discovery_message(self, uuid: str) -> Coroutine:
         """Return a single discovery data message.
 
         This method returns a coroutine.
@@ -409,15 +478,29 @@ class HassIO:
         return self.send_command(f"/discovery/{uuid}", method="get")
 
     @api_data
-    def get_resolution_info(self):
+    def get_resolution_info(self) -> Coroutine:
         """Return data for Supervisor resolution center.
 
         This method returns a coroutine.
         """
         return self.send_command("/resolution/info", method="get")
 
+    @api_data
+    def get_suggestions_for_issue(
+        self, issue_id: str
+    ) -> Coroutine[Any, Any, dict[str, Any]]:
+        """Return suggestions for issue from Supervisor resolution center.
+
+        This method returns a coroutine.
+        """
+        return self.send_command(
+            f"/resolution/issue/{issue_id}/suggestions", method="get"
+        )
+
     @_api_bool
-    async def update_hass_api(self, http_config, refresh_token):
+    async def update_hass_api(
+        self, http_config: dict[str, Any], refresh_token: RefreshToken
+    ):
         """Update Home Assistant API data on Hass.io."""
         port = http_config.get(CONF_SERVER_PORT) or SERVER_PORT
         options = {
@@ -437,7 +520,7 @@ class HassIO:
         return await self.send_command("/homeassistant/options", payload=options)
 
     @_api_bool
-    def update_hass_timezone(self, timezone):
+    def update_hass_timezone(self, timezone: str) -> Coroutine:
         """Update Home-Assistant timezone data on Hass.io.
 
         This method returns a coroutine.
@@ -445,7 +528,7 @@ class HassIO:
         return self.send_command("/supervisor/options", payload={"timezone": timezone})
 
     @_api_bool
-    def update_diagnostics(self, diagnostics: bool):
+    def update_diagnostics(self, diagnostics: bool) -> Coroutine:
         """Update Supervisor diagnostics setting.
 
         This method returns a coroutine.
@@ -454,24 +537,43 @@ class HassIO:
             "/supervisor/options", payload={"diagnostics": diagnostics}
         )
 
+    @_api_bool
+    def apply_suggestion(self, suggestion_uuid: str) -> Coroutine:
+        """Apply a suggestion from supervisor's resolution center.
+
+        This method returns a coroutine.
+        """
+        return self.send_command(f"/resolution/suggestion/{suggestion_uuid}")
+
     async def send_command(
         self,
-        command,
-        method="post",
-        payload=None,
-        timeout=10,
-        return_text=False,
+        command: str,
+        method: str = "post",
+        payload: Any | None = None,
+        timeout: int | None = 10,
+        return_text: bool = False,
         *,
-        source="core.handler",
-    ):
+        source: str = "core.handler",
+    ) -> Any:
         """Send API command to Hass.io.
 
         This method is a coroutine.
         """
+        url = f"http://{self._ip}{command}"
+        joined_url = self._base_url.join(URL(command))
+        # This check is to make sure the normalized URL string
+        # is the same as the URL string that was passed in. If
+        # they are different, then the passed in command URL
+        # contained characters that were removed by the normalization
+        # such as ../../../../etc/passwd
+        if url != str(joined_url):
+            _LOGGER.error("Invalid request %s", command)
+            raise HassioAPIError()
+
         try:
             request = await self.websession.request(
                 method,
-                f"http://{self._ip}{command}",
+                joined_url,
                 json=payload,
                 headers={
                     aiohttp.hdrs.AUTHORIZATION: (
@@ -489,7 +591,7 @@ class HassIO:
             if return_text:
                 return await request.text(encoding="utf-8")
 
-            return await request.json()
+            return await request.json(encoding="utf-8")
 
         except asyncio.TimeoutError:
             _LOGGER.error("Timeout on %s request", command)
