@@ -1,13 +1,16 @@
 """Provide info to system health."""
+from __future__ import annotations
+
 import os
+from typing import Any
 
 from homeassistant.components import system_health
 from homeassistant.core import HomeAssistant, callback
 
 from . import get_host_info, get_info, get_os_info, get_supervisor_info
 
-SUPERVISOR_PING = f"http://{os.environ['HASSIO']}/supervisor/ping"
-OBSERVER_URL = f"http://{os.environ['HASSIO']}:4357"
+SUPERVISOR_PING = f"http://{os.environ['SUPERVISOR']}/supervisor/ping"
+OBSERVER_URL = f"http://{os.environ['SUPERVISOR']}:4357"
 
 
 @callback
@@ -18,13 +21,14 @@ def async_register(
     register.async_register_info(system_health_info)
 
 
-async def system_health_info(hass: HomeAssistant):
+async def system_health_info(hass: HomeAssistant) -> dict[str, Any]:
     """Get info for the info page."""
-    info = get_info(hass)
-    host_info = get_host_info(hass)
+    info = get_info(hass) or {}
+    host_info = get_host_info(hass) or {}
     supervisor_info = get_supervisor_info(hass)
 
-    if supervisor_info.get("healthy"):
+    healthy: bool | dict[str, str]
+    if supervisor_info is not None and supervisor_info.get("healthy"):
         healthy = True
     else:
         healthy = {
@@ -32,7 +36,8 @@ async def system_health_info(hass: HomeAssistant):
             "error": "Unhealthy",
         }
 
-    if supervisor_info.get("supported"):
+    supported: bool | dict[str, str]
+    if supervisor_info is not None and supervisor_info.get("supported"):
         supported = True
     else:
         supported = {
@@ -53,7 +58,7 @@ async def system_health_info(hass: HomeAssistant):
     }
 
     if info.get("hassos") is not None:
-        os_info = get_os_info(hass)
+        os_info = get_os_info(hass) or {}
         information["board"] = os_info.get("board")
 
     information["supervisor_api"] = system_health.async_check_can_reach_url(
@@ -66,7 +71,7 @@ async def system_health_info(hass: HomeAssistant):
 
     information["installed_addons"] = ", ".join(
         f"{addon['name']} ({addon['version']})"
-        for addon in supervisor_info.get("addons", [])
+        for addon in (supervisor_info or {}).get("addons", [])
     )
 
     return information
