@@ -1,12 +1,10 @@
 """The HERE Travel Time integration."""
 from __future__ import annotations
 
-import logging
-
 from homeassistant.config_entries import ConfigEntry
 from homeassistant.const import CONF_API_KEY, CONF_MODE, Platform
 from homeassistant.core import HomeAssistant
-from homeassistant.util import dt
+from homeassistant.util import dt as dt_util
 
 from .const import (
     CONF_ARRIVAL_TIME,
@@ -28,23 +26,14 @@ from .coordinator import (
 from .model import HERETravelTimeConfig
 
 PLATFORMS = [Platform.SENSOR]
-_LOGGER = logging.getLogger(__name__)
 
 
 async def async_setup_entry(hass: HomeAssistant, config_entry: ConfigEntry) -> bool:
     """Set up HERE Travel Time from a config entry."""
     api_key = config_entry.data[CONF_API_KEY]
 
-    arrival = (
-        dt.parse_time(config_entry.options[CONF_ARRIVAL_TIME])
-        if config_entry.options[CONF_ARRIVAL_TIME] is not None
-        else None
-    )
-    departure = (
-        dt.parse_time(config_entry.options[CONF_DEPARTURE_TIME])
-        if config_entry.options[CONF_DEPARTURE_TIME] is not None
-        else None
-    )
+    arrival = dt_util.parse_time(config_entry.options.get(CONF_ARRIVAL_TIME, ""))
+    departure = dt_util.parse_time(config_entry.options.get(CONF_DEPARTURE_TIME, ""))
 
     here_travel_time_config = HERETravelTimeConfig(
         destination_latitude=config_entry.data.get(CONF_DESTINATION_LATITUDE),
