@@ -1,6 +1,8 @@
 """Class to hold all media player accessories."""
 import logging
+from typing import Any
 
+from pyhap.characteristic import Characteristic
 from pyhap.const import CATEGORY_SWITCH
 
 from homeassistant.components.media_player import (
@@ -10,12 +12,7 @@ from homeassistant.components.media_player import (
     ATTR_MEDIA_VOLUME_MUTED,
     DOMAIN,
     SERVICE_SELECT_SOURCE,
-    SUPPORT_PAUSE,
-    SUPPORT_PLAY,
-    SUPPORT_SELECT_SOURCE,
-    SUPPORT_VOLUME_MUTE,
-    SUPPORT_VOLUME_SET,
-    SUPPORT_VOLUME_STEP,
+    MediaPlayerEntityFeature,
 )
 from homeassistant.const import (
     ATTR_ENTITY_ID,
@@ -36,11 +33,12 @@ from homeassistant.const import (
     STATE_STANDBY,
     STATE_UNKNOWN,
 )
-from homeassistant.core import callback
+from homeassistant.core import State, callback
 
 from .accessories import TYPES, HomeAccessory
 from .const import (
     ATTR_KEY_NAME,
+    CATEGORY_RECEIVER,
     CHAR_ACTIVE,
     CHAR_MUTE,
     CHAR_NAME,
@@ -59,7 +57,7 @@ from .const import (
     SERV_TELEVISION_SPEAKER,
 )
 from .type_remotes import REMOTE_KEYS, RemoteInputSelectAccessory
-from .util import get_media_player_features
+from .util import cleanup_name_for_homekit, get_media_player_features
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -85,11 +83,12 @@ MEDIA_PLAYER_OFF_STATES = (
 class MediaPlayer(HomeAccessory):
     """Generate a Media Player accessory."""
 
-    def __init__(self, *args):
+    def __init__(self, *args: Any) -> None:
         """Initialize a Switch accessory object."""
         super().__init__(*args, category=CATEGORY_SWITCH)
         state = self.hass.states.get(self.entity_id)
-        self.chars = {
+        assert state
+        self.chars: dict[str, Characteristic | None] = {
             FEATURE_ON_OFF: None,
             FEATURE_PLAY_PAUSE: None,
             FEATURE_PLAY_STOP: None,
@@ -101,7 +100,9 @@ class MediaPlayer(HomeAccessory):
 
         if FEATURE_ON_OFF in feature_list:
             name = self.generate_service_name(FEATURE_ON_OFF)
-            serv_on_off = self.add_preload_service(SERV_SWITCH, CHAR_NAME)
+            serv_on_off = self.add_preload_service(
+                SERV_SWITCH, CHAR_NAME, unique_id=FEATURE_ON_OFF
+            )
             serv_on_off.configure_char(CHAR_NAME, value=name)
             self.chars[FEATURE_ON_OFF] = serv_on_off.configure_char(
                 CHAR_ON, value=False, setter_callback=self.set_on_off
@@ -109,7 +110,9 @@ class MediaPlayer(HomeAccessory):
 
         if FEATURE_PLAY_PAUSE in feature_list:
             name = self.generate_service_name(FEATURE_PLAY_PAUSE)
-            serv_play_pause = self.add_preload_service(SERV_SWITCH, CHAR_NAME)
+            serv_play_pause = self.add_preload_service(
+                SERV_SWITCH, CHAR_NAME, unique_id=FEATURE_PLAY_PAUSE
+            )
             serv_play_pause.configure_char(CHAR_NAME, value=name)
             self.chars[FEATURE_PLAY_PAUSE] = serv_play_pause.configure_char(
                 CHAR_ON, value=False, setter_callback=self.set_play_pause
@@ -117,7 +120,9 @@ class MediaPlayer(HomeAccessory):
 
         if FEATURE_PLAY_STOP in feature_list:
             name = self.generate_service_name(FEATURE_PLAY_STOP)
-            serv_play_stop = self.add_preload_service(SERV_SWITCH, CHAR_NAME)
+            serv_play_stop = self.add_preload_service(
+                SERV_SWITCH, CHAR_NAME, unique_id=FEATURE_PLAY_STOP
+            )
             serv_play_stop.configure_char(CHAR_NAME, value=name)
             self.chars[FEATURE_PLAY_STOP] = serv_play_stop.configure_char(
                 CHAR_ON, value=False, setter_callback=self.set_play_stop
@@ -125,25 +130,29 @@ class MediaPlayer(HomeAccessory):
 
         if FEATURE_TOGGLE_MUTE in feature_list:
             name = self.generate_service_name(FEATURE_TOGGLE_MUTE)
-            serv_toggle_mute = self.add_preload_service(SERV_SWITCH, CHAR_NAME)
+            serv_toggle_mute = self.add_preload_service(
+                SERV_SWITCH, CHAR_NAME, unique_id=FEATURE_TOGGLE_MUTE
+            )
             serv_toggle_mute.configure_char(CHAR_NAME, value=name)
             self.chars[FEATURE_TOGGLE_MUTE] = serv_toggle_mute.configure_char(
                 CHAR_ON, value=False, setter_callback=self.set_toggle_mute
             )
         self.async_update_state(state)
 
-    def generate_service_name(self, mode):
+    def generate_service_name(self, mode: str) -> str:
         """Generate name for individual service."""
-        return f"{self.display_name} {MODE_FRIENDLY_NAME[mode]}"
+        return cleanup_name_for_homekit(
+            f"{self.display_name} {MODE_FRIENDLY_NAME[mode]}"
+        )
 
-    def set_on_off(self, value):
+    def set_on_off(self, value: bool) -> None:
         """Move switch state to value if call came from HomeKit."""
         _LOGGER.debug('%s: Set switch state for "on_off" to %s', self.entity_id, value)
         service = SERVICE_TURN_ON if value else SERVICE_TURN_OFF
         params = {ATTR_ENTITY_ID: self.entity_id}
         self.async_call_service(DOMAIN, service, params)
 
-    def set_play_pause(self, value):
+    def set_play_pause(self, value: bool) -> None:
         """Move switch state to value if call came from HomeKit."""
         _LOGGER.debug(
             '%s: Set switch state for "play_pause" to %s', self.entity_id, value
@@ -152,7 +161,7 @@ class MediaPlayer(HomeAccessory):
         params = {ATTR_ENTITY_ID: self.entity_id}
         self.async_call_service(DOMAIN, service, params)
 
-    def set_play_stop(self, value):
+    def set_play_stop(self, value: bool) -> None:
         """Move switch state to value if call came from HomeKit."""
         _LOGGER.debug(
             '%s: Set switch state for "play_stop" to %s', self.entity_id, value
@@ -161,7 +170,7 @@ class MediaPlayer(HomeAccessory):
         params = {ATTR_ENTITY_ID: self.entity_id}
         self.async_call_service(DOMAIN, service, params)
 
-    def set_toggle_mute(self, value):
+    def set_toggle_mute(self, value: bool) -> None:
         """Move switch state to value if call came from HomeKit."""
         _LOGGER.debug(
             '%s: Set switch state for "toggle_mute" to %s', self.entity_id, value
@@ -170,72 +179,75 @@ class MediaPlayer(HomeAccessory):
         self.async_call_service(DOMAIN, SERVICE_VOLUME_MUTE, params)
 
     @callback
-    def async_update_state(self, new_state):
+    def async_update_state(self, new_state: State) -> None:
         """Update switch state after state changed."""
         current_state = new_state.state
 
-        if self.chars[FEATURE_ON_OFF]:
+        if on_off_char := self.chars[FEATURE_ON_OFF]:
             hk_state = current_state not in MEDIA_PLAYER_OFF_STATES
             _LOGGER.debug(
                 '%s: Set current state for "on_off" to %s', self.entity_id, hk_state
             )
-            if self.chars[FEATURE_ON_OFF].value != hk_state:
-                self.chars[FEATURE_ON_OFF].set_value(hk_state)
+            on_off_char.set_value(hk_state)
 
-        if self.chars[FEATURE_PLAY_PAUSE]:
+        if play_pause_char := self.chars[FEATURE_PLAY_PAUSE]:
             hk_state = current_state == STATE_PLAYING
             _LOGGER.debug(
                 '%s: Set current state for "play_pause" to %s',
                 self.entity_id,
                 hk_state,
             )
-            if self.chars[FEATURE_PLAY_PAUSE].value != hk_state:
-                self.chars[FEATURE_PLAY_PAUSE].set_value(hk_state)
+            play_pause_char.set_value(hk_state)
 
-        if self.chars[FEATURE_PLAY_STOP]:
+        if play_stop_char := self.chars[FEATURE_PLAY_STOP]:
             hk_state = current_state == STATE_PLAYING
             _LOGGER.debug(
                 '%s: Set current state for "play_stop" to %s',
                 self.entity_id,
                 hk_state,
             )
-            if self.chars[FEATURE_PLAY_STOP].value != hk_state:
-                self.chars[FEATURE_PLAY_STOP].set_value(hk_state)
+            play_stop_char.set_value(hk_state)
 
-        if self.chars[FEATURE_TOGGLE_MUTE]:
-            current_state = bool(new_state.attributes.get(ATTR_MEDIA_VOLUME_MUTED))
+        if toggle_mute_char := self.chars[FEATURE_TOGGLE_MUTE]:
+            mute_state = bool(new_state.attributes.get(ATTR_MEDIA_VOLUME_MUTED))
             _LOGGER.debug(
                 '%s: Set current state for "toggle_mute" to %s',
                 self.entity_id,
-                current_state,
+                mute_state,
             )
-            if self.chars[FEATURE_TOGGLE_MUTE].value != current_state:
-                self.chars[FEATURE_TOGGLE_MUTE].set_value(current_state)
+            toggle_mute_char.set_value(mute_state)
 
 
 @TYPES.register("TelevisionMediaPlayer")
 class TelevisionMediaPlayer(RemoteInputSelectAccessory):
     """Generate a Television Media Player accessory."""
 
-    def __init__(self, *args):
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         """Initialize a Television Media Player accessory object."""
         super().__init__(
-            SUPPORT_SELECT_SOURCE,
+            MediaPlayerEntityFeature.SELECT_SOURCE,
             ATTR_INPUT_SOURCE,
             ATTR_INPUT_SOURCE_LIST,
             *args,
+            **kwargs,
         )
         state = self.hass.states.get(self.entity_id)
+        assert state
         features = state.attributes.get(ATTR_SUPPORTED_FEATURES, 0)
 
-        self.chars_speaker = []
+        self.chars_speaker: list[str] = []
 
-        self._supports_play_pause = features & (SUPPORT_PLAY | SUPPORT_PAUSE)
-        if features & SUPPORT_VOLUME_MUTE or features & SUPPORT_VOLUME_STEP:
+        self._supports_play_pause = features & (
+            MediaPlayerEntityFeature.PLAY | MediaPlayerEntityFeature.PAUSE
+        )
+        if (
+            features & MediaPlayerEntityFeature.VOLUME_MUTE
+            or features & MediaPlayerEntityFeature.VOLUME_STEP
+        ):
             self.chars_speaker.extend(
                 (CHAR_NAME, CHAR_ACTIVE, CHAR_VOLUME_CONTROL_TYPE, CHAR_VOLUME_SELECTOR)
             )
-            if features & SUPPORT_VOLUME_SET:
+            if features & MediaPlayerEntityFeature.VOLUME_SET:
                 self.chars_speaker.append(CHAR_VOLUME)
 
         if CHAR_VOLUME_SELECTOR in self.chars_speaker:
@@ -268,14 +280,14 @@ class TelevisionMediaPlayer(RemoteInputSelectAccessory):
 
         self.async_update_state(state)
 
-    def set_on_off(self, value):
+    def set_on_off(self, value: bool) -> None:
         """Move switch state to value if call came from HomeKit."""
         _LOGGER.debug('%s: Set switch state for "on_off" to %s', self.entity_id, value)
         service = SERVICE_TURN_ON if value else SERVICE_TURN_OFF
         params = {ATTR_ENTITY_ID: self.entity_id}
         self.async_call_service(DOMAIN, service, params)
 
-    def set_mute(self, value):
+    def set_mute(self, value: bool) -> None:
         """Move switch state to value if call came from HomeKit."""
         _LOGGER.debug(
             '%s: Set switch state for "toggle_mute" to %s', self.entity_id, value
@@ -283,37 +295,38 @@ class TelevisionMediaPlayer(RemoteInputSelectAccessory):
         params = {ATTR_ENTITY_ID: self.entity_id, ATTR_MEDIA_VOLUME_MUTED: value}
         self.async_call_service(DOMAIN, SERVICE_VOLUME_MUTE, params)
 
-    def set_volume(self, value):
+    def set_volume(self, value: bool) -> None:
         """Send volume step value if call came from HomeKit."""
         _LOGGER.debug("%s: Set volume to %s", self.entity_id, value)
         params = {ATTR_ENTITY_ID: self.entity_id, ATTR_MEDIA_VOLUME_LEVEL: value}
         self.async_call_service(DOMAIN, SERVICE_VOLUME_SET, params)
 
-    def set_volume_step(self, value):
+    def set_volume_step(self, value: bool) -> None:
         """Send volume step value if call came from HomeKit."""
         _LOGGER.debug("%s: Step volume by %s", self.entity_id, value)
         service = SERVICE_VOLUME_DOWN if value else SERVICE_VOLUME_UP
         params = {ATTR_ENTITY_ID: self.entity_id}
         self.async_call_service(DOMAIN, service, params)
 
-    def set_input_source(self, value):
+    def set_input_source(self, value: int) -> None:
         """Send input set value if call came from HomeKit."""
         _LOGGER.debug("%s: Set current input to %s", self.entity_id, value)
-        source = self.sources[value]
-        params = {ATTR_ENTITY_ID: self.entity_id, ATTR_INPUT_SOURCE: source}
+        source_name = self._mapped_sources[self.sources[value]]
+        params = {ATTR_ENTITY_ID: self.entity_id, ATTR_INPUT_SOURCE: source_name}
         self.async_call_service(DOMAIN, SERVICE_SELECT_SOURCE, params)
 
-    def set_remote_key(self, value):
+    def set_remote_key(self, value: int) -> None:
         """Send remote key value if call came from HomeKit."""
         _LOGGER.debug("%s: Set remote key to %s", self.entity_id, value)
-        key_name = REMOTE_KEYS.get(value)
-        if key_name is None:
+        if (key_name := REMOTE_KEYS.get(value)) is None:
             _LOGGER.warning("%s: Unhandled key press for %s", self.entity_id, value)
             return
 
         if key_name == KEY_PLAY_PAUSE and self._supports_play_pause:
             # Handle Play Pause by directly updating the media player entity.
-            state = self.hass.states.get(self.entity_id).state
+            state_obj = self.hass.states.get(self.entity_id)
+            assert state_obj
+            state = state_obj.state
             if state in (STATE_PLAYING, STATE_PAUSED):
                 service = (
                     SERVICE_MEDIA_PLAY if state == STATE_PAUSED else SERVICE_MEDIA_PAUSE
@@ -331,7 +344,7 @@ class TelevisionMediaPlayer(RemoteInputSelectAccessory):
         )
 
     @callback
-    def async_update_state(self, new_state):
+    def async_update_state(self, new_state: State) -> None:
         """Update Television state after state changed."""
         current_state = new_state.state
 
@@ -340,8 +353,7 @@ class TelevisionMediaPlayer(RemoteInputSelectAccessory):
         if current_state not in MEDIA_PLAYER_OFF_STATES:
             hk_state = 1
         _LOGGER.debug("%s: Set current active state to %s", self.entity_id, hk_state)
-        if self.char_active.value != hk_state:
-            self.char_active.set_value(hk_state)
+        self.char_active.set_value(hk_state)
 
         # Set mute state
         if CHAR_VOLUME_SELECTOR in self.chars_speaker:
@@ -351,7 +363,20 @@ class TelevisionMediaPlayer(RemoteInputSelectAccessory):
                 self.entity_id,
                 current_mute_state,
             )
-            if self.char_mute.value != current_mute_state:
-                self.char_mute.set_value(current_mute_state)
+            self.char_mute.set_value(current_mute_state)
 
         self._async_update_input_state(hk_state, new_state)
+
+
+@TYPES.register("ReceiverMediaPlayer")
+class ReceiverMediaPlayer(TelevisionMediaPlayer):
+    """Generate a Receiver Media Player accessory.
+
+    For HomeKit, a Receiver Media Player is exactly the same as a
+    Television Media Player except it has a different category
+    which will tell HomeKit how to render the device.
+    """
+
+    def __init__(self, *args: Any) -> None:
+        """Initialize a Receiver Media Player accessory object."""
+        super().__init__(*args, category=CATEGORY_RECEIVER)
