@@ -25,8 +25,9 @@ from homeassistant.components.lock import DOMAIN as LOCK_DOMAIN
 from homeassistant.components.media_player import (
     DOMAIN as MEDIA_PLAYER_DOMAIN,
     MediaPlayerDeviceClass,
+    MediaPlayerEntityFeature,
 )
-from homeassistant.components.remote import DOMAIN as REMOTE_DOMAIN, SUPPORT_ACTIVITY
+from homeassistant.components.remote import DOMAIN as REMOTE_DOMAIN, RemoteEntityFeature
 from homeassistant.const import (
     ATTR_CODE,
     ATTR_DEVICE_CLASS,
@@ -34,12 +35,14 @@ from homeassistant.const import (
     CONF_NAME,
     CONF_PORT,
     CONF_TYPE,
-    TEMP_CELSIUS,
+    UnitOfTemperature,
 )
 from homeassistant.core import HomeAssistant, State, callback, split_entity_id
 import homeassistant.helpers.config_validation as cv
+from homeassistant.helpers.event import EventStateChangedData
 from homeassistant.helpers.storage import STORAGE_DIR
-import homeassistant.util.temperature as temp_util
+from homeassistant.helpers.typing import EventType
+from homeassistant.util.unit_conversion import TemperatureConverter
 
 from .const import (
     AUDIO_CODEC_COPY,
@@ -83,8 +86,6 @@ from .const import (
     FEATURE_PLAY_PAUSE,
     FEATURE_PLAY_STOP,
     FEATURE_TOGGLE_MUTE,
-    HOMEKIT_PAIRING_QR,
-    HOMEKIT_PAIRING_QR_SECRET,
     MAX_NAME_LENGTH,
     TYPE_FAUCET,
     TYPE_OUTLET,
@@ -94,8 +95,10 @@ from .const import (
     TYPE_VALVE,
     VIDEO_CODEC_COPY,
     VIDEO_CODEC_H264_OMX,
+    VIDEO_CODEC_H264_V4L2M2M,
     VIDEO_CODEC_LIBX264,
 )
+from .models import HomeKitEntryData
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -106,7 +109,12 @@ MAX_VERSION_PART = 2**32 - 1
 
 
 MAX_PORT = 65535
-VALID_VIDEO_CODECS = [VIDEO_CODEC_LIBX264, VIDEO_CODEC_H264_OMX, AUDIO_CODEC_COPY]
+VALID_VIDEO_CODECS = [
+    VIDEO_CODEC_LIBX264,
+    VIDEO_CODEC_H264_OMX,
+    VIDEO_CODEC_H264_V4L2M2M,
+    AUDIO_CODEC_COPY,
+]
 VALID_AUDIO_CODECS = [AUDIO_CODEC_OPUS, VIDEO_CODEC_COPY]
 
 BASIC_INFO_SCHEMA = vol.Schema(
@@ -296,14 +304,14 @@ def get_media_player_features(state: State) -> list[str]:
 
     supported_modes = []
     if features & (
-        media_player.const.SUPPORT_TURN_ON | media_player.const.SUPPORT_TURN_OFF
+        MediaPlayerEntityFeature.TURN_ON | MediaPlayerEntityFeature.TURN_OFF
     ):
         supported_modes.append(FEATURE_ON_OFF)
-    if features & (media_player.const.SUPPORT_PLAY | media_player.const.SUPPORT_PAUSE):
+    if features & (MediaPlayerEntityFeature.PLAY | MediaPlayerEntityFeature.PAUSE):
         supported_modes.append(FEATURE_PLAY_PAUSE)
-    if features & (media_player.const.SUPPORT_PLAY | media_player.const.SUPPORT_STOP):
+    if features & (MediaPlayerEntityFeature.PLAY | MediaPlayerEntityFeature.STOP):
         supported_modes.append(FEATURE_PLAY_STOP)
-    if features & media_player.const.SUPPORT_VOLUME_MUTE:
+    if features & MediaPlayerEntityFeature.VOLUME_MUTE:
         supported_modes.append(FEATURE_TOGGLE_MUTE)
     return supported_modes
 
@@ -343,12 +351,14 @@ def async_show_setup_message(
     url.svg(buffer, scale=5, module_color="#000", background="#FFF")
     pairing_secret = secrets.token_hex(32)
 
-    hass.data[DOMAIN][entry_id][HOMEKIT_PAIRING_QR] = buffer.getvalue()
-    hass.data[DOMAIN][entry_id][HOMEKIT_PAIRING_QR_SECRET] = pairing_secret
+    entry_data: HomeKitEntryData = hass.data[DOMAIN][entry_id]
+
+    entry_data.pairing_qr = buffer.getvalue()
+    entry_data.pairing_qr_secret = pairing_secret
 
     message = (
         f"To set up {bridge_name} in the Home App, "
-        f"scan the QR code or enter the following code:\n"
+        "scan the QR code or enter the following code:\n"
         f"### {pin}\n"
         f"![image](/api/homekit/pairingqr?{entry_id}-{pairing_secret})"
     )
@@ -390,51 +400,77 @@ def cleanup_name_for_homekit(name: str | None) -> str:
 
 def temperature_to_homekit(temperature: float | int, unit: str) -> float:
     """Convert temperature to Celsius for HomeKit."""
-    return round(temp_util.convert(temperature, unit, TEMP_CELSIUS), 1)
+    return round(
+        TemperatureConverter.convert(temperature, unit, UnitOfTemperature.CELSIUS), 1
+    )
 
 
 def temperature_to_states(temperature: float | int, unit: str) -> float:
     """Convert temperature back from Celsius to Home Assistant unit."""
-    return round(temp_util.convert(temperature, TEMP_CELSIUS, unit) * 2) / 2
+    return (
+        round(
+            TemperatureConverter.convert(temperature, UnitOfTemperature.CELSIUS, unit)
+            * 2
+        )
+        / 2
+    )
 
 
 def density_to_air_quality(density: float) -> int:
-    """Map PM2.5 density to HomeKit AirQuality level."""
-    if density <= 35:
+    """Map PM2.5 µg/m3 density to HomeKit AirQuality level."""
+    if density <= 12:  # US AQI 0-50 (HomeKit: Excellent)
         return 1
-    if density <= 75:
+    if density <= 35.4:  # US AQI 51-100 (HomeKit: Good)
         return 2
-    if density <= 115:
+    if density <= 55.4:  # US AQI 101-150 (HomeKit: Fair)
         return 3
-    if density <= 150:
+    if density <= 150.4:  # US AQI 151-200 (HomeKit: Inferior)
         return 4
-    return 5
+    return 5  # US AQI 201+ (HomeKit: Poor)
 
 
 def density_to_air_quality_pm10(density: float) -> int:
-    """Map PM10 density to HomeKit AirQuality level."""
-    if density <= 40:
+    """Map PM10 µg/m3 density to HomeKit AirQuality level."""
+    if density <= 54:  # US AQI 0-50 (HomeKit: Excellent)
         return 1
+    if density <= 154:  # US AQI 51-100 (HomeKit: Good)
+        return 2
+    if density <= 254:  # US AQI 101-150 (HomeKit: Fair)
+        return 3
+    if density <= 354:  # US AQI 151-200 (HomeKit: Inferior)
+        return 4
+    return 5  # US AQI 201+ (HomeKit: Poor)
+
+
+def density_to_air_quality_nitrogen_dioxide(density: float) -> int:
+    """Map nitrogen dioxide µg/m3 to HomeKit AirQuality level."""
+    if density <= 30:
+        return 1
+    if density <= 60:
+        return 2
     if density <= 80:
-        return 2
-    if density <= 120:
         return 3
-    if density <= 300:
+    if density <= 90:
         return 4
     return 5
 
 
-def density_to_air_quality_pm25(density: float) -> int:
-    """Map PM2.5 density to HomeKit AirQuality level."""
-    if density <= 25:
+def density_to_air_quality_voc(density: float) -> int:
+    """Map VOCs µg/m3 to HomeKit AirQuality level.
+
+    The VOC mappings use the IAQ guidelines for Europe released by the WHO (World Health Organization).
+    Referenced from Sensirion_Gas_Sensors_SGP3x_TVOC_Concept.pdf
+    https://github.com/paulvha/svm30/blob/master/extras/Sensirion_Gas_Sensors_SGP3x_TVOC_Concept.pdf
+    """
+    if density <= 250:  # WHO IAQ 1 (HomeKit: Excellent)
         return 1
-    if density <= 50:
+    if density <= 500:  # WHO IAQ 2 (HomeKit: Good)
         return 2
-    if density <= 100:
+    if density <= 1000:  # WHO IAQ 3 (HomeKit: Fair)
         return 3
-    if density <= 300:
+    if density <= 3000:  # WHO IAQ 4 (HomeKit: Inferior)
         return 4
-    return 5
+    return 5  # WHOA IAQ 5 (HomeKit: Poor)
 
 
 def get_persist_filename_for_entry_id(entry_id: str) -> str:
@@ -443,10 +479,15 @@ def get_persist_filename_for_entry_id(entry_id: str) -> str:
 
 
 def get_aid_storage_filename_for_entry_id(entry_id: str) -> str:
-    """Determine the ilename of homekit aid storage file."""
+    """Determine the filename of homekit aid storage file."""
     return f"{DOMAIN}.{entry_id}.aids"
 
 
+def get_iid_storage_filename_for_entry_id(entry_id: str) -> str:
+    """Determine the filename of homekit iid storage file."""
+    return f"{DOMAIN}.{entry_id}.iids"
+
+
 def get_persist_fullpath_for_entry_id(hass: HomeAssistant, entry_id: str) -> str:
     """Determine the path to the homekit state file."""
     return hass.config.path(STORAGE_DIR, get_persist_filename_for_entry_id(entry_id))
@@ -459,6 +500,13 @@ def get_aid_storage_fullpath_for_entry_id(hass: HomeAssistant, entry_id: str) ->
     )
 
 
+def get_iid_storage_fullpath_for_entry_id(hass: HomeAssistant, entry_id: str) -> str:
+    """Determine the path to the homekit iid storage file."""
+    return hass.config.path(
+        STORAGE_DIR, get_iid_storage_filename_for_entry_id(entry_id)
+    )
+
+
 def _format_version_part(version_part: str) -> str:
     return str(max(0, min(MAX_VERSION_PART, coerce_int(version_part))))
 
@@ -478,14 +526,15 @@ def _is_zero_but_true(value: Any) -> bool:
     return convert_to_float(value) == 0
 
 
-def remove_state_files_for_entry_id(hass: HomeAssistant, entry_id: str) -> bool:
+def remove_state_files_for_entry_id(hass: HomeAssistant, entry_id: str) -> None:
     """Remove the state files from disk."""
-    persist_file_path = get_persist_fullpath_for_entry_id(hass, entry_id)
-    aid_storage_path = get_aid_storage_fullpath_for_entry_id(hass, entry_id)
-    os.unlink(persist_file_path)
-    if os.path.exists(aid_storage_path):
-        os.unlink(aid_storage_path)
-    return True
+    for path in (
+        get_persist_fullpath_for_entry_id(hass, entry_id),
+        get_aid_storage_fullpath_for_entry_id(hass, entry_id),
+        get_iid_storage_fullpath_for_entry_id(hass, entry_id),
+    ):
+        if os.path.exists(path):
+            os.unlink(path)
 
 
 def _get_test_socket() -> socket.socket:
@@ -566,7 +615,17 @@ def state_needs_accessory_mode(state: State) -> bool:
 
     return (
         state.domain == MEDIA_PLAYER_DOMAIN
-        and state.attributes.get(ATTR_DEVICE_CLASS) == MediaPlayerDeviceClass.TV
+        and state.attributes.get(ATTR_DEVICE_CLASS)
+        in (MediaPlayerDeviceClass.TV, MediaPlayerDeviceClass.RECEIVER)
         or state.domain == REMOTE_DOMAIN
-        and state.attributes.get(ATTR_SUPPORTED_FEATURES, 0) & SUPPORT_ACTIVITY
+        and state.attributes.get(ATTR_SUPPORTED_FEATURES, 0)
+        & RemoteEntityFeature.ACTIVITY
     )
+
+
+def state_changed_event_is_same_state(event: EventType[EventStateChangedData]) -> bool:
+    """Check if a state changed event is the same state."""
+    event_data = event.data
+    old_state = event_data["old_state"]
+    new_state = event_data["new_state"]
+    return bool(new_state and old_state and new_state.state == old_state.state)
