@@ -2,7 +2,7 @@
 from __future__ import annotations
 
 import logging
-from typing import Any, Final
+from typing import TYPE_CHECKING, Any, Final
 
 from aiohomekit.model.characteristics import (
     ActivationStateValues,
@@ -23,6 +23,10 @@ from homeassistant.components.climate import (
     DEFAULT_MAX_TEMP,
     DEFAULT_MIN_TEMP,
     FAN_AUTO,
+    FAN_HIGH,
+    FAN_LOW,
+    FAN_MEDIUM,
+    FAN_OFF,
     FAN_ON,
     SWING_OFF,
     SWING_VERTICAL,
@@ -32,14 +36,24 @@ from homeassistant.components.climate import (
     HVACMode,
 )
 from homeassistant.config_entries import ConfigEntry
-from homeassistant.const import ATTR_TEMPERATURE, TEMP_CELSIUS, Platform
+from homeassistant.const import ATTR_TEMPERATURE, Platform, UnitOfTemperature
 from homeassistant.core import HomeAssistant, callback
 from homeassistant.helpers.entity_platform import AddEntitiesCallback
+from homeassistant.util.percentage import (
+    percentage_to_ranged_value,
+    ranged_value_to_percentage,
+)
 
 from . import KNOWN_DEVICES
 from .connection import HKDevice
 from .entity import HomeKitEntity
 
+if TYPE_CHECKING:
+    from functools import cached_property
+else:
+    from homeassistant.backports.functools import cached_property
+
+
 _LOGGER = logging.getLogger(__name__)
 
 # Map of Homekit operation modes to hass modes
@@ -86,6 +100,16 @@ SWING_MODE_HASS_TO_HOMEKIT = {v: k for k, v in SWING_MODE_HOMEKIT_TO_HASS.items(
 
 DEFAULT_MIN_STEP: Final = 1.0
 
+ROTATION_SPEED_LOW = 33
+ROTATION_SPEED_MEDIUM = 66
+ROTATION_SPEED_HIGH = 100
+
+HASS_FAN_MODE_TO_HOMEKIT_ROTATION = {
+    FAN_LOW: ROTATION_SPEED_LOW,
+    FAN_MEDIUM: ROTATION_SPEED_MEDIUM,
+    FAN_HIGH: ROTATION_SPEED_HIGH,
+}
+
 
 async def async_setup_entry(
     hass: HomeAssistant,
@@ -114,7 +138,14 @@ async def async_setup_entry(
 class HomeKitBaseClimateEntity(HomeKitEntity, ClimateEntity):
     """The base HomeKit Controller climate entity."""
 
-    _attr_temperature_unit = TEMP_CELSIUS
+    _attr_temperature_unit = UnitOfTemperature.CELSIUS
+    _enable_turn_on_off_backwards_compatibility = False
+
+    @callback
+    def _async_reconfigure(self) -> None:
+        """Reconfigure entity."""
+        self._async_clear_property_cache(("supported_features", "fan_modes"))
+        super()._async_reconfigure()
 
     def get_characteristic_types(self) -> list[str]:
         """Define the homekit characteristics the entity cares about."""
@@ -128,7 +159,7 @@ class HomeKitBaseClimateEntity(HomeKitEntity, ClimateEntity):
         """Return the current temperature."""
         return self.service.value(CharacteristicsTypes.TEMPERATURE_CURRENT)
 
-    @property
+    @cached_property
     def fan_modes(self) -> list[str] | None:
         """Return the available fan modes."""
         if self.service.has(CharacteristicsTypes.FAN_STATE_TARGET):
@@ -147,10 +178,10 @@ class HomeKitBaseClimateEntity(HomeKitEntity, ClimateEntity):
             {CharacteristicsTypes.FAN_STATE_TARGET: int(fan_mode == FAN_AUTO)}
         )
 
-    @property
-    def supported_features(self) -> int:
+    @cached_property
+    def supported_features(self) -> ClimateEntityFeature:
         """Return the list of supported features."""
-        features = 0
+        features = ClimateEntityFeature.TURN_OFF | ClimateEntityFeature.TURN_ON
 
         if self.service.has(CharacteristicsTypes.FAN_STATE_TARGET):
             features |= ClimateEntityFeature.FAN_MODE
@@ -161,6 +192,12 @@ class HomeKitBaseClimateEntity(HomeKitEntity, ClimateEntity):
 class HomeKitHeaterCoolerEntity(HomeKitBaseClimateEntity):
     """Representation of a Homekit climate device."""
 
+    @callback
+    def _async_reconfigure(self) -> None:
+        """Reconfigure entity."""
+        self._async_clear_property_cache(("hvac_modes", "swing_modes"))
+        super()._async_reconfigure()
+
     def get_characteristic_types(self) -> list[str]:
         """Define the homekit characteristics the entity cares about."""
         return super().get_characteristic_types() + [
@@ -170,8 +207,45 @@ class HomeKitHeaterCoolerEntity(HomeKitBaseClimateEntity):
             CharacteristicsTypes.TEMPERATURE_COOLING_THRESHOLD,
             CharacteristicsTypes.TEMPERATURE_HEATING_THRESHOLD,
             CharacteristicsTypes.SWING_MODE,
+            CharacteristicsTypes.ROTATION_SPEED,
         ]
 
+    def _get_rotation_speed_range(self) -> tuple[float, float]:
+        rotation_speed = self.service[CharacteristicsTypes.ROTATION_SPEED]
+        return round(rotation_speed.minValue or 0) + 1, round(
+            rotation_speed.maxValue or 100
+        )
+
+    @cached_property
+    def fan_modes(self) -> list[str]:
+        """Return the available fan modes."""
+        return [FAN_OFF, FAN_LOW, FAN_MEDIUM, FAN_HIGH]
+
+    @property
+    def fan_mode(self) -> str | None:
+        """Return the current fan mode."""
+        speed_range = self._get_rotation_speed_range()
+        speed_percentage = ranged_value_to_percentage(
+            speed_range, self.service.value(CharacteristicsTypes.ROTATION_SPEED)
+        )
+        # homekit value 0 33 66 100
+        if speed_percentage > ROTATION_SPEED_MEDIUM:
+            return FAN_HIGH
+        if speed_percentage > ROTATION_SPEED_LOW:
+            return FAN_MEDIUM
+        if speed_percentage > 0:
+            return FAN_LOW
+        return FAN_OFF
+
+    async def async_set_fan_mode(self, fan_mode: str) -> None:
+        """Set new target fan mode."""
+        rotation = HASS_FAN_MODE_TO_HOMEKIT_ROTATION.get(fan_mode, 0)
+        speed_range = self._get_rotation_speed_range()
+        speed = round(percentage_to_ranged_value(speed_range, rotation))
+        await self.async_put_characteristics(
+            {CharacteristicsTypes.ROTATION_SPEED: speed}
+        )
+
     async def async_set_temperature(self, **kwargs: Any) -> None:
         """Set new target temperature."""
         temp = kwargs.get(ATTR_TEMPERATURE)
@@ -187,8 +261,11 @@ class HomeKitHeaterCoolerEntity(HomeKitBaseClimateEntity):
         else:
             hvac_mode = TARGET_HEATER_COOLER_STATE_HOMEKIT_TO_HASS.get(state)
             _LOGGER.warning(
-                "HomeKit device %s: Setting temperature in %s mode is not supported yet;"
-                " Consider raising a ticket if you have this device and want to help us implement this feature",
+                (
+                    "HomeKit device %s: Setting temperature in %s mode is not supported"
+                    " yet; Consider raising a ticket if you have this device and want"
+                    " to help us implement this feature"
+                ),
                 self.entity_id,
                 hvac_mode,
             )
@@ -202,8 +279,11 @@ class HomeKitHeaterCoolerEntity(HomeKitBaseClimateEntity):
             return
         if hvac_mode not in {HVACMode.HEAT, HVACMode.COOL}:
             _LOGGER.warning(
-                "HomeKit device %s: Setting temperature in %s mode is not supported yet;"
-                " Consider raising a ticket if you have this device and want to help us implement this feature",
+                (
+                    "HomeKit device %s: Setting temperature in %s mode is not supported"
+                    " yet; Consider raising a ticket if you have this device and want"
+                    " to help us implement this feature"
+                ),
                 self.entity_id,
                 hvac_mode,
             )
@@ -327,7 +407,7 @@ class HomeKitHeaterCoolerEntity(HomeKitBaseClimateEntity):
         value = self.service.value(CharacteristicsTypes.TARGET_HEATER_COOLER_STATE)
         return TARGET_HEATER_COOLER_STATE_HOMEKIT_TO_HASS[value]
 
-    @property
+    @cached_property
     def hvac_modes(self) -> list[HVACMode]:
         """Return the list of available hvac operation modes."""
         valid_values = clamp_enum_to_char(
@@ -349,7 +429,7 @@ class HomeKitHeaterCoolerEntity(HomeKitBaseClimateEntity):
         value = self.service.value(CharacteristicsTypes.SWING_MODE)
         return SWING_MODE_HOMEKIT_TO_HASS[value]
 
-    @property
+    @cached_property
     def swing_modes(self) -> list[str]:
         """Return the list of available swing modes.
 
@@ -367,8 +447,8 @@ class HomeKitHeaterCoolerEntity(HomeKitBaseClimateEntity):
             {CharacteristicsTypes.SWING_MODE: SWING_MODE_HASS_TO_HOMEKIT[swing_mode]}
         )
 
-    @property
-    def supported_features(self) -> int:
+    @cached_property
+    def supported_features(self) -> ClimateEntityFeature:
         """Return the list of supported features."""
         features = super().supported_features
 
@@ -381,12 +461,21 @@ class HomeKitHeaterCoolerEntity(HomeKitBaseClimateEntity):
         if self.service.has(CharacteristicsTypes.SWING_MODE):
             features |= ClimateEntityFeature.SWING_MODE
 
+        if self.service.has(CharacteristicsTypes.ROTATION_SPEED):
+            features |= ClimateEntityFeature.FAN_MODE
+
         return features
 
 
 class HomeKitClimateEntity(HomeKitBaseClimateEntity):
     """Representation of a Homekit climate device."""
 
+    @callback
+    def _async_reconfigure(self) -> None:
+        """Reconfigure entity."""
+        self._async_clear_property_cache(("hvac_modes",))
+        super()._async_reconfigure()
+
     def get_characteristic_types(self) -> list[str]:
         """Define the homekit characteristics the entity cares about."""
         return super().get_characteristic_types() + [
@@ -419,7 +508,7 @@ class HomeKitClimateEntity(HomeKitBaseClimateEntity):
         if (
             (mode == HVACMode.HEAT_COOL)
             and (
-                ClimateEntityFeature.TARGET_TEMPERATURE_RANGE & self.supported_features
+                ClimateEntityFeature.TARGET_TEMPERATURE_RANGE in self.supported_features
             )
             and heat_temp
             and cool_temp
@@ -460,9 +549,8 @@ class HomeKitClimateEntity(HomeKitBaseClimateEntity):
         value = self.service.value(CharacteristicsTypes.HEATING_COOLING_TARGET)
         if (MODE_HOMEKIT_TO_HASS.get(value) in {HVACMode.HEAT, HVACMode.COOL}) or (
             (MODE_HOMEKIT_TO_HASS.get(value) in {HVACMode.HEAT_COOL})
-            and not (
-                ClimateEntityFeature.TARGET_TEMPERATURE_RANGE & self.supported_features
-            )
+            and ClimateEntityFeature.TARGET_TEMPERATURE_RANGE
+            not in self.supported_features
         ):
             return self.service.value(CharacteristicsTypes.TEMPERATURE_TARGET)
         return None
@@ -472,7 +560,7 @@ class HomeKitClimateEntity(HomeKitBaseClimateEntity):
         """Return the highbound target temperature we try to reach."""
         value = self.service.value(CharacteristicsTypes.HEATING_COOLING_TARGET)
         if (MODE_HOMEKIT_TO_HASS.get(value) in {HVACMode.HEAT_COOL}) and (
-            ClimateEntityFeature.TARGET_TEMPERATURE_RANGE & self.supported_features
+            ClimateEntityFeature.TARGET_TEMPERATURE_RANGE in self.supported_features
         ):
             return self.service.value(
                 CharacteristicsTypes.TEMPERATURE_COOLING_THRESHOLD
@@ -484,7 +572,7 @@ class HomeKitClimateEntity(HomeKitBaseClimateEntity):
         """Return the lowbound target temperature we try to reach."""
         value = self.service.value(CharacteristicsTypes.HEATING_COOLING_TARGET)
         if (MODE_HOMEKIT_TO_HASS.get(value) in {HVACMode.HEAT_COOL}) and (
-            ClimateEntityFeature.TARGET_TEMPERATURE_RANGE & self.supported_features
+            ClimateEntityFeature.TARGET_TEMPERATURE_RANGE in self.supported_features
         ):
             return self.service.value(
                 CharacteristicsTypes.TEMPERATURE_HEATING_THRESHOLD
@@ -496,7 +584,7 @@ class HomeKitClimateEntity(HomeKitBaseClimateEntity):
         """Return the minimum target temp."""
         value = self.service.value(CharacteristicsTypes.HEATING_COOLING_TARGET)
         if (MODE_HOMEKIT_TO_HASS.get(value) in {HVACMode.HEAT_COOL}) and (
-            ClimateEntityFeature.TARGET_TEMPERATURE_RANGE & self.supported_features
+            ClimateEntityFeature.TARGET_TEMPERATURE_RANGE in self.supported_features
         ):
             min_temp = self.service[
                 CharacteristicsTypes.TEMPERATURE_HEATING_THRESHOLD
@@ -518,7 +606,7 @@ class HomeKitClimateEntity(HomeKitBaseClimateEntity):
         """Return the maximum target temp."""
         value = self.service.value(CharacteristicsTypes.HEATING_COOLING_TARGET)
         if (MODE_HOMEKIT_TO_HASS.get(value) in {HVACMode.HEAT_COOL}) and (
-            ClimateEntityFeature.TARGET_TEMPERATURE_RANGE & self.supported_features
+            ClimateEntityFeature.TARGET_TEMPERATURE_RANGE in self.supported_features
         ):
             max_temp = self.service[
                 CharacteristicsTypes.TEMPERATURE_COOLING_THRESHOLD
@@ -592,7 +680,7 @@ class HomeKitClimateEntity(HomeKitBaseClimateEntity):
         value = self.service.value(CharacteristicsTypes.HEATING_COOLING_TARGET)
         return MODE_HOMEKIT_TO_HASS[value]
 
-    @property
+    @cached_property
     def hvac_modes(self) -> list[HVACMode]:
         """Return the list of available hvac operation modes."""
         valid_values = clamp_enum_to_char(
@@ -601,8 +689,8 @@ class HomeKitClimateEntity(HomeKitBaseClimateEntity):
         )
         return [MODE_HOMEKIT_TO_HASS[mode] for mode in valid_values]
 
-    @property
-    def supported_features(self) -> int:
+    @cached_property
+    def supported_features(self) -> ClimateEntityFeature:
         """Return the list of supported features."""
         features = super().supported_features
 
