@@ -4,7 +4,9 @@ from __future__ import annotations
 import asyncio
 from collections.abc import Callable, Iterable
 from datetime import datetime, timedelta
+from functools import partial
 import logging
+from operator import attrgetter
 from types import MappingProxyType
 from typing import Any
 
@@ -22,13 +24,12 @@ from aiohomekit.model.services import Service, ServicesTypes
 from homeassistant.components.thread.dataset_store import async_get_preferred_dataset
 from homeassistant.config_entries import ConfigEntry
 from homeassistant.const import ATTR_VIA_DEVICE, EVENT_HOMEASSISTANT_STARTED
-from homeassistant.core import CoreState, Event, HomeAssistant, callback
+from homeassistant.core import CALLBACK_TYPE, CoreState, Event, HomeAssistant, callback
 from homeassistant.exceptions import HomeAssistantError
 from homeassistant.helpers import device_registry as dr, entity_registry as er
 from homeassistant.helpers.debounce import Debouncer
-from homeassistant.helpers.dispatcher import async_dispatcher_send
-from homeassistant.helpers.entity import DeviceInfo
-from homeassistant.helpers.event import async_track_time_interval
+from homeassistant.helpers.device_registry import DeviceInfo
+from homeassistant.helpers.event import async_call_later, async_track_time_interval
 
 from .config_flow import normalize_hkid
 from .const import (
@@ -42,6 +43,7 @@ from .const import (
     IDENTIFIER_LEGACY_SERIAL_NUMBER,
     IDENTIFIER_SERIAL_NUMBER,
     STARTUP_EXCEPTIONS,
+    SUBSCRIBE_COOLDOWN,
 )
 from .device_trigger import async_fire_triggers, async_setup_triggers_for_entry
 
@@ -96,6 +98,13 @@ class HKDevice:
         # A list of callbacks that turn HK service metadata into entities
         self.listeners: list[AddServiceCb] = []
 
+        # A list of callbacks that turn HK service metadata into triggers
+        self.trigger_factories: list[AddServiceCb] = []
+
+        # Track aid/iid pairs so we know if we already handle triggers for a HK
+        # service.
+        self._triggers: set[tuple[int, int]] = set()
+
         # A list of callbacks that turn HK characteristics into entities
         self.char_factories: list[AddCharacteristicCb] = []
 
@@ -108,7 +117,7 @@ class HKDevice:
 
         # This just tracks aid/iid pairs so we know if a HK service has been
         # mapped to a HA entity.
-        self.entities: list[tuple[int, int | None, int | None]] = []
+        self.entities: set[tuple[int, int | None, int | None]] = set()
 
         # A map of aid -> device_id
         # Useful when routing events to triggers
@@ -116,9 +125,7 @@ class HKDevice:
 
         self.available = False
 
-        self.signal_state_updated = "_".join((DOMAIN, self.unique_id, "state_updated"))
-
-        self.pollable_characteristics: list[tuple[int, int]] = []
+        self.pollable_characteristics: set[tuple[int, int]] = set()
 
         # Never allow concurrent polling of the same accessory or bridge
         self._polling_lock = asyncio.Lock()
@@ -128,7 +135,7 @@ class HKDevice:
         # This is set to True if we can't rely on serial numbers to be unique
         self.unreliable_serial_numbers = False
 
-        self.watchable_characteristics: list[tuple[int, int]] = []
+        self.watchable_characteristics: set[tuple[int, int]] = set()
 
         self._debounced_update = Debouncer(
             hass,
@@ -138,6 +145,12 @@ class HKDevice:
             function=self.async_update,
         )
 
+        self._availability_callbacks: set[CALLBACK_TYPE] = set()
+        self._config_changed_callbacks: set[CALLBACK_TYPE] = set()
+        self._subscriptions: dict[tuple[int, int], set[CALLBACK_TYPE]] = {}
+        self._pending_subscribes: set[tuple[int, int]] = set()
+        self._subscribe_timer: CALLBACK_TYPE | None = None
+
     @property
     def entity_map(self) -> Accessories:
         """Return the accessories from the pairing."""
@@ -152,26 +165,51 @@ class HKDevice:
         self, characteristics: list[tuple[int, int]]
     ) -> None:
         """Add (aid, iid) pairs that we need to poll."""
-        self.pollable_characteristics.extend(characteristics)
+        self.pollable_characteristics.update(characteristics)
 
-    def remove_pollable_characteristics(self, accessory_id: int) -> None:
+    def remove_pollable_characteristics(
+        self, characteristics: list[tuple[int, int]]
+    ) -> None:
         """Remove all pollable characteristics by accessory id."""
-        self.pollable_characteristics = [
-            char for char in self.pollable_characteristics if char[0] != accessory_id
-        ]
+        for aid_iid in characteristics:
+            self.pollable_characteristics.discard(aid_iid)
 
-    async def add_watchable_characteristics(
+    def add_watchable_characteristics(
         self, characteristics: list[tuple[int, int]]
     ) -> None:
         """Add (aid, iid) pairs that we need to poll."""
-        self.watchable_characteristics.extend(characteristics)
-        await self.pairing.subscribe(characteristics)
+        self.watchable_characteristics.update(characteristics)
+        self._pending_subscribes.update(characteristics)
+        # Try to subscribe to the characteristics all at once
+        if not self._subscribe_timer:
+            self._subscribe_timer = async_call_later(
+                self.hass,
+                SUBSCRIBE_COOLDOWN,
+                self._async_subscribe,
+            )
 
-    def remove_watchable_characteristics(self, accessory_id: int) -> None:
+    @callback
+    def _async_cancel_subscription_timer(self) -> None:
+        """Cancel the subscribe timer."""
+        if self._subscribe_timer:
+            self._subscribe_timer()
+            self._subscribe_timer = None
+
+    async def _async_subscribe(self, _now: datetime) -> None:
+        """Subscribe to characteristics."""
+        self._subscribe_timer = None
+        if self._pending_subscribes:
+            subscribes = self._pending_subscribes.copy()
+            self._pending_subscribes.clear()
+            await self.pairing.subscribe(subscribes)
+
+    def remove_watchable_characteristics(
+        self, characteristics: list[tuple[int, int]]
+    ) -> None:
         """Remove all pollable characteristics by accessory id."""
-        self.watchable_characteristics = [
-            char for char in self.watchable_characteristics if char[0] != accessory_id
-        ]
+        for aid_iid in characteristics:
+            self.watchable_characteristics.discard(aid_iid)
+            self._pending_subscribes.discard(aid_iid)
 
     @callback
     def async_set_available_state(self, available: bool) -> None:
@@ -182,7 +220,8 @@ class HKDevice:
         if self.available == available:
             return
         self.available = available
-        async_dispatcher_send(self.hass, self.signal_state_updated)
+        for callback_ in self._availability_callbacks:
+            callback_()
 
     async def _async_populate_ble_accessory_state(self, event: Event) -> None:
         """Populate the BLE accessory state without blocking startup.
@@ -222,7 +261,7 @@ class HKDevice:
         # Ideally we would know which entities we are about to add
         # so we only poll those chars but that is not possible
         # yet.
-        attempts = None if self.hass.state == CoreState.running else 1
+        attempts = None if self.hass.state is CoreState.running else 1
         if (
             transport == Transport.BLE
             and pairing.accessories
@@ -253,6 +292,7 @@ class HKDevice:
         entry.async_on_unload(
             pairing.dispatcher_availability_changed(self.async_set_available_state)
         )
+        entry.async_on_unload(self._async_cancel_subscription_timer)
 
         await self.async_process_entity_map()
 
@@ -272,7 +312,7 @@ class HKDevice:
                     self.hass,
                     self.async_update_available_state,
                     timedelta(seconds=BLE_AVAILABILITY_CHECK_INTERVAL),
-                    name=f"HomeKit Controller {self.unique_id} BLE availability "
+                    name=f"HomeKit Device {self.unique_id} BLE availability "
                     "check poll",
                 )
             )
@@ -291,7 +331,7 @@ class HKDevice:
                 self.hass,
                 self.async_request_update,
                 self.pairing.poll_interval,
-                name=f"HomeKit Controller {self.unique_id} availability check poll",
+                name=f"HomeKit Device {self.unique_id} availability check poll",
             )
         )
 
@@ -397,9 +437,10 @@ class HKDevice:
 
     @callback
     def async_migrate_unique_id(
-        self, old_unique_id: str, new_unique_id: str, platform: str
+        self, old_unique_id: str, new_unique_id: str | None, platform: str
     ) -> None:
         """Migrate legacy unique IDs to new format."""
+        assert new_unique_id is not None
         _LOGGER.debug(
             "Checking if unique ID %s on %s needs to be migrated",
             old_unique_id,
@@ -500,9 +541,7 @@ class HKDevice:
 
         # Accessories need to be created in the correct order or setting up
         # relationships with ATTR_VIA_DEVICE may fail.
-        for accessory in sorted(
-            self.entity_map.accessories, key=lambda accessory: accessory.aid
-        ):
+        for accessory in sorted(self.entity_map.accessories, key=attrgetter("aid")):
             device_info = self.device_info_for_accessory(accessory)
 
             device = device_registry.async_get_or_create(
@@ -596,23 +635,32 @@ class HKDevice:
     async def async_update_new_accessories_state(self) -> None:
         """Process a change in the pairings accessories state."""
         await self.async_process_entity_map()
-        if self.watchable_characteristics:
-            await self.pairing.subscribe(self.watchable_characteristics)
+        for callback_ in self._config_changed_callbacks:
+            callback_()
         await self.async_update()
         await self.async_add_new_entities()
 
-    def add_accessory_factory(self, add_entities_cb) -> None:
+    @callback
+    def async_entity_key_removed(
+        self, entity_key: tuple[int, int | None, int | None]
+    ) -> None:
+        """Handle an entity being removed.
+
+        Releases the entity from self.entities so it can be added again.
+        """
+        self.entities.discard(entity_key)
+
+    def add_accessory_factory(self, add_entities_cb: AddAccessoryCb) -> None:
         """Add a callback to run when discovering new entities for accessories."""
         self.accessory_factories.append(add_entities_cb)
         self._add_new_entities_for_accessory([add_entities_cb])
 
-    def _add_new_entities_for_accessory(self, handlers) -> None:
+    def _add_new_entities_for_accessory(self, handlers: list[AddAccessoryCb]) -> None:
         for accessory in self.entity_map.accessories:
+            entity_key = (accessory.aid, None, None)
             for handler in handlers:
-                if (accessory.aid, None, None) in self.entities:
-                    continue
-                if handler(accessory):
-                    self.entities.append((accessory.aid, None, None))
+                if entity_key not in self.entities and handler(accessory):
+                    self.entities.add(entity_key)
                     break
 
     def add_char_factory(self, add_entities_cb: AddCharacteristicCb) -> None:
@@ -620,15 +668,14 @@ class HKDevice:
         self.char_factories.append(add_entities_cb)
         self._add_new_entities_for_char([add_entities_cb])
 
-    def _add_new_entities_for_char(self, handlers) -> None:
+    def _add_new_entities_for_char(self, handlers: list[AddCharacteristicCb]) -> None:
         for accessory in self.entity_map.accessories:
             for service in accessory.services:
                 for char in service.characteristics:
+                    entity_key = (accessory.aid, service.iid, char.iid)
                     for handler in handlers:
-                        if (accessory.aid, service.iid, char.iid) in self.entities:
-                            continue
-                        if handler(char):
-                            self.entities.append((accessory.aid, service.iid, char.iid))
+                        if entity_key not in self.entities and handler(char):
+                            self.entities.add(entity_key)
                             break
 
     def add_listener(self, add_entities_cb: AddServiceCb) -> None:
@@ -636,25 +683,47 @@ class HKDevice:
         self.listeners.append(add_entities_cb)
         self._add_new_entities([add_entities_cb])
 
+    def add_trigger_factory(self, add_triggers_cb: AddServiceCb) -> None:
+        """Add a callback to run when discovering new triggers for services."""
+        self.trigger_factories.append(add_triggers_cb)
+        self._add_new_triggers([add_triggers_cb])
+
+    def _add_new_triggers(self, callbacks: list[AddServiceCb]) -> None:
+        for accessory in self.entity_map.accessories:
+            aid = accessory.aid
+            for service in accessory.services:
+                iid = service.iid
+                entity_key = (aid, iid)
+
+                if entity_key in self._triggers:
+                    # Don't add the same trigger again
+                    continue
+
+                for add_trigger_cb in callbacks:
+                    if add_trigger_cb(service):
+                        self._triggers.add(entity_key)
+                        break
+
     def add_entities(self) -> None:
         """Process the entity map and create HA entities."""
         self._add_new_entities(self.listeners)
         self._add_new_entities_for_accessory(self.accessory_factories)
         self._add_new_entities_for_char(self.char_factories)
+        self._add_new_triggers(self.trigger_factories)
 
-    def _add_new_entities(self, callbacks) -> None:
+    def _add_new_entities(self, callbacks: list[AddServiceCb]) -> None:
         for accessory in self.entity_map.accessories:
             aid = accessory.aid
             for service in accessory.services:
-                iid = service.iid
+                entity_key = (aid, None, service.iid)
 
-                if (aid, None, iid) in self.entities:
+                if entity_key in self.entities:
                     # Don't add the same entity again
                     continue
 
                 for listener in callbacks:
                     if listener(service):
-                        self.entities.append((aid, None, iid))
+                        self.entities.add(entity_key)
                         break
 
     async def async_load_platform(self, platform: str) -> None:
@@ -701,7 +770,7 @@ class HKDevice:
         """Request an debounced update from the accessory."""
         await self._debounced_update.async_call()
 
-    async def async_update(self, now=None):
+    async def async_update(self, now: datetime | None = None) -> None:
         """Poll state of all entities attached to this bridge/accessory."""
         if not self.pollable_characteristics:
             self.async_update_available_state()
@@ -714,7 +783,7 @@ class HKDevice:
             if not self._polling_lock_warned:
                 _LOGGER.warning(
                     (
-                        "HomeKit controller update skipped as previous poll still in"
+                        "HomeKit device update skipped as previous poll still in"
                         " flight: %s"
                     ),
                     self.unique_id,
@@ -725,7 +794,7 @@ class HKDevice:
         if self._polling_lock_warned:
             _LOGGER.info(
                 (
-                    "HomeKit controller no longer detecting back pressure - not"
+                    "HomeKit device no longer detecting back pressure - not"
                     " skipping poll: %s"
                 ),
                 self.unique_id,
@@ -733,7 +802,7 @@ class HKDevice:
             self._polling_lock_warned = False
 
         async with self._polling_lock:
-            _LOGGER.debug("Starting HomeKit controller update: %s", self.unique_id)
+            _LOGGER.debug("Starting HomeKit device update: %s", self.unique_id)
 
             try:
                 new_values_dict = await self.get_characteristics(
@@ -755,7 +824,7 @@ class HKDevice:
             self._poll_failures = 0
             self.process_new_events(new_values_dict)
 
-            _LOGGER.debug("Finished HomeKit controller update: %s", self.unique_id)
+            _LOGGER.debug("Finished HomeKit device update: %s", self.unique_id)
 
     def process_new_events(
         self, new_values_dict: dict[tuple[int, int], dict[str, Any]]
@@ -766,11 +835,60 @@ class HKDevice:
         # Process any stateless events (via device_triggers)
         async_fire_triggers(self, new_values_dict)
 
-        self.entity_map.process_changes(new_values_dict)
+        to_callback: set[CALLBACK_TYPE] = set()
+        for aid_iid in self.entity_map.process_changes(new_values_dict):
+            if callbacks := self._subscriptions.get(aid_iid):
+                to_callback.update(callbacks)
 
-        async_dispatcher_send(self.hass, self.signal_state_updated)
+        for callback_ in to_callback:
+            callback_()
 
-    async def get_characteristics(self, *args: Any, **kwargs: Any) -> dict[str, Any]:
+    @callback
+    def _remove_characteristics_callback(
+        self, characteristics: set[tuple[int, int]], callback_: CALLBACK_TYPE
+    ) -> None:
+        """Remove a characteristics callback."""
+        for aid_iid in characteristics:
+            self._subscriptions[aid_iid].remove(callback_)
+            if not self._subscriptions[aid_iid]:
+                del self._subscriptions[aid_iid]
+
+    @callback
+    def async_subscribe(
+        self, characteristics: set[tuple[int, int]], callback_: CALLBACK_TYPE
+    ) -> CALLBACK_TYPE:
+        """Add characteristics to the watch list."""
+        for aid_iid in characteristics:
+            self._subscriptions.setdefault(aid_iid, set()).add(callback_)
+        return partial(
+            self._remove_characteristics_callback, characteristics, callback_
+        )
+
+    @callback
+    def _remove_availability_callback(self, callback_: CALLBACK_TYPE) -> None:
+        """Remove an availability callback."""
+        self._availability_callbacks.remove(callback_)
+
+    @callback
+    def async_subscribe_availability(self, callback_: CALLBACK_TYPE) -> CALLBACK_TYPE:
+        """Add characteristics to the watch list."""
+        self._availability_callbacks.add(callback_)
+        return partial(self._remove_availability_callback, callback_)
+
+    @callback
+    def _remove_config_changed_callback(self, callback_: CALLBACK_TYPE) -> None:
+        """Remove an availability callback."""
+        self._config_changed_callbacks.remove(callback_)
+
+    @callback
+    def async_subscribe_config_changed(self, callback_: CALLBACK_TYPE) -> CALLBACK_TYPE:
+        """Subscribe to config of the accessory being changed aka c# changes."""
+        self._config_changed_callbacks.add(callback_)
+        return partial(self._remove_config_changed_callback, callback_)
+
+    async def get_characteristics(
+        self, *args: Any, **kwargs: Any
+    ) -> dict[tuple[int, int], dict[str, Any]]:
         """Read latest state from homekit accessory."""
         return await self.pairing.get_characteristics(*args, **kwargs)
 
