@@ -1,5 +1,4 @@
-"""
-Support for Homekit number ranges.
+"""Support for Homekit number ranges.
 
 These are mostly used where a HomeKit accessory exposes additional non-standard
 characteristics that don't map to a Home Assistant feature.
@@ -16,8 +15,8 @@ from homeassistant.components.number import (
     NumberEntityDescription,
 )
 from homeassistant.config_entries import ConfigEntry
+from homeassistant.const import EntityCategory, Platform
 from homeassistant.core import HomeAssistant, callback
-from homeassistant.helpers.entity import EntityCategory
 from homeassistant.helpers.entity_platform import AddEntitiesCallback
 from homeassistant.helpers.typing import ConfigType
 
@@ -50,6 +49,18 @@ NUMBER_ENTITIES: dict[str, NumberEntityDescription] = {
         icon="mdi:volume-high",
         entity_category=EntityCategory.CONFIG,
     ),
+    CharacteristicsTypes.VENDOR_EVE_MOTION_DURATION: NumberEntityDescription(
+        key=CharacteristicsTypes.VENDOR_EVE_MOTION_DURATION,
+        name="Duration",
+        icon="mdi:timer",
+        entity_category=EntityCategory.CONFIG,
+    ),
+    CharacteristicsTypes.VENDOR_EVE_MOTION_SENSITIVITY: NumberEntityDescription(
+        key=CharacteristicsTypes.VENDOR_EVE_MOTION_SENSITIVITY,
+        name="Sensitivity",
+        icon="mdi:knob",
+        entity_category=EntityCategory.CONFIG,
+    ),
 }
 
 
@@ -59,12 +70,12 @@ async def async_setup_entry(
     async_add_entities: AddEntitiesCallback,
 ) -> None:
     """Set up Homekit numbers."""
-    hkid = config_entry.data["AccessoryPairingID"]
+    hkid: str = config_entry.data["AccessoryPairingID"]
     conn: HKDevice = hass.data[KNOWN_DEVICES][hkid]
 
     @callback
     def async_add_characteristic(char: Characteristic) -> bool:
-        entities = []
+        entities: list[HomeKitNumber] = []
         info = {"aid": char.service.accessory.aid, "iid": char.service.iid}
 
         if description := NUMBER_ENTITIES.get(char.type):
@@ -72,7 +83,12 @@ async def async_setup_entry(
         else:
             return False
 
-        async_add_entities(entities, True)
+        for entity in entities:
+            conn.async_migrate_unique_id(
+                entity.old_unique_id, entity.unique_id, Platform.NUMBER
+            )
+
+        async_add_entities(entities)
         return True
 
     conn.add_char_factory(async_add_characteristic)
