@@ -1,24 +1,32 @@
 """Support for Homematic lights."""
+from __future__ import annotations
+
+from typing import Any
+
 from homeassistant.components.light import (
     ATTR_BRIGHTNESS,
     ATTR_COLOR_TEMP,
     ATTR_EFFECT,
     ATTR_HS_COLOR,
     ATTR_TRANSITION,
-    SUPPORT_BRIGHTNESS,
-    SUPPORT_COLOR,
-    SUPPORT_COLOR_TEMP,
-    SUPPORT_EFFECT,
+    ColorMode,
     LightEntity,
+    LightEntityFeature,
 )
+from homeassistant.core import HomeAssistant
+from homeassistant.helpers.entity_platform import AddEntitiesCallback
+from homeassistant.helpers.typing import ConfigType, DiscoveryInfoType
 
 from .const import ATTR_DISCOVER_DEVICES
 from .entity import HMDevice
 
-SUPPORT_HOMEMATIC = SUPPORT_BRIGHTNESS
 
-
-def setup_platform(hass, config, add_entities, discovery_info=None):
+def setup_platform(
+    hass: HomeAssistant,
+    config: ConfigType,
+    add_entities: AddEntitiesCallback,
+    discovery_info: DiscoveryInfoType | None = None,
+) -> None:
     """Set up the Homematic light platform."""
     if discovery_info is None:
         return
@@ -51,21 +59,40 @@ class HMLight(HMDevice, LightEntity):
             return False
 
     @property
-    def supported_features(self):
-        """Flag supported features."""
-        features = SUPPORT_BRIGHTNESS
+    def color_mode(self) -> ColorMode:
+        """Return the color mode of the light."""
         if "COLOR" in self._hmdevice.WRITENODE:
-            features |= SUPPORT_COLOR
-        if "PROGRAM" in self._hmdevice.WRITENODE:
-            features |= SUPPORT_EFFECT
+            return ColorMode.HS
         if hasattr(self._hmdevice, "get_color_temp"):
-            features |= SUPPORT_COLOR_TEMP
+            return ColorMode.COLOR_TEMP
+        return ColorMode.BRIGHTNESS
+
+    @property
+    def supported_color_modes(self) -> set[ColorMode]:
+        """Flag supported color modes."""
+        color_modes: set[ColorMode] = set()
+
+        if "COLOR" in self._hmdevice.WRITENODE:
+            color_modes.add(ColorMode.HS)
+        if hasattr(self._hmdevice, "get_color_temp"):
+            color_modes.add(ColorMode.COLOR_TEMP)
+        if not color_modes:
+            color_modes.add(ColorMode.BRIGHTNESS)
+
+        return color_modes
+
+    @property
+    def supported_features(self) -> LightEntityFeature:
+        """Flag supported features."""
+        features = LightEntityFeature.TRANSITION
+        if "PROGRAM" in self._hmdevice.WRITENODE:
+            features |= LightEntityFeature.EFFECT
         return features
 
     @property
     def hs_color(self):
         """Return the hue and saturation color value [float, float]."""
-        if not self.supported_features & SUPPORT_COLOR:
+        if ColorMode.HS not in self.supported_color_modes:
             return None
         hue, sat = self._hmdevice.get_hs_color(self._channel)
         return hue * 360.0, sat * 100.0
@@ -73,7 +100,7 @@ class HMLight(HMDevice, LightEntity):
     @property
     def color_temp(self):
         """Return the color temp in mireds [int]."""
-        if not self.supported_features & SUPPORT_COLOR_TEMP:
+        if ColorMode.COLOR_TEMP not in self.supported_color_modes:
             return None
         hm_color_temp = self._hmdevice.get_color_temp(self._channel)
         return self.max_mireds - (self.max_mireds - self.min_mireds) * hm_color_temp
@@ -81,21 +108,21 @@ class HMLight(HMDevice, LightEntity):
     @property
     def effect_list(self):
         """Return the list of supported effects."""
-        if not self.supported_features & SUPPORT_EFFECT:
+        if not self.supported_features & LightEntityFeature.EFFECT:
             return None
         return self._hmdevice.get_effect_list()
 
     @property
     def effect(self):
         """Return the current color change program of the light."""
-        if not self.supported_features & SUPPORT_EFFECT:
+        if not self.supported_features & LightEntityFeature.EFFECT:
             return None
         return self._hmdevice.get_effect()
 
-    def turn_on(self, **kwargs):
+    def turn_on(self, **kwargs: Any) -> None:
         """Turn the light on and/or change color or color effect settings."""
         if ATTR_TRANSITION in kwargs:
-            self._hmdevice.setValue("RAMP_TIME", kwargs[ATTR_TRANSITION])
+            self._hmdevice.setValue("RAMP_TIME", kwargs[ATTR_TRANSITION], self._channel)
 
         if ATTR_BRIGHTNESS in kwargs and self._state == "LEVEL":
             percent_bright = float(kwargs[ATTR_BRIGHTNESS]) / 255
@@ -107,7 +134,7 @@ class HMLight(HMDevice, LightEntity):
         ):
             self._hmdevice.on(self._channel)
 
-        if ATTR_HS_COLOR in kwargs and self.supported_features & SUPPORT_COLOR:
+        if ATTR_HS_COLOR in kwargs:
             self._hmdevice.set_hs_color(
                 hue=kwargs[ATTR_HS_COLOR][0] / 360.0,
                 saturation=kwargs[ATTR_HS_COLOR][1] / 100.0,
@@ -121,8 +148,11 @@ class HMLight(HMDevice, LightEntity):
         if ATTR_EFFECT in kwargs:
             self._hmdevice.set_effect(kwargs[ATTR_EFFECT])
 
-    def turn_off(self, **kwargs):
+    def turn_off(self, **kwargs: Any) -> None:
         """Turn the light off."""
+        if ATTR_TRANSITION in kwargs:
+            self._hmdevice.setValue("RAMP_TIME", kwargs[ATTR_TRANSITION], self._channel)
+
         self._hmdevice.off(self._channel)
 
     def _init_data_struct(self):
@@ -131,7 +161,7 @@ class HMLight(HMDevice, LightEntity):
         self._state = "LEVEL"
         self._data[self._state] = None
 
-        if self.supported_features & SUPPORT_COLOR:
+        if ColorMode.HS in self.supported_color_modes:
             self._data.update({"COLOR": None})
-        if self.supported_features & SUPPORT_EFFECT:
+        if self.supported_features & LightEntityFeature.EFFECT:
             self._data.update({"PROGRAM": None})
