@@ -5,7 +5,7 @@ DOMAIN = "honeywell"
 
 CONF_COOL_AWAY_TEMPERATURE = "away_cool_temperature"
 CONF_HEAT_AWAY_TEMPERATURE = "away_heat_temperature"
-CONF_DEV_ID = "thermostat"
-CONF_LOC_ID = "location"
-
+DEFAULT_COOL_AWAY_TEMPERATURE = 88
+DEFAULT_HEAT_AWAY_TEMPERATURE = 61
 _LOGGER = logging.getLogger(__name__)
+RETRY = 3
