@@ -1,6 +1,7 @@
 """Support for Hue groups (room/zone)."""
 from __future__ import annotations
 
+import asyncio
 from typing import Any
 
 from aiohue.v2 import HueBridgeV2
@@ -14,19 +15,15 @@ from homeassistant.components.light import (
     ATTR_FLASH,
     ATTR_TRANSITION,
     ATTR_XY_COLOR,
-    COLOR_MODE_BRIGHTNESS,
-    COLOR_MODE_COLOR_TEMP,
-    COLOR_MODE_ONOFF,
-    COLOR_MODE_XY,
     FLASH_SHORT,
-    SUPPORT_FLASH,
-    SUPPORT_TRANSITION,
+    ColorMode,
     LightEntity,
+    LightEntityDescription,
+    LightEntityFeature,
 )
 from homeassistant.config_entries import ConfigEntry
 from homeassistant.core import HomeAssistant, callback
-from homeassistant.helpers.device_registry import DeviceEntryType
-from homeassistant.helpers.entity import DeviceInfo
+from homeassistant.helpers.device_registry import DeviceInfo
 from homeassistant.helpers.entity_platform import AddEntitiesCallback
 
 from ..bridge import HueBridge
@@ -48,18 +45,26 @@ async def async_setup_entry(
     bridge: HueBridge = hass.data[DOMAIN][config_entry.entry_id]
     api: HueBridgeV2 = bridge.api
 
-    @callback
-    def async_add_light(event_type: EventType, resource: GroupedLight) -> None:
+    async def async_add_light(event_type: EventType, resource: GroupedLight) -> None:
         """Add Grouped Light for Hue Room/Zone."""
-        group = api.groups.grouped_light.get_zone(resource.id)
+        # delay group creation a bit due to a race condition where the
+        # grouped_light resource is created before the zone/room
+        retries = 5
+        while (
+            retries
+            and (group := api.groups.grouped_light.get_zone(resource.id)) is None
+        ):
+            retries -= 1
+            await asyncio.sleep(0.5)
         if group is None:
+            # guard, just in case
             return
         light = GroupedHueLight(bridge, resource, group)
         async_add_entities([light])
 
     # add current items
     for item in api.groups.grouped_light.items:
-        async_add_light(EventType.RESOURCE_ADDED, item)
+        await async_add_light(EventType.RESOURCE_ADDED, item)
 
     # register listener for new grouped_light
     config_entry.async_on_unload(
@@ -72,7 +77,12 @@ async def async_setup_entry(
 class GroupedHueLight(HueBaseEntity, LightEntity):
     """Representation of a Grouped Hue light."""
 
-    _attr_icon = "mdi:lightbulb-group"
+    entity_description = LightEntityDescription(
+        key="hue_grouped_light",
+        icon="mdi:lightbulb-group",
+        has_entity_name=True,
+        name=None,
+    )
 
     def __init__(
         self, bridge: HueBridge, resource: GroupedLight, group: Room | Zone
@@ -84,9 +94,15 @@ class GroupedHueLight(HueBaseEntity, LightEntity):
         self.group = group
         self.controller = controller
         self.api: HueBridgeV2 = bridge.api
-        self._attr_supported_features |= SUPPORT_FLASH
-        self._attr_supported_features |= SUPPORT_TRANSITION
-
+        self._attr_supported_features |= LightEntityFeature.FLASH
+        self._attr_supported_features |= LightEntityFeature.TRANSITION
+        self._restore_brightness: float | None = None
+        self._brightness_pct: float = 0
+        # we create a virtual service/device for Hue zones/rooms
+        # so we have a parent for grouped lights and scenes
+        self._attr_device_info = DeviceInfo(
+            identifiers={(DOMAIN, self.group.id)},
+        )
         self._dynamic_mode_active = False
         self._update_values()
 
@@ -108,11 +124,6 @@ class GroupedHueLight(HueBaseEntity, LightEntity):
                 self.api.lights.subscribe(self._handle_event, light_ids)
             )
 
-    @property
-    def name(self) -> str:
-        """Return name of room/zone for this grouped light."""
-        return self.group.metadata.name
-
     @property
     def is_on(self) -> bool:
         """Return true if light is on."""
@@ -124,7 +135,10 @@ class GroupedHueLight(HueBaseEntity, LightEntity):
         scenes = {
             x.metadata.name for x in self.api.scenes if x.group.rid == self.group.id
         }
-        lights = {x.metadata.name for x in self.controller.get_lights(self.resource.id)}
+        lights = {
+            self.controller.get_device(x.id).metadata.name
+            for x in self.controller.get_lights(self.resource.id)
+        }
         return {
             "is_hue_group": True,
             "hue_scenes": scenes,
@@ -133,22 +147,6 @@ class GroupedHueLight(HueBaseEntity, LightEntity):
             "dynamics": self._dynamic_mode_active,
         }
 
-    @property
-    def device_info(self) -> DeviceInfo:
-        """Return device (service) info."""
-        # we create a virtual service/device for Hue zones/rooms
-        # so we have a parent for grouped lights and scenes
-        model = self.group.type.value.title()
-        return DeviceInfo(
-            identifiers={(DOMAIN, self.group.id)},
-            entry_type=DeviceEntryType.SERVICE,
-            name=self.group.metadata.name,
-            manufacturer=self.api.config.bridge_device.product_data.manufacturer_name,
-            model=model,
-            suggested_area=self.group.metadata.name if model == "Room" else None,
-            via_device=(DOMAIN, self.api.config.bridge_device.id),
-        )
-
     async def async_turn_on(self, **kwargs: Any) -> None:
         """Turn the grouped_light on."""
         transition = normalize_hue_transition(kwargs.get(ATTR_TRANSITION))
@@ -157,6 +155,18 @@ class GroupedHueLight(HueBaseEntity, LightEntity):
         brightness = normalize_hue_brightness(kwargs.get(ATTR_BRIGHTNESS))
         flash = kwargs.get(ATTR_FLASH)
 
+        if self._restore_brightness and brightness is None:
+            # The Hue bridge sets the brightness to 1% when turning on a bulb
+            # when a transition was used to turn off the bulb.
+            # This issue has been reported on the Hue forum several times:
+            # https://developers.meethue.com/forum/t/brightness-turns-down-to-1-automatically-shortly-after-sending-off-signal-hue-bug/5692
+            # https://developers.meethue.com/forum/t/lights-turn-on-with-lowest-brightness-via-siri-if-turned-off-via-api/6700
+            # https://developers.meethue.com/forum/t/using-transitiontime-with-on-false-resets-bri-to-1/4585
+            # https://developers.meethue.com/forum/t/bri-value-changing-in-switching-lights-on-off/6323
+            # https://developers.meethue.com/forum/t/fade-in-fade-out/6673
+            brightness = self._restore_brightness
+            self._restore_brightness = None
+
         if flash is not None:
             await self.async_set_flash(flash)
             return
@@ -174,11 +184,13 @@ class GroupedHueLight(HueBaseEntity, LightEntity):
     async def async_turn_off(self, **kwargs: Any) -> None:
         """Turn the light off."""
         transition = normalize_hue_transition(kwargs.get(ATTR_TRANSITION))
+        if transition is not None:
+            self._restore_brightness = self._brightness_pct
         flash = kwargs.get(ATTR_FLASH)
 
         if flash is not None:
             await self.async_set_flash(flash)
-            # flash can not be sent with other commands at the same time
+            # flash cannot be sent with other commands at the same time
             return
 
         await self.bridge.async_request_call(
@@ -204,7 +216,7 @@ class GroupedHueLight(HueBaseEntity, LightEntity):
     @callback
     def _update_values(self) -> None:
         """Set base values from underlying lights of a group."""
-        supported_color_modes = set()
+        supported_color_modes: set[ColorMode | str] = set()
         lights_with_color_support = 0
         lights_with_color_temp_support = 0
         lights_with_dimming_support = 0
@@ -241,18 +253,19 @@ class GroupedHueLight(HueBaseEntity, LightEntity):
         # this means that the state is derived from only some of the lights
         # and will never be 100% accurate but it will be close
         if lights_with_color_support > 0:
-            supported_color_modes.add(COLOR_MODE_XY)
+            supported_color_modes.add(ColorMode.XY)
         if lights_with_color_temp_support > 0:
-            supported_color_modes.add(COLOR_MODE_COLOR_TEMP)
+            supported_color_modes.add(ColorMode.COLOR_TEMP)
         if lights_with_dimming_support > 0:
             if len(supported_color_modes) == 0:
                 # only add color mode brightness if no color variants
-                supported_color_modes.add(COLOR_MODE_BRIGHTNESS)
+                supported_color_modes.add(ColorMode.BRIGHTNESS)
+            self._brightness_pct = total_brightness / lights_with_dimming_support
             self._attr_brightness = round(
                 ((total_brightness / lights_with_dimming_support) / 100) * 255
             )
         else:
-            supported_color_modes.add(COLOR_MODE_ONOFF)
+            supported_color_modes.add(ColorMode.ONOFF)
         self._dynamic_mode_active = lights_in_dynamic_mode > 0
         self._attr_supported_color_modes = supported_color_modes
         # pick a winner for the current colormode
@@ -260,10 +273,10 @@ class GroupedHueLight(HueBaseEntity, LightEntity):
             lights_with_color_temp_support > 0
             and lights_in_colortemp_mode == lights_with_color_temp_support
         ):
-            self._attr_color_mode = COLOR_MODE_COLOR_TEMP
+            self._attr_color_mode = ColorMode.COLOR_TEMP
         elif lights_with_color_support > 0:
-            self._attr_color_mode = COLOR_MODE_XY
+            self._attr_color_mode = ColorMode.XY
         elif lights_with_dimming_support > 0:
-            self._attr_color_mode = COLOR_MODE_BRIGHTNESS
+            self._attr_color_mode = ColorMode.BRIGHTNESS
         else:
-            self._attr_color_mode = COLOR_MODE_ONOFF
+            self._attr_color_mode = ColorMode.ONOFF
