@@ -2,6 +2,7 @@
 from __future__ import annotations
 
 import asyncio
+from collections.abc import Mapping
 from contextlib import suppress
 import logging
 from typing import Any
@@ -10,9 +11,8 @@ from urllib.parse import urlparse
 from hyperion import client, const
 import voluptuous as vol
 
-from homeassistant.components.ssdp import ATTR_SSDP_LOCATION, ATTR_UPNP_SERIAL
+from homeassistant.components import ssdp
 from homeassistant.config_entries import (
-    CONN_CLASS_LOCAL_PUSH,
     SOURCE_REAUTH,
     ConfigEntry,
     ConfigFlow,
@@ -29,7 +29,6 @@ from homeassistant.const import (
 from homeassistant.core import callback
 from homeassistant.data_entry_flow import FlowResult
 import homeassistant.helpers.config_validation as cv
-from homeassistant.helpers.typing import ConfigType
 
 from . import create_hyperion_client
 from .const import (
@@ -110,7 +109,6 @@ class HyperionConfigFlow(ConfigFlow, domain=DOMAIN):
     """Handle a Hyperion config flow."""
 
     VERSION = 1
-    CONNECTION_CLASS = CONN_CLASS_LOCAL_PUSH
 
     def __init__(self) -> None:
         """Instantiate config flow."""
@@ -143,18 +141,15 @@ class HyperionConfigFlow(ConfigFlow, domain=DOMAIN):
             return await self.async_step_auth()
         return await self.async_step_confirm()
 
-    async def async_step_reauth(
-        self,
-        config_data: ConfigType,
-    ) -> FlowResult:
+    async def async_step_reauth(self, entry_data: Mapping[str, Any]) -> FlowResult:
         """Handle a reauthentication flow."""
-        self._data = dict(config_data)
+        self._data = dict(entry_data)
         async with self._create_client(raw_connection=True) as hyperion_client:
             if not hyperion_client:
                 return self.async_abort(reason="cannot_connect")
             return await self._advance_to_auth_step_if_necessary(hyperion_client)
 
-    async def async_step_ssdp(self, discovery_info: dict[str, Any]) -> FlowResult:
+    async def async_step_ssdp(self, discovery_info: ssdp.SsdpServiceInfo) -> FlowResult:
         """Handle a flow initiated by SSDP."""
         # Sample data provided by SSDP: {
         #   'ssdp_location': 'http://192.168.0.1:8090/description.xml',
@@ -191,23 +186,24 @@ class HyperionConfigFlow(ConfigFlow, domain=DOMAIN):
 
         # SSDP requires user confirmation.
         self._require_confirm = True
-        self._data[CONF_HOST] = urlparse(discovery_info[ATTR_SSDP_LOCATION]).hostname
+        self._data[CONF_HOST] = urlparse(discovery_info.ssdp_location).hostname
         try:
-            self._port_ui = urlparse(discovery_info[ATTR_SSDP_LOCATION]).port
+            self._port_ui = (
+                urlparse(discovery_info.ssdp_location).port or const.DEFAULT_PORT_UI
+            )
         except ValueError:
             self._port_ui = const.DEFAULT_PORT_UI
 
         try:
             self._data[CONF_PORT] = int(
-                discovery_info.get("ports", {}).get(
+                discovery_info.upnp.get("ports", {}).get(
                     "jsonServer", const.DEFAULT_PORT_JSON
                 )
             )
         except ValueError:
             self._data[CONF_PORT] = const.DEFAULT_PORT_JSON
 
-        hyperion_id = discovery_info.get(ATTR_UPNP_SERIAL)
-        if not hyperion_id:
+        if not (hyperion_id := discovery_info.upnp.get(ssdp.ATTR_UPNP_SERIAL)):
             return self.async_abort(reason="no_id")
 
         # For discovery mechanisms, we set the unique_id as early as possible to
@@ -224,7 +220,7 @@ class HyperionConfigFlow(ConfigFlow, domain=DOMAIN):
 
     async def async_step_user(
         self,
-        user_input: ConfigType | None = None,
+        user_input: dict[str, Any] | None = None,
     ) -> FlowResult:
         """Handle a flow initiated by the user."""
         errors = {}
@@ -295,7 +291,7 @@ class HyperionConfigFlow(ConfigFlow, domain=DOMAIN):
 
     async def async_step_auth(
         self,
-        user_input: ConfigType | None = None,
+        user_input: dict[str, Any] | None = None,
     ) -> FlowResult:
         """Handle the auth step of a flow."""
         errors = {}
@@ -324,7 +320,7 @@ class HyperionConfigFlow(ConfigFlow, domain=DOMAIN):
         )
 
     async def async_step_create_token(
-        self, user_input: ConfigType | None = None
+        self, user_input: dict[str, Any] | None = None
     ) -> FlowResult:
         """Send a request for a new token."""
         if user_input is None:
@@ -350,7 +346,7 @@ class HyperionConfigFlow(ConfigFlow, domain=DOMAIN):
         )
 
     async def async_step_create_token_external(
-        self, auth_resp: ConfigType | None = None
+        self, auth_resp: dict[str, Any] | None = None
     ) -> FlowResult:
         """Handle completion of the request for a new token."""
         if auth_resp is not None and client.ResponseOK(auth_resp):
@@ -363,7 +359,7 @@ class HyperionConfigFlow(ConfigFlow, domain=DOMAIN):
         return self.async_external_step_done(next_step_id="create_token_fail")
 
     async def async_step_create_token_success(
-        self, _: ConfigType | None = None
+        self, _: dict[str, Any] | None = None
     ) -> FlowResult:
         """Create an entry after successful token creation."""
         # Clean-up the request task.
@@ -379,7 +375,7 @@ class HyperionConfigFlow(ConfigFlow, domain=DOMAIN):
         return await self.async_step_confirm()
 
     async def async_step_create_token_fail(
-        self, _: ConfigType | None = None
+        self, _: dict[str, Any] | None = None
     ) -> FlowResult:
         """Show an error on the auth form."""
         # Clean-up the request task.
@@ -387,7 +383,7 @@ class HyperionConfigFlow(ConfigFlow, domain=DOMAIN):
         return self.async_abort(reason="auth_new_token_not_granted_error")
 
     async def async_step_confirm(
-        self, user_input: ConfigType | None = None
+        self, user_input: dict[str, Any] | None = None
     ) -> FlowResult:
         """Get final confirmation before entry creation."""
         if user_input is None and self._require_confirm:
@@ -434,7 +430,7 @@ class HyperionConfigFlow(ConfigFlow, domain=DOMAIN):
 class HyperionOptionsFlow(OptionsFlow):
     """Hyperion options flow."""
 
-    def __init__(self, config_entry: ConfigEntry):
+    def __init__(self, config_entry: ConfigEntry) -> None:
         """Initialize a Hyperion options flow."""
         self._config_entry = config_entry
 
@@ -451,7 +447,7 @@ class HyperionOptionsFlow(OptionsFlow):
     ) -> FlowResult:
         """Manage the options."""
 
-        effects = {source: source for source in const.KEY_COMPONENTID_EXTERNAL_SOURCES}
+        effects = {}
         async with self._create_client() as hyperion_client:
             if not hyperion_client:
                 return self.async_abort(reason="cannot_connect")
