@@ -22,9 +22,17 @@ import voluptuous as vol
 from homeassistant.const import (
     CONF_DOMAIN,
     CONF_ENTITY_ID,
+    CONF_HOST,
+    CONF_PASSWORD,
+    CONF_PATH,
+    CONF_PORT,
+    CONF_SSL,
     CONF_TIMEOUT,
+    CONF_TOKEN,
     CONF_UNIT_OF_MEASUREMENT,
     CONF_URL,
+    CONF_USERNAME,
+    CONF_VERIFY_SSL,
     EVENT_HOMEASSISTANT_STOP,
     EVENT_STATE_CHANGED,
     STATE_UNAVAILABLE,
@@ -56,23 +64,15 @@ from .const import (
     CONF_COMPONENT_CONFIG_GLOB,
     CONF_DB_NAME,
     CONF_DEFAULT_MEASUREMENT,
-    CONF_HOST,
     CONF_IGNORE_ATTRIBUTES,
     CONF_MEASUREMENT_ATTR,
     CONF_ORG,
     CONF_OVERRIDE_MEASUREMENT,
-    CONF_PASSWORD,
-    CONF_PATH,
-    CONF_PORT,
     CONF_PRECISION,
     CONF_RETRY_COUNT,
-    CONF_SSL,
     CONF_SSL_CA_CERT,
     CONF_TAGS,
     CONF_TAGS_ATTRIBUTES,
-    CONF_TOKEN,
-    CONF_USERNAME,
-    CONF_VERIFY_SSL,
     CONNECTION_ERROR,
     DEFAULT_API_VERSION,
     DEFAULT_HOST_V2,
@@ -135,19 +135,21 @@ def validate_version_specific_config(conf: dict) -> dict:
     if conf[CONF_API_VERSION] == API_VERSION_2:
         if CONF_TOKEN not in conf:
             raise vol.Invalid(
-                f"{CONF_TOKEN} and {CONF_BUCKET} are required when {CONF_API_VERSION} is {API_VERSION_2}"
+                f"{CONF_TOKEN} and {CONF_BUCKET} are required when"
+                f" {CONF_API_VERSION} is {API_VERSION_2}"
             )
 
         if CONF_USERNAME in conf:
             raise vol.Invalid(
-                f"{CONF_USERNAME} and {CONF_PASSWORD} are only allowed when {CONF_API_VERSION} is {DEFAULT_API_VERSION}"
+                f"{CONF_USERNAME} and {CONF_PASSWORD} are only allowed when"
+                f" {CONF_API_VERSION} is {DEFAULT_API_VERSION}"
             )
 
-    else:
-        if CONF_TOKEN in conf:
-            raise vol.Invalid(
-                f"{CONF_TOKEN} and {CONF_BUCKET} are only allowed when {CONF_API_VERSION} is {API_VERSION_2}"
-            )
+    elif CONF_TOKEN in conf:
+        raise vol.Invalid(
+            f"{CONF_TOKEN} and {CONF_BUCKET} are only allowed when"
+            f" {CONF_API_VERSION} is {API_VERSION_2}"
+        )
 
     return conf
 
@@ -218,7 +220,7 @@ def _generate_event_to_json(conf: dict) -> Callable[[Event], dict[str, Any] | No
         state: State | None = event.data.get(EVENT_NEW_STATE)
         if (
             state is None
-            or state.state in (STATE_UNKNOWN, "", STATE_UNAVAILABLE)
+            or state.state in (STATE_UNKNOWN, "", STATE_UNAVAILABLE, None)
             or not entity_filter(state.entity_id)
         ):
             return None
@@ -292,7 +294,7 @@ def _generate_event_to_json(conf: dict) -> Callable[[Event], dict[str, Any] | No
                     key = f"{key}_"
                 # Prevent column data errors in influxDB.
                 # For each value we try to cast it as float
-                # But if we can not do it we store the value
+                # But if we cannot do it we store the value
                 # as string add "_str" postfix to the field key
                 try:
                     json[INFLUX_CONF_FIELDS][key] = float(value)
