@@ -12,8 +12,8 @@ from homeassistant.components.binary_sensor import (
     BinarySensorEntityDescription,
 )
 from homeassistant.config_entries import ConfigEntry
+from homeassistant.const import EntityCategory
 from homeassistant.core import HomeAssistant
-from homeassistant.helpers.entity import EntityCategory
 from homeassistant.helpers.entity_platform import AddEntitiesCallback
 
 from . import IntellifireDataUpdateCoordinator
@@ -21,14 +21,14 @@ from .const import DOMAIN
 from .entity import IntellifireEntity
 
 
-@dataclass
+@dataclass(frozen=True)
 class IntellifireBinarySensorRequiredKeysMixin:
     """Mixin for required keys."""
 
     value_fn: Callable[[IntellifirePollData], bool]
 
 
-@dataclass
+@dataclass(frozen=True)
 class IntellifireBinarySensorEntityDescription(
     BinarySensorEntityDescription, IntellifireBinarySensorRequiredKeysMixin
 ):
@@ -38,45 +38,45 @@ class IntellifireBinarySensorEntityDescription(
 INTELLIFIRE_BINARY_SENSORS: tuple[IntellifireBinarySensorEntityDescription, ...] = (
     IntellifireBinarySensorEntityDescription(
         key="on_off",  # This is the sensor name
-        name="Flame",  # This is the human readable name
+        translation_key="flame",  # This is the translation key
         icon="mdi:fire",
         value_fn=lambda data: data.is_on,
     ),
     IntellifireBinarySensorEntityDescription(
         key="timer_on",
-        name="Timer On",
+        translation_key="timer_on",
         icon="mdi:camera-timer",
         value_fn=lambda data: data.timer_on,
     ),
     IntellifireBinarySensorEntityDescription(
         key="pilot_light_on",
-        name="Pilot Light On",
+        translation_key="pilot_light_on",
         icon="mdi:fire-alert",
         value_fn=lambda data: data.pilot_on,
     ),
     IntellifireBinarySensorEntityDescription(
         key="thermostat_on",
-        name="Thermostat On",
+        translation_key="thermostat_on",
         icon="mdi:home-thermometer-outline",
         value_fn=lambda data: data.thermostat_on,
     ),
     IntellifireBinarySensorEntityDescription(
         key="error_pilot_flame",
-        name="Pilot Flame Error",
+        translation_key="pilot_flame_error",
         entity_category=EntityCategory.DIAGNOSTIC,
         value_fn=lambda data: data.error_pilot_flame,
         device_class=BinarySensorDeviceClass.PROBLEM,
     ),
     IntellifireBinarySensorEntityDescription(
         key="error_flame",
-        name="Flame Error",
+        translation_key="flame_error",
         entity_category=EntityCategory.DIAGNOSTIC,
         value_fn=lambda data: data.error_flame,
         device_class=BinarySensorDeviceClass.PROBLEM,
     ),
     IntellifireBinarySensorEntityDescription(
         key="error_fan_delay",
-        name="Fan Delay Error",
+        translation_key="fan_delay_error",
         icon="mdi:fan-alert",
         entity_category=EntityCategory.DIAGNOSTIC,
         value_fn=lambda data: data.error_fan_delay,
@@ -84,21 +84,21 @@ INTELLIFIRE_BINARY_SENSORS: tuple[IntellifireBinarySensorEntityDescription, ...]
     ),
     IntellifireBinarySensorEntityDescription(
         key="error_maintenance",
-        name="Maintenance Error",
+        translation_key="maintenance_error",
         entity_category=EntityCategory.DIAGNOSTIC,
         value_fn=lambda data: data.error_maintenance,
         device_class=BinarySensorDeviceClass.PROBLEM,
     ),
     IntellifireBinarySensorEntityDescription(
         key="error_disabled",
-        name="Disabled Error",
+        translation_key="disabled_error",
         entity_category=EntityCategory.DIAGNOSTIC,
         value_fn=lambda data: data.error_disabled,
         device_class=BinarySensorDeviceClass.PROBLEM,
     ),
     IntellifireBinarySensorEntityDescription(
         key="error_fan",
-        name="Fan Error",
+        translation_key="fan_error",
         icon="mdi:fan-alert",
         entity_category=EntityCategory.DIAGNOSTIC,
         value_fn=lambda data: data.error_fan,
@@ -106,35 +106,35 @@ INTELLIFIRE_BINARY_SENSORS: tuple[IntellifireBinarySensorEntityDescription, ...]
     ),
     IntellifireBinarySensorEntityDescription(
         key="error_lights",
-        name="Lights Error",
+        translation_key="lights_error",
         entity_category=EntityCategory.DIAGNOSTIC,
         value_fn=lambda data: data.error_lights,
         device_class=BinarySensorDeviceClass.PROBLEM,
     ),
     IntellifireBinarySensorEntityDescription(
         key="error_accessory",
-        name="Accessory Error",
+        translation_key="accessory_error",
         entity_category=EntityCategory.DIAGNOSTIC,
         value_fn=lambda data: data.error_accessory,
         device_class=BinarySensorDeviceClass.PROBLEM,
     ),
     IntellifireBinarySensorEntityDescription(
         key="error_soft_lock_out",
-        name="Soft Lock Out Error",
+        translation_key="soft_lock_out_error",
         entity_category=EntityCategory.DIAGNOSTIC,
         value_fn=lambda data: data.error_soft_lock_out,
         device_class=BinarySensorDeviceClass.PROBLEM,
     ),
     IntellifireBinarySensorEntityDescription(
         key="error_ecm_offline",
-        name="ECM Offline Error",
+        translation_key="ecm_offline_error",
         entity_category=EntityCategory.DIAGNOSTIC,
         value_fn=lambda data: data.error_ecm_offline,
         device_class=BinarySensorDeviceClass.PROBLEM,
     ),
     IntellifireBinarySensorEntityDescription(
         key="error_offline",
-        name="Offline Error",
+        translation_key="offline_error",
         entity_category=EntityCategory.DIAGNOSTIC,
         value_fn=lambda data: data.error_offline,
         device_class=BinarySensorDeviceClass.PROBLEM,
