@@ -1,65 +1,73 @@
 """Support for IntesisHome and airconwithme Smart AC Controllers."""
+from __future__ import annotations
+
 import logging
 from random import randrange
+from typing import Any, NamedTuple
 
 from pyintesishome import IHAuthenticationError, IHConnectionError, IntesisHome
 import voluptuous as vol
 
-from homeassistant.components.climate import PLATFORM_SCHEMA, ClimateEntity
-from homeassistant.components.climate.const import (
+from homeassistant.components.climate import (
     ATTR_HVAC_MODE,
-    HVAC_MODE_COOL,
-    HVAC_MODE_DRY,
-    HVAC_MODE_FAN_ONLY,
-    HVAC_MODE_HEAT,
-    HVAC_MODE_HEAT_COOL,
-    HVAC_MODE_OFF,
+    PLATFORM_SCHEMA,
     PRESET_BOOST,
     PRESET_COMFORT,
     PRESET_ECO,
-    SUPPORT_FAN_MODE,
-    SUPPORT_PRESET_MODE,
-    SUPPORT_SWING_MODE,
-    SUPPORT_TARGET_TEMPERATURE,
     SWING_BOTH,
     SWING_HORIZONTAL,
     SWING_OFF,
     SWING_VERTICAL,
+    ClimateEntity,
+    ClimateEntityFeature,
+    HVACMode,
 )
 from homeassistant.const import (
     ATTR_TEMPERATURE,
     CONF_DEVICE,
     CONF_PASSWORD,
     CONF_USERNAME,
-    TEMP_CELSIUS,
+    UnitOfTemperature,
 )
+from homeassistant.core import HomeAssistant
 from homeassistant.exceptions import PlatformNotReady
 from homeassistant.helpers.aiohttp_client import async_get_clientsession
 import homeassistant.helpers.config_validation as cv
+from homeassistant.helpers.entity_platform import AddEntitiesCallback
 from homeassistant.helpers.event import async_call_later
+from homeassistant.helpers.typing import ConfigType, DiscoveryInfoType
 
 _LOGGER = logging.getLogger(__name__)
 
 IH_DEVICE_INTESISHOME = "IntesisHome"
 IH_DEVICE_AIRCONWITHME = "airconwithme"
+IH_DEVICE_ANYWAIR = "anywair"
 
 PLATFORM_SCHEMA = PLATFORM_SCHEMA.extend(
     {
         vol.Required(CONF_USERNAME): cv.string,
         vol.Required(CONF_PASSWORD): cv.string,
         vol.Optional(CONF_DEVICE, default=IH_DEVICE_INTESISHOME): vol.In(
-            [IH_DEVICE_AIRCONWITHME, IH_DEVICE_INTESISHOME]
+            [IH_DEVICE_AIRCONWITHME, IH_DEVICE_ANYWAIR, IH_DEVICE_INTESISHOME]
         ),
     }
 )
 
+
+class SwingSettings(NamedTuple):
+    """Settings for swing mode."""
+
+    vvane: str
+    hvane: str
+
+
 MAP_IH_TO_HVAC_MODE = {
-    "auto": HVAC_MODE_HEAT_COOL,
-    "cool": HVAC_MODE_COOL,
-    "dry": HVAC_MODE_DRY,
-    "fan": HVAC_MODE_FAN_ONLY,
-    "heat": HVAC_MODE_HEAT,
-    "off": HVAC_MODE_OFF,
+    "auto": HVACMode.HEAT_COOL,
+    "cool": HVACMode.COOL,
+    "dry": HVACMode.DRY,
+    "fan": HVACMode.FAN_ONLY,
+    "heat": HVACMode.HEAT,
+    "off": HVACMode.OFF,
 }
 MAP_HVAC_MODE_TO_IH = {v: k for k, v in MAP_IH_TO_HVAC_MODE.items()}
 
@@ -73,23 +81,28 @@ MAP_PRESET_MODE_TO_IH = {v: k for k, v in MAP_IH_TO_PRESET_MODE.items()}
 IH_SWING_STOP = "auto/stop"
 IH_SWING_SWING = "swing"
 MAP_SWING_TO_IH = {
-    SWING_OFF: {"vvane": IH_SWING_STOP, "hvane": IH_SWING_STOP},
-    SWING_BOTH: {"vvane": IH_SWING_SWING, "hvane": IH_SWING_SWING},
-    SWING_HORIZONTAL: {"vvane": IH_SWING_STOP, "hvane": IH_SWING_SWING},
-    SWING_VERTICAL: {"vvane": IH_SWING_SWING, "hvane": IH_SWING_STOP},
+    SWING_OFF: SwingSettings(vvane=IH_SWING_STOP, hvane=IH_SWING_STOP),
+    SWING_BOTH: SwingSettings(vvane=IH_SWING_SWING, hvane=IH_SWING_SWING),
+    SWING_HORIZONTAL: SwingSettings(vvane=IH_SWING_STOP, hvane=IH_SWING_SWING),
+    SWING_VERTICAL: SwingSettings(vvane=IH_SWING_SWING, hvane=IH_SWING_STOP),
 }
 
 
 MAP_STATE_ICONS = {
-    HVAC_MODE_COOL: "mdi:snowflake",
-    HVAC_MODE_DRY: "mdi:water-off",
-    HVAC_MODE_FAN_ONLY: "mdi:fan",
-    HVAC_MODE_HEAT: "mdi:white-balance-sunny",
-    HVAC_MODE_HEAT_COOL: "mdi:cached",
+    HVACMode.COOL: "mdi:snowflake",
+    HVACMode.DRY: "mdi:water-off",
+    HVACMode.FAN_ONLY: "mdi:fan",
+    HVACMode.HEAT: "mdi:white-balance-sunny",
+    HVACMode.HEAT_COOL: "mdi:cached",
 }
 
 
-async def async_setup_platform(hass, config, async_add_entities, discovery_info=None):
+async def async_setup_platform(
+    hass: HomeAssistant,
+    config: ConfigType,
+    async_add_entities: AddEntitiesCallback,
+    discovery_info: DiscoveryInfoType | None = None,
+) -> None:
     """Create the IntesisHome climate devices."""
     ih_user = config[CONF_USERNAME]
     ih_pass = config[CONF_PASSWORD]
@@ -107,12 +120,11 @@ async def async_setup_platform(hass, config, async_add_entities, discovery_info=
     except IHAuthenticationError:
         _LOGGER.error("Invalid username or password")
         return
-    except IHConnectionError:
+    except IHConnectionError as ex:
         _LOGGER.error("Error connecting to the %s server", device_type)
-        raise PlatformNotReady
+        raise PlatformNotReady from ex
 
-    ih_devices = controller.get_devices()
-    if ih_devices:
+    if ih_devices := controller.get_devices():
         async_add_entities(
             [
                 IntesisAC(ih_device_id, device, controller)
@@ -132,6 +144,10 @@ async def async_setup_platform(hass, config, async_add_entities, discovery_info=
 class IntesisAC(ClimateEntity):
     """Represents an Intesishome air conditioning device."""
 
+    _attr_should_poll = False
+    _attr_temperature_unit = UnitOfTemperature.CELSIUS
+    _enable_turn_on_off_backwards_compatibility = False
+
     def __init__(self, ih_device_id, ih_device, controller):
         """Initialize the thermostat."""
         self._controller = controller
@@ -143,7 +159,7 @@ class IntesisAC(ClimateEntity):
         self._setpoint_step = 1
         self._current_temp = None
         self._max_temp = None
-        self._hvac_mode_list = []
+        self._attr_hvac_modes = []
         self._min_temp = None
         self._target_temp = None
         self._outdoor_temp = None
@@ -157,13 +173,12 @@ class IntesisAC(ClimateEntity):
         self._hvane = None
         self._power = False
         self._fan_speed = None
-        self._support = 0
         self._power_consumption_heat = None
         self._power_consumption_cool = None
 
         # Setpoint support
         if controller.has_setpoint_control(ih_device_id):
-            self._support |= SUPPORT_TARGET_TEMPERATURE
+            self._attr_supported_features |= ClimateEntityFeature.TARGET_TEMPERATURE
 
         # Setup swing list
         if controller.has_vertical_swing(ih_device_id):
@@ -173,25 +188,29 @@ class IntesisAC(ClimateEntity):
         if SWING_HORIZONTAL in self._swing_list and SWING_VERTICAL in self._swing_list:
             self._swing_list.append(SWING_BOTH)
         if len(self._swing_list) > 1:
-            self._support |= SUPPORT_SWING_MODE
+            self._attr_supported_features |= ClimateEntityFeature.SWING_MODE
 
         # Setup fan speeds
         self._fan_modes = controller.get_fan_speed_list(ih_device_id)
         if self._fan_modes:
-            self._support |= SUPPORT_FAN_MODE
+            self._attr_supported_features |= ClimateEntityFeature.FAN_MODE
 
         # Preset support
         if ih_device.get("climate_working_mode"):
-            self._support |= SUPPORT_PRESET_MODE
+            self._attr_supported_features |= ClimateEntityFeature.PRESET_MODE
 
         # Setup HVAC modes
-        modes = controller.get_mode_list(ih_device_id)
-        if modes:
+        if modes := controller.get_mode_list(ih_device_id):
             mode_list = [MAP_IH_TO_HVAC_MODE[mode] for mode in modes]
-            self._hvac_mode_list.extend(mode_list)
-        self._hvac_mode_list.append(HVAC_MODE_OFF)
+            self._attr_hvac_modes.extend(mode_list)
+        self._attr_hvac_modes.append(HVACMode.OFF)
 
-    async def async_added_to_hass(self):
+        if len(self.hvac_modes) > 1:
+            self._attr_supported_features |= (
+                ClimateEntityFeature.TURN_OFF | ClimateEntityFeature.TURN_ON
+            )
+
+    async def async_added_to_hass(self) -> None:
         """Subscribe to event updates."""
         _LOGGER.debug("Added climate device with state: %s", repr(self._ih_device))
         await self._controller.add_update_callback(self.async_update_callback)
@@ -199,6 +218,7 @@ class IntesisAC(ClimateEntity):
             await self._controller.connect()
         except IHConnectionError as ex:
             _LOGGER.error("Exception connecting to IntesisHome: %s", ex)
+            raise PlatformNotReady from ex
 
     @property
     def name(self):
@@ -206,12 +226,7 @@ class IntesisAC(ClimateEntity):
         return self._device_name
 
     @property
-    def temperature_unit(self):
-        """Intesishome API uses celsius on the backend."""
-        return TEMP_CELSIUS
-
-    @property
-    def device_state_attributes(self):
+    def extra_state_attributes(self):
         """Return the device specific state attributes."""
         attrs = {}
         if self._outdoor_temp:
@@ -247,15 +262,12 @@ class IntesisAC(ClimateEntity):
         """Return the current preset mode."""
         return self._preset
 
-    async def async_set_temperature(self, **kwargs):
+    async def async_set_temperature(self, **kwargs: Any) -> None:
         """Set new target temperature."""
-        temperature = kwargs.get(ATTR_TEMPERATURE)
-        hvac_mode = kwargs.get(ATTR_HVAC_MODE)
-
-        if hvac_mode:
+        if hvac_mode := kwargs.get(ATTR_HVAC_MODE):
             await self.async_set_hvac_mode(hvac_mode)
 
-        if temperature:
+        if temperature := kwargs.get(ATTR_TEMPERATURE):
             _LOGGER.debug("Setting %s to %s degrees", self._device_type, temperature)
             await self._controller.set_temperature(self._device_id, temperature)
             self._target_temp = temperature
@@ -263,10 +275,10 @@ class IntesisAC(ClimateEntity):
         # Write updated temperature to HA state to avoid flapping (API confirmation is slow)
         self.async_write_ha_state()
 
-    async def async_set_hvac_mode(self, hvac_mode):
+    async def async_set_hvac_mode(self, hvac_mode: HVACMode) -> None:
         """Set operation mode."""
         _LOGGER.debug("Setting %s to %s mode", self._device_type, hvac_mode)
-        if hvac_mode == HVAC_MODE_OFF:
+        if hvac_mode == HVACMode.OFF:
             self._power = False
             await self._controller.set_power_off(self._device_id)
             # Write changes to HA, API can be slow to push changes
@@ -289,7 +301,7 @@ class IntesisAC(ClimateEntity):
         self._hvac_mode = hvac_mode
         self.async_write_ha_state()
 
-    async def async_set_fan_mode(self, fan_mode):
+    async def async_set_fan_mode(self, fan_mode: str) -> None:
         """Set fan mode (from quiet, low, medium, high, auto)."""
         await self._controller.set_fan_speed(self._device_id, fan_mode)
 
@@ -297,23 +309,22 @@ class IntesisAC(ClimateEntity):
         self._fan_speed = fan_mode
         self.async_write_ha_state()
 
-    async def async_set_preset_mode(self, preset_mode):
+    async def async_set_preset_mode(self, preset_mode: str) -> None:
         """Set preset mode."""
         ih_preset_mode = MAP_PRESET_MODE_TO_IH.get(preset_mode)
         await self._controller.set_preset_mode(self._device_id, ih_preset_mode)
 
-    async def async_set_swing_mode(self, swing_mode):
+    async def async_set_swing_mode(self, swing_mode: str) -> None:
         """Set the vertical vane."""
-        swing_settings = MAP_SWING_TO_IH.get(swing_mode)
-        if swing_settings:
+        if swing_settings := MAP_SWING_TO_IH.get(swing_mode):
             await self._controller.set_vertical_vane(
-                self._device_id, swing_settings.get("vvane")
+                self._device_id, swing_settings.vvane
             )
             await self._controller.set_horizontal_vane(
-                self._device_id, swing_settings.get("hvane")
+                self._device_id, swing_settings.hvane
             )
 
-    async def async_update(self):
+    async def async_update(self) -> None:
         """Copy values from controller dictionary to climate device."""
         # Update values from controller's device dictionary
         self._connected = self._controller.is_connected
@@ -348,7 +359,7 @@ class IntesisAC(ClimateEntity):
             self._device_id
         )
 
-    async def async_will_remove_from_hass(self):
+    async def async_will_remove_from_hass(self) -> None:
         """Shutdown the controller when the device is being removed."""
         await self._controller.stop()
 
@@ -373,9 +384,11 @@ class IntesisAC(ClimateEntity):
                 reconnect_minutes,
             )
             # Schedule reconnection
-            async_call_later(
-                self.hass, reconnect_minutes * 60, self._controller.connect()
-            )
+
+            async def try_connect(_now):
+                await self._controller.connect()
+
+            async_call_later(self.hass, reconnect_minutes * 60, try_connect)
 
         if self._controller.is_connected and not self._connected:
             # Connection has been restored
@@ -401,16 +414,6 @@ class IntesisAC(ClimateEntity):
         """Return the maximum temperature for the current mode of operation."""
         return self._max_temp
 
-    @property
-    def should_poll(self):
-        """Poll for updates if pyIntesisHome doesn't have a socket open."""
-        return False
-
-    @property
-    def hvac_modes(self):
-        """List of available operation modes."""
-        return self._hvac_mode_list
-
     @property
     def fan_mode(self):
         """Return whether the fan is on."""
@@ -450,18 +453,13 @@ class IntesisAC(ClimateEntity):
         return self._current_temp
 
     @property
-    def hvac_mode(self):
+    def hvac_mode(self) -> HVACMode:
         """Return the current mode of operation if unit is on."""
         if self._power:
             return self._hvac_mode
-        return HVAC_MODE_OFF
+        return HVACMode.OFF
 
     @property
     def target_temperature(self):
         """Return the current setpoint temperature if unit is on."""
         return self._target_temp
-
-    @property
-    def supported_features(self):
-        """Return the list of supported features."""
-        return self._support
