@@ -2,7 +2,6 @@
 import asyncio
 import logging
 
-import async_timeout
 from pyipma import IPMAException
 from pyipma.api import IPMA_API
 from pyipma.location import Location
@@ -18,7 +17,7 @@ from .const import DATA_API, DATA_LOCATION, DOMAIN
 
 DEFAULT_NAME = "ipma"
 
-PLATFORMS = [Platform.WEATHER]
+PLATFORMS = [Platform.SENSOR, Platform.WEATHER]
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -32,7 +31,7 @@ async def async_setup_entry(hass: HomeAssistant, config_entry: ConfigEntry) -> b
     api = IPMA_API(async_get_clientsession(hass))
 
     try:
-        async with async_timeout.timeout(30):
+        async with asyncio.timeout(30):
             location = await Location.get(api, float(latitude), float(longitude))
     except (IPMAException, asyncio.TimeoutError) as err:
         raise ConfigEntryNotReady(
@@ -57,4 +56,11 @@ async def async_setup_entry(hass: HomeAssistant, config_entry: ConfigEntry) -> b
 
 async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
     """Unload a config entry."""
-    return await hass.config_entries.async_unload_platforms(entry, PLATFORMS)
+
+    if unload_ok := await hass.config_entries.async_unload_platforms(entry, PLATFORMS):
+        hass.data[DOMAIN].pop(entry.entry_id)
+
+    if not hass.data[DOMAIN]:
+        hass.data.pop(DOMAIN)
+
+    return unload_ok
