@@ -1,16 +1,21 @@
 """Support for IQVIA."""
+from __future__ import annotations
+
 import asyncio
+from collections.abc import Callable, Coroutine
 from datetime import timedelta
 from functools import partial
+from typing import Any
 
 from pyiqvia import Client
 from pyiqvia.errors import IQVIAError
 
-from homeassistant.components.sensor import SensorEntity
-from homeassistant.const import ATTR_ATTRIBUTION
-from homeassistant.core import callback
+from homeassistant.config_entries import ConfigEntry
+from homeassistant.const import Platform
+from homeassistant.core import HomeAssistant, callback
 from homeassistant.exceptions import ConfigEntryNotReady
 from homeassistant.helpers import aiohttp_client
+from homeassistant.helpers.entity import EntityDescription
 from homeassistant.helpers.update_coordinator import (
     CoordinatorEntity,
     DataUpdateCoordinator,
@@ -19,7 +24,6 @@ from homeassistant.helpers.update_coordinator import (
 
 from .const import (
     CONF_ZIP_CODE,
-    DATA_COORDINATOR,
     DOMAIN,
     LOGGER,
     TYPE_ALLERGY_FORECAST,
@@ -34,14 +38,11 @@ from .const import (
 DEFAULT_ATTRIBUTION = "Data provided by IQVIA™"
 DEFAULT_SCAN_INTERVAL = timedelta(minutes=30)
 
-PLATFORMS = ["sensor"]
+PLATFORMS = [Platform.SENSOR]
 
 
-async def async_setup_entry(hass, entry):
+async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
     """Set up IQVIA as config entry."""
-    hass.data.setdefault(DOMAIN, {})
-    coordinators = {}
-
     if not entry.unique_id:
         # If the config entry doesn't already have a unique ID, set one:
         hass.config_entries.async_update_entry(
@@ -51,14 +52,22 @@ async def async_setup_entry(hass, entry):
     websession = aiohttp_client.async_get_clientsession(hass)
     client = Client(entry.data[CONF_ZIP_CODE], session=websession)
 
-    async def async_get_data_from_api(api_coro):
+    # We disable the client's request retry abilities here to avoid a lengthy (and
+    # blocking) startup:
+    client.disable_request_retries()
+
+    async def async_get_data_from_api(
+        api_coro: Callable[..., Coroutine[Any, Any, dict[str, Any]]],
+    ) -> dict[str, Any]:
         """Get data from a particular API coroutine."""
         try:
             return await api_coro()
         except IQVIAError as err:
             raise UpdateFailed from err
 
+    coordinators = {}
     init_data_update_tasks = []
+
     for sensor_type, api_coro in (
         (TYPE_ALLERGY_FORECAST, client.allergens.extended),
         (TYPE_ALLERGY_INDEX, client.allergens.current),
@@ -84,34 +93,44 @@ async def async_setup_entry(hass, entry):
         # API calls fail:
         raise ConfigEntryNotReady()
 
-    hass.data[DOMAIN].setdefault(DATA_COORDINATOR, {})[entry.entry_id] = coordinators
-    hass.config_entries.async_setup_platforms(entry, PLATFORMS)
+    # Once we've successfully authenticated, we re-enable client request retries:
+    client.enable_request_retries()
+
+    hass.data.setdefault(DOMAIN, {})
+    hass.data[DOMAIN][entry.entry_id] = coordinators
+
+    await hass.config_entries.async_forward_entry_setups(entry, PLATFORMS)
 
     return True
 
 
-async def async_unload_entry(hass, entry):
+async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
     """Unload an OpenUV config entry."""
     unload_ok = await hass.config_entries.async_unload_platforms(entry, PLATFORMS)
     if unload_ok:
-        hass.data[DOMAIN][DATA_COORDINATOR].pop(entry.entry_id)
+        hass.data[DOMAIN].pop(entry.entry_id)
+
     return unload_ok
 
 
-class IQVIAEntity(CoordinatorEntity, SensorEntity):
+class IQVIAEntity(CoordinatorEntity[DataUpdateCoordinator[dict[str, Any]]]):
     """Define a base IQVIA entity."""
 
-    def __init__(self, coordinator, entry, sensor_type, name, icon):
+    _attr_has_entity_name = True
+
+    def __init__(
+        self,
+        coordinator: DataUpdateCoordinator[dict[str, Any]],
+        entry: ConfigEntry,
+        description: EntityDescription,
+    ) -> None:
         """Initialize."""
         super().__init__(coordinator)
 
-        self._attr_extra_state_attributes = {ATTR_ATTRIBUTION: DEFAULT_ATTRIBUTION}
-        self._attr_icon = icon
-        self._attr_name = name
-        self._attr_unique_id = f"{entry.data[CONF_ZIP_CODE]}_{sensor_type}"
-        self._attr_native_unit_of_measurement = "index"
+        self._attr_extra_state_attributes = {}
+        self._attr_unique_id = f"{entry.data[CONF_ZIP_CODE]}_{description.key}"
         self._entry = entry
-        self._type = sensor_type
+        self.entity_description = description
 
     @callback
     def _handle_coordinator_update(self) -> None:
@@ -122,13 +141,13 @@ class IQVIAEntity(CoordinatorEntity, SensorEntity):
         self.update_from_latest_data()
         self.async_write_ha_state()
 
-    async def async_added_to_hass(self):
+    async def async_added_to_hass(self) -> None:
         """Register callbacks."""
         await super().async_added_to_hass()
 
-        if self._type == TYPE_ALLERGY_FORECAST:
+        if self.entity_description.key == TYPE_ALLERGY_FORECAST:
             self.async_on_remove(
-                self.hass.data[DOMAIN][DATA_COORDINATOR][self._entry.entry_id][
+                self.hass.data[DOMAIN][self._entry.entry_id][
                     TYPE_ALLERGY_OUTLOOK
                 ].async_add_listener(self._handle_coordinator_update)
             )
@@ -136,6 +155,6 @@ class IQVIAEntity(CoordinatorEntity, SensorEntity):
         self.update_from_latest_data()
 
     @callback
-    def update_from_latest_data(self):
+    def update_from_latest_data(self) -> None:
         """Update the entity from the latest data."""
         raise NotImplementedError
