@@ -17,11 +17,14 @@ from homeassistant.const import (
     STATE_UNAVAILABLE,
     STATE_UNKNOWN,
 )
-from homeassistant.core import Event, HomeAssistant, State, callback
-from homeassistant.helpers.event import async_track_state_change_event
-from homeassistant.helpers.typing import ConfigType, StateType
+from homeassistant.core import HomeAssistant, State, callback
+from homeassistant.helpers.event import (
+    EventStateChangedData,
+    async_track_state_change_event,
+)
+from homeassistant.helpers.typing import ConfigType, EventType, StateType
 
-from .const import KNX_ADDRESS
+from .const import CONF_RESPOND_TO_READ, KNX_ADDRESS
 from .schema import ExposeSchema
 
 _LOGGER = logging.getLogger(__name__)
@@ -32,27 +35,23 @@ def create_knx_exposure(
     hass: HomeAssistant, xknx: XKNX, config: ConfigType
 ) -> KNXExposeSensor | KNXExposeTime:
     """Create exposures from config."""
-    address = config[KNX_ADDRESS]
+
     expose_type = config[ExposeSchema.CONF_KNX_EXPOSE_TYPE]
-    attribute = config.get(ExposeSchema.CONF_KNX_EXPOSE_ATTRIBUTE)
-    default = config.get(ExposeSchema.CONF_KNX_EXPOSE_DEFAULT)
 
     exposure: KNXExposeSensor | KNXExposeTime
     if (
         isinstance(expose_type, str)
         and expose_type.lower() in ExposeSchema.EXPOSE_TIME_TYPES
     ):
-        exposure = KNXExposeTime(xknx, expose_type, address)
+        exposure = KNXExposeTime(
+            xknx=xknx,
+            config=config,
+        )
     else:
-        entity_id = config[CONF_ENTITY_ID]
         exposure = KNXExposeSensor(
             hass,
-            xknx,
-            expose_type,
-            entity_id,
-            attribute,
-            default,
-            address,
+            xknx=xknx,
+            config=config,
         )
     return exposure
 
@@ -64,36 +63,37 @@ class KNXExposeSensor:
         self,
         hass: HomeAssistant,
         xknx: XKNX,
-        expose_type: int | str,
-        entity_id: str,
-        attribute: str | None,
-        default: StateType,
-        address: str,
+        config: ConfigType,
     ) -> None:
         """Initialize of Expose class."""
         self.hass = hass
         self.xknx = xknx
-        self.type = expose_type
-        self.entity_id = entity_id
-        self.expose_attribute = attribute
-        self.expose_default = default
-        self.address = address
+
+        self.entity_id: str = config[CONF_ENTITY_ID]
+        self.expose_attribute: str | None = config.get(
+            ExposeSchema.CONF_KNX_EXPOSE_ATTRIBUTE
+        )
+        self.expose_default = config.get(ExposeSchema.CONF_KNX_EXPOSE_DEFAULT)
+        self.expose_type: int | str = config[ExposeSchema.CONF_KNX_EXPOSE_TYPE]
+
         self._remove_listener: Callable[[], None] | None = None
-        self.device: ExposeSensor = self.async_register()
+        self.device: ExposeSensor = self.async_register(config)
         self._init_expose_state()
 
     @callback
-    def async_register(self) -> ExposeSensor:
+    def async_register(self, config: ConfigType) -> ExposeSensor:
         """Register listener."""
         if self.expose_attribute is not None:
             _name = self.entity_id + "__" + self.expose_attribute
         else:
             _name = self.entity_id
         device = ExposeSensor(
-            self.xknx,
+            xknx=self.xknx,
             name=_name,
-            group_address=self.address,
-            value_type=self.type,
+            group_address=config[KNX_ADDRESS],
+            respond_to_read=config[CONF_RESPOND_TO_READ],
+            value_type=self.expose_type,
+            cooldown=config[ExposeSchema.CONF_KNX_EXPOSE_COOLDOWN],
         )
         self._remove_listener = async_track_state_change_event(
             self.hass, [self.entity_id], self._async_entity_changed
@@ -118,17 +118,17 @@ class KNXExposeSensor:
             self._remove_listener = None
         self.device.shutdown()
 
-    def _get_expose_value(self, state: State | None) -> StateType:
+    def _get_expose_value(self, state: State | None) -> bool | int | float | str | None:
         """Extract value from state."""
         if state is None or state.state in (STATE_UNKNOWN, STATE_UNAVAILABLE):
             value = self.expose_default
+        elif self.expose_attribute is not None:
+            _attr = state.attributes.get(self.expose_attribute)
+            value = _attr if _attr is not None else self.expose_default
         else:
-            value = (
-                state.state
-                if self.expose_attribute is None
-                else state.attributes.get(self.expose_attribute, self.expose_default)
-            )
-        if self.type == "binary":
+            value = state.state
+
+        if self.expose_type == "binary":
             if value in (1, STATE_ON, "True"):
                 return True
             if value in (0, STATE_OFF, "False"):
@@ -148,12 +148,14 @@ class KNXExposeSensor:
             return str(value)[:14]
         return value
 
-    async def _async_entity_changed(self, event: Event) -> None:
+    async def _async_entity_changed(
+        self, event: EventType[EventStateChangedData]
+    ) -> None:
         """Handle entity change."""
-        new_state = event.data.get("new_state")
+        new_state = event.data["new_state"]
         if (new_value := self._get_expose_value(new_state)) is None:
             return
-        old_state = event.data.get("old_state")
+        old_state = event.data["old_state"]
         # don't use default value for comparison on first state change (old_state is None)
         old_value = self._get_expose_value(old_state) if old_state is not None else None
         # don't send same value sequentially
@@ -164,29 +166,34 @@ class KNXExposeSensor:
         """Set new value on xknx ExposeSensor."""
         try:
             await self.device.set(value)
-        except ConversionError:
-            _LOGGER.exception("Error during sending of expose sensor value")
+        except ConversionError as err:
+            _LOGGER.warning(
+                'Could not expose %s %s value "%s" to KNX: %s',
+                self.entity_id,
+                self.expose_attribute or "state",
+                value,
+                err,
+            )
 
 
 class KNXExposeTime:
     """Object to Expose Time/Date object to KNX bus."""
 
-    def __init__(self, xknx: XKNX, expose_type: str, address: str) -> None:
+    def __init__(self, xknx: XKNX, config: ConfigType) -> None:
         """Initialize of Expose class."""
         self.xknx = xknx
-        self.expose_type = expose_type
-        self.address = address
-        self.device: DateTime = self.async_register()
+        self.device: DateTime = self.async_register(config)
 
     @callback
-    def async_register(self) -> DateTime:
+    def async_register(self, config: ConfigType) -> DateTime:
         """Register listener."""
+        expose_type = config[ExposeSchema.CONF_KNX_EXPOSE_TYPE]
         return DateTime(
             self.xknx,
-            name=self.expose_type.capitalize(),
-            broadcast_type=self.expose_type.upper(),
+            name=expose_type.capitalize(),
+            broadcast_type=expose_type.upper(),
             localtime=True,
-            group_address=self.address,
+            group_address=config[KNX_ADDRESS],
         )
 
     @callback
