@@ -3,7 +3,7 @@ from __future__ import annotations
 
 from typing import cast
 
-from xknx.devices import Climate as XknxClimate, Device as XknxDevice
+from xknx.devices import Device as XknxDevice
 
 from homeassistant.helpers.entity import Entity
 
@@ -14,10 +14,11 @@ from .const import DOMAIN
 class KnxEntity(Entity):
     """Representation of a KNX entity."""
 
+    _attr_should_poll = False
+
     def __init__(self, device: XknxDevice) -> None:
         """Set up device."""
         self._device = device
-        self._unique_id: str | None = None
 
     @property
     def name(self) -> str:
@@ -30,16 +31,6 @@ class KnxEntity(Entity):
         knx_module = cast(KNXModule, self.hass.data[DOMAIN])
         return knx_module.connected
 
-    @property
-    def should_poll(self) -> bool:
-        """No polling needed within KNX."""
-        return False
-
-    @property
-    def unique_id(self) -> str | None:
-        """Return the unique id of the device."""
-        return self._unique_id
-
     async def async_update(self) -> None:
         """Request a state update from KNX bus."""
         await self._device.sync()
@@ -51,13 +42,5 @@ class KnxEntity(Entity):
     async def async_added_to_hass(self) -> None:
         """Store register state change callback."""
         self._device.register_device_updated_cb(self.after_update_callback)
-
-        if isinstance(self._device, XknxClimate) and self._device.mode is not None:
-            self._device.mode.register_device_updated_cb(self.after_update_callback)
-
-    async def async_will_remove_from_hass(self) -> None:
-        """Disconnect device object when removed."""
-        self._device.unregister_device_updated_cb(self.after_update_callback)
-
-        if isinstance(self._device, XknxClimate) and self._device.mode is not None:
-            self._device.mode.unregister_device_updated_cb(self.after_update_callback)
+        # will remove all callbacks and xknx tasks
+        self.async_on_remove(self._device.shutdown)
