@@ -11,8 +11,11 @@ from homeassistant.components.media_player import (
 from homeassistant.config_entries import ConfigEntry
 from homeassistant.const import CONF_HOST, CONF_PORT
 from homeassistant.core import HomeAssistant
+from homeassistant.helpers.device_registry import DeviceInfo
 from homeassistant.helpers.entity_platform import AddEntitiesCallback
 
+from .const import DOMAIN
+
 
 async def async_setup_entry(
     hass: HomeAssistant,
@@ -25,7 +28,7 @@ async def async_setup_entry(
             LGDevice(
                 config_entry.data[CONF_HOST],
                 config_entry.data[CONF_PORT],
-                config_entry.unique_id,
+                config_entry.unique_id or config_entry.entry_id,
             )
         ]
     )
@@ -42,6 +45,8 @@ class LGDevice(MediaPlayerEntity):
         | MediaPlayerEntityFeature.SELECT_SOURCE
         | MediaPlayerEntityFeature.SELECT_SOUND_MODE
     )
+    _attr_has_entity_name = True
+    _attr_name = None
 
     def __init__(self, host, port, unique_id):
         """Initialize the LG speakers."""
@@ -66,6 +71,9 @@ class LGDevice(MediaPlayerEntity):
         self._bass = 0
         self._treble = 0
         self._device = None
+        self._attr_device_info = DeviceInfo(
+            identifiers={(DOMAIN, unique_id)}, name=host
+        )
 
     async def async_added_to_hass(self) -> None:
         """Register the callback after hass is ready for it."""
@@ -82,7 +90,7 @@ class LGDevice(MediaPlayerEntity):
 
     def handle_event(self, response):
         """Handle responses from the speakers."""
-        data = response["data"]
+        data = response["data"] if "data" in response else {}
         if response["msg"] == "EQ_VIEW_INFO":
             if "i_bass" in data:
                 self._bass = data["i_bass"]
