@@ -2,14 +2,16 @@
 import asyncio
 
 from aiohttp.client_exceptions import ClientResponseError
-import async_timeout
 from logi_circle import LogiCircle
 from logi_circle.exception import AuthorizationFailed
 import voluptuous as vol
 
 from homeassistant import config_entries
-from homeassistant.components.camera import ATTR_FILENAME, CAMERA_SERVICE_SCHEMA
+from homeassistant.components import persistent_notification
+from homeassistant.components.camera import ATTR_FILENAME
+from homeassistant.config_entries import ConfigEntry
 from homeassistant.const import (
+    ATTR_ENTITY_ID,
     ATTR_MODE,
     CONF_API_KEY,
     CONF_CLIENT_ID,
@@ -17,9 +19,12 @@ from homeassistant.const import (
     CONF_MONITORED_CONDITIONS,
     CONF_SENSORS,
     EVENT_HOMEASSISTANT_STOP,
+    Platform,
 )
+from homeassistant.core import HomeAssistant, ServiceCall
 from homeassistant.helpers import config_validation as cv
 from homeassistant.helpers.dispatcher import async_dispatcher_send
+from homeassistant.helpers.typing import ConfigType
 
 from . import config_flow
 from .const import (
@@ -28,12 +33,12 @@ from .const import (
     DEFAULT_CACHEDB,
     DOMAIN,
     LED_MODE_KEY,
-    LOGI_SENSORS,
     RECORDING_MODE_KEY,
     SIGNAL_LOGI_CIRCLE_RECONFIGURE,
     SIGNAL_LOGI_CIRCLE_RECORD,
     SIGNAL_LOGI_CIRCLE_SNAPSHOT,
 )
+from .sensor import SENSOR_TYPES
 
 NOTIFICATION_ID = "logi_circle_notification"
 NOTIFICATION_TITLE = "Logi Circle Setup"
@@ -47,10 +52,14 @@ SERVICE_LIVESTREAM_RECORD = "livestream_record"
 ATTR_VALUE = "value"
 ATTR_DURATION = "duration"
 
+PLATFORMS = [Platform.CAMERA, Platform.SENSOR]
+
+SENSOR_KEYS = [desc.key for desc in SENSOR_TYPES]
+
 SENSOR_SCHEMA = vol.Schema(
     {
-        vol.Optional(CONF_MONITORED_CONDITIONS, default=list(LOGI_SENSORS)): vol.All(
-            cv.ensure_list, [vol.In(LOGI_SENSORS)]
+        vol.Optional(CONF_MONITORED_CONDITIONS, default=SENSOR_KEYS): vol.All(
+            cv.ensure_list, [vol.In(SENSOR_KEYS)]
         )
     }
 )
@@ -70,26 +79,31 @@ CONFIG_SCHEMA = vol.Schema(
     extra=vol.ALLOW_EXTRA,
 )
 
-LOGI_CIRCLE_SERVICE_SET_CONFIG = CAMERA_SERVICE_SCHEMA.extend(
+LOGI_CIRCLE_SERVICE_SET_CONFIG = vol.Schema(
     {
+        vol.Optional(ATTR_ENTITY_ID): cv.comp_entity_ids,
         vol.Required(ATTR_MODE): vol.In([LED_MODE_KEY, RECORDING_MODE_KEY]),
         vol.Required(ATTR_VALUE): cv.boolean,
     }
 )
 
-LOGI_CIRCLE_SERVICE_SNAPSHOT = CAMERA_SERVICE_SCHEMA.extend(
-    {vol.Required(ATTR_FILENAME): cv.template}
+LOGI_CIRCLE_SERVICE_SNAPSHOT = vol.Schema(
+    {
+        vol.Optional(ATTR_ENTITY_ID): cv.comp_entity_ids,
+        vol.Required(ATTR_FILENAME): cv.template,
+    }
 )
 
-LOGI_CIRCLE_SERVICE_RECORD = CAMERA_SERVICE_SCHEMA.extend(
+LOGI_CIRCLE_SERVICE_RECORD = vol.Schema(
     {
+        vol.Optional(ATTR_ENTITY_ID): cv.comp_entity_ids,
         vol.Required(ATTR_FILENAME): cv.template,
         vol.Required(ATTR_DURATION): cv.positive_int,
     }
 )
 
 
-async def async_setup(hass, config):
+async def async_setup(hass: HomeAssistant, config: ConfigType) -> bool:
     """Set up configured Logi Circle component."""
     if DOMAIN not in config:
         return True
@@ -115,7 +129,7 @@ async def async_setup(hass, config):
     return True
 
 
-async def async_setup_entry(hass, entry):
+async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
     """Set up Logi Circle from a config entry."""
     logi_circle = LogiCircle(
         client_id=entry.data[CONF_CLIENT_ID],
@@ -126,10 +140,12 @@ async def async_setup_entry(hass, entry):
     )
 
     if not logi_circle.authorized:
-        hass.components.persistent_notification.create(
+        persistent_notification.create(
+            hass,
             (
-                f"Error: The cached access tokens are missing from {DEFAULT_CACHEDB}.<br />"
-                f"Please unload then re-add the Logi Circle integration to resolve."
+                "Error: The cached access tokens are missing from"
+                f" {DEFAULT_CACHEDB}.<br />Please unload then re-add the Logi Circle"
+                " integration to resolve."
             ),
             title=NOTIFICATION_TITLE,
             notification_id=NOTIFICATION_ID,
@@ -137,16 +153,19 @@ async def async_setup_entry(hass, entry):
         return False
 
     try:
-        with async_timeout.timeout(_TIMEOUT):
+        async with asyncio.timeout(_TIMEOUT):
             # Ensure the cameras property returns the same Camera objects for
             # all devices. Performs implicit login and session validation.
             await logi_circle.synchronize_cameras()
     except AuthorizationFailed:
-        hass.components.persistent_notification.create(
-            "Error: Failed to obtain an access token from the cached "
-            "refresh token.<br />"
-            "Token may have expired or been revoked.<br />"
-            "Please unload then re-add the Logi Circle integration to resolve",
+        persistent_notification.create(
+            hass,
+            (
+                "Error: Failed to obtain an access token from the cached "
+                "refresh token.<br />"
+                "Token may have expired or been revoked.<br />"
+                "Please unload then re-add the Logi Circle integration to resolve"
+            ),
             title=NOTIFICATION_TITLE,
             notification_id=NOTIFICATION_ID,
         )
@@ -155,14 +174,16 @@ async def async_setup_entry(hass, entry):
         # The TimeoutError exception object returns nothing when casted to a
         # string, so we'll handle it separately.
         err = f"{_TIMEOUT}s timeout exceeded when connecting to Logi Circle API"
-        hass.components.persistent_notification.create(
+        persistent_notification.create(
+            hass,
             f"Error: {err}<br />You will need to restart hass after fixing.",
             title=NOTIFICATION_TITLE,
             notification_id=NOTIFICATION_ID,
         )
         return False
     except ClientResponseError as ex:
-        hass.components.persistent_notification.create(
+        persistent_notification.create(
+            hass,
             f"Error: {ex}<br />You will need to restart hass after fixing.",
             title=NOTIFICATION_TITLE,
             notification_id=NOTIFICATION_ID,
@@ -171,12 +192,9 @@ async def async_setup_entry(hass, entry):
 
     hass.data[DATA_LOGI] = logi_circle
 
-    for component in "camera", "sensor":
-        hass.async_create_task(
-            hass.config_entries.async_forward_entry_setup(entry, component)
-        )
+    await hass.config_entries.async_forward_entry_setups(entry, PLATFORMS)
 
-    async def service_handler(service):
+    async def service_handler(service: ServiceCall) -> None:
         """Dispatch service calls to target entities."""
         params = dict(service.data)
 
@@ -212,15 +230,16 @@ async def async_setup_entry(hass, entry):
         """Close Logi Circle aiohttp session."""
         await logi_circle.auth_provider.close()
 
-    hass.bus.async_listen_once(EVENT_HOMEASSISTANT_STOP, shut_down)
+    entry.async_on_unload(
+        hass.bus.async_listen_once(EVENT_HOMEASSISTANT_STOP, shut_down)
+    )
 
     return True
 
 
-async def async_unload_entry(hass, entry):
+async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
     """Unload a config entry."""
-    for component in "camera", "sensor":
-        await hass.config_entries.async_forward_entry_unload(entry, component)
+    unload_ok = await hass.config_entries.async_unload_platforms(entry, PLATFORMS)
 
     logi_circle = hass.data.pop(DATA_LOGI)
 
@@ -228,4 +247,4 @@ async def async_unload_entry(hass, entry):
     # and clear all locally cached tokens
     await logi_circle.auth_provider.clear_authorization()
 
-    return True
+    return unload_ok
