@@ -51,10 +51,7 @@ AUTHORITIES = [
     "Westminster",
 ]
 
-URL = (
-    "http://api.erg.kcl.ac.uk/AirQuality/Hourly/"
-    "MonitoringIndex/GroupName=London/Json"
-)
+URL = "http://api.erg.kcl.ac.uk/AirQuality/Hourly/MonitoringIndex/GroupName=London/Json"
 
 PLATFORM_SCHEMA = PLATFORM_SCHEMA.extend(
     {
@@ -84,7 +81,7 @@ def setup_platform(
 class APIData:
     """Get the latest data for all authorities."""
 
-    def __init__(self):
+    def __init__(self) -> None:
         """Initialize the AirData object."""
         self.data = None
 
@@ -221,11 +218,12 @@ def parse_api_response(response):
     for authority in AUTHORITIES:
         for entry in response["HourlyAirQualityIndex"]["LocalAuthority"]:
             if entry["@LocalAuthorityName"] == authority:
-
-                if isinstance(entry["Site"], dict):
-                    entry_sites_data = [entry["Site"]]
-                else:
-                    entry_sites_data = entry["Site"]
+                entry_sites_data = []
+                if "Site" in entry:
+                    if isinstance(entry["Site"], dict):
+                        entry_sites_data = [entry["Site"]]
+                    else:
+                        entry_sites_data = entry["Site"]
 
                 data[authority] = parse_site(entry_sites_data)
 
