@@ -3,8 +3,11 @@ import logging
 
 import voluptuous as vol
 
-from homeassistant.components import frontend
-from homeassistant.config import async_hass_config_yaml, async_process_component_config
+from homeassistant.components import frontend, websocket_api
+from homeassistant.config import (
+    async_hass_config_yaml,
+    async_process_component_and_handle_errors,
+)
 from homeassistant.const import CONF_FILENAME, CONF_MODE, CONF_RESOURCES
 from homeassistant.core import HomeAssistant, ServiceCall, callback
 from homeassistant.exceptions import HomeAssistantError
@@ -14,7 +17,7 @@ from homeassistant.helpers.typing import ConfigType
 from homeassistant.loader import async_get_integration
 
 from . import dashboard, resources, websocket
-from .const import (
+from .const import (  # noqa: F401
     CONF_ICON,
     CONF_REQUIRE_ADMIN,
     CONF_SHOW_IN_SIDEBAR,
@@ -23,6 +26,7 @@ from .const import (
     DASHBOARD_BASE_CREATE_FIELDS,
     DEFAULT_ICON,
     DOMAIN,
+    EVENT_LOVELACE_UPDATED,
     MODE_STORAGE,
     MODE_YAML,
     RESOURCE_CREATE_FIELDS,
@@ -67,7 +71,7 @@ CONFIG_SCHEMA = vol.Schema(
 )
 
 
-async def async_setup(hass: HomeAssistant, config: ConfigType):
+async def async_setup(hass: HomeAssistant, config: ConfigType) -> bool:
     """Set up the Lovelace commands."""
     mode = config[DOMAIN][CONF_MODE]
     yaml_resources = config[DOMAIN].get(CONF_RESOURCES)
@@ -84,13 +88,19 @@ async def async_setup(hass: HomeAssistant, config: ConfigType):
 
         integration = await async_get_integration(hass, DOMAIN)
 
-        config = await async_process_component_config(hass, conf, integration)
+        config = await async_process_component_and_handle_errors(
+            hass, conf, integration
+        )
+
+        if config is None:
+            raise HomeAssistantError("Config validation failed")
 
         resource_collection = await create_yaml_resource_col(
             hass, config[DOMAIN].get(CONF_RESOURCES)
         )
         hass.data[DOMAIN]["resources"] = resource_collection
 
+    default_config: dashboard.LovelaceConfig
     if mode == MODE_YAML:
         default_config = dashboard.LovelaceYAML(hass, None, None)
         resource_collection = await create_yaml_resource_col(hass, yaml_resources)
@@ -108,12 +118,13 @@ async def async_setup(hass: HomeAssistant, config: ConfigType):
 
         if yaml_resources is not None:
             _LOGGER.warning(
-                "Lovelace is running in storage mode. Define resources via user interface"
+                "Lovelace is running in storage mode. Define resources via user"
+                " interface"
             )
 
         resource_collection = resources.ResourceStorageCollection(hass, default_config)
 
-        collection.StorageCollectionWebsocket(
+        collection.DictStorageCollectionWebsocket(
             resource_collection,
             "lovelace/resources",
             "resource",
@@ -121,31 +132,24 @@ async def async_setup(hass: HomeAssistant, config: ConfigType):
             RESOURCE_UPDATE_FIELDS,
         ).async_setup(hass, create_list=False)
 
-    hass.components.websocket_api.async_register_command(
-        websocket.websocket_lovelace_config
-    )
-    hass.components.websocket_api.async_register_command(
-        websocket.websocket_lovelace_save_config
-    )
-    hass.components.websocket_api.async_register_command(
-        websocket.websocket_lovelace_delete_config
-    )
-    hass.components.websocket_api.async_register_command(
-        websocket.websocket_lovelace_resources
+    websocket_api.async_register_command(hass, websocket.websocket_lovelace_config)
+    websocket_api.async_register_command(hass, websocket.websocket_lovelace_save_config)
+    websocket_api.async_register_command(
+        hass, websocket.websocket_lovelace_delete_config
     )
+    websocket_api.async_register_command(hass, websocket.websocket_lovelace_resources)
 
-    hass.components.websocket_api.async_register_command(
-        websocket.websocket_lovelace_dashboards
-    )
+    websocket_api.async_register_command(hass, websocket.websocket_lovelace_dashboards)
 
     hass.data[DOMAIN] = {
         # We store a dictionary mapping url_path: config. None is the default.
+        "mode": mode,
         "dashboards": {None: default_config},
         "resources": resource_collection,
         "yaml_dashboards": config[DOMAIN].get(CONF_DASHBOARDS, {}),
     }
 
-    if hass.config.safe_mode:
+    if hass.config.recovery_mode:
         return True
 
     async def storage_dashboard_changed(change_type, item_id, item):
@@ -158,7 +162,6 @@ async def async_setup(hass: HomeAssistant, config: ConfigType):
             return
 
         if change_type == collection.CHANGE_ADDED:
-
             existing = hass.data[DOMAIN]["dashboards"].get(url_path)
 
             if existing:
@@ -186,8 +189,8 @@ async def async_setup(hass: HomeAssistant, config: ConfigType):
     # Process YAML dashboards
     for url_path, dashboard_conf in hass.data[DOMAIN]["yaml_dashboards"].items():
         # For now always mode=yaml
-        config = dashboard.LovelaceYAML(hass, url_path, dashboard_conf)
-        hass.data[DOMAIN]["dashboards"][url_path] = config
+        lovelace_config = dashboard.LovelaceYAML(hass, url_path, dashboard_conf)
+        hass.data[DOMAIN]["dashboards"][url_path] = lovelace_config
 
         try:
             _register_panel(hass, url_path, MODE_YAML, dashboard_conf, False)
@@ -200,7 +203,7 @@ async def async_setup(hass: HomeAssistant, config: ConfigType):
     dashboards_collection.async_add_listener(storage_dashboard_changed)
     await dashboards_collection.async_load()
 
-    collection.StorageCollectionWebsocket(
+    collection.DictStorageCollectionWebsocket(
         dashboards_collection,
         "lovelace/dashboards",
         "dashboard",
@@ -222,7 +225,8 @@ async def create_yaml_resource_col(hass, yaml_resources):
         else:
             if CONF_RESOURCES in ll_conf:
                 _LOGGER.warning(
-                    "Resources need to be specified in your configuration.yaml. Please see the docs"
+                    "Resources need to be specified in your configuration.yaml. Please"
+                    " see the docs"
                 )
                 yaml_resources = ll_conf[CONF_RESOURCES]
 
