@@ -1,48 +1,51 @@
 """Support for Lutron Caseta fans."""
-import logging
+from __future__ import annotations
+
+from typing import Any
 
 from pylutron_caseta import FAN_HIGH, FAN_LOW, FAN_MEDIUM, FAN_MEDIUM_HIGH, FAN_OFF
 
-from homeassistant.components.fan import DOMAIN, SUPPORT_SET_SPEED, FanEntity
+from homeassistant.components.fan import DOMAIN, FanEntity, FanEntityFeature
+from homeassistant.config_entries import ConfigEntry
+from homeassistant.core import HomeAssistant
+from homeassistant.helpers.entity_platform import AddEntitiesCallback
 from homeassistant.util.percentage import (
     ordered_list_item_to_percentage,
     percentage_to_ordered_list_item,
 )
 
-from . import LutronCasetaDevice
-from .const import BRIDGE_DEVICE, BRIDGE_LEAP, DOMAIN as CASETA_DOMAIN
-
-_LOGGER = logging.getLogger(__name__)
+from . import LutronCasetaDeviceUpdatableEntity
+from .const import DOMAIN as CASETA_DOMAIN
+from .models import LutronCasetaData
 
 DEFAULT_ON_PERCENTAGE = 50
 ORDERED_NAMED_FAN_SPEEDS = [FAN_LOW, FAN_MEDIUM, FAN_MEDIUM_HIGH, FAN_HIGH]
 
 
-async def async_setup_entry(hass, config_entry, async_add_entities):
+async def async_setup_entry(
+    hass: HomeAssistant,
+    config_entry: ConfigEntry,
+    async_add_entities: AddEntitiesCallback,
+) -> None:
     """Set up the Lutron Caseta fan platform.
 
     Adds fan controllers from the Caseta bridge associated with the config_entry
     as fan entities.
     """
-
-    entities = []
-    data = hass.data[CASETA_DOMAIN][config_entry.entry_id]
-    bridge = data[BRIDGE_LEAP]
-    bridge_device = data[BRIDGE_DEVICE]
+    data: LutronCasetaData = hass.data[CASETA_DOMAIN][config_entry.entry_id]
+    bridge = data.bridge
     fan_devices = bridge.get_devices_by_domain(DOMAIN)
-
-    for fan_device in fan_devices:
-        entity = LutronCasetaFan(fan_device, bridge, bridge_device)
-        entities.append(entity)
-
-    async_add_entities(entities, True)
+    async_add_entities(LutronCasetaFan(fan_device, data) for fan_device in fan_devices)
 
 
-class LutronCasetaFan(LutronCasetaDevice, FanEntity):
+class LutronCasetaFan(LutronCasetaDeviceUpdatableEntity, FanEntity):
     """Representation of a Lutron Caseta fan. Including Fan Speed."""
 
+    _attr_supported_features = FanEntityFeature.SET_SPEED
+    _attr_speed_count = len(ORDERED_NAMED_FAN_SPEEDS)
+
     @property
-    def percentage(self) -> str:
+    def percentage(self) -> int | None:
         """Return the current speed percentage."""
         if self._device["fan_speed"] is None:
             return None
@@ -52,30 +55,19 @@ class LutronCasetaFan(LutronCasetaDevice, FanEntity):
             ORDERED_NAMED_FAN_SPEEDS, self._device["fan_speed"]
         )
 
-    @property
-    def speed_count(self) -> int:
-        """Return the number of speeds the fan supports."""
-        return len(ORDERED_NAMED_FAN_SPEEDS)
-
-    @property
-    def supported_features(self) -> int:
-        """Flag supported features. Speed Only."""
-        return SUPPORT_SET_SPEED
-
     async def async_turn_on(
         self,
-        speed: str = None,
-        percentage: int = None,
-        preset_mode: str = None,
-        **kwargs,
-    ):
+        percentage: int | None = None,
+        preset_mode: str | None = None,
+        **kwargs: Any,
+    ) -> None:
         """Turn the fan on."""
         if percentage is None:
             percentage = DEFAULT_ON_PERCENTAGE
 
         await self.async_set_percentage(percentage)
 
-    async def async_turn_off(self, **kwargs):
+    async def async_turn_off(self, **kwargs: Any) -> None:
         """Turn the fan off."""
         await self.async_set_percentage(0)
 
@@ -91,11 +83,6 @@ class LutronCasetaFan(LutronCasetaDevice, FanEntity):
         await self._smartbridge.set_fan(self.device_id, named_speed)
 
     @property
-    def is_on(self):
+    def is_on(self) -> bool:
         """Return true if device is on."""
-        return self.percentage and self.percentage > 0
-
-    async def async_update(self):
-        """Update when forcing a refresh of the device."""
-        self._device = self._smartbridge.get_device_by_id(self.device_id)
-        _LOGGER.debug("State of this lutron fan device is %s", self._device)
+        return bool(self.percentage)
