@@ -14,8 +14,9 @@ from homeassistant.components.sensor import (
     SensorStateClass,
 )
 from homeassistant.config_entries import ConfigEntry
-from homeassistant.const import TEMP_CELSIUS
+from homeassistant.const import UnitOfTemperature
 from homeassistant.core import HomeAssistant, callback
+from homeassistant.helpers.device_registry import DeviceInfo
 from homeassistant.helpers.entity_platform import AddEntitiesCallback
 from homeassistant.helpers.update_coordinator import (
     CoordinatorEntity,
@@ -26,7 +27,7 @@ from homeassistant.util import dt as dt_util
 from .const import DOMAIN
 
 
-@dataclass
+@dataclass(frozen=True)
 class MeaterSensorEntityDescriptionMixin:
     """Mixin for MeaterSensorEntityDescription."""
 
@@ -34,7 +35,7 @@ class MeaterSensorEntityDescriptionMixin:
     value: Callable[[MeaterProbe], datetime | float | str | None]
 
 
-@dataclass
+@dataclass(frozen=True)
 class MeaterSensorEntityDescription(
     SensorEntityDescription, MeaterSensorEntityDescriptionMixin
 ):
@@ -43,25 +44,29 @@ class MeaterSensorEntityDescription(
 
 def _elapsed_time_to_timestamp(probe: MeaterProbe) -> datetime | None:
     """Convert elapsed time to timestamp."""
-    if not probe.cook:
+    if not probe.cook or not hasattr(probe.cook, "time_elapsed"):
         return None
     return dt_util.utcnow() - timedelta(seconds=probe.cook.time_elapsed)
 
 
 def _remaining_time_to_timestamp(probe: MeaterProbe) -> datetime | None:
     """Convert remaining time to timestamp."""
-    if not probe.cook or probe.cook.time_remaining < 0:
+    if (
+        not probe.cook
+        or not hasattr(probe.cook, "time_remaining")
+        or probe.cook.time_remaining < 0
+    ):
         return None
-    return dt_util.utcnow() + timedelta(probe.cook.time_remaining)
+    return dt_util.utcnow() + timedelta(seconds=probe.cook.time_remaining)
 
 
 SENSOR_TYPES = (
     # Ambient temperature
     MeaterSensorEntityDescription(
         key="ambient",
+        translation_key="ambient",
         device_class=SensorDeviceClass.TEMPERATURE,
-        name="Ambient",
-        native_unit_of_measurement=TEMP_CELSIUS,
+        native_unit_of_measurement=UnitOfTemperature.CELSIUS,
         state_class=SensorStateClass.MEASUREMENT,
         available=lambda probe: probe is not None,
         value=lambda probe: probe.ambient_temperature,
@@ -69,9 +74,9 @@ SENSOR_TYPES = (
     # Internal temperature (probe tip)
     MeaterSensorEntityDescription(
         key="internal",
+        translation_key="internal",
         device_class=SensorDeviceClass.TEMPERATURE,
-        name="Internal",
-        native_unit_of_measurement=TEMP_CELSIUS,
+        native_unit_of_measurement=UnitOfTemperature.CELSIUS,
         state_class=SensorStateClass.MEASUREMENT,
         available=lambda probe: probe is not None,
         value=lambda probe: probe.internal_temperature,
@@ -79,7 +84,7 @@ SENSOR_TYPES = (
     # Name of selected meat in user language or user given custom name
     MeaterSensorEntityDescription(
         key="cook_name",
-        name="Cooking",
+        translation_key="cook_name",
         available=lambda probe: probe is not None and probe.cook is not None,
         value=lambda probe: probe.cook.name if probe.cook else None,
     ),
@@ -87,36 +92,40 @@ SENSOR_TYPES = (
     # Slightly Underdone, Finished, Slightly Overdone, OVERCOOK!. Not translated.
     MeaterSensorEntityDescription(
         key="cook_state",
-        name="Cook state",
+        translation_key="cook_state",
         available=lambda probe: probe is not None and probe.cook is not None,
         value=lambda probe: probe.cook.state if probe.cook else None,
     ),
     # Target temperature
     MeaterSensorEntityDescription(
         key="cook_target_temp",
+        translation_key="cook_target_temp",
         device_class=SensorDeviceClass.TEMPERATURE,
-        name="Target",
-        native_unit_of_measurement=TEMP_CELSIUS,
+        native_unit_of_measurement=UnitOfTemperature.CELSIUS,
         state_class=SensorStateClass.MEASUREMENT,
         available=lambda probe: probe is not None and probe.cook is not None,
-        value=lambda probe: probe.cook.target_temperature if probe.cook else None,
+        value=lambda probe: probe.cook.target_temperature
+        if probe.cook and hasattr(probe.cook, "target_temperature")
+        else None,
     ),
     # Peak temperature
     MeaterSensorEntityDescription(
         key="cook_peak_temp",
+        translation_key="cook_peak_temp",
         device_class=SensorDeviceClass.TEMPERATURE,
-        name="Peak",
-        native_unit_of_measurement=TEMP_CELSIUS,
+        native_unit_of_measurement=UnitOfTemperature.CELSIUS,
         state_class=SensorStateClass.MEASUREMENT,
         available=lambda probe: probe is not None and probe.cook is not None,
-        value=lambda probe: probe.cook.peak_temperature if probe.cook else None,
+        value=lambda probe: probe.cook.peak_temperature
+        if probe.cook and hasattr(probe.cook, "peak_temperature")
+        else None,
     ),
     # Remaining time in seconds. When unknown/calculating default is used. Default: -1
     # Exposed as a TIMESTAMP sensor where the timestamp is current time + remaining time.
     MeaterSensorEntityDescription(
         key="cook_time_remaining",
+        translation_key="cook_time_remaining",
         device_class=SensorDeviceClass.TIMESTAMP,
-        name="Remaining time",
         available=lambda probe: probe is not None and probe.cook is not None,
         value=_remaining_time_to_timestamp,
     ),
@@ -124,8 +133,8 @@ SENSOR_TYPES = (
     # where the timestamp is current time - elapsed time.
     MeaterSensorEntityDescription(
         key="cook_time_elapsed",
+        translation_key="cook_time_elapsed",
         device_class=SensorDeviceClass.TIMESTAMP,
-        name="Elapsed time",
         available=lambda probe: probe is not None and probe.cook is not None,
         value=_elapsed_time_to_timestamp,
     ),
@@ -136,9 +145,9 @@ async def async_setup_entry(
     hass: HomeAssistant, entry: ConfigEntry, async_add_entities: AddEntitiesCallback
 ) -> None:
     """Set up the entry."""
-    coordinator: DataUpdateCoordinator = hass.data[DOMAIN][entry.entry_id][
-        "coordinator"
-    ]
+    coordinator: DataUpdateCoordinator[dict[str, MeaterProbe]] = hass.data[DOMAIN][
+        entry.entry_id
+    ]["coordinator"]
 
     @callback
     def async_update_data():
@@ -146,22 +155,22 @@ async def async_setup_entry(
         if not coordinator.last_update_success:
             return
 
-        devices: dict[str, MeaterProbe] = coordinator.data
+        devices = coordinator.data
         entities = []
         known_probes: set = hass.data[DOMAIN]["known_probes"]
 
         # Add entities for temperature probes which we've not yet seen
-        for dev in devices:
-            if dev in known_probes:
+        for device_id in devices:
+            if device_id in known_probes:
                 continue
 
             entities.extend(
                 [
-                    MeaterProbeTemperature(coordinator, dev, sensor_description)
+                    MeaterProbeTemperature(coordinator, device_id, sensor_description)
                     for sensor_description in SENSOR_TYPES
                 ]
             )
-            known_probes.add(dev)
+            known_probes.add(device_id)
 
         async_add_entities(entities)
 
@@ -176,8 +185,6 @@ class MeaterProbeTemperature(
 ):
     """Meater Temperature Sensor Entity."""
 
-    _attr_device_class = SensorDeviceClass.TEMPERATURE
-    _attr_native_unit_of_measurement = TEMP_CELSIUS
     entity_description: MeaterSensorEntityDescription
 
     def __init__(
@@ -185,16 +192,15 @@ class MeaterProbeTemperature(
     ) -> None:
         """Initialise the sensor."""
         super().__init__(coordinator)
-        self._attr_name = f"Meater Probe {description.name}"
-        self._attr_device_info = {
-            "identifiers": {
+        self._attr_device_info = DeviceInfo(
+            identifiers={
                 # Serial numbers are unique identifiers within a specific domain
                 (DOMAIN, device_id)
             },
-            "manufacturer": "Apption Labs",
-            "model": "Meater Probe",
-            "name": f"Meater Probe {device_id}",
-        }
+            manufacturer="Apption Labs",
+            model="Meater Probe",
+            name=f"Meater Probe {device_id}",
+        )
         self._attr_unique_id = f"{device_id}-{description.key}"
 
         self.device_id = device_id
@@ -209,7 +215,7 @@ class MeaterProbeTemperature(
         return self.entity_description.value(device)
 
     @property
-    def available(self):
+    def available(self) -> bool:
         """Return if entity is available."""
         # See if the device was returned from the API. If not, it's offline
         return (
