@@ -1,4 +1,7 @@
 """Support for the Meraki CMX location service."""
+from __future__ import annotations
+
+from http import HTTPStatus
 import json
 import logging
 
@@ -6,12 +9,13 @@ import voluptuous as vol
 
 from homeassistant.components.device_tracker import (
     PLATFORM_SCHEMA as PARENT_PLATFORM_SCHEMA,
-    SOURCE_TYPE_ROUTER,
+    AsyncSeeCallback,
+    SourceType,
 )
 from homeassistant.components.http import HomeAssistantView
-from homeassistant.const import HTTP_BAD_REQUEST, HTTP_UNPROCESSABLE_ENTITY
-from homeassistant.core import callback
+from homeassistant.core import HomeAssistant, callback
 import homeassistant.helpers.config_validation as cv
+from homeassistant.helpers.typing import ConfigType, DiscoveryInfoType
 
 CONF_VALIDATOR = "validator"
 CONF_SECRET = "secret"
@@ -27,7 +31,12 @@ PLATFORM_SCHEMA = PARENT_PLATFORM_SCHEMA.extend(
 )
 
 
-async def async_setup_scanner(hass, config, async_see, discovery_info=None):
+async def async_setup_scanner(
+    hass: HomeAssistant,
+    config: ConfigType,
+    async_see: AsyncSeeCallback,
+    discovery_info: DiscoveryInfoType | None = None,
+) -> bool:
     """Set up an endpoint for the Meraki tracker."""
     hass.http.register_view(MerakiView(config, async_see))
 
@@ -41,7 +50,7 @@ class MerakiView(HomeAssistantView):
     name = "api:meraki"
     requires_auth = False
 
-    def __init__(self, config, async_see):
+    def __init__(self, config: ConfigType, async_see: AsyncSeeCallback) -> None:
         """Initialize Meraki URL endpoints."""
         self.async_see = async_see
         self.validator = config[CONF_VALIDATOR]
@@ -56,21 +65,23 @@ class MerakiView(HomeAssistantView):
         try:
             data = await request.json()
         except ValueError:
-            return self.json_message("Invalid JSON", HTTP_BAD_REQUEST)
+            return self.json_message("Invalid JSON", HTTPStatus.BAD_REQUEST)
         _LOGGER.debug("Meraki Data from Post: %s", json.dumps(data))
         if not data.get("secret", False):
             _LOGGER.error("The secret is invalid")
-            return self.json_message("No secret", HTTP_UNPROCESSABLE_ENTITY)
+            return self.json_message("No secret", HTTPStatus.UNPROCESSABLE_ENTITY)
         if data["secret"] != self.secret:
             _LOGGER.error("Invalid Secret received from Meraki")
-            return self.json_message("Invalid secret", HTTP_UNPROCESSABLE_ENTITY)
+            return self.json_message("Invalid secret", HTTPStatus.UNPROCESSABLE_ENTITY)
         if data["version"] != VERSION:
             _LOGGER.error("Invalid API version: %s", data["version"])
-            return self.json_message("Invalid version", HTTP_UNPROCESSABLE_ENTITY)
+            return self.json_message("Invalid version", HTTPStatus.UNPROCESSABLE_ENTITY)
         _LOGGER.debug("Valid Secret")
         if data["type"] not in ("DevicesSeen", "BluetoothDevicesSeen"):
             _LOGGER.error("Unknown Device %s", data["type"])
-            return self.json_message("Invalid device type", HTTP_UNPROCESSABLE_ENTITY)
+            return self.json_message(
+                "Invalid device type", HTTPStatus.UNPROCESSABLE_ENTITY
+            )
         _LOGGER.debug("Processing %s", data["type"])
         if not data["data"]["observations"]:
             _LOGGER.debug("No observations found")
@@ -116,7 +127,7 @@ class MerakiView(HomeAssistantView):
                 self.async_see(
                     gps=gps_location,
                     mac=mac,
-                    source_type=SOURCE_TYPE_ROUTER,
+                    source_type=SourceType.ROUTER,
                     gps_accuracy=accuracy,
                     attributes=attrs,
                 )
