@@ -1,8 +1,6 @@
 """Constants used in the Mikrotik components."""
 from typing import Final
 
-from homeassistant.const import Platform
-
 DOMAIN: Final = "mikrotik"
 DEFAULT_NAME: Final = "Mikrotik"
 DEFAULT_API_PORT: Final = 8728
@@ -26,8 +24,13 @@ ARP: Final = "arp"
 CAPSMAN: Final = "capsman"
 DHCP: Final = "dhcp"
 WIRELESS: Final = "wireless"
+WIFIWAVE2: Final = "wifiwave2"
+WIFI: Final = "wifi"
 IS_WIRELESS: Final = "is_wireless"
 IS_CAPSMAN: Final = "is_capsman"
+IS_WIFIWAVE2: Final = "is_wifiwave2"
+IS_WIFI: Final = "is_wifi"
+
 
 MIKROTIK_SERVICES: Final = {
     ARP: "/ip/arp/getall",
@@ -36,11 +39,14 @@ MIKROTIK_SERVICES: Final = {
     IDENTITY: "/system/identity/getall",
     INFO: "/system/routerboard/getall",
     WIRELESS: "/interface/wireless/registration-table/getall",
+    WIFIWAVE2: "/interface/wifiwave2/registration-table/print",
+    WIFI: "/interface/wifi/registration-table/print",
     IS_WIRELESS: "/interface/wireless/print",
     IS_CAPSMAN: "/caps-man/interface/print",
+    IS_WIFIWAVE2: "/interface/wifiwave2/print",
+    IS_WIFI: "/interface/wifi/print",
 }
 
-PLATFORMS: Final = [Platform.DEVICE_TRACKER]
 
 ATTR_DEVICE_TRACKER: Final = [
     "comment",
