@@ -1,20 +1,28 @@
 """Webhook handlers for mobile_app."""
+from __future__ import annotations
+
 import asyncio
+from collections.abc import Callable, Coroutine
 from contextlib import suppress
-from functools import wraps
+from functools import lru_cache, wraps
 from http import HTTPStatus
 import logging
 import secrets
+from typing import Any
 
 from aiohttp.web import HTTPBadRequest, Request, Response, json_response
 from nacl.exceptions import CryptoError
 from nacl.secret import SecretBox
 import voluptuous as vol
 
-from homeassistant.components import camera, cloud, notify as hass_notify, tag
-from homeassistant.components.binary_sensor import (
-    DEVICE_CLASSES as BINARY_SENSOR_CLASSES,
+from homeassistant.components import (
+    camera,
+    cloud,
+    conversation,
+    notify as hass_notify,
+    tag,
 )
+from homeassistant.components.binary_sensor import BinarySensorDeviceClass
 from homeassistant.components.camera import CameraEntityFeature
 from homeassistant.components.device_tracker import (
     ATTR_BATTERY,
@@ -23,21 +31,22 @@ from homeassistant.components.device_tracker import (
     ATTR_LOCATION_NAME,
 )
 from homeassistant.components.frontend import MANIFEST_JSON
-from homeassistant.components.sensor import (
-    DEVICE_CLASSES as SENSOR_CLASSES,
-    STATE_CLASSES as SENSOSR_STATE_CLASSES,
-)
-from homeassistant.components.zone.const import DOMAIN as ZONE_DOMAIN
+from homeassistant.components.sensor import SensorDeviceClass, SensorStateClass
+from homeassistant.components.zone import DOMAIN as ZONE_DOMAIN
+from homeassistant.config_entries import ConfigEntry
 from homeassistant.const import (
     ATTR_DEVICE_ID,
     ATTR_DOMAIN,
     ATTR_SERVICE,
     ATTR_SERVICE_DATA,
     ATTR_SUPPORTED_FEATURES,
+    CONF_NAME,
+    CONF_UNIQUE_ID,
     CONF_WEBHOOK_ID,
+    EntityCategory,
 )
 from homeassistant.core import EventOrigin, HomeAssistant
-from homeassistant.exceptions import HomeAssistantError, ServiceNotFound
+from homeassistant.exceptions import HomeAssistantError, ServiceNotFound, TemplateError
 from homeassistant.helpers import (
     config_validation as cv,
     device_registry as dr,
@@ -45,7 +54,6 @@ from homeassistant.helpers import (
     template,
 )
 from homeassistant.helpers.dispatcher import async_dispatcher_send
-from homeassistant.helpers.entity import ENTITY_CATEGORIES_SCHEMA
 from homeassistant.util.decorator import Registry
 
 from .const import (
@@ -63,6 +71,7 @@ from .const import (
     ATTR_OS_VERSION,
     ATTR_SENSOR_ATTRIBUTES,
     ATTR_SENSOR_DEVICE_CLASS,
+    ATTR_SENSOR_DISABLED,
     ATTR_SENSOR_ENTITY_CATEGORY,
     ATTR_SENSOR_ICON,
     ATTR_SENSOR_NAME,
@@ -87,6 +96,7 @@ from .const import (
     CONF_SECRET,
     DATA_CONFIG_ENTRIES,
     DATA_DELETED_IDS,
+    DATA_DEVICES,
     DOMAIN,
     ERR_ENCRYPTION_ALREADY_ENABLED,
     ERR_ENCRYPTION_NOT_AVAILABLE,
@@ -98,8 +108,8 @@ from .const import (
     SIGNAL_SENSOR_UPDATE,
 )
 from .helpers import (
-    _decrypt_payload,
-    _decrypt_payload_legacy,
+    decrypt_payload,
+    decrypt_payload_legacy,
     empty_okay_response,
     error_response,
     registration_context,
@@ -112,17 +122,35 @@ _LOGGER = logging.getLogger(__name__)
 
 DELAY_SAVE = 10
 
-WEBHOOK_COMMANDS = Registry()  # type: ignore[var-annotated]
+WEBHOOK_COMMANDS: Registry[
+    str, Callable[[HomeAssistant, ConfigEntry, Any], Coroutine[Any, Any, Response]]
+] = Registry()
 
-COMBINED_CLASSES = set(BINARY_SENSOR_CLASSES + SENSOR_CLASSES)
-SENSOR_TYPES = [ATTR_SENSOR_TYPE_BINARY_SENSOR, ATTR_SENSOR_TYPE_SENSOR]
+SENSOR_TYPES = (ATTR_SENSOR_TYPE_BINARY_SENSOR, ATTR_SENSOR_TYPE_SENSOR)
 
-WEBHOOK_PAYLOAD_SCHEMA = vol.Schema(
+WEBHOOK_PAYLOAD_SCHEMA = vol.Any(
+    vol.Schema(
+        {
+            vol.Required(ATTR_WEBHOOK_TYPE): cv.string,
+            vol.Optional(ATTR_WEBHOOK_DATA): vol.Any(dict, list),
+        }
+    ),
+    vol.Schema(
+        {
+            vol.Required(ATTR_WEBHOOK_TYPE): cv.string,
+            vol.Required(ATTR_WEBHOOK_ENCRYPTED): True,
+            vol.Optional(ATTR_WEBHOOK_ENCRYPTED_DATA): cv.string,
+        }
+    ),
+)
+
+SENSOR_SCHEMA_FULL = vol.Schema(
     {
-        vol.Required(ATTR_WEBHOOK_TYPE): cv.string,
-        vol.Required(ATTR_WEBHOOK_DATA, default={}): vol.Any(dict, list),
-        vol.Optional(ATTR_WEBHOOK_ENCRYPTED, default=False): cv.boolean,
-        vol.Optional(ATTR_WEBHOOK_ENCRYPTED_DATA): cv.string,
+        vol.Optional(ATTR_SENSOR_ATTRIBUTES, default={}): dict,
+        vol.Optional(ATTR_SENSOR_ICON, default="mdi:cellphone"): vol.Any(None, cv.icon),
+        vol.Required(ATTR_SENSOR_STATE): vol.Any(None, bool, int, float, str),
+        vol.Required(ATTR_SENSOR_TYPE): vol.In(SENSOR_TYPES),
+        vol.Required(ATTR_SENSOR_UNIQUE_ID): cv.string,
     }
 )
 
@@ -159,9 +187,9 @@ async def handle_webhook(
     if webhook_id in hass.data[DOMAIN][DATA_DELETED_IDS]:
         return Response(status=410)
 
-    config_entry = hass.data[DOMAIN][DATA_CONFIG_ENTRIES][webhook_id]
+    config_entry: ConfigEntry = hass.data[DOMAIN][DATA_CONFIG_ENTRIES][webhook_id]
 
-    device_name = config_entry.data[ATTR_DEVICE_NAME]
+    device_name: str = config_entry.data[ATTR_DEVICE_NAME]
 
     try:
         req_data = await request.json()
@@ -190,19 +218,19 @@ async def handle_webhook(
 
     webhook_type = req_data[ATTR_WEBHOOK_TYPE]
 
-    webhook_payload = req_data.get(ATTR_WEBHOOK_DATA, {})
+    webhook_payload = None
 
-    if req_data[ATTR_WEBHOOK_ENCRYPTED]:
+    if ATTR_WEBHOOK_ENCRYPTED in req_data:
         enc_data = req_data[ATTR_WEBHOOK_ENCRYPTED_DATA]
         try:
-            webhook_payload = _decrypt_payload(config_entry.data[CONF_SECRET], enc_data)
+            webhook_payload = decrypt_payload(config_entry.data[CONF_SECRET], enc_data)
             if ATTR_NO_LEGACY_ENCRYPTION not in config_entry.data:
                 data = {**config_entry.data, ATTR_NO_LEGACY_ENCRYPTION: True}
                 hass.config_entries.async_update_entry(config_entry, data=data)
         except CryptoError:
             if ATTR_NO_LEGACY_ENCRYPTION not in config_entry.data:
                 try:
-                    webhook_payload = _decrypt_payload_legacy(
+                    webhook_payload = decrypt_payload_legacy(
                         config_entry.data[CONF_SECRET], enc_data
                     )
                 except CryptoError:
@@ -210,11 +238,16 @@ async def handle_webhook(
                         "Ignoring encrypted payload because unable to decrypt"
                     )
                 except ValueError:
-                    _LOGGER.warning("Ignoring invalid encrypted payload")
+                    _LOGGER.warning("Ignoring invalid JSON in encrypted payload")
             else:
                 _LOGGER.warning("Ignoring encrypted payload because unable to decrypt")
-        except ValueError:
-            _LOGGER.warning("Ignoring invalid encrypted payload")
+        except ValueError as err:
+            _LOGGER.warning("Ignoring invalid JSON in encrypted payload: %s", err)
+    else:
+        webhook_payload = req_data.get(ATTR_WEBHOOK_DATA, {})
+
+    if webhook_payload is None:
+        return empty_okay_response()
 
     if webhook_type not in WEBHOOK_COMMANDS:
         _LOGGER.error(
@@ -243,7 +276,9 @@ async def handle_webhook(
         vol.Optional(ATTR_SERVICE_DATA, default={}): dict,
     }
 )
-async def webhook_call_service(hass, config_entry, data):
+async def webhook_call_service(
+    hass: HomeAssistant, config_entry: ConfigEntry, data: dict[str, Any]
+) -> Response:
     """Handle a call service webhook."""
     try:
         await hass.services.async_call(
@@ -255,8 +290,10 @@ async def webhook_call_service(hass, config_entry, data):
         )
     except (vol.Invalid, ServiceNotFound, Exception) as ex:
         _LOGGER.error(
-            "Error when calling service during mobile_app "
-            "webhook (device name: %s): %s",
+            (
+                "Error when calling service during mobile_app "
+                "webhook (device name: %s): %s"
+            ),
             config_entry.data[ATTR_DEVICE_NAME],
             ex,
         )
@@ -272,9 +309,11 @@ async def webhook_call_service(hass, config_entry, data):
         vol.Optional(ATTR_EVENT_DATA, default={}): dict,
     }
 )
-async def webhook_fire_event(hass, config_entry, data):
+async def webhook_fire_event(
+    hass: HomeAssistant, config_entry: ConfigEntry, data: dict[str, Any]
+) -> Response:
     """Handle a fire event webhook."""
-    event_type = data[ATTR_EVENT_TYPE]
+    event_type: str = data[ATTR_EVENT_TYPE]
     hass.bus.async_fire(
         event_type,
         data[ATTR_EVENT_DATA],
@@ -284,9 +323,33 @@ async def webhook_fire_event(hass, config_entry, data):
     return empty_okay_response()
 
 
+@WEBHOOK_COMMANDS.register("conversation_process")
+@validate_schema(
+    {
+        vol.Required("text"): cv.string,
+        vol.Optional("language"): cv.string,
+        vol.Optional("conversation_id"): cv.string,
+    }
+)
+async def webhook_conversation_process(
+    hass: HomeAssistant, config_entry: ConfigEntry, data: dict[str, Any]
+) -> Response:
+    """Handle a conversation process webhook."""
+    result = await conversation.async_converse(
+        hass,
+        text=data["text"],
+        language=data.get("language"),
+        conversation_id=data.get("conversation_id"),
+        context=registration_context(config_entry.data),
+    )
+    return webhook_response(result.as_dict(), registration=config_entry.data)
+
+
 @WEBHOOK_COMMANDS.register("stream_camera")
 @validate_schema({vol.Required(ATTR_CAMERA_ENTITY_ID): cv.string})
-async def webhook_stream_camera(hass, config_entry, data):
+async def webhook_stream_camera(
+    hass: HomeAssistant, config_entry: ConfigEntry, data: dict[str, str]
+) -> Response:
     """Handle a request to HLS-stream a camera."""
     if (camera_state := hass.states.get(data[ATTR_CAMERA_ENTITY_ID])) is None:
         return webhook_response(
@@ -295,7 +358,9 @@ async def webhook_stream_camera(hass, config_entry, data):
             status=HTTPStatus.BAD_REQUEST,
         )
 
-    resp = {"mjpeg_path": f"/api/camera_proxy_stream/{camera_state.entity_id}"}
+    resp: dict[str, Any] = {
+        "mjpeg_path": f"/api/camera_proxy_stream/{camera_state.entity_id}"
+    }
 
     if camera_state.attributes[ATTR_SUPPORTED_FEATURES] & CameraEntityFeature.STREAM:
         try:
@@ -310,6 +375,12 @@ async def webhook_stream_camera(hass, config_entry, data):
     return webhook_response(resp, registration=config_entry.data)
 
 
+@lru_cache
+def _cached_template(template_str: str, hass: HomeAssistant) -> template.Template:
+    """Return a cached template."""
+    return template.Template(template_str, hass)
+
+
 @WEBHOOK_COMMANDS.register("render_template")
 @validate_schema(
     {
@@ -319,14 +390,16 @@ async def webhook_stream_camera(hass, config_entry, data):
         }
     }
 )
-async def webhook_render_template(hass, config_entry, data):
+async def webhook_render_template(
+    hass: HomeAssistant, config_entry: ConfigEntry, data: dict[str, Any]
+) -> Response:
     """Handle a render template webhook."""
     resp = {}
     for key, item in data.items():
         try:
-            tpl = template.Template(item[ATTR_TEMPLATE], hass)
+            tpl = _cached_template(item[ATTR_TEMPLATE], hass)
             resp[key] = tpl.async_render(item.get(ATTR_TEMPLATE_VARIABLES))
-        except template.TemplateError as ex:
+        except TemplateError as ex:
             resp[key] = {"error": str(ex)}
 
     return webhook_response(resp, registration=config_entry.data)
@@ -348,10 +421,12 @@ async def webhook_render_template(hass, config_entry, data):
         },
     )
 )
-async def webhook_update_location(hass, config_entry, data):
+async def webhook_update_location(
+    hass: HomeAssistant, config_entry: ConfigEntry, data: dict[str, Any]
+) -> Response:
     """Handle an update location webhook."""
-    hass.helpers.dispatcher.async_dispatcher_send(
-        SIGNAL_LOCATION_UPDATE.format(config_entry.entry_id), data
+    async_dispatcher_send(
+        hass, SIGNAL_LOCATION_UPDATE.format(config_entry.entry_id), data
     )
     return empty_okay_response()
 
@@ -367,7 +442,9 @@ async def webhook_update_location(hass, config_entry, data):
         vol.Optional(ATTR_OS_VERSION): cv.string,
     }
 )
-async def webhook_update_registration(hass, config_entry, data):
+async def webhook_update_registration(
+    hass: HomeAssistant, config_entry: ConfigEntry, data: dict[str, Any]
+) -> Response:
     """Handle an update registration webhook."""
     new_registration = {**config_entry.data, **data}
 
@@ -393,7 +470,9 @@ async def webhook_update_registration(hass, config_entry, data):
 
 
 @WEBHOOK_COMMANDS.register("enable_encryption")
-async def webhook_enable_encryption(hass, config_entry, data):
+async def webhook_enable_encryption(
+    hass: HomeAssistant, config_entry: ConfigEntry, data: Any
+) -> Response:
     """Handle a encryption enable webhook."""
     if config_entry.data[ATTR_SUPPORTS_ENCRYPTION]:
         _LOGGER.warning(
@@ -413,14 +492,18 @@ async def webhook_enable_encryption(hass, config_entry, data):
 
     secret = secrets.token_hex(SecretBox.KEY_SIZE)
 
-    data = {**config_entry.data, ATTR_SUPPORTS_ENCRYPTION: True, CONF_SECRET: secret}
+    update_data = {
+        **config_entry.data,
+        ATTR_SUPPORTS_ENCRYPTION: True,
+        CONF_SECRET: secret,
+    }
 
-    hass.config_entries.async_update_entry(config_entry, data=data)
+    hass.config_entries.async_update_entry(config_entry, data=update_data)
 
     return json_response({"secret": secret})
 
 
-def _validate_state_class_sensor(value: dict):
+def _validate_state_class_sensor(value: dict[str, Any]) -> dict[str, Any]:
     """Validate we only set state class for sensors."""
     if (
         ATTR_SENSOR_STATE_CLASS in value
@@ -431,35 +514,56 @@ def _validate_state_class_sensor(value: dict):
     return value
 
 
+def _gen_unique_id(webhook_id: str, sensor_unique_id: str) -> str:
+    """Return a unique sensor ID."""
+    return f"{webhook_id}_{sensor_unique_id}"
+
+
+def _extract_sensor_unique_id(webhook_id: str, unique_id: str) -> str:
+    """Return a unique sensor ID."""
+    return unique_id[len(webhook_id) + 1 :]
+
+
 @WEBHOOK_COMMANDS.register("register_sensor")
 @validate_schema(
     vol.All(
         {
             vol.Optional(ATTR_SENSOR_ATTRIBUTES, default={}): dict,
-            vol.Optional(ATTR_SENSOR_DEVICE_CLASS): vol.All(
-                vol.Lower, vol.In(COMBINED_CLASSES)
+            vol.Optional(ATTR_SENSOR_DEVICE_CLASS): vol.Any(
+                None,
+                vol.All(vol.Lower, vol.Coerce(BinarySensorDeviceClass)),
+                vol.All(vol.Lower, vol.Coerce(SensorDeviceClass)),
             ),
             vol.Required(ATTR_SENSOR_NAME): cv.string,
             vol.Required(ATTR_SENSOR_TYPE): vol.In(SENSOR_TYPES),
             vol.Required(ATTR_SENSOR_UNIQUE_ID): cv.string,
-            vol.Optional(ATTR_SENSOR_UOM): cv.string,
+            vol.Optional(ATTR_SENSOR_UOM): vol.Any(None, cv.string),
             vol.Optional(ATTR_SENSOR_STATE, default=None): vol.Any(
-                None, bool, str, int, float
+                None, bool, int, float, str
             ),
-            vol.Optional(ATTR_SENSOR_ENTITY_CATEGORY): ENTITY_CATEGORIES_SCHEMA,
-            vol.Optional(ATTR_SENSOR_ICON, default="mdi:cellphone"): cv.icon,
-            vol.Optional(ATTR_SENSOR_STATE_CLASS): vol.In(SENSOSR_STATE_CLASSES),
+            vol.Optional(ATTR_SENSOR_ENTITY_CATEGORY): vol.Any(
+                None, vol.Coerce(EntityCategory)
+            ),
+            vol.Optional(ATTR_SENSOR_ICON, default="mdi:cellphone"): vol.Any(
+                None, cv.icon
+            ),
+            vol.Optional(ATTR_SENSOR_STATE_CLASS): vol.Any(
+                None, vol.Coerce(SensorStateClass)
+            ),
+            vol.Optional(ATTR_SENSOR_DISABLED): bool,
         },
         _validate_state_class_sensor,
     )
 )
-async def webhook_register_sensor(hass, config_entry, data):
+async def webhook_register_sensor(
+    hass: HomeAssistant, config_entry: ConfigEntry, data: dict[str, Any]
+) -> Response:
     """Handle a register sensor webhook."""
-    entity_type = data[ATTR_SENSOR_TYPE]
-    unique_id = data[ATTR_SENSOR_UNIQUE_ID]
-    device_name = config_entry.data[ATTR_DEVICE_NAME]
+    entity_type: str = data[ATTR_SENSOR_TYPE]
+    unique_id: str = data[ATTR_SENSOR_UNIQUE_ID]
+    device_name: str = config_entry.data[ATTR_DEVICE_NAME]
 
-    unique_store_key = f"{config_entry.data[CONF_WEBHOOK_ID]}_{unique_id}"
+    unique_store_key = _gen_unique_id(config_entry.data[CONF_WEBHOOK_ID], unique_id)
     entity_registry = er.async_get(hass)
     existing_sensor = entity_registry.async_get_entity_id(
         entity_type, DOMAIN, unique_store_key
@@ -474,13 +578,23 @@ async def webhook_register_sensor(hass, config_entry, data):
         )
 
         entry = entity_registry.async_get(existing_sensor)
-        changes = {}
+        assert entry is not None
+        changes: dict[str, Any] = {}
 
         if (
             new_name := f"{device_name} {data[ATTR_SENSOR_NAME]}"
         ) != entry.original_name:
             changes["original_name"] = new_name
 
+        if (
+            should_be_disabled := data.get(ATTR_SENSOR_DISABLED)
+        ) is None or should_be_disabled == entry.disabled:
+            pass
+        elif should_be_disabled:
+            changes["disabled_by"] = er.RegistryEntryDisabler.INTEGRATION
+        else:
+            changes["disabled_by"] = None
+
         for ent_reg_key, data_key in (
             ("device_class", ATTR_SENSOR_DEVICE_CLASS),
             ("unit_of_measurement", ATTR_SENSOR_UOM),
@@ -493,8 +607,13 @@ async def webhook_register_sensor(hass, config_entry, data):
         if changes:
             entity_registry.async_update_entity(existing_sensor, **changes)
 
-        async_dispatcher_send(hass, SIGNAL_SENSOR_UPDATE, data)
+        async_dispatcher_send(hass, f"{SIGNAL_SENSOR_UPDATE}-{unique_store_key}", data)
     else:
+        data[CONF_UNIQUE_ID] = unique_store_key
+        data[
+            CONF_NAME
+        ] = f"{config_entry.data[ATTR_DEVICE_NAME]} {data[ATTR_SENSOR_NAME]}"
+
         register_signal = f"{DOMAIN}_{data[ATTR_SENSOR_TYPE]}_register"
         async_dispatcher_send(hass, register_signal, data)
 
@@ -523,33 +642,27 @@ async def webhook_register_sensor(hass, config_entry, data):
         ],
     )
 )
-async def webhook_update_sensor_states(hass, config_entry, data):
+async def webhook_update_sensor_states(
+    hass: HomeAssistant, config_entry: ConfigEntry, data: list[dict[str, Any]]
+) -> Response:
     """Handle an update sensor states webhook."""
-    sensor_schema_full = vol.Schema(
-        {
-            vol.Optional(ATTR_SENSOR_ATTRIBUTES, default={}): dict,
-            vol.Optional(ATTR_SENSOR_ICON, default="mdi:cellphone"): cv.icon,
-            vol.Required(ATTR_SENSOR_STATE): vol.Any(None, bool, str, int, float),
-            vol.Required(ATTR_SENSOR_TYPE): vol.In(SENSOR_TYPES),
-            vol.Required(ATTR_SENSOR_UNIQUE_ID): cv.string,
-        }
-    )
-
-    device_name = config_entry.data[ATTR_DEVICE_NAME]
-    resp = {}
+    device_name: str = config_entry.data[ATTR_DEVICE_NAME]
+    resp: dict[str, Any] = {}
+    entity_registry = er.async_get(hass)
 
     for sensor in data:
-        entity_type = sensor[ATTR_SENSOR_TYPE]
+        entity_type: str = sensor[ATTR_SENSOR_TYPE]
 
-        unique_id = sensor[ATTR_SENSOR_UNIQUE_ID]
+        unique_id: str = sensor[ATTR_SENSOR_UNIQUE_ID]
 
-        unique_store_key = f"{config_entry.data[CONF_WEBHOOK_ID]}_{unique_id}"
+        unique_store_key = _gen_unique_id(config_entry.data[CONF_WEBHOOK_ID], unique_id)
 
-        entity_registry = er.async_get(hass)
-        if not entity_registry.async_get_entity_id(
-            entity_type, DOMAIN, unique_store_key
+        if not (
+            entity_id := entity_registry.async_get_entity_id(
+                entity_type, DOMAIN, unique_store_key
+            )
         ):
-            _LOGGER.error(
+            _LOGGER.debug(
                 "Refusing to update %s non-registered sensor: %s",
                 device_name,
                 unique_store_key,
@@ -562,7 +675,7 @@ async def webhook_update_sensor_states(hass, config_entry, data):
             continue
 
         try:
-            sensor = sensor_schema_full(sensor)
+            sensor = SENSOR_SCHEMA_FULL(sensor)
         except vol.Invalid as err:
             err_msg = vol.humanize.humanize_error(sensor, err)
             _LOGGER.error(
@@ -578,15 +691,27 @@ async def webhook_update_sensor_states(hass, config_entry, data):
             continue
 
         sensor[CONF_WEBHOOK_ID] = config_entry.data[CONF_WEBHOOK_ID]
-        async_dispatcher_send(hass, SIGNAL_SENSOR_UPDATE, sensor)
+        async_dispatcher_send(
+            hass,
+            f"{SIGNAL_SENSOR_UPDATE}-{unique_store_key}",
+            sensor,
+        )
 
         resp[unique_id] = {"success": True}
 
+        # Check if disabled
+        entry = entity_registry.async_get(entity_id)
+
+        if entry and entry.disabled_by:
+            resp[unique_id]["is_disabled"] = True
+
     return webhook_response(resp, registration=config_entry.data)
 
 
 @WEBHOOK_COMMANDS.register("get_zones")
-async def webhook_get_zones(hass, config_entry, data):
+async def webhook_get_zones(
+    hass: HomeAssistant, config_entry: ConfigEntry, data: Any
+) -> Response:
     """Handle a get zones webhook."""
     zones = [
         hass.states.get(entity_id)
@@ -596,7 +721,9 @@ async def webhook_get_zones(hass, config_entry, data):
 
 
 @WEBHOOK_COMMANDS.register("get_config")
-async def webhook_get_config(hass, config_entry, data):
+async def webhook_get_config(
+    hass: HomeAssistant, config_entry: ConfigEntry, data: Any
+) -> Response:
     """Handle a get config webhook."""
     hass_config = hass.config.as_dict()
 
@@ -615,20 +742,38 @@ async def webhook_get_config(hass, config_entry, data):
     if CONF_CLOUDHOOK_URL in config_entry.data:
         resp[CONF_CLOUDHOOK_URL] = config_entry.data[CONF_CLOUDHOOK_URL]
 
-    with suppress(hass.components.cloud.CloudNotAvailable):
-        resp[CONF_REMOTE_UI_URL] = cloud.async_remote_ui_url(hass)
+    if cloud.async_active_subscription(hass):
+        with suppress(hass.components.cloud.CloudNotAvailable):
+            resp[CONF_REMOTE_UI_URL] = cloud.async_remote_ui_url(hass)
+
+    webhook_id = config_entry.data[CONF_WEBHOOK_ID]
+
+    entities = {}
+    for entry in er.async_entries_for_config_entry(
+        er.async_get(hass), config_entry.entry_id
+    ):
+        if entry.domain in ("binary_sensor", "sensor"):
+            unique_id = _extract_sensor_unique_id(webhook_id, entry.unique_id)
+        else:
+            unique_id = entry.unique_id
+
+        entities[unique_id] = {"disabled": entry.disabled}
+
+    resp["entities"] = entities
 
     return webhook_response(resp, registration=config_entry.data)
 
 
 @WEBHOOK_COMMANDS.register("scan_tag")
 @validate_schema({vol.Required("tag_id"): cv.string})
-async def webhook_scan_tag(hass, config_entry, data):
+async def webhook_scan_tag(
+    hass: HomeAssistant, config_entry: ConfigEntry, data: dict[str, str]
+) -> Response:
     """Handle a fire event webhook."""
     await tag.async_scan_tag(
         hass,
         data["tag_id"],
-        config_entry.data[ATTR_DEVICE_ID],
+        hass.data[DOMAIN][DATA_DEVICES][config_entry.data[CONF_WEBHOOK_ID]].id,
         registration_context(config_entry.data),
     )
     return empty_okay_response()
