@@ -8,7 +8,7 @@ import aiohttp
 from moehlenhoff_alpha2 import Alpha2Base
 
 from homeassistant.config_entries import ConfigEntry
-from homeassistant.const import Platform
+from homeassistant.const import CONF_HOST, Platform
 from homeassistant.core import HomeAssistant
 from homeassistant.exceptions import HomeAssistantError
 from homeassistant.helpers.update_coordinator import DataUpdateCoordinator
@@ -17,14 +17,14 @@ from .const import DOMAIN
 
 _LOGGER = logging.getLogger(__name__)
 
-PLATFORMS = [Platform.CLIMATE, Platform.SENSOR, Platform.BINARY_SENSOR]
+PLATFORMS = [Platform.BINARY_SENSOR, Platform.BUTTON, Platform.CLIMATE, Platform.SENSOR]
 
 UPDATE_INTERVAL = timedelta(seconds=60)
 
 
 async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
     """Set up a config entry."""
-    base = Alpha2Base(entry.data["host"])
+    base = Alpha2Base(entry.data[CONF_HOST])
     coordinator = Alpha2BaseCoordinator(hass, base)
 
     await coordinator.async_config_entry_first_refresh()
@@ -52,7 +52,7 @@ async def update_listener(hass: HomeAssistant, entry: ConfigEntry) -> None:
     await hass.config_entries.async_reload(entry.entry_id)
 
 
-class Alpha2BaseCoordinator(DataUpdateCoordinator[dict[str, dict]]):
+class Alpha2BaseCoordinator(DataUpdateCoordinator[dict[str, dict]]):  # pylint: disable=hass-enforce-coordinator-module
     """Keep the base instance in one place and centralize the update."""
 
     def __init__(self, hass: HomeAssistant, base: Alpha2Base) -> None:
@@ -124,7 +124,7 @@ class Alpha2BaseCoordinator(DataUpdateCoordinator[dict[str, dict]]):
         """Set the mode of the given heat area."""
         # HEATAREA_MODE: 0=Auto, 1=Tag, 2=Nacht
         if heat_area_mode not in (0, 1, 2):
-            ValueError(f"Invalid heat area mode: {heat_area_mode}")
+            raise ValueError(f"Invalid heat area mode: {heat_area_mode}")
         _LOGGER.debug(
             "Setting mode of heat area %s to %d",
             heat_area_id,
