@@ -1,6 +1,7 @@
 """The motionEye integration."""
 from __future__ import annotations
 
+from contextlib import suppress
 from types import MappingProxyType
 from typing import Any
 
@@ -11,7 +12,6 @@ from motioneye_client.const import (
     DEFAULT_SURVEILLANCE_USERNAME,
     KEY_ACTION_SNAPSHOT,
     KEY_MOTION_DETECTION,
-    KEY_NAME,
     KEY_STREAMING_AUTH_MODE,
     KEY_TEXT_OVERLAY_CAMERA_NAME,
     KEY_TEXT_OVERLAY_CUSTOM_TEXT,
@@ -24,10 +24,9 @@ from motioneye_client.const import (
 )
 import voluptuous as vol
 
-from homeassistant.components.mjpeg.camera import (
+from homeassistant.components.mjpeg import (
     CONF_MJPEG_URL,
     CONF_STILL_IMAGE_URL,
-    CONF_VERIFY_SSL,
     MjpegCamera,
 )
 from homeassistant.config_entries import ConfigEntry
@@ -38,6 +37,7 @@ from homeassistant.const import (
     CONF_USERNAME,
     HTTP_BASIC_AUTHENTICATION,
     HTTP_DIGEST_AUTHENTICATION,
+    Platform,
 )
 from homeassistant.core import HomeAssistant, callback
 from homeassistant.helpers import config_validation as cv, entity_platform
@@ -65,7 +65,7 @@ from .const import (
     TYPE_MOTIONEYE_MJPEG_CAMERA,
 )
 
-PLATFORMS = ["camera"]
+PLATFORMS = [Platform.CAMERA]
 
 SCHEMA_TEXT_OVERLAY = vol.In(
     [
@@ -143,6 +143,10 @@ async def async_setup_entry(
 class MotionEyeMjpegCamera(MotionEyeEntity, MjpegCamera):
     """motionEye mjpeg camera."""
 
+    _attr_brand = MOTIONEYE_MANUFACTURER
+    # motionEye cameras are always streaming or unavailable.
+    _attr_is_streaming = True
+
     def __init__(
         self,
         config_entry_id: str,
@@ -158,9 +162,6 @@ class MotionEyeMjpegCamera(MotionEyeEntity, MjpegCamera):
         self._surveillance_password = password
         self._motion_detection_enabled: bool = camera.get(KEY_MOTION_DETECTION, False)
 
-        # motionEye cameras are always streaming or unavailable.
-        self._attr_is_streaming = True
-
         MotionEyeEntity.__init__(
             self,
             config_entry_id,
@@ -172,10 +173,8 @@ class MotionEyeMjpegCamera(MotionEyeEntity, MjpegCamera):
         )
         MjpegCamera.__init__(
             self,
-            {
-                CONF_VERIFY_SSL: False,
-                **self._get_mjpeg_camera_properties_for_camera(camera),
-            },
+            verify_ssl=False,
+            **self._get_mjpeg_camera_properties_for_camera(camera),
         )
 
     @callback
@@ -198,15 +197,13 @@ class MotionEyeMjpegCamera(MotionEyeEntity, MjpegCamera):
             # which is not available during entity construction.
             streaming_url = Template(streaming_template).render(**camera)
         else:
-            try:
+            with suppress(MotionEyeClientURLParseError):
                 streaming_url = self._client.get_camera_stream_url(camera)
-            except MotionEyeClientURLParseError:
-                pass
 
         return {
-            CONF_NAME: camera[KEY_NAME],
+            CONF_NAME: None,
             CONF_USERNAME: self._surveillance_username if auth is not None else None,
-            CONF_PASSWORD: self._surveillance_password if auth is not None else None,
+            CONF_PASSWORD: self._surveillance_password if auth is not None else "",
             CONF_MJPEG_URL: streaming_url or "",
             CONF_STILL_IMAGE_URL: self._client.get_camera_snapshot_url(camera),
             CONF_AUTHENTICATION: auth,
@@ -219,14 +216,16 @@ class MotionEyeMjpegCamera(MotionEyeEntity, MjpegCamera):
         # Sets the state of the underlying (inherited) MjpegCamera based on the updated
         # MotionEye camera dictionary.
         properties = self._get_mjpeg_camera_properties_for_camera(camera)
-        self._name = properties[CONF_NAME]
         self._username = properties[CONF_USERNAME]
         self._password = properties[CONF_PASSWORD]
         self._mjpeg_url = properties[CONF_MJPEG_URL]
         self._still_image_url = properties[CONF_STILL_IMAGE_URL]
         self._authentication = properties[CONF_AUTHENTICATION]
 
-        if self._authentication == HTTP_BASIC_AUTHENTICATION:
+        if (
+            self._authentication == HTTP_BASIC_AUTHENTICATION
+            and self._username is not None
+        ):
             self._auth = aiohttp.BasicAuth(self._username, password=self._password)
 
     def _is_acceptable_streaming_camera(self) -> bool:
@@ -251,11 +250,6 @@ class MotionEyeMjpegCamera(MotionEyeEntity, MjpegCamera):
             )
         super()._handle_coordinator_update()
 
-    @property
-    def brand(self) -> str:
-        """Return the camera brand."""
-        return MOTIONEYE_MANUFACTURER
-
     @property
     def motion_detection_enabled(self) -> bool:
         """Return the camera motion detection status."""
@@ -263,10 +257,10 @@ class MotionEyeMjpegCamera(MotionEyeEntity, MjpegCamera):
 
     async def async_set_text_overlay(
         self,
-        left_text: str = None,
-        right_text: str = None,
-        custom_left_text: str = None,
-        custom_right_text: str = None,
+        left_text: str | None = None,
+        right_text: str | None = None,
+        custom_left_text: str | None = None,
+        custom_right_text: str | None = None,
     ) -> None:
         """Set text overlay for a camera."""
         # Fetch the very latest camera config to reduce the risk of updating with a
