@@ -1,9 +1,7 @@
-"""This platform enables the possibility to control a MQTT alarm."""
+"""Control a MQTT alarm."""
 from __future__ import annotations
 
-import functools
 import logging
-import re
 
 import voluptuous as vol
 
@@ -28,7 +26,7 @@ from homeassistant.const import (
 from homeassistant.core import HomeAssistant, callback
 import homeassistant.helpers.config_validation as cv
 from homeassistant.helpers.entity_platform import AddEntitiesCallback
-from homeassistant.helpers.typing import ConfigType, DiscoveryInfoType
+from homeassistant.helpers.typing import ConfigType
 
 from . import subscription
 from .config import DEFAULT_RETAIN, MQTT_BASE_SCHEMA
@@ -39,21 +37,29 @@ from .const import (
     CONF_QOS,
     CONF_RETAIN,
     CONF_STATE_TOPIC,
+    CONF_SUPPORTED_FEATURES,
 )
 from .debug_info import log_messages
 from .mixins import (
     MQTT_ENTITY_COMMON_SCHEMA,
     MqttEntity,
-    async_setup_entry_helper,
-    async_setup_platform_discovery,
-    async_setup_platform_helper,
-    warn_for_legacy_schema,
+    async_setup_entity_entry_helper,
+    write_state_on_attr_change,
 )
-from .models import MqttCommandTemplate, MqttValueTemplate
+from .models import MqttCommandTemplate, MqttValueTemplate, ReceiveMessage
 from .util import valid_publish_topic, valid_subscribe_topic
 
 _LOGGER = logging.getLogger(__name__)
 
+_SUPPORTED_FEATURES = {
+    "arm_home": AlarmControlPanelEntityFeature.ARM_HOME,
+    "arm_away": AlarmControlPanelEntityFeature.ARM_AWAY,
+    "arm_night": AlarmControlPanelEntityFeature.ARM_NIGHT,
+    "arm_vacation": AlarmControlPanelEntityFeature.ARM_VACATION,
+    "arm_custom_bypass": AlarmControlPanelEntityFeature.ARM_CUSTOM_BYPASS,
+    "trigger": AlarmControlPanelEntityFeature.TRIGGER,
+}
+
 CONF_CODE_ARM_REQUIRED = "code_arm_required"
 CONF_CODE_DISARM_REQUIRED = "code_disarm_required"
 CONF_CODE_TRIGGER_REQUIRED = "code_trigger_required"
@@ -88,6 +94,9 @@ REMOTE_CODE_TEXT = "REMOTE_CODE_TEXT"
 
 PLATFORM_SCHEMA_MODERN = MQTT_BASE_SCHEMA.extend(
     {
+        vol.Optional(CONF_SUPPORTED_FEATURES, default=list(_SUPPORTED_FEATURES)): [
+            vol.In(_SUPPORTED_FEATURES)
+        ],
         vol.Optional(CONF_CODE): cv.string,
         vol.Optional(CONF_CODE_ARM_REQUIRED, default=True): cv.boolean,
         vol.Optional(CONF_CODE_DISARM_REQUIRED, default=True): cv.boolean,
@@ -96,7 +105,7 @@ PLATFORM_SCHEMA_MODERN = MQTT_BASE_SCHEMA.extend(
             CONF_COMMAND_TEMPLATE, default=DEFAULT_COMMAND_TEMPLATE
         ): cv.template,
         vol.Required(CONF_COMMAND_TOPIC): valid_publish_topic,
-        vol.Optional(CONF_NAME, default=DEFAULT_NAME): cv.string,
+        vol.Optional(CONF_NAME): vol.Any(cv.string, None),
         vol.Optional(CONF_PAYLOAD_ARM_AWAY, default=DEFAULT_ARM_AWAY): cv.string,
         vol.Optional(CONF_PAYLOAD_ARM_HOME, default=DEFAULT_ARM_HOME): cv.string,
         vol.Optional(CONF_PAYLOAD_ARM_NIGHT, default=DEFAULT_ARM_NIGHT): cv.string,
@@ -114,88 +123,66 @@ PLATFORM_SCHEMA_MODERN = MQTT_BASE_SCHEMA.extend(
     }
 ).extend(MQTT_ENTITY_COMMON_SCHEMA.schema)
 
-# Configuring MQTT alarm control panels under the alarm_control_panel platform key is deprecated in HA Core 2022.6
-PLATFORM_SCHEMA = vol.All(
-    cv.PLATFORM_SCHEMA.extend(PLATFORM_SCHEMA_MODERN.schema),
-    warn_for_legacy_schema(alarm.DOMAIN),
-)
-
 DISCOVERY_SCHEMA = PLATFORM_SCHEMA_MODERN.extend({}, extra=vol.REMOVE_EXTRA)
 
 
-async def async_setup_platform(
-    hass: HomeAssistant,
-    config: ConfigType,
-    async_add_entities: AddEntitiesCallback,
-    discovery_info: DiscoveryInfoType | None = None,
-) -> None:
-    """Set up MQTT alarm control panel configured under the alarm_control_panel key (deprecated)."""
-    # Deprecated in HA Core 2022.6
-    await async_setup_platform_helper(
-        hass,
-        alarm.DOMAIN,
-        discovery_info or config,
-        async_add_entities,
-        _async_setup_entity,
-    )
-
-
 async def async_setup_entry(
     hass: HomeAssistant,
     config_entry: ConfigEntry,
     async_add_entities: AddEntitiesCallback,
 ) -> None:
-    """Set up MQTT alarm control panel through configuration.yaml and dynamically through MQTT discovery."""
-    # load and initialize platform config from configuration.yaml
-    config_entry.async_on_unload(
-        await async_setup_platform_discovery(hass, alarm.DOMAIN)
+    """Set up MQTT alarm control panel through YAML and through MQTT discovery."""
+    await async_setup_entity_entry_helper(
+        hass,
+        config_entry,
+        MqttAlarm,
+        alarm.DOMAIN,
+        async_add_entities,
+        DISCOVERY_SCHEMA,
+        PLATFORM_SCHEMA_MODERN,
     )
-    # setup for discovery
-    setup = functools.partial(
-        _async_setup_entity, hass, async_add_entities, config_entry=config_entry
-    )
-    await async_setup_entry_helper(hass, alarm.DOMAIN, setup, DISCOVERY_SCHEMA)
-
-
-async def _async_setup_entity(
-    hass, async_add_entities, config, config_entry=None, discovery_data=None
-):
-    """Set up the MQTT Alarm Control Panel platform."""
-    async_add_entities([MqttAlarm(hass, config, config_entry, discovery_data)])
 
 
 class MqttAlarm(MqttEntity, alarm.AlarmControlPanelEntity):
     """Representation of a MQTT alarm status."""
 
+    _default_name = DEFAULT_NAME
     _entity_id_format = alarm.ENTITY_ID_FORMAT
     _attributes_extra_blocked = MQTT_ALARM_ATTRIBUTES_BLOCKED
 
-    def __init__(self, hass, config, config_entry, discovery_data):
-        """Init the MQTT Alarm Control Panel."""
-        self._state = None
-
-        MqttEntity.__init__(self, hass, config, config_entry, discovery_data)
-
     @staticmethod
-    def config_schema():
+    def config_schema() -> vol.Schema:
         """Return the config schema."""
         return DISCOVERY_SCHEMA
 
-    def _setup_from_config(self, config):
+    def _setup_from_config(self, config: ConfigType) -> None:
+        """(Re)Setup the entity."""
         self._value_template = MqttValueTemplate(
-            self._config.get(CONF_VALUE_TEMPLATE),
+            config.get(CONF_VALUE_TEMPLATE),
             entity=self,
         ).async_render_with_possible_json_value
         self._command_template = MqttCommandTemplate(
-            self._config[CONF_COMMAND_TEMPLATE], entity=self
+            config[CONF_COMMAND_TEMPLATE], entity=self
         ).async_render
 
-    def _prepare_subscribe_topics(self):
+        for feature in self._config[CONF_SUPPORTED_FEATURES]:
+            self._attr_supported_features |= _SUPPORTED_FEATURES[feature]
+
+        if (code := self._config.get(CONF_CODE)) is None:
+            self._attr_code_format = None
+        elif code == REMOTE_CODE or str(code).isdigit():
+            self._attr_code_format = alarm.CodeFormat.NUMBER
+        else:
+            self._attr_code_format = alarm.CodeFormat.TEXT
+        self._attr_code_arm_required = bool(self._config[CONF_CODE_ARM_REQUIRED])
+
+    def _prepare_subscribe_topics(self) -> None:
         """(Re)Subscribe to topics."""
 
         @callback
         @log_messages(self.hass, self.entity_id)
-        def message_received(msg):
+        @write_state_on_attr_change(self, {"_attr_state"})
+        def message_received(msg: ReceiveMessage) -> None:
             """Run when new MQTT message has been received."""
             payload = self._value_template(msg.payload)
             if payload not in (
@@ -212,8 +199,7 @@ class MqttAlarm(MqttEntity, alarm.AlarmControlPanelEntity):
             ):
                 _LOGGER.warning("Received unexpected payload: %s", msg.payload)
                 return
-            self._state = payload
-            self.async_write_ha_state()
+            self._attr_state = str(payload)
 
         self._sub_state = subscription.async_prepare_subscribe_topics(
             self.hass,
@@ -228,120 +214,88 @@ class MqttAlarm(MqttEntity, alarm.AlarmControlPanelEntity):
             },
         )
 
-    async def _subscribe_topics(self):
+    async def _subscribe_topics(self) -> None:
         """(Re)Subscribe to topics."""
         await subscription.async_subscribe_topics(self.hass, self._sub_state)
 
-    @property
-    def state(self):
-        """Return the state of the device."""
-        return self._state
-
-    @property
-    def supported_features(self) -> int:
-        """Return the list of supported features."""
-        return (
-            AlarmControlPanelEntityFeature.ARM_HOME
-            | AlarmControlPanelEntityFeature.ARM_AWAY
-            | AlarmControlPanelEntityFeature.ARM_NIGHT
-            | AlarmControlPanelEntityFeature.ARM_VACATION
-            | AlarmControlPanelEntityFeature.ARM_CUSTOM_BYPASS
-            | AlarmControlPanelEntityFeature.TRIGGER
-        )
-
-    @property
-    def code_format(self):
-        """Return one or more digits/characters."""
-        if (code := self._config.get(CONF_CODE)) is None:
-            return None
-        if code == REMOTE_CODE or (isinstance(code, str) and re.search("^\\d+$", code)):
-            return alarm.CodeFormat.NUMBER
-        return alarm.CodeFormat.TEXT
-
-    @property
-    def code_arm_required(self):
-        """Whether the code is required for arm actions."""
-        code_required = self._config.get(CONF_CODE_ARM_REQUIRED)
-        return code_required
-
-    async def async_alarm_disarm(self, code=None):
+    async def async_alarm_disarm(self, code: str | None = None) -> None:
         """Send disarm command.
 
         This method is a coroutine.
         """
-        code_required = self._config[CONF_CODE_DISARM_REQUIRED]
+        code_required: bool = self._config[CONF_CODE_DISARM_REQUIRED]
         if code_required and not self._validate_code(code, "disarming"):
             return
-        payload = self._config[CONF_PAYLOAD_DISARM]
+        payload: str = self._config[CONF_PAYLOAD_DISARM]
         await self._publish(code, payload)
 
-    async def async_alarm_arm_home(self, code=None):
+    async def async_alarm_arm_home(self, code: str | None = None) -> None:
         """Send arm home command.
 
         This method is a coroutine.
         """
-        code_required = self._config[CONF_CODE_ARM_REQUIRED]
+        code_required: bool = self._config[CONF_CODE_ARM_REQUIRED]
         if code_required and not self._validate_code(code, "arming home"):
             return
-        action = self._config[CONF_PAYLOAD_ARM_HOME]
+        action: str = self._config[CONF_PAYLOAD_ARM_HOME]
         await self._publish(code, action)
 
-    async def async_alarm_arm_away(self, code=None):
+    async def async_alarm_arm_away(self, code: str | None = None) -> None:
         """Send arm away command.
 
         This method is a coroutine.
         """
-        code_required = self._config[CONF_CODE_ARM_REQUIRED]
+        code_required: bool = self._config[CONF_CODE_ARM_REQUIRED]
         if code_required and not self._validate_code(code, "arming away"):
             return
-        action = self._config[CONF_PAYLOAD_ARM_AWAY]
+        action: str = self._config[CONF_PAYLOAD_ARM_AWAY]
         await self._publish(code, action)
 
-    async def async_alarm_arm_night(self, code=None):
+    async def async_alarm_arm_night(self, code: str | None = None) -> None:
         """Send arm night command.
 
         This method is a coroutine.
         """
-        code_required = self._config[CONF_CODE_ARM_REQUIRED]
+        code_required: bool = self._config[CONF_CODE_ARM_REQUIRED]
         if code_required and not self._validate_code(code, "arming night"):
             return
-        action = self._config[CONF_PAYLOAD_ARM_NIGHT]
+        action: str = self._config[CONF_PAYLOAD_ARM_NIGHT]
         await self._publish(code, action)
 
-    async def async_alarm_arm_vacation(self, code=None):
+    async def async_alarm_arm_vacation(self, code: str | None = None) -> None:
         """Send arm vacation command.
 
         This method is a coroutine.
         """
-        code_required = self._config[CONF_CODE_ARM_REQUIRED]
+        code_required: bool = self._config[CONF_CODE_ARM_REQUIRED]
         if code_required and not self._validate_code(code, "arming vacation"):
             return
-        action = self._config[CONF_PAYLOAD_ARM_VACATION]
+        action: str = self._config[CONF_PAYLOAD_ARM_VACATION]
         await self._publish(code, action)
 
-    async def async_alarm_arm_custom_bypass(self, code=None):
+    async def async_alarm_arm_custom_bypass(self, code: str | None = None) -> None:
         """Send arm custom bypass command.
 
         This method is a coroutine.
         """
-        code_required = self._config[CONF_CODE_ARM_REQUIRED]
+        code_required: bool = self._config[CONF_CODE_ARM_REQUIRED]
         if code_required and not self._validate_code(code, "arming custom bypass"):
             return
-        action = self._config[CONF_PAYLOAD_ARM_CUSTOM_BYPASS]
+        action: str = self._config[CONF_PAYLOAD_ARM_CUSTOM_BYPASS]
         await self._publish(code, action)
 
-    async def async_alarm_trigger(self, code=None):
+    async def async_alarm_trigger(self, code: str | None = None) -> None:
         """Send trigger command.
 
         This method is a coroutine.
         """
-        code_required = self._config[CONF_CODE_TRIGGER_REQUIRED]
+        code_required: bool = self._config[CONF_CODE_TRIGGER_REQUIRED]
         if code_required and not self._validate_code(code, "triggering"):
             return
-        action = self._config[CONF_PAYLOAD_TRIGGER]
+        action: str = self._config[CONF_PAYLOAD_TRIGGER]
         await self._publish(code, action)
 
-    async def _publish(self, code, action):
+    async def _publish(self, code: str | None, action: str) -> None:
         """Publish via mqtt."""
         variables = {"action": action, "code": code}
         payload = self._command_template(None, variables=variables)
@@ -353,10 +307,10 @@ class MqttAlarm(MqttEntity, alarm.AlarmControlPanelEntity):
             self._config[CONF_ENCODING],
         )
 
-    def _validate_code(self, code, state):
+    def _validate_code(self, code: str | None, state: str) -> bool:
         """Validate given code."""
-        conf_code = self._config.get(CONF_CODE)
-        check = (
+        conf_code: str | None = self._config.get(CONF_CODE)
+        check = bool(
             conf_code is None
             or code == conf_code
             or (conf_code == REMOTE_CODE and code)
