@@ -7,33 +7,38 @@ import functools
 import logging
 import re
 import time
-from typing import Any
+from typing import TYPE_CHECKING, Any
+
+import voluptuous as vol
 
 from homeassistant.config_entries import ConfigEntry
-from homeassistant.const import CONF_DEVICE, CONF_PLATFORM
-from homeassistant.core import HomeAssistant
+from homeassistant.const import CONF_DEVICE, CONF_NAME, CONF_PLATFORM
+from homeassistant.core import HomeAssistant, callback
 from homeassistant.data_entry_flow import FlowResultType
 import homeassistant.helpers.config_validation as cv
 from homeassistant.helpers.dispatcher import (
     async_dispatcher_connect,
     async_dispatcher_send,
 )
-from homeassistant.helpers.json import json_loads
 from homeassistant.helpers.service_info.mqtt import MqttServiceInfo
 from homeassistant.helpers.typing import DiscoveryInfoType
 from homeassistant.loader import async_get_mqtt
+from homeassistant.util.json import json_loads_object
 
 from .. import mqtt
-from .abbreviations import ABBREVIATIONS, DEVICE_ABBREVIATIONS
+from .abbreviations import ABBREVIATIONS, DEVICE_ABBREVIATIONS, ORIGIN_ABBREVIATIONS
 from .const import (
     ATTR_DISCOVERY_HASH,
     ATTR_DISCOVERY_PAYLOAD,
     ATTR_DISCOVERY_TOPIC,
     CONF_AVAILABILITY,
+    CONF_ORIGIN,
+    CONF_SUPPORT_URL,
+    CONF_SW_VERSION,
     CONF_TOPIC,
     DOMAIN,
 )
-from .models import ReceiveMessage
+from .models import MqttOriginInfo, ReceiveMessage
 from .util import get_mqtt_data
 
 _LOGGER = logging.getLogger(__name__)
@@ -43,7 +48,7 @@ TOPIC_MATCHER = re.compile(
     r"?(?P<object_id>[a-zA-Z0-9_-]+)/config"
 )
 
-SUPPORTED_COMPONENTS = [
+SUPPORTED_COMPONENTS = {
     "alarm_control_panel",
     "binary_sensor",
     "button",
@@ -52,8 +57,11 @@ SUPPORTED_COMPONENTS = [
     "cover",
     "device_automation",
     "device_tracker",
+    "event",
     "fan",
     "humidifier",
+    "image",
+    "lawn_mower",
     "light",
     "lock",
     "number",
@@ -66,7 +74,9 @@ SUPPORTED_COMPONENTS = [
     "text",
     "update",
     "vacuum",
-]
+    "valve",
+    "water_heater",
+}
 
 MQTT_DISCOVERY_UPDATED = "mqtt_discovery_updated_{}"
 MQTT_DISCOVERY_NEW = "mqtt_discovery_new_{}_{}"
@@ -74,6 +84,16 @@ MQTT_DISCOVERY_DONE = "mqtt_discovery_done_{}"
 
 TOPIC_BASE = "~"
 
+MQTT_ORIGIN_INFO_SCHEMA = vol.All(
+    vol.Schema(
+        {
+            vol.Required(CONF_NAME): cv.string,
+            vol.Optional(CONF_SW_VERSION): cv.string,
+            vol.Optional(CONF_SUPPORT_URL): cv.configuration_url,
+        }
+    ),
+)
+
 
 class MQTTDiscoveryPayload(dict[str, Any]):
     """Class to hold and MQTT discovery payload and discovery data."""
@@ -91,6 +111,30 @@ def set_discovery_hash(hass: HomeAssistant, discovery_hash: tuple[str, str]) ->
     get_mqtt_data(hass).discovery_already_discovered.add(discovery_hash)
 
 
+@callback
+def async_log_discovery_origin_info(
+    message: str, discovery_payload: MQTTDiscoveryPayload
+) -> None:
+    """Log information about the discovery and origin."""
+    if CONF_ORIGIN not in discovery_payload:
+        _LOGGER.info(message)
+        return
+    origin_info: MqttOriginInfo = discovery_payload[CONF_ORIGIN]
+    sw_version_log = ""
+    if sw_version := origin_info.get("sw_version"):
+        sw_version_log = f", version: {sw_version}"
+    support_url_log = ""
+    if support_url := origin_info.get("support_url"):
+        support_url_log = f", support URL: {support_url}"
+    _LOGGER.info(
+        "%s from external application %s%s%s",
+        message,
+        origin_info["name"],
+        sw_version_log,
+        support_url_log,
+    )
+
+
 async def async_start(  # noqa: C901
     hass: HomeAssistant, discovery_topic: str, config_entry: ConfigEntry
 ) -> None:
@@ -98,7 +142,8 @@ async def async_start(  # noqa: C901
     mqtt_data = get_mqtt_data(hass)
     mqtt_integrations = {}
 
-    async def async_discovery_message_received(msg: ReceiveMessage) -> None:
+    @callback
+    def async_discovery_message_received(msg: ReceiveMessage) -> None:  # noqa: C901
         """Process the received message."""
         mqtt_data.last_discovery = time.time()
         payload = msg.payload
@@ -112,7 +157,7 @@ async def async_start(  # noqa: C901
                         "Received message on illegal discovery topic '%s'. The topic"
                         " contains "
                         "not allowed characters. For more information see "
-                        "https://www.home-assistant.io/docs/mqtt/discovery/#discovery-topic"
+                        "https://www.home-assistant.io/integrations/mqtt/#discovery-topic"
                     ),
                     topic,
                 )
@@ -126,7 +171,7 @@ async def async_start(  # noqa: C901
 
         if payload:
             try:
-                discovery_payload = MQTTDiscoveryPayload(json_loads(payload))
+                discovery_payload = MQTTDiscoveryPayload(json_loads_object(payload))
             except ValueError:
                 _LOGGER.warning("Unable to parse JSON %s: '%s'", object_id, payload)
                 return
@@ -145,6 +190,22 @@ async def async_start(  # noqa: C901
                 key = DEVICE_ABBREVIATIONS.get(key, key)
                 device[key] = device.pop(abbreviated_key)
 
+        if CONF_ORIGIN in discovery_payload:
+            origin_info: dict[str, Any] = discovery_payload[CONF_ORIGIN]
+            try:
+                for key in list(origin_info):
+                    abbreviated_key = key
+                    key = ORIGIN_ABBREVIATIONS.get(key, key)
+                    origin_info[key] = origin_info.pop(abbreviated_key)
+                MQTT_ORIGIN_INFO_SCHEMA(discovery_payload[CONF_ORIGIN])
+            except Exception:  # pylint: disable=broad-except
+                _LOGGER.warning(
+                    "Unable to parse origin information "
+                    "from discovery message, got %s",
+                    discovery_payload[CONF_ORIGIN],
+                )
+                return
+
         if CONF_AVAILABILITY in discovery_payload:
             for availability_conf in cv.ensure_list(
                 discovery_payload[CONF_AVAILABILITY]
@@ -205,56 +266,54 @@ async def async_start(  # noqa: C901
             )
             return
 
-        await async_process_discovery_payload(
-            component, discovery_id, discovery_payload
-        )
+        async_process_discovery_payload(component, discovery_id, discovery_payload)
 
-    async def async_process_discovery_payload(
+    @callback
+    def async_process_discovery_payload(
         component: str, discovery_id: str, payload: MQTTDiscoveryPayload
     ) -> None:
         """Process the payload of a new discovery."""
 
         _LOGGER.debug("Process discovery payload %s", payload)
         discovery_hash = (component, discovery_id)
-        if discovery_hash in mqtt_data.discovery_already_discovered or payload:
 
-            async def discovery_done(_: Any) -> None:
-                pending = mqtt_data.discovery_pending_discovered[discovery_hash][
-                    "pending"
-                ]
+        already_discovered = discovery_hash in mqtt_data.discovery_already_discovered
+        if (
+            already_discovered or payload
+        ) and discovery_hash not in mqtt_data.discovery_pending_discovered:
+            discovery_pending_discovered = mqtt_data.discovery_pending_discovered
+
+            @callback
+            def discovery_done(_: Any) -> None:
+                pending = discovery_pending_discovered[discovery_hash]["pending"]
                 _LOGGER.debug("Pending discovery for %s: %s", discovery_hash, pending)
                 if not pending:
-                    mqtt_data.discovery_pending_discovered[discovery_hash]["unsub"]()
-                    mqtt_data.discovery_pending_discovered.pop(discovery_hash)
+                    discovery_pending_discovered[discovery_hash]["unsub"]()
+                    discovery_pending_discovered.pop(discovery_hash)
                 else:
                     payload = pending.pop()
-                    await async_process_discovery_payload(
-                        component, discovery_id, payload
-                    )
+                    async_process_discovery_payload(component, discovery_id, payload)
 
-            if discovery_hash not in mqtt_data.discovery_pending_discovered:
-                mqtt_data.discovery_pending_discovered[discovery_hash] = {
-                    "unsub": async_dispatcher_connect(
-                        hass,
-                        MQTT_DISCOVERY_DONE.format(discovery_hash),
-                        discovery_done,
-                    ),
-                    "pending": deque([]),
-                }
+            discovery_pending_discovered[discovery_hash] = {
+                "unsub": async_dispatcher_connect(
+                    hass,
+                    MQTT_DISCOVERY_DONE.format(discovery_hash),
+                    discovery_done,
+                ),
+                "pending": deque([]),
+            }
 
-        if discovery_hash in mqtt_data.discovery_already_discovered:
+        if already_discovered:
             # Dispatch update
-            _LOGGER.info(
-                "Component has already been discovered: %s %s, sending update",
-                component,
-                discovery_id,
-            )
+            message = f"Component has already been discovered: {component} {discovery_id}, sending update"
+            async_log_discovery_origin_info(message, payload)
             async_dispatcher_send(
                 hass, MQTT_DISCOVERY_UPDATED.format(discovery_hash), payload
             )
         elif payload:
             # Add component
-            _LOGGER.info("Found new component: %s %s", component, discovery_id)
+            message = f"Found new component: {component} {discovery_id}"
+            async_log_discovery_origin_info(message, payload)
             mqtt_data.discovery_already_discovered.add(discovery_hash)
             async_dispatcher_send(
                 hass, MQTT_DISCOVERY_NEW.format(component, "mqtt"), payload
@@ -279,13 +338,14 @@ async def async_start(  # noqa: C901
     mqtt_data.last_discovery = time.time()
     mqtt_integrations = await async_get_mqtt(hass)
 
-    for (integration, topics) in mqtt_integrations.items():
+    for integration, topics in mqtt_integrations.items():
 
         async def async_integration_message_received(
             integration: str, msg: ReceiveMessage
         ) -> None:
             """Process the received message."""
-            assert mqtt_data.data_config_flow_lock
+            if TYPE_CHECKING:
+                assert mqtt_data.data_config_flow_lock
             key = f"{integration}_{msg.subscribed_topic}"
 
             # Lock to prevent initiating many parallel config flows.
@@ -312,10 +372,7 @@ async def async_start(  # noqa: C901
                     and result["reason"]
                     in ("already_configured", "single_instance_allowed")
                 ):
-                    unsub = mqtt_data.integration_unsubscribe.pop(key, None)
-                    if unsub is None:
-                        return
-                    unsub()
+                    mqtt_data.integration_unsubscribe.pop(key)()
 
         for topic in topics:
             key = f"{integration}_{topic}"
