@@ -1,14 +1,16 @@
 """Support for MQTT fans."""
 from __future__ import annotations
 
-import functools
+from collections.abc import Callable
 import logging
 import math
+from typing import Any
 
 import voluptuous as vol
 
 from homeassistant.components import fan
 from homeassistant.components.fan import (
+    ATTR_DIRECTION,
     ATTR_OSCILLATING,
     ATTR_PERCENTAGE,
     ATTR_PRESET_MODE,
@@ -26,12 +28,13 @@ from homeassistant.const import (
 from homeassistant.core import HomeAssistant, callback
 import homeassistant.helpers.config_validation as cv
 from homeassistant.helpers.entity_platform import AddEntitiesCallback
-from homeassistant.helpers.typing import ConfigType, DiscoveryInfoType
+from homeassistant.helpers.template import Template
+from homeassistant.helpers.typing import ConfigType
 from homeassistant.util.percentage import (
-    int_states_in_range,
     percentage_to_ranged_value,
     ranged_value_to_percentage,
 )
+from homeassistant.util.scaling import int_states_in_range
 
 from . import subscription
 from .config import MQTT_RW_SCHEMA
@@ -49,14 +52,23 @@ from .debug_info import log_messages
 from .mixins import (
     MQTT_ENTITY_COMMON_SCHEMA,
     MqttEntity,
-    async_setup_entry_helper,
-    async_setup_platform_discovery,
-    async_setup_platform_helper,
-    warn_for_legacy_schema,
+    async_setup_entity_entry_helper,
+    write_state_on_attr_change,
+)
+from .models import (
+    MessageCallbackType,
+    MqttCommandTemplate,
+    MqttValueTemplate,
+    PublishPayloadType,
+    ReceiveMessage,
+    ReceivePayloadType,
 )
-from .models import MqttCommandTemplate, MqttValueTemplate
 from .util import valid_publish_topic, valid_subscribe_topic
 
+CONF_DIRECTION_STATE_TOPIC = "direction_state_topic"
+CONF_DIRECTION_COMMAND_TOPIC = "direction_command_topic"
+CONF_DIRECTION_VALUE_TEMPLATE = "direction_value_template"
+CONF_DIRECTION_COMMAND_TEMPLATE = "direction_command_template"
 CONF_PERCENTAGE_STATE_TOPIC = "percentage_state_topic"
 CONF_PERCENTAGE_COMMAND_TOPIC = "percentage_command_topic"
 CONF_PERCENTAGE_VALUE_TEMPLATE = "percentage_value_template"
@@ -70,26 +82,17 @@ CONF_PRESET_MODE_VALUE_TEMPLATE = "preset_mode_value_template"
 CONF_PRESET_MODE_COMMAND_TEMPLATE = "preset_mode_command_template"
 CONF_PRESET_MODES_LIST = "preset_modes"
 CONF_PAYLOAD_RESET_PRESET_MODE = "payload_reset_preset_mode"
-CONF_SPEED_STATE_TOPIC = "speed_state_topic"
-CONF_SPEED_COMMAND_TOPIC = "speed_command_topic"
-CONF_SPEED_VALUE_TEMPLATE = "speed_value_template"
 CONF_OSCILLATION_STATE_TOPIC = "oscillation_state_topic"
 CONF_OSCILLATION_COMMAND_TOPIC = "oscillation_command_topic"
 CONF_OSCILLATION_VALUE_TEMPLATE = "oscillation_value_template"
 CONF_OSCILLATION_COMMAND_TEMPLATE = "oscillation_command_template"
 CONF_PAYLOAD_OSCILLATION_ON = "payload_oscillation_on"
 CONF_PAYLOAD_OSCILLATION_OFF = "payload_oscillation_off"
-CONF_PAYLOAD_OFF_SPEED = "payload_off_speed"
-CONF_PAYLOAD_LOW_SPEED = "payload_low_speed"
-CONF_PAYLOAD_MEDIUM_SPEED = "payload_medium_speed"
-CONF_PAYLOAD_HIGH_SPEED = "payload_high_speed"
-CONF_SPEED_LIST = "speeds"
 
 DEFAULT_NAME = "MQTT Fan"
 DEFAULT_PAYLOAD_ON = "ON"
 DEFAULT_PAYLOAD_OFF = "OFF"
 DEFAULT_PAYLOAD_RESET = "None"
-DEFAULT_OPTIMISTIC = False
 DEFAULT_SPEED_RANGE_MIN = 1
 DEFAULT_SPEED_RANGE_MAX = 100
 
@@ -110,27 +113,30 @@ MQTT_FAN_ATTRIBUTES_BLOCKED = frozenset(
 _LOGGER = logging.getLogger(__name__)
 
 
-def valid_speed_range_configuration(config):
+def valid_speed_range_configuration(config: ConfigType) -> ConfigType:
     """Validate that the fan speed_range configuration is valid, throws if it isn't."""
-    if config.get(CONF_SPEED_RANGE_MIN) == 0:
-        raise ValueError("speed_range_min must be > 0")
-    if config.get(CONF_SPEED_RANGE_MIN) >= config.get(CONF_SPEED_RANGE_MAX):
-        raise ValueError("speed_range_max must be > speed_range_min")
+    if config[CONF_SPEED_RANGE_MIN] == 0:
+        raise vol.Invalid("speed_range_min must be > 0")
+    if config[CONF_SPEED_RANGE_MIN] >= config[CONF_SPEED_RANGE_MAX]:
+        raise vol.Invalid("speed_range_max must be > speed_range_min")
     return config
 
 
-def valid_preset_mode_configuration(config):
+def valid_preset_mode_configuration(config: ConfigType) -> ConfigType:
     """Validate that the preset mode reset payload is not one of the preset modes."""
-    if config.get(CONF_PAYLOAD_RESET_PRESET_MODE) in config.get(CONF_PRESET_MODES_LIST):
-        raise ValueError("preset_modes must not contain payload_reset_preset_mode")
+    if config[CONF_PAYLOAD_RESET_PRESET_MODE] in config[CONF_PRESET_MODES_LIST]:
+        raise vol.Invalid("preset_modes must not contain payload_reset_preset_mode")
     return config
 
 
 _PLATFORM_SCHEMA_BASE = MQTT_RW_SCHEMA.extend(
     {
-        vol.Optional(CONF_NAME, default=DEFAULT_NAME): cv.string,
-        vol.Optional(CONF_OPTIMISTIC, default=DEFAULT_OPTIMISTIC): cv.boolean,
+        vol.Optional(CONF_NAME): vol.Any(cv.string, None),
         vol.Optional(CONF_COMMAND_TEMPLATE): cv.template,
+        vol.Optional(CONF_DIRECTION_COMMAND_TOPIC): valid_publish_topic,
+        vol.Optional(CONF_DIRECTION_COMMAND_TEMPLATE): cv.template,
+        vol.Optional(CONF_DIRECTION_STATE_TOPIC): valid_subscribe_topic,
+        vol.Optional(CONF_DIRECTION_VALUE_TEMPLATE): cv.template,
         vol.Optional(CONF_OSCILLATION_COMMAND_TOPIC): valid_publish_topic,
         vol.Optional(CONF_OSCILLATION_COMMAND_TEMPLATE): cv.template,
         vol.Optional(CONF_OSCILLATION_STATE_TOPIC): valid_subscribe_topic,
@@ -139,7 +145,8 @@ _PLATFORM_SCHEMA_BASE = MQTT_RW_SCHEMA.extend(
         vol.Optional(CONF_PERCENTAGE_COMMAND_TEMPLATE): cv.template,
         vol.Optional(CONF_PERCENTAGE_STATE_TOPIC): valid_subscribe_topic,
         vol.Optional(CONF_PERCENTAGE_VALUE_TEMPLATE): cv.template,
-        # CONF_PRESET_MODE_COMMAND_TOPIC and CONF_PRESET_MODES_LIST must be used together
+        # CONF_PRESET_MODE_COMMAND_TOPIC and CONF_PRESET_MODES_LIST
+        # must be used together
         vol.Inclusive(
             CONF_PRESET_MODE_COMMAND_TOPIC, "preset_modes"
         ): valid_publish_topic,
@@ -169,21 +176,10 @@ _PLATFORM_SCHEMA_BASE = MQTT_RW_SCHEMA.extend(
         vol.Optional(
             CONF_PAYLOAD_OSCILLATION_ON, default=OSCILLATE_ON_PAYLOAD
         ): cv.string,
-        vol.Optional(CONF_SPEED_COMMAND_TOPIC): valid_publish_topic,
-        vol.Optional(CONF_SPEED_STATE_TOPIC): valid_subscribe_topic,
-        vol.Optional(CONF_SPEED_VALUE_TEMPLATE): cv.template,
         vol.Optional(CONF_STATE_VALUE_TEMPLATE): cv.template,
     }
 ).extend(MQTT_ENTITY_COMMON_SCHEMA.schema)
 
-# Configuring MQTT Fans under the fan platform key is deprecated in HA Core 2022.6
-PLATFORM_SCHEMA = vol.All(
-    cv.PLATFORM_SCHEMA.extend(_PLATFORM_SCHEMA_BASE.schema),
-    valid_speed_range_configuration,
-    valid_preset_mode_configuration,
-    warn_for_legacy_schema(fan.DOMAIN),
-)
-
 PLATFORM_SCHEMA_MODERN = vol.All(
     _PLATFORM_SCHEMA_BASE,
     valid_speed_range_configuration,
@@ -191,102 +187,70 @@ PLATFORM_SCHEMA_MODERN = vol.All(
 )
 
 DISCOVERY_SCHEMA = vol.All(
-    # CONF_SPEED_COMMAND_TOPIC, CONF_SPEED_LIST, CONF_SPEED_STATE_TOPIC, CONF_SPEED_VALUE_TEMPLATE and
-    # Speeds SPEED_LOW, SPEED_MEDIUM, SPEED_HIGH SPEED_OFF,
-    # are no longer supported, support was removed in release 2021.12
-    cv.removed(CONF_PAYLOAD_HIGH_SPEED),
-    cv.removed(CONF_PAYLOAD_LOW_SPEED),
-    cv.removed(CONF_PAYLOAD_MEDIUM_SPEED),
-    cv.removed(CONF_SPEED_COMMAND_TOPIC),
-    cv.removed(CONF_SPEED_LIST),
-    cv.removed(CONF_SPEED_STATE_TOPIC),
-    cv.removed(CONF_SPEED_VALUE_TEMPLATE),
     _PLATFORM_SCHEMA_BASE.extend({}, extra=vol.REMOVE_EXTRA),
     valid_speed_range_configuration,
     valid_preset_mode_configuration,
 )
 
 
-async def async_setup_platform(
-    hass: HomeAssistant,
-    config: ConfigType,
-    async_add_entities: AddEntitiesCallback,
-    discovery_info: DiscoveryInfoType | None = None,
-) -> None:
-    """Set up MQTT fans configured under the fan platform key (deprecated)."""
-    # Deprecated in HA Core 2022.6
-    await async_setup_platform_helper(
-        hass,
-        fan.DOMAIN,
-        discovery_info or config,
-        async_add_entities,
-        _async_setup_entity,
-    )
-
-
 async def async_setup_entry(
     hass: HomeAssistant,
     config_entry: ConfigEntry,
     async_add_entities: AddEntitiesCallback,
 ) -> None:
-    """Set up MQTT fan through configuration.yaml and dynamically through MQTT discovery."""
-    # load and initialize platform config from configuration.yaml
-    config_entry.async_on_unload(await async_setup_platform_discovery(hass, fan.DOMAIN))
-    # setup for discovery
-    setup = functools.partial(
-        _async_setup_entity, hass, async_add_entities, config_entry=config_entry
+    """Set up MQTT fan through YAML and through MQTT discovery."""
+    await async_setup_entity_entry_helper(
+        hass,
+        config_entry,
+        MqttFan,
+        fan.DOMAIN,
+        async_add_entities,
+        DISCOVERY_SCHEMA,
+        PLATFORM_SCHEMA_MODERN,
     )
-    await async_setup_entry_helper(hass, fan.DOMAIN, setup, DISCOVERY_SCHEMA)
-
-
-async def _async_setup_entity(
-    hass, async_add_entities, config, config_entry=None, discovery_data=None
-):
-    """Set up the MQTT fan."""
-    async_add_entities([MqttFan(hass, config, config_entry, discovery_data)])
 
 
 class MqttFan(MqttEntity, FanEntity):
     """A MQTT fan component."""
 
+    _attr_percentage: int | None = None
+    _attr_preset_mode: str | None = None
+
+    _default_name = DEFAULT_NAME
     _entity_id_format = fan.ENTITY_ID_FORMAT
     _attributes_extra_blocked = MQTT_FAN_ATTRIBUTES_BLOCKED
 
-    def __init__(self, hass, config, config_entry, discovery_data):
-        """Initialize the MQTT fan."""
-        self._state = None
-        self._percentage = None
-        self._preset_mode = None
-        self._oscillation = None
-        self._supported_features = 0
-
-        self._topic = None
-        self._payload = None
-        self._value_templates = None
-        self._command_templates = None
-        self._optimistic = None
-        self._optimistic_oscillation = None
-        self._optimistic_percentage = None
-        self._optimistic_preset_mode = None
-
-        MqttEntity.__init__(self, hass, config, config_entry, discovery_data)
+    _command_templates: dict[str, Callable[[PublishPayloadType], PublishPayloadType]]
+    _value_templates: dict[str, Callable[[ReceivePayloadType], ReceivePayloadType]]
+    _feature_percentage: bool
+    _feature_preset_mode: bool
+    _topic: dict[str, Any]
+    _optimistic: bool
+    _optimistic_direction: bool
+    _optimistic_oscillation: bool
+    _optimistic_percentage: bool
+    _optimistic_preset_mode: bool
+    _payload: dict[str, Any]
+    _speed_range: tuple[int, int]
 
     @staticmethod
-    def config_schema():
+    def config_schema() -> vol.Schema:
         """Return the config schema."""
         return DISCOVERY_SCHEMA
 
-    def _setup_from_config(self, config):
+    def _setup_from_config(self, config: ConfigType) -> None:
         """(Re)Setup the entity."""
         self._speed_range = (
-            config.get(CONF_SPEED_RANGE_MIN),
-            config.get(CONF_SPEED_RANGE_MAX),
+            config[CONF_SPEED_RANGE_MIN],
+            config[CONF_SPEED_RANGE_MAX],
         )
         self._topic = {
             key: config.get(key)
             for key in (
                 CONF_STATE_TOPIC,
                 CONF_COMMAND_TOPIC,
+                CONF_DIRECTION_STATE_TOPIC,
+                CONF_DIRECTION_COMMAND_TOPIC,
                 CONF_PERCENTAGE_STATE_TOPIC,
                 CONF_PERCENTAGE_COMMAND_TOPIC,
                 CONF_PRESET_MODE_STATE_TOPIC,
@@ -295,18 +259,6 @@ class MqttFan(MqttEntity, FanEntity):
                 CONF_OSCILLATION_COMMAND_TOPIC,
             )
         }
-        self._value_templates = {
-            CONF_STATE: config.get(CONF_STATE_VALUE_TEMPLATE),
-            ATTR_PERCENTAGE: config.get(CONF_PERCENTAGE_VALUE_TEMPLATE),
-            ATTR_PRESET_MODE: config.get(CONF_PRESET_MODE_VALUE_TEMPLATE),
-            ATTR_OSCILLATING: config.get(CONF_OSCILLATION_VALUE_TEMPLATE),
-        }
-        self._command_templates = {
-            CONF_STATE: config.get(CONF_COMMAND_TEMPLATE),
-            ATTR_PERCENTAGE: config.get(CONF_PERCENTAGE_COMMAND_TEMPLATE),
-            ATTR_PRESET_MODE: config.get(CONF_PRESET_MODE_COMMAND_TEMPLATE),
-            ATTR_OSCILLATING: config.get(CONF_OSCILLATION_COMMAND_TEMPLATE),
-        }
         self._payload = {
             "STATE_ON": config[CONF_PAYLOAD_ON],
             "STATE_OFF": config[CONF_PAYLOAD_OFF],
@@ -319,11 +271,11 @@ class MqttFan(MqttEntity, FanEntity):
         self._feature_percentage = CONF_PERCENTAGE_COMMAND_TOPIC in config
         self._feature_preset_mode = CONF_PRESET_MODE_COMMAND_TOPIC in config
         if self._feature_preset_mode:
-            self._preset_modes = config[CONF_PRESET_MODES_LIST]
+            self._attr_preset_modes = config[CONF_PRESET_MODES_LIST]
         else:
-            self._preset_modes = []
+            self._attr_preset_modes = []
 
-        self._speed_count = (
+        self._attr_speed_count = (
             min(int_states_in_range(self._speed_range), 100)
             if self._feature_percentage
             else 100
@@ -331,6 +283,10 @@ class MqttFan(MqttEntity, FanEntity):
 
         optimistic = config[CONF_OPTIMISTIC]
         self._optimistic = optimistic or self._topic[CONF_STATE_TOPIC] is None
+        self._attr_assumed_state = bool(self._optimistic)
+        self._optimistic_direction = (
+            optimistic or self._topic[CONF_DIRECTION_STATE_TOPIC] is None
+        )
         self._optimistic_oscillation = (
             optimistic or self._topic[CONF_OSCILLATION_STATE_TOPIC] is None
         )
@@ -341,58 +297,84 @@ class MqttFan(MqttEntity, FanEntity):
             optimistic or self._topic[CONF_PRESET_MODE_STATE_TOPIC] is None
         )
 
-        self._supported_features = 0
-        self._supported_features |= (
+        self._attr_supported_features = FanEntityFeature(0)
+        self._attr_supported_features |= (
             self._topic[CONF_OSCILLATION_COMMAND_TOPIC] is not None
             and FanEntityFeature.OSCILLATE
         )
+        self._attr_supported_features |= (
+            self._topic[CONF_DIRECTION_COMMAND_TOPIC] is not None
+            and FanEntityFeature.DIRECTION
+        )
         if self._feature_percentage:
-            self._supported_features |= FanEntityFeature.SET_SPEED
+            self._attr_supported_features |= FanEntityFeature.SET_SPEED
         if self._feature_preset_mode:
-            self._supported_features |= FanEntityFeature.PRESET_MODE
+            self._attr_supported_features |= FanEntityFeature.PRESET_MODE
 
-        for key, tpl in self._command_templates.items():
+        command_templates: dict[str, Template | None] = {
+            CONF_STATE: config.get(CONF_COMMAND_TEMPLATE),
+            ATTR_DIRECTION: config.get(CONF_DIRECTION_COMMAND_TEMPLATE),
+            ATTR_PERCENTAGE: config.get(CONF_PERCENTAGE_COMMAND_TEMPLATE),
+            ATTR_PRESET_MODE: config.get(CONF_PRESET_MODE_COMMAND_TEMPLATE),
+            ATTR_OSCILLATING: config.get(CONF_OSCILLATION_COMMAND_TEMPLATE),
+        }
+        self._command_templates = {}
+        for key, tpl in command_templates.items():
             self._command_templates[key] = MqttCommandTemplate(
                 tpl, entity=self
             ).async_render
 
-        for key, tpl in self._value_templates.items():
+        self._value_templates = {}
+        value_templates: dict[str, Template | None] = {
+            CONF_STATE: config.get(CONF_STATE_VALUE_TEMPLATE),
+            ATTR_DIRECTION: config.get(CONF_DIRECTION_VALUE_TEMPLATE),
+            ATTR_PERCENTAGE: config.get(CONF_PERCENTAGE_VALUE_TEMPLATE),
+            ATTR_PRESET_MODE: config.get(CONF_PRESET_MODE_VALUE_TEMPLATE),
+            ATTR_OSCILLATING: config.get(CONF_OSCILLATION_VALUE_TEMPLATE),
+        }
+        for key, tpl in value_templates.items():
             self._value_templates[key] = MqttValueTemplate(
                 tpl,
                 entity=self,
             ).async_render_with_possible_json_value
 
-    def _prepare_subscribe_topics(self):
+    def _prepare_subscribe_topics(self) -> None:
         """(Re)Subscribe to topics."""
-        topics = {}
+        topics: dict[str, Any] = {}
+
+        def add_subscribe_topic(topic: str, msg_callback: MessageCallbackType) -> bool:
+            """Add a topic to subscribe to."""
+            if has_topic := self._topic[topic] is not None:
+                topics[topic] = {
+                    "topic": self._topic[topic],
+                    "msg_callback": msg_callback,
+                    "qos": self._config[CONF_QOS],
+                    "encoding": self._config[CONF_ENCODING] or None,
+                }
+            return has_topic
 
         @callback
         @log_messages(self.hass, self.entity_id)
-        def state_received(msg):
+        @write_state_on_attr_change(self, {"_attr_is_on"})
+        def state_received(msg: ReceiveMessage) -> None:
             """Handle new received MQTT message."""
             payload = self._value_templates[CONF_STATE](msg.payload)
             if not payload:
                 _LOGGER.debug("Ignoring empty state from '%s'", msg.topic)
                 return
             if payload == self._payload["STATE_ON"]:
-                self._state = True
+                self._attr_is_on = True
             elif payload == self._payload["STATE_OFF"]:
-                self._state = False
+                self._attr_is_on = False
             elif payload == PAYLOAD_NONE:
-                self._state = None
-            self.async_write_ha_state()
+                self._attr_is_on = None
 
-        if self._topic[CONF_STATE_TOPIC] is not None:
-            topics[CONF_STATE_TOPIC] = {
-                "topic": self._topic[CONF_STATE_TOPIC],
-                "msg_callback": state_received,
-                "qos": self._config[CONF_QOS],
-                "encoding": self._config[CONF_ENCODING] or None,
-            }
+        add_subscribe_topic(CONF_STATE_TOPIC, state_received)
 
         @callback
         @log_messages(self.hass, self.entity_id)
-        def percentage_received(msg):
+        @write_state_on_attr_change(self, {"_attr_percentage"})
+        def percentage_received(msg: ReceiveMessage) -> None:
             """Handle new received MQTT message for the percentage."""
             rendered_percentage_payload = self._value_templates[ATTR_PERCENTAGE](
                 msg.payload
@@ -401,8 +383,7 @@ class MqttFan(MqttEntity, FanEntity):
                 _LOGGER.debug("Ignoring empty speed from '%s'", msg.topic)
                 return
             if rendered_percentage_payload == self._payload["PERCENTAGE_RESET"]:
-                self._percentage = None
-                self.async_write_ha_state()
+                self._attr_percentage = None
                 return
             try:
                 percentage = ranged_value_to_percentage(
@@ -410,7 +391,10 @@ class MqttFan(MqttEntity, FanEntity):
                 )
             except ValueError:
                 _LOGGER.warning(
-                    "'%s' received on topic %s. '%s' is not a valid speed within the speed range",
+                    (
+                        "'%s' received on topic %s. '%s' is not a valid speed within"
+                        " the speed range"
+                    ),
                     msg.payload,
                     msg.topic,
                     rendered_percentage_payload,
@@ -418,37 +402,32 @@ class MqttFan(MqttEntity, FanEntity):
                 return
             if percentage < 0 or percentage > 100:
                 _LOGGER.warning(
-                    "'%s' received on topic %s. '%s' is not a valid speed within the speed range",
+                    (
+                        "'%s' received on topic %s. '%s' is not a valid speed within"
+                        " the speed range"
+                    ),
                     msg.payload,
                     msg.topic,
                     rendered_percentage_payload,
                 )
                 return
-            self._percentage = percentage
-            self.async_write_ha_state()
+            self._attr_percentage = percentage
 
-        if self._topic[CONF_PERCENTAGE_STATE_TOPIC] is not None:
-            topics[CONF_PERCENTAGE_STATE_TOPIC] = {
-                "topic": self._topic[CONF_PERCENTAGE_STATE_TOPIC],
-                "msg_callback": percentage_received,
-                "qos": self._config[CONF_QOS],
-                "encoding": self._config[CONF_ENCODING] or None,
-            }
-            self._percentage = None
+        add_subscribe_topic(CONF_PERCENTAGE_STATE_TOPIC, percentage_received)
 
         @callback
         @log_messages(self.hass, self.entity_id)
-        def preset_mode_received(msg):
+        @write_state_on_attr_change(self, {"_attr_preset_mode"})
+        def preset_mode_received(msg: ReceiveMessage) -> None:
             """Handle new received MQTT message for preset mode."""
-            preset_mode = self._value_templates[ATTR_PRESET_MODE](msg.payload)
+            preset_mode = str(self._value_templates[ATTR_PRESET_MODE](msg.payload))
             if preset_mode == self._payload["PRESET_MODE_RESET"]:
-                self._preset_mode = None
-                self.async_write_ha_state()
+                self._attr_preset_mode = None
                 return
             if not preset_mode:
                 _LOGGER.debug("Ignoring empty preset_mode from '%s'", msg.topic)
                 return
-            if preset_mode not in self.preset_modes:
+            if not self.preset_modes or preset_mode not in self.preset_modes:
                 _LOGGER.warning(
                     "'%s' received on topic %s. '%s' is not a valid preset mode",
                     msg.payload,
@@ -457,95 +436,59 @@ class MqttFan(MqttEntity, FanEntity):
                 )
                 return
 
-            self._preset_mode = preset_mode
-            self.async_write_ha_state()
+            self._attr_preset_mode = preset_mode
 
-        if self._topic[CONF_PRESET_MODE_STATE_TOPIC] is not None:
-            topics[CONF_PRESET_MODE_STATE_TOPIC] = {
-                "topic": self._topic[CONF_PRESET_MODE_STATE_TOPIC],
-                "msg_callback": preset_mode_received,
-                "qos": self._config[CONF_QOS],
-                "encoding": self._config[CONF_ENCODING] or None,
-            }
-            self._preset_mode = None
+        add_subscribe_topic(CONF_PRESET_MODE_STATE_TOPIC, preset_mode_received)
 
         @callback
         @log_messages(self.hass, self.entity_id)
-        def oscillation_received(msg):
+        @write_state_on_attr_change(self, {"_attr_oscillating"})
+        def oscillation_received(msg: ReceiveMessage) -> None:
             """Handle new received MQTT message for the oscillation."""
             payload = self._value_templates[ATTR_OSCILLATING](msg.payload)
             if not payload:
                 _LOGGER.debug("Ignoring empty oscillation from '%s'", msg.topic)
                 return
             if payload == self._payload["OSCILLATE_ON_PAYLOAD"]:
-                self._oscillation = True
+                self._attr_oscillating = True
             elif payload == self._payload["OSCILLATE_OFF_PAYLOAD"]:
-                self._oscillation = False
-            self.async_write_ha_state()
+                self._attr_oscillating = False
 
-        if self._topic[CONF_OSCILLATION_STATE_TOPIC] is not None:
-            topics[CONF_OSCILLATION_STATE_TOPIC] = {
-                "topic": self._topic[CONF_OSCILLATION_STATE_TOPIC],
-                "msg_callback": oscillation_received,
-                "qos": self._config[CONF_QOS],
-                "encoding": self._config[CONF_ENCODING] or None,
-            }
-            self._oscillation = False
+        if add_subscribe_topic(CONF_OSCILLATION_STATE_TOPIC, oscillation_received):
+            self._attr_oscillating = False
+
+        @callback
+        @log_messages(self.hass, self.entity_id)
+        @write_state_on_attr_change(self, {"_attr_current_direction"})
+        def direction_received(msg: ReceiveMessage) -> None:
+            """Handle new received MQTT message for the direction."""
+            direction = self._value_templates[ATTR_DIRECTION](msg.payload)
+            if not direction:
+                _LOGGER.debug("Ignoring empty direction from '%s'", msg.topic)
+                return
+            self._attr_current_direction = str(direction)
+
+        add_subscribe_topic(CONF_DIRECTION_STATE_TOPIC, direction_received)
 
         self._sub_state = subscription.async_prepare_subscribe_topics(
             self.hass, self._sub_state, topics
         )
 
-    async def _subscribe_topics(self):
+    async def _subscribe_topics(self) -> None:
         """(Re)Subscribe to topics."""
         await subscription.async_subscribe_topics(self.hass, self._sub_state)
 
-    @property
-    def assumed_state(self):
-        """Return true if we do optimistic updates."""
-        return self._optimistic
-
     @property
     def is_on(self) -> bool | None:
         """Return true if device is on."""
-        return self._state
+        # The default for FanEntity is to compute it based on percentage
+        return self._attr_is_on
 
-    @property
-    def percentage(self):
-        """Return the current percentage."""
-        return self._percentage
-
-    @property
-    def preset_mode(self):
-        """Return the current preset _mode."""
-        return self._preset_mode
-
-    @property
-    def preset_modes(self) -> list:
-        """Get the list of available preset modes."""
-        return self._preset_modes
-
-    @property
-    def supported_features(self) -> int:
-        """Flag supported features."""
-        return self._supported_features
-
-    @property
-    def speed_count(self) -> int:
-        """Return the number of speeds the fan supports."""
-        return self._speed_count
-
-    @property
-    def oscillating(self):
-        """Return the oscillation state."""
-        return self._oscillation
-
-    # The speed attribute deprecated in the schema, support will be removed after a quarter (2021.7)
     async def async_turn_on(
         self,
-        percentage: int = None,
-        preset_mode: str = None,
-        **kwargs,
+        percentage: int | None = None,
+        preset_mode: str | None = None,
+        **kwargs: Any,
     ) -> None:
         """Turn on the entity.
 
@@ -564,10 +507,10 @@ class MqttFan(MqttEntity, FanEntity):
         if preset_mode:
             await self.async_set_preset_mode(preset_mode)
         if self._optimistic:
-            self._state = True
+            self._attr_is_on = True
             self.async_write_ha_state()
 
-    async def async_turn_off(self, **kwargs) -> None:
+    async def async_turn_off(self, **kwargs: Any) -> None:
         """Turn off the entity.
 
         This method is a coroutine.
@@ -581,7 +524,7 @@ class MqttFan(MqttEntity, FanEntity):
             self._config[CONF_ENCODING],
         )
         if self._optimistic:
-            self._state = False
+            self._attr_is_on = False
             self.async_write_ha_state()
 
     async def async_set_percentage(self, percentage: int) -> None:
@@ -602,7 +545,7 @@ class MqttFan(MqttEntity, FanEntity):
         )
 
         if self._optimistic_percentage:
-            self._percentage = percentage
+            self._attr_percentage = percentage
             self.async_write_ha_state()
 
     async def async_set_preset_mode(self, preset_mode: str) -> None:
@@ -610,8 +553,6 @@ class MqttFan(MqttEntity, FanEntity):
 
         This method is a coroutine.
         """
-        self._valid_preset_mode_or_raise(preset_mode)
-
         mqtt_payload = self._command_templates[ATTR_PRESET_MODE](preset_mode)
 
         await self.async_publish(
@@ -623,7 +564,7 @@ class MqttFan(MqttEntity, FanEntity):
         )
 
         if self._optimistic_preset_mode:
-            self._preset_mode = preset_mode
+            self._attr_preset_mode = preset_mode
             self.async_write_ha_state()
 
     async def async_oscillate(self, oscillating: bool) -> None:
@@ -649,5 +590,24 @@ class MqttFan(MqttEntity, FanEntity):
         )
 
         if self._optimistic_oscillation:
-            self._oscillation = oscillating
+            self._attr_oscillating = oscillating
+            self.async_write_ha_state()
+
+    async def async_set_direction(self, direction: str) -> None:
+        """Set direction.
+
+        This method is a coroutine.
+        """
+        mqtt_payload = self._command_templates[ATTR_DIRECTION](direction)
+
+        await self.async_publish(
+            self._topic[CONF_DIRECTION_COMMAND_TOPIC],
+            mqtt_payload,
+            self._config[CONF_QOS],
+            self._config[CONF_RETAIN],
+            self._config[CONF_ENCODING],
+        )
+
+        if self._optimistic_direction:
+            self._attr_current_direction = direction
             self.async_write_ha_state()
