@@ -2,14 +2,14 @@
 from __future__ import annotations
 
 from collections.abc import Callable, Coroutine
-from typing import Any
+from typing import TYPE_CHECKING, Any
 
 import attr
 
 from homeassistant.core import HomeAssistant
 
-from . import debug_info
 from .. import mqtt
+from . import debug_info
 from .const import DEFAULT_QOS
 from .models import MessageCallbackType
 
@@ -19,16 +19,20 @@ class EntitySubscription:
     """Class to hold data about an active entity topic subscription."""
 
     hass: HomeAssistant = attr.ib()
-    topic: str = attr.ib()
+    topic: str | None = attr.ib()
     message_callback: MessageCallbackType = attr.ib()
-    subscribe_task: Coroutine | None = attr.ib()
+    subscribe_task: Coroutine[Any, Any, Callable[[], None]] | None = attr.ib()
     unsubscribe_callback: Callable[[], None] | None = attr.ib()
     qos: int = attr.ib(default=0)
     encoding: str = attr.ib(default="utf-8")
 
-    def resubscribe_if_necessary(self, hass, other):
+    def resubscribe_if_necessary(
+        self, hass: HomeAssistant, other: EntitySubscription | None
+    ) -> None:
         """Re-subscribe to the new topic if necessary."""
         if not self._should_resubscribe(other):
+            if TYPE_CHECKING:
+                assert other
             self.unsubscribe_callback = other.unsubscribe_callback
             return
 
@@ -36,7 +40,7 @@ class EntitySubscription:
             other.unsubscribe_callback()
             # Clear debug data if it exists
             debug_info.remove_subscription(
-                self.hass, other.message_callback, other.topic
+                self.hass, other.message_callback, str(other.topic)
             )
 
         if self.topic is None:
@@ -50,18 +54,22 @@ class EntitySubscription:
             hass, self.topic, self.message_callback, self.qos, self.encoding
         )
 
-    async def subscribe(self):
+    async def subscribe(self) -> None:
         """Subscribe to a topic."""
         if not self.subscribe_task:
             return
         self.unsubscribe_callback = await self.subscribe_task
 
-    def _should_resubscribe(self, other):
+    def _should_resubscribe(self, other: EntitySubscription | None) -> bool:
         """Check if we should re-subscribe to the topic using the old state."""
         if other is None:
             return True
 
-        return (self.topic, self.qos, self.encoding,) != (
+        return (
+            self.topic,
+            self.qos,
+            self.encoding,
+        ) != (
             other.topic,
             other.qos,
             other.encoding,
@@ -109,7 +117,7 @@ def async_prepare_subscribe_topics(
             remaining.unsubscribe_callback()
             # Clear debug data if it exists
             debug_info.remove_subscription(
-                hass, remaining.message_callback, remaining.topic
+                hass, remaining.message_callback, str(remaining.topic)
             )
 
     return new_state
@@ -117,11 +125,9 @@ def async_prepare_subscribe_topics(
 
 async def async_subscribe_topics(
     hass: HomeAssistant,
-    sub_state: dict[str, EntitySubscription] | None,
+    sub_state: dict[str, EntitySubscription],
 ) -> None:
     """(Re)Subscribe to a set of MQTT topics."""
-    if sub_state is None:
-        return
     for sub in sub_state.values():
         await sub.subscribe()
 
