@@ -1,25 +1,20 @@
 """Config flow for Mullvad VPN integration."""
-import logging
-
 from mullvad_api import MullvadAPI, MullvadAPIError
 
 from homeassistant import config_entries
+from homeassistant.data_entry_flow import FlowResult
 
-from .const import DOMAIN  # pylint:disable=unused-import
-
-_LOGGER = logging.getLogger(__name__)
+from .const import DOMAIN
 
 
 class ConfigFlow(config_entries.ConfigFlow, domain=DOMAIN):
     """Handle a config flow for Mullvad VPN."""
 
     VERSION = 1
-    CONNECTION_CLASS = config_entries.CONN_CLASS_CLOUD_POLL
 
-    async def async_step_user(self, user_input=None):
+    async def async_step_user(self, user_input=None) -> FlowResult:
         """Handle the initial step."""
-        if self.hass.config_entries.async_entries(DOMAIN):
-            return self.async_abort(reason="already_configured")
+        self._async_abort_entries_match()
 
         errors = {}
         if user_input is not None:
