@@ -2,22 +2,19 @@
 from __future__ import annotations
 
 from enum import Enum, unique
-import logging
 from typing import Any
 
-from homeassistant.components import mysensors
-from homeassistant.components.cover import ATTR_POSITION, DOMAIN, CoverEntity
-from homeassistant.components.mysensors.const import MYSENSORS_DISCOVERY, DiscoveryInfo
+from homeassistant.components.cover import ATTR_POSITION, CoverEntity
 from homeassistant.config_entries import ConfigEntry
-from homeassistant.const import STATE_OFF, STATE_ON
+from homeassistant.const import STATE_OFF, STATE_ON, Platform
 from homeassistant.core import HomeAssistant
 from homeassistant.helpers.dispatcher import async_dispatcher_connect
 from homeassistant.helpers.entity_platform import AddEntitiesCallback
 
+from .. import mysensors
+from .const import MYSENSORS_DISCOVERY, DiscoveryInfo
 from .helpers import on_unload
 
-_LOGGER = logging.getLogger(__name__)
-
 
 @unique
 class CoverState(Enum):
@@ -40,7 +37,7 @@ async def async_setup_entry(
         """Discover and add a MySensors cover."""
         mysensors.setup_mysensors_platform(
             hass,
-            DOMAIN,
+            Platform.COVER,
             discovery_info,
             MySensorsCover,
             async_add_entities=async_add_entities,
@@ -51,13 +48,13 @@ async def async_setup_entry(
         config_entry.entry_id,
         async_dispatcher_connect(
             hass,
-            MYSENSORS_DISCOVERY.format(config_entry.entry_id, DOMAIN),
+            MYSENSORS_DISCOVERY.format(config_entry.entry_id, Platform.COVER),
             async_discover,
         ),
     )
 
 
-class MySensorsCover(mysensors.device.MySensorsEntity, CoverEntity):
+class MySensorsCover(mysensors.device.MySensorsChildEntity, CoverEntity):
     """Representation of the value of a MySensors Cover child node."""
 
     def get_cover_state(self) -> CoverState:
