@@ -1,21 +1,28 @@
 """Handle MySensors devices."""
 from __future__ import annotations
 
-from functools import partial
+from abc import abstractmethod
 import logging
 from typing import Any
 
 from mysensors import BaseAsyncGateway, Sensor
 from mysensors.sensor import ChildSensor
 
-from homeassistant.const import ATTR_BATTERY_LEVEL, STATE_OFF, STATE_ON
+from homeassistant.const import (
+    ATTR_BATTERY_LEVEL,
+    CONF_DEVICE,
+    STATE_OFF,
+    STATE_ON,
+    Platform,
+)
 from homeassistant.core import HomeAssistant, callback
+from homeassistant.helpers.debounce import Debouncer
+from homeassistant.helpers.device_registry import DeviceInfo
 from homeassistant.helpers.dispatcher import async_dispatcher_connect
-from homeassistant.helpers.entity import DeviceInfo, Entity
+from homeassistant.helpers.entity import Entity
 
 from .const import (
     CHILD_CALLBACK,
-    CONF_DEVICE,
     DOMAIN,
     NODE_CALLBACK,
     PLATFORM_TYPES,
@@ -34,61 +41,24 @@ ATTR_HEARTBEAT = "heartbeat"
 MYSENSORS_PLATFORM_DEVICES = "mysensors_devices_{}"
 
 
-class MySensorsDevice:
+class MySensorNodeEntity(Entity):
     """Representation of a MySensors device."""
 
     hass: HomeAssistant
 
     def __init__(
-        self,
-        gateway_id: GatewayId,
-        gateway: BaseAsyncGateway,
-        node_id: int,
-        child_id: int,
-        value_type: int,
+        self, gateway_id: GatewayId, gateway: BaseAsyncGateway, node_id: int
     ) -> None:
-        """Set up the MySensors device."""
+        """Set up the MySensors node entity."""
         self.gateway_id: GatewayId = gateway_id
         self.gateway: BaseAsyncGateway = gateway
         self.node_id: int = node_id
-        self.child_id: int = child_id
-        self.value_type: int = value_type  # value_type as int. string variant can be looked up in gateway consts
-        self.child_type = self._child.type
-        self._values: dict[int, Any] = {}
-        self._update_scheduled = False
-
-    @property
-    def dev_id(self) -> DevId:
-        """Return the DevId of this device.
-
-        It is used to route incoming MySensors messages to the correct device/entity.
-        """
-        return self.gateway_id, self.node_id, self.child_id, self.value_type
-
-    @property
-    def _logger(self) -> logging.Logger:
-        return logging.getLogger(f"{__name__}.{self.name}")
-
-    async def async_will_remove_from_hass(self) -> None:
-        """Remove this entity from home assistant."""
-        for platform in PLATFORM_TYPES:
-            platform_str = MYSENSORS_PLATFORM_DEVICES.format(platform)
-            if platform_str in self.hass.data[DOMAIN]:
-                platform_dict = self.hass.data[DOMAIN][platform_str]
-                if self.dev_id in platform_dict:
-                    del platform_dict[self.dev_id]
-                    self._logger.debug(
-                        "deleted %s from platform %s", self.dev_id, platform
-                    )
+        self._debouncer: Debouncer | None = None
 
     @property
     def _node(self) -> Sensor:
         return self.gateway.sensors[self.node_id]
 
-    @property
-    def _child(self) -> ChildSensor:
-        return self._node.children[self.child_id]
-
     @property
     def sketch_name(self) -> str:
         """Return the name of the sketch running on the whole node.
@@ -113,11 +83,6 @@ class MySensorsDevice:
         """
         return f"{self.sketch_name} {self.node_id}"
 
-    @property
-    def unique_id(self) -> str:
-        """Return a unique ID for use in home assistant."""
-        return f"{self.gateway_id}-{self.node_id}-{self.child_id}-{self.value_type}"
-
     @property
     def device_info(self) -> DeviceInfo:
         """Return the device info."""
@@ -128,6 +93,96 @@ class MySensorsDevice:
             sw_version=self.sketch_version,
         )
 
+    @property
+    def extra_state_attributes(self) -> dict[str, Any]:
+        """Return device specific attributes."""
+        node = self.gateway.sensors[self.node_id]
+
+        return {
+            ATTR_HEARTBEAT: node.heartbeat,
+            ATTR_NODE_ID: self.node_id,
+        }
+
+    @callback
+    @abstractmethod
+    def _async_update_callback(self) -> None:
+        """Update the device."""
+
+    async def async_update_callback(self) -> None:
+        """Update the device after delay."""
+        if not self._debouncer:
+            self._debouncer = Debouncer(
+                self.hass,
+                _LOGGER,
+                cooldown=UPDATE_DELAY,
+                immediate=False,
+                function=self._async_update_callback,
+            )
+
+        await self._debouncer.async_call()
+
+    async def async_added_to_hass(self) -> None:
+        """Register update callback."""
+        self.async_on_remove(
+            async_dispatcher_connect(
+                self.hass,
+                NODE_CALLBACK.format(self.gateway_id, self.node_id),
+                self.async_update_callback,
+            )
+        )
+        self._async_update_callback()
+
+
+def get_mysensors_devices(
+    hass: HomeAssistant, domain: Platform
+) -> dict[DevId, MySensorsChildEntity]:
+    """Return MySensors devices for a hass platform name."""
+    if MYSENSORS_PLATFORM_DEVICES.format(domain) not in hass.data[DOMAIN]:
+        hass.data[DOMAIN][MYSENSORS_PLATFORM_DEVICES.format(domain)] = {}
+    devices: dict[DevId, MySensorsChildEntity] = hass.data[DOMAIN][
+        MYSENSORS_PLATFORM_DEVICES.format(domain)
+    ]
+    return devices
+
+
+class MySensorsChildEntity(MySensorNodeEntity):
+    """Representation of a MySensors entity."""
+
+    _attr_should_poll = False
+
+    def __init__(
+        self,
+        gateway_id: GatewayId,
+        gateway: BaseAsyncGateway,
+        node_id: int,
+        child_id: int,
+        value_type: int,
+    ) -> None:
+        """Set up the MySensors child entity."""
+        super().__init__(gateway_id, gateway, node_id)
+        self.child_id: int = child_id
+        # value_type as int. string variant can be looked up in gateway consts
+        self.value_type: int = value_type
+        self.child_type = self._child.type
+        self._values: dict[int, Any] = {}
+
+    @property
+    def dev_id(self) -> DevId:
+        """Return the DevId of this device.
+
+        It is used to route incoming MySensors messages to the correct device/entity.
+        """
+        return self.gateway_id, self.node_id, self.child_id, self.value_type
+
+    @property
+    def _child(self) -> ChildSensor:
+        return self._node.children[self.child_id]
+
+    @property
+    def unique_id(self) -> str:
+        """Return a unique ID for use in home assistant."""
+        return f"{self.gateway_id}-{self.node_id}-{self.child_id}-{self.value_type}"
+
     @property
     def name(self) -> str:
         """Return the name of this entity."""
@@ -137,32 +192,45 @@ class MySensorsDevice:
             return str(child.description)
         return f"{self.node_name} {self.child_id}"
 
+    async def async_will_remove_from_hass(self) -> None:
+        """Remove this entity from home assistant."""
+        for platform in PLATFORM_TYPES:
+            platform_str = MYSENSORS_PLATFORM_DEVICES.format(platform)
+            if platform_str in self.hass.data[DOMAIN]:
+                platform_dict = self.hass.data[DOMAIN][platform_str]
+                if self.dev_id in platform_dict:
+                    del platform_dict[self.dev_id]
+                    _LOGGER.debug("Deleted %s from platform %s", self.dev_id, platform)
+
     @property
-    def _extra_attributes(self) -> dict[str, Any]:
-        """Return device specific attributes."""
-        node = self.gateway.sensors[self.node_id]
-        child = node.children[self.child_id]
-        attr = {
-            ATTR_BATTERY_LEVEL: node.battery_level,
-            ATTR_HEARTBEAT: node.heartbeat,
-            ATTR_CHILD_ID: self.child_id,
-            ATTR_DESCRIPTION: child.description,
-            ATTR_NODE_ID: self.node_id,
-        }
+    def available(self) -> bool:
+        """Return true if entity is available."""
+        return self.value_type in self._values
+
+    @property
+    def extra_state_attributes(self) -> dict[str, Any]:
+        """Return entity and device specific state attributes."""
+        attr = super().extra_state_attributes
+
+        assert self.platform.config_entry
+        attr[ATTR_DEVICE] = self.platform.config_entry.data[CONF_DEVICE]
+
+        attr[ATTR_CHILD_ID] = self.child_id
+        attr[ATTR_DESCRIPTION] = self._child.description
+        # We should deprecate the battery level attribute in the future.
+        attr[ATTR_BATTERY_LEVEL] = self._node.battery_level
 
         set_req = self.gateway.const.SetReq
-
         for value_type, value in self._values.items():
             attr[set_req(value_type).name] = value
 
         return attr
 
-    async def async_update(self) -> None:
+    @callback
+    def _async_update(self) -> None:
         """Update the controller with the latest value from a sensor."""
-        node = self.gateway.sensors[self.node_id]
-        child = node.children[self.child_id]
         set_req = self.gateway.const.SetReq
-        for value_type, value in child.values.items():
+        for value_type, value in self._child.values.items():
             _LOGGER.debug(
                 "Entity update: %s: value_type %s, value = %s",
                 self.name,
@@ -184,72 +252,15 @@ class MySensorsDevice:
             else:
                 self._values[value_type] = value
 
-    async def _async_update_callback(self) -> None:
-        """Update the device."""
-        raise NotImplementedError
-
     @callback
-    def async_update_callback(self) -> None:
-        """Update the device after delay."""
-        if self._update_scheduled:
-            return
-
-        async def update() -> None:
-            """Perform update."""
-            try:
-                await self._async_update_callback()
-            except Exception:  # pylint: disable=broad-except
-                _LOGGER.exception("Error updating %s", self.name)
-            finally:
-                self._update_scheduled = False
-
-        self._update_scheduled = True
-        delayed_update = partial(self.hass.async_create_task, update())
-        self.hass.loop.call_later(UPDATE_DELAY, delayed_update)
-
-
-def get_mysensors_devices(
-    hass: HomeAssistant, domain: str
-) -> dict[DevId, MySensorsDevice]:
-    """Return MySensors devices for a hass platform name."""
-    if MYSENSORS_PLATFORM_DEVICES.format(domain) not in hass.data[DOMAIN]:
-        hass.data[DOMAIN][MYSENSORS_PLATFORM_DEVICES.format(domain)] = {}
-    devices: dict[DevId, MySensorsDevice] = hass.data[DOMAIN][
-        MYSENSORS_PLATFORM_DEVICES.format(domain)
-    ]
-    return devices
-
-
-class MySensorsEntity(MySensorsDevice, Entity):
-    """Representation of a MySensors entity."""
-
-    @property
-    def should_poll(self) -> bool:
-        """Return the polling state. The gateway pushes its states."""
-        return False
-
-    @property
-    def available(self) -> bool:
-        """Return true if entity is available."""
-        return self.value_type in self._values
-
-    @property
-    def extra_state_attributes(self) -> dict[str, Any]:
-        """Return entity specific state attributes."""
-        attr = self._extra_attributes
-
-        assert self.platform
-        assert self.platform.config_entry
-        attr[ATTR_DEVICE] = self.platform.config_entry.data[CONF_DEVICE]
-
-        return attr
-
-    async def _async_update_callback(self) -> None:
+    def _async_update_callback(self) -> None:
         """Update the entity."""
-        await self.async_update_ha_state(True)
+        self._async_update()
+        self.async_write_ha_state()
 
     async def async_added_to_hass(self) -> None:
         """Register update callback."""
+        await super().async_added_to_hass()
         self.async_on_remove(
             async_dispatcher_connect(
                 self.hass,
@@ -257,10 +268,3 @@ class MySensorsEntity(MySensorsDevice, Entity):
                 self.async_update_callback,
             )
         )
-        self.async_on_remove(
-            async_dispatcher_connect(
-                self.hass,
-                NODE_CALLBACK.format(self.gateway_id, self.node_id),
-                self.async_update_callback,
-            )
-        )
