@@ -1,33 +1,33 @@
 """The Nettigo Air Monitor component."""
 from __future__ import annotations
 
+import asyncio
 import logging
 from typing import cast
 
-from aiohttp import ClientSession
-from aiohttp.client_exceptions import ClientConnectorError
-import async_timeout
+from aiohttp.client_exceptions import ClientConnectorError, ClientError
 from nettigo_air_monitor import (
     ApiError,
-    InvalidSensorData,
+    AuthFailedError,
+    ConnectionOptions,
+    InvalidSensorDataError,
     NAMSensors,
     NettigoAirMonitor,
 )
 
 from homeassistant.components.air_quality import DOMAIN as AIR_QUALITY_PLATFORM
 from homeassistant.config_entries import ConfigEntry
-from homeassistant.const import CONF_HOST
+from homeassistant.const import CONF_HOST, CONF_PASSWORD, CONF_USERNAME, Platform
 from homeassistant.core import HomeAssistant
-from homeassistant.helpers import entity_registry
+from homeassistant.exceptions import ConfigEntryAuthFailed, ConfigEntryNotReady
+from homeassistant.helpers import device_registry as dr, entity_registry as er
 from homeassistant.helpers.aiohttp_client import async_get_clientsession
-from homeassistant.helpers.device_registry import CONNECTION_NETWORK_MAC
-from homeassistant.helpers.entity import DeviceInfo
+from homeassistant.helpers.device_registry import DeviceInfo
 from homeassistant.helpers.update_coordinator import DataUpdateCoordinator, UpdateFailed
 
 from .const import (
     ATTR_SDS011,
     ATTR_SPS30,
-    DEFAULT_NAME,
     DEFAULT_UPDATE_INTERVAL,
     DOMAIN,
     MANUFACTURER,
@@ -35,25 +35,40 @@ from .const import (
 
 _LOGGER = logging.getLogger(__name__)
 
-PLATFORMS = ["sensor"]
+PLATFORMS = [Platform.BUTTON, Platform.SENSOR]
 
 
 async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
     """Set up Nettigo as config entry."""
     host: str = entry.data[CONF_HOST]
+    username: str | None = entry.data.get(CONF_USERNAME)
+    password: str | None = entry.data.get(CONF_PASSWORD)
 
     websession = async_get_clientsession(hass)
 
-    coordinator = NAMDataUpdateCoordinator(hass, websession, host, entry.unique_id)
+    options = ConnectionOptions(host=host, username=username, password=password)
+    try:
+        nam = await NettigoAirMonitor.create(websession, options)
+    except (ApiError, ClientError, ClientConnectorError, asyncio.TimeoutError) as err:
+        raise ConfigEntryNotReady from err
+
+    try:
+        await nam.async_check_credentials()
+    except ApiError as err:
+        raise ConfigEntryNotReady from err
+    except AuthFailedError as err:
+        raise ConfigEntryAuthFailed from err
+
+    coordinator = NAMDataUpdateCoordinator(hass, nam, entry.unique_id)
     await coordinator.async_config_entry_first_refresh()
 
     hass.data.setdefault(DOMAIN, {})
     hass.data[DOMAIN][entry.entry_id] = coordinator
 
-    hass.config_entries.async_setup_platforms(entry, PLATFORMS)
+    await hass.config_entries.async_forward_entry_setups(entry, PLATFORMS)
 
     # Remove air_quality entities from registry if they exist
-    ent_reg = entity_registry.async_get(hass)
+    ent_reg = er.async_get(hass)
     for sensor_type in ("sds", ATTR_SDS011, ATTR_SPS30):
         unique_id = f"{coordinator.unique_id}-{sensor_type}"
         if entity_id := ent_reg.async_get_entity_id(
@@ -75,20 +90,18 @@ async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
     return unload_ok
 
 
-class NAMDataUpdateCoordinator(DataUpdateCoordinator):
+class NAMDataUpdateCoordinator(DataUpdateCoordinator[NAMSensors]):  # pylint: disable=hass-enforce-coordinator-module
     """Class to manage fetching Nettigo Air Monitor data."""
 
     def __init__(
         self,
         hass: HomeAssistant,
-        session: ClientSession,
-        host: str,
+        nam: NettigoAirMonitor,
         unique_id: str | None,
     ) -> None:
         """Initialize."""
-        self.host = host
-        self.nam = NettigoAirMonitor(session, host)
         self._unique_id = unique_id
+        self.nam = nam
 
         super().__init__(
             hass, _LOGGER, name=DOMAIN, update_interval=DEFAULT_UPDATE_INTERVAL
@@ -97,12 +110,11 @@ class NAMDataUpdateCoordinator(DataUpdateCoordinator):
     async def _async_update_data(self) -> NAMSensors:
         """Update data via library."""
         try:
-            # Device firmware uses synchronous code and doesn't respond to http queries
-            # when reading data from sensors. The nettigo-air-quality library tries to
-            # get the data 4 times, so we use a longer than usual timeout here.
-            with async_timeout.timeout(30):
+            async with asyncio.timeout(10):
                 data = await self.nam.async_update()
-        except (ApiError, ClientConnectorError, InvalidSensorData) as error:
+        # We do not need to catch AuthFailed exception here because sensor data is
+        # always available without authorization.
+        except (ApiError, ClientConnectorError, InvalidSensorDataError) as error:
             raise UpdateFailed(error) from error
 
         return data
@@ -116,9 +128,9 @@ class NAMDataUpdateCoordinator(DataUpdateCoordinator):
     def device_info(self) -> DeviceInfo:
         """Return the device info."""
         return DeviceInfo(
-            connections={(CONNECTION_NETWORK_MAC, cast(str, self._unique_id))},
-            name=DEFAULT_NAME,
+            connections={(dr.CONNECTION_NETWORK_MAC, cast(str, self._unique_id))},
+            name="Nettigo Air Monitor",
             sw_version=self.nam.software_version,
             manufacturer=MANUFACTURER,
-            configuration_url=f"http://{self.host}/",
+            configuration_url=f"http://{self.nam.host}/",
         )
