@@ -8,15 +8,15 @@ from typing import Any
 from pybotvac.exceptions import NeatoRobotException
 from pybotvac.robot import Robot
 
-from homeassistant.components.neato import NeatoHub
-from homeassistant.components.sensor import DEVICE_CLASS_BATTERY, SensorEntity
+from homeassistant.components.sensor import SensorDeviceClass, SensorEntity
 from homeassistant.config_entries import ConfigEntry
-from homeassistant.const import ENTITY_CATEGORY_DIAGNOSTIC, PERCENTAGE
+from homeassistant.const import PERCENTAGE, EntityCategory
 from homeassistant.core import HomeAssistant
-from homeassistant.helpers.entity import DeviceInfo
 from homeassistant.helpers.entity_platform import AddEntitiesCallback
 
-from .const import NEATO_DOMAIN, NEATO_LOGIN, NEATO_ROBOTS, SCAN_INTERVAL_MINUTES
+from .const import NEATO_LOGIN, NEATO_ROBOTS, SCAN_INTERVAL_MINUTES
+from .entity import NeatoEntity
+from .hub import NeatoHub
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -41,15 +41,19 @@ async def async_setup_entry(
     async_add_entities(dev, True)
 
 
-class NeatoSensor(SensorEntity):
+class NeatoSensor(NeatoEntity, SensorEntity):
     """Neato sensor."""
 
+    _attr_device_class = SensorDeviceClass.BATTERY
+    _attr_entity_category = EntityCategory.DIAGNOSTIC
+    _attr_native_unit_of_measurement = PERCENTAGE
+    _attr_available: bool = False
+
     def __init__(self, neato: NeatoHub, robot: Robot) -> None:
         """Initialize Neato sensor."""
-        self.robot = robot
-        self._available: bool = False
-        self._robot_name: str = f"{self.robot.name} {BATTERY}"
+        super().__init__(robot)
         self._robot_serial: str = self.robot.serial
+        self._attr_unique_id = self.robot.serial
         self._state: dict[str, Any] | None = None
 
     def update(self) -> None:
@@ -57,55 +61,20 @@ class NeatoSensor(SensorEntity):
         try:
             self._state = self.robot.state
         except NeatoRobotException as ex:
-            if self._available:
+            if self._attr_available:
                 _LOGGER.error(
                     "Neato sensor connection error for '%s': %s", self.entity_id, ex
                 )
             self._state = None
-            self._available = False
+            self._attr_available = False
             return
 
-        self._available = True
+        self._attr_available = True
         _LOGGER.debug("self._state=%s", self._state)
 
-    @property
-    def name(self) -> str:
-        """Return the name of this sensor."""
-        return self._robot_name
-
-    @property
-    def unique_id(self) -> str:
-        """Return unique ID."""
-        return self._robot_serial
-
-    @property
-    def device_class(self) -> str:
-        """Return the device class."""
-        return DEVICE_CLASS_BATTERY
-
-    @property
-    def entity_category(self) -> str:
-        """Device entity category."""
-        return ENTITY_CATEGORY_DIAGNOSTIC
-
-    @property
-    def available(self) -> bool:
-        """Return availability."""
-        return self._available
-
     @property
     def native_value(self) -> str | None:
         """Return the state."""
         if self._state is not None:
             return str(self._state["details"]["charge"])
         return None
-
-    @property
-    def native_unit_of_measurement(self) -> str:
-        """Return unit of measurement."""
-        return PERCENTAGE
-
-    @property
-    def device_info(self) -> DeviceInfo:
-        """Device info for neato robot."""
-        return DeviceInfo(identifiers={(NEATO_DOMAIN, self._robot_serial)})
