@@ -22,7 +22,7 @@ from homeassistant.config_entries import ConfigEntry
 from homeassistant.const import ATTR_MODE, STATE_IDLE, STATE_PAUSED
 from homeassistant.core import HomeAssistant
 from homeassistant.helpers import config_validation as cv, entity_platform
-from homeassistant.helpers.entity import DeviceInfo
+from homeassistant.helpers.device_registry import DeviceInfo
 from homeassistant.helpers.entity_platform import AddEntitiesCallback
 
 from .const import (
@@ -30,13 +30,13 @@ from .const import (
     ALERTS,
     ERRORS,
     MODE,
-    NEATO_DOMAIN,
     NEATO_LOGIN,
     NEATO_MAP_DATA,
     NEATO_PERSISTENT_MAPS,
     NEATO_ROBOTS,
     SCAN_INTERVAL_MINUTES,
 )
+from .entity import NeatoEntity
 from .hub import NeatoHub
 
 _LOGGER = logging.getLogger(__name__)
@@ -91,7 +91,7 @@ async def async_setup_entry(
     )
 
 
-class NeatoConnectedVacuum(StateVacuumEntity):
+class NeatoConnectedVacuum(NeatoEntity, StateVacuumEntity):
     """Representation of a Neato Connected Vacuum."""
 
     _attr_icon = "mdi:robot-vacuum-variant"
@@ -106,6 +106,7 @@ class NeatoConnectedVacuum(StateVacuumEntity):
         | VacuumEntityFeature.MAP
         | VacuumEntityFeature.LOCATE
     )
+    _attr_name = None
 
     def __init__(
         self,
@@ -115,16 +116,14 @@ class NeatoConnectedVacuum(StateVacuumEntity):
         persistent_maps: dict[str, Any] | None,
     ) -> None:
         """Initialize the Neato Connected Vacuum."""
-        self.robot = robot
+        super().__init__(robot)
         self._attr_available: bool = neato is not None
         self._mapdata = mapdata
-        self._attr_name: str = self.robot.name
         self._robot_has_map: bool = self.robot.has_persistent_maps
         self._robot_maps = persistent_maps
         self._robot_serial: str = self.robot.serial
         self._attr_unique_id: str = self.robot.serial
         self._status_state: str | None = None
-        self._clean_state: str | None = None
         self._state: dict[str, Any] | None = None
         self._clean_time_start: str | None = None
         self._clean_time_stop: str | None = None
@@ -169,23 +168,23 @@ class NeatoConnectedVacuum(StateVacuumEntity):
             robot_alert = None
         if self._state["state"] == 1:
             if self._state["details"]["isCharging"]:
-                self._clean_state = STATE_DOCKED
+                self._attr_state = STATE_DOCKED
                 self._status_state = "Charging"
             elif (
                 self._state["details"]["isDocked"]
                 and not self._state["details"]["isCharging"]
             ):
-                self._clean_state = STATE_DOCKED
+                self._attr_state = STATE_DOCKED
                 self._status_state = "Docked"
             else:
-                self._clean_state = STATE_IDLE
+                self._attr_state = STATE_IDLE
                 self._status_state = "Stopped"
 
             if robot_alert is not None:
                 self._status_state = robot_alert
         elif self._state["state"] == 2:
             if robot_alert is None:
-                self._clean_state = STATE_CLEANING
+                self._attr_state = STATE_CLEANING
                 self._status_state = (
                     f"{MODE.get(self._state['cleaning']['mode'])} "
                     f"{ACTION.get(self._state['action'])}"
@@ -200,10 +199,10 @@ class NeatoConnectedVacuum(StateVacuumEntity):
             else:
                 self._status_state = robot_alert
         elif self._state["state"] == 3:
-            self._clean_state = STATE_PAUSED
+            self._attr_state = STATE_PAUSED
             self._status_state = "Paused"
         elif self._state["state"] == 4:
-            self._clean_state = STATE_ERROR
+            self._attr_state = STATE_ERROR
             self._status_state = ERRORS.get(self._state["error"])
 
         self._attr_battery_level = self._state["details"]["charge"]
@@ -261,11 +260,6 @@ class NeatoConnectedVacuum(StateVacuumEntity):
                         self._robot_boundaries,
                     )
 
-    @property
-    def state(self) -> str | None:
-        """Return the status of the vacuum cleaner."""
-        return self._clean_state
-
     @property
     def extra_state_attributes(self) -> dict[str, Any]:
         """Return the state attributes of the vacuum cleaner."""
@@ -299,14 +293,12 @@ class NeatoConnectedVacuum(StateVacuumEntity):
     @property
     def device_info(self) -> DeviceInfo:
         """Device info for neato robot."""
-        stats = self._robot_stats
-        return DeviceInfo(
-            identifiers={(NEATO_DOMAIN, self._robot_serial)},
-            manufacturer=stats["battery"]["vendor"] if stats else None,
-            model=stats["model"] if stats else None,
-            name=self._attr_name,
-            sw_version=stats["firmware"] if stats else None,
-        )
+        device_info = self._attr_device_info
+        if self._robot_stats:
+            device_info["manufacturer"] = self._robot_stats["battery"]["vendor"]
+            device_info["model"] = self._robot_stats["model"]
+            device_info["sw_version"] = self._robot_stats["firmware"]
+        return device_info
 
     def start(self) -> None:
         """Start cleaning or resume cleaning."""
@@ -333,9 +325,9 @@ class NeatoConnectedVacuum(StateVacuumEntity):
     def return_to_base(self, **kwargs: Any) -> None:
         """Set the vacuum cleaner to return to the dock."""
         try:
-            if self._clean_state == STATE_CLEANING:
+            if self._attr_state == STATE_CLEANING:
                 self.robot.pause_cleaning()
-            self._clean_state = STATE_RETURNING
+            self._attr_state = STATE_RETURNING
             self.robot.send_to_base()
         except NeatoRobotException as ex:
             _LOGGER.error(
@@ -385,7 +377,7 @@ class NeatoConnectedVacuum(StateVacuumEntity):
                 return
             _LOGGER.info("Start cleaning zone '%s' with robot %s", zone, self.entity_id)
 
-        self._clean_state = STATE_CLEANING
+        self._attr_state = STATE_CLEANING
         try:
             self.robot.start_cleaning(mode, navigation, category, boundary_id)
         except NeatoRobotException as ex:
