@@ -10,9 +10,14 @@ import logging
 from time import time
 from typing import Any
 
+import aiohttp
 import pyatmo
-from pyatmo.modules.device_types import DeviceCategory as NetatmoDeviceCategory
+from pyatmo.modules.device_types import (
+    DeviceCategory as NetatmoDeviceCategory,
+    DeviceType as NetatmoDeviceType,
+)
 
+from homeassistant.components import cloud
 from homeassistant.config_entries import ConfigEntry
 from homeassistant.core import CALLBACK_TYPE, HomeAssistant, callback
 from homeassistant.helpers.dispatcher import (
@@ -32,6 +37,7 @@ from .const import (
     NETATMO_CREATE_CAMERA_LIGHT,
     NETATMO_CREATE_CLIMATE,
     NETATMO_CREATE_COVER,
+    NETATMO_CREATE_FAN,
     NETATMO_CREATE_LIGHT,
     NETATMO_CREATE_ROOM_SENSOR,
     NETATMO_CREATE_SELECT,
@@ -52,7 +58,7 @@ ACCOUNT = "account"
 HOME = "home"
 WEATHER = "weather"
 AIR_CARE = "air_care"
-PUBLIC = "public"
+PUBLIC = NetatmoDeviceType.public
 EVENT = "event"
 
 PUBLISHERS = {
@@ -65,6 +71,10 @@ PUBLISHERS = {
 }
 
 BATCH_SIZE = 3
+DEV_FACTOR = 7
+DEV_LIMIT = 400
+CLOUD_FACTOR = 2
+CLOUD_LIMIT = 150
 DEFAULT_INTERVALS = {
     ACCOUNT: 10800,
     HOME: 300,
@@ -122,6 +132,7 @@ class NetatmoDataHandler:
     """Manages the Netatmo data handling."""
 
     account: pyatmo.AsyncAccount
+    _interval_factor: int
 
     def __init__(self, hass: HomeAssistant, config_entry: ConfigEntry) -> None:
         """Initialize self."""
@@ -131,11 +142,21 @@ class NetatmoDataHandler:
         self.publisher: dict[str, NetatmoPublisher] = {}
         self._queue: deque = deque()
         self._webhook: bool = False
+        if config_entry.data["auth_implementation"] == cloud.DOMAIN:
+            self._interval_factor = CLOUD_FACTOR
+            self._rate_limit = CLOUD_LIMIT
+        else:
+            self._interval_factor = DEV_FACTOR
+            self._rate_limit = DEV_LIMIT
+        self.poll_start = time()
+        self.poll_count = 0
 
     async def async_setup(self) -> None:
         """Set up the Netatmo data handler."""
-        async_track_time_interval(
-            self.hass, self.async_update, timedelta(seconds=SCAN_INTERVAL)
+        self.config_entry.async_on_unload(
+            async_track_time_interval(
+                self.hass, self.async_update, timedelta(seconds=SCAN_INTERVAL)
+            )
         )
 
         self.config_entry.async_on_unload(
@@ -156,28 +177,40 @@ class NetatmoDataHandler:
         await self.async_dispatch()
 
     async def async_update(self, event_time: datetime) -> None:
-        """
-        Update device.
+        """Update device.
 
         We do up to BATCH_SIZE calls in one update in order
         to minimize the calls on the api service.
         """
-        for data_class in islice(self._queue, 0, BATCH_SIZE):
+        for data_class in islice(self._queue, 0, BATCH_SIZE * self._interval_factor):
             if data_class.next_scan > time():
                 continue
 
             if publisher := data_class.name:
-                self.publisher[publisher].next_scan = time() + data_class.interval
+                error = await self.async_fetch_data(publisher)
 
-                await self.async_fetch_data(publisher)
+                if error:
+                    self.publisher[publisher].next_scan = (
+                        time() + data_class.interval * 10
+                    )
+                else:
+                    self.publisher[publisher].next_scan = time() + data_class.interval
 
         self._queue.rotate(BATCH_SIZE)
+        cph = self.poll_count / (time() - self.poll_start) * 3600
+        _LOGGER.debug("Calls per hour: %i", cph)
+        if cph > self._rate_limit:
+            for publisher in self.publisher.values():
+                publisher.next_scan += 60
+        if (time() - self.poll_start) > 3600:
+            self.poll_start = time()
+            self.poll_count = 0
 
     @callback
     def async_force_update(self, signal_name: str) -> None:
         """Prioritize data retrieval for given data class entry."""
-        # self.publisher[signal_name].next_scan = time()
-        # self._queue.rotate(-(self._queue.index(self.publisher[signal_name])))
+        self.publisher[signal_name].next_scan = time()
+        self._queue.rotate(-(self._queue.index(self.publisher[signal_name])))
 
     async def handle_event(self, event: dict) -> None:
         """Handle webhook events."""
@@ -193,27 +226,29 @@ class NetatmoDataHandler:
             _LOGGER.debug("%s camera reconnected", MANUFACTURER)
             self.async_force_update(ACCOUNT)
 
-    async def async_fetch_data(self, signal_name: str) -> None:
+    async def async_fetch_data(self, signal_name: str) -> bool:
         """Fetch data and notify."""
+        self.poll_count += 1
+        has_error = False
         try:
             await getattr(self.account, self.publisher[signal_name].method)(
                 **self.publisher[signal_name].kwargs
             )
 
-        except pyatmo.NoDevice as err:
+        except (pyatmo.NoDevice, pyatmo.ApiError) as err:
             _LOGGER.debug(err)
+            has_error = True
 
-        except pyatmo.ApiError as err:
+        except (asyncio.TimeoutError, aiohttp.ClientConnectorError) as err:
             _LOGGER.debug(err)
-
-        except asyncio.TimeoutError as err:
-            _LOGGER.debug(err)
-            return
+            return True
 
         for update_callback in self.publisher[signal_name].subscriptions:
             if update_callback:
                 update_callback()
 
+        return has_error
+
     async def subscribe(
         self,
         publisher: str,
@@ -230,10 +265,11 @@ class NetatmoDataHandler:
         if publisher == "public":
             kwargs = {"area_id": self.account.register_public_weather_area(**kwargs)}
 
+        interval = int(DEFAULT_INTERVALS[publisher] / self._interval_factor)
         self.publisher[signal_name] = NetatmoPublisher(
             name=signal_name,
-            interval=DEFAULT_INTERVALS[publisher],
-            next_scan=time() + DEFAULT_INTERVALS[publisher],
+            interval=interval,
+            next_scan=time() + interval,
             subscriptions={update_callback},
             method=PUBLISHERS[publisher],
             kwargs=kwargs,
@@ -252,7 +288,7 @@ class NetatmoDataHandler:
         self, signal_name: str, update_callback: CALLBACK_TYPE | None
     ) -> None:
         """Unsubscribe from publisher."""
-        if update_callback in self.publisher[signal_name].subscriptions:
+        if update_callback not in self.publisher[signal_name].subscriptions:
             return
 
         self.publisher[signal_name].subscriptions.remove(update_callback)
@@ -288,6 +324,9 @@ class NetatmoDataHandler:
                 person.entity_id: person.pseudo for person in home.persons.values()
             }
 
+        await self.unsubscribe(WEATHER, None)
+        await self.unsubscribe(AIR_CARE, None)
+
     def setup_air_care(self) -> None:
         """Set up home coach/air care modules."""
         for module in self.account.modules.values():
@@ -318,6 +357,7 @@ class NetatmoDataHandler:
                 NETATMO_CREATE_SENSOR,
             ],
             NetatmoDeviceCategory.meter: [NETATMO_CREATE_SENSOR],
+            NetatmoDeviceCategory.fan: [NETATMO_CREATE_FAN],
         }
         for module in home.modules.values():
             if not module.device_category:
