@@ -1,15 +1,28 @@
 """Onboarding views."""
-import asyncio
+from __future__ import annotations
 
+import asyncio
+from http import HTTPStatus
+from typing import TYPE_CHECKING, Any, cast
+
+from aiohttp import web
+from aiohttp.web_exceptions import HTTPUnauthorized
 import voluptuous as vol
 
 from homeassistant.auth.const import GROUP_ID_ADMIN
+from homeassistant.auth.providers.homeassistant import HassAuthProvider
+from homeassistant.components import person
 from homeassistant.components.auth import indieauth
-from homeassistant.components.http.const import KEY_HASS_REFRESH_TOKEN_ID
+from homeassistant.components.http import KEY_HASS_REFRESH_TOKEN_ID
 from homeassistant.components.http.data_validator import RequestDataValidator
 from homeassistant.components.http.view import HomeAssistantView
-from homeassistant.const import HTTP_BAD_REQUEST, HTTP_FORBIDDEN
-from homeassistant.core import callback
+from homeassistant.core import HomeAssistant, callback
+from homeassistant.helpers import area_registry as ar
+from homeassistant.helpers.system_info import async_get_system_info
+from homeassistant.helpers.translation import async_get_translations
+
+if TYPE_CHECKING:
+    from . import OnboadingStorage
 
 from .const import (
     DEFAULT_AREAS,
@@ -22,9 +35,12 @@ from .const import (
 )
 
 
-async def async_setup(hass, data, store):
+async def async_setup(
+    hass: HomeAssistant, data: dict[str, list[str]], store: OnboadingStorage
+) -> None:
     """Set up the onboarding view."""
     hass.http.register_view(OnboardingView(data, store))
+    hass.http.register_view(InstallationTypeOnboardingView(data))
     hass.http.register_view(UserOnboardingView(data, store))
     hass.http.register_view(CoreConfigOnboardingView(data, store))
     hass.http.register_view(IntegrationOnboardingView(data, store))
@@ -38,35 +54,56 @@ class OnboardingView(HomeAssistantView):
     url = "/api/onboarding"
     name = "api:onboarding"
 
-    def __init__(self, data, store):
+    def __init__(self, data: dict[str, list[str]], store: OnboadingStorage) -> None:
         """Initialize the onboarding view."""
         self._store = store
         self._data = data
 
-    async def get(self, request):
+    async def get(self, request: web.Request) -> web.Response:
         """Return the onboarding status."""
         return self.json(
             [{"step": key, "done": key in self._data["done"]} for key in STEPS]
         )
 
 
+class InstallationTypeOnboardingView(HomeAssistantView):
+    """Return the installation type during onboarding."""
+
+    requires_auth = False
+    url = "/api/onboarding/installation_type"
+    name = "api:onboarding:installation_type"
+
+    def __init__(self, data: dict[str, list[str]]) -> None:
+        """Initialize the onboarding installation type view."""
+        self._data = data
+
+    async def get(self, request: web.Request) -> web.Response:
+        """Return the onboarding status."""
+        if self._data["done"]:
+            raise HTTPUnauthorized()
+
+        hass: HomeAssistant = request.app["hass"]
+        info = await async_get_system_info(hass)
+        return self.json({"installation_type": info["installation_type"]})
+
+
 class _BaseOnboardingView(HomeAssistantView):
     """Base class for onboarding."""
 
-    step = None
+    step: str
 
-    def __init__(self, data, store):
+    def __init__(self, data: dict[str, list[str]], store: OnboadingStorage) -> None:
         """Initialize the onboarding view."""
         self._store = store
         self._data = data
         self._lock = asyncio.Lock()
 
     @callback
-    def _async_is_done(self):
+    def _async_is_done(self) -> bool:
         """Return if this step is done."""
         return self.step in self._data["done"]
 
-    async def _async_mark_done(self, hass):
+    async def _async_mark_done(self, hass: HomeAssistant) -> None:
         """Mark step as done."""
         self._data["done"].append(self.step)
         await self._store.async_save(self._data)
@@ -94,50 +131,50 @@ class UserOnboardingView(_BaseOnboardingView):
             }
         )
     )
-    async def post(self, request, data):
+    async def post(self, request: web.Request, data: dict[str, str]) -> web.Response:
         """Handle user creation, area creation."""
-        hass = request.app["hass"]
+        hass: HomeAssistant = request.app["hass"]
 
         async with self._lock:
             if self._async_is_done():
-                return self.json_message("User step already done", HTTP_FORBIDDEN)
+                return self.json_message("User step already done", HTTPStatus.FORBIDDEN)
 
             provider = _async_get_hass_provider(hass)
             await provider.async_initialize()
 
-            user = await hass.auth.async_create_user(data["name"], [GROUP_ID_ADMIN])
-            await hass.async_add_executor_job(
-                provider.data.add_auth, data["username"], data["password"]
+            user = await hass.auth.async_create_user(
+                data["name"], group_ids=[GROUP_ID_ADMIN]
             )
+            await provider.async_add_auth(data["username"], data["password"])
             credentials = await provider.async_get_or_create_credentials(
                 {"username": data["username"]}
             )
-            await provider.data.async_save()
             await hass.auth.async_link_user(user, credentials)
             if "person" in hass.config.components:
-                await hass.components.person.async_create_person(
-                    data["name"], user_id=user.id
-                )
+                await person.async_create_person(hass, data["name"], user_id=user.id)
 
             # Create default areas using the users supplied language.
-            translations = await hass.helpers.translation.async_get_translations(
-                data["language"], "area", DOMAIN
+            translations = await async_get_translations(
+                hass, data["language"], "area", {DOMAIN}
             )
 
-            area_registry = await hass.helpers.area_registry.async_get_registry()
+            area_registry = ar.async_get(hass)
 
             for area in DEFAULT_AREAS:
-                area_registry.async_create(
-                    translations[f"component.onboarding.area.{area}"]
-                )
+                name = translations[f"component.onboarding.area.{area}"]
+                # Guard because area might have been created by an automatically
+                # set up integration.
+                if not area_registry.async_get_area_by_name(name):
+                    area_registry.async_create(name)
 
             await self._async_mark_done(hass)
 
             # Return authorization code for fetching tokens and connect
             # during onboarding.
-            auth_code = hass.components.auth.create_auth_code(
-                data["client_id"], credentials
-            )
+            # pylint: disable-next=import-outside-toplevel
+            from homeassistant.components.auth import create_auth_code
+
+            auth_code = create_auth_code(hass, data["client_id"], credentials)
             return self.json({"auth_code": auth_code})
 
 
@@ -148,29 +185,45 @@ class CoreConfigOnboardingView(_BaseOnboardingView):
     name = "api:onboarding:core_config"
     step = STEP_CORE_CONFIG
 
-    async def post(self, request):
+    async def post(self, request: web.Request) -> web.Response:
         """Handle finishing core config step."""
-        hass = request.app["hass"]
+        hass: HomeAssistant = request.app["hass"]
 
         async with self._lock:
             if self._async_is_done():
                 return self.json_message(
-                    "Core config step already done", HTTP_FORBIDDEN
+                    "Core config step already done", HTTPStatus.FORBIDDEN
                 )
 
             await self._async_mark_done(hass)
 
-            await hass.config_entries.flow.async_init(
-                "met", context={"source": "onboarding"}
-            )
+            # Integrations to set up when finishing onboarding
+            onboard_integrations = [
+                "google_translate",
+                "met",
+                "radio_browser",
+                "shopping_list",
+            ]
+
+            # pylint: disable-next=import-outside-toplevel
+            from homeassistant.components import hassio
 
             if (
-                hass.components.hassio.is_hassio()
-                and "raspberrypi" in hass.components.hassio.get_core_info()["machine"]
+                hassio.is_hassio(hass)
+                and (core_info := hassio.get_core_info(hass))
+                and "raspberrypi" in core_info["machine"]
             ):
-                await hass.config_entries.flow.async_init(
-                    "rpi_power", context={"source": "onboarding"}
+                onboard_integrations.append("rpi_power")
+
+            # Set up integrations after onboarding
+            await asyncio.gather(
+                *(
+                    hass.config_entries.flow.async_init(
+                        domain, context={"source": "onboarding"}
+                    )
+                    for domain in onboard_integrations
                 )
+            )
 
             return self.json({})
 
@@ -185,15 +238,15 @@ class IntegrationOnboardingView(_BaseOnboardingView):
     @RequestDataValidator(
         vol.Schema({vol.Required("client_id"): str, vol.Required("redirect_uri"): str})
     )
-    async def post(self, request, data):
+    async def post(self, request: web.Request, data: dict[str, Any]) -> web.Response:
         """Handle token creation."""
-        hass = request.app["hass"]
+        hass: HomeAssistant = request.app["hass"]
         refresh_token_id = request[KEY_HASS_REFRESH_TOKEN_ID]
 
         async with self._lock:
             if self._async_is_done():
                 return self.json_message(
-                    "Integration step already done", HTTP_FORBIDDEN
+                    "Integration step already done", HTTPStatus.FORBIDDEN
                 )
 
             await self._async_mark_done(hass)
@@ -203,18 +256,21 @@ class IntegrationOnboardingView(_BaseOnboardingView):
                 request.app["hass"], data["client_id"], data["redirect_uri"]
             ):
                 return self.json_message(
-                    "invalid client id or redirect uri", HTTP_BAD_REQUEST
+                    "invalid client id or redirect uri", HTTPStatus.BAD_REQUEST
                 )
 
-            refresh_token = await hass.auth.async_get_refresh_token(refresh_token_id)
+            refresh_token = hass.auth.async_get_refresh_token(refresh_token_id)
             if refresh_token is None or refresh_token.credential is None:
                 return self.json_message(
-                    "Credentials for user not available", HTTP_FORBIDDEN
+                    "Credentials for user not available", HTTPStatus.FORBIDDEN
                 )
 
             # Return authorization code so we can redirect user and log them in
-            auth_code = hass.components.auth.create_auth_code(
-                data["client_id"], refresh_token.credential
+            # pylint: disable-next=import-outside-toplevel
+            from homeassistant.components.auth import create_auth_code
+
+            auth_code = create_auth_code(
+                hass, data["client_id"], refresh_token.credential
             )
             return self.json({"auth_code": auth_code})
 
@@ -226,14 +282,14 @@ class AnalyticsOnboardingView(_BaseOnboardingView):
     name = "api:onboarding:analytics"
     step = STEP_ANALYTICS
 
-    async def post(self, request):
+    async def post(self, request: web.Request) -> web.Response:
         """Handle finishing analytics step."""
-        hass = request.app["hass"]
+        hass: HomeAssistant = request.app["hass"]
 
         async with self._lock:
             if self._async_is_done():
                 return self.json_message(
-                    "Analytics config step already done", HTTP_FORBIDDEN
+                    "Analytics config step already done", HTTPStatus.FORBIDDEN
                 )
 
             await self._async_mark_done(hass)
@@ -242,10 +298,10 @@ class AnalyticsOnboardingView(_BaseOnboardingView):
 
 
 @callback
-def _async_get_hass_provider(hass):
+def _async_get_hass_provider(hass: HomeAssistant) -> HassAuthProvider:
     """Get the Home Assistant auth provider."""
     for prv in hass.auth.auth_providers:
         if prv.type == "homeassistant":
-            return prv
+            return cast(HassAuthProvider, prv)
 
     raise RuntimeError("No Home Assistant provider found")
