@@ -3,6 +3,7 @@ from __future__ import annotations
 
 from datetime import timedelta
 import logging
+from typing import Any
 
 from ondilo import OndiloError
 
@@ -15,12 +16,12 @@ from homeassistant.components.sensor import (
 from homeassistant.config_entries import ConfigEntry
 from homeassistant.const import (
     CONCENTRATION_PARTS_PER_MILLION,
-    ELECTRIC_POTENTIAL_MILLIVOLT,
     PERCENTAGE,
-    TEMP_CELSIUS,
+    UnitOfElectricPotential,
+    UnitOfTemperature,
 )
 from homeassistant.core import HomeAssistant
-from homeassistant.helpers.entity import DeviceInfo
+from homeassistant.helpers.device_registry import DeviceInfo
 from homeassistant.helpers.entity_platform import AddEntitiesCallback
 from homeassistant.helpers.update_coordinator import (
     CoordinatorEntity,
@@ -28,62 +29,54 @@ from homeassistant.helpers.update_coordinator import (
     UpdateFailed,
 )
 
+from .api import OndiloClient
 from .const import DOMAIN
 
 SENSOR_TYPES: tuple[SensorEntityDescription, ...] = (
     SensorEntityDescription(
         key="temperature",
-        name="Temperature",
-        native_unit_of_measurement=TEMP_CELSIUS,
-        icon=None,
+        native_unit_of_measurement=UnitOfTemperature.CELSIUS,
         device_class=SensorDeviceClass.TEMPERATURE,
         state_class=SensorStateClass.MEASUREMENT,
     ),
     SensorEntityDescription(
         key="orp",
-        name="Oxydo Reduction Potential",
-        native_unit_of_measurement=ELECTRIC_POTENTIAL_MILLIVOLT,
+        translation_key="oxydo_reduction_potential",
+        native_unit_of_measurement=UnitOfElectricPotential.MILLIVOLT,
         icon="mdi:pool",
-        device_class=None,
         state_class=SensorStateClass.MEASUREMENT,
     ),
     SensorEntityDescription(
         key="ph",
-        name="pH",
+        translation_key="ph",
         icon="mdi:pool",
-        device_class=None,
         state_class=SensorStateClass.MEASUREMENT,
     ),
     SensorEntityDescription(
         key="tds",
-        name="TDS",
+        translation_key="tds",
         native_unit_of_measurement=CONCENTRATION_PARTS_PER_MILLION,
         icon="mdi:pool",
-        device_class=None,
         state_class=SensorStateClass.MEASUREMENT,
     ),
     SensorEntityDescription(
         key="battery",
-        name="Battery",
         native_unit_of_measurement=PERCENTAGE,
-        icon=None,
         device_class=SensorDeviceClass.BATTERY,
         state_class=SensorStateClass.MEASUREMENT,
     ),
     SensorEntityDescription(
         key="rssi",
-        name="RSSI",
+        translation_key="rssi",
+        icon="mdi:wifi",
         native_unit_of_measurement=PERCENTAGE,
-        icon=None,
-        device_class=SensorDeviceClass.SIGNAL_STRENGTH,
         state_class=SensorStateClass.MEASUREMENT,
     ),
     SensorEntityDescription(
         key="salt",
-        name="Salt",
+        translation_key="salt",
         native_unit_of_measurement="mg/L",
         icon="mdi:pool",
-        device_class=None,
         state_class=SensorStateClass.MEASUREMENT,
     ),
 )
@@ -98,9 +91,9 @@ async def async_setup_entry(
 ) -> None:
     """Set up the Ondilo ICO sensors."""
 
-    api = hass.data[DOMAIN][entry.entry_id]
+    api: OndiloClient = hass.data[DOMAIN][entry.entry_id]
 
-    async def async_update_data():
+    async def async_update_data() -> list[dict[str, Any]]:
         """Fetch data from API endpoint.
 
         This is the place to pre-process the data to lookup tables
@@ -139,12 +132,16 @@ async def async_setup_entry(
     async_add_entities(entities)
 
 
-class OndiloICO(CoordinatorEntity, SensorEntity):
+class OndiloICO(
+    CoordinatorEntity[DataUpdateCoordinator[list[dict[str, Any]]]], SensorEntity
+):
     """Representation of a Sensor."""
 
+    _attr_has_entity_name = True
+
     def __init__(
         self,
-        coordinator: DataUpdateCoordinator,
+        coordinator: DataUpdateCoordinator[list[dict[str, Any]]],
         poolidx: int,
         description: SensorEntityDescription,
     ) -> None:
@@ -156,8 +153,13 @@ class OndiloICO(CoordinatorEntity, SensorEntity):
 
         pooldata = self._pooldata()
         self._attr_unique_id = f"{pooldata['ICO']['serial_number']}-{description.key}"
-        self._device_name = pooldata["name"]
-        self._attr_name = f"{self._device_name} {description.name}"
+        self._attr_device_info = DeviceInfo(
+            identifiers={(DOMAIN, pooldata["ICO"]["serial_number"])},
+            manufacturer="Ondilo",
+            model="ICO",
+            name=pooldata["name"],
+            sw_version=pooldata["ICO"]["sw_version"],
+        )
 
     def _pooldata(self):
         """Get pool data dict."""
@@ -181,15 +183,3 @@ class OndiloICO(CoordinatorEntity, SensorEntity):
     def native_value(self):
         """Last value of the sensor."""
         return self._devdata()["value"]
-
-    @property
-    def device_info(self) -> DeviceInfo:
-        """Return the device info for the sensor."""
-        pooldata = self._pooldata()
-        return DeviceInfo(
-            identifiers={(DOMAIN, pooldata["ICO"]["serial_number"])},
-            manufacturer="Ondilo",
-            model="ICO",
-            name=self._device_name,
-            sw_version=pooldata["ICO"]["sw_version"],
-        )
