@@ -2,8 +2,7 @@
 from __future__ import annotations
 
 from collections.abc import Callable, Mapping
-import copy
-from dataclasses import dataclass
+import dataclasses
 import logging
 import os
 from types import MappingProxyType
@@ -43,7 +42,7 @@ from .onewire_entities import OneWireEntity, OneWireEntityDescription
 from .onewirehub import OneWireHub
 
 
-@dataclass
+@dataclasses.dataclass(frozen=True)
 class OneWireSensorEntityDescription(OneWireEntityDescription, SensorEntityDescription):
     """Class describing OneWire sensor entities."""
 
@@ -73,7 +72,6 @@ SIMPLE_TEMPERATURE_SENSOR_DESCRIPTION = OneWireSensorEntityDescription(
     native_unit_of_measurement=UnitOfTemperature.CELSIUS,
     read_mode=READ_MODE_FLOAT,
     state_class=SensorStateClass.MEASUREMENT,
-    translation_key="temperature",
 )
 
 _LOGGER = logging.getLogger(__name__)
@@ -89,7 +87,6 @@ DEVICE_SENSORS: dict[str, tuple[OneWireSensorEntityDescription, ...]] = {
             native_unit_of_measurement=UnitOfTemperature.CELSIUS,
             read_mode=READ_MODE_FLOAT,
             state_class=SensorStateClass.MEASUREMENT,
-            translation_key="temperature",
         ),
         OneWireSensorEntityDescription(
             key="TAI8570/pressure",
@@ -98,7 +95,6 @@ DEVICE_SENSORS: dict[str, tuple[OneWireSensorEntityDescription, ...]] = {
             native_unit_of_measurement=UnitOfPressure.MBAR,
             read_mode=READ_MODE_FLOAT,
             state_class=SensorStateClass.MEASUREMENT,
-            translation_key="pressure",
         ),
     ),
     "22": (SIMPLE_TEMPERATURE_SENSOR_DESCRIPTION,),
@@ -111,7 +107,6 @@ DEVICE_SENSORS: dict[str, tuple[OneWireSensorEntityDescription, ...]] = {
             native_unit_of_measurement=PERCENTAGE,
             read_mode=READ_MODE_FLOAT,
             state_class=SensorStateClass.MEASUREMENT,
-            translation_key="humidity",
         ),
         OneWireSensorEntityDescription(
             key="HIH3600/humidity",
@@ -156,7 +151,6 @@ DEVICE_SENSORS: dict[str, tuple[OneWireSensorEntityDescription, ...]] = {
             native_unit_of_measurement=UnitOfPressure.MBAR,
             read_mode=READ_MODE_FLOAT,
             state_class=SensorStateClass.MEASUREMENT,
-            translation_key="pressure",
         ),
         OneWireSensorEntityDescription(
             key="S3-R1-A/illuminance",
@@ -165,7 +159,6 @@ DEVICE_SENSORS: dict[str, tuple[OneWireSensorEntityDescription, ...]] = {
             native_unit_of_measurement=LIGHT_LUX,
             read_mode=READ_MODE_FLOAT,
             state_class=SensorStateClass.MEASUREMENT,
-            translation_key="illuminance",
         ),
         OneWireSensorEntityDescription(
             key="VAD",
@@ -203,7 +196,6 @@ DEVICE_SENSORS: dict[str, tuple[OneWireSensorEntityDescription, ...]] = {
             override_key=_get_sensor_precision_family_28,
             read_mode=READ_MODE_FLOAT,
             state_class=SensorStateClass.MEASUREMENT,
-            translation_key="temperature",
         ),
     ),
     "30": (
@@ -225,7 +217,6 @@ DEVICE_SENSORS: dict[str, tuple[OneWireSensorEntityDescription, ...]] = {
             native_unit_of_measurement=UnitOfElectricPotential.VOLT,
             read_mode=READ_MODE_FLOAT,
             state_class=SensorStateClass.MEASUREMENT,
-            translation_key="voltage",
         ),
         OneWireSensorEntityDescription(
             key="vis",
@@ -261,7 +252,6 @@ HOBBYBOARD_EF: dict[str, tuple[OneWireSensorEntityDescription, ...]] = {
             native_unit_of_measurement=PERCENTAGE,
             read_mode=READ_MODE_FLOAT,
             state_class=SensorStateClass.MEASUREMENT,
-            translation_key="humidity",
         ),
         OneWireSensorEntityDescription(
             key="humidity/humidity_raw",
@@ -277,7 +267,6 @@ HOBBYBOARD_EF: dict[str, tuple[OneWireSensorEntityDescription, ...]] = {
             native_unit_of_measurement=UnitOfTemperature.CELSIUS,
             read_mode=READ_MODE_FLOAT,
             state_class=SensorStateClass.MEASUREMENT,
-            translation_key="temperature",
         ),
     ),
     "HB_MOISTURE_METER": tuple(
@@ -303,7 +292,6 @@ EDS_SENSORS: dict[str, tuple[OneWireSensorEntityDescription, ...]] = {
             native_unit_of_measurement=UnitOfTemperature.CELSIUS,
             read_mode=READ_MODE_FLOAT,
             state_class=SensorStateClass.MEASUREMENT,
-            translation_key="temperature",
         ),
         OneWireSensorEntityDescription(
             key="EDS0066/pressure",
@@ -311,7 +299,6 @@ EDS_SENSORS: dict[str, tuple[OneWireSensorEntityDescription, ...]] = {
             native_unit_of_measurement=UnitOfPressure.MBAR,
             read_mode=READ_MODE_FLOAT,
             state_class=SensorStateClass.MEASUREMENT,
-            translation_key="pressure",
         ),
     ),
     "EDS0068": (
@@ -321,7 +308,6 @@ EDS_SENSORS: dict[str, tuple[OneWireSensorEntityDescription, ...]] = {
             native_unit_of_measurement=UnitOfTemperature.CELSIUS,
             read_mode=READ_MODE_FLOAT,
             state_class=SensorStateClass.MEASUREMENT,
-            translation_key="temperature",
         ),
         OneWireSensorEntityDescription(
             key="EDS0068/pressure",
@@ -329,7 +315,6 @@ EDS_SENSORS: dict[str, tuple[OneWireSensorEntityDescription, ...]] = {
             native_unit_of_measurement=UnitOfPressure.MBAR,
             read_mode=READ_MODE_FLOAT,
             state_class=SensorStateClass.MEASUREMENT,
-            translation_key="pressure",
         ),
         OneWireSensorEntityDescription(
             key="EDS0068/light",
@@ -337,7 +322,6 @@ EDS_SENSORS: dict[str, tuple[OneWireSensorEntityDescription, ...]] = {
             native_unit_of_measurement=LIGHT_LUX,
             read_mode=READ_MODE_FLOAT,
             state_class=SensorStateClass.MEASUREMENT,
-            translation_key="illuminance",
         ),
         OneWireSensorEntityDescription(
             key="EDS0068/humidity",
@@ -345,7 +329,6 @@ EDS_SENSORS: dict[str, tuple[OneWireSensorEntityDescription, ...]] = {
             native_unit_of_measurement=PERCENTAGE,
             read_mode=READ_MODE_FLOAT,
             state_class=SensorStateClass.MEASUREMENT,
-            translation_key="humidity",
         ),
     ),
 }
@@ -409,11 +392,12 @@ def get_entities(
                     ).decode()
                 )
                 if is_leaf:
-                    description = copy.deepcopy(description)
-                    description.device_class = SensorDeviceClass.HUMIDITY
-                    description.native_unit_of_measurement = PERCENTAGE
-                    description.translation_key = f"wetness_{s_id}"
-                    _LOGGER.info(description.translation_key)
+                    description = dataclasses.replace(
+                        description,
+                        device_class=SensorDeviceClass.HUMIDITY,
+                        native_unit_of_measurement=PERCENTAGE,
+                        translation_key=f"wetness_{s_id}",
+                    )
             override_key = None
             if description.override_key:
                 override_key = description.override_key(device_id, options)
