@@ -7,10 +7,11 @@ from homeassistant.components.binary_sensor import BinarySensorDeviceClass
 from homeassistant.components.sensor import SensorDeviceClass
 from homeassistant.const import (
     PERCENTAGE,
-    PRESSURE_BAR,
-    TEMP_CELSIUS,
-    TIME_HOURS,
-    TIME_MINUTES,
+    UnitOfPower,
+    UnitOfPressure,
+    UnitOfTemperature,
+    UnitOfTime,
+    UnitOfVolume,
 )
 
 ATTR_GW_ID = "gateway_id"
@@ -50,8 +51,7 @@ TRANSLATE_SOURCE = {
     gw_vars.THERMOSTAT: "Thermostat",
 }
 
-UNIT_KW = "kW"
-UNIT_L_MIN = f"L/{TIME_MINUTES}"
+SENSOR_FLOAT_SUGGESTED_DISPLAY_PRECISION = 1
 
 BINARY_SENSOR_INFO: dict[str, list] = {
     # [device_class, friendly_name format, [status source, ...]]
@@ -216,427 +216,453 @@ BINARY_SENSOR_INFO: dict[str, list] = {
 }
 
 SENSOR_INFO: dict[str, list] = {
-    # [device_class, unit, friendly_name, [status source, ...]]
+    # [device_class, unit, friendly_name, suggested_display_precision, [status source, ...]]
     gw_vars.DATA_CONTROL_SETPOINT: [
         SensorDeviceClass.TEMPERATURE,
-        TEMP_CELSIUS,
+        UnitOfTemperature.CELSIUS,
         "Control Setpoint {}",
+        SENSOR_FLOAT_SUGGESTED_DISPLAY_PRECISION,
         [gw_vars.BOILER, gw_vars.THERMOSTAT],
     ],
     gw_vars.DATA_MASTER_MEMBERID: [
         None,
         None,
         "Thermostat Member ID {}",
+        None,
         [gw_vars.BOILER, gw_vars.THERMOSTAT],
     ],
     gw_vars.DATA_SLAVE_MEMBERID: [
         None,
         None,
         "Boiler Member ID {}",
+        None,
         [gw_vars.BOILER, gw_vars.THERMOSTAT],
     ],
     gw_vars.DATA_SLAVE_OEM_FAULT: [
         None,
         None,
         "Boiler OEM Fault Code {}",
+        None,
         [gw_vars.BOILER, gw_vars.THERMOSTAT],
     ],
     gw_vars.DATA_COOLING_CONTROL: [
         None,
         PERCENTAGE,
         "Cooling Control Signal {}",
+        SENSOR_FLOAT_SUGGESTED_DISPLAY_PRECISION,
         [gw_vars.BOILER, gw_vars.THERMOSTAT],
     ],
     gw_vars.DATA_CONTROL_SETPOINT_2: [
         SensorDeviceClass.TEMPERATURE,
-        TEMP_CELSIUS,
+        UnitOfTemperature.CELSIUS,
         "Control Setpoint 2 {}",
+        SENSOR_FLOAT_SUGGESTED_DISPLAY_PRECISION,
         [gw_vars.BOILER, gw_vars.THERMOSTAT],
     ],
     gw_vars.DATA_ROOM_SETPOINT_OVRD: [
         SensorDeviceClass.TEMPERATURE,
-        TEMP_CELSIUS,
+        UnitOfTemperature.CELSIUS,
         "Room Setpoint Override {}",
+        SENSOR_FLOAT_SUGGESTED_DISPLAY_PRECISION,
         [gw_vars.BOILER, gw_vars.THERMOSTAT],
     ],
     gw_vars.DATA_SLAVE_MAX_RELATIVE_MOD: [
         None,
         PERCENTAGE,
         "Boiler Maximum Relative Modulation {}",
+        SENSOR_FLOAT_SUGGESTED_DISPLAY_PRECISION,
         [gw_vars.BOILER, gw_vars.THERMOSTAT],
     ],
     gw_vars.DATA_SLAVE_MAX_CAPACITY: [
-        None,
-        UNIT_KW,
+        SensorDeviceClass.POWER,
+        UnitOfPower.KILO_WATT,
         "Boiler Maximum Capacity {}",
+        None,
         [gw_vars.BOILER, gw_vars.THERMOSTAT],
     ],
     gw_vars.DATA_SLAVE_MIN_MOD_LEVEL: [
         None,
         PERCENTAGE,
         "Boiler Minimum Modulation Level {}",
+        None,
         [gw_vars.BOILER, gw_vars.THERMOSTAT],
     ],
     gw_vars.DATA_ROOM_SETPOINT: [
         SensorDeviceClass.TEMPERATURE,
-        TEMP_CELSIUS,
+        UnitOfTemperature.CELSIUS,
         "Room Setpoint {}",
+        SENSOR_FLOAT_SUGGESTED_DISPLAY_PRECISION,
         [gw_vars.BOILER, gw_vars.THERMOSTAT],
     ],
     gw_vars.DATA_REL_MOD_LEVEL: [
         None,
         PERCENTAGE,
         "Relative Modulation Level {}",
+        SENSOR_FLOAT_SUGGESTED_DISPLAY_PRECISION,
         [gw_vars.BOILER, gw_vars.THERMOSTAT],
     ],
     gw_vars.DATA_CH_WATER_PRESS: [
-        None,
-        PRESSURE_BAR,
+        SensorDeviceClass.PRESSURE,
+        UnitOfPressure.BAR,
         "Central Heating Water Pressure {}",
+        SENSOR_FLOAT_SUGGESTED_DISPLAY_PRECISION,
         [gw_vars.BOILER, gw_vars.THERMOSTAT],
     ],
     gw_vars.DATA_DHW_FLOW_RATE: [
         None,
-        UNIT_L_MIN,
+        f"{UnitOfVolume.LITERS}/{UnitOfTime.MINUTES}",
         "Hot Water Flow Rate {}",
+        SENSOR_FLOAT_SUGGESTED_DISPLAY_PRECISION,
         [gw_vars.BOILER, gw_vars.THERMOSTAT],
     ],
     gw_vars.DATA_ROOM_SETPOINT_2: [
         SensorDeviceClass.TEMPERATURE,
-        TEMP_CELSIUS,
+        UnitOfTemperature.CELSIUS,
         "Room Setpoint 2 {}",
+        SENSOR_FLOAT_SUGGESTED_DISPLAY_PRECISION,
         [gw_vars.BOILER, gw_vars.THERMOSTAT],
     ],
     gw_vars.DATA_ROOM_TEMP: [
         SensorDeviceClass.TEMPERATURE,
-        TEMP_CELSIUS,
+        UnitOfTemperature.CELSIUS,
         "Room Temperature {}",
+        SENSOR_FLOAT_SUGGESTED_DISPLAY_PRECISION,
         [gw_vars.BOILER, gw_vars.THERMOSTAT],
     ],
     gw_vars.DATA_CH_WATER_TEMP: [
         SensorDeviceClass.TEMPERATURE,
-        TEMP_CELSIUS,
+        UnitOfTemperature.CELSIUS,
         "Central Heating Water Temperature {}",
+        SENSOR_FLOAT_SUGGESTED_DISPLAY_PRECISION,
         [gw_vars.BOILER, gw_vars.THERMOSTAT],
     ],
     gw_vars.DATA_DHW_TEMP: [
         SensorDeviceClass.TEMPERATURE,
-        TEMP_CELSIUS,
+        UnitOfTemperature.CELSIUS,
         "Hot Water Temperature {}",
+        SENSOR_FLOAT_SUGGESTED_DISPLAY_PRECISION,
         [gw_vars.BOILER, gw_vars.THERMOSTAT],
     ],
     gw_vars.DATA_OUTSIDE_TEMP: [
         SensorDeviceClass.TEMPERATURE,
-        TEMP_CELSIUS,
+        UnitOfTemperature.CELSIUS,
         "Outside Temperature {}",
+        SENSOR_FLOAT_SUGGESTED_DISPLAY_PRECISION,
         [gw_vars.BOILER, gw_vars.THERMOSTAT],
     ],
     gw_vars.DATA_RETURN_WATER_TEMP: [
         SensorDeviceClass.TEMPERATURE,
-        TEMP_CELSIUS,
+        UnitOfTemperature.CELSIUS,
         "Return Water Temperature {}",
+        SENSOR_FLOAT_SUGGESTED_DISPLAY_PRECISION,
         [gw_vars.BOILER, gw_vars.THERMOSTAT],
     ],
     gw_vars.DATA_SOLAR_STORAGE_TEMP: [
         SensorDeviceClass.TEMPERATURE,
-        TEMP_CELSIUS,
+        UnitOfTemperature.CELSIUS,
         "Solar Storage Temperature {}",
+        SENSOR_FLOAT_SUGGESTED_DISPLAY_PRECISION,
         [gw_vars.BOILER, gw_vars.THERMOSTAT],
     ],
     gw_vars.DATA_SOLAR_COLL_TEMP: [
         SensorDeviceClass.TEMPERATURE,
-        TEMP_CELSIUS,
+        UnitOfTemperature.CELSIUS,
         "Solar Collector Temperature {}",
+        SENSOR_FLOAT_SUGGESTED_DISPLAY_PRECISION,
         [gw_vars.BOILER, gw_vars.THERMOSTAT],
     ],
     gw_vars.DATA_CH_WATER_TEMP_2: [
         SensorDeviceClass.TEMPERATURE,
-        TEMP_CELSIUS,
+        UnitOfTemperature.CELSIUS,
         "Central Heating 2 Water Temperature {}",
+        SENSOR_FLOAT_SUGGESTED_DISPLAY_PRECISION,
         [gw_vars.BOILER, gw_vars.THERMOSTAT],
     ],
     gw_vars.DATA_DHW_TEMP_2: [
         SensorDeviceClass.TEMPERATURE,
-        TEMP_CELSIUS,
+        UnitOfTemperature.CELSIUS,
         "Hot Water 2 Temperature {}",
+        SENSOR_FLOAT_SUGGESTED_DISPLAY_PRECISION,
         [gw_vars.BOILER, gw_vars.THERMOSTAT],
     ],
     gw_vars.DATA_EXHAUST_TEMP: [
         SensorDeviceClass.TEMPERATURE,
-        TEMP_CELSIUS,
+        UnitOfTemperature.CELSIUS,
         "Exhaust Temperature {}",
+        None,
         [gw_vars.BOILER, gw_vars.THERMOSTAT],
     ],
     gw_vars.DATA_SLAVE_DHW_MAX_SETP: [
         SensorDeviceClass.TEMPERATURE,
-        TEMP_CELSIUS,
+        UnitOfTemperature.CELSIUS,
         "Hot Water Maximum Setpoint {}",
+        None,
         [gw_vars.BOILER, gw_vars.THERMOSTAT],
     ],
     gw_vars.DATA_SLAVE_DHW_MIN_SETP: [
         SensorDeviceClass.TEMPERATURE,
-        TEMP_CELSIUS,
+        UnitOfTemperature.CELSIUS,
         "Hot Water Minimum Setpoint {}",
+        None,
         [gw_vars.BOILER, gw_vars.THERMOSTAT],
     ],
     gw_vars.DATA_SLAVE_CH_MAX_SETP: [
         SensorDeviceClass.TEMPERATURE,
-        TEMP_CELSIUS,
+        UnitOfTemperature.CELSIUS,
         "Boiler Maximum Central Heating Setpoint {}",
+        None,
         [gw_vars.BOILER, gw_vars.THERMOSTAT],
     ],
     gw_vars.DATA_SLAVE_CH_MIN_SETP: [
         SensorDeviceClass.TEMPERATURE,
-        TEMP_CELSIUS,
+        UnitOfTemperature.CELSIUS,
         "Boiler Minimum Central Heating Setpoint {}",
+        None,
         [gw_vars.BOILER, gw_vars.THERMOSTAT],
     ],
     gw_vars.DATA_DHW_SETPOINT: [
         SensorDeviceClass.TEMPERATURE,
-        TEMP_CELSIUS,
+        UnitOfTemperature.CELSIUS,
         "Hot Water Setpoint {}",
+        SENSOR_FLOAT_SUGGESTED_DISPLAY_PRECISION,
         [gw_vars.BOILER, gw_vars.THERMOSTAT],
     ],
     gw_vars.DATA_MAX_CH_SETPOINT: [
         SensorDeviceClass.TEMPERATURE,
-        TEMP_CELSIUS,
+        UnitOfTemperature.CELSIUS,
         "Maximum Central Heating Setpoint {}",
+        SENSOR_FLOAT_SUGGESTED_DISPLAY_PRECISION,
         [gw_vars.BOILER, gw_vars.THERMOSTAT],
     ],
     gw_vars.DATA_OEM_DIAG: [
         None,
         None,
         "OEM Diagnostic Code {}",
+        None,
         [gw_vars.BOILER, gw_vars.THERMOSTAT],
     ],
     gw_vars.DATA_TOTAL_BURNER_STARTS: [
         None,
-        None,
+        "starts",
         "Total Burner Starts {}",
+        None,
         [gw_vars.BOILER, gw_vars.THERMOSTAT],
     ],
     gw_vars.DATA_CH_PUMP_STARTS: [
         None,
-        None,
+        "starts",
         "Central Heating Pump Starts {}",
+        None,
         [gw_vars.BOILER, gw_vars.THERMOSTAT],
     ],
     gw_vars.DATA_DHW_PUMP_STARTS: [
         None,
-        None,
+        "starts",
         "Hot Water Pump Starts {}",
+        None,
         [gw_vars.BOILER, gw_vars.THERMOSTAT],
     ],
     gw_vars.DATA_DHW_BURNER_STARTS: [
         None,
-        None,
+        "starts",
         "Hot Water Burner Starts {}",
+        None,
         [gw_vars.BOILER, gw_vars.THERMOSTAT],
     ],
     gw_vars.DATA_TOTAL_BURNER_HOURS: [
-        None,
-        TIME_HOURS,
+        SensorDeviceClass.DURATION,
+        UnitOfTime.HOURS,
         "Total Burner Hours {}",
+        None,
         [gw_vars.BOILER, gw_vars.THERMOSTAT],
     ],
     gw_vars.DATA_CH_PUMP_HOURS: [
-        None,
-        TIME_HOURS,
+        SensorDeviceClass.DURATION,
+        UnitOfTime.HOURS,
         "Central Heating Pump Hours {}",
+        None,
         [gw_vars.BOILER, gw_vars.THERMOSTAT],
     ],
     gw_vars.DATA_DHW_PUMP_HOURS: [
-        None,
-        TIME_HOURS,
+        SensorDeviceClass.DURATION,
+        UnitOfTime.HOURS,
         "Hot Water Pump Hours {}",
+        None,
         [gw_vars.BOILER, gw_vars.THERMOSTAT],
     ],
     gw_vars.DATA_DHW_BURNER_HOURS: [
-        None,
-        TIME_HOURS,
+        SensorDeviceClass.DURATION,
+        UnitOfTime.HOURS,
         "Hot Water Burner Hours {}",
+        None,
         [gw_vars.BOILER, gw_vars.THERMOSTAT],
     ],
     gw_vars.DATA_MASTER_OT_VERSION: [
         None,
         None,
         "Thermostat OpenTherm Version {}",
+        SENSOR_FLOAT_SUGGESTED_DISPLAY_PRECISION,
         [gw_vars.BOILER, gw_vars.THERMOSTAT],
     ],
     gw_vars.DATA_SLAVE_OT_VERSION: [
         None,
         None,
         "Boiler OpenTherm Version {}",
+        SENSOR_FLOAT_SUGGESTED_DISPLAY_PRECISION,
         [gw_vars.BOILER, gw_vars.THERMOSTAT],
     ],
     gw_vars.DATA_MASTER_PRODUCT_TYPE: [
         None,
         None,
         "Thermostat Product Type {}",
+        None,
         [gw_vars.BOILER, gw_vars.THERMOSTAT],
     ],
     gw_vars.DATA_MASTER_PRODUCT_VERSION: [
         None,
         None,
         "Thermostat Product Version {}",
+        None,
         [gw_vars.BOILER, gw_vars.THERMOSTAT],
     ],
     gw_vars.DATA_SLAVE_PRODUCT_TYPE: [
         None,
         None,
         "Boiler Product Type {}",
+        None,
         [gw_vars.BOILER, gw_vars.THERMOSTAT],
     ],
     gw_vars.DATA_SLAVE_PRODUCT_VERSION: [
         None,
         None,
         "Boiler Product Version {}",
+        None,
         [gw_vars.BOILER, gw_vars.THERMOSTAT],
     ],
-    gw_vars.OTGW_MODE: [None, None, "Gateway/Monitor Mode {}", [gw_vars.OTGW]],
+    gw_vars.OTGW_MODE: [
+        None,
+        None,
+        "Gateway/Monitor Mode {}",
+        None,
+        [gw_vars.OTGW],
+    ],
     gw_vars.OTGW_DHW_OVRD: [
         None,
         None,
         "Gateway Hot Water Override Mode {}",
+        None,
+        [gw_vars.OTGW],
+    ],
+    gw_vars.OTGW_ABOUT: [
+        None,
+        None,
+        "Gateway Firmware Version {}",
+        None,
+        [gw_vars.OTGW],
+    ],
+    gw_vars.OTGW_BUILD: [
+        None,
+        None,
+        "Gateway Firmware Build {}",
+        None,
+        [gw_vars.OTGW],
+    ],
+    gw_vars.OTGW_CLOCKMHZ: [
+        None,
+        None,
+        "Gateway Clock Speed {}",
+        None,
+        [gw_vars.OTGW],
+    ],
+    gw_vars.OTGW_LED_A: [
+        None,
+        None,
+        "Gateway LED A Mode {}",
+        None,
+        [gw_vars.OTGW],
+    ],
+    gw_vars.OTGW_LED_B: [
+        None,
+        None,
+        "Gateway LED B Mode {}",
+        None,
+        [gw_vars.OTGW],
+    ],
+    gw_vars.OTGW_LED_C: [
+        None,
+        None,
+        "Gateway LED C Mode {}",
+        None,
+        [gw_vars.OTGW],
+    ],
+    gw_vars.OTGW_LED_D: [
+        None,
+        None,
+        "Gateway LED D Mode {}",
+        None,
+        [gw_vars.OTGW],
+    ],
+    gw_vars.OTGW_LED_E: [
+        None,
+        None,
+        "Gateway LED E Mode {}",
+        None,
+        [gw_vars.OTGW],
+    ],
+    gw_vars.OTGW_LED_F: [
+        None,
+        None,
+        "Gateway LED F Mode {}",
+        None,
+        [gw_vars.OTGW],
+    ],
+    gw_vars.OTGW_GPIO_A: [
+        None,
+        None,
+        "Gateway GPIO A Mode {}",
+        None,
+        [gw_vars.OTGW],
+    ],
+    gw_vars.OTGW_GPIO_B: [
+        None,
+        None,
+        "Gateway GPIO B Mode {}",
+        None,
         [gw_vars.OTGW],
     ],
-    gw_vars.OTGW_ABOUT: [None, None, "Gateway Firmware Version {}", [gw_vars.OTGW]],
-    gw_vars.OTGW_BUILD: [None, None, "Gateway Firmware Build {}", [gw_vars.OTGW]],
-    gw_vars.OTGW_CLOCKMHZ: [None, None, "Gateway Clock Speed {}", [gw_vars.OTGW]],
-    gw_vars.OTGW_LED_A: [None, None, "Gateway LED A Mode {}", [gw_vars.OTGW]],
-    gw_vars.OTGW_LED_B: [None, None, "Gateway LED B Mode {}", [gw_vars.OTGW]],
-    gw_vars.OTGW_LED_C: [None, None, "Gateway LED C Mode {}", [gw_vars.OTGW]],
-    gw_vars.OTGW_LED_D: [None, None, "Gateway LED D Mode {}", [gw_vars.OTGW]],
-    gw_vars.OTGW_LED_E: [None, None, "Gateway LED E Mode {}", [gw_vars.OTGW]],
-    gw_vars.OTGW_LED_F: [None, None, "Gateway LED F Mode {}", [gw_vars.OTGW]],
-    gw_vars.OTGW_GPIO_A: [None, None, "Gateway GPIO A Mode {}", [gw_vars.OTGW]],
-    gw_vars.OTGW_GPIO_B: [None, None, "Gateway GPIO B Mode {}", [gw_vars.OTGW]],
     gw_vars.OTGW_SB_TEMP: [
         SensorDeviceClass.TEMPERATURE,
-        TEMP_CELSIUS,
+        UnitOfTemperature.CELSIUS,
         "Gateway Setback Temperature {}",
+        SENSOR_FLOAT_SUGGESTED_DISPLAY_PRECISION,
         [gw_vars.OTGW],
     ],
     gw_vars.OTGW_SETP_OVRD_MODE: [
         None,
         None,
         "Gateway Room Setpoint Override Mode {}",
+        None,
+        [gw_vars.OTGW],
+    ],
+    gw_vars.OTGW_SMART_PWR: [
+        None,
+        None,
+        "Gateway Smart Power Mode {}",
+        None,
         [gw_vars.OTGW],
     ],
-    gw_vars.OTGW_SMART_PWR: [None, None, "Gateway Smart Power Mode {}", [gw_vars.OTGW]],
     gw_vars.OTGW_THRM_DETECT: [
         None,
         None,
         "Gateway Thermostat Detection {}",
+        None,
         [gw_vars.OTGW],
     ],
     gw_vars.OTGW_VREF: [
         None,
         None,
         "Gateway Reference Voltage Setting {}",
+        None,
         [gw_vars.OTGW],
     ],
 }
-
-DEPRECATED_BINARY_SENSOR_SOURCE_LOOKUP = {
-    gw_vars.DATA_MASTER_CH_ENABLED: gw_vars.THERMOSTAT,
-    gw_vars.DATA_MASTER_DHW_ENABLED: gw_vars.THERMOSTAT,
-    gw_vars.DATA_MASTER_OTC_ENABLED: gw_vars.THERMOSTAT,
-    gw_vars.DATA_MASTER_CH2_ENABLED: gw_vars.THERMOSTAT,
-    gw_vars.DATA_SLAVE_FAULT_IND: gw_vars.BOILER,
-    gw_vars.DATA_SLAVE_CH_ACTIVE: gw_vars.BOILER,
-    gw_vars.DATA_SLAVE_DHW_ACTIVE: gw_vars.BOILER,
-    gw_vars.DATA_SLAVE_FLAME_ON: gw_vars.BOILER,
-    gw_vars.DATA_SLAVE_COOLING_ACTIVE: gw_vars.BOILER,
-    gw_vars.DATA_SLAVE_CH2_ACTIVE: gw_vars.BOILER,
-    gw_vars.DATA_SLAVE_DIAG_IND: gw_vars.BOILER,
-    gw_vars.DATA_SLAVE_DHW_PRESENT: gw_vars.BOILER,
-    gw_vars.DATA_SLAVE_CONTROL_TYPE: gw_vars.BOILER,
-    gw_vars.DATA_SLAVE_COOLING_SUPPORTED: gw_vars.BOILER,
-    gw_vars.DATA_SLAVE_DHW_CONFIG: gw_vars.BOILER,
-    gw_vars.DATA_SLAVE_MASTER_LOW_OFF_PUMP: gw_vars.BOILER,
-    gw_vars.DATA_SLAVE_CH2_PRESENT: gw_vars.BOILER,
-    gw_vars.DATA_SLAVE_SERVICE_REQ: gw_vars.BOILER,
-    gw_vars.DATA_SLAVE_REMOTE_RESET: gw_vars.BOILER,
-    gw_vars.DATA_SLAVE_LOW_WATER_PRESS: gw_vars.BOILER,
-    gw_vars.DATA_SLAVE_GAS_FAULT: gw_vars.BOILER,
-    gw_vars.DATA_SLAVE_AIR_PRESS_FAULT: gw_vars.BOILER,
-    gw_vars.DATA_SLAVE_WATER_OVERTEMP: gw_vars.BOILER,
-    gw_vars.DATA_REMOTE_TRANSFER_DHW: gw_vars.BOILER,
-    gw_vars.DATA_REMOTE_TRANSFER_MAX_CH: gw_vars.BOILER,
-    gw_vars.DATA_REMOTE_RW_DHW: gw_vars.BOILER,
-    gw_vars.DATA_REMOTE_RW_MAX_CH: gw_vars.BOILER,
-    gw_vars.DATA_ROVRD_MAN_PRIO: gw_vars.THERMOSTAT,
-    gw_vars.DATA_ROVRD_AUTO_PRIO: gw_vars.THERMOSTAT,
-    gw_vars.OTGW_GPIO_A_STATE: gw_vars.OTGW,
-    gw_vars.OTGW_GPIO_B_STATE: gw_vars.OTGW,
-    gw_vars.OTGW_IGNORE_TRANSITIONS: gw_vars.OTGW,
-    gw_vars.OTGW_OVRD_HB: gw_vars.OTGW,
-}
-
-DEPRECATED_SENSOR_SOURCE_LOOKUP = {
-    gw_vars.DATA_CONTROL_SETPOINT: gw_vars.BOILER,
-    gw_vars.DATA_MASTER_MEMBERID: gw_vars.THERMOSTAT,
-    gw_vars.DATA_SLAVE_MEMBERID: gw_vars.BOILER,
-    gw_vars.DATA_SLAVE_OEM_FAULT: gw_vars.BOILER,
-    gw_vars.DATA_COOLING_CONTROL: gw_vars.BOILER,
-    gw_vars.DATA_CONTROL_SETPOINT_2: gw_vars.BOILER,
-    gw_vars.DATA_ROOM_SETPOINT_OVRD: gw_vars.THERMOSTAT,
-    gw_vars.DATA_SLAVE_MAX_RELATIVE_MOD: gw_vars.BOILER,
-    gw_vars.DATA_SLAVE_MAX_CAPACITY: gw_vars.BOILER,
-    gw_vars.DATA_SLAVE_MIN_MOD_LEVEL: gw_vars.BOILER,
-    gw_vars.DATA_ROOM_SETPOINT: gw_vars.THERMOSTAT,
-    gw_vars.DATA_REL_MOD_LEVEL: gw_vars.BOILER,
-    gw_vars.DATA_CH_WATER_PRESS: gw_vars.BOILER,
-    gw_vars.DATA_DHW_FLOW_RATE: gw_vars.BOILER,
-    gw_vars.DATA_ROOM_SETPOINT_2: gw_vars.THERMOSTAT,
-    gw_vars.DATA_ROOM_TEMP: gw_vars.THERMOSTAT,
-    gw_vars.DATA_CH_WATER_TEMP: gw_vars.BOILER,
-    gw_vars.DATA_DHW_TEMP: gw_vars.BOILER,
-    gw_vars.DATA_OUTSIDE_TEMP: gw_vars.THERMOSTAT,
-    gw_vars.DATA_RETURN_WATER_TEMP: gw_vars.BOILER,
-    gw_vars.DATA_SOLAR_STORAGE_TEMP: gw_vars.BOILER,
-    gw_vars.DATA_SOLAR_COLL_TEMP: gw_vars.BOILER,
-    gw_vars.DATA_CH_WATER_TEMP_2: gw_vars.BOILER,
-    gw_vars.DATA_DHW_TEMP_2: gw_vars.BOILER,
-    gw_vars.DATA_EXHAUST_TEMP: gw_vars.BOILER,
-    gw_vars.DATA_SLAVE_DHW_MAX_SETP: gw_vars.BOILER,
-    gw_vars.DATA_SLAVE_DHW_MIN_SETP: gw_vars.BOILER,
-    gw_vars.DATA_SLAVE_CH_MAX_SETP: gw_vars.BOILER,
-    gw_vars.DATA_SLAVE_CH_MIN_SETP: gw_vars.BOILER,
-    gw_vars.DATA_DHW_SETPOINT: gw_vars.BOILER,
-    gw_vars.DATA_MAX_CH_SETPOINT: gw_vars.BOILER,
-    gw_vars.DATA_OEM_DIAG: gw_vars.BOILER,
-    gw_vars.DATA_TOTAL_BURNER_STARTS: gw_vars.BOILER,
-    gw_vars.DATA_CH_PUMP_STARTS: gw_vars.BOILER,
-    gw_vars.DATA_DHW_PUMP_STARTS: gw_vars.BOILER,
-    gw_vars.DATA_DHW_BURNER_STARTS: gw_vars.BOILER,
-    gw_vars.DATA_TOTAL_BURNER_HOURS: gw_vars.BOILER,
-    gw_vars.DATA_CH_PUMP_HOURS: gw_vars.BOILER,
-    gw_vars.DATA_DHW_PUMP_HOURS: gw_vars.BOILER,
-    gw_vars.DATA_DHW_BURNER_HOURS: gw_vars.BOILER,
-    gw_vars.DATA_MASTER_OT_VERSION: gw_vars.THERMOSTAT,
-    gw_vars.DATA_SLAVE_OT_VERSION: gw_vars.BOILER,
-    gw_vars.DATA_MASTER_PRODUCT_TYPE: gw_vars.THERMOSTAT,
-    gw_vars.DATA_MASTER_PRODUCT_VERSION: gw_vars.THERMOSTAT,
-    gw_vars.DATA_SLAVE_PRODUCT_TYPE: gw_vars.BOILER,
-    gw_vars.DATA_SLAVE_PRODUCT_VERSION: gw_vars.BOILER,
-    gw_vars.OTGW_MODE: gw_vars.OTGW,
-    gw_vars.OTGW_DHW_OVRD: gw_vars.OTGW,
-    gw_vars.OTGW_ABOUT: gw_vars.OTGW,
-    gw_vars.OTGW_BUILD: gw_vars.OTGW,
-    gw_vars.OTGW_CLOCKMHZ: gw_vars.OTGW,
-    gw_vars.OTGW_LED_A: gw_vars.OTGW,
-    gw_vars.OTGW_LED_B: gw_vars.OTGW,
-    gw_vars.OTGW_LED_C: gw_vars.OTGW,
-    gw_vars.OTGW_LED_D: gw_vars.OTGW,
-    gw_vars.OTGW_LED_E: gw_vars.OTGW,
-    gw_vars.OTGW_LED_F: gw_vars.OTGW,
-    gw_vars.OTGW_GPIO_A: gw_vars.OTGW,
-    gw_vars.OTGW_GPIO_B: gw_vars.OTGW,
-    gw_vars.OTGW_SB_TEMP: gw_vars.OTGW,
-    gw_vars.OTGW_SETP_OVRD_MODE: gw_vars.OTGW,
-    gw_vars.OTGW_SMART_PWR: gw_vars.OTGW,
-    gw_vars.OTGW_THRM_DETECT: gw_vars.OTGW,
-    gw_vars.OTGW_VREF: gw_vars.OTGW,
-}
