@@ -1,10 +1,13 @@
 """Support for Orvibo S20 Wifi Smart Switches."""
+from __future__ import annotations
+
 import logging
+from typing import Any
 
 from orvibo.s20 import S20, S20Exception, discover
 import voluptuous as vol
 
-from homeassistant.components.switch import PLATFORM_SCHEMA, SwitchDevice
+from homeassistant.components.switch import PLATFORM_SCHEMA, SwitchEntity
 from homeassistant.const import (
     CONF_DISCOVERY,
     CONF_HOST,
@@ -12,7 +15,10 @@ from homeassistant.const import (
     CONF_NAME,
     CONF_SWITCHES,
 )
+from homeassistant.core import HomeAssistant
 import homeassistant.helpers.config_validation as cv
+from homeassistant.helpers.entity_platform import AddEntitiesCallback
+from homeassistant.helpers.typing import ConfigType, DiscoveryInfoType
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -36,7 +42,12 @@ PLATFORM_SCHEMA = PLATFORM_SCHEMA.extend(
 )
 
 
-def setup_platform(hass, config, add_entities_callback, discovery_info=None):
+def setup_platform(
+    hass: HomeAssistant,
+    config: ConfigType,
+    add_entities_callback: AddEntitiesCallback,
+    discovery_info: DiscoveryInfoType | None = None,
+) -> None:
     """Set up S20 switches."""
 
     switch_data = {}
@@ -44,7 +55,7 @@ def setup_platform(hass, config, add_entities_callback, discovery_info=None):
     switch_conf = config.get(CONF_SWITCHES, [config])
 
     if config.get(CONF_DISCOVERY):
-        _LOGGER.info("Discovering S20 switches ...")
+        _LOGGER.info("Discovering S20 switches")
         switch_data.update(discover())
 
     for switch in switch_conf:
@@ -62,7 +73,7 @@ def setup_platform(hass, config, add_entities_callback, discovery_info=None):
     add_entities_callback(switches)
 
 
-class S20Switch(SwitchDevice):
+class S20Switch(SwitchEntity):
     """Representation of an S20 switch."""
 
     def __init__(self, name, s20):
@@ -73,11 +84,6 @@ class S20Switch(SwitchDevice):
         self._state = False
         self._exc = S20Exception
 
-    @property
-    def should_poll(self):
-        """Return the polling state."""
-        return True
-
     @property
     def name(self):
         """Return the name of the switch."""
@@ -88,21 +94,21 @@ class S20Switch(SwitchDevice):
         """Return true if device is on."""
         return self._state
 
-    def update(self):
+    def update(self) -> None:
         """Update device state."""
         try:
             self._state = self._s20.on
         except self._exc:
             _LOGGER.exception("Error while fetching S20 state")
 
-    def turn_on(self, **kwargs):
+    def turn_on(self, **kwargs: Any) -> None:
         """Turn the device on."""
         try:
             self._s20.on = True
         except self._exc:
             _LOGGER.exception("Error while turning on S20")
 
-    def turn_off(self, **kwargs):
+    def turn_off(self, **kwargs: Any) -> None:
         """Turn the device off."""
         try:
             self._s20.on = False
