@@ -1,10 +1,14 @@
 """Register a custom front end panel."""
+from __future__ import annotations
+
 import logging
-import os
 
 import voluptuous as vol
 
+from homeassistant.components import frontend
+from homeassistant.core import HomeAssistant
 import homeassistant.helpers.config_validation as cv
+from homeassistant.helpers.typing import ConfigType
 from homeassistant.loader import bind_hass
 
 _LOGGER = logging.getLogger(__name__)
@@ -15,7 +19,6 @@ CONF_SIDEBAR_TITLE = "sidebar_title"
 CONF_SIDEBAR_ICON = "sidebar_icon"
 CONF_URL_PATH = "url_path"
 CONF_CONFIG = "config"
-CONF_WEBCOMPONENT_PATH = "webcomponent_path"
 CONF_JS_URL = "js_url"
 CONF_MODULE_URL = "module_url"
 CONF_EMBED_IFRAME = "embed_iframe"
@@ -23,8 +26,6 @@ CONF_TRUST_EXTERNAL_SCRIPT = "trust_external_script"
 CONF_URL_EXCLUSIVE_GROUP = "url_exclusive_group"
 CONF_REQUIRE_ADMIN = "require_admin"
 
-MSG_URL_CONFLICT = "Pass in only one of webcomponent_path, module_url or js_url"
-
 DEFAULT_EMBED_IFRAME = False
 DEFAULT_TRUST_EXTERNAL = False
 
@@ -33,6 +34,7 @@ LEGACY_URL = "/api/panel_custom/{}"
 
 PANEL_DIR = "panels"
 
+
 CONFIG_SCHEMA = vol.Schema(
     {
         DOMAIN: vol.All(
@@ -45,28 +47,22 @@ CONFIG_SCHEMA = vol.Schema(
                         vol.Optional(CONF_SIDEBAR_ICON, default=DEFAULT_ICON): cv.icon,
                         vol.Optional(CONF_URL_PATH): cv.string,
                         vol.Optional(CONF_CONFIG): dict,
-                        vol.Exclusive(
-                            CONF_WEBCOMPONENT_PATH,
-                            CONF_URL_EXCLUSIVE_GROUP,
-                            msg=MSG_URL_CONFLICT,
+                        vol.Optional(
+                            CONF_JS_URL,
                         ): cv.string,
-                        vol.Exclusive(
-                            CONF_JS_URL, CONF_URL_EXCLUSIVE_GROUP, msg=MSG_URL_CONFLICT
-                        ): cv.string,
-                        vol.Exclusive(
+                        vol.Optional(
                             CONF_MODULE_URL,
-                            CONF_URL_EXCLUSIVE_GROUP,
-                            msg=MSG_URL_CONFLICT,
                         ): cv.string,
                         vol.Optional(
                             CONF_EMBED_IFRAME, default=DEFAULT_EMBED_IFRAME
                         ): cv.boolean,
                         vol.Optional(
-                            CONF_TRUST_EXTERNAL_SCRIPT, default=DEFAULT_TRUST_EXTERNAL
+                            CONF_TRUST_EXTERNAL_SCRIPT,
+                            default=DEFAULT_TRUST_EXTERNAL,
                         ): cv.boolean,
                         vol.Optional(CONF_REQUIRE_ADMIN, default=False): cv.boolean,
                     }
-                )
+                ),
             ],
         )
     },
@@ -76,35 +72,32 @@ CONFIG_SCHEMA = vol.Schema(
 
 @bind_hass
 async def async_register_panel(
-    hass,
+    hass: HomeAssistant,
     # The url to serve the panel
-    frontend_url_path,
+    frontend_url_path: str,
     # The webcomponent name that loads your panel
-    webcomponent_name,
+    webcomponent_name: str,
     # Title/icon for sidebar
-    sidebar_title=None,
-    sidebar_icon=None,
-    # HTML source of your panel
-    html_url=None,
+    sidebar_title: str | None = None,
+    sidebar_icon: str | None = None,
     # JS source of your panel
-    js_url=None,
+    js_url: str | None = None,
     # JS module of your panel
-    module_url=None,
+    module_url: str | None = None,
     # If your panel should be run inside an iframe
-    embed_iframe=DEFAULT_EMBED_IFRAME,
+    embed_iframe: bool = DEFAULT_EMBED_IFRAME,
     # Should user be asked for confirmation when loading external source
-    trust_external=DEFAULT_TRUST_EXTERNAL,
+    trust_external: bool = DEFAULT_TRUST_EXTERNAL,
     # Configuration to be passed to the panel
-    config=None,
+    config: ConfigType | None = None,
     # If your panel should only be shown to admin users
-    require_admin=False,
-):
+    require_admin: bool = False,
+    # If your panel is used to configure an integration, needs the domain of the integration
+    config_panel_domain: str | None = None,
+) -> None:
     """Register a new custom panel."""
-    if js_url is None and html_url is None and module_url is None:
+    if js_url is None and module_url is None:
         raise ValueError("Either js_url, module_url or html_url is required.")
-    if (js_url and html_url) or (module_url and html_url):
-        raise ValueError("Pass in only one of JS url, Module url or HTML url.")
-
     if config is not None and not isinstance(config, dict):
         raise ValueError("Config needs to be a dictionary.")
 
@@ -120,9 +113,6 @@ async def async_register_panel(
     if module_url is not None:
         custom_panel_config["module_url"] = module_url
 
-    if html_url is not None:
-        custom_panel_config["html_url"] = html_url
-
     if config is not None:
         # Make copy because we're mutating it
         config = dict(config)
@@ -131,17 +121,19 @@ async def async_register_panel(
 
     config["_panel_custom"] = custom_panel_config
 
-    hass.components.frontend.async_register_built_in_panel(
+    frontend.async_register_built_in_panel(
+        hass,
         component_name="custom",
         sidebar_title=sidebar_title,
         sidebar_icon=sidebar_icon,
         frontend_url_path=frontend_url_path,
         config=config,
         require_admin=require_admin,
+        config_panel_domain=config_panel_domain,
     )
 
 
-async def async_setup(hass, config):
+async def async_setup(hass: HomeAssistant, config: ConfigType) -> bool:
     """Initialize custom panel."""
     if DOMAIN not in config:
         return True
@@ -160,26 +152,12 @@ async def async_setup(hass, config):
             "require_admin": panel[CONF_REQUIRE_ADMIN],
         }
 
-        panel_path = panel.get(CONF_WEBCOMPONENT_PATH)
-
-        if panel_path is None:
-            panel_path = hass.config.path(PANEL_DIR, f"{name}.html")
-
         if CONF_JS_URL in panel:
             kwargs["js_url"] = panel[CONF_JS_URL]
 
-        elif CONF_MODULE_URL in panel:
+        if CONF_MODULE_URL in panel:
             kwargs["module_url"] = panel[CONF_MODULE_URL]
 
-        elif not await hass.async_add_job(os.path.isfile, panel_path):
-            _LOGGER.error("Unable to find webcomponent for %s: %s", name, panel_path)
-            continue
-
-        else:
-            url = LEGACY_URL.format(name)
-            hass.http.register_static_path(url, panel_path)
-            kwargs["html_url"] = url
-
         try:
             await async_register_panel(hass, **kwargs)
         except ValueError as err:
