@@ -1,9 +1,13 @@
 """Constants for the Plex component."""
-from homeassistant.const import __version__
+from datetime import timedelta
+from typing import Final
+
+from homeassistant.const import Platform, __version__
 
 DOMAIN = "plex"
 NAME_FORMAT = "Plex ({})"
 COMMON_PLAYERS = ["Plex Web"]
+TRANSIENT_DEVICE_MODELS = ["Plex Web", "Plex for Sonos"]
 
 DEFAULT_PORT = 32400
 DEFAULT_SSL = False
@@ -11,21 +15,25 @@ DEFAULT_VERIFY_SSL = True
 
 PLEXTV_THROTTLE = 60
 
+CLIENT_SCAN_INTERVAL = timedelta(minutes=10)
 DEBOUNCE_TIMEOUT = 1
-DISPATCHERS = "dispatchers"
-GDM_DEBOUNCER = "gdm_debouncer"
-GDM_SCANNER = "gdm_scanner"
-PLATFORMS = frozenset(["media_player", "sensor"])
-PLATFORMS_COMPLETED = "platforms_completed"
+DISPATCHERS: Final = "dispatchers"
+GDM_DEBOUNCER: Final = "gdm_debouncer"
+GDM_SCANNER: Final = "gdm_scanner"
+PLATFORMS = frozenset(
+    [Platform.BUTTON, Platform.MEDIA_PLAYER, Platform.SENSOR, Platform.UPDATE]
+)
+PLATFORMS_COMPLETED: Final = "platforms_completed"
 PLAYER_SOURCE = "player_source"
-SERVERS = "servers"
-WEBSOCKETS = "websockets"
+SERVERS: Final = "servers"
+WEBSOCKETS: Final = "websockets"
 
 PLEX_SERVER_CONFIG = "server_config"
 
 PLEX_NEW_MP_SIGNAL = "plex_new_mp_signal.{}"
 PLEX_UPDATE_MEDIA_PLAYER_SESSION_SIGNAL = "plex_update_session_signal.{}"
 PLEX_UPDATE_MEDIA_PLAYER_SIGNAL = "plex_update_mp_signal.{}"
+PLEX_UPDATE_LIBRARY_SIGNAL = "plex_update_libraries_signal.{}"
 PLEX_UPDATE_PLATFORMS_SIGNAL = "plex_update_platforms_signal.{}"
 PLEX_UPDATE_SENSOR_SIGNAL = "plex_update_sensor_signal.{}"
 
@@ -51,3 +59,5 @@ SERVICE_REFRESH_LIBRARY = "refresh_library"
 SERVICE_SCAN_CLIENTS = "scan_for_clients"
 
 PLEX_URI_SCHEME = "plex://"
+
+INVALID_TOKEN_MESSAGE = "Invalid token"
