@@ -3,7 +3,7 @@ from __future__ import annotations
 
 from datetime import timedelta
 import logging
-from typing import Final
+from typing import Final, Literal
 
 from homeassistant.const import Platform
 
@@ -21,7 +21,7 @@ SMILE: Final = "smile"
 STRETCH: Final = "stretch"
 STRETCH_USERNAME: Final = "stretch"
 
-PLATFORMS_GATEWAY: Final[list[str]] = [
+PLATFORMS: Final[list[str]] = [
     Platform.BINARY_SENSOR,
     Platform.CLIMATE,
     Platform.NUMBER,
@@ -36,6 +36,25 @@ ZEROCONF_MAP: Final[dict[str, str]] = {
     "stretch": "Stretch",
 }
 
+NumberType = Literal[
+    "maximum_boiler_temperature",
+    "max_dhw_temperature",
+    "temperature_offset",
+]
+
+SelectType = Literal[
+    "select_dhw_mode",
+    "select_gateway_mode",
+    "select_regulation_mode",
+    "select_schedule",
+]
+SelectOptionsType = Literal[
+    "dhw_modes",
+    "gateway_modes",
+    "regulation_modes",
+    "available_schedules",
+]
+
 # Default directives
 DEFAULT_MAX_TEMP: Final = 30
 DEFAULT_MIN_TEMP: Final = 4
