@@ -13,46 +13,53 @@ from homeassistant.components.number import (
     NumberMode,
 )
 from homeassistant.config_entries import ConfigEntry
-from homeassistant.const import TEMP_CELSIUS
+from homeassistant.const import EntityCategory, UnitOfTemperature
 from homeassistant.core import HomeAssistant
-from homeassistant.helpers.entity import EntityCategory
 from homeassistant.helpers.entity_platform import AddEntitiesCallback
 
-from .const import DOMAIN
+from .const import DOMAIN, NumberType
 from .coordinator import PlugwiseDataUpdateCoordinator
 from .entity import PlugwiseEntity
 
 
-@dataclass
-class PlugwiseEntityDescriptionMixin:
-    """Mixin values for Plugwse entities."""
-
-    command: Callable[[Smile, str, float], Awaitable[None]]
-    native_max_value_key: str
-    native_min_value_key: str
-    native_step_key: str
-    native_value_key: str
-
-
-@dataclass
-class PlugwiseNumberEntityDescription(
-    NumberEntityDescription, PlugwiseEntityDescriptionMixin
-):
+@dataclass(frozen=True, kw_only=True)
+class PlugwiseNumberEntityDescription(NumberEntityDescription):
     """Class describing Plugwise Number entities."""
 
+    command: Callable[[Smile, str, str, float], Awaitable[None]]
+    key: NumberType
+
 
 NUMBER_TYPES = (
     PlugwiseNumberEntityDescription(
         key="maximum_boiler_temperature",
-        command=lambda api, number, value: api.set_number_setpoint(number, value),
+        translation_key="maximum_boiler_temperature",
+        command=lambda api, number, dev_id, value: api.set_number_setpoint(
+            number, dev_id, value
+        ),
         device_class=NumberDeviceClass.TEMPERATURE,
-        name="Maximum boiler temperature setpoint",
         entity_category=EntityCategory.CONFIG,
-        native_max_value_key="upper_bound",
-        native_min_value_key="lower_bound",
-        native_step_key="resolution",
-        native_unit_of_measurement=TEMP_CELSIUS,
-        native_value_key="setpoint",
+        native_unit_of_measurement=UnitOfTemperature.CELSIUS,
+    ),
+    PlugwiseNumberEntityDescription(
+        key="max_dhw_temperature",
+        translation_key="max_dhw_temperature",
+        command=lambda api, number, dev_id, value: api.set_number_setpoint(
+            number, dev_id, value
+        ),
+        device_class=NumberDeviceClass.TEMPERATURE,
+        entity_category=EntityCategory.CONFIG,
+        native_unit_of_measurement=UnitOfTemperature.CELSIUS,
+    ),
+    PlugwiseNumberEntityDescription(
+        key="temperature_offset",
+        translation_key="temperature_offset",
+        command=lambda api, number, dev_id, value: api.set_temperature_offset(
+            number, dev_id, value
+        ),
+        device_class=NumberDeviceClass.TEMPERATURE,
+        entity_category=EntityCategory.CONFIG,
+        native_unit_of_measurement=UnitOfTemperature.CELSIUS,
     ),
 )
 
@@ -71,7 +78,7 @@ async def async_setup_entry(
     entities: list[PlugwiseNumberEntity] = []
     for device_id, device in coordinator.data.devices.items():
         for description in NUMBER_TYPES:
-            if description.key in device and "setpoint" in device[description.key]:
+            if description.key in device:
                 entities.append(
                     PlugwiseNumberEntity(coordinator, device_id, description)
                 )
@@ -92,44 +99,26 @@ class PlugwiseNumberEntity(PlugwiseEntity, NumberEntity):
     ) -> None:
         """Initiate Plugwise Number."""
         super().__init__(coordinator, device_id)
+        self.device_id = device_id
         self.entity_description = description
         self._attr_unique_id = f"{device_id}-{description.key}"
         self._attr_mode = NumberMode.BOX
+        self._attr_native_max_value = self.device[description.key]["upper_bound"]
+        self._attr_native_min_value = self.device[description.key]["lower_bound"]
 
-    @property
-    def native_step(self) -> float:
-        """Return the setpoint step value."""
-        return max(
-            self.device[self.entity_description.key][
-                self.entity_description.native_step_key
-            ],
-            1,
-        )
+        native_step = self.device[description.key]["resolution"]
+        if description.key != "temperature_offset":
+            native_step = max(native_step, 0.5)
+        self._attr_native_step = native_step
 
     @property
     def native_value(self) -> float:
         """Return the present setpoint value."""
-        return self.device[self.entity_description.key][
-            self.entity_description.native_value_key
-        ]
-
-    @property
-    def native_min_value(self) -> float:
-        """Return the setpoint min. value."""
-        return self.device[self.entity_description.key][
-            self.entity_description.native_min_value_key
-        ]
-
-    @property
-    def native_max_value(self) -> float:
-        """Return the setpoint max. value."""
-        return self.device[self.entity_description.key][
-            self.entity_description.native_max_value_key
-        ]
+        return self.device[self.entity_description.key]["setpoint"]
 
     async def async_set_native_value(self, value: float) -> None:
         """Change to the new setpoint value."""
         await self.entity_description.command(
-            self.coordinator.api, self.entity_description.key, value
+            self.coordinator.api, self.entity_description.key, self.device_id, value
         )
         await self.coordinator.async_request_refresh()
