@@ -1,31 +1,27 @@
-"""This platform provides binary sensors for key RainMachine data."""
+"""Binary sensors for key RainMachine data."""
 from dataclasses import dataclass
 
 from homeassistant.components.binary_sensor import (
+    DOMAIN as BINARY_SENSOR_DOMAIN,
     BinarySensorEntity,
     BinarySensorEntityDescription,
 )
 from homeassistant.config_entries import ConfigEntry
+from homeassistant.const import EntityCategory
 from homeassistant.core import HomeAssistant, callback
-from homeassistant.helpers.entity import EntityCategory
 from homeassistant.helpers.entity_platform import AddEntitiesCallback
 
-from . import RainMachineEntity
-from .const import (
-    DATA_CONTROLLER,
-    DATA_COORDINATOR,
-    DATA_PROVISION_SETTINGS,
-    DATA_RESTRICTIONS_CURRENT,
-    DATA_RESTRICTIONS_UNIVERSAL,
-    DOMAIN,
+from . import RainMachineData, RainMachineEntity
+from .const import DATA_PROVISION_SETTINGS, DATA_RESTRICTIONS_CURRENT, DOMAIN
+from .model import RainMachineEntityDescription
+from .util import (
+    EntityDomainReplacementStrategy,
+    async_finish_entity_domain_replacements,
+    key_exists,
 )
-from .model import RainMachineDescriptionMixinApiCategory
-from .util import key_exists
 
 TYPE_FLOW_SENSOR = "flow_sensor"
 TYPE_FREEZE = "freeze"
-TYPE_FREEZE_PROTECTION = "freeze_protection"
-TYPE_HOT_DAYS = "extra_water_on_hot_days"
 TYPE_HOURLY = "hourly"
 TYPE_MONTH = "month"
 TYPE_RAINDELAY = "raindelay"
@@ -33,75 +29,58 @@ TYPE_RAINSENSOR = "rainsensor"
 TYPE_WEEKDAY = "weekday"
 
 
-@dataclass
+@dataclass(frozen=True, kw_only=True)
 class RainMachineBinarySensorDescription(
-    BinarySensorEntityDescription, RainMachineDescriptionMixinApiCategory
+    BinarySensorEntityDescription, RainMachineEntityDescription
 ):
     """Describe a RainMachine binary sensor."""
 
+    data_key: str
+
 
 BINARY_SENSOR_DESCRIPTIONS = (
     RainMachineBinarySensorDescription(
         key=TYPE_FLOW_SENSOR,
-        name="Flow Sensor",
+        translation_key=TYPE_FLOW_SENSOR,
         icon="mdi:water-pump",
         api_category=DATA_PROVISION_SETTINGS,
         data_key="useFlowSensor",
     ),
     RainMachineBinarySensorDescription(
         key=TYPE_FREEZE,
-        name="Freeze Restrictions",
+        translation_key=TYPE_FREEZE,
         icon="mdi:cancel",
         entity_category=EntityCategory.DIAGNOSTIC,
         api_category=DATA_RESTRICTIONS_CURRENT,
         data_key="freeze",
     ),
-    RainMachineBinarySensorDescription(
-        key=TYPE_FREEZE_PROTECTION,
-        name="Freeze Protection",
-        icon="mdi:weather-snowy",
-        entity_category=EntityCategory.DIAGNOSTIC,
-        api_category=DATA_RESTRICTIONS_UNIVERSAL,
-        data_key="freezeProtectEnabled",
-    ),
-    RainMachineBinarySensorDescription(
-        key=TYPE_HOT_DAYS,
-        name="Extra Water on Hot Days",
-        icon="mdi:thermometer-lines",
-        entity_category=EntityCategory.DIAGNOSTIC,
-        api_category=DATA_RESTRICTIONS_UNIVERSAL,
-        data_key="hotDaysExtraWatering",
-    ),
     RainMachineBinarySensorDescription(
         key=TYPE_HOURLY,
-        name="Hourly Restrictions",
+        translation_key=TYPE_HOURLY,
         icon="mdi:cancel",
         entity_category=EntityCategory.DIAGNOSTIC,
-        entity_registry_enabled_default=False,
         api_category=DATA_RESTRICTIONS_CURRENT,
         data_key="hourly",
     ),
     RainMachineBinarySensorDescription(
         key=TYPE_MONTH,
-        name="Month Restrictions",
+        translation_key=TYPE_MONTH,
         icon="mdi:cancel",
         entity_category=EntityCategory.DIAGNOSTIC,
-        entity_registry_enabled_default=False,
         api_category=DATA_RESTRICTIONS_CURRENT,
         data_key="month",
     ),
     RainMachineBinarySensorDescription(
         key=TYPE_RAINDELAY,
-        name="Rain Delay Restrictions",
+        translation_key=TYPE_RAINDELAY,
         icon="mdi:cancel",
         entity_category=EntityCategory.DIAGNOSTIC,
-        entity_registry_enabled_default=False,
         api_category=DATA_RESTRICTIONS_CURRENT,
         data_key="rainDelay",
     ),
     RainMachineBinarySensorDescription(
         key=TYPE_RAINSENSOR,
-        name="Rain Sensor Restrictions",
+        translation_key=TYPE_RAINSENSOR,
         icon="mdi:cancel",
         entity_category=EntityCategory.DIAGNOSTIC,
         entity_registry_enabled_default=False,
@@ -110,10 +89,9 @@ BINARY_SENSOR_DESCRIPTIONS = (
     ),
     RainMachineBinarySensorDescription(
         key=TYPE_WEEKDAY,
-        name="Weekday Restrictions",
+        translation_key=TYPE_WEEKDAY,
         icon="mdi:cancel",
         entity_category=EntityCategory.DIAGNOSTIC,
-        entity_registry_enabled_default=False,
         api_category=DATA_RESTRICTIONS_CURRENT,
         data_key="weekDay",
     ),
@@ -124,23 +102,43 @@ async def async_setup_entry(
     hass: HomeAssistant, entry: ConfigEntry, async_add_entities: AddEntitiesCallback
 ) -> None:
     """Set up RainMachine binary sensors based on a config entry."""
-    controller = hass.data[DOMAIN][entry.entry_id][DATA_CONTROLLER]
-    coordinators = hass.data[DOMAIN][entry.entry_id][DATA_COORDINATOR]
+    data: RainMachineData = hass.data[DOMAIN][entry.entry_id]
+
+    async_finish_entity_domain_replacements(
+        hass,
+        entry,
+        (
+            EntityDomainReplacementStrategy(
+                BINARY_SENSOR_DOMAIN,
+                f"{data.controller.mac}_freeze_protection",
+                f"switch.{data.controller.name.lower()}_freeze_protect_enabled",
+                breaks_in_ha_version="2022.12.0",
+                remove_old_entity=True,
+            ),
+            EntityDomainReplacementStrategy(
+                BINARY_SENSOR_DOMAIN,
+                f"{data.controller.mac}_extra_water_on_hot_days",
+                f"switch.{data.controller.name.lower()}_hot_days_extra_watering",
+                breaks_in_ha_version="2022.12.0",
+                remove_old_entity=True,
+            ),
+        ),
+    )
 
     api_category_sensor_map = {
         DATA_PROVISION_SETTINGS: ProvisionSettingsBinarySensor,
         DATA_RESTRICTIONS_CURRENT: CurrentRestrictionsBinarySensor,
-        DATA_RESTRICTIONS_UNIVERSAL: UniversalRestrictionsBinarySensor,
     }
 
     async_add_entities(
         [
-            api_category_sensor_map[description.api_category](
-                entry, coordinator, controller, description
-            )
+            api_category_sensor_map[description.api_category](entry, data, description)
             for description in BINARY_SENSOR_DESCRIPTIONS
-            if (coordinator := coordinators[description.api_category]) is not None
-            and key_exists(coordinator.data, description.data_key)
+            if (
+                (coordinator := data.coordinators[description.api_category]) is not None
+                and coordinator.data
+                and key_exists(coordinator.data, description.data_key)
+            )
         ]
     )
 
@@ -148,6 +146,8 @@ async def async_setup_entry(
 class CurrentRestrictionsBinarySensor(RainMachineEntity, BinarySensorEntity):
     """Define a binary sensor that handles current restrictions data."""
 
+    entity_description: RainMachineBinarySensorDescription
+
     @callback
     def update_from_latest_data(self) -> None:
         """Update the state."""
@@ -168,20 +168,12 @@ class CurrentRestrictionsBinarySensor(RainMachineEntity, BinarySensorEntity):
 class ProvisionSettingsBinarySensor(RainMachineEntity, BinarySensorEntity):
     """Define a binary sensor that handles provisioning data."""
 
+    entity_description: RainMachineBinarySensorDescription
+
     @callback
     def update_from_latest_data(self) -> None:
         """Update the state."""
         if self.entity_description.key == TYPE_FLOW_SENSOR:
-            self._attr_is_on = self.coordinator.data["system"].get("useFlowSensor")
-
-
-class UniversalRestrictionsBinarySensor(RainMachineEntity, BinarySensorEntity):
-    """Define a binary sensor that handles universal restrictions data."""
-
-    @callback
-    def update_from_latest_data(self) -> None:
-        """Update the state."""
-        if self.entity_description.key == TYPE_FREEZE_PROTECTION:
-            self._attr_is_on = self.coordinator.data.get("freezeProtectEnabled")
-        elif self.entity_description.key == TYPE_HOT_DAYS:
-            self._attr_is_on = self.coordinator.data.get("hotDaysExtraWatering")
+            self._attr_is_on = self.coordinator.data.get("system", {}).get(
+                "useFlowSensor"
+            )
