@@ -1,27 +1,26 @@
-"""This component provides support for RainMachine programs and zones."""
+"""Component providing support for RainMachine programs and zones."""
 from __future__ import annotations
 
 import asyncio
 from collections.abc import Awaitable, Callable, Coroutine
 from dataclasses import dataclass
 from datetime import datetime
-from typing import Any, TypeVar
+from typing import Any, Concatenate, ParamSpec, TypeVar
 
 from regenmaschine.errors import RainMachineError
-from typing_extensions import Concatenate, ParamSpec
 import voluptuous as vol
 
 from homeassistant.components.switch import SwitchEntity, SwitchEntityDescription
 from homeassistant.config_entries import ConfigEntry
-from homeassistant.const import ATTR_ID
+from homeassistant.const import ATTR_ID, EntityCategory
 from homeassistant.core import HomeAssistant, callback
 from homeassistant.exceptions import HomeAssistantError
 from homeassistant.helpers import config_validation as cv, entity_platform
-from homeassistant.helpers.entity import EntityCategory
 from homeassistant.helpers.entity_platform import AddEntitiesCallback
 
 from . import RainMachineData, RainMachineEntity, async_update_programs_and_zones
 from .const import (
+    CONF_ALLOW_INACTIVE_ZONES_TO_RUN,
     CONF_DEFAULT_ZONE_RUN_TIME,
     CONF_DURATION,
     CONF_USE_APP_RUN_TIMES,
@@ -32,11 +31,7 @@ from .const import (
     DEFAULT_ZONE_RUN,
     DOMAIN,
 )
-from .model import (
-    RainMachineEntityDescription,
-    RainMachineEntityDescriptionMixinDataKey,
-    RainMachineEntityDescriptionMixinUid,
-)
+from .model import RainMachineEntityDescription
 from .util import RUN_STATE_MAP, key_exists
 
 ATTR_AREA = "area"
@@ -119,7 +114,7 @@ _P = ParamSpec("_P")
 
 
 def raise_on_request_error(
-    func: Callable[Concatenate[_T, _P], Awaitable[None]]
+    func: Callable[Concatenate[_T, _P], Awaitable[None]],
 ) -> Callable[Concatenate[_T, _P], Coroutine[Any, Any, None]]:
     """Define a decorator to raise on a request error."""
 
@@ -135,27 +130,26 @@ def raise_on_request_error(
     return decorator
 
 
-@dataclass
+@dataclass(frozen=True, kw_only=True)
 class RainMachineSwitchDescription(
-    SwitchEntityDescription,
-    RainMachineEntityDescription,
+    SwitchEntityDescription, RainMachineEntityDescription
 ):
     """Describe a RainMachine switch."""
 
 
-@dataclass
-class RainMachineActivitySwitchDescription(
-    RainMachineSwitchDescription, RainMachineEntityDescriptionMixinUid
-):
+@dataclass(frozen=True, kw_only=True)
+class RainMachineActivitySwitchDescription(RainMachineSwitchDescription):
     """Describe a RainMachine activity (program/zone) switch."""
 
+    uid: int
 
-@dataclass
-class RainMachineRestrictionSwitchDescription(
-    RainMachineSwitchDescription, RainMachineEntityDescriptionMixinDataKey
-):
+
+@dataclass(frozen=True, kw_only=True)
+class RainMachineRestrictionSwitchDescription(RainMachineSwitchDescription):
     """Describe a RainMachine restriction switch."""
 
+    data_key: str
+
 
 TYPE_RESTRICTIONS_FREEZE_PROTECT_ENABLED = "freeze_protect_enabled"
 TYPE_RESTRICTIONS_HOT_DAYS_EXTRA_WATERING = "hot_days_extra_watering"
@@ -163,14 +157,14 @@ TYPE_RESTRICTIONS_HOT_DAYS_EXTRA_WATERING = "hot_days_extra_watering"
 RESTRICTIONS_SWITCH_DESCRIPTIONS = (
     RainMachineRestrictionSwitchDescription(
         key=TYPE_RESTRICTIONS_FREEZE_PROTECT_ENABLED,
-        name="Freeze protection",
+        translation_key=TYPE_RESTRICTIONS_FREEZE_PROTECT_ENABLED,
         icon="mdi:snowflake-alert",
         api_category=DATA_RESTRICTIONS_UNIVERSAL,
         data_key="freezeProtectEnabled",
     ),
     RainMachineRestrictionSwitchDescription(
         key=TYPE_RESTRICTIONS_HOT_DAYS_EXTRA_WATERING,
-        name="Extra water on hot days",
+        translation_key=TYPE_RESTRICTIONS_HOT_DAYS_EXTRA_WATERING,
         icon="mdi:heat-wave",
         api_category=DATA_RESTRICTIONS_UNIVERSAL,
         data_key="hotDaysExtraWatering",
@@ -302,7 +296,10 @@ class RainMachineActivitySwitch(RainMachineBaseSwitch):
         The only way this could occur is if someone rapidly turns a disabled activity
         off right after turning it on.
         """
-        if not self.coordinator.data[self.entity_description.uid]["active"]:
+        if (
+            not self._entry.options[CONF_ALLOW_INACTIVE_ZONES_TO_RUN]
+            and not self.coordinator.data[self.entity_description.uid]["active"]
+        ):
             raise HomeAssistantError(
                 f"Cannot turn off an inactive program/zone: {self.name}"
             )
@@ -316,7 +313,10 @@ class RainMachineActivitySwitch(RainMachineBaseSwitch):
 
     async def async_turn_on(self, **kwargs: Any) -> None:
         """Turn the switch on."""
-        if not self.coordinator.data[self.entity_description.uid]["active"]:
+        if (
+            not self._entry.options[CONF_ALLOW_INACTIVE_ZONES_TO_RUN]
+            and not self.coordinator.data[self.entity_description.uid]["active"]
+        ):
             self._attr_is_on = False
             self.async_write_ha_state()
             raise HomeAssistantError(
