@@ -4,6 +4,8 @@ from __future__ import annotations
 import dataclasses
 from typing import Any
 
+from aiorecollect.client import PickupEvent
+
 from homeassistant.components.diagnostics import async_redact_data
 from homeassistant.config_entries import ConfigEntry
 from homeassistant.const import CONF_UNIQUE_ID
@@ -28,7 +30,9 @@ async def async_get_config_entry_diagnostics(
     hass: HomeAssistant, entry: ConfigEntry
 ) -> dict[str, Any]:
     """Return diagnostics for a config entry."""
-    coordinator: DataUpdateCoordinator = hass.data[DOMAIN][entry.entry_id]
+    coordinator: DataUpdateCoordinator[list[PickupEvent]] = hass.data[DOMAIN][
+        entry.entry_id
+    ]
 
     return async_redact_data(
         {
