@@ -1,11 +1,17 @@
 """Support for binary sensor using RPi GPIO."""
+from __future__ import annotations
+
 import requests
 import voluptuous as vol
 
 from homeassistant.components.binary_sensor import PLATFORM_SCHEMA, BinarySensorEntity
 from homeassistant.const import CONF_HOST
+from homeassistant.core import HomeAssistant
 import homeassistant.helpers.config_validation as cv
+from homeassistant.helpers.entity_platform import AddEntitiesCallback
+from homeassistant.helpers.typing import ConfigType, DiscoveryInfoType
 
+from .. import remote_rpi_gpio
 from . import (
     CONF_BOUNCETIME,
     CONF_INVERT_LOGIC,
@@ -14,7 +20,6 @@ from . import (
     DEFAULT_INVERT_LOGIC,
     DEFAULT_PULL_MODE,
 )
-from .. import remote_rpi_gpio
 
 CONF_PORTS = "ports"
 
@@ -31,7 +36,12 @@ PLATFORM_SCHEMA = PLATFORM_SCHEMA.extend(
 )
 
 
-def setup_platform(hass, config, add_entities, discovery_info=None):
+def setup_platform(
+    hass: HomeAssistant,
+    config: ConfigType,
+    add_entities: AddEntitiesCallback,
+    discovery_info: DiscoveryInfoType | None = None,
+) -> None:
     """Set up the Raspberry PI GPIO devices."""
     address = config["host"]
     invert_logic = config[CONF_INVERT_LOGIC]
@@ -56,6 +66,8 @@ def setup_platform(hass, config, add_entities, discovery_info=None):
 class RemoteRPiGPIOBinarySensor(BinarySensorEntity):
     """Represent a binary sensor that uses a Remote Raspberry Pi GPIO."""
 
+    _attr_should_poll = False
+
     def __init__(self, name, sensor, invert_logic):
         """Initialize the RPi binary sensor."""
         self._name = name
@@ -63,7 +75,7 @@ class RemoteRPiGPIOBinarySensor(BinarySensorEntity):
         self._state = False
         self._sensor = sensor
 
-    async def async_added_to_hass(self):
+    async def async_added_to_hass(self) -> None:
         """Run when entity about to be added to hass."""
 
         def read_gpio():
@@ -74,11 +86,6 @@ class RemoteRPiGPIOBinarySensor(BinarySensorEntity):
         self._sensor.when_deactivated = read_gpio
         self._sensor.when_activated = read_gpio
 
-    @property
-    def should_poll(self):
-        """No polling needed."""
-        return False
-
     @property
     def name(self):
         """Return the name of the sensor."""
@@ -94,7 +101,7 @@ class RemoteRPiGPIOBinarySensor(BinarySensorEntity):
         """Return the class of this sensor, from DEVICE_CLASSES."""
         return
 
-    def update(self):
+    def update(self) -> None:
         """Update the GPIO state."""
         try:
             self._state = remote_rpi_gpio.read_input(self._sensor)
