@@ -10,15 +10,22 @@ import voluptuous as vol
 
 from homeassistant import config_entries
 from homeassistant.components import dhcp
-from homeassistant.const import CONF_HOST, CONF_PASSWORD, CONF_PORT, CONF_USERNAME
+from homeassistant.const import (
+    CONF_HOST,
+    CONF_PASSWORD,
+    CONF_PORT,
+    CONF_PROTOCOL,
+    CONF_USERNAME,
+)
 from homeassistant.core import callback
-from homeassistant.data_entry_flow import FlowResult
+from homeassistant.data_entry_flow import AbortFlow, FlowResult
 from homeassistant.helpers import config_validation as cv
 from homeassistant.helpers.device_registry import format_mac
 
-from .const import CONF_PROTOCOL, CONF_USE_HTTPS, DOMAIN
-from .exceptions import ReolinkException, UserNotAdmin
+from .const import CONF_USE_HTTPS, DOMAIN
+from .exceptions import ReolinkException, ReolinkWebhookException, UserNotAdmin
 from .host import ReolinkHost
+from .util import is_connected
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -79,6 +86,10 @@ class ReolinkFlowHandler(config_entries.ConfigFlow, domain=DOMAIN):
         self._username = entry_data[CONF_USERNAME]
         self._password = entry_data[CONF_PASSWORD]
         self._reauth = True
+        self.context["title_placeholders"]["ip_address"] = entry_data[CONF_HOST]
+        self.context["title_placeholders"]["hostname"] = self.context[
+            "title_placeholders"
+        ]["name"]
         return await self.async_step_reauth_confirm()
 
     async def async_step_reauth_confirm(
@@ -92,13 +103,53 @@ class ReolinkFlowHandler(config_entries.ConfigFlow, domain=DOMAIN):
     async def async_step_dhcp(self, discovery_info: dhcp.DhcpServiceInfo) -> FlowResult:
         """Handle discovery via dhcp."""
         mac_address = format_mac(discovery_info.macaddress)
-        await self.async_set_unique_id(mac_address)
+        existing_entry = await self.async_set_unique_id(mac_address)
+        if (
+            existing_entry
+            and CONF_PASSWORD in existing_entry.data
+            and existing_entry.data[CONF_HOST] != discovery_info.ip
+        ):
+            if is_connected(self.hass, existing_entry):
+                _LOGGER.debug(
+                    "Reolink DHCP reported new IP '%s', "
+                    "but connection to camera seems to be okay, so sticking to IP '%s'",
+                    discovery_info.ip,
+                    existing_entry.data[CONF_HOST],
+                )
+                raise AbortFlow("already_configured")
+
+            # check if the camera is reachable at the new IP
+            new_config = dict(existing_entry.data)
+            new_config[CONF_HOST] = discovery_info.ip
+            host = ReolinkHost(self.hass, new_config, existing_entry.options)
+            try:
+                await host.api.get_state("GetLocalLink")
+                await host.api.logout()
+            except ReolinkError as err:
+                _LOGGER.debug(
+                    "Reolink DHCP reported new IP '%s', "
+                    "but got error '%s' trying to connect, so sticking to IP '%s'",
+                    discovery_info.ip,
+                    err,
+                    existing_entry.data[CONF_HOST],
+                )
+                raise AbortFlow("already_configured") from err
+            if format_mac(host.api.mac_address) != mac_address:
+                _LOGGER.debug(
+                    "Reolink mac address '%s' at new IP '%s' from DHCP, "
+                    "does not match mac '%s' of config entry, so sticking to IP '%s'",
+                    format_mac(host.api.mac_address),
+                    discovery_info.ip,
+                    mac_address,
+                    existing_entry.data[CONF_HOST],
+                )
+                raise AbortFlow("already_configured")
+
         self._abort_if_unique_id_configured(updates={CONF_HOST: discovery_info.ip})
 
-        short_mac = mac_address[-8:].upper()
         self.context["title_placeholders"] = {
-            "short_mac": short_mac,
             "ip_address": discovery_info.ip,
+            "hostname": discovery_info.hostname,
         }
 
         self._host = discovery_info.ip
@@ -109,7 +160,10 @@ class ReolinkFlowHandler(config_entries.ConfigFlow, domain=DOMAIN):
     ) -> FlowResult:
         """Handle the initial step."""
         errors = {}
-        placeholders = {"error": ""}
+        placeholders = {
+            "error": "",
+            "troubleshooting_link": "https://www.home-assistant.io/integrations/reolink/#troubleshooting",
+        }
 
         if user_input is not None:
             if CONF_HOST not in user_input:
@@ -127,6 +181,12 @@ class ReolinkFlowHandler(config_entries.ConfigFlow, domain=DOMAIN):
             except ApiError as err:
                 placeholders["error"] = str(err)
                 errors[CONF_HOST] = "api_error"
+            except ReolinkWebhookException as err:
+                placeholders["error"] = str(err)
+                placeholders[
+                    "more_info"
+                ] = "https://www.home-assistant.io/more-info/no-url-available/#configuring-the-instance-url"
+                errors["base"] = "webhook_exception"
             except (ReolinkError, ReolinkException) as err:
                 placeholders["error"] = str(err)
                 errors[CONF_HOST] = "cannot_connect"
@@ -176,7 +236,7 @@ class ReolinkFlowHandler(config_entries.ConfigFlow, domain=DOMAIN):
             data_schema = data_schema.extend(
                 {
                     vol.Optional(CONF_PORT): cv.positive_int,
-                    vol.Optional(CONF_USE_HTTPS): bool,
+                    vol.Required(CONF_USE_HTTPS, default=False): bool,
                 }
             )
 
