@@ -1,10 +1,11 @@
 """Support for Rflink devices."""
+from __future__ import annotations
+
 import asyncio
 from collections import defaultdict
 import logging
 
-import async_timeout
-from rflink.protocol import create_rflink_connection
+from rflink.protocol import ProtocolBase, create_rflink_connection
 from serial import SerialException
 import voluptuous as vol
 
@@ -18,7 +19,7 @@ from homeassistant.const import (
     EVENT_HOMEASSISTANT_STOP,
     STATE_ON,
 )
-from homeassistant.core import CoreState, callback
+from homeassistant.core import CoreState, HassJob, HomeAssistant, ServiceCall, callback
 from homeassistant.exceptions import HomeAssistantError
 import homeassistant.helpers.config_validation as cv
 from homeassistant.helpers.dispatcher import (
@@ -26,7 +27,11 @@ from homeassistant.helpers.dispatcher import (
     async_dispatcher_send,
 )
 from homeassistant.helpers.entity import Entity
+from homeassistant.helpers.event import async_call_later
 from homeassistant.helpers.restore_state import RestoreEntity
+from homeassistant.helpers.typing import ConfigType
+
+from .utils import brightness_to_rflink
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -67,6 +72,7 @@ SERVICE_SEND_COMMAND = "send_command"
 
 SIGNAL_AVAILABILITY = "rflink_device_available"
 SIGNAL_HANDLE_EVENT = "rflink_handle_event_{}"
+SIGNAL_EVENT = "rflink_event"
 
 TMP_ENTITY = "tmp.{}"
 
@@ -118,7 +124,7 @@ def identify_event_type(event):
     return "unknown"
 
 
-async def async_setup(hass, config):
+async def async_setup(hass: HomeAssistant, config: ConfigType) -> bool:
     """Set up the Rflink component."""
     # Allow entities to register themselves by device_id to be looked up when
     # new rflink events arrive to be handled
@@ -131,7 +137,7 @@ async def async_setup(hass, config):
     # Allow platform to specify function to register new unknown devices
     hass.data[DATA_DEVICE_REGISTER] = {}
 
-    async def async_send_command(call):
+    async def async_send_command(call: ServiceCall) -> None:
         """Send Rflink command."""
         _LOGGER.debug("Rflink command for %s", str(call.data))
         if not (
@@ -140,6 +146,15 @@ async def async_setup(hass, config):
             )
         ):
             _LOGGER.error("Failed Rflink command for %s", str(call.data))
+        else:
+            async_dispatcher_send(
+                hass,
+                SIGNAL_EVENT,
+                {
+                    EVENT_KEY_ID: call.data.get(CONF_DEVICE_ID),
+                    EVENT_KEY_COMMAND: call.data.get(CONF_COMMAND),
+                },
+            )
 
     hass.services.async_register(
         DOMAIN, SERVICE_SEND_COMMAND, async_send_command, schema=SEND_COMMAND_SCHEMA
@@ -210,9 +225,11 @@ async def async_setup(hass, config):
         keepalive_idle_timer = config[DOMAIN][CONF_KEEPALIVE_IDLE]
         if keepalive_idle_timer < 0:
             _LOGGER.error(
-                "A bogus TCP Keepalive IDLE timer was provided (%d secs), "
-                "it will be disabled. "
-                "Recommended values: 60-3600 (seconds)",
+                (
+                    "A bogus TCP Keepalive IDLE timer was provided (%d secs), "
+                    "it will be disabled. "
+                    "Recommended values: 60-3600 (seconds)"
+                ),
                 keepalive_idle_timer,
             )
             keepalive_idle_timer = None
@@ -220,14 +237,16 @@ async def async_setup(hass, config):
             keepalive_idle_timer = None
         elif keepalive_idle_timer <= 30:
             _LOGGER.warning(
-                "A very short TCP Keepalive IDLE timer was provided (%d secs) "
-                "and may produce unexpected disconnections from RFlink device."
-                " Recommended values: 60-3600 (seconds)",
+                (
+                    "A very short TCP Keepalive IDLE timer was provided (%d secs) "
+                    "and may produce unexpected disconnections from RFlink device."
+                    " Recommended values: 60-3600 (seconds)"
+                ),
                 keepalive_idle_timer,
             )
 
     @callback
-    def reconnect(exc=None):
+    def reconnect(_: Exception | None = None) -> None:
         """Schedule reconnect after connection has been unexpectedly lost."""
         # Reset protocol binding before starting reconnect
         RflinkCommand.set_rflink_protocol(None)
@@ -235,10 +254,12 @@ async def async_setup(hass, config):
         async_dispatcher_send(hass, SIGNAL_AVAILABILITY, False)
 
         # If HA is not stopping, initiate new connection
-        if hass.state != CoreState.stopping:
-            _LOGGER.warning("disconnected from Rflink, reconnecting")
+        if hass.state is not CoreState.stopping:
+            _LOGGER.warning("Disconnected from Rflink, reconnecting")
             hass.async_create_task(connect())
 
+    _reconnect_job = HassJob(reconnect, "Rflink reconnect", cancel_on_shutdown=True)
+
     async def connect():
         """Set up connection and hook it into HA for reconnect/shutdown."""
         _LOGGER.info("Initiating Rflink connection")
@@ -258,16 +279,14 @@ async def async_setup(hass, config):
         )
 
         try:
-            with async_timeout.timeout(CONNECTION_TIMEOUT):
+            async with asyncio.timeout(CONNECTION_TIMEOUT):
                 transport, protocol = await connection
 
         except (
             SerialException,
-            ConnectionRefusedError,
-            TimeoutError,
             OSError,
             asyncio.TimeoutError,
-        ) as exc:
+        ):
             reconnect_interval = config[DOMAIN][CONF_RECONNECT_INTERVAL]
             _LOGGER.exception(
                 "Error connecting to Rflink, reconnecting in %s", reconnect_interval
@@ -275,7 +294,7 @@ async def async_setup(hass, config):
             # Connection to Rflink device is lost, make entities unavailable
             async_dispatcher_send(hass, SIGNAL_AVAILABILITY, False)
 
-            hass.loop.call_later(reconnect_interval, reconnect, exc)
+            async_call_later(hass, reconnect_interval, _reconnect_job)
             return
 
         # There is a valid connection to a Rflink device now so
@@ -293,6 +312,7 @@ async def async_setup(hass, config):
         _LOGGER.info("Connected to Rflink")
 
     hass.async_create_task(connect())
+    async_dispatcher_connect(hass, SIGNAL_EVENT, event_callback)
     return True
 
 
@@ -302,9 +322,9 @@ class RflinkDevice(Entity):
     Contains the common logic for Rflink entities.
     """
 
-    platform = None
-    _state = None
+    _state: bool | None = None
     _available = True
+    _attr_should_poll = False
 
     def __init__(
         self,
@@ -322,6 +342,7 @@ class RflinkDevice(Entity):
         # Rflink specific attributes for every component type
         self._initial_event = initial_event
         self._device_id = device_id
+        self._attr_unique_id = device_id
         if name:
             self._name = name
         else:
@@ -357,11 +378,6 @@ class RflinkDevice(Entity):
         """Platform specific event handler."""
         raise NotImplementedError()
 
-    @property
-    def should_poll(self):
-        """No polling needed."""
-        return False
-
     @property
     def name(self):
         """Return a name for the device."""
@@ -463,12 +479,16 @@ class RflinkCommand(RflinkDevice):
 
     # Keep repetition tasks to cancel if state is changed before repetitions
     # are sent
-    _repetition_task = None
+    _repetition_task: asyncio.Task[None] | None = None
 
-    _protocol = None
+    _protocol: ProtocolBase | None = None
+
+    _wait_ack: bool | None = None
 
     @classmethod
-    def set_rflink_protocol(cls, protocol, wait_ack=None):
+    def set_rflink_protocol(
+        cls, protocol: ProtocolBase | None, wait_ack: bool | None = None
+    ) -> None:
         """Set the Rflink asyncio protocol as a class variable."""
         cls._protocol = protocol
         if wait_ack is not None:
@@ -498,7 +518,7 @@ class RflinkCommand(RflinkDevice):
 
         elif command == "dim":
             # convert brightness to rflink dim level
-            cmd = str(int(args[0] / 17))
+            cmd = str(brightness_to_rflink(args[0]))
             self._state = True
 
         elif command == "toggle":
@@ -569,9 +589,7 @@ class SwitchableRflinkDevice(RflinkCommand, RestoreEntity):
     async def async_added_to_hass(self):
         """Restore RFLink device state (ON/OFF)."""
         await super().async_added_to_hass()
-
-        old_state = await self.async_get_last_state()
-        if old_state is not None:
+        if (old_state := await self.async_get_last_state()) is not None:
             self._state = old_state.state == STATE_ON
 
     def _handle_event(self, event):
