@@ -1,13 +1,19 @@
 """Config flow for RFXCOM RFXtrx integration."""
+from __future__ import annotations
+
+import asyncio
+from contextlib import suppress
 import copy
+import itertools
 import os
+from typing import Any, TypedDict, cast
 
 import RFXtrx as rfxtrxmod
 import serial
 import serial.tools.list_ports
 import voluptuous as vol
 
-from homeassistant import config_entries, exceptions
+from homeassistant import config_entries, data_entry_flow, exceptions
 from homeassistant.config_entries import ConfigEntry
 from homeassistant.const import (
     CONF_COMMAND_OFF,
@@ -19,42 +25,49 @@ from homeassistant.const import (
     CONF_PORT,
     CONF_TYPE,
 )
-from homeassistant.core import callback
-from homeassistant.helpers import config_validation as cv
-from homeassistant.helpers.device_registry import (
-    async_entries_for_config_entry,
-    async_get_registry as async_get_device_registry,
-)
-from homeassistant.helpers.entity_registry import (
-    async_entries_for_device,
-    async_get_registry as async_get_entity_registry,
+from homeassistant.core import State, callback
+from homeassistant.helpers import (
+    config_validation as cv,
+    device_registry as dr,
+    entity_registry as er,
 )
+from homeassistant.helpers.event import async_track_state_change
 
-from . import DOMAIN, get_device_id, get_rfx_object
+from . import (
+    DOMAIN,
+    DeviceTuple,
+    get_device_id,
+    get_device_tuple_from_identifiers,
+    get_rfx_object,
+)
 from .binary_sensor import supported as binary_supported
 from .const import (
     CONF_AUTOMATIC_ADD,
     CONF_DATA_BITS,
-    CONF_FIRE_EVENT,
     CONF_OFF_DELAY,
-    CONF_REMOVE_DEVICE,
+    CONF_PROTOCOLS,
     CONF_REPLACE_DEVICE,
-    CONF_SIGNAL_REPETITIONS,
     CONF_VENETIAN_BLIND_MODE,
     CONST_VENETIAN_BLIND_MODE_DEFAULT,
     CONST_VENETIAN_BLIND_MODE_EU,
     CONST_VENETIAN_BLIND_MODE_US,
     DEVICE_PACKET_TYPE_LIGHTING4,
 )
-from .cover import supported as cover_supported
-from .light import supported as light_supported
-from .switch import supported as switch_supported
 
 CONF_EVENT_CODE = "event_code"
 CONF_MANUAL_PATH = "Enter Manually"
 
+RECV_MODES = sorted(itertools.chain(*rfxtrxmod.lowlevel.Status.RECMODES))
 
-def none_or_int(value, base):
+
+class DeviceData(TypedDict):
+    """Dict data representing a device entry."""
+
+    event_code: str | None
+    device_id: DeviceTuple
+
+
+def none_or_int(value: str | None, base: int) -> int | None:
     """Check if strin is one otherwise convert to int."""
     if value is None:
         return None
@@ -64,62 +77,51 @@ def none_or_int(value, base):
 class OptionsFlow(config_entries.OptionsFlow):
     """Handle Rfxtrx options."""
 
+    _device_registry: dr.DeviceRegistry
+    _device_entries: list[dr.DeviceEntry]
+
     def __init__(self, config_entry: ConfigEntry) -> None:
         """Initialize rfxtrx options flow."""
         self._config_entry = config_entry
-        self._global_options = None
-        self._selected_device = None
-        self._selected_device_entry_id = None
-        self._selected_device_event_code = None
-        self._selected_device_object = None
-        self._device_entries = None
-        self._device_registry = None
+        self._global_options: dict[str, Any] = {}
+        self._selected_device: dict[str, Any] = {}
+        self._selected_device_entry_id: str | None = None
+        self._selected_device_event_code: str | None = None
+        self._selected_device_object: rfxtrxmod.RFXtrxEvent | None = None
 
-    async def async_step_init(self, user_input=None):
+    async def async_step_init(
+        self, user_input: dict[str, Any] | None = None
+    ) -> data_entry_flow.FlowResult:
         """Manage the options."""
         return await self.async_step_prompt_options()
 
-    async def async_step_prompt_options(self, user_input=None):
+    async def async_step_prompt_options(
+        self, user_input: dict[str, Any] | None = None
+    ) -> data_entry_flow.FlowResult:
         """Prompt for options."""
         errors = {}
 
         if user_input is not None:
             self._global_options = {
                 CONF_AUTOMATIC_ADD: user_input[CONF_AUTOMATIC_ADD],
+                CONF_PROTOCOLS: user_input[CONF_PROTOCOLS] or None,
             }
             if CONF_DEVICE in user_input:
                 entry_id = user_input[CONF_DEVICE]
                 device_data = self._get_device_data(entry_id)
                 self._selected_device_entry_id = entry_id
-                event_code = device_data[CONF_EVENT_CODE]
+                event_code = device_data["event_code"]
+                assert event_code
                 self._selected_device_event_code = event_code
                 self._selected_device = self._config_entry.data[CONF_DEVICES][
                     event_code
                 ]
                 self._selected_device_object = get_rfx_object(event_code)
                 return await self.async_step_set_device_options()
-            if CONF_REMOVE_DEVICE in user_input:
-                remove_devices = user_input[CONF_REMOVE_DEVICE]
-                devices = {}
-                for entry_id in remove_devices:
-                    device_data = self._get_device_data(entry_id)
-
-                    event_code = device_data[CONF_EVENT_CODE]
-                    device_id = device_data[CONF_DEVICE_ID]
-                    self.hass.helpers.dispatcher.async_dispatcher_send(
-                        f"{DOMAIN}_{CONF_REMOVE_DEVICE}_{device_id}"
-                    )
-                    self._device_registry.async_remove_device(entry_id)
-                    if event_code is not None:
-                        devices[event_code] = None
-
-                self.update_config_data(
-                    global_options=self._global_options, devices=devices
-                )
-
-                return self.async_create_entry(title="", data={})
             if CONF_EVENT_CODE in user_input:
-                self._selected_device_event_code = user_input[CONF_EVENT_CODE]
+                self._selected_device_event_code = cast(
+                    str, user_input[CONF_EVENT_CODE]
+                )
                 self._selected_device = {}
                 selected_device_object = get_rfx_object(
                     self._selected_device_event_code
@@ -137,18 +139,13 @@ class OptionsFlow(config_entries.OptionsFlow):
 
                 return self.async_create_entry(title="", data={})
 
-        device_registry = await async_get_device_registry(self.hass)
-        device_entries = async_entries_for_config_entry(
+        device_registry = dr.async_get(self.hass)
+        device_entries = dr.async_entries_for_config_entry(
             device_registry, self._config_entry.entry_id
         )
         self._device_registry = device_registry
         self._device_entries = device_entries
 
-        remove_devices = {
-            entry.id: entry.name_by_user if entry.name_by_user else entry.name
-            for entry in device_entries
-        }
-
         configure_devices = {
             entry.id: entry.name_by_user if entry.name_by_user else entry.name
             for entry in device_entries
@@ -160,20 +157,29 @@ class OptionsFlow(config_entries.OptionsFlow):
                 CONF_AUTOMATIC_ADD,
                 default=self._config_entry.data[CONF_AUTOMATIC_ADD],
             ): bool,
+            vol.Optional(
+                CONF_PROTOCOLS,
+                default=self._config_entry.data.get(CONF_PROTOCOLS) or [],
+            ): cv.multi_select(RECV_MODES),
             vol.Optional(CONF_EVENT_CODE): str,
             vol.Optional(CONF_DEVICE): vol.In(configure_devices),
-            vol.Optional(CONF_REMOVE_DEVICE): cv.multi_select(remove_devices),
         }
 
         return self.async_show_form(
             step_id="prompt_options", data_schema=vol.Schema(options), errors=errors
         )
 
-    async def async_step_set_device_options(self, user_input=None):
+    async def async_step_set_device_options(
+        self, user_input: dict[str, Any] | None = None
+    ) -> data_entry_flow.FlowResult:
         """Manage device options."""
         errors = {}
+        assert self._selected_device_object
+        assert self._selected_device_event_code
 
         if user_input is not None:
+            devices: dict[str, dict[str, Any] | None] = {}
+            device: dict[str, Any]
             device_id = get_device_id(
                 self._selected_device_object.device,
                 data_bits=user_input.get(CONF_DATA_BITS),
@@ -207,9 +213,7 @@ class OptionsFlow(config_entries.OptionsFlow):
             if not errors:
                 devices = {}
                 device = {
-                    CONF_DEVICE_ID: device_id,
-                    CONF_FIRE_EVENT: user_input.get(CONF_FIRE_EVENT, False),
-                    CONF_SIGNAL_REPETITIONS: user_input.get(CONF_SIGNAL_REPETITIONS, 1),
+                    CONF_DEVICE_ID: list(device_id),
                 }
 
                 devices[self._selected_device_event_code] = device
@@ -235,11 +239,7 @@ class OptionsFlow(config_entries.OptionsFlow):
 
         device_data = self._selected_device
 
-        data_schema = {
-            vol.Optional(
-                CONF_FIRE_EVENT, default=device_data.get(CONF_FIRE_EVENT, False)
-            ): bool,
-        }
+        data_schema = {}
 
         if binary_supported(self._selected_device_object):
             if device_data.get(CONF_OFF_DELAY):
@@ -255,21 +255,6 @@ class OptionsFlow(config_entries.OptionsFlow):
                 }
             data_schema.update(off_delay_schema)
 
-        if (
-            binary_supported(self._selected_device_object)
-            or cover_supported(self._selected_device_object)
-            or light_supported(self._selected_device_object)
-            or switch_supported(self._selected_device_object)
-        ):
-            data_schema.update(
-                {
-                    vol.Optional(
-                        CONF_SIGNAL_REPETITIONS,
-                        default=device_data.get(CONF_SIGNAL_REPETITIONS, 1),
-                    ): int,
-                }
-            )
-
         if (
             self._selected_device_object.device.packettype
             == DEVICE_PACKET_TYPE_LIGHTING4
@@ -307,16 +292,16 @@ class OptionsFlow(config_entries.OptionsFlow):
                     ),
                 }
             )
-        devices = {
+        replace_devices = {
             entry.id: entry.name_by_user if entry.name_by_user else entry.name
             for entry in self._device_entries
             if self._can_replace_device(entry.id)
         }
 
-        if devices:
+        if replace_devices:
             data_schema.update(
                 {
-                    vol.Optional(CONF_REPLACE_DEVICE): vol.In(devices),
+                    vol.Optional(CONF_REPLACE_DEVICE): vol.In(replace_devices),
                 }
             )
 
@@ -326,11 +311,13 @@ class OptionsFlow(config_entries.OptionsFlow):
             errors=errors,
         )
 
-    async def _async_replace_device(self, replace_device):
+    async def _async_replace_device(self, replace_device: str) -> None:
         """Migrate properties of a device into another."""
         device_registry = self._device_registry
         old_device = self._selected_device_entry_id
+        assert old_device
         old_entry = device_registry.async_get(old_device)
+        assert old_entry
         device_registry.async_update_device(
             replace_device,
             area_id=old_entry.area_id,
@@ -343,8 +330,8 @@ class OptionsFlow(config_entries.OptionsFlow):
         old_device_id = "_".join(x for x in old_device_data[CONF_DEVICE_ID])
         new_device_id = "_".join(x for x in new_device_data[CONF_DEVICE_ID])
 
-        entity_registry = await async_get_entity_registry(self.hass)
-        entity_entries = async_entries_for_device(
+        entity_registry = er.async_get(self.hass)
+        entity_entries = er.async_entries_for_device(
             entity_registry, old_device, include_disabled_entities=True
         )
         entity_migration_map = {}
@@ -359,9 +346,58 @@ class OptionsFlow(config_entries.OptionsFlow):
             if new_entity_id is not None:
                 entity_migration_map[new_entity_id] = entry
 
+        @callback
+        def _handle_state_removed(
+            entity_id: str, old_state: State | None, new_state: State | None
+        ) -> None:
+            # Wait for entities to finish cleanup
+            if new_state is None and entity_id in entities_to_be_removed:
+                entities_to_be_removed.remove(entity_id)
+            if not entities_to_be_removed:
+                wait_for_entities.set()
+
+        # Create a set with entities to be removed which are currently in the state
+        # machine
+        entities_to_be_removed = {
+            entry.entity_id
+            for entry in entity_migration_map.values()
+            if not self.hass.states.async_available(entry.entity_id)
+        }
+        wait_for_entities = asyncio.Event()
+        remove_track_state_changes = async_track_state_change(
+            self.hass, entities_to_be_removed, _handle_state_removed
+        )
+
         for entry in entity_migration_map.values():
             entity_registry.async_remove(entry.entity_id)
 
+        # Wait for entities to finish cleanup
+        with suppress(asyncio.TimeoutError):
+            async with asyncio.timeout(10):
+                await wait_for_entities.wait()
+        remove_track_state_changes()
+
+        @callback
+        def _handle_state_added(
+            entity_id: str, old_state: State | None, new_state: State | None
+        ) -> None:
+            # Wait for entities to be added
+            if old_state is None and entity_id in entities_to_be_added:
+                entities_to_be_added.remove(entity_id)
+            if not entities_to_be_added:
+                wait_for_entities.set()
+
+        # Create a set with entities to be added to the state machine
+        entities_to_be_added = {
+            entry.entity_id
+            for entry in entity_migration_map.values()
+            if self.hass.states.async_available(entry.entity_id)
+        }
+        wait_for_entities = asyncio.Event()
+        remove_track_state_changes = async_track_state_change(
+            self.hass, entities_to_be_added, _handle_state_added
+        )
+
         for entity_id, entry in entity_migration_map.items():
             entity_registry.async_update_entity(
                 entity_id,
@@ -370,26 +406,37 @@ class OptionsFlow(config_entries.OptionsFlow):
                 icon=entry.icon,
             )
 
+        # Wait for entities to finish renaming
+        with suppress(asyncio.TimeoutError):
+            async with asyncio.timeout(10):
+                await wait_for_entities.wait()
+        remove_track_state_changes()
+
         device_registry.async_remove_device(old_device)
 
-    def _can_add_device(self, new_rfx_obj):
+    def _can_add_device(self, new_rfx_obj: rfxtrxmod.RFXtrxEvent) -> bool:
         """Check if device does not already exist."""
         new_device_id = get_device_id(new_rfx_obj.device)
         for packet_id, entity_info in self._config_entry.data[CONF_DEVICES].items():
             rfx_obj = get_rfx_object(packet_id)
+            assert rfx_obj
+
             device_id = get_device_id(rfx_obj.device, entity_info.get(CONF_DATA_BITS))
             if new_device_id == device_id:
                 return False
 
         return True
 
-    def _can_replace_device(self, entry_id):
+    def _can_replace_device(self, entry_id: str) -> bool:
         """Check if device can be replaced with selected device."""
-        device_data = self._get_device_data(entry_id)
-        event_code = device_data[CONF_EVENT_CODE]
+        assert self._selected_device_object
 
-        if event_code is not None:
+        device_data = self._get_device_data(entry_id)
+
+        if (event_code := device_data["event_code"]) is not None:
             rfx_obj = get_rfx_object(event_code)
+            assert rfx_obj
+
             if (
                 rfx_obj.device.packettype
                 == self._selected_device_object.device.packettype
@@ -401,28 +448,30 @@ class OptionsFlow(config_entries.OptionsFlow):
 
         return False
 
-    def _get_device_event_code(self, entry_id):
+    def _get_device_event_code(self, entry_id: str) -> str | None:
         data = self._get_device_data(entry_id)
 
-        return data[CONF_EVENT_CODE]
+        return data["event_code"]
 
-    def _get_device_data(self, entry_id):
+    def _get_device_data(self, entry_id: str) -> DeviceData:
         """Get event code based on device identifier."""
-        event_code = None
-        device_id = None
+        event_code: str | None = None
         entry = self._device_registry.async_get(entry_id)
-        device_id = next(iter(entry.identifiers))[1:]
+        assert entry
+        device_id = get_device_tuple_from_identifiers(entry.identifiers)
+        assert device_id
         for packet_id, entity_info in self._config_entry.data[CONF_DEVICES].items():
             if tuple(entity_info.get(CONF_DEVICE_ID)) == device_id:
-                event_code = packet_id
+                event_code = cast(str, packet_id)
                 break
-
-        data = {CONF_EVENT_CODE: event_code, CONF_DEVICE_ID: device_id}
-
-        return data
+        return DeviceData(event_code=event_code, device_id=device_id)
 
     @callback
-    def update_config_data(self, global_options=None, devices=None):
+    def update_config_data(
+        self,
+        global_options: dict[str, Any] | None = None,
+        devices: dict[str, Any] | None = None,
+    ) -> None:
         """Update data in ConfigEntry."""
         entry_data = self._config_entry.data.copy()
         entry_data[CONF_DEVICES] = copy.deepcopy(self._config_entry.data[CONF_DEVICES])
@@ -444,17 +493,17 @@ class ConfigFlow(config_entries.ConfigFlow, domain=DOMAIN):
     """Handle a config flow for RFXCOM RFXtrx."""
 
     VERSION = 1
-    CONNECTION_CLASS = config_entries.CONN_CLASS_LOCAL_PUSH
 
-    async def async_step_user(self, user_input=None):
+    async def async_step_user(
+        self, user_input: dict[str, Any] | None = None
+    ) -> data_entry_flow.FlowResult:
         """Step when user initializes a integration."""
         await self.async_set_unique_id(DOMAIN)
         self._abort_if_unique_id_configured()
 
-        errors = {}
+        errors: dict[str, str] = {}
         if user_input is not None:
-            user_selection = user_input[CONF_TYPE]
-            if user_selection == "Serial":
+            if user_input[CONF_TYPE] == "Serial":
                 return await self.async_step_setup_serial()
 
             return await self.async_step_setup_network()
@@ -464,9 +513,11 @@ class ConfigFlow(config_entries.ConfigFlow, domain=DOMAIN):
         schema = vol.Schema({vol.Required(CONF_TYPE): vol.In(list_of_types)})
         return self.async_show_form(step_id="user", data_schema=schema, errors=errors)
 
-    async def async_step_setup_network(self, user_input=None):
+    async def async_step_setup_network(
+        self, user_input: dict[str, Any] | None = None
+    ) -> data_entry_flow.FlowResult:
         """Step when setting up network configuration."""
-        errors = {}
+        errors: dict[str, str] = {}
 
         if user_input is not None:
             host = user_input[CONF_HOST]
@@ -489,9 +540,11 @@ class ConfigFlow(config_entries.ConfigFlow, domain=DOMAIN):
             errors=errors,
         )
 
-    async def async_step_setup_serial(self, user_input=None):
+    async def async_step_setup_serial(
+        self, user_input: dict[str, Any] | None = None
+    ) -> data_entry_flow.FlowResult:
         """Step when setting up serial configuration."""
-        errors = {}
+        errors: dict[str, str] = {}
 
         if user_input is not None:
             user_selection = user_input[CONF_DEVICE]
@@ -513,10 +566,9 @@ class ConfigFlow(config_entries.ConfigFlow, domain=DOMAIN):
         ports = await self.hass.async_add_executor_job(serial.tools.list_ports.comports)
         list_of_ports = {}
         for port in ports:
-            list_of_ports[
-                port.device
-            ] = f"{port}, s/n: {port.serial_number or 'n/a'}" + (
-                f" - {port.manufacturer}" if port.manufacturer else ""
+            list_of_ports[port.device] = (
+                f"{port}, s/n: {port.serial_number or 'n/a'}"
+                + (f" - {port.manufacturer}" if port.manufacturer else "")
             )
         list_of_ports[CONF_MANUAL_PATH] = CONF_MANUAL_PATH
 
@@ -527,9 +579,11 @@ class ConfigFlow(config_entries.ConfigFlow, domain=DOMAIN):
             errors=errors,
         )
 
-    async def async_step_setup_serial_manual_path(self, user_input=None):
+    async def async_step_setup_serial_manual_path(
+        self, user_input: dict[str, Any] | None = None
+    ) -> data_entry_flow.FlowResult:
         """Select path manually."""
-        errors = {}
+        errors: dict[str, str] = {}
 
         if user_input is not None:
             device = user_input[CONF_DEVICE]
@@ -548,31 +602,12 @@ class ConfigFlow(config_entries.ConfigFlow, domain=DOMAIN):
             errors=errors,
         )
 
-    async def async_step_import(self, import_config=None):
-        """Handle the initial step."""
-        entry = await self.async_set_unique_id(DOMAIN)
-        if entry:
-            if CONF_DEVICES not in entry.data:
-                # In version 0.113, devices key was not written to config entry. Update the entry with import data
-                self._abort_if_unique_id_configured(import_config)
-            else:
-                self._abort_if_unique_id_configured()
-
-        host = import_config[CONF_HOST]
-        port = import_config[CONF_PORT]
-        device = import_config[CONF_DEVICE]
-
-        try:
-            if host is not None:
-                await self.async_validate_rfx(host=host, port=port)
-            else:
-                await self.async_validate_rfx(device=device)
-        except CannotConnect:
-            return self.async_abort(reason="cannot_connect")
-
-        return self.async_create_entry(title="RFXTRX", data=import_config)
-
-    async def async_validate_rfx(self, host=None, port=None, device=None):
+    async def async_validate_rfx(
+        self,
+        host: str | None = None,
+        port: int | None = None,
+        device: str | None = None,
+    ) -> dict[str, Any]:
         """Create data for rfxtrx entry."""
         success = await self.hass.async_add_executor_job(
             _test_transport, host, port, device
@@ -580,7 +615,7 @@ class ConfigFlow(config_entries.ConfigFlow, domain=DOMAIN):
         if not success:
             raise CannotConnect
 
-        data = {
+        data: dict[str, Any] = {
             CONF_HOST: host,
             CONF_PORT: port,
             CONF_DEVICE: device,
@@ -596,7 +631,7 @@ class ConfigFlow(config_entries.ConfigFlow, domain=DOMAIN):
         return OptionsFlow(config_entry)
 
 
-def _test_transport(host, port, device):
+def _test_transport(host: str | None, port: int | None, device: str | None) -> bool:
     """Construct a rfx object based on config."""
     if port is not None:
         try:
@@ -608,7 +643,7 @@ def _test_transport(host, port, device):
     else:
         try:
             conn = rfxtrxmod.PySerialTransport(device)
-        except serial.serialutil.SerialException:
+        except serial.SerialException:
             return False
 
         if conn.serial is None:
