@@ -1,9 +1,12 @@
 """Sensor for Risco Events."""
 from __future__ import annotations
 
-from collections.abc import Mapping
+from collections.abc import Collection, Mapping
+from datetime import datetime
 from typing import Any
 
+from pyrisco.cloud.event import Event
+
 from homeassistant.components.binary_sensor import DOMAIN as BS_DOMAIN
 from homeassistant.components.sensor import SensorDeviceClass, SensorEntity
 from homeassistant.config_entries import ConfigEntry
@@ -15,7 +18,7 @@ from homeassistant.util import dt as dt_util
 
 from . import RiscoEventsDataUpdateCoordinator, is_local
 from .const import DOMAIN, EVENTS_COORDINATOR
-from .entity import binary_sensor_unique_id
+from .entity import zone_unique_id
 
 CATEGORIES = {
     2: "Alarm",
@@ -63,30 +66,36 @@ async def async_setup_entry(
     async_add_entities(sensors)
 
 
-class RiscoSensor(CoordinatorEntity, SensorEntity):
+class RiscoSensor(CoordinatorEntity[RiscoEventsDataUpdateCoordinator], SensorEntity):
     """Sensor for Risco events."""
 
-    def __init__(self, coordinator, category_id, excludes, name, entry_id) -> None:
+    _entity_registry: er.EntityRegistry
+
+    def __init__(
+        self,
+        coordinator: RiscoEventsDataUpdateCoordinator,
+        category_id: int | None,
+        excludes: Collection[int],
+        name: str,
+        entry_id: str,
+    ) -> None:
         """Initialize sensor."""
         super().__init__(coordinator)
-        self._event = None
+        self._event: Event | None = None
         self._category_id = category_id
         self._excludes = excludes
         self._name = name
         self._entry_id = entry_id
-        self._entity_registry: er.EntityRegistry | None = None
         self._attr_unique_id = f"events_{name}_{self.coordinator.risco.site_uuid}"
         self._attr_name = f"Risco {self.coordinator.risco.site_name} {name} Events"
         self._attr_device_class = SensorDeviceClass.TIMESTAMP
 
     async def async_added_to_hass(self) -> None:
         """When entity is added to hass."""
+        await super().async_added_to_hass()
         self._entity_registry = er.async_get(self.hass)
-        self.async_on_remove(
-            self.coordinator.async_add_listener(self._refresh_from_coordinator)
-        )
 
-    def _refresh_from_coordinator(self):
+    def _handle_coordinator_update(self) -> None:
         events = self.coordinator.data
         for event in reversed(events):
             if event.category_id in self._excludes:
@@ -98,14 +107,14 @@ class RiscoSensor(CoordinatorEntity, SensorEntity):
             self.async_write_ha_state()
 
     @property
-    def native_value(self):
+    def native_value(self) -> datetime | None:
         """Value of sensor."""
         if self._event is None:
             return None
 
-        return dt_util.parse_datetime(self._event.time).replace(
-            tzinfo=dt_util.DEFAULT_TIME_ZONE
-        )
+        if res := dt_util.parse_datetime(self._event.time):
+            return res.replace(tzinfo=dt_util.DEFAULT_TIME_ZONE)
+        return None
 
     @property
     def extra_state_attributes(self) -> Mapping[str, Any] | None:
@@ -115,11 +124,9 @@ class RiscoSensor(CoordinatorEntity, SensorEntity):
 
         attrs = {atr: getattr(self._event, atr, None) for atr in EVENT_ATTRIBUTES}
         if self._event.zone_id is not None:
-            zone_unique_id = binary_sensor_unique_id(
-                self.coordinator.risco, self._event.zone_id
-            )
+            uid = zone_unique_id(self.coordinator.risco, self._event.zone_id)
             zone_entity_id = self._entity_registry.async_get_entity_id(
-                BS_DOMAIN, DOMAIN, zone_unique_id
+                BS_DOMAIN, DOMAIN, uid
             )
             if zone_entity_id is not None:
                 attrs["zone_entity_id"] = zone_entity_id
