@@ -1,23 +1,26 @@
 """Support for Ruckus Unleashed devices."""
 from __future__ import annotations
 
-from homeassistant.components.device_tracker import SourceType
-from homeassistant.components.device_tracker.config_entry import ScannerEntity
+import logging
+
+from homeassistant.components.device_tracker import ScannerEntity, SourceType
 from homeassistant.config_entries import ConfigEntry
 from homeassistant.core import HomeAssistant, callback
-from homeassistant.helpers import entity_registry
+from homeassistant.helpers import entity_registry as er
 from homeassistant.helpers.entity_platform import AddEntitiesCallback
 from homeassistant.helpers.update_coordinator import CoordinatorEntity
 
 from .const import (
-    API_CLIENTS,
-    API_NAME,
+    API_CLIENT_HOSTNAME,
+    API_CLIENT_IP,
     COORDINATOR,
     DOMAIN,
-    MANUFACTURER,
+    KEY_SYS_CLIENTS,
     UNDO_UPDATE_LISTENERS,
 )
 
+_LOGGER = logging.getLogger(__package__)
+
 
 async def async_setup_entry(
     hass: HomeAssistant, entry: ConfigEntry, async_add_entities: AddEntitiesCallback
@@ -38,7 +41,7 @@ async def async_setup_entry(
         coordinator.async_add_listener(router_update)
     )
 
-    registry = entity_registry.async_get(hass)
+    registry = er.async_get(hass)
     restore_entities(registry, coordinator, entry, async_add_entities, tracked)
 
 
@@ -47,12 +50,15 @@ def add_new_entities(coordinator, async_add_entities, tracked):
     """Add new tracker entities from the router."""
     new_tracked = []
 
-    for mac in coordinator.data[API_CLIENTS]:
+    for mac in coordinator.data[KEY_SYS_CLIENTS]:
         if mac in tracked:
             continue
 
-        device = coordinator.data[API_CLIENTS][mac]
-        new_tracked.append(RuckusUnleashedDevice(coordinator, mac, device[API_NAME]))
+        device = coordinator.data[KEY_SYS_CLIENTS][mac]
+        _LOGGER.debug("adding new device: [%s] %s", mac, device[API_CLIENT_HOSTNAME])
+        new_tracked.append(
+            RuckusUnleashedDevice(coordinator, mac, device[API_CLIENT_HOSTNAME])
+        )
         tracked.add(mac)
 
     async_add_entities(new_tracked)
@@ -67,7 +73,7 @@ def restore_entities(registry, coordinator, entry, async_add_entities, tracked):
         if (
             entity.config_entry_id == entry.entry_id
             and entity.platform == DOMAIN
-            and entity.unique_id not in coordinator.data[API_CLIENTS]
+            and entity.unique_id not in coordinator.data[KEY_SYS_CLIENTS]
         ):
             missing.append(
                 RuckusUnleashedDevice(
@@ -76,6 +82,7 @@ def restore_entities(registry, coordinator, entry, async_add_entities, tracked):
             )
             tracked.add(entity.unique_id)
 
+    _LOGGER.debug("added %d missing devices", len(missing))
     async_add_entities(missing)
 
 
@@ -96,17 +103,21 @@ class RuckusUnleashedDevice(CoordinatorEntity, ScannerEntity):
     @property
     def name(self) -> str:
         """Return the name."""
-        if self.is_connected:
-            return (
-                self.coordinator.data[API_CLIENTS][self._mac][API_NAME]
-                or f"{MANUFACTURER} {self._mac}"
-            )
-        return self._name
+        if not self.is_connected:
+            return self._name
+        return self.coordinator.data[KEY_SYS_CLIENTS][self._mac][API_CLIENT_HOSTNAME]
+
+    @property
+    def ip_address(self) -> str | None:
+        """Return the ip address."""
+        if not self.is_connected:
+            return None
+        return self.coordinator.data[KEY_SYS_CLIENTS][self._mac][API_CLIENT_IP]
 
     @property
     def is_connected(self) -> bool:
         """Return true if the device is connected to the network."""
-        return self._mac in self.coordinator.data[API_CLIENTS]
+        return self._mac in self.coordinator.data[KEY_SYS_CLIENTS]
 
     @property
     def source_type(self) -> SourceType:
