@@ -3,7 +3,6 @@ from __future__ import annotations
 
 import asyncio
 from collections.abc import Coroutine, Sequence
-from datetime import datetime, timedelta
 from typing import Any
 
 from async_upnp_client.aiohttp import AiohttpNotifyServer, AiohttpSessionRequester
@@ -30,26 +29,17 @@ from homeassistant.components.media_player import (
     MediaType,
 )
 from homeassistant.config_entries import SOURCE_REAUTH, ConfigEntry
-from homeassistant.const import CONF_HOST, CONF_MAC, CONF_MODEL, CONF_NAME
+from homeassistant.const import CONF_HOST
 from homeassistant.core import HomeAssistant, callback
-from homeassistant.helpers import entity_component
+from homeassistant.helpers import config_validation as cv
 from homeassistant.helpers.aiohttp_client import async_get_clientsession
-import homeassistant.helpers.config_validation as cv
-from homeassistant.helpers.device_registry import CONNECTION_NETWORK_MAC
-from homeassistant.helpers.entity import DeviceInfo
 from homeassistant.helpers.entity_platform import AddEntitiesCallback
-from homeassistant.helpers.script import Script
-from homeassistant.util import dt as dt_util
+from homeassistant.helpers.trigger import PluggableAction
 
 from .bridge import SamsungTVBridge, SamsungTVWSBridge
-from .const import (
-    CONF_MANUFACTURER,
-    CONF_ON_ACTION,
-    CONF_SSDP_RENDERING_CONTROL_LOCATION,
-    DEFAULT_NAME,
-    DOMAIN,
-    LOGGER,
-)
+from .const import CONF_SSDP_RENDERING_CONTROL_LOCATION, DOMAIN, LOGGER
+from .entity import SamsungTVEntity
+from .triggers.turn_on import async_get_turn_on_trigger
 
 SOURCES = {"TV": "KEY_TV", "HDMI": "KEY_HDMI"}
 
@@ -65,12 +55,6 @@ SUPPORT_SAMSUNGTV = (
     | MediaPlayerEntityFeature.PLAY_MEDIA
 )
 
-# Since the TV will take a few seconds to go to sleep
-# and actually be seen as off, we need to wait just a bit
-# more than the next scan interval
-SCAN_INTERVAL_PLUS_OFF_TIME = entity_component.DEFAULT_SCAN_INTERVAL + timedelta(
-    seconds=5
-)
 
 # Max delay waiting for app_list to return, as some TVs simply ignore the request
 APP_LIST_DELAY = 3
@@ -81,71 +65,45 @@ async def async_setup_entry(
 ) -> None:
     """Set up the Samsung TV from a config entry."""
     bridge = hass.data[DOMAIN][entry.entry_id]
-
-    host = entry.data[CONF_HOST]
-    on_script = None
-    data = hass.data[DOMAIN]
-    if turn_on_action := data.get(host, {}).get(CONF_ON_ACTION):
-        on_script = Script(
-            hass, turn_on_action, entry.data.get(CONF_NAME, DEFAULT_NAME), DOMAIN
-        )
-
-    async_add_entities([SamsungTVDevice(bridge, entry, on_script)], True)
+    async_add_entities([SamsungTVDevice(bridge, entry)], True)
 
 
-class SamsungTVDevice(MediaPlayerEntity):
+class SamsungTVDevice(SamsungTVEntity, MediaPlayerEntity):
     """Representation of a Samsung TV."""
 
     _attr_source_list: list[str]
+    _attr_name = None
+    _attr_device_class = MediaPlayerDeviceClass.TV
 
     def __init__(
         self,
         bridge: SamsungTVBridge,
         config_entry: ConfigEntry,
-        on_script: Script | None,
     ) -> None:
         """Initialize the Samsung device."""
+        super().__init__(bridge=bridge, config_entry=config_entry)
         self._config_entry = config_entry
         self._host: str | None = config_entry.data[CONF_HOST]
-        self._mac: str | None = config_entry.data.get(CONF_MAC)
         self._ssdp_rendering_control_location: str | None = config_entry.data.get(
             CONF_SSDP_RENDERING_CONTROL_LOCATION
         )
-        self._on_script = on_script
+        self._turn_on = PluggableAction(self.async_write_ha_state)
         # Assume that the TV is in Play mode
         self._playing: bool = True
 
-        self._attr_name: str | None = config_entry.data.get(CONF_NAME)
-        self._attr_unique_id = config_entry.unique_id
         self._attr_is_volume_muted: bool = False
-        self._attr_device_class = MediaPlayerDeviceClass.TV
         self._attr_source_list = list(SOURCES)
         self._app_list: dict[str, str] | None = None
         self._app_list_event: asyncio.Event = asyncio.Event()
 
         self._attr_supported_features = SUPPORT_SAMSUNGTV
-        if self._on_script or self._mac:
-            # Add turn-on if on_script or mac is available
+        if self._mac:
+            # (deprecated) add turn-on if mac is available
+            # Triggers have not yet been registered so this is adjusted in the property
             self._attr_supported_features |= MediaPlayerEntityFeature.TURN_ON
         if self._ssdp_rendering_control_location:
             self._attr_supported_features |= MediaPlayerEntityFeature.VOLUME_SET
 
-        self._attr_device_info = DeviceInfo(
-            name=self.name,
-            manufacturer=config_entry.data.get(CONF_MANUFACTURER),
-            model=config_entry.data.get(CONF_MODEL),
-        )
-        if self.unique_id:
-            self._attr_device_info["identifiers"] = {(DOMAIN, self.unique_id)}
-        if self._mac:
-            self._attr_device_info["connections"] = {
-                (CONNECTION_NETWORK_MAC, self._mac)
-            }
-
-        # Mark the end of a shutdown command (need to wait 15 seconds before
-        # sending the next command to avoid turning the TV back ON).
-        self._end_of_power_off: datetime | None = None
-        self._bridge = bridge
         self._auth_failed = False
         self._bridge.register_reauth_callback(self.access_denied)
         self._bridge.register_app_list_callback(self._app_list_callback)
@@ -153,6 +111,15 @@ class SamsungTVDevice(MediaPlayerEntity):
         self._dmr_device: DmrDevice | None = None
         self._upnp_server: AiohttpNotifyServer | None = None
 
+    @property
+    def supported_features(self) -> MediaPlayerEntityFeature:
+        """Flag media player features that are supported."""
+        # `turn_on` triggers are not yet registered during initialisation,
+        # so this property needs to be dynamic
+        if self._turn_on:
+            return self._attr_supported_features | MediaPlayerEntityFeature.TURN_ON
+        return self._attr_supported_features
+
     def _update_sources(self) -> None:
         self._attr_source_list = list(SOURCES)
         if app_list := self._app_list:
@@ -188,7 +155,7 @@ class SamsungTVDevice(MediaPlayerEntity):
         if self._auth_failed or self.hass.is_stopping:
             return
         old_state = self._attr_state
-        if self._power_off_in_progress():
+        if self._bridge.power_off_in_progress:
             self._attr_state = MediaPlayerState.OFF
         else:
             self._attr_state = (
@@ -250,7 +217,8 @@ class SamsungTVDevice(MediaPlayerEntity):
             # enter it unless we have to (Python 3.11 will have zero cost try)
             return
         try:
-            await asyncio.wait_for(self._app_list_event.wait(), APP_LIST_DELAY)
+            async with asyncio.timeout(APP_LIST_DELAY):
+                await self._app_list_event.wait()
         except asyncio.TimeoutError as err:
             # No need to try again
             self._app_list_event.set()
@@ -330,7 +298,7 @@ class SamsungTVDevice(MediaPlayerEntity):
 
     async def _async_launch_app(self, app_id: str) -> None:
         """Send launch_app to the tv."""
-        if self._power_off_in_progress():
+        if self._bridge.power_off_in_progress:
             LOGGER.info("TV is powering off, not sending launch_app command")
             return
         assert isinstance(self._bridge, SamsungTVWSBridge)
@@ -339,17 +307,11 @@ class SamsungTVDevice(MediaPlayerEntity):
     async def _async_send_keys(self, keys: list[str]) -> None:
         """Send a key to the tv and handles exceptions."""
         assert keys
-        if self._power_off_in_progress() and keys[0] != "KEY_POWEROFF":
+        if self._bridge.power_off_in_progress and keys[0] != "KEY_POWEROFF":
             LOGGER.info("TV is powering off, not sending keys: %s", keys)
             return
         await self._bridge.async_send_keys(keys)
 
-    def _power_off_in_progress(self) -> bool:
-        return (
-            self._end_of_power_off is not None
-            and self._end_of_power_off > dt_util.utcnow()
-        )
-
     @property
     def available(self) -> bool:
         """Return the availability of the device."""
@@ -357,14 +319,24 @@ class SamsungTVDevice(MediaPlayerEntity):
             return False
         return (
             self.state == MediaPlayerState.ON
-            or self._on_script is not None
+            or bool(self._turn_on)
             or self._mac is not None
-            or self._power_off_in_progress()
+            or self._bridge.power_off_in_progress
         )
 
+    async def async_added_to_hass(self) -> None:
+        """Connect and subscribe to dispatcher signals and state updates."""
+        await super().async_added_to_hass()
+
+        if (entry := self.registry_entry) and entry.device_id:
+            self.async_on_remove(
+                self._turn_on.async_register(
+                    self.hass, async_get_turn_on_trigger(entry.device_id)
+                )
+            )
+
     async def async_turn_off(self) -> None:
         """Turn off media player."""
-        self._end_of_power_off = dt_util.utcnow() + SCAN_INTERVAL_PLUS_OFF_TIME
         await self._bridge.async_power_off()
 
     async def async_set_volume_level(self, volume: float) -> None:
@@ -415,7 +387,7 @@ class SamsungTVDevice(MediaPlayerEntity):
         await self._async_send_keys(["KEY_CHDOWN"])
 
     async def async_play_media(
-        self, media_type: str, media_id: str, **kwargs: Any
+        self, media_type: MediaType | str, media_id: str, **kwargs: Any
     ) -> None:
         """Support changing a channel."""
         if media_type == MediaType.APP:
@@ -446,8 +418,8 @@ class SamsungTVDevice(MediaPlayerEntity):
 
     async def async_turn_on(self) -> None:
         """Turn the media player on."""
-        if self._on_script:
-            await self._on_script.async_run(context=self._context)
+        if self._turn_on:
+            await self._turn_on.async_run(self.hass, self._context)
         elif self._mac:
             await self.hass.async_add_executor_job(self._wake_on_lan)
 
