@@ -19,11 +19,10 @@ from homeassistant.const import (
     ATTR_STATE,
     ATTR_TEMPERATURE,
     PRECISION_TENTHS,
-    TEMP_CELSIUS,
-    TEMP_FAHRENHEIT,
+    UnitOfTemperature,
 )
 from homeassistant.core import HomeAssistant
-from homeassistant.exceptions import HomeAssistantError
+from homeassistant.exceptions import HomeAssistantError, ServiceValidationError
 from homeassistant.helpers import config_validation as cv, entity_platform
 from homeassistant.helpers.entity_platform import AddEntitiesCallback
 from homeassistant.util.unit_conversion import TemperatureConverter
@@ -55,6 +54,31 @@ ATTR_HORIZONTAL_SWING_MODE = "horizontal_swing_mode"
 ATTR_LIGHT = "light"
 BOOST_INCLUSIVE = "boost_inclusive"
 
+AVAILABLE_FAN_MODES = {
+    "quiet",
+    "low",
+    "medium_low",
+    "medium",
+    "medium_high",
+    "high",
+    "strong",
+    "auto",
+}
+AVAILABLE_SWING_MODES = {
+    "stopped",
+    "fixedtop",
+    "fixedmiddletop",
+    "fixedmiddle",
+    "fixedmiddlebottom",
+    "fixedbottom",
+    "rangetop",
+    "rangemiddle",
+    "rangebottom",
+    "rangefull",
+    "horizontal",
+    "both",
+}
+
 PARALLEL_UPDATES = 0
 
 FIELD_TO_FLAG = {
@@ -164,6 +188,11 @@ async def async_setup_entry(
 class SensiboClimate(SensiboDeviceBaseEntity, ClimateEntity):
     """Representation of a Sensibo device."""
 
+    _attr_name = None
+    _attr_precision = PRECISION_TENTHS
+    _attr_translation_key = "climate_device"
+    _enable_turn_on_off_backwards_compatibility = False
+
     def __init__(
         self, coordinator: SensiboDataUpdateCoordinator, device_id: str
     ) -> None:
@@ -171,14 +200,15 @@ class SensiboClimate(SensiboDeviceBaseEntity, ClimateEntity):
         super().__init__(coordinator, device_id)
         self._attr_unique_id = device_id
         self._attr_temperature_unit = (
-            TEMP_CELSIUS if self.device_data.temp_unit == "C" else TEMP_FAHRENHEIT
+            UnitOfTemperature.CELSIUS
+            if self.device_data.temp_unit == "C"
+            else UnitOfTemperature.FAHRENHEIT
         )
         self._attr_supported_features = self.get_features()
-        self._attr_precision = PRECISION_TENTHS
 
-    def get_features(self) -> int:
+    def get_features(self) -> ClimateEntityFeature:
         """Get supported features."""
-        features = 0
+        features = ClimateEntityFeature.TURN_OFF | ClimateEntityFeature.TURN_ON
         for key in self.device_data.full_features:
             if key in FIELD_TO_FLAG:
                 features |= FIELD_TO_FLAG[key]
@@ -212,7 +242,7 @@ class SensiboClimate(SensiboDeviceBaseEntity, ClimateEntity):
         if self.device_data.temp:
             return TemperatureConverter.convert(
                 self.device_data.temp,
-                TEMP_CELSIUS,
+                UnitOfTemperature.CELSIUS,
                 self.temperature_unit,
             )
         return None
@@ -220,7 +250,11 @@ class SensiboClimate(SensiboDeviceBaseEntity, ClimateEntity):
     @property
     def temperature_unit(self) -> str:
         """Return temperature unit."""
-        return TEMP_CELSIUS if self.device_data.temp_unit == "C" else TEMP_FAHRENHEIT
+        return (
+            UnitOfTemperature.CELSIUS
+            if self.device_data.temp_unit == "C"
+            else UnitOfTemperature.FAHRENHEIT
+        )
 
     @property
     def target_temperature(self) -> float | None:
@@ -281,11 +315,17 @@ class SensiboClimate(SensiboDeviceBaseEntity, ClimateEntity):
         """Set new target temperature."""
         if "targetTemperature" not in self.device_data.active_features:
             raise HomeAssistantError(
-                "Current mode doesn't support setting Target Temperature"
+                "Current mode doesn't support setting Target Temperature",
+                translation_domain=DOMAIN,
+                translation_key="no_target_temperature_in_features",
             )
 
         if (temperature := kwargs.get(ATTR_TEMPERATURE)) is None:
-            raise ValueError("No target temperature provided")
+            raise ServiceValidationError(
+                "No target temperature provided",
+                translation_domain=DOMAIN,
+                translation_key="no_target_temperature",
+            )
 
         if temperature == self.target_temperature:
             return
@@ -301,13 +341,26 @@ class SensiboClimate(SensiboDeviceBaseEntity, ClimateEntity):
     async def async_set_fan_mode(self, fan_mode: str) -> None:
         """Set new target fan mode."""
         if "fanLevel" not in self.device_data.active_features:
-            raise HomeAssistantError("Current mode doesn't support setting Fanlevel")
+            raise HomeAssistantError(
+                "Current mode doesn't support setting Fanlevel",
+                translation_domain=DOMAIN,
+                translation_key="no_fan_level_in_features",
+            )
+        if fan_mode not in AVAILABLE_FAN_MODES:
+            raise HomeAssistantError(
+                f"Climate fan mode {fan_mode} is not supported by the integration, please open an issue",
+                translation_domain=DOMAIN,
+                translation_key="fan_mode_not_supported",
+                translation_placeholders={"fan_mode": fan_mode},
+            )
 
+        transformation = self.device_data.fan_modes_translated
         await self.async_send_api_call(
             key=AC_STATE_TO_DATA["fanLevel"],
             value=fan_mode,
             name="fanLevel",
             assumed_state=False,
+            transformation=transformation,
         )
 
     async def async_set_hvac_mode(self, hvac_mode: HVACMode) -> None:
@@ -340,13 +393,26 @@ class SensiboClimate(SensiboDeviceBaseEntity, ClimateEntity):
     async def async_set_swing_mode(self, swing_mode: str) -> None:
         """Set new target swing operation."""
         if "swing" not in self.device_data.active_features:
-            raise HomeAssistantError("Current mode doesn't support setting Swing")
+            raise HomeAssistantError(
+                "Current mode doesn't support setting Swing",
+                translation_domain=DOMAIN,
+                translation_key="no_swing_in_features",
+            )
+        if swing_mode not in AVAILABLE_SWING_MODES:
+            raise HomeAssistantError(
+                f"Climate swing mode {swing_mode} is not supported by the integration, please open an issue",
+                translation_domain=DOMAIN,
+                translation_key="swing_not_supported",
+                translation_placeholders={"swing_mode": swing_mode},
+            )
 
+        transformation = self.device_data.swing_modes_translated
         await self.async_send_api_call(
             key=AC_STATE_TO_DATA["swing"],
             value=swing_mode,
             name="swing",
             assumed_state=False,
+            transformation=transformation,
         )
 
     async def async_turn_on(self) -> None:
@@ -464,10 +530,14 @@ class SensiboClimate(SensiboDeviceBaseEntity, ClimateEntity):
 
         if high_temperature_state.get("temperatureUnit") == "F":
             high_temp = TemperatureConverter.convert(
-                high_temperature_threshold, TEMP_FAHRENHEIT, TEMP_CELSIUS
+                high_temperature_threshold,
+                UnitOfTemperature.FAHRENHEIT,
+                UnitOfTemperature.CELSIUS,
             )
             low_temp = TemperatureConverter.convert(
-                low_temperature_threshold, TEMP_FAHRENHEIT, TEMP_CELSIUS
+                low_temperature_threshold,
+                UnitOfTemperature.FAHRENHEIT,
+                UnitOfTemperature.CELSIUS,
             )
 
         params: dict[str, str | bool | float | dict] = {
@@ -493,8 +563,11 @@ class SensiboClimate(SensiboDeviceBaseEntity, ClimateEntity):
         value: Any,
         name: str,
         assumed_state: bool = False,
+        transformation: dict | None = None,
     ) -> bool:
         """Make service call to api."""
+        if transformation:
+            value = transformation[value]
         result = await self._client.async_set_ac_state_property(
             self._device_id,
             name,
