@@ -24,7 +24,7 @@ from .entity import SensiboDeviceBaseEntity, async_handle_api_call
 PARALLEL_UPDATES = 0
 
 
-@dataclass
+@dataclass(frozen=True)
 class DeviceBaseEntityDescriptionMixin:
     """Mixin for required Sensibo Device description keys."""
 
@@ -35,7 +35,7 @@ class DeviceBaseEntityDescriptionMixin:
     data_key: str
 
 
-@dataclass
+@dataclass(frozen=True)
 class SensiboDeviceSwitchEntityDescription(
     SwitchEntityDescription, DeviceBaseEntityDescriptionMixin
 ):
@@ -45,8 +45,8 @@ class SensiboDeviceSwitchEntityDescription(
 DEVICE_SWITCH_TYPES: tuple[SensiboDeviceSwitchEntityDescription, ...] = (
     SensiboDeviceSwitchEntityDescription(
         key="timer_on_switch",
+        translation_key="timer_on_switch",
         device_class=SwitchDeviceClass.SWITCH,
-        name="Timer",
         icon="mdi:timer",
         value_fn=lambda data: data.timer_on,
         extra_fn=lambda data: {"id": data.timer_id, "turn_on": data.timer_state_on},
@@ -56,8 +56,8 @@ DEVICE_SWITCH_TYPES: tuple[SensiboDeviceSwitchEntityDescription, ...] = (
     ),
     SensiboDeviceSwitchEntityDescription(
         key="climate_react_switch",
+        translation_key="climate_react_switch",
         device_class=SwitchDeviceClass.SWITCH,
-        name="Climate React",
         icon="mdi:wizard-hat",
         value_fn=lambda data: data.smart_on,
         extra_fn=lambda data: {"type": data.smart_type},
@@ -70,8 +70,8 @@ DEVICE_SWITCH_TYPES: tuple[SensiboDeviceSwitchEntityDescription, ...] = (
 PURE_SWITCH_TYPES: tuple[SensiboDeviceSwitchEntityDescription, ...] = (
     SensiboDeviceSwitchEntityDescription(
         key="pure_boost_switch",
+        translation_key="pure_boost_switch",
         device_class=SwitchDeviceClass.SWITCH,
-        name="Pure Boost",
         value_fn=lambda data: data.pure_boost_enabled,
         extra_fn=None,
         command_on="async_turn_on_off_pure_boost",
@@ -138,7 +138,7 @@ class SensiboDeviceSwitch(SensiboDeviceBaseEntity, SwitchEntity):
         await func(
             self,
             key=self.entity_description.data_key,
-            value=True,
+            value=False,
         )
 
     @property
@@ -149,9 +149,9 @@ class SensiboDeviceSwitch(SensiboDeviceBaseEntity, SwitchEntity):
         return None
 
     @async_handle_api_call
-    async def async_turn_on_timer(self, key: str, value: Any) -> bool:
+    async def async_turn_on_timer(self, key: str, value: bool) -> bool:
         """Make service call to api for setting timer."""
-        new_state = bool(self.device_data.ac_states["on"] is False)
+        new_state = not self.device_data.device_on
         data = {
             "minutesFromNow": 60,
             "acState": {**self.device_data.ac_states, "on": new_state},
@@ -160,16 +160,15 @@ class SensiboDeviceSwitch(SensiboDeviceBaseEntity, SwitchEntity):
         return bool(result.get("status") == "success")
 
     @async_handle_api_call
-    async def async_turn_off_timer(self, key: str, value: Any) -> bool:
+    async def async_turn_off_timer(self, key: str, value: bool) -> bool:
         """Make service call to api for deleting timer."""
         result = await self._client.async_del_timer(self._device_id)
         return bool(result.get("status") == "success")
 
     @async_handle_api_call
-    async def async_turn_on_off_pure_boost(self, key: str, value: Any) -> bool:
+    async def async_turn_on_off_pure_boost(self, key: str, value: bool) -> bool:
         """Make service call to api for setting Pure Boost."""
-        new_state = bool(self.device_data.pure_boost_enabled is False)
-        data: dict[str, Any] = {"enabled": new_state}
+        data: dict[str, Any] = {"enabled": value}
         if self.device_data.pure_measure_integration is None:
             data["sensitivity"] = "N"
             data["measurementsIntegration"] = True
@@ -180,13 +179,15 @@ class SensiboDeviceSwitch(SensiboDeviceBaseEntity, SwitchEntity):
         return bool(result.get("status") == "success")
 
     @async_handle_api_call
-    async def async_turn_on_off_smart(self, key: str, value: Any) -> bool:
+    async def async_turn_on_off_smart(self, key: str, value: bool) -> bool:
         """Make service call to api for setting Climate React."""
         if self.device_data.smart_type is None:
             raise HomeAssistantError(
-                "Use Sensibo Enable Climate React Service once to enable switch or the Sensibo app"
+                "Use Sensibo Enable Climate React Service once to enable switch or the"
+                " Sensibo app",
+                translation_domain=DOMAIN,
+                translation_key="climate_react_not_available",
             )
-        new_state = bool(self.device_data.smart_on is False)
-        data: dict[str, Any] = {"enabled": new_state}
+        data: dict[str, Any] = {"enabled": value}
         result = await self._client.async_enable_climate_react(self._device_id, data)
         return bool(result.get("status") == "success")
