@@ -1,43 +1,29 @@
 """Shelly entity helper."""
 from __future__ import annotations
 
-import asyncio
 from collections.abc import Callable, Mapping
 from dataclasses import dataclass
 from typing import Any, cast
 
 from aioshelly.block_device import Block
-import async_timeout
+from aioshelly.exceptions import DeviceConnectionError, InvalidAuthError, RpcCallError
 
 from homeassistant.config_entries import ConfigEntry
-from homeassistant.core import HomeAssistant, callback
-from homeassistant.helpers import (
-    device_registry,
-    entity,
-    entity_registry,
-    update_coordinator,
-)
-from homeassistant.helpers.entity import DeviceInfo, EntityDescription
+from homeassistant.core import HomeAssistant, State, callback
+from homeassistant.exceptions import HomeAssistantError
+from homeassistant.helpers.device_registry import CONNECTION_NETWORK_MAC, DeviceInfo
+from homeassistant.helpers.entity import Entity, EntityDescription
 from homeassistant.helpers.entity_platform import AddEntitiesCallback
-from homeassistant.helpers.restore_state import RestoreEntity
+from homeassistant.helpers.entity_registry import (
+    RegistryEntry,
+    async_entries_for_config_entry,
+    async_get as er_async_get,
+)
 from homeassistant.helpers.typing import StateType
+from homeassistant.helpers.update_coordinator import CoordinatorEntity
 
-from . import (
-    BlockDeviceWrapper,
-    RpcDeviceWrapper,
-    RpcPollingWrapper,
-    ShellyDeviceRestWrapper,
-)
-from .const import (
-    AIOSHELLY_DEVICE_TIMEOUT_SEC,
-    BLOCK,
-    DATA_CONFIG_ENTRY,
-    DOMAIN,
-    LOGGER,
-    REST,
-    RPC,
-    RPC_POLL,
-)
+from .const import CONF_SLEEP_PERIOD, LOGGER
+from .coordinator import ShellyBlockCoordinator, ShellyRpcCoordinator, get_entry_data
 from .utils import (
     async_remove_shelly_entity,
     get_block_entity_name,
@@ -53,28 +39,22 @@ def async_setup_entry_attribute_entities(
     async_add_entities: AddEntitiesCallback,
     sensors: Mapping[tuple[str, str], BlockEntityDescription],
     sensor_class: Callable,
-    description_class: Callable[
-        [entity_registry.RegistryEntry], BlockEntityDescription
-    ],
 ) -> None:
     """Set up entities for attributes."""
-    wrapper: BlockDeviceWrapper = hass.data[DOMAIN][DATA_CONFIG_ENTRY][
-        config_entry.entry_id
-    ][BLOCK]
-
-    if wrapper.device.initialized:
+    coordinator = get_entry_data(hass)[config_entry.entry_id].block
+    assert coordinator
+    if coordinator.device.initialized:
         async_setup_block_attribute_entities(
-            hass, async_add_entities, wrapper, sensors, sensor_class
+            hass, async_add_entities, coordinator, sensors, sensor_class
         )
     else:
         async_restore_block_attribute_entities(
             hass,
             config_entry,
             async_add_entities,
-            wrapper,
+            coordinator,
             sensors,
             sensor_class,
-            description_class,
         )
 
 
@@ -82,16 +62,16 @@ def async_setup_entry_attribute_entities(
 def async_setup_block_attribute_entities(
     hass: HomeAssistant,
     async_add_entities: AddEntitiesCallback,
-    wrapper: BlockDeviceWrapper,
+    coordinator: ShellyBlockCoordinator,
     sensors: Mapping[tuple[str, str], BlockEntityDescription],
     sensor_class: Callable,
 ) -> None:
     """Set up entities for block attributes."""
     blocks = []
 
-    assert wrapper.device.blocks
+    assert coordinator.device.blocks
 
-    for block in wrapper.device.blocks:
+    for block in coordinator.device.blocks:
         for sensor_id in block.sensor_ids:
             description = sensors.get((block.type, sensor_id))
             if description is None:
@@ -101,12 +81,13 @@ def async_setup_block_attribute_entities(
             if getattr(block, sensor_id, None) in (-1, None):
                 continue
 
-            # Filter and remove entities that according to settings should not create an entity
+            # Filter and remove entities that according to settings
+            # should not create an entity
             if description.removal_condition and description.removal_condition(
-                wrapper.device.settings, block
+                coordinator.device.settings, block
             ):
                 domain = sensor_class.__module__.split(".")[-1]
-                unique_id = f"{wrapper.mac}-{block.description}-{sensor_id}"
+                unique_id = f"{coordinator.mac}-{block.description}-{sensor_id}"
                 async_remove_shelly_entity(hass, domain, unique_id)
             else:
                 blocks.append((block, sensor_id, description))
@@ -116,7 +97,7 @@ def async_setup_block_attribute_entities(
 
     async_add_entities(
         [
-            sensor_class(wrapper, block, sensor_id, description)
+            sensor_class(coordinator, block, sensor_id, description)
             for block, sensor_id, description in blocks
         ]
     )
@@ -127,20 +108,15 @@ def async_restore_block_attribute_entities(
     hass: HomeAssistant,
     config_entry: ConfigEntry,
     async_add_entities: AddEntitiesCallback,
-    wrapper: BlockDeviceWrapper,
+    coordinator: ShellyBlockCoordinator,
     sensors: Mapping[tuple[str, str], BlockEntityDescription],
     sensor_class: Callable,
-    description_class: Callable[
-        [entity_registry.RegistryEntry], BlockEntityDescription
-    ],
 ) -> None:
     """Restore block attributes entities."""
     entities = []
 
-    ent_reg = entity_registry.async_get(hass)
-    entries = entity_registry.async_entries_for_config_entry(
-        ent_reg, config_entry.entry_id
-    )
+    ent_reg = er_async_get(hass)
+    entries = async_entries_for_config_entry(ent_reg, config_entry.entry_id)
 
     domain = sensor_class.__module__.split(".")[-1]
 
@@ -149,11 +125,12 @@ def async_restore_block_attribute_entities(
             continue
 
         attribute = entry.unique_id.split("-")[-1]
-        description = description_class(entry)
+        block_type = entry.unique_id.split("-")[-2].split("_")[0]
 
-        entities.append(
-            sensor_class(wrapper, None, attribute, description, entry, sensors)
-        )
+        if description := sensors.get((block_type, attribute)):
+            entities.append(
+                sensor_class(coordinator, None, attribute, description, entry)
+            )
 
     if not entities:
         return
@@ -169,41 +146,100 @@ def async_setup_entry_rpc(
     sensors: Mapping[str, RpcEntityDescription],
     sensor_class: Callable,
 ) -> None:
-    """Set up entities for REST sensors."""
-    wrapper: RpcDeviceWrapper = hass.data[DOMAIN][DATA_CONFIG_ENTRY][
-        config_entry.entry_id
-    ][RPC]
+    """Set up entities for RPC sensors."""
+    coordinator = get_entry_data(hass)[config_entry.entry_id].rpc
+    assert coordinator
 
-    polling_wrapper: RpcPollingWrapper = hass.data[DOMAIN][DATA_CONFIG_ENTRY][
-        config_entry.entry_id
-    ][RPC_POLL]
+    if coordinator.device.initialized:
+        async_setup_rpc_attribute_entities(
+            hass, config_entry, async_add_entities, sensors, sensor_class
+        )
+    else:
+        async_restore_rpc_attribute_entities(
+            hass, config_entry, async_add_entities, coordinator, sensors, sensor_class
+        )
+
+
+@callback
+def async_setup_rpc_attribute_entities(
+    hass: HomeAssistant,
+    config_entry: ConfigEntry,
+    async_add_entities: AddEntitiesCallback,
+    sensors: Mapping[str, RpcEntityDescription],
+    sensor_class: Callable,
+) -> None:
+    """Set up entities for RPC attributes."""
+    coordinator = get_entry_data(hass)[config_entry.entry_id].rpc
+    assert coordinator
+
+    polling_coordinator = None
+    if not (sleep_period := config_entry.data[CONF_SLEEP_PERIOD]):
+        polling_coordinator = get_entry_data(hass)[config_entry.entry_id].rpc_poll
+        assert polling_coordinator
 
     entities = []
     for sensor_id in sensors:
         description = sensors[sensor_id]
-        key_instances = get_rpc_key_instances(wrapper.device.status, description.key)
+        key_instances = get_rpc_key_instances(
+            coordinator.device.status, description.key
+        )
 
         for key in key_instances:
             # Filter non-existing sensors
-            if description.sub_key not in wrapper.device.status[
+            if description.sub_key not in coordinator.device.status[
                 key
-            ] and not description.supported(wrapper.device.status[key]):
+            ] and not description.supported(coordinator.device.status[key]):
                 continue
 
-            # Filter and remove entities that according to settings should not create an entity
+            # Filter and remove entities that according to settings/status
+            # should not create an entity
             if description.removal_condition and description.removal_condition(
-                wrapper.device.config, key
+                coordinator.device.config, coordinator.device.status, key
             ):
                 domain = sensor_class.__module__.split(".")[-1]
-                unique_id = f"{wrapper.mac}-{key}-{sensor_id}"
+                unique_id = f"{coordinator.mac}-{key}-{sensor_id}"
                 async_remove_shelly_entity(hass, domain, unique_id)
-            else:
-                if description.use_polling_wrapper:
+            elif description.use_polling_coordinator:
+                if not sleep_period:
                     entities.append(
-                        sensor_class(polling_wrapper, key, sensor_id, description)
+                        sensor_class(polling_coordinator, key, sensor_id, description)
                     )
-                else:
-                    entities.append(sensor_class(wrapper, key, sensor_id, description))
+            else:
+                entities.append(sensor_class(coordinator, key, sensor_id, description))
+    if not entities:
+        return
+
+    async_add_entities(entities)
+
+
+@callback
+def async_restore_rpc_attribute_entities(
+    hass: HomeAssistant,
+    config_entry: ConfigEntry,
+    async_add_entities: AddEntitiesCallback,
+    coordinator: ShellyRpcCoordinator,
+    sensors: Mapping[str, RpcEntityDescription],
+    sensor_class: Callable,
+) -> None:
+    """Restore block attributes entities."""
+    entities = []
+
+    ent_reg = er_async_get(hass)
+    entries = async_entries_for_config_entry(ent_reg, config_entry.entry_id)
+
+    domain = sensor_class.__module__.split(".")[-1]
+
+    for entry in entries:
+        if entry.domain != domain:
+            continue
+
+        key = entry.unique_id.split("-")[-2]
+        attribute = entry.unique_id.split("-")[-1]
+
+        if description := sensors.get(attribute):
+            entities.append(
+                sensor_class(coordinator, key, attribute, description, entry)
+            )
 
     if not entities:
         return
@@ -220,32 +256,23 @@ def async_setup_entry_rest(
     sensor_class: Callable,
 ) -> None:
     """Set up entities for REST sensors."""
-    wrapper: ShellyDeviceRestWrapper = hass.data[DOMAIN][DATA_CONFIG_ENTRY][
-        config_entry.entry_id
-    ][REST]
-
-    entities = []
-    for sensor_id in sensors:
-        description = sensors.get(sensor_id)
-
-        if not wrapper.device.settings.get("sleep_mode"):
-            entities.append((sensor_id, description))
-
-    if not entities:
-        return
+    coordinator = get_entry_data(hass)[config_entry.entry_id].rest
+    assert coordinator
 
     async_add_entities(
-        [
-            sensor_class(wrapper, sensor_id, description)
-            for sensor_id, description in entities
-        ]
+        sensor_class(coordinator, sensor_id, sensors[sensor_id])
+        for sensor_id in sensors
     )
 
 
-@dataclass
+@dataclass(frozen=True)
 class BlockEntityDescription(EntityDescription):
     """Class to describe a BLOCK entity."""
 
+    # BlockEntity does not support UNDEFINED or None,
+    # restrict the type to str.
+    name: str = ""
+
     icon_fn: Callable[[dict], str] | None = None
     unit_fn: Callable[[dict], str] | None = None
     value: Callable[[Any], Any] = lambda val: val
@@ -255,59 +282,58 @@ class BlockEntityDescription(EntityDescription):
     extra_state_attributes: Callable[[Block], dict | None] | None = None
 
 
-@dataclass
+@dataclass(frozen=True)
 class RpcEntityRequiredKeysMixin:
     """Class for RPC entity required keys."""
 
     sub_key: str
 
 
-@dataclass
+@dataclass(frozen=True)
 class RpcEntityDescription(EntityDescription, RpcEntityRequiredKeysMixin):
     """Class to describe a RPC entity."""
 
+    # BlockEntity does not support UNDEFINED or None,
+    # restrict the type to str.
+    name: str = ""
+
     value: Callable[[Any, Any], Any] | None = None
     available: Callable[[dict], bool] | None = None
-    removal_condition: Callable[[dict, str], bool] | None = None
+    removal_condition: Callable[[dict, dict, str], bool] | None = None
     extra_state_attributes: Callable[[dict, dict], dict | None] | None = None
-    use_polling_wrapper: bool = False
+    use_polling_coordinator: bool = False
     supported: Callable = lambda _: False
 
 
-@dataclass
+@dataclass(frozen=True)
 class RestEntityDescription(EntityDescription):
     """Class to describe a REST entity."""
 
+    # BlockEntity does not support UNDEFINED or None,
+    # restrict the type to str.
+    name: str = ""
+
     value: Callable[[dict, Any], Any] | None = None
     extra_state_attributes: Callable[[dict], dict | None] | None = None
 
 
-class ShellyBlockEntity(entity.Entity):
+class ShellyBlockEntity(CoordinatorEntity[ShellyBlockCoordinator]):
     """Helper class to represent a block entity."""
 
-    def __init__(self, wrapper: BlockDeviceWrapper, block: Block) -> None:
+    def __init__(self, coordinator: ShellyBlockCoordinator, block: Block) -> None:
         """Initialize Shelly entity."""
-        self.wrapper = wrapper
+        super().__init__(coordinator)
         self.block = block
-        self._attr_name = get_block_entity_name(wrapper.device, block)
-        self._attr_should_poll = False
+        self._attr_name = get_block_entity_name(coordinator.device, block)
         self._attr_device_info = DeviceInfo(
-            connections={(device_registry.CONNECTION_NETWORK_MAC, wrapper.mac)}
+            connections={(CONNECTION_NETWORK_MAC, coordinator.mac)}
         )
-        self._attr_unique_id = f"{wrapper.mac}-{block.description}"
-
-    @property
-    def available(self) -> bool:
-        """Available."""
-        return self.wrapper.last_update_success
+        self._attr_unique_id = f"{coordinator.mac}-{block.description}"
 
+    # pylint: disable-next=hass-missing-super-call
     async def async_added_to_hass(self) -> None:
         """When entity is added to HASS."""
-        self.async_on_remove(self.wrapper.async_add_listener(self._update_callback))
-
-    async def async_update(self) -> None:
-        """Update entity with latest info."""
-        await self.wrapper.async_request_refresh()
+        self.async_on_remove(self.coordinator.async_add_listener(self._update_callback))
 
     @callback
     def _update_callback(self) -> None:
@@ -318,54 +344,39 @@ class ShellyBlockEntity(entity.Entity):
         """Set block state (HTTP request)."""
         LOGGER.debug("Setting state for entity %s, state: %s", self.name, kwargs)
         try:
-            async with async_timeout.timeout(AIOSHELLY_DEVICE_TIMEOUT_SEC):
-                return await self.block.set_state(**kwargs)
-        except (asyncio.TimeoutError, OSError) as err:
-            LOGGER.error(
-                "Setting state for entity %s failed, state: %s, error: %s",
-                self.name,
-                kwargs,
-                repr(err),
-            )
-            self.wrapper.last_update_success = False
-            return None
+            return await self.block.set_state(**kwargs)
+        except DeviceConnectionError as err:
+            self.coordinator.last_update_success = False
+            raise HomeAssistantError(
+                f"Setting state for entity {self.name} failed, state: {kwargs}, error:"
+                f" {repr(err)}"
+            ) from err
+        except InvalidAuthError:
+            self.coordinator.entry.async_start_reauth(self.hass)
 
 
-class ShellyRpcEntity(entity.Entity):
+class ShellyRpcEntity(CoordinatorEntity[ShellyRpcCoordinator]):
     """Helper class to represent a rpc entity."""
 
-    def __init__(
-        self,
-        wrapper: RpcDeviceWrapper | RpcPollingWrapper,
-        key: str,
-    ) -> None:
+    def __init__(self, coordinator: ShellyRpcCoordinator, key: str) -> None:
         """Initialize Shelly entity."""
-        self.wrapper = wrapper
+        super().__init__(coordinator)
         self.key = key
-        self._attr_should_poll = False
         self._attr_device_info = {
-            "connections": {(device_registry.CONNECTION_NETWORK_MAC, wrapper.mac)}
+            "connections": {(CONNECTION_NETWORK_MAC, coordinator.mac)}
         }
-        self._attr_unique_id = f"{wrapper.mac}-{key}"
-        self._attr_name = get_rpc_entity_name(wrapper.device, key)
-
-    @property
-    def available(self) -> bool:
-        """Available."""
-        return self.wrapper.device.connected
+        self._attr_unique_id = f"{coordinator.mac}-{key}"
+        self._attr_name = get_rpc_entity_name(coordinator.device, key)
 
     @property
     def status(self) -> dict:
         """Device status by entity key."""
-        return cast(dict, self.wrapper.device.status[self.key])
+        return cast(dict, self.coordinator.device.status[self.key])
 
+    # pylint: disable-next=hass-missing-super-call
     async def async_added_to_hass(self) -> None:
         """When entity is added to HASS."""
-        self.async_on_remove(self.wrapper.async_add_listener(self._update_callback))
-
-    async def async_update(self) -> None:
-        """Update entity with latest info."""
-        await self.wrapper.async_request_refresh()
+        self.async_on_remove(self.coordinator.async_add_listener(self._update_callback))
 
     @callback
     def _update_callback(self) -> None:
@@ -381,39 +392,43 @@ class ShellyRpcEntity(entity.Entity):
             params,
         )
         try:
-            async with async_timeout.timeout(AIOSHELLY_DEVICE_TIMEOUT_SEC):
-                return await self.wrapper.device.call_rpc(method, params)
-        except asyncio.TimeoutError as err:
-            LOGGER.error(
-                "Call RPC for entity %s failed, method: %s, params: %s, error: %s",
-                self.name,
-                method,
-                params,
-                repr(err),
-            )
-            self.wrapper.last_update_success = False
-            return None
+            return await self.coordinator.device.call_rpc(method, params)
+        except DeviceConnectionError as err:
+            self.coordinator.last_update_success = False
+            raise HomeAssistantError(
+                f"Call RPC for {self.name} connection error, method: {method}, params:"
+                f" {params}, error: {repr(err)}"
+            ) from err
+        except RpcCallError as err:
+            raise HomeAssistantError(
+                f"Call RPC for {self.name} request error, method: {method}, params:"
+                f" {params}, error: {repr(err)}"
+            ) from err
+        except InvalidAuthError:
+            self.coordinator.entry.async_start_reauth(self.hass)
 
 
-class ShellyBlockAttributeEntity(ShellyBlockEntity, entity.Entity):
+class ShellyBlockAttributeEntity(ShellyBlockEntity, Entity):
     """Helper class to represent a block attribute."""
 
     entity_description: BlockEntityDescription
 
     def __init__(
         self,
-        wrapper: BlockDeviceWrapper,
+        coordinator: ShellyBlockCoordinator,
         block: Block,
         attribute: str,
         description: BlockEntityDescription,
     ) -> None:
         """Initialize sensor."""
-        super().__init__(wrapper, block)
+        super().__init__(coordinator, block)
         self.attribute = attribute
         self.entity_description = description
 
         self._attr_unique_id: str = f"{super().unique_id}-{self.attribute}"
-        self._attr_name = get_block_entity_name(wrapper.device, block, description.name)
+        self._attr_name = get_block_entity_name(
+            coordinator.device, block, description.name
+        )
 
     @property
     def attribute_value(self) -> StateType:
@@ -428,7 +443,7 @@ class ShellyBlockAttributeEntity(ShellyBlockEntity, entity.Entity):
         """Available."""
         available = super().available
 
-        if not available or not self.entity_description.available:
+        if not available or not self.entity_description.available or self.block is None:
             return available
 
         return self.entity_description.available(self.block)
@@ -442,89 +457,82 @@ class ShellyBlockAttributeEntity(ShellyBlockEntity, entity.Entity):
         return self.entity_description.extra_state_attributes(self.block)
 
 
-class ShellyRestAttributeEntity(update_coordinator.CoordinatorEntity):
+class ShellyRestAttributeEntity(CoordinatorEntity[ShellyBlockCoordinator]):
     """Class to load info from REST."""
 
     entity_description: RestEntityDescription
 
     def __init__(
         self,
-        wrapper: BlockDeviceWrapper,
+        coordinator: ShellyBlockCoordinator,
         attribute: str,
         description: RestEntityDescription,
     ) -> None:
         """Initialize sensor."""
-        super().__init__(wrapper)
-        self.wrapper = wrapper
+        super().__init__(coordinator)
+        self.block_coordinator = coordinator
         self.attribute = attribute
         self.entity_description = description
-        self._attr_name = get_block_entity_name(wrapper.device, None, description.name)
-        self._attr_unique_id = f"{wrapper.mac}-{attribute}"
+        self._attr_name = get_block_entity_name(
+            coordinator.device, None, description.name
+        )
+        self._attr_unique_id = f"{coordinator.mac}-{attribute}"
         self._attr_device_info = DeviceInfo(
-            connections={(device_registry.CONNECTION_NETWORK_MAC, wrapper.mac)}
+            connections={(CONNECTION_NETWORK_MAC, coordinator.mac)}
         )
         self._last_value = None
 
     @property
     def available(self) -> bool:
         """Available."""
-        return self.wrapper.last_update_success
+        return self.block_coordinator.last_update_success
 
     @property
     def attribute_value(self) -> StateType:
         """Value of sensor."""
         if callable(self.entity_description.value):
             self._last_value = self.entity_description.value(
-                self.wrapper.device.status, self._last_value
+                self.block_coordinator.device.status, self._last_value
             )
         return self._last_value
 
-    @property
-    def extra_state_attributes(self) -> dict[str, Any] | None:
-        """Return the state attributes."""
-        if self.entity_description.extra_state_attributes is None:
-            return None
 
-        return self.entity_description.extra_state_attributes(
-            self.wrapper.device.status
-        )
-
-
-class ShellyRpcAttributeEntity(ShellyRpcEntity, entity.Entity):
+class ShellyRpcAttributeEntity(ShellyRpcEntity, Entity):
     """Helper class to represent a rpc attribute."""
 
     entity_description: RpcEntityDescription
 
     def __init__(
         self,
-        wrapper: RpcDeviceWrapper,
+        coordinator: ShellyRpcCoordinator,
         key: str,
         attribute: str,
         description: RpcEntityDescription,
     ) -> None:
         """Initialize sensor."""
-        super().__init__(wrapper, key)
+        super().__init__(coordinator, key)
         self.attribute = attribute
         self.entity_description = description
 
         self._attr_unique_id = f"{super().unique_id}-{attribute}"
-        self._attr_name = get_rpc_entity_name(wrapper.device, key, description.name)
+        self._attr_name = get_rpc_entity_name(coordinator.device, key, description.name)
         self._last_value = None
 
+    @property
+    def sub_status(self) -> Any:
+        """Device status by entity key."""
+        return self.status[self.entity_description.sub_key]
+
     @property
     def attribute_value(self) -> StateType:
         """Value of sensor."""
         if callable(self.entity_description.value):
+            # using "get" here since subkey might not exist (e.g. "errors" sub_key)
             self._last_value = self.entity_description.value(
-                self.wrapper.device.status[self.key].get(
-                    self.entity_description.sub_key
-                ),
-                self._last_value,
+                self.status.get(self.entity_description.sub_key), self._last_value
             )
         else:
-            self._last_value = self.wrapper.device.status[self.key][
-                self.entity_description.sub_key
-            ]
+            self._last_value = self.sub_status
 
         return self._last_value
 
@@ -536,84 +544,55 @@ class ShellyRpcAttributeEntity(ShellyRpcEntity, entity.Entity):
         if not available or not self.entity_description.available:
             return available
 
-        return self.entity_description.available(
-            self.wrapper.device.status[self.key][self.entity_description.sub_key]
-        )
-
-    @property
-    def extra_state_attributes(self) -> dict[str, Any] | None:
-        """Return the state attributes."""
-        if self.entity_description.extra_state_attributes is None:
-            return None
-
-        assert self.wrapper.device.shelly
-
-        return self.entity_description.extra_state_attributes(
-            self.wrapper.device.status[self.key][self.entity_description.sub_key],
-            self.wrapper.device.shelly,
-        )
+        return self.entity_description.available(self.sub_status)
 
 
-class ShellySleepingBlockAttributeEntity(ShellyBlockAttributeEntity, RestoreEntity):
+class ShellySleepingBlockAttributeEntity(ShellyBlockAttributeEntity):
     """Represent a shelly sleeping block attribute entity."""
 
-    # pylint: disable=super-init-not-called
+    # pylint: disable-next=super-init-not-called
     def __init__(
         self,
-        wrapper: BlockDeviceWrapper,
+        coordinator: ShellyBlockCoordinator,
         block: Block | None,
         attribute: str,
         description: BlockEntityDescription,
-        entry: entity_registry.RegistryEntry | None = None,
-        sensors: Mapping[tuple[str, str], BlockEntityDescription] | None = None,
+        entry: RegistryEntry | None = None,
     ) -> None:
         """Initialize the sleeping sensor."""
-        self.sensors = sensors
-        self.last_state: StateType = None
-        self.wrapper = wrapper
+        self.last_state: State | None = None
+        self.coordinator = coordinator
         self.attribute = attribute
         self.block: Block | None = block  # type: ignore[assignment]
         self.entity_description = description
 
-        self._attr_should_poll = False
         self._attr_device_info = DeviceInfo(
-            connections={(device_registry.CONNECTION_NETWORK_MAC, wrapper.mac)}
+            connections={(CONNECTION_NETWORK_MAC, coordinator.mac)}
         )
 
         if block is not None:
-            self._attr_unique_id = f"{self.wrapper.mac}-{block.description}-{attribute}"
+            self._attr_unique_id = (
+                f"{self.coordinator.mac}-{block.description}-{attribute}"
+            )
             self._attr_name = get_block_entity_name(
-                self.wrapper.device, block, self.entity_description.name
+                self.coordinator.device, block, self.entity_description.name
             )
         elif entry is not None:
             self._attr_unique_id = entry.unique_id
             self._attr_name = cast(str, entry.original_name)
 
-    async def async_added_to_hass(self) -> None:
-        """Handle entity which will be added."""
-        await super().async_added_to_hass()
-
-        last_state = await self.async_get_last_state()
-
-        if last_state is not None:
-            self.last_state = last_state.state
-
     @callback
     def _update_callback(self) -> None:
         """Handle device update."""
-        if (
-            self.block is not None
-            or not self.wrapper.device.initialized
-            or self.sensors is None
-        ):
+        if self.block is not None or not self.coordinator.device.initialized:
             super()._update_callback()
             return
 
         _, entity_block, entity_sensor = self._attr_unique_id.split("-")
 
-        assert self.wrapper.device.blocks
+        assert self.coordinator.device.blocks
 
-        for block in self.wrapper.device.blocks:
+        for block in self.coordinator.device.blocks:
             if block.description != entity_block:
                 continue
 
@@ -621,13 +600,58 @@ class ShellySleepingBlockAttributeEntity(ShellyBlockAttributeEntity, RestoreEnti
                 if sensor_id != entity_sensor:
                     continue
 
-                description = self.sensors.get((block.type, sensor_id))
-                if description is None:
-                    continue
-
                 self.block = block
-                self.entity_description = description
-
                 LOGGER.debug("Entity %s attached to block", self.name)
                 super()._update_callback()
                 return
+
+    async def async_update(self) -> None:
+        """Update the entity."""
+        LOGGER.info(
+            "Entity %s comes from a sleeping device, update is not possible",
+            self.entity_id,
+        )
+
+
+class ShellySleepingRpcAttributeEntity(ShellyRpcAttributeEntity):
+    """Helper class to represent a sleeping rpc attribute."""
+
+    entity_description: RpcEntityDescription
+
+    # pylint: disable-next=super-init-not-called
+    def __init__(
+        self,
+        coordinator: ShellyRpcCoordinator,
+        key: str,
+        attribute: str,
+        description: RpcEntityDescription,
+        entry: RegistryEntry | None = None,
+    ) -> None:
+        """Initialize the sleeping sensor."""
+        self.last_state: State | None = None
+        self.coordinator = coordinator
+        self.key = key
+        self.attribute = attribute
+        self.entity_description = description
+
+        self._attr_device_info = DeviceInfo(
+            connections={(CONNECTION_NETWORK_MAC, coordinator.mac)}
+        )
+        self._attr_unique_id = (
+            self._attr_unique_id
+        ) = f"{coordinator.mac}-{key}-{attribute}"
+        self._last_value = None
+
+        if coordinator.device.initialized:
+            self._attr_name = get_rpc_entity_name(
+                coordinator.device, key, description.name
+            )
+        elif entry is not None:
+            self._attr_name = cast(str, entry.original_name)
+
+    async def async_update(self) -> None:
+        """Update the entity."""
+        LOGGER.info(
+            "Entity %s comes from a sleeping device, update is not possible",
+            self.entity_id,
+        )
