@@ -3,7 +3,7 @@ from __future__ import annotations
 
 from collections.abc import Callable
 from dataclasses import dataclass
-from typing import Any
+from datetime import datetime
 
 from aioskybell import SkybellDevice
 from aioskybell.helpers import const as CONST
@@ -14,44 +14,52 @@ from homeassistant.components.sensor import (
     SensorEntityDescription,
 )
 from homeassistant.config_entries import ConfigEntry
+from homeassistant.const import EntityCategory
 from homeassistant.core import HomeAssistant
-from homeassistant.helpers.entity import EntityCategory
 from homeassistant.helpers.entity_platform import AddEntitiesCallback
+from homeassistant.helpers.typing import StateType
 
 from .entity import DOMAIN, SkybellEntity
 
 
-@dataclass
-class SkybellSensorEntityDescription(SensorEntityDescription):
-    """Class to describe a Skybell sensor."""
+@dataclass(frozen=True)
+class SkybellSensorEntityDescriptionMixIn:
+    """Mixin for Skybell sensor."""
 
-    value_fn: Callable[[SkybellDevice], Any] = lambda val: val
+    value_fn: Callable[[SkybellDevice], StateType | datetime]
+
+
+@dataclass(frozen=True)
+class SkybellSensorEntityDescription(
+    SensorEntityDescription, SkybellSensorEntityDescriptionMixIn
+):
+    """Class to describe a Skybell sensor."""
 
 
 SENSOR_TYPES: tuple[SkybellSensorEntityDescription, ...] = (
     SkybellSensorEntityDescription(
         key="chime_level",
-        name="Chime level",
+        translation_key="chime_level",
         icon="mdi:bell-ring",
         value_fn=lambda device: device.outdoor_chime_level,
     ),
     SkybellSensorEntityDescription(
         key="last_button_event",
-        name="Last button event",
+        translation_key="last_button_event",
         icon="mdi:clock",
         device_class=SensorDeviceClass.TIMESTAMP,
         value_fn=lambda device: device.latest("button").get(CONST.CREATED_AT),
     ),
     SkybellSensorEntityDescription(
         key="last_motion_event",
-        name="Last motion event",
+        translation_key="last_motion_event",
         icon="mdi:clock",
         device_class=SensorDeviceClass.TIMESTAMP,
         value_fn=lambda device: device.latest("motion").get(CONST.CREATED_AT),
     ),
     SkybellSensorEntityDescription(
         key=CONST.ATTR_LAST_CHECK_IN,
-        name="Last check in",
+        translation_key="last_check_in",
         icon="mdi:clock",
         entity_registry_enabled_default=False,
         device_class=SensorDeviceClass.TIMESTAMP,
@@ -60,7 +68,7 @@ SENSOR_TYPES: tuple[SkybellSensorEntityDescription, ...] = (
     ),
     SkybellSensorEntityDescription(
         key="motion_threshold",
-        name="Motion threshold",
+        translation_key="motion_threshold",
         icon="mdi:walk",
         entity_registry_enabled_default=False,
         entity_category=EntityCategory.DIAGNOSTIC,
@@ -68,14 +76,14 @@ SENSOR_TYPES: tuple[SkybellSensorEntityDescription, ...] = (
     ),
     SkybellSensorEntityDescription(
         key="video_profile",
-        name="Video profile",
+        translation_key="video_profile",
         entity_registry_enabled_default=False,
         entity_category=EntityCategory.DIAGNOSTIC,
         value_fn=lambda device: device.video_profile,
     ),
     SkybellSensorEntityDescription(
         key=CONST.ATTR_WIFI_SSID,
-        name="Wifi SSID",
+        translation_key="wifi_ssid",
         icon="mdi:wifi-settings",
         entity_registry_enabled_default=False,
         entity_category=EntityCategory.DIAGNOSTIC,
@@ -83,7 +91,7 @@ SENSOR_TYPES: tuple[SkybellSensorEntityDescription, ...] = (
     ),
     SkybellSensorEntityDescription(
         key=CONST.ATTR_WIFI_STATUS,
-        name="Wifi status",
+        translation_key="wifi_status",
         icon="mdi:wifi-strength-3",
         entity_registry_enabled_default=False,
         entity_category=EntityCategory.DIAGNOSTIC,
@@ -110,6 +118,6 @@ class SkybellSensor(SkybellEntity, SensorEntity):
     entity_description: SkybellSensorEntityDescription
 
     @property
-    def native_value(self) -> int:
+    def native_value(self) -> StateType | datetime:
         """Return the state of the sensor."""
         return self.entity_description.value_fn(self._device)
