@@ -12,32 +12,32 @@ from homeassistant.components.sensor import (
 from homeassistant.config_entries import ConfigEntry
 from homeassistant.const import UnitOfElectricPotential, UnitOfEnergy, UnitOfPower
 from homeassistant.core import HomeAssistant
-from homeassistant.helpers.entity import DeviceInfo
+from homeassistant.helpers.device_registry import DeviceInfo
 from homeassistant.helpers.entity_platform import AddEntitiesCallback
 
 from .const import DOMAIN
 
 
-@dataclass
+@dataclass(frozen=True)
 class SmappeeRequiredKeysMixin:
     """Mixin for required keys."""
 
     sensor_id: str
 
 
-@dataclass
+@dataclass(frozen=True)
 class SmappeeSensorEntityDescription(SensorEntityDescription, SmappeeRequiredKeysMixin):
     """Describes Smappee sensor entity."""
 
 
-@dataclass
+@dataclass(frozen=True)
 class SmappeePollingSensorEntityDescription(SmappeeSensorEntityDescription):
     """Describes Smappee sensor entity."""
 
     local_polling: bool = False
 
 
-@dataclass
+@dataclass(frozen=True)
 class SmappeeVoltageSensorEntityDescription(SmappeeSensorEntityDescription):
     """Describes Smappee sensor entity."""
 
@@ -336,11 +336,18 @@ class SmappeeSensor(SensorEntity):
         smappee_base,
         service_location,
         description: SmappeeSensorEntityDescription,
-    ):
+    ) -> None:
         """Initialize the Smappee sensor."""
         self.entity_description = description
         self._smappee_base = smappee_base
         self._service_location = service_location
+        self._attr_device_info = DeviceInfo(
+            identifiers={(DOMAIN, service_location.device_serial_number)},
+            manufacturer="Smappee",
+            model=service_location.device_model,
+            name=service_location.service_location_name,
+            sw_version=service_location.firmware_version,
+        )
 
     @property
     def name(self):
@@ -372,17 +379,6 @@ class SmappeeSensor(SensorEntity):
             f"{sensor_key}"
         )
 
-    @property
-    def device_info(self) -> DeviceInfo:
-        """Return the device info for this sensor."""
-        return DeviceInfo(
-            identifiers={(DOMAIN, self._service_location.device_serial_number)},
-            manufacturer="Smappee",
-            model=self._service_location.device_model,
-            name=self._service_location.service_location_name,
-            sw_version=self._service_location.firmware_version,
-        )
-
     async def async_update(self) -> None:
         """Get the latest data from Smappee and update the state."""
         await self._smappee_base.async_update()
