@@ -1,14 +1,22 @@
 """Support for interacting with Smappee Comport Plugs, Switches and Output Modules."""
+from typing import Any
+
 from homeassistant.components.switch import SwitchEntity
-from homeassistant.helpers.entity import DeviceInfo
+from homeassistant.config_entries import ConfigEntry
+from homeassistant.core import HomeAssistant
+from homeassistant.helpers.device_registry import DeviceInfo
+from homeassistant.helpers.entity_platform import AddEntitiesCallback
 
 from .const import DOMAIN
 
 SWITCH_PREFIX = "Switch"
-ICON = "mdi:toggle-switch"
 
 
-async def async_setup_entry(hass, config_entry, async_add_entities):
+async def async_setup_entry(
+    hass: HomeAssistant,
+    config_entry: ConfigEntry,
+    async_add_entities: AddEntitiesCallback,
+) -> None:
     """Set up the Smappee Comfort Plugs."""
     smappee_base = hass.data[DOMAIN][config_entry.entry_id]
 
@@ -46,6 +54,8 @@ async def async_setup_entry(hass, config_entry, async_add_entities):
 class SmappeeActuator(SwitchEntity):
     """Representation of a Smappee Comport Plug."""
 
+    _attr_icon = "mdi:toggle-switch"
+
     def __init__(
         self,
         smappee_base,
@@ -64,10 +74,17 @@ class SmappeeActuator(SwitchEntity):
         self._actuator_type = actuator_type
         self._actuator_serialnumber = actuator_serialnumber
         self._actuator_state_option = actuator_state_option
-        self._state = self._service_location.actuators.get(actuator_id).state
-        self._connection_state = self._service_location.actuators.get(
+        self._state = service_location.actuators.get(actuator_id).state
+        self._connection_state = service_location.actuators.get(
             actuator_id
         ).connection_state
+        self._attr_device_info = DeviceInfo(
+            identifiers={(DOMAIN, service_location.device_serial_number)},
+            manufacturer="Smappee",
+            model=service_location.device_model,
+            name=service_location.service_location_name,
+            sw_version=service_location.firmware_version,
+        )
 
     @property
     def name(self):
@@ -96,12 +113,7 @@ class SmappeeActuator(SwitchEntity):
         # Switch or comfort plug
         return self._state == "ON_ON"
 
-    @property
-    def icon(self):
-        """Icon to use in the frontend."""
-        return ICON
-
-    def turn_on(self, **kwargs):
+    def turn_on(self, **kwargs: Any) -> None:
         """Turn on Comport Plug."""
         if self._actuator_type in ("SWITCH", "COMFORT_PLUG"):
             self._service_location.set_actuator_state(self._actuator_id, state="ON_ON")
@@ -110,7 +122,7 @@ class SmappeeActuator(SwitchEntity):
                 self._actuator_id, state=self._actuator_state_option
             )
 
-    def turn_off(self, **kwargs):
+    def turn_off(self, **kwargs: Any) -> None:
         """Turn off Comport Plug."""
         if self._actuator_type in ("SWITCH", "COMFORT_PLUG"):
             self._service_location.set_actuator_state(
@@ -122,7 +134,7 @@ class SmappeeActuator(SwitchEntity):
             )
 
     @property
-    def available(self):
+    def available(self) -> bool:
         """Return True if entity is available. Unavailable for COMFORT_PLUGS."""
         return (
             self._connection_state == "CONNECTED"
@@ -148,18 +160,7 @@ class SmappeeActuator(SwitchEntity):
             f"{self._actuator_id}"
         )
 
-    @property
-    def device_info(self) -> DeviceInfo:
-        """Return the device info for this switch."""
-        return DeviceInfo(
-            identifiers={(DOMAIN, self._service_location.device_serial_number)},
-            manufacturer="Smappee",
-            model=self._service_location.device_model,
-            name=self._service_location.service_location_name,
-            sw_version=self._service_location.firmware_version,
-        )
-
-    async def async_update(self):
+    async def async_update(self) -> None:
         """Get the latest data from Smappee and update the state."""
         await self._smappee_base.async_update()
 
