@@ -16,7 +16,9 @@ from homeassistant.config_entries import SOURCE_IMPORT, ConfigEntry
 from homeassistant.const import CONF_ACCESS_TOKEN, CONF_CLIENT_ID, CONF_CLIENT_SECRET
 from homeassistant.core import HomeAssistant
 from homeassistant.exceptions import ConfigEntryNotReady
+from homeassistant.helpers import config_validation as cv
 from homeassistant.helpers.aiohttp_client import async_get_clientsession
+from homeassistant.helpers.device_registry import DeviceInfo
 from homeassistant.helpers.dispatcher import (
     async_dispatcher_connect,
     async_dispatcher_send,
@@ -51,10 +53,12 @@ from .smartapp import (
 
 _LOGGER = logging.getLogger(__name__)
 
+CONFIG_SCHEMA = cv.config_entry_only_config_schema(DOMAIN)
+
 
 async def async_setup(hass: HomeAssistant, config: ConfigType) -> bool:
     """Initialize the SmartThings platform."""
-    await setup_smartapp_endpoint(hass)
+    await setup_smartapp_endpoint(hass, False)
     return True
 
 
@@ -92,7 +96,8 @@ async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
 
     if not validate_webhook_requirements(hass):
         _LOGGER.warning(
-            "The 'base_url' of the 'http' integration must be configured and start with 'https://'"
+            "The 'base_url' of the 'http' integration must be configured and start with"
+            " 'https://'"
         )
         return False
 
@@ -135,7 +140,10 @@ async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
                 await device.status.refresh()
             except ClientResponseError:
                 _LOGGER.debug(
-                    "Unable to update status for device: %s (%s), the device will be excluded",
+                    (
+                        "Unable to update status for device: %s (%s), the device will"
+                        " be excluded"
+                    ),
                     device.label,
                     device.device_id,
                     exc_info=True,
@@ -161,7 +169,10 @@ async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
     except ClientResponseError as ex:
         if ex.status in (HTTPStatus.UNAUTHORIZED, HTTPStatus.FORBIDDEN):
             _LOGGER.exception(
-                "Unable to setup configuration entry '%s' - please reconfigure the integration",
+                (
+                    "Unable to setup configuration entry '%s' - please reconfigure the"
+                    " integration"
+                ),
                 entry.title,
             )
             remove_entry = True
@@ -183,7 +194,7 @@ async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
             )
         return False
 
-    hass.config_entries.async_setup_platforms(entry, PLATFORMS)
+    await hass.config_entries.async_forward_entry_setups(entry, PLATFORMS)
     return True
 
 
@@ -194,7 +205,10 @@ async def async_get_entry_scenes(entry: ConfigEntry, api):
     except ClientResponseError as ex:
         if ex.status == HTTPStatus.FORBIDDEN:
             _LOGGER.exception(
-                "Unable to load scenes for configuration entry '%s' because the access token does not have the required access",
+                (
+                    "Unable to load scenes for configuration entry '%s' because the"
+                    " access token does not have the required access"
+                ),
                 entry.title,
             )
         else:
@@ -237,7 +251,10 @@ async def async_remove_entry(hass: HomeAssistant, entry: ConfigEntry) -> None:
     app_count = sum(1 for entry in all_entries if entry.data[CONF_APP_ID] == app_id)
     if app_count > 1:
         _LOGGER.debug(
-            "App %s was not removed because it is in use by other configuration entries",
+            (
+                "App %s was not removed because it is in use by other configuration"
+                " entries"
+            ),
             app_id,
         )
         return
@@ -266,7 +283,7 @@ class DeviceBroker:
         smart_app,
         devices: Iterable,
         scenes: Iterable,
-    ):
+    ) -> None:
         """Create a new instance of the DeviceBroker."""
         self._hass = hass
         self._entry = entry
@@ -305,6 +322,7 @@ class DeviceBroker:
 
     def connect(self):
         """Connect handlers/listeners for device/lifecycle events."""
+
         # Setup interval to regenerate the refresh token on a periodic basis.
         # Tokens expire in 30 days and once expired, cannot be recovered.
         async def regenerate_refresh_token(now):
@@ -405,10 +423,23 @@ class DeviceBroker:
 class SmartThingsEntity(Entity):
     """Defines a SmartThings entity."""
 
+    _attr_should_poll = False
+
     def __init__(self, device: DeviceEntity) -> None:
         """Initialize the instance."""
         self._device = device
         self._dispatcher_remove = None
+        self._attr_name = device.label
+        self._attr_unique_id = device.device_id
+        self._attr_device_info = DeviceInfo(
+            configuration_url="https://account.smartthings.com",
+            identifiers={(DOMAIN, device.device_id)},
+            manufacturer=device.status.ocf_manufacturer_name,
+            model=device.status.ocf_model_number,
+            name=device.label,
+            hw_version=device.status.ocf_hardware_version,
+            sw_version=device.status.ocf_firmware_version,
+        )
 
     async def async_added_to_hass(self):
         """Device added to hass."""
@@ -426,28 +457,3 @@ class SmartThingsEntity(Entity):
         """Disconnect the device when removed."""
         if self._dispatcher_remove:
             self._dispatcher_remove()
-
-    @property
-    def device_info(self):
-        """Get attributes about the device."""
-        return {
-            "identifiers": {(DOMAIN, self._device.device_id)},
-            "name": self._device.label,
-            "model": self._device.device_type_name,
-            "manufacturer": "Unavailable",
-        }
-
-    @property
-    def name(self) -> str:
-        """Return the name of the device."""
-        return self._device.label
-
-    @property
-    def should_poll(self) -> bool:
-        """No polling needed for this device."""
-        return False
-
-    @property
-    def unique_id(self) -> str:
-        """Return a unique ID."""
-        return self._device.device_id
