@@ -4,85 +4,101 @@ from __future__ import annotations
 import asyncio
 from collections.abc import Iterable, Sequence
 import logging
+from typing import Any
 
 from pysmartthings import Attribute, Capability
 
-from homeassistant.components.climate import DOMAIN as CLIMATE_DOMAIN, ClimateEntity
-from homeassistant.components.climate.const import (
+from homeassistant.components.climate import (
     ATTR_HVAC_MODE,
     ATTR_TARGET_TEMP_HIGH,
     ATTR_TARGET_TEMP_LOW,
-    CURRENT_HVAC_COOL,
-    CURRENT_HVAC_FAN,
-    CURRENT_HVAC_HEAT,
-    CURRENT_HVAC_IDLE,
-    HVAC_MODE_AUTO,
-    HVAC_MODE_COOL,
-    HVAC_MODE_DRY,
-    HVAC_MODE_FAN_ONLY,
-    HVAC_MODE_HEAT,
-    HVAC_MODE_HEAT_COOL,
-    HVAC_MODE_OFF,
-    SUPPORT_FAN_MODE,
-    SUPPORT_TARGET_TEMPERATURE,
-    SUPPORT_TARGET_TEMPERATURE_RANGE,
+    DOMAIN as CLIMATE_DOMAIN,
+    SWING_BOTH,
+    SWING_HORIZONTAL,
+    SWING_OFF,
+    SWING_VERTICAL,
+    ClimateEntity,
+    ClimateEntityFeature,
+    HVACAction,
+    HVACMode,
 )
-from homeassistant.const import ATTR_TEMPERATURE, TEMP_CELSIUS, TEMP_FAHRENHEIT
+from homeassistant.config_entries import ConfigEntry
+from homeassistant.const import ATTR_TEMPERATURE, UnitOfTemperature
+from homeassistant.core import HomeAssistant
+from homeassistant.helpers.entity_platform import AddEntitiesCallback
 
 from . import SmartThingsEntity
 from .const import DATA_BROKERS, DOMAIN
 
 ATTR_OPERATION_STATE = "operation_state"
 MODE_TO_STATE = {
-    "auto": HVAC_MODE_HEAT_COOL,
-    "cool": HVAC_MODE_COOL,
-    "eco": HVAC_MODE_AUTO,
-    "rush hour": HVAC_MODE_AUTO,
-    "emergency heat": HVAC_MODE_HEAT,
-    "heat": HVAC_MODE_HEAT,
-    "off": HVAC_MODE_OFF,
+    "auto": HVACMode.HEAT_COOL,
+    "cool": HVACMode.COOL,
+    "eco": HVACMode.AUTO,
+    "rush hour": HVACMode.AUTO,
+    "emergency heat": HVACMode.HEAT,
+    "heat": HVACMode.HEAT,
+    "off": HVACMode.OFF,
 }
 STATE_TO_MODE = {
-    HVAC_MODE_HEAT_COOL: "auto",
-    HVAC_MODE_COOL: "cool",
-    HVAC_MODE_HEAT: "heat",
-    HVAC_MODE_OFF: "off",
+    HVACMode.HEAT_COOL: "auto",
+    HVACMode.COOL: "cool",
+    HVACMode.HEAT: "heat",
+    HVACMode.OFF: "off",
 }
 
 OPERATING_STATE_TO_ACTION = {
-    "cooling": CURRENT_HVAC_COOL,
-    "fan only": CURRENT_HVAC_FAN,
-    "heating": CURRENT_HVAC_HEAT,
-    "idle": CURRENT_HVAC_IDLE,
-    "pending cool": CURRENT_HVAC_COOL,
-    "pending heat": CURRENT_HVAC_HEAT,
-    "vent economizer": CURRENT_HVAC_FAN,
+    "cooling": HVACAction.COOLING,
+    "fan only": HVACAction.FAN,
+    "heating": HVACAction.HEATING,
+    "idle": HVACAction.IDLE,
+    "pending cool": HVACAction.COOLING,
+    "pending heat": HVACAction.HEATING,
+    "vent economizer": HVACAction.FAN,
 }
 
 AC_MODE_TO_STATE = {
-    "auto": HVAC_MODE_HEAT_COOL,
-    "cool": HVAC_MODE_COOL,
-    "dry": HVAC_MODE_DRY,
-    "coolClean": HVAC_MODE_COOL,
-    "dryClean": HVAC_MODE_DRY,
-    "heat": HVAC_MODE_HEAT,
-    "heatClean": HVAC_MODE_HEAT,
-    "fanOnly": HVAC_MODE_FAN_ONLY,
+    "auto": HVACMode.HEAT_COOL,
+    "cool": HVACMode.COOL,
+    "dry": HVACMode.DRY,
+    "coolClean": HVACMode.COOL,
+    "dryClean": HVACMode.DRY,
+    "heat": HVACMode.HEAT,
+    "heatClean": HVACMode.HEAT,
+    "fanOnly": HVACMode.FAN_ONLY,
 }
 STATE_TO_AC_MODE = {
-    HVAC_MODE_HEAT_COOL: "auto",
-    HVAC_MODE_COOL: "cool",
-    HVAC_MODE_DRY: "dry",
-    HVAC_MODE_HEAT: "heat",
-    HVAC_MODE_FAN_ONLY: "fanOnly",
+    HVACMode.HEAT_COOL: "auto",
+    HVACMode.COOL: "cool",
+    HVACMode.DRY: "dry",
+    HVACMode.HEAT: "heat",
+    HVACMode.FAN_ONLY: "fanOnly",
 }
 
-UNIT_MAP = {"C": TEMP_CELSIUS, "F": TEMP_FAHRENHEIT}
+SWING_TO_FAN_OSCILLATION = {
+    SWING_BOTH: "all",
+    SWING_HORIZONTAL: "horizontal",
+    SWING_VERTICAL: "vertical",
+    SWING_OFF: "fixed",
+}
+
+FAN_OSCILLATION_TO_SWING = {
+    value: key for key, value in SWING_TO_FAN_OSCILLATION.items()
+}
+
+
+WINDFREE = "windFree"
+
+UNIT_MAP = {"C": UnitOfTemperature.CELSIUS, "F": UnitOfTemperature.FAHRENHEIT}
 
 _LOGGER = logging.getLogger(__name__)
 
 
-async def async_setup_entry(hass, config_entry, async_add_entities):
+async def async_setup_entry(
+    hass: HomeAssistant,
+    config_entry: ConfigEntry,
+    async_add_entities: AddEntitiesCallback,
+) -> None:
     """Add climate entities for a config entry."""
     ac_capabilities = [
         Capability.air_conditioner_mode,
@@ -93,7 +109,7 @@ async def async_setup_entry(hass, config_entry, async_add_entities):
     ]
 
     broker = hass.data[DOMAIN][DATA_BROKERS][config_entry.entry_id]
-    entities = []
+    entities: list[ClimateEntity] = []
     for device in broker.devices.values():
         if not broker.any_assigned(device.device_id, CLIMATE_DOMAIN):
             continue
@@ -110,10 +126,7 @@ def get_capabilities(capabilities: Sequence[str]) -> Sequence[str] | None:
         Capability.air_conditioner_mode,
         Capability.demand_response_load_control,
         Capability.air_conditioner_fan_mode,
-        Capability.power_consumption_report,
-        Capability.relative_humidity_measurement,
         Capability.switch,
-        Capability.temperature_measurement,
         Capability.thermostat,
         Capability.thermostat_cooling_setpoint,
         Capability.thermostat_fan_mode,
@@ -149,22 +162,29 @@ def get_capabilities(capabilities: Sequence[str]) -> Sequence[str] | None:
 class SmartThingsThermostat(SmartThingsEntity, ClimateEntity):
     """Define a SmartThings climate entities."""
 
+    _enable_turn_on_off_backwards_compatibility = False
+
     def __init__(self, device):
         """Init the class."""
         super().__init__(device)
-        self._supported_features = self._determine_features()
+        self._attr_supported_features = self._determine_features()
         self._hvac_mode = None
         self._hvac_modes = None
 
     def _determine_features(self):
-        flags = SUPPORT_TARGET_TEMPERATURE | SUPPORT_TARGET_TEMPERATURE_RANGE
+        flags = (
+            ClimateEntityFeature.TARGET_TEMPERATURE
+            | ClimateEntityFeature.TARGET_TEMPERATURE_RANGE
+            | ClimateEntityFeature.TURN_OFF
+            | ClimateEntityFeature.TURN_ON
+        )
         if self._device.get_capability(
             Capability.thermostat_fan_mode, Capability.thermostat
         ):
-            flags |= SUPPORT_FAN_MODE
+            flags |= ClimateEntityFeature.FAN_MODE
         return flags
 
-    async def async_set_fan_mode(self, fan_mode):
+    async def async_set_fan_mode(self, fan_mode: str) -> None:
         """Set new target fan mode."""
         await self._device.set_thermostat_fan_mode(fan_mode, set_status=True)
 
@@ -172,7 +192,7 @@ class SmartThingsThermostat(SmartThingsEntity, ClimateEntity):
         # the entity state ahead of receiving the confirming push updates
         self.async_schedule_update_ha_state(True)
 
-    async def async_set_hvac_mode(self, hvac_mode):
+    async def async_set_hvac_mode(self, hvac_mode: HVACMode) -> None:
         """Set new target operation mode."""
         mode = STATE_TO_MODE[hvac_mode]
         await self._device.set_thermostat_mode(mode, set_status=True)
@@ -181,11 +201,10 @@ class SmartThingsThermostat(SmartThingsEntity, ClimateEntity):
         # the entity state ahead of receiving the confirming push updates
         self.async_schedule_update_ha_state(True)
 
-    async def async_set_temperature(self, **kwargs):
+    async def async_set_temperature(self, **kwargs: Any) -> None:
         """Set new operation mode and target temperatures."""
         # Operation state
-        operation_state = kwargs.get(ATTR_HVAC_MODE)
-        if operation_state:
+        if operation_state := kwargs.get(ATTR_HVAC_MODE):
             mode = STATE_TO_MODE[operation_state]
             await self._device.set_thermostat_mode(mode, set_status=True)
             await self.async_update()
@@ -193,9 +212,9 @@ class SmartThingsThermostat(SmartThingsEntity, ClimateEntity):
         # Heat/cool setpoint
         heating_setpoint = None
         cooling_setpoint = None
-        if self.hvac_mode == HVAC_MODE_HEAT:
+        if self.hvac_mode == HVACMode.HEAT:
             heating_setpoint = kwargs.get(ATTR_TEMPERATURE)
-        elif self.hvac_mode == HVAC_MODE_COOL:
+        elif self.hvac_mode == HVACMode.COOL:
             cooling_setpoint = kwargs.get(ATTR_TEMPERATURE)
         else:
             heating_setpoint = kwargs.get(ATTR_TARGET_TEMP_LOW)
@@ -219,7 +238,7 @@ class SmartThingsThermostat(SmartThingsEntity, ClimateEntity):
         # the entity state ahead of receiving the confirming push updates
         self.async_schedule_update_ha_state(True)
 
-    async def async_update(self):
+    async def async_update(self) -> None:
         """Update the attributes of the climate device."""
         thermostat_mode = self._device.status.thermostat_mode
         self._hvac_mode = MODE_TO_STATE.get(thermostat_mode)
@@ -235,12 +254,14 @@ class SmartThingsThermostat(SmartThingsEntity, ClimateEntity):
         supported_modes = self._device.status.supported_thermostat_modes
         if isinstance(supported_modes, Iterable):
             for mode in supported_modes:
-                state = MODE_TO_STATE.get(mode)
-                if state is not None:
+                if (state := MODE_TO_STATE.get(mode)) is not None:
                     modes.add(state)
                 else:
                     _LOGGER.debug(
-                        "Device %s (%s) returned an invalid supported thermostat mode: %s",
+                        (
+                            "Device %s (%s) returned an invalid supported thermostat"
+                            " mode: %s"
+                        ),
                         self._device.label,
                         self._device.device_id,
                         mode,
@@ -275,47 +296,42 @@ class SmartThingsThermostat(SmartThingsEntity, ClimateEntity):
         return self._device.status.supported_thermostat_fan_modes
 
     @property
-    def hvac_action(self) -> str | None:
+    def hvac_action(self) -> HVACAction | None:
         """Return the current running hvac operation if supported."""
         return OPERATING_STATE_TO_ACTION.get(
             self._device.status.thermostat_operating_state
         )
 
     @property
-    def hvac_mode(self):
+    def hvac_mode(self) -> HVACMode:
         """Return current operation ie. heat, cool, idle."""
         return self._hvac_mode
 
     @property
-    def hvac_modes(self):
+    def hvac_modes(self) -> list[HVACMode]:
         """Return the list of available operation modes."""
         return self._hvac_modes
 
-    @property
-    def supported_features(self):
-        """Return the supported features."""
-        return self._supported_features
-
     @property
     def target_temperature(self):
         """Return the temperature we try to reach."""
-        if self.hvac_mode == HVAC_MODE_COOL:
+        if self.hvac_mode == HVACMode.COOL:
             return self._device.status.cooling_setpoint
-        if self.hvac_mode == HVAC_MODE_HEAT:
+        if self.hvac_mode == HVACMode.HEAT:
             return self._device.status.heating_setpoint
         return None
 
     @property
     def target_temperature_high(self):
         """Return the highbound target temperature we try to reach."""
-        if self.hvac_mode == HVAC_MODE_HEAT_COOL:
+        if self.hvac_mode == HVACMode.HEAT_COOL:
             return self._device.status.cooling_setpoint
         return None
 
     @property
     def target_temperature_low(self):
         """Return the lowbound target temperature we try to reach."""
-        if self.hvac_mode == HVAC_MODE_HEAT_COOL:
+        if self.hvac_mode == HVACMode.HEAT_COOL:
             return self._device.status.heating_setpoint
         return None
 
@@ -328,21 +344,45 @@ class SmartThingsThermostat(SmartThingsEntity, ClimateEntity):
 class SmartThingsAirConditioner(SmartThingsEntity, ClimateEntity):
     """Define a SmartThings Air Conditioner."""
 
-    def __init__(self, device):
+    _hvac_modes: list[HVACMode]
+    _enable_turn_on_off_backwards_compatibility = False
+
+    def __init__(self, device) -> None:
         """Init the class."""
         super().__init__(device)
-        self._hvac_modes = None
+        self._hvac_modes = []
+        self._attr_preset_mode = None
+        self._attr_preset_modes = self._determine_preset_modes()
+        self._attr_swing_modes = self._determine_swing_modes()
+        self._attr_supported_features = self._determine_supported_features()
 
-    async def async_set_fan_mode(self, fan_mode):
+    def _determine_supported_features(self) -> ClimateEntityFeature:
+        features = (
+            ClimateEntityFeature.TARGET_TEMPERATURE
+            | ClimateEntityFeature.FAN_MODE
+            | ClimateEntityFeature.TURN_OFF
+            | ClimateEntityFeature.TURN_ON
+        )
+        if self._device.get_capability(Capability.fan_oscillation_mode):
+            features |= ClimateEntityFeature.SWING_MODE
+        if (self._attr_preset_modes is not None) and len(self._attr_preset_modes) > 0:
+            features |= ClimateEntityFeature.PRESET_MODE
+        return features
+
+    async def async_set_fan_mode(self, fan_mode: str) -> None:
         """Set new target fan mode."""
         await self._device.set_fan_mode(fan_mode, set_status=True)
+
+        # setting the fan must reset the preset mode (it deactivates the windFree function)
+        self._attr_preset_mode = None
+
         # State is set optimistically in the command above, therefore update
         # the entity state ahead of receiving the confirming push updates
         self.async_write_ha_state()
 
-    async def async_set_hvac_mode(self, hvac_mode):
+    async def async_set_hvac_mode(self, hvac_mode: HVACMode) -> None:
         """Set new target operation mode."""
-        if hvac_mode == HVAC_MODE_OFF:
+        if hvac_mode == HVACMode.OFF:
             await self.async_turn_off()
             return
         tasks = []
@@ -359,13 +399,12 @@ class SmartThingsAirConditioner(SmartThingsEntity, ClimateEntity):
         # the entity state ahead of receiving the confirming push updates
         self.async_write_ha_state()
 
-    async def async_set_temperature(self, **kwargs):
+    async def async_set_temperature(self, **kwargs: Any) -> None:
         """Set new target temperature."""
         tasks = []
         # operation mode
-        operation_mode = kwargs.get(ATTR_HVAC_MODE)
-        if operation_mode:
-            if operation_mode == HVAC_MODE_OFF:
+        if operation_mode := kwargs.get(ATTR_HVAC_MODE):
+            if operation_mode == HVACMode.OFF:
                 tasks.append(self._device.switch_off(set_status=True))
             else:
                 if not self._device.status.switch:
@@ -380,26 +419,25 @@ class SmartThingsAirConditioner(SmartThingsEntity, ClimateEntity):
         # the entity state ahead of receiving the confirming push updates
         self.async_write_ha_state()
 
-    async def async_turn_on(self):
+    async def async_turn_on(self) -> None:
         """Turn device on."""
         await self._device.switch_on(set_status=True)
         # State is set optimistically in the command above, therefore update
         # the entity state ahead of receiving the confirming push updates
         self.async_write_ha_state()
 
-    async def async_turn_off(self):
+    async def async_turn_off(self) -> None:
         """Turn device off."""
         await self._device.switch_off(set_status=True)
         # State is set optimistically in the command above, therefore update
         # the entity state ahead of receiving the confirming push updates
         self.async_write_ha_state()
 
-    async def async_update(self):
+    async def async_update(self) -> None:
         """Update the calculated fields of the AC."""
-        modes = {HVAC_MODE_OFF}
+        modes = {HVACMode.OFF}
         for mode in self._device.status.supported_ac_modes:
-            state = AC_MODE_TO_STATE.get(mode)
-            if state is not None:
+            if (state := AC_MODE_TO_STATE.get(mode)) is not None:
                 modes.add(state)
             else:
                 _LOGGER.debug(
@@ -411,14 +449,13 @@ class SmartThingsAirConditioner(SmartThingsEntity, ClimateEntity):
         self._hvac_modes = list(modes)
 
     @property
-    def current_temperature(self):
+    def current_temperature(self) -> float | None:
         """Return the current temperature."""
         return self._device.status.temperature
 
     @property
-    def extra_state_attributes(self):
-        """
-        Return device specific state attributes.
+    def extra_state_attributes(self) -> dict[str, Any]:
+        """Return device specific state attributes.
 
         Include attributes from the Demand Response Load Control (drlc)
         and Power Consumption capabilities.
@@ -428,10 +465,6 @@ class SmartThingsAirConditioner(SmartThingsEntity, ClimateEntity):
             "drlc_status_level",
             "drlc_status_start",
             "drlc_status_override",
-            "power_consumption_start",
-            "power_consumption_power",
-            "power_consumption_energy",
-            "power_consumption_end",
         ]
         state_attributes = {}
         for attribute in attributes:
@@ -441,38 +474,86 @@ class SmartThingsAirConditioner(SmartThingsEntity, ClimateEntity):
         return state_attributes
 
     @property
-    def fan_mode(self):
+    def fan_mode(self) -> str:
         """Return the fan setting."""
         return self._device.status.fan_mode
 
     @property
-    def fan_modes(self):
+    def fan_modes(self) -> list[str]:
         """Return the list of available fan modes."""
         return self._device.status.supported_ac_fan_modes
 
     @property
-    def hvac_mode(self):
+    def hvac_mode(self) -> HVACMode | None:
         """Return current operation ie. heat, cool, idle."""
         if not self._device.status.switch:
-            return HVAC_MODE_OFF
+            return HVACMode.OFF
         return AC_MODE_TO_STATE.get(self._device.status.air_conditioner_mode)
 
     @property
-    def hvac_modes(self):
+    def hvac_modes(self) -> list[HVACMode]:
         """Return the list of available operation modes."""
         return self._hvac_modes
 
     @property
-    def supported_features(self):
-        """Return the supported features."""
-        return SUPPORT_TARGET_TEMPERATURE | SUPPORT_FAN_MODE
-
-    @property
-    def target_temperature(self):
+    def target_temperature(self) -> float:
         """Return the temperature we try to reach."""
         return self._device.status.cooling_setpoint
 
     @property
-    def temperature_unit(self):
+    def temperature_unit(self) -> str:
         """Return the unit of measurement."""
-        return UNIT_MAP.get(self._device.status.attributes[Attribute.temperature].unit)
+        return UNIT_MAP[self._device.status.attributes[Attribute.temperature].unit]
+
+    def _determine_swing_modes(self) -> list[str] | None:
+        """Return the list of available swing modes."""
+        supported_swings = None
+        supported_modes = self._device.status.attributes[
+            Attribute.supported_fan_oscillation_modes
+        ][0]
+        if supported_modes is not None:
+            supported_swings = [
+                FAN_OSCILLATION_TO_SWING.get(m, SWING_OFF) for m in supported_modes
+            ]
+        return supported_swings
+
+    async def async_set_swing_mode(self, swing_mode: str) -> None:
+        """Set swing mode."""
+        fan_oscillation_mode = SWING_TO_FAN_OSCILLATION[swing_mode]
+        await self._device.set_fan_oscillation_mode(fan_oscillation_mode)
+
+        # setting the fan must reset the preset mode (it deactivates the windFree function)
+        self._attr_preset_mode = None
+
+        self.async_schedule_update_ha_state(True)
+
+    @property
+    def swing_mode(self) -> str:
+        """Return the swing setting."""
+        return FAN_OSCILLATION_TO_SWING.get(
+            self._device.status.fan_oscillation_mode, SWING_OFF
+        )
+
+    def _determine_preset_modes(self) -> list[str] | None:
+        """Return a list of available preset modes."""
+        supported_modes: list | None = self._device.status.attributes[
+            "supportedAcOptionalMode"
+        ].value
+        if supported_modes and WINDFREE in supported_modes:
+            return [WINDFREE]
+        return None
+
+    async def async_set_preset_mode(self, preset_mode: str) -> None:
+        """Set special modes (currently only windFree is supported)."""
+        result = await self._device.command(
+            "main",
+            "custom.airConditionerOptionalMode",
+            "setAcOptionalMode",
+            [preset_mode],
+        )
+        if result:
+            self._device.status.update_attribute_value("acOptionalMode", preset_mode)
+
+        self._attr_preset_mode = preset_mode
+
+        self.async_write_ha_state()
