@@ -5,7 +5,6 @@ import asyncio
 from collections import OrderedDict
 import logging
 
-import async_timeout
 from songpal import (
     ConnectChange,
     ContentChange,
@@ -17,6 +16,7 @@ from songpal import (
 import voluptuous as vol
 
 from homeassistant.components.media_player import (
+    MediaPlayerDeviceClass,
     MediaPlayerEntity,
     MediaPlayerEntityFeature,
     MediaPlayerState,
@@ -30,11 +30,11 @@ from homeassistant.helpers import (
     device_registry as dr,
     entity_platform,
 )
-from homeassistant.helpers.entity import DeviceInfo
+from homeassistant.helpers.device_registry import DeviceInfo
 from homeassistant.helpers.entity_platform import AddEntitiesCallback
 from homeassistant.helpers.typing import ConfigType, DiscoveryInfoType
 
-from .const import CONF_ENDPOINT, DOMAIN, SET_SOUND_SETTING
+from .const import CONF_ENDPOINT, DOMAIN, ERROR_REQUEST_RETRY, SET_SOUND_SETTING
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -52,7 +52,8 @@ async def async_setup_platform(
 ) -> None:
     """Set up from legacy configuration file. Obsolete."""
     _LOGGER.error(
-        "Configuring Songpal through media_player platform is no longer supported. Convert to songpal platform or UI configuration"
+        "Configuring Songpal through media_player platform is no longer supported."
+        " Convert to songpal platform or UI configuration"
     )
 
 
@@ -67,7 +68,7 @@ async def async_setup_entry(
 
     device = Device(endpoint)
     try:
-        async with async_timeout.timeout(
+        async with asyncio.timeout(
             10
         ):  # set timeout to avoid blocking the setup process
             await device.get_supported_methods()
@@ -91,6 +92,7 @@ class SongpalEntity(MediaPlayerEntity):
     """Class representing a Songpal device."""
 
     _attr_should_poll = False
+    _attr_device_class = MediaPlayerDeviceClass.RECEIVER
     _attr_supported_features = (
         MediaPlayerEntityFeature.VOLUME_SET
         | MediaPlayerEntityFeature.VOLUME_STEP
@@ -99,6 +101,8 @@ class SongpalEntity(MediaPlayerEntity):
         | MediaPlayerEntityFeature.TURN_ON
         | MediaPlayerEntityFeature.TURN_OFF
     )
+    _attr_has_entity_name = True
+    _attr_name = None
 
     def __init__(self, name, device):
         """Init."""
@@ -108,14 +112,14 @@ class SongpalEntity(MediaPlayerEntity):
         self._model = None
 
         self._state = False
-        self._available = False
+        self._attr_available = False
         self._initialized = False
 
         self._volume_control = None
         self._volume_min = 0
         self._volume_max = 1
         self._volume = 0
-        self._is_muted = False
+        self._attr_is_volume_muted = False
 
         self._active_source = None
         self._sources = {}
@@ -135,7 +139,7 @@ class SongpalEntity(MediaPlayerEntity):
         async def _volume_changed(volume: VolumeChange):
             _LOGGER.debug("Volume changed: %s", volume)
             self._volume = volume.volume
-            self._is_muted = volume.mute
+            self._attr_is_volume_muted = volume.mute
             self.async_write_ha_state()
 
         async def _source_changed(content: ContentChange):
@@ -159,13 +163,13 @@ class SongpalEntity(MediaPlayerEntity):
                 self._dev.endpoint,
             )
             _LOGGER.debug("Disconnected: %s", connect.exception)
-            self._available = False
+            self._attr_available = False
             self.async_write_ha_state()
 
             # Try to reconnect forever, a successful reconnect will initialize
             # the websocket connection again.
             delay = INITIAL_RETRY_DELAY
-            while not self._available:
+            while not self._attr_available:
                 _LOGGER.debug("Trying to reconnect in %s seconds", delay)
                 await asyncio.sleep(delay)
 
@@ -196,11 +200,6 @@ class SongpalEntity(MediaPlayerEntity):
 
         self.hass.loop.create_task(self._dev.listen_notifications())
 
-    @property
-    def name(self):
-        """Return name of the device."""
-        return self._name
-
     @property
     def unique_id(self):
         """Return a unique ID."""
@@ -219,15 +218,10 @@ class SongpalEntity(MediaPlayerEntity):
             identifiers={(DOMAIN, self.unique_id)},
             manufacturer="Sony Corporation",
             model=self._model,
-            name=self.name,
+            name=self._name,
             sw_version=self._sysinfo.version,
         )
 
-    @property
-    def available(self):
-        """Return availability of the device."""
-        return self._available
-
     async def async_set_sound_setting(self, name, value):
         """Change a setting on the device."""
         _LOGGER.debug("Calling set_sound_setting with %s: %s", name, value)
@@ -246,7 +240,7 @@ class SongpalEntity(MediaPlayerEntity):
             volumes = await self._dev.get_volume_information()
             if not volumes:
                 _LOGGER.error("Got no volume controls, bailing out")
-                self._available = False
+                self._attr_available = False
                 return
 
             if len(volumes) > 1:
@@ -259,7 +253,7 @@ class SongpalEntity(MediaPlayerEntity):
             self._volume_min = volume.minVolume
             self._volume = volume.volume
             self._volume_control = volume
-            self._is_muted = self._volume_control.is_muted
+            self._attr_is_volume_muted = self._volume_control.is_muted
 
             status = await self._dev.get_power()
             self._state = status.status
@@ -276,11 +270,11 @@ class SongpalEntity(MediaPlayerEntity):
 
             _LOGGER.debug("Active source: %s", self._active_source)
 
-            self._available = True
+            self._attr_available = True
 
         except SongpalException as ex:
             _LOGGER.error("Unable to update: %s", ex)
-            self._available = False
+            self._attr_available = False
 
     async def async_select_source(self, source: str) -> None:
         """Select source."""
@@ -312,8 +306,7 @@ class SongpalEntity(MediaPlayerEntity):
     @property
     def volume_level(self):
         """Return volume level."""
-        volume = self._volume / self._volume_max
-        return volume
+        return self._volume / self._volume_max
 
     async def async_set_volume_level(self, volume: float) -> None:
         """Set volume level."""
@@ -331,18 +324,29 @@ class SongpalEntity(MediaPlayerEntity):
 
     async def async_turn_on(self) -> None:
         """Turn the device on."""
-        return await self._dev.set_power(True)
+        try:
+            return await self._dev.set_power(True)
+        except SongpalException as ex:
+            if ex.code == ERROR_REQUEST_RETRY:
+                _LOGGER.debug(
+                    "Swallowing %s, the device might be already in the wanted state", ex
+                )
+                return
+            raise
 
     async def async_turn_off(self) -> None:
         """Turn the device off."""
-        return await self._dev.set_power(False)
+        try:
+            return await self._dev.set_power(False)
+        except SongpalException as ex:
+            if ex.code == ERROR_REQUEST_RETRY:
+                _LOGGER.debug(
+                    "Swallowing %s, the device might be already in the wanted state", ex
+                )
+                return
+            raise
 
     async def async_mute_volume(self, mute: bool) -> None:
         """Mute or unmute the device."""
         _LOGGER.debug("Set mute: %s", mute)
         return await self._volume_control.set_mute(mute)
-
-    @property
-    def is_volume_muted(self):
-        """Return whether the device is muted."""
-        return self._is_muted
