@@ -1,186 +1,63 @@
-"""Support for interface with a Bose Soundtouch."""
+"""Support for interface with a Bose SoundTouch."""
 from __future__ import annotations
 
 from functools import partial
 import logging
-import re
+from typing import Any
 
-from libsoundtouch import soundtouch_device
+from libsoundtouch.device import SoundTouchDevice
 from libsoundtouch.utils import Source
-import voluptuous as vol
 
 from homeassistant.components import media_source
 from homeassistant.components.media_player import (
-    PLATFORM_SCHEMA,
+    BrowseMedia,
+    MediaPlayerDeviceClass,
     MediaPlayerEntity,
     MediaPlayerEntityFeature,
-)
-from homeassistant.components.media_player.browse_media import (
+    MediaPlayerState,
+    MediaType,
     async_process_play_media_url,
 )
-from homeassistant.const import (
-    CONF_HOST,
-    CONF_NAME,
-    CONF_PORT,
-    EVENT_HOMEASSISTANT_START,
-    STATE_OFF,
-    STATE_PAUSED,
-    STATE_PLAYING,
-    STATE_UNAVAILABLE,
+from homeassistant.config_entries import ConfigEntry
+from homeassistant.const import EVENT_HOMEASSISTANT_START
+from homeassistant.core import HomeAssistant, callback
+from homeassistant.helpers.device_registry import (
+    CONNECTION_NETWORK_MAC,
+    DeviceInfo,
+    format_mac,
 )
-from homeassistant.core import HomeAssistant, ServiceCall, callback
-import homeassistant.helpers.config_validation as cv
 from homeassistant.helpers.entity_platform import AddEntitiesCallback
-from homeassistant.helpers.typing import ConfigType, DiscoveryInfoType
 
-from .const import (
-    DOMAIN,
-    SERVICE_ADD_ZONE_SLAVE,
-    SERVICE_CREATE_ZONE,
-    SERVICE_PLAY_EVERYWHERE,
-    SERVICE_REMOVE_ZONE_SLAVE,
-)
+from .const import DOMAIN
 
 _LOGGER = logging.getLogger(__name__)
 
 MAP_STATUS = {
-    "PLAY_STATE": STATE_PLAYING,
-    "BUFFERING_STATE": STATE_PLAYING,
-    "PAUSE_STATE": STATE_PAUSED,
-    "STOP_STATE": STATE_OFF,
+    "PLAY_STATE": MediaPlayerState.PLAYING,
+    "BUFFERING_STATE": MediaPlayerState.PLAYING,
+    "PAUSE_STATE": MediaPlayerState.PAUSED,
+    "STOP_STATE": MediaPlayerState.OFF,
 }
 
-DATA_SOUNDTOUCH = "soundtouch"
 ATTR_SOUNDTOUCH_GROUP = "soundtouch_group"
 ATTR_SOUNDTOUCH_ZONE = "soundtouch_zone"
 
-SOUNDTOUCH_PLAY_EVERYWHERE = vol.Schema({vol.Required("master"): cv.entity_id})
 
-SOUNDTOUCH_CREATE_ZONE_SCHEMA = vol.Schema(
-    {vol.Required("master"): cv.entity_id, vol.Required("slaves"): cv.entity_ids}
-)
-
-SOUNDTOUCH_ADD_ZONE_SCHEMA = vol.Schema(
-    {vol.Required("master"): cv.entity_id, vol.Required("slaves"): cv.entity_ids}
-)
-
-SOUNDTOUCH_REMOVE_ZONE_SCHEMA = vol.Schema(
-    {vol.Required("master"): cv.entity_id, vol.Required("slaves"): cv.entity_ids}
-)
-
-DEFAULT_NAME = "Bose Soundtouch"
-DEFAULT_PORT = 8090
-
-PLATFORM_SCHEMA = PLATFORM_SCHEMA.extend(
-    {
-        vol.Required(CONF_HOST): cv.string,
-        vol.Optional(CONF_NAME, default=DEFAULT_NAME): cv.string,
-        vol.Optional(CONF_PORT, default=DEFAULT_PORT): cv.port,
-    }
-)
-
-
-def setup_platform(
+async def async_setup_entry(
     hass: HomeAssistant,
-    config: ConfigType,
-    add_entities: AddEntitiesCallback,
-    discovery_info: DiscoveryInfoType | None = None,
+    entry: ConfigEntry,
+    async_add_entities: AddEntitiesCallback,
 ) -> None:
-    """Set up the Bose Soundtouch platform."""
-    if DATA_SOUNDTOUCH not in hass.data:
-        hass.data[DATA_SOUNDTOUCH] = []
+    """Set up the Bose SoundTouch media player based on a config entry."""
+    device = hass.data[DOMAIN][entry.entry_id].device
+    media_player = SoundTouchMediaPlayer(device)
 
-    if discovery_info:
-        host = discovery_info["host"]
-        port = int(discovery_info["port"])
+    async_add_entities([media_player], True)
 
-        # if device already exists by config
-        if host in [device.config["host"] for device in hass.data[DATA_SOUNDTOUCH]]:
-            return
-
-        remote_config = {"id": "ha.component.soundtouch", "host": host, "port": port}
-        bose_soundtouch_entity = SoundTouchDevice(None, remote_config)
-        hass.data[DATA_SOUNDTOUCH].append(bose_soundtouch_entity)
-        add_entities([bose_soundtouch_entity], True)
-    else:
-        name = config.get(CONF_NAME)
-        remote_config = {
-            "id": "ha.component.soundtouch",
-            "port": config.get(CONF_PORT),
-            "host": config.get(CONF_HOST),
-        }
-        bose_soundtouch_entity = SoundTouchDevice(name, remote_config)
-        hass.data[DATA_SOUNDTOUCH].append(bose_soundtouch_entity)
-        add_entities([bose_soundtouch_entity], True)
-
-    def service_handle(service: ServiceCall) -> None:
-        """Handle the applying of a service."""
-        master_device_id = service.data.get("master")
-        slaves_ids = service.data.get("slaves")
-        slaves = []
-        if slaves_ids:
-            slaves = [
-                device
-                for device in hass.data[DATA_SOUNDTOUCH]
-                if device.entity_id in slaves_ids
-            ]
-
-        master = next(
-            iter(
-                [
-                    device
-                    for device in hass.data[DATA_SOUNDTOUCH]
-                    if device.entity_id == master_device_id
-                ]
-            ),
-            None,
-        )
-
-        if master is None:
-            _LOGGER.warning(
-                "Unable to find master with entity_id: %s", str(master_device_id)
-            )
-            return
-
-        if service.service == SERVICE_PLAY_EVERYWHERE:
-            slaves = [
-                d for d in hass.data[DATA_SOUNDTOUCH] if d.entity_id != master_device_id
-            ]
-            master.create_zone(slaves)
-        elif service.service == SERVICE_CREATE_ZONE:
-            master.create_zone(slaves)
-        elif service.service == SERVICE_REMOVE_ZONE_SLAVE:
-            master.remove_zone_slave(slaves)
-        elif service.service == SERVICE_ADD_ZONE_SLAVE:
-            master.add_zone_slave(slaves)
-
-    hass.services.register(
-        DOMAIN,
-        SERVICE_PLAY_EVERYWHERE,
-        service_handle,
-        schema=SOUNDTOUCH_PLAY_EVERYWHERE,
-    )
-    hass.services.register(
-        DOMAIN,
-        SERVICE_CREATE_ZONE,
-        service_handle,
-        schema=SOUNDTOUCH_CREATE_ZONE_SCHEMA,
-    )
-    hass.services.register(
-        DOMAIN,
-        SERVICE_REMOVE_ZONE_SLAVE,
-        service_handle,
-        schema=SOUNDTOUCH_REMOVE_ZONE_SCHEMA,
-    )
-    hass.services.register(
-        DOMAIN,
-        SERVICE_ADD_ZONE_SLAVE,
-        service_handle,
-        schema=SOUNDTOUCH_ADD_ZONE_SCHEMA,
-    )
+    hass.data[DOMAIN][entry.entry_id].media_player = media_player
 
 
-class SoundTouchDevice(MediaPlayerEntity):
+class SoundTouchMediaPlayer(MediaPlayerEntity):
     """Representation of a SoundTouch Bose device."""
 
     _attr_supported_features = (
@@ -197,31 +74,40 @@ class SoundTouchDevice(MediaPlayerEntity):
         | MediaPlayerEntityFeature.SELECT_SOURCE
         | MediaPlayerEntityFeature.BROWSE_MEDIA
     )
+    _attr_device_class = MediaPlayerDeviceClass.SPEAKER
+    _attr_has_entity_name = True
+    _attr_name = None
+    _attr_source_list = [
+        Source.AUX.value,
+        Source.BLUETOOTH.value,
+    ]
 
-    def __init__(self, name, config):
-        """Create Soundtouch Entity."""
+    def __init__(self, device: SoundTouchDevice) -> None:
+        """Create SoundTouch media player entity."""
+
+        self._device = device
+
+        self._attr_unique_id = device.config.device_id
+        self._attr_device_info = DeviceInfo(
+            identifiers={(DOMAIN, device.config.device_id)},
+            connections={
+                (CONNECTION_NETWORK_MAC, format_mac(device.config.mac_address))
+            },
+            manufacturer="Bose Corporation",
+            model=device.config.type,
+            name=device.config.name,
+        )
 
-        self._device = soundtouch_device(config["host"], config["port"])
-        if name is None:
-            self._name = self._device.config.name
-        else:
-            self._name = name
         self._status = None
         self._volume = None
-        self._config = config
         self._zone = None
 
-    @property
-    def config(self):
-        """Return specific soundtouch configuration."""
-        return self._config
-
     @property
     def device(self):
-        """Return Soundtouch device."""
+        """Return SoundTouch device."""
         return self._device
 
-    def update(self):
+    def update(self) -> None:
         """Retrieve the latest data."""
         self._status = self._device.status()
         self._volume = self._device.volume()
@@ -233,77 +119,67 @@ class SoundTouchDevice(MediaPlayerEntity):
         return self._volume.actual / 100
 
     @property
-    def name(self):
-        """Return the name of the device."""
-        return self._name
-
-    @property
-    def state(self):
+    def state(self) -> MediaPlayerState | None:
         """Return the state of the device."""
-        if self._status.source == "STANDBY":
-            return STATE_OFF
+        if self._status is None or self._status.source == "STANDBY":
+            return MediaPlayerState.OFF
 
-        return MAP_STATUS.get(self._status.play_status, STATE_UNAVAILABLE)
+        if self._status.source == "INVALID_SOURCE":
+            return None
+
+        return MAP_STATUS.get(self._status.play_status)
 
     @property
     def source(self):
         """Name of the current input source."""
         return self._status.source
 
-    @property
-    def source_list(self):
-        """List of available input sources."""
-        return [
-            Source.AUX.value,
-            Source.BLUETOOTH.value,
-        ]
-
     @property
     def is_volume_muted(self):
         """Boolean if volume is currently muted."""
         return self._volume.muted
 
-    def turn_off(self):
+    def turn_off(self) -> None:
         """Turn off media player."""
         self._device.power_off()
 
-    def turn_on(self):
+    def turn_on(self) -> None:
         """Turn on media player."""
         self._device.power_on()
 
-    def volume_up(self):
+    def volume_up(self) -> None:
         """Volume up the media player."""
         self._device.volume_up()
 
-    def volume_down(self):
+    def volume_down(self) -> None:
         """Volume down media player."""
         self._device.volume_down()
 
-    def set_volume_level(self, volume):
+    def set_volume_level(self, volume: float) -> None:
         """Set volume level, range 0..1."""
         self._device.set_volume(int(volume * 100))
 
-    def mute_volume(self, mute):
+    def mute_volume(self, mute: bool) -> None:
         """Send mute command."""
         self._device.mute()
 
-    def media_play_pause(self):
+    def media_play_pause(self) -> None:
         """Simulate play pause media player."""
         self._device.play_pause()
 
-    def media_play(self):
+    def media_play(self) -> None:
         """Send play command."""
         self._device.play()
 
-    def media_pause(self):
+    def media_pause(self) -> None:
         """Send media pause command to media player."""
         self._device.pause()
 
-    def media_next_track(self):
+    def media_next_track(self) -> None:
         """Send next track command."""
         self._device.next_track()
 
-    def media_previous_track(self):
+    def media_previous_track(self) -> None:
         """Send the previous track command."""
         self._device.previous_track()
 
@@ -342,7 +218,7 @@ class SoundTouchDevice(MediaPlayerEntity):
         """Album name of current playing media."""
         return self._status.album
 
-    async def async_added_to_hass(self):
+    async def async_added_to_hass(self) -> None:
         """Populate zone info which requires entity_id."""
 
         @callback
@@ -354,7 +230,9 @@ class SoundTouchDevice(MediaPlayerEntity):
             EVENT_HOMEASSISTANT_START, async_update_on_start
         )
 
-    async def async_play_media(self, media_type, media_id, **kwargs):
+    async def async_play_media(
+        self, media_type: MediaType | str, media_id: str, **kwargs: Any
+    ) -> None:
         """Play a piece of media."""
         if media_source.is_media_source_id(media_id):
             play_item = await media_source.async_resolve_media(
@@ -366,10 +244,12 @@ class SoundTouchDevice(MediaPlayerEntity):
             partial(self.play_media, media_type, media_id, **kwargs)
         )
 
-    def play_media(self, media_type, media_id, **kwargs):
+    def play_media(
+        self, media_type: MediaType | str, media_id: str, **kwargs: Any
+    ) -> None:
         """Play a piece of media."""
         _LOGGER.debug("Starting media with media_id: %s", media_id)
-        if re.match(r"http?://", str(media_id)):
+        if str(media_id).lower().startswith("http://"):  # no https support
             # URL
             _LOGGER.debug("Playing URL %s", str(media_id))
             self._device.play_url(str(media_id))
@@ -388,7 +268,7 @@ class SoundTouchDevice(MediaPlayerEntity):
             else:
                 _LOGGER.warning("Unable to find preset with id %s", media_id)
 
-    def select_source(self, source):
+    def select_source(self, source: str) -> None:
         """Select input source."""
         if source == Source.AUX.value:
             _LOGGER.debug("Selecting source AUX")
@@ -400,8 +280,7 @@ class SoundTouchDevice(MediaPlayerEntity):
             _LOGGER.warning("Source %s is not supported", source)
 
     def create_zone(self, slaves):
-        """
-        Create a zone (multi-room)  and play on selected devices.
+        """Create a zone (multi-room)  and play on selected devices.
 
         :param slaves: slaves on which to play
 
@@ -413,10 +292,9 @@ class SoundTouchDevice(MediaPlayerEntity):
             self._device.create_zone([slave.device for slave in slaves])
 
     def remove_zone_slave(self, slaves):
-        """
-        Remove slave(s) from and existing zone (multi-room).
+        """Remove slave(s) from and existing zone (multi-room).
 
-        Zone must already exist and slaves array can not be empty.
+        Zone must already exist and slaves array cannot be empty.
         Note: If removing last slave, the zone will be deleted and you'll have
         to create a new one. You will not be able to add a new slave anymore
 
@@ -438,10 +316,9 @@ class SoundTouchDevice(MediaPlayerEntity):
                     self._device.remove_zone_slave([slave.device])
 
     def add_zone_slave(self, slaves):
-        """
-        Add slave(s) to and existing zone (multi-room).
+        """Add slave(s) to and existing zone (multi-room).
 
-        Zone must already exist and slaves array can not be empty.
+        Zone must already exist and slaves array cannot be empty.
 
         :param slaves:slaves to add
 
@@ -468,7 +345,11 @@ class SoundTouchDevice(MediaPlayerEntity):
 
         return attributes
 
-    async def async_browse_media(self, media_content_type=None, media_content_id=None):
+    async def async_browse_media(
+        self,
+        media_content_type: MediaType | str | None = None,
+        media_content_id: str | None = None,
+    ) -> BrowseMedia:
         """Implement the websocket media browsing helper."""
         return await media_source.async_browse_media(self.hass, media_content_id)
 
@@ -478,20 +359,20 @@ class SoundTouchDevice(MediaPlayerEntity):
         if not zone_status:
             return None
 
-        # Due to a bug in the SoundTouch API itself client devices do NOT return their
-        # siblings as part of the "slaves" list. Only the master has the full list of
-        # slaves for some reason. To compensate for this shortcoming we have to fetch
-        # the zone info from the master when the current device is a slave until this is
-        # fixed in the SoundTouch API or libsoundtouch, or of course until somebody has a
-        # better idea on how to fix this.
+        # Client devices do NOT return their siblings as part of the "slaves" list.
+        # Only the master has the full list of slaves. To compensate for this
+        # shortcoming we have to fetch the zone info from the master when the current
+        # device is a slave.
         # In addition to this shortcoming, libsoundtouch seems to report the "is_master"
-        # property wrong on some slaves, so the only reliable way to detect if the current
-        # devices is the master, is by comparing the master_id of the zone with the device_id
+        # property wrong on some slaves, so the only reliable way to detect
+        # if the current devices is the master, is by comparing the master_id
+        # of the zone with the device_id.
         if zone_status.master_id == self._device.config.device_id:
             return self._build_zone_info(self.entity_id, zone_status.slaves)
 
-        # The master device has to be searched by it's ID and not IP since libsoundtouch / BOSE API
-        # do not return the IP of the master for some slave objects/responses
+        # The master device has to be searched by it's ID and not IP since
+        # libsoundtouch / BOSE API do not return the IP of the master
+        # for some slave objects/responses
         master_instance = self._get_instance_by_id(zone_status.master_id)
         if master_instance is not None:
             master_zone_status = master_instance.device.zone_status()
@@ -499,22 +380,23 @@ class SoundTouchDevice(MediaPlayerEntity):
                 master_instance.entity_id, master_zone_status.slaves
             )
 
-        # We should never end up here since this means we haven't found a master device to get the
-        # correct zone info from. In this case, assume current device is master
+        # We should never end up here since this means we haven't found a master
+        # device to get the correct zone info from. In this case,
+        # assume current device is master
         return self._build_zone_info(self.entity_id, zone_status.slaves)
 
     def _get_instance_by_ip(self, ip_address):
         """Search and return a SoundTouchDevice instance by it's IP address."""
-        for instance in self.hass.data[DATA_SOUNDTOUCH]:
-            if instance and instance.config["host"] == ip_address:
-                return instance
+        for data in self.hass.data[DOMAIN].values():
+            if data.device.config.device_ip == ip_address:
+                return data.media_player
         return None
 
     def _get_instance_by_id(self, instance_id):
         """Search and return a SoundTouchDevice instance by it's ID (aka MAC address)."""
-        for instance in self.hass.data[DATA_SOUNDTOUCH]:
-            if instance and instance.device.config.device_id == instance_id:
-                return instance
+        for data in self.hass.data[DOMAIN].values():
+            if data.device.config.device_id == instance_id:
+                return data.media_player
         return None
 
     def _build_zone_info(self, master, zone_slaves):
@@ -523,7 +405,7 @@ class SoundTouchDevice(MediaPlayerEntity):
 
         for slave in zone_slaves:
             slave_instance = self._get_instance_by_ip(slave.device_ip)
-            if slave_instance:
+            if slave_instance and slave_instance.entity_id != master:
                 slaves.append(slave_instance.entity_id)
 
         attributes = {
