@@ -1,7 +1,9 @@
 """Support for Vanderbilt (formerly Siemens) SPC alarm systems."""
 from __future__ import annotations
 
+from pyspcwebgw import SpcWebGateway
 from pyspcwebgw.const import ZoneInput, ZoneType
+from pyspcwebgw.zone import Zone
 
 from homeassistant.components.binary_sensor import (
     BinarySensorDeviceClass,
@@ -15,12 +17,12 @@ from homeassistant.helpers.typing import ConfigType, DiscoveryInfoType
 from . import DATA_API, SIGNAL_UPDATE_SENSOR
 
 
-def _get_device_class(zone_type):
+def _get_device_class(zone_type: ZoneType) -> BinarySensorDeviceClass | None:
     return {
         ZoneType.ALARM: BinarySensorDeviceClass.MOTION,
         ZoneType.ENTRY_EXIT: BinarySensorDeviceClass.OPENING,
         ZoneType.FIRE: BinarySensorDeviceClass.SMOKE,
-        ZoneType.TECHNICAL: "power",
+        ZoneType.TECHNICAL: BinarySensorDeviceClass.POWER,
     }.get(zone_type)
 
 
@@ -33,7 +35,7 @@ async def async_setup_platform(
     """Set up the SPC binary sensor."""
     if discovery_info is None:
         return
-    api = hass.data[DATA_API]
+    api: SpcWebGateway = hass.data[DATA_API]
     async_add_entities(
         [
             SpcBinarySensor(zone)
@@ -46,11 +48,15 @@ async def async_setup_platform(
 class SpcBinarySensor(BinarySensorEntity):
     """Representation of a sensor based on a SPC zone."""
 
-    def __init__(self, zone):
+    _attr_should_poll = False
+
+    def __init__(self, zone: Zone) -> None:
         """Initialize the sensor device."""
         self._zone = zone
+        self._attr_name = zone.name
+        self._attr_device_class = _get_device_class(zone.type)
 
-    async def async_added_to_hass(self):
+    async def async_added_to_hass(self) -> None:
         """Call for adding new entities."""
         self.async_on_remove(
             async_dispatcher_connect(
@@ -61,26 +67,11 @@ class SpcBinarySensor(BinarySensorEntity):
         )
 
     @callback
-    def _update_callback(self):
+    def _update_callback(self) -> None:
         """Call update method."""
         self.async_schedule_update_ha_state(True)
 
     @property
-    def name(self):
-        """Return the name of the device."""
-        return self._zone.name
-
-    @property
-    def is_on(self):
+    def is_on(self) -> bool:
         """Whether the device is switched on."""
         return self._zone.input == ZoneInput.OPEN
-
-    @property
-    def should_poll(self):
-        """No polling needed."""
-        return False
-
-    @property
-    def device_class(self):
-        """Return the device class."""
-        return _get_device_class(self._zone.type)
