@@ -3,7 +3,11 @@ from __future__ import annotations
 
 from typing import Any
 
-from homeassistant.components.cover import CoverEntity, CoverEntityFeature
+from homeassistant.components.cover import (
+    DOMAIN as COVER_DOMAIN,
+    CoverEntity,
+    CoverEntityFeature,
+)
 from homeassistant.components.switch import DOMAIN as SWITCH_DOMAIN
 from homeassistant.config_entries import ConfigEntry
 from homeassistant.const import (
@@ -13,11 +17,14 @@ from homeassistant.const import (
     SERVICE_TURN_ON,
     STATE_ON,
 )
-from homeassistant.core import Event, HomeAssistant, callback
+from homeassistant.core import HomeAssistant, callback
 from homeassistant.helpers import entity_registry as er
 from homeassistant.helpers.entity_platform import AddEntitiesCallback
+from homeassistant.helpers.event import EventStateChangedData
+from homeassistant.helpers.typing import EventType
 
-from .entity import BaseEntity
+from .const import CONF_INVERT
+from .entity import BaseInvertableEntity
 
 
 async def async_setup_entry(
@@ -36,6 +43,8 @@ async def async_setup_entry(
             CoverSwitch(
                 hass,
                 config_entry.title,
+                COVER_DOMAIN,
+                config_entry.options[CONF_INVERT],
                 entity_id,
                 config_entry.entry_id,
             )
@@ -43,7 +52,7 @@ async def async_setup_entry(
     )
 
 
-class CoverSwitch(BaseEntity, CoverEntity):
+class CoverSwitch(BaseInvertableEntity, CoverEntity):
     """Represents a Switch as a Cover."""
 
     _attr_supported_features = CoverEntityFeature.OPEN | CoverEntityFeature.CLOSE
@@ -52,7 +61,7 @@ class CoverSwitch(BaseEntity, CoverEntity):
         """Open the cover."""
         await self.hass.services.async_call(
             SWITCH_DOMAIN,
-            SERVICE_TURN_ON,
+            SERVICE_TURN_OFF if self._invert_state else SERVICE_TURN_ON,
             {ATTR_ENTITY_ID: self._switch_entity_id},
             blocking=True,
             context=self._context,
@@ -62,14 +71,16 @@ class CoverSwitch(BaseEntity, CoverEntity):
         """Close cover."""
         await self.hass.services.async_call(
             SWITCH_DOMAIN,
-            SERVICE_TURN_OFF,
+            SERVICE_TURN_ON if self._invert_state else SERVICE_TURN_OFF,
             {ATTR_ENTITY_ID: self._switch_entity_id},
             blocking=True,
             context=self._context,
         )
 
     @callback
-    def async_state_changed_listener(self, event: Event | None = None) -> None:
+    def async_state_changed_listener(
+        self, event: EventType[EventStateChangedData] | None = None
+    ) -> None:
         """Handle child updates."""
         super().async_state_changed_listener(event)
         if (
@@ -78,4 +89,7 @@ class CoverSwitch(BaseEntity, CoverEntity):
         ):
             return
 
-        self._attr_is_closed = state.state != STATE_ON
+        if self._invert_state:
+            self._attr_is_closed = state.state == STATE_ON
+        else:
+            self._attr_is_closed = state.state != STATE_ON
