@@ -6,27 +6,24 @@ from datetime import timedelta
 import logging
 from typing import Any
 
-from aioswitcher.api import Command, SwitcherApi, SwitcherBaseResponse
+from aioswitcher.api import Command, SwitcherBaseResponse, SwitcherType1Api
 from aioswitcher.device import DeviceCategory, DeviceState
 import voluptuous as vol
 
-from homeassistant.components.switch import (
-    DEVICE_CLASS_OUTLET,
-    DEVICE_CLASS_SWITCH,
-    SwitchEntity,
-)
+from homeassistant.components.switch import SwitchDeviceClass, SwitchEntity
 from homeassistant.config_entries import ConfigEntry
 from homeassistant.core import HomeAssistant, callback
 from homeassistant.helpers import (
     config_validation as cv,
-    device_registry,
+    device_registry as dr,
     entity_platform,
 )
+from homeassistant.helpers.device_registry import DeviceInfo
 from homeassistant.helpers.dispatcher import async_dispatcher_connect
 from homeassistant.helpers.entity_platform import AddEntitiesCallback
 from homeassistant.helpers.update_coordinator import CoordinatorEntity
 
-from . import SwitcherDeviceWrapper
+from . import SwitcherDataUpdateCoordinator
 from .const import (
     CONF_AUTO_OFF,
     CONF_TIMER_MINUTES,
@@ -69,35 +66,36 @@ async def async_setup_entry(
     )
 
     @callback
-    def async_add_switch(wrapper: SwitcherDeviceWrapper) -> None:
+    def async_add_switch(coordinator: SwitcherDataUpdateCoordinator) -> None:
         """Add switch from Switcher device."""
-        if wrapper.data.device_type.category == DeviceCategory.POWER_PLUG:
-            async_add_entities([SwitcherPowerPlugSwitchEntity(wrapper)])
-        elif wrapper.data.device_type.category == DeviceCategory.WATER_HEATER:
-            async_add_entities([SwitcherWaterHeaterSwitchEntity(wrapper)])
+        if coordinator.data.device_type.category == DeviceCategory.POWER_PLUG:
+            async_add_entities([SwitcherPowerPlugSwitchEntity(coordinator)])
+        elif coordinator.data.device_type.category == DeviceCategory.WATER_HEATER:
+            async_add_entities([SwitcherWaterHeaterSwitchEntity(coordinator)])
 
     config_entry.async_on_unload(
         async_dispatcher_connect(hass, SIGNAL_DEVICE_ADD, async_add_switch)
     )
 
 
-class SwitcherBaseSwitchEntity(CoordinatorEntity, SwitchEntity):
+class SwitcherBaseSwitchEntity(
+    CoordinatorEntity[SwitcherDataUpdateCoordinator], SwitchEntity
+):
     """Representation of a Switcher switch entity."""
 
-    def __init__(self, wrapper: SwitcherDeviceWrapper) -> None:
+    _attr_has_entity_name = True
+    _attr_name = None
+
+    def __init__(self, coordinator: SwitcherDataUpdateCoordinator) -> None:
         """Initialize the entity."""
-        super().__init__(wrapper)
-        self.wrapper = wrapper
+        super().__init__(coordinator)
         self.control_result: bool | None = None
 
         # Entity class attributes
-        self._attr_name = wrapper.name
-        self._attr_unique_id = f"{wrapper.device_id}-{wrapper.mac_address}"
-        self._attr_device_info = {
-            "connections": {
-                (device_registry.CONNECTION_NETWORK_MAC, wrapper.mac_address)
-            }
-        }
+        self._attr_unique_id = f"{coordinator.device_id}-{coordinator.mac_address}"
+        self._attr_device_info = DeviceInfo(
+            connections={(dr.CONNECTION_NETWORK_MAC, coordinator.mac_address)}
+        )
 
     @callback
     def _handle_coordinator_update(self) -> None:
@@ -107,13 +105,17 @@ class SwitcherBaseSwitchEntity(CoordinatorEntity, SwitchEntity):
 
     async def _async_call_api(self, api: str, *args: Any) -> None:
         """Call Switcher API."""
-        _LOGGER.debug("Calling api for %s, api: '%s', args: %s", self.name, api, args)
+        _LOGGER.debug(
+            "Calling api for %s, api: '%s', args: %s", self.coordinator.name, api, args
+        )
         response: SwitcherBaseResponse = None
         error = None
 
         try:
-            async with SwitcherApi(
-                self.wrapper.data.ip_address, self.wrapper.device_id
+            async with SwitcherType1Api(
+                self.coordinator.data.ip_address,
+                self.coordinator.data.device_id,
+                self.coordinator.data.device_key,
             ) as swapi:
                 response = await getattr(swapi, api)(*args)
         except (asyncio.TimeoutError, OSError, RuntimeError) as err:
@@ -122,12 +124,12 @@ class SwitcherBaseSwitchEntity(CoordinatorEntity, SwitchEntity):
         if error or not response or not response.successful:
             _LOGGER.error(
                 "Call api for %s failed, api: '%s', args: %s, response/error: %s",
-                self.name,
+                self.coordinator.name,
                 api,
                 args,
                 response or error,
             )
-            self.wrapper.last_update_success = False
+            self.coordinator.last_update_success = False
 
     @property
     def is_on(self) -> bool:
@@ -135,15 +137,15 @@ class SwitcherBaseSwitchEntity(CoordinatorEntity, SwitchEntity):
         if self.control_result is not None:
             return self.control_result
 
-        return bool(self.wrapper.data.device_state == DeviceState.ON)
+        return bool(self.coordinator.data.device_state == DeviceState.ON)
 
-    async def async_turn_on(self, **kwargs: dict) -> None:
+    async def async_turn_on(self, **kwargs: Any) -> None:
         """Turn the entity on."""
         await self._async_call_api("control_device", Command.ON)
         self.control_result = True
         self.async_write_ha_state()
 
-    async def async_turn_off(self, **kwargs: dict) -> None:
+    async def async_turn_off(self, **kwargs: Any) -> None:
         """Turn the entity off."""
         await self._async_call_api("control_device", Command.OFF)
         self.control_result = False
@@ -154,7 +156,7 @@ class SwitcherBaseSwitchEntity(CoordinatorEntity, SwitchEntity):
         _LOGGER.warning(
             "Service '%s' is not supported by %s",
             SERVICE_SET_AUTO_OFF_NAME,
-            self.name,
+            self.coordinator.name,
         )
 
     async def async_turn_on_with_timer_service(self, timer_minutes: int) -> None:
@@ -162,20 +164,20 @@ class SwitcherBaseSwitchEntity(CoordinatorEntity, SwitchEntity):
         _LOGGER.warning(
             "Service '%s' is not supported by %s",
             SERVICE_TURN_ON_WITH_TIMER_NAME,
-            self.name,
+            self.coordinator.name,
         )
 
 
 class SwitcherPowerPlugSwitchEntity(SwitcherBaseSwitchEntity):
     """Representation of a Switcher power plug switch entity."""
 
-    _attr_device_class = DEVICE_CLASS_OUTLET
+    _attr_device_class = SwitchDeviceClass.OUTLET
 
 
 class SwitcherWaterHeaterSwitchEntity(SwitcherBaseSwitchEntity):
     """Representation of a Switcher water heater switch entity."""
 
-    _attr_device_class = DEVICE_CLASS_SWITCH
+    _attr_device_class = SwitchDeviceClass.SWITCH
 
     async def async_set_auto_off_service(self, auto_off: timedelta) -> None:
         """Use for handling setting device auto-off service calls."""
