@@ -11,23 +11,31 @@ from synology_dsm.exceptions import (
 )
 
 from homeassistant.components.camera import (
-    SUPPORT_STREAM,
     Camera,
     CameraEntityDescription,
+    CameraEntityFeature,
 )
 from homeassistant.config_entries import ConfigEntry
-from homeassistant.core import HomeAssistant
-from homeassistant.helpers.entity import DeviceInfo
+from homeassistant.core import HomeAssistant, callback
+from homeassistant.helpers.device_registry import DeviceInfo
+from homeassistant.helpers.dispatcher import async_dispatcher_connect
 from homeassistant.helpers.entity_platform import AddEntitiesCallback
-from homeassistant.helpers.update_coordinator import DataUpdateCoordinator
 
-from . import SynoApi, SynologyDSMBaseEntity
-from .const import COORDINATOR_CAMERAS, DOMAIN, SYNO_API, SynologyDSMEntityDescription
+from . import SynoApi
+from .const import (
+    CONF_SNAPSHOT_QUALITY,
+    DEFAULT_SNAPSHOT_QUALITY,
+    DOMAIN,
+    SIGNAL_CAMERA_SOURCE_CHANGED,
+)
+from .coordinator import SynologyDSMCameraUpdateCoordinator
+from .entity import SynologyDSMBaseEntity, SynologyDSMEntityDescription
+from .models import SynologyDSMData
 
 _LOGGER = logging.getLogger(__name__)
 
 
-@dataclass
+@dataclass(frozen=True)
 class SynologyDSMCameraEntityDescription(
     CameraEntityDescription, SynologyDSMEntityDescription
 ):
@@ -38,35 +46,24 @@ async def async_setup_entry(
     hass: HomeAssistant, entry: ConfigEntry, async_add_entities: AddEntitiesCallback
 ) -> None:
     """Set up the Synology NAS cameras."""
-
-    data = hass.data[DOMAIN][entry.unique_id]
-    api: SynoApi = data[SYNO_API]
-
-    if SynoSurveillanceStation.CAMERA_API_KEY not in api.dsm.apis:
-        return
-
-    # initial data fetch
-    coordinator: DataUpdateCoordinator[dict[str, dict[str, SynoCamera]]] = data[
-        COORDINATOR_CAMERAS
-    ]
-    await coordinator.async_config_entry_first_refresh()
-
-    async_add_entities(
-        SynoDSMCamera(api, coordinator, camera_id)
-        for camera_id in coordinator.data["cameras"]
-    )
+    data: SynologyDSMData = hass.data[DOMAIN][entry.unique_id]
+    if coordinator := data.coordinator_cameras:
+        async_add_entities(
+            SynoDSMCamera(data.api, coordinator, camera_id)
+            for camera_id in coordinator.data["cameras"]
+        )
 
 
-class SynoDSMCamera(SynologyDSMBaseEntity, Camera):
+class SynoDSMCamera(SynologyDSMBaseEntity[SynologyDSMCameraUpdateCoordinator], Camera):
     """Representation a Synology camera."""
 
-    coordinator: DataUpdateCoordinator[dict[str, dict[str, SynoCamera]]]
+    _attr_supported_features = CameraEntityFeature.STREAM
     entity_description: SynologyDSMCameraEntityDescription
 
     def __init__(
         self,
         api: SynoApi,
-        coordinator: DataUpdateCoordinator[dict[str, dict[str, SynoCamera]]],
+        coordinator: SynologyDSMCameraUpdateCoordinator,
         camera_id: str,
     ) -> None:
         """Initialize a Synology camera."""
@@ -78,6 +75,9 @@ class SynoDSMCamera(SynologyDSMBaseEntity, Camera):
                 camera_id
             ].is_enabled,
         )
+        self.snapshot_quality = api._entry.options.get(
+            CONF_SNAPSHOT_QUALITY, DEFAULT_SNAPSHOT_QUALITY
+        )
         super().__init__(api, coordinator, description)
         Camera.__init__(self)
 
@@ -109,11 +109,6 @@ class SynoDSMCamera(SynologyDSMBaseEntity, Camera):
         """Return the availability of the camera."""
         return self.camera_data.is_enabled and self.coordinator.last_update_success
 
-    @property
-    def supported_features(self) -> int:
-        """Return supported features of this camera."""
-        return SUPPORT_STREAM
-
     @property
     def is_recording(self) -> bool:
         """Return true if the device is recording."""
@@ -124,7 +119,30 @@ class SynoDSMCamera(SynologyDSMBaseEntity, Camera):
         """Return the camera motion detection status."""
         return self.camera_data.is_motion_detection_enabled  # type: ignore[no-any-return]
 
-    def camera_image(
+    def _listen_source_updates(self) -> None:
+        """Listen for camera source changed events."""
+
+        @callback
+        def _handle_signal(url: str) -> None:
+            if self.stream:
+                _LOGGER.debug("Update stream URL for camera %s", self.camera_data.name)
+                self.stream.update_source(url)
+
+        assert self.platform.config_entry
+        self.async_on_remove(
+            async_dispatcher_connect(
+                self.hass,
+                f"{SIGNAL_CAMERA_SOURCE_CHANGED}_{self.platform.config_entry.entry_id}_{self.camera_data.id}",
+                _handle_signal,
+            )
+        )
+
+    async def async_added_to_hass(self) -> None:
+        """Subscribe to signal."""
+        self._listen_source_updates()
+        await super().async_added_to_hass()
+
+    async def async_camera_image(
         self, width: int | None = None, height: int | None = None
     ) -> bytes | None:
         """Return bytes of camera image."""
@@ -135,7 +153,9 @@ class SynoDSMCamera(SynologyDSMBaseEntity, Camera):
         if not self.available:
             return None
         try:
-            return self._api.surveillance_station.get_camera_image(self.entity_description.key)  # type: ignore[no-any-return]
+            return await self._api.surveillance_station.get_camera_image(  # type: ignore[no-any-return]
+                self.entity_description.key, self.snapshot_quality
+            )
         except (
             SynologyDSMAPIErrorException,
             SynologyDSMRequestException,
@@ -156,24 +176,25 @@ class SynoDSMCamera(SynologyDSMBaseEntity, Camera):
         )
         if not self.available:
             return None
+
         return self.camera_data.live_view.rtsp  # type: ignore[no-any-return]
 
-    def enable_motion_detection(self) -> None:
+    async def async_enable_motion_detection(self) -> None:
         """Enable motion detection in the camera."""
         _LOGGER.debug(
             "SynoDSMCamera.enable_motion_detection(%s)",
             self.camera_data.name,
         )
-        self._api.surveillance_station.enable_motion_detection(
+        await self._api.surveillance_station.enable_motion_detection(
             self.entity_description.key
         )
 
-    def disable_motion_detection(self) -> None:
+    async def async_disable_motion_detection(self) -> None:
         """Disable motion detection in camera."""
         _LOGGER.debug(
             "SynoDSMCamera.disable_motion_detection(%s)",
             self.camera_data.name,
         )
-        self._api.surveillance_station.disable_motion_detection(
+        await self._api.surveillance_station.disable_motion_detection(
             self.entity_description.key
         )
