@@ -1,8 +1,10 @@
 """Config flow to configure the Synology DSM integration."""
 from __future__ import annotations
 
+from collections.abc import Mapping
+from ipaddress import ip_address as ip
 import logging
-from typing import Any
+from typing import Any, cast
 from urllib.parse import urlparse
 
 from synology_dsm import SynologyDSM
@@ -16,7 +18,7 @@ from synology_dsm.exceptions import (
 import voluptuous as vol
 
 from homeassistant import exceptions
-from homeassistant.components import ssdp
+from homeassistant.components import ssdp, zeroconf
 from homeassistant.config_entries import ConfigEntry, ConfigFlow, OptionsFlow
 from homeassistant.const import (
     CONF_DISKS,
@@ -33,15 +35,19 @@ from homeassistant.const import (
 )
 from homeassistant.core import callback
 from homeassistant.data_entry_flow import FlowResult
+from homeassistant.helpers.aiohttp_client import async_get_clientsession
 import homeassistant.helpers.config_validation as cv
 from homeassistant.helpers.typing import DiscoveryInfoType
+from homeassistant.util.network import is_ip_address as is_ip
 
 from .const import (
     CONF_DEVICE_TOKEN,
+    CONF_SNAPSHOT_QUALITY,
     CONF_VOLUMES,
     DEFAULT_PORT,
     DEFAULT_PORT_SSL,
     DEFAULT_SCAN_INTERVAL,
+    DEFAULT_SNAPSHOT_QUALITY,
     DEFAULT_TIMEOUT,
     DEFAULT_USE_SSL,
     DEFAULT_VERIFY_SSL,
@@ -52,11 +58,22 @@ _LOGGER = logging.getLogger(__name__)
 
 CONF_OTP_CODE = "otp_code"
 
+HTTP_SUFFIX = "._http._tcp.local."
+
 
 def _discovery_schema_with_defaults(discovery_info: DiscoveryInfoType) -> vol.Schema:
     return vol.Schema(_ordered_shared_schema(discovery_info))
 
 
+def _reauth_schema() -> vol.Schema:
+    return vol.Schema(
+        {
+            vol.Required(CONF_USERNAME): str,
+            vol.Required(CONF_PASSWORD): str,
+        }
+    )
+
+
 def _user_schema_with_defaults(user_input: dict[str, Any]) -> vol.Schema:
     user_schema = {
         vol.Required(CONF_HOST, default=user_input.get(CONF_HOST, "")): str,
@@ -67,7 +84,7 @@ def _user_schema_with_defaults(user_input: dict[str, Any]) -> vol.Schema:
 
 
 def _ordered_shared_schema(
-    schema_input: dict[str, Any]
+    schema_input: dict[str, Any],
 ) -> dict[vol.Required | vol.Optional, Any]:
     return {
         vol.Required(CONF_USERNAME, default=schema_input.get(CONF_USERNAME, "")): str,
@@ -83,6 +100,11 @@ def _ordered_shared_schema(
     }
 
 
+def format_synology_mac(mac: str) -> str:
+    """Format a mac address to the format used by Synology DSM."""
+    return mac.replace(":", "").replace("-", "").upper()
+
+
 class SynologyDSMFlowHandler(ConfigFlow, domain=DOMAIN):
     """Handle a config flow."""
 
@@ -100,9 +122,12 @@ class SynologyDSMFlowHandler(ConfigFlow, domain=DOMAIN):
         """Initialize the synology_dsm config flow."""
         self.saved_user_input: dict[str, Any] = {}
         self.discovered_conf: dict[str, Any] = {}
+        self.reauth_conf: Mapping[str, Any] = {}
+        self.reauth_reason: str | None = None
 
-    async def _show_setup_form(
+    def _show_form(
         self,
+        step_id: str,
         user_input: dict[str, Any] | None = None,
         errors: dict[str, str] | None = None,
     ) -> FlowResult:
@@ -110,33 +135,29 @@ class SynologyDSMFlowHandler(ConfigFlow, domain=DOMAIN):
         if not user_input:
             user_input = {}
 
-        if self.discovered_conf:
+        description_placeholders = {}
+        data_schema = {}
+
+        if step_id == "link":
             user_input.update(self.discovered_conf)
-            step_id = "link"
             data_schema = _discovery_schema_with_defaults(user_input)
-        else:
-            step_id = "user"
+            description_placeholders = self.discovered_conf
+        elif step_id == "reauth_confirm":
+            data_schema = _reauth_schema()
+        elif step_id == "user":
             data_schema = _user_schema_with_defaults(user_input)
 
         return self.async_show_form(
             step_id=step_id,
             data_schema=data_schema,
             errors=errors or {},
-            description_placeholders=self.discovered_conf or {},
+            description_placeholders=description_placeholders,
         )
 
-    async def async_step_user(
-        self, user_input: dict[str, Any] | None = None
+    async def async_validate_input_create_entry(
+        self, user_input: dict[str, Any], step_id: str
     ) -> FlowResult:
-        """Handle a flow initiated by the user."""
-        errors = {}
-
-        if user_input is None:
-            return await self._show_setup_form(user_input, None)
-
-        if self.discovered_conf:
-            user_input.update(self.discovered_conf)
-
+        """Process user input and create new or update existing config entry."""
         host = user_input[CONF_HOST]
         port = user_input.get(CONF_PORT)
         username = user_input[CONF_USERNAME]
@@ -144,6 +165,7 @@ class SynologyDSMFlowHandler(ConfigFlow, domain=DOMAIN):
         use_ssl = user_input.get(CONF_SSL, DEFAULT_USE_SSL)
         verify_ssl = user_input.get(CONF_VERIFY_SSL, DEFAULT_VERIFY_SSL)
         otp_code = user_input.get(CONF_OTP_CODE)
+        friendly_name = user_input.get(CONF_NAME)
 
         if not port:
             if use_ssl is True:
@@ -151,14 +173,12 @@ class SynologyDSMFlowHandler(ConfigFlow, domain=DOMAIN):
             else:
                 port = DEFAULT_PORT
 
-        api = SynologyDSM(
-            host, port, username, password, use_ssl, verify_ssl, timeout=30
-        )
+        session = async_get_clientsession(self.hass, verify_ssl)
+        api = SynologyDSM(session, host, port, username, password, use_ssl, timeout=30)
 
+        errors = {}
         try:
-            serial = await self.hass.async_add_executor_job(
-                _login_and_fetch_syno_info, api, otp_code
-            )
+            serial = await _login_and_fetch_syno_info(api, otp_code)
         except SynologyDSMLogin2SARequiredException:
             return await self.async_step_2sa(user_input)
         except SynologyDSMLogin2SAFailedException:
@@ -178,13 +198,10 @@ class SynologyDSMFlowHandler(ConfigFlow, domain=DOMAIN):
             errors["base"] = "missing_data"
 
         if errors:
-            return await self._show_setup_form(user_input, errors)
+            return self._show_form(step_id, user_input, errors)
 
         # unique_id should be serial for services purpose
-        await self.async_set_unique_id(serial, raise_on_progress=False)
-
-        # Check if already configured
-        self._abort_if_unique_id_configured()
+        existing_entry = await self.async_set_unique_id(serial, raise_on_progress=False)
 
         config_data = {
             CONF_HOST: host,
@@ -202,40 +219,118 @@ class SynologyDSMFlowHandler(ConfigFlow, domain=DOMAIN):
         if user_input.get(CONF_VOLUMES):
             config_data[CONF_VOLUMES] = user_input[CONF_VOLUMES]
 
-        return self.async_create_entry(title=host, data=config_data)
+        if existing_entry:
+            reason = (
+                "reauth_successful" if self.reauth_conf else "reconfigure_successful"
+            )
+            return self.async_update_reload_and_abort(
+                existing_entry, data=config_data, reason=reason
+            )
 
-    async def async_step_ssdp(self, discovery_info: DiscoveryInfoType) -> FlowResult:
-        """Handle a discovered synology_dsm."""
-        parsed_url = urlparse(discovery_info[ssdp.ATTR_SSDP_LOCATION])
-        friendly_name = (
-            discovery_info[ssdp.ATTR_UPNP_FRIENDLY_NAME].split("(", 1)[0].strip()
-        )
+        return self.async_create_entry(title=friendly_name or host, data=config_data)
 
-        mac = discovery_info[ssdp.ATTR_UPNP_SERIAL].upper()
+    async def async_step_user(
+        self, user_input: dict[str, Any] | None = None
+    ) -> FlowResult:
+        """Handle a flow initiated by the user."""
+        step = "user"
+        if not user_input:
+            return self._show_form(step)
+        return await self.async_validate_input_create_entry(user_input, step_id=step)
+
+    async def async_step_zeroconf(
+        self, discovery_info: zeroconf.ZeroconfServiceInfo
+    ) -> FlowResult:
+        """Handle a discovered synology_dsm via zeroconf."""
+        discovered_macs = [
+            format_synology_mac(mac)
+            for mac in discovery_info.properties.get("mac_address", "").split("|")
+            if mac
+        ]
+        if not discovered_macs:
+            return self.async_abort(reason="no_mac_address")
+        host = discovery_info.host
+        friendly_name = discovery_info.name.removesuffix(HTTP_SUFFIX)
+        return await self._async_from_discovery(host, friendly_name, discovered_macs)
+
+    async def async_step_ssdp(self, discovery_info: ssdp.SsdpServiceInfo) -> FlowResult:
+        """Handle a discovered synology_dsm via ssdp."""
+        parsed_url = urlparse(discovery_info.ssdp_location)
+        upnp_friendly_name: str = discovery_info.upnp[ssdp.ATTR_UPNP_FRIENDLY_NAME]
+        friendly_name = upnp_friendly_name.split("(", 1)[0].strip()
+        mac_address = discovery_info.upnp[ssdp.ATTR_UPNP_SERIAL]
+        discovered_macs = [format_synology_mac(mac_address)]
         # Synology NAS can broadcast on multiple IP addresses, since they can be connected to multiple ethernets.
         # The serial of the NAS is actually its MAC address.
-        if self._mac_already_configured(mac):
-            return self.async_abort(reason="already_configured")
+        host = cast(str, parsed_url.hostname)
+        return await self._async_from_discovery(host, friendly_name, discovered_macs)
 
-        await self.async_set_unique_id(mac)
-        self._abort_if_unique_id_configured()
+    async def _async_from_discovery(
+        self, host: str, friendly_name: str, discovered_macs: list[str]
+    ) -> FlowResult:
+        """Handle a discovered synology_dsm via zeroconf or ssdp."""
+        existing_entry = None
+        for discovered_mac in discovered_macs:
+            await self.async_set_unique_id(discovered_mac)
+            if existing_entry := self._async_get_existing_entry(discovered_mac):
+                break
+            self._abort_if_unique_id_configured()
+
+        if (
+            existing_entry
+            and is_ip(existing_entry.data[CONF_HOST])
+            and is_ip(host)
+            and existing_entry.data[CONF_HOST] != host
+            and ip(existing_entry.data[CONF_HOST]).version == ip(host).version
+        ):
+            _LOGGER.info(
+                "Update host from '%s' to '%s' for NAS '%s' via discovery",
+                existing_entry.data[CONF_HOST],
+                host,
+                existing_entry.unique_id,
+            )
+            self.hass.config_entries.async_update_entry(
+                existing_entry,
+                data={**existing_entry.data, CONF_HOST: host},
+            )
+            return self.async_abort(reason="reconfigure_successful")
+
+        if existing_entry:
+            return self.async_abort(reason="already_configured")
 
         self.discovered_conf = {
             CONF_NAME: friendly_name,
-            CONF_HOST: parsed_url.hostname,
+            CONF_HOST: host,
         }
         self.context["title_placeholders"] = self.discovered_conf
-        return await self.async_step_user()
+        return await self.async_step_link()
 
-    async def async_step_import(
+    async def async_step_link(
         self, user_input: dict[str, Any] | None = None
     ) -> FlowResult:
-        """Import a config entry."""
-        return await self.async_step_user(user_input)
-
-    async def async_step_link(self, user_input: dict[str, Any]) -> FlowResult:
         """Link a config entry from discovery."""
-        return await self.async_step_user(user_input)
+        step = "link"
+        if not user_input:
+            return self._show_form(step)
+        user_input = {**self.discovered_conf, **user_input}
+        return await self.async_validate_input_create_entry(user_input, step_id=step)
+
+    async def async_step_reauth(self, entry_data: Mapping[str, Any]) -> FlowResult:
+        """Perform reauth upon an API authentication error."""
+        self.reauth_conf = entry_data
+        self.context["title_placeholders"][CONF_HOST] = entry_data[CONF_HOST]
+
+        return await self.async_step_reauth_confirm()
+
+    async def async_step_reauth_confirm(
+        self, user_input: dict[str, Any] | None = None
+    ) -> FlowResult:
+        """Perform reauth confirm upon an API authentication error."""
+        step = "reauth_confirm"
+        if not user_input:
+            return self._show_form(step)
+        user_input = {**self.reauth_conf, **user_input}
+        return await self.async_validate_input_create_entry(user_input, step_id=step)
 
     async def async_step_2sa(
         self, user_input: dict[str, Any], errors: dict[str, str] | None = None
@@ -256,14 +351,14 @@ class SynologyDSMFlowHandler(ConfigFlow, domain=DOMAIN):
 
         return await self.async_step_user(user_input)
 
-    def _mac_already_configured(self, mac: str) -> bool:
-        """See if we already have configured a NAS with this MAC address."""
-        existing_macs = [
-            mac.replace("-", "")
-            for entry in self._async_current_entries()
-            for mac in entry.data.get(CONF_MAC, [])
-        ]
-        return mac in existing_macs
+    def _async_get_existing_entry(self, discovered_mac: str) -> ConfigEntry | None:
+        """See if we already have a configured NAS with this MAC address."""
+        for entry in self._async_current_entries():
+            if discovered_mac in [
+                format_synology_mac(mac) for mac in entry.data.get(CONF_MAC, [])
+            ]:
+                return entry
+        return None
 
 
 class SynologyDSMOptionsFlowHandler(OptionsFlow):
@@ -282,30 +377,36 @@ class SynologyDSMOptionsFlowHandler(OptionsFlow):
 
         data_schema = vol.Schema(
             {
-                vol.Optional(
+                vol.Required(
                     CONF_SCAN_INTERVAL,
                     default=self.config_entry.options.get(
                         CONF_SCAN_INTERVAL, DEFAULT_SCAN_INTERVAL
                     ),
                 ): cv.positive_int,
-                vol.Optional(
+                vol.Required(
                     CONF_TIMEOUT,
                     default=self.config_entry.options.get(
                         CONF_TIMEOUT, DEFAULT_TIMEOUT
                     ),
                 ): cv.positive_int,
+                vol.Required(
+                    CONF_SNAPSHOT_QUALITY,
+                    default=self.config_entry.options.get(
+                        CONF_SNAPSHOT_QUALITY, DEFAULT_SNAPSHOT_QUALITY
+                    ),
+                ): vol.All(vol.Coerce(int), vol.Range(min=0, max=2)),
             }
         )
         return self.async_show_form(step_id="init", data_schema=data_schema)
 
 
-def _login_and_fetch_syno_info(api: SynologyDSM, otp_code: str) -> str:
+async def _login_and_fetch_syno_info(api: SynologyDSM, otp_code: str | None) -> str:
     """Login to the NAS and fetch basic data."""
     # These do i/o
-    api.login(otp_code)
-    api.utilisation.update()
-    api.storage.update()
-    api.network.update()
+    await api.login(otp_code)
+    await api.utilisation.update()
+    await api.storage.update()
+    await api.network.update()
 
     if (
         not api.information.serial
