@@ -10,43 +10,51 @@ from PyTado.const import (
     CONST_HVAC_OFF,
 )
 
-from homeassistant.components.climate.const import (
-    CURRENT_HVAC_COOL,
-    CURRENT_HVAC_DRY,
-    CURRENT_HVAC_FAN,
-    CURRENT_HVAC_HEAT,
-    CURRENT_HVAC_IDLE,
-    CURRENT_HVAC_OFF,
+from homeassistant.components.climate import (
     FAN_AUTO,
     FAN_HIGH,
     FAN_LOW,
     FAN_MEDIUM,
     FAN_OFF,
-    HVAC_MODE_AUTO,
-    HVAC_MODE_COOL,
-    HVAC_MODE_DRY,
-    HVAC_MODE_FAN_ONLY,
-    HVAC_MODE_HEAT,
-    HVAC_MODE_HEAT_COOL,
-    HVAC_MODE_OFF,
     PRESET_AWAY,
     PRESET_HOME,
+    SWING_OFF,
+    SWING_ON,
+    HVACAction,
+    HVACMode,
 )
 
 TADO_HVAC_ACTION_TO_HA_HVAC_ACTION = {
-    CONST_HVAC_HEAT: CURRENT_HVAC_HEAT,
-    CONST_HVAC_DRY: CURRENT_HVAC_DRY,
-    CONST_HVAC_FAN: CURRENT_HVAC_FAN,
-    CONST_HVAC_COOL: CURRENT_HVAC_COOL,
-    CONST_HVAC_IDLE: CURRENT_HVAC_IDLE,
-    CONST_HVAC_OFF: CURRENT_HVAC_OFF,
-    CONST_HVAC_HOT_WATER: CURRENT_HVAC_HEAT,
+    CONST_HVAC_HEAT: HVACAction.HEATING,
+    CONST_HVAC_DRY: HVACAction.DRYING,
+    CONST_HVAC_FAN: HVACAction.FAN,
+    CONST_HVAC_COOL: HVACAction.COOLING,
+    CONST_HVAC_IDLE: HVACAction.IDLE,
+    CONST_HVAC_OFF: HVACAction.OFF,
+    CONST_HVAC_HOT_WATER: HVACAction.HEATING,
 }
 
 # Configuration
 CONF_FALLBACK = "fallback"
+CONF_HOME_ID = "home_id"
 DATA = "data"
 UPDATE_TRACK = "update_track"
+UPDATE_MOBILE_DEVICE_TRACK = "update_mobile_device_track"
+
+# Weather
+CONDITIONS_MAP = {
+    "clear-night": {"NIGHT_CLEAR"},
+    "cloudy": {"CLOUDY", "CLOUDY_MOSTLY", "NIGHT_CLOUDY"},
+    "fog": {"FOGGY"},
+    "hail": {"HAIL", "RAIN_HAIL"},
+    "lightning": {"THUNDERSTORM"},
+    "partlycloudy": {"CLOUDY_PARTLY"},
+    "rainy": {"DRIZZLE", "RAIN", "SCATTERED_RAIN"},
+    "snowy": {"FREEZING", "SCATTERED_SNOW", "SNOW"},
+    "snowy-rainy": {"RAIN_SNOW", "SCATTERED_RAIN_SNOW"},
+    "sunny": {"SUN"},
+    "windy": {"WIND"},
+}
 
 # Types
 TYPE_AIR_CONDITIONING = "AIR_CONDITIONING"
@@ -80,6 +88,17 @@ CONST_OVERLAY_TADO_MODE = (
 )
 CONST_OVERLAY_MANUAL = "MANUAL"  # the user has change the temperature or mode manually
 CONST_OVERLAY_TIMER = "TIMER"  # the temperature will be reset after a timespan
+CONST_OVERLAY_TADO_DEFAULT = (
+    "TADO_DEFAULT"  # use the setting from tado zone itself (set in Tado app or webapp)
+)
+CONST_OVERLAY_TADO_OPTIONS = [
+    CONST_OVERLAY_TADO_MODE,
+    CONST_OVERLAY_MANUAL,
+    CONST_OVERLAY_TADO_DEFAULT,
+]
+CONST_EXCLUSIVE_OVERLAY_GROUP = (
+    "overlay_group"  # Overlay group for set_climate_timer service
+)
 
 
 # Heat always comes first since we get the
@@ -95,14 +114,14 @@ ORDERED_KNOWN_TADO_MODES = [
 ]
 
 TADO_MODES_TO_HA_CURRENT_HVAC_ACTION = {
-    CONST_MODE_HEAT: CURRENT_HVAC_HEAT,
-    CONST_MODE_DRY: CURRENT_HVAC_DRY,
-    CONST_MODE_FAN: CURRENT_HVAC_FAN,
-    CONST_MODE_COOL: CURRENT_HVAC_COOL,
+    CONST_MODE_HEAT: HVACAction.HEATING,
+    CONST_MODE_DRY: HVACAction.DRYING,
+    CONST_MODE_FAN: HVACAction.FAN,
+    CONST_MODE_COOL: HVACAction.COOLING,
 }
 
 # These modes will not allow a temp to be set
-TADO_MODES_WITH_NO_TEMP_SETTING = [CONST_MODE_AUTO, CONST_MODE_DRY, CONST_MODE_FAN]
+TADO_MODES_WITH_NO_TEMP_SETTING = [CONST_MODE_AUTO, CONST_MODE_FAN]
 #
 # HVAC_MODE_HEAT_COOL is mapped to CONST_MODE_AUTO
 #    This lets tado decide on a temp
@@ -111,13 +130,13 @@ TADO_MODES_WITH_NO_TEMP_SETTING = [CONST_MODE_AUTO, CONST_MODE_DRY, CONST_MODE_F
 #    This runs the smart schedule
 #
 HA_TO_TADO_HVAC_MODE_MAP = {
-    HVAC_MODE_OFF: CONST_MODE_OFF,
-    HVAC_MODE_HEAT_COOL: CONST_MODE_AUTO,
-    HVAC_MODE_AUTO: CONST_MODE_SMART_SCHEDULE,
-    HVAC_MODE_HEAT: CONST_MODE_HEAT,
-    HVAC_MODE_COOL: CONST_MODE_COOL,
-    HVAC_MODE_DRY: CONST_MODE_DRY,
-    HVAC_MODE_FAN_ONLY: CONST_MODE_FAN,
+    HVACMode.OFF: CONST_MODE_OFF,
+    HVACMode.HEAT_COOL: CONST_MODE_AUTO,
+    HVACMode.AUTO: CONST_MODE_SMART_SCHEDULE,
+    HVACMode.HEAT: CONST_MODE_HEAT,
+    HVACMode.COOL: CONST_MODE_COOL,
+    HVACMode.DRY: CONST_MODE_DRY,
+    HVACMode.FAN_ONLY: CONST_MODE_FAN,
 }
 
 HA_TO_TADO_FAN_MODE_MAP = {
@@ -136,19 +155,36 @@ TADO_TO_HA_FAN_MODE_MAP = {value: key for key, value in HA_TO_TADO_FAN_MODE_MAP.
 
 DEFAULT_TADO_PRECISION = 0.1
 
-SUPPORT_PRESET = [PRESET_AWAY, PRESET_HOME]
+# Constant for Auto Geolocation mode
+PRESET_AUTO = "auto"
 
+SUPPORT_PRESET_AUTO = [PRESET_AWAY, PRESET_HOME, PRESET_AUTO]
+SUPPORT_PRESET_MANUAL = [PRESET_AWAY, PRESET_HOME]
+
+SENSOR_DATA_CATEGORY_WEATHER = "weather"
+SENSOR_DATA_CATEGORY_GEOFENCE = "geofence"
 
 TADO_SWING_OFF = "OFF"
 TADO_SWING_ON = "ON"
 
+HA_TO_TADO_SWING_MODE_MAP = {
+    SWING_OFF: TADO_SWING_OFF,
+    SWING_ON: TADO_SWING_ON,
+}
+
+TADO_TO_HA_SWING_MODE_MAP = {
+    value: key for key, value in HA_TO_TADO_SWING_MODE_MAP.items()
+}
+
 DOMAIN = "tado"
 
 SIGNAL_TADO_UPDATE_RECEIVED = "tado_update_received_{}_{}_{}"
+SIGNAL_TADO_MOBILE_DEVICE_UPDATE_RECEIVED = "tado_mobile_device_update_received_{}"
 UNIQUE_ID = "unique_id"
 
 DEFAULT_NAME = "Tado"
 
+TADO_HOME = "Home"
 TADO_ZONE = "Zone"
 
 UPDATE_LISTENER = "update_listener"
@@ -164,3 +200,7 @@ TADO_TO_HA_OFFSET_MAP = {
     TADO_OFFSET_CELSIUS: HA_OFFSET_CELSIUS,
     TADO_OFFSET_FAHRENHEIT: HA_OFFSET_FAHRENHEIT,
 }
+
+# Constants for Overlay Default settings
+HA_TERMINATION_TYPE = "default_overlay_type"
+HA_TERMINATION_DURATION = "default_overlay_seconds"
