@@ -1,9 +1,20 @@
 """Support for Tasmota binary sensors."""
+from __future__ import annotations
+
+from collections.abc import Callable
+from datetime import datetime
+from typing import Any
+
+from hatasmota import switch as tasmota_switch
+from hatasmota.entity import TasmotaEntity as HATasmotaEntity
+from hatasmota.models import DiscoveryHashType
 
 from homeassistant.components import binary_sensor
 from homeassistant.components.binary_sensor import BinarySensorEntity
-from homeassistant.core import callback
+from homeassistant.config_entries import ConfigEntry
+from homeassistant.core import HomeAssistant, callback
 from homeassistant.helpers.dispatcher import async_dispatcher_connect
+from homeassistant.helpers.entity_platform import AddEntitiesCallback
 import homeassistant.helpers.event as evt
 
 from .const import DATA_REMOVE_DISCOVER_COMPONENT
@@ -11,11 +22,17 @@ from .discovery import TASMOTA_DISCOVERY_ENTITY_NEW
 from .mixins import TasmotaAvailability, TasmotaDiscoveryUpdate
 
 
-async def async_setup_entry(hass, config_entry, async_add_entities):
+async def async_setup_entry(
+    hass: HomeAssistant,
+    config_entry: ConfigEntry,
+    async_add_entities: AddEntitiesCallback,
+) -> None:
     """Set up Tasmota binary sensor dynamically through discovery."""
 
     @callback
-    def async_discover(tasmota_entity, discovery_hash):
+    def async_discover(
+        tasmota_entity: HATasmotaEntity, discovery_hash: DiscoveryHashType
+    ) -> None:
         """Discover and add a Tasmota binary sensor."""
         async_add_entities(
             [
@@ -41,33 +58,41 @@ class TasmotaBinarySensor(
 ):
     """Representation a Tasmota binary sensor."""
 
-    def __init__(self, **kwds):
-        """Initialize the Tasmota binary sensor."""
-        self._delay_listener = None
-        self._state = None
+    _delay_listener: Callable | None = None
+    _on_off_state: bool | None = None
+    _tasmota_entity: tasmota_switch.TasmotaSwitch
 
+    def __init__(self, **kwds: Any) -> None:
+        """Initialize the Tasmota binary sensor."""
         super().__init__(
             **kwds,
         )
+        if self._tasmota_entity.off_delay is not None:
+            self._attr_force_update = True
+
+    async def async_added_to_hass(self) -> None:
+        """Subscribe to MQTT events."""
+        self._tasmota_entity.set_on_state_callback(self.on_off_state_updated)
+        await super().async_added_to_hass()
 
     @callback
-    def off_delay_listener(self, now):
+    def off_delay_listener(self, now: datetime) -> None:
         """Switch device off after a delay."""
         self._delay_listener = None
-        self._state = False
+        self._on_off_state = False
         self.async_write_ha_state()
 
     @callback
-    def state_updated(self, state, **kwargs):
+    def on_off_state_updated(self, state: bool, **kwargs: Any) -> None:
         """Handle state updates."""
-        self._state = state
+        self._on_off_state = state
 
         if self._delay_listener is not None:
             self._delay_listener()
             self._delay_listener = None
 
         off_delay = self._tasmota_entity.off_delay
-        if self._state and off_delay is not None:
+        if self._on_off_state and off_delay is not None:
             self._delay_listener = evt.async_call_later(
                 self.hass, off_delay, self.off_delay_listener
             )
@@ -75,11 +100,6 @@ class TasmotaBinarySensor(
         self.async_write_ha_state()
 
     @property
-    def force_update(self):
-        """Force update."""
-        return True
-
-    @property
-    def is_on(self):
+    def is_on(self) -> bool | None:
         """Return true if the binary sensor is on."""
-        return self._state
+        return self._on_off_state
