@@ -2,21 +2,25 @@
 from __future__ import annotations
 
 from datetime import date, datetime
+import logging
 from typing import Any
 
 import voluptuous as vol
 
 from homeassistant.components.sensor import (
+    ATTR_LAST_RESET,
     CONF_STATE_CLASS,
     DEVICE_CLASSES_SCHEMA,
     DOMAIN as SENSOR_DOMAIN,
     ENTITY_ID_FORMAT,
     PLATFORM_SCHEMA,
-    STATE_CLASSES_SCHEMA,
+    RestoreSensor,
     SensorDeviceClass,
     SensorEntity,
+    SensorStateClass,
 )
 from homeassistant.components.sensor.helpers import async_parse_date_datetime
+from homeassistant.config_entries import ConfigEntry
 from homeassistant.const import (
     ATTR_ENTITY_ID,
     CONF_DEVICE_CLASS,
@@ -30,14 +34,19 @@ from homeassistant.const import (
     CONF_UNIQUE_ID,
     CONF_UNIT_OF_MEASUREMENT,
     CONF_VALUE_TEMPLATE,
+    STATE_UNAVAILABLE,
+    STATE_UNKNOWN,
 )
 from homeassistant.core import HomeAssistant, callback
 from homeassistant.exceptions import TemplateError
 from homeassistant.helpers import config_validation as cv, template
 from homeassistant.helpers.entity import async_generate_entity_id
 from homeassistant.helpers.entity_platform import AddEntitiesCallback
+from homeassistant.helpers.trigger_template_entity import TEMPLATE_SENSOR_BASE_SCHEMA
 from homeassistant.helpers.typing import ConfigType, DiscoveryInfoType
+from homeassistant.util import dt as dt_util
 
+from . import TriggerUpdateCoordinator
 from .const import (
     CONF_ATTRIBUTE_TEMPLATES,
     CONF_AVAILABILITY_TEMPLATE,
@@ -58,16 +67,30 @@ LEGACY_FIELDS = {
 }
 
 
-SENSOR_SCHEMA = vol.Schema(
-    {
-        vol.Optional(CONF_DEVICE_CLASS): DEVICE_CLASSES_SCHEMA,
-        vol.Optional(CONF_NAME): cv.template,
-        vol.Optional(CONF_STATE_CLASS): STATE_CLASSES_SCHEMA,
-        vol.Required(CONF_STATE): cv.template,
-        vol.Optional(CONF_UNIQUE_ID): cv.string,
-        vol.Optional(CONF_UNIT_OF_MEASUREMENT): cv.string,
-    }
-).extend(TEMPLATE_ENTITY_COMMON_SCHEMA.schema)
+def validate_last_reset(val):
+    """Run extra validation checks."""
+    if (
+        val.get(ATTR_LAST_RESET) is not None
+        and val.get(CONF_STATE_CLASS) != SensorStateClass.TOTAL
+    ):
+        raise vol.Invalid(
+            "last_reset is only valid for template sensors with state_class 'total'"
+        )
+
+    return val
+
+
+SENSOR_SCHEMA = vol.All(
+    vol.Schema(
+        {
+            vol.Required(CONF_STATE): cv.template,
+            vol.Optional(ATTR_LAST_RESET): cv.template,
+        }
+    )
+    .extend(TEMPLATE_SENSOR_BASE_SCHEMA.schema)
+    .extend(TEMPLATE_ENTITY_COMMON_SCHEMA.schema),
+    validate_last_reset,
+)
 
 
 LEGACY_SENSOR_SCHEMA = vol.All(
@@ -96,8 +119,9 @@ def extra_validation_checks(val):
     """Run extra validation checks."""
     if CONF_TRIGGER in val:
         raise vol.Invalid(
-            "You can only add triggers to template entities if they are defined under `template:`. "
-            "See the template documentation for more information: https://www.home-assistant.io/integrations/template/"
+            "You can only add triggers to template entities if they are defined under"
+            " `template:`. See the template documentation for more information:"
+            " https://www.home-assistant.io/integrations/template/"
         )
 
     if CONF_SENSORS not in val and SENSOR_DOMAIN not in val:
@@ -133,11 +157,16 @@ PLATFORM_SCHEMA = vol.All(
     extra_validation_checks,
 )
 
+_LOGGER = logging.getLogger(__name__)
+
 
 @callback
 def _async_create_template_tracking_entities(
-    async_add_entities, hass, definitions: list[dict], unique_id_prefix: str | None
-):
+    async_add_entities: AddEntitiesCallback,
+    hass: HomeAssistant,
+    definitions: list[dict],
+    unique_id_prefix: str | None,
+) -> None:
     """Create the template sensors."""
     sensors = []
 
@@ -189,9 +218,33 @@ async def async_setup_platform(
     )
 
 
+async def async_setup_entry(
+    hass: HomeAssistant,
+    config_entry: ConfigEntry,
+    async_add_entities: AddEntitiesCallback,
+) -> None:
+    """Initialize config entry."""
+    _options = dict(config_entry.options)
+    _options.pop("template_type")
+    validated_config = SENSOR_SCHEMA(_options)
+    async_add_entities([SensorTemplate(hass, validated_config, config_entry.entry_id)])
+
+
+@callback
+def async_create_preview_sensor(
+    hass: HomeAssistant, name: str, config: dict[str, Any]
+) -> SensorTemplate:
+    """Create a preview sensor."""
+    validated_config = SENSOR_SCHEMA(config | {CONF_NAME: name})
+    entity = SensorTemplate(hass, validated_config, None)
+    return entity
+
+
 class SensorTemplate(TemplateEntity, SensorEntity):
     """Representation of a Template Sensor."""
 
+    _attr_should_poll = False
+
     def __init__(
         self,
         hass: HomeAssistant,
@@ -199,24 +252,38 @@ class SensorTemplate(TemplateEntity, SensorEntity):
         unique_id: str | None,
     ) -> None:
         """Initialize the sensor."""
-        super().__init__(hass, config=config, unique_id=unique_id)
+        super().__init__(hass, config=config, fallback_name=None, unique_id=unique_id)
+        self._attr_native_unit_of_measurement = config.get(CONF_UNIT_OF_MEASUREMENT)
+        self._attr_device_class = config.get(CONF_DEVICE_CLASS)
+        self._attr_state_class = config.get(CONF_STATE_CLASS)
+        self._template: template.Template = config[CONF_STATE]
+        self._attr_last_reset_template: None | template.Template = config.get(
+            ATTR_LAST_RESET
+        )
         if (object_id := config.get(CONF_OBJECT_ID)) is not None:
             self.entity_id = async_generate_entity_id(
                 ENTITY_ID_FORMAT, object_id, hass=hass
             )
 
-        self._attr_native_unit_of_measurement = config.get(CONF_UNIT_OF_MEASUREMENT)
-        self._template = config.get(CONF_STATE)
-        self._attr_device_class = config.get(CONF_DEVICE_CLASS)
-        self._attr_state_class = config.get(CONF_STATE_CLASS)
-
-    async def async_added_to_hass(self):
-        """Register callbacks."""
+    @callback
+    def _async_setup_templates(self) -> None:
+        """Set up templates."""
         self.add_template_attribute(
             "_attr_native_value", self._template, None, self._update_state
         )
+        if self._attr_last_reset_template is not None:
+            self.add_template_attribute(
+                "_attr_last_reset",
+                self._attr_last_reset_template,
+                cv.datetime,
+                self._update_last_reset,
+            )
 
-        await super().async_added_to_hass()
+        super()._async_setup_templates()
+
+    @callback
+    def _update_last_reset(self, result):
+        self._attr_last_reset = result
 
     @callback
     def _update_state(self, result):
@@ -237,32 +304,66 @@ class SensorTemplate(TemplateEntity, SensorEntity):
         )
 
 
-class TriggerSensorEntity(TriggerEntity, SensorEntity):
+class TriggerSensorEntity(TriggerEntity, RestoreSensor):
     """Sensor entity based on trigger data."""
 
     domain = SENSOR_DOMAIN
     extra_template_keys = (CONF_STATE,)
 
+    def __init__(
+        self,
+        hass: HomeAssistant,
+        coordinator: TriggerUpdateCoordinator,
+        config: ConfigType,
+    ) -> None:
+        """Initialize."""
+        super().__init__(hass, coordinator, config)
+
+        if (last_reset_template := config.get(ATTR_LAST_RESET)) is not None:
+            if last_reset_template.is_static:
+                self._static_rendered[ATTR_LAST_RESET] = last_reset_template.template
+            else:
+                self._to_render_simple.append(ATTR_LAST_RESET)
+
+        self._attr_state_class = config.get(CONF_STATE_CLASS)
+        self._attr_native_unit_of_measurement = config.get(CONF_UNIT_OF_MEASUREMENT)
+
+    async def async_added_to_hass(self) -> None:
+        """Restore last state."""
+        await super().async_added_to_hass()
+        if (
+            (last_state := await self.async_get_last_state()) is not None
+            and (extra_data := await self.async_get_last_sensor_data()) is not None
+            and last_state.state not in (STATE_UNKNOWN, STATE_UNAVAILABLE)
+            # The trigger might have fired already while we waited for stored data,
+            # then we should not restore state
+            and CONF_STATE not in self._rendered
+        ):
+            self._rendered[CONF_STATE] = extra_data.native_value
+            self.restore_attributes(last_state)
+
     @property
     def native_value(self) -> str | datetime | date | None:
         """Return state of the sensor."""
         return self._rendered.get(CONF_STATE)
 
-    @property
-    def state_class(self) -> str | None:
-        """Sensor state class."""
-        return self._config.get(CONF_STATE_CLASS)
-
-    @property
-    def native_unit_of_measurement(self) -> str | None:
-        """Return the unit of measurement of the sensor, if any."""
-        return self._config.get(CONF_UNIT_OF_MEASUREMENT)
-
     @callback
     def _process_data(self) -> None:
         """Process new data."""
         super()._process_data()
 
+        # Update last_reset
+        if ATTR_LAST_RESET in self._rendered:
+            parsed_timestamp = dt_util.parse_datetime(self._rendered[ATTR_LAST_RESET])
+            if parsed_timestamp is None:
+                _LOGGER.warning(
+                    "%s rendered invalid timestamp for last_reset attribute: %s",
+                    self.entity_id,
+                    self._rendered.get(ATTR_LAST_RESET),
+                )
+            else:
+                self._attr_last_reset = parsed_timestamp
+
         if (
             state := self._rendered.get(CONF_STATE)
         ) is None or self.device_class not in (
