@@ -1,17 +1,21 @@
 """Support for THOMSON routers."""
+from __future__ import annotations
+
 import logging
 import re
-import telnetlib
+import telnetlib  # pylint: disable=deprecated-module
 
 import voluptuous as vol
 
 from homeassistant.components.device_tracker import (
     DOMAIN,
-    PLATFORM_SCHEMA,
+    PLATFORM_SCHEMA as PARENT_PLATFORM_SCHEMA,
     DeviceScanner,
 )
 from homeassistant.const import CONF_HOST, CONF_PASSWORD, CONF_USERNAME
+from homeassistant.core import HomeAssistant
 import homeassistant.helpers.config_validation as cv
+from homeassistant.helpers.typing import ConfigType
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -25,7 +29,7 @@ _DEVICES_REGEX = re.compile(
     r"(?P<host>([^\s]+))"
 )
 
-PLATFORM_SCHEMA = PLATFORM_SCHEMA.extend(
+PLATFORM_SCHEMA = PARENT_PLATFORM_SCHEMA.extend(
     {
         vol.Required(CONF_HOST): cv.string,
         vol.Required(CONF_PASSWORD): cv.string,
@@ -34,7 +38,7 @@ PLATFORM_SCHEMA = PLATFORM_SCHEMA.extend(
 )
 
 
-def get_scanner(hass, config):
+def get_scanner(hass: HomeAssistant, config: ConfigType) -> ThomsonDeviceScanner | None:
     """Validate the configuration and return a THOMSON scanner."""
     scanner = ThomsonDeviceScanner(config[DOMAIN])
 
@@ -42,7 +46,7 @@ def get_scanner(hass, config):
 
 
 class ThomsonDeviceScanner(DeviceScanner):
-    """This class queries a router running THOMSON firmware."""
+    """Class which queries a router running THOMSON firmware."""
 
     def __init__(self, config):
         """Initialize the scanner."""
@@ -78,8 +82,7 @@ class ThomsonDeviceScanner(DeviceScanner):
             return False
 
         _LOGGER.info("Checking ARP")
-        data = self.get_thomson_data()
-        if not data:
+        if not (data := self.get_thomson_data()):
             return False
 
         # Flag C stands for CONNECTED
@@ -110,8 +113,7 @@ class ThomsonDeviceScanner(DeviceScanner):
 
         devices = {}
         for device in devices_result:
-            match = _DEVICES_REGEX.search(device.decode("utf-8"))
-            if match:
+            if match := _DEVICES_REGEX.search(device.decode("utf-8")):
                 devices[match.group("ip")] = {
                     "ip": match.group("ip"),
                     "mac": match.group("mac").upper(),
