@@ -9,7 +9,7 @@ from tmb import IBus
 import voluptuous as vol
 
 from homeassistant.components.sensor import PLATFORM_SCHEMA, SensorEntity
-from homeassistant.const import ATTR_ATTRIBUTION, CONF_NAME, TIME_MINUTES
+from homeassistant.const import CONF_NAME, UnitOfTime
 from homeassistant.core import HomeAssistant
 import homeassistant.helpers.config_validation as cv
 from homeassistant.helpers.entity_platform import AddEntitiesCallback
@@ -18,10 +18,6 @@ from homeassistant.util import Throttle
 
 _LOGGER = logging.getLogger(__name__)
 
-ATTRIBUTION = "Data provided by Transport Metropolitans de Barcelona"
-
-ICON = "mdi:bus-clock"
-
 CONF_APP_ID = "app_id"
 CONF_APP_KEY = "app_key"
 CONF_LINE = "line"
@@ -75,13 +71,16 @@ def setup_platform(
 class TMBSensor(SensorEntity):
     """Implementation of a TMB line/stop Sensor."""
 
+    _attr_attribution = "Data provided by Transport Metropolitans de Barcelona"
+    _attr_icon = "mdi:bus-clock"
+
     def __init__(self, ibus_client, stop, line, name):
         """Initialize the sensor."""
         self._ibus_client = ibus_client
         self._stop = stop
         self._line = line.upper()
         self._name = name
-        self._unit = TIME_MINUTES
+        self._unit = UnitOfTime.MINUTES
         self._state = None
 
     @property
@@ -89,11 +88,6 @@ class TMBSensor(SensorEntity):
         """Return the name of the sensor."""
         return self._name
 
-    @property
-    def icon(self):
-        """Return the icon for the frontend."""
-        return ICON
-
     @property
     def native_unit_of_measurement(self):
         """Return the unit of measurement."""
@@ -113,7 +107,6 @@ class TMBSensor(SensorEntity):
     def extra_state_attributes(self):
         """Return the state attributes of the last update."""
         return {
-            ATTR_ATTRIBUTION: ATTRIBUTION,
             ATTR_BUS_STOP: self._stop,
             ATTR_LINE: self._line,
         }
@@ -125,5 +118,6 @@ class TMBSensor(SensorEntity):
             self._state = self._ibus_client.get_stop_forecast(self._stop, self._line)
         except HTTPError:
             _LOGGER.error(
-                "Unable to fetch data from TMB API. Please check your API keys are valid"
+                "Unable to fetch data from TMB API. Please check your API keys are"
+                " valid"
             )
