@@ -23,21 +23,27 @@ from homeassistant.helpers.aiohttp_client import async_get_clientsession
 from homeassistant.helpers.dispatcher import async_dispatcher_send
 
 from .const import (
-    ATTR_BUZZER,
+    ATTR_ACTIVITY_LABEL,
+    ATTR_CALORIES,
     ATTR_DAILY_GOAL,
-    ATTR_LED,
-    ATTR_LIVE_TRACKING,
     ATTR_MINUTES_ACTIVE,
+    ATTR_MINUTES_DAY_SLEEP,
+    ATTR_MINUTES_NIGHT_SLEEP,
+    ATTR_MINUTES_REST,
+    ATTR_SLEEP_LABEL,
     ATTR_TRACKER_STATE,
     CLIENT,
     CLIENT_ID,
     DOMAIN,
     RECONNECT_INTERVAL,
     SERVER_UNAVAILABLE,
+    SWITCH_KEY_MAP,
     TRACKABLES,
     TRACKER_ACTIVITY_STATUS_UPDATED,
     TRACKER_HARDWARE_STATUS_UPDATED,
     TRACKER_POSITION_UPDATED,
+    TRACKER_SWITCH_STATUS_UPDATED,
+    TRACKER_WELLNESS_STATUS_UPDATED,
 )
 
 PLATFORMS = [
@@ -84,7 +90,6 @@ async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
         raise ConfigEntryNotReady from error
 
     tractive = TractiveClient(hass, client, creds["user_id"], entry)
-    tractive.subscribe()
 
     try:
         trackable_objects = await client.trackable_objects()
@@ -92,7 +97,6 @@ async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
             *(_generate_trackables(client, item) for item in trackable_objects)
         )
     except aiotractive.exceptions.TractiveError as error:
-        await tractive.unsubscribe()
         raise ConfigEntryNotReady from error
 
     # When the pet defined in Tractive has no tracker linked we get None as `trackable`.
@@ -125,6 +129,13 @@ async def _generate_trackables(
     if not trackable["device_id"]:
         return None
 
+    if "details" not in trackable:
+        _LOGGER.info(
+            "Tracker %s has no details and will be skipped. This happens for shared trackers",
+            trackable["device_id"],
+        )
+        return None
+
     tracker = client.tracker(trackable["device_id"])
 
     tracker_details, hw_info, pos_report = await asyncio.gather(
@@ -168,6 +179,14 @@ class TractiveClient:
         """Return user id."""
         return self._user_id
 
+    @property
+    def subscribed(self) -> bool:
+        """Return True if subscribed."""
+        if self._listen_task is None:
+            return False
+
+        return not self._listen_task.cancelled()
+
     async def trackable_objects(
         self,
     ) -> list[aiotractive.trackable_object.TrackableObject]:
@@ -196,25 +215,32 @@ class TractiveClient:
         while True:
             try:
                 async for event in self._client.events():
+                    _LOGGER.debug("Received event: %s", event)
                     if server_was_unavailable:
                         _LOGGER.debug("Tractive is back online")
                         server_was_unavailable = False
                     if event["message"] == "activity_update":
                         self._send_activity_update(event)
                         continue
+                    if event["message"] == "wellness_overview":
+                        self._send_wellness_update(event)
+                        continue
                     if (
                         "hardware" in event
                         and self._last_hw_time != event["hardware"]["time"]
                     ):
                         self._last_hw_time = event["hardware"]["time"]
                         self._send_hardware_update(event)
-
                     if (
                         "position" in event
                         and self._last_pos_time != event["position"]["time"]
                     ):
                         self._last_pos_time = event["position"]["time"]
                         self._send_position_update(event)
+                    # If any key belonging to the switch is present in the event,
+                    # we send a switch status update
+                    if bool(set(SWITCH_KEY_MAP.values()).intersection(event)):
+                        self._send_switch_update(event)
             except aiotractive.exceptions.UnauthorizedError:
                 self._config_entry.async_start_reauth(self._hass)
                 await self.unsubscribe()
@@ -223,10 +249,15 @@ class TractiveClient:
                     self._config_entry.data[CONF_EMAIL],
                 )
                 return
-
+            except (KeyError, TypeError) as error:
+                _LOGGER.error("Error while listening for events: %s", error)
+                continue
             except aiotractive.exceptions.TractiveError:
                 _LOGGER.debug(
-                    "Tractive is not available. Internet connection is down? Sleeping %i seconds and retrying",
+                    (
+                        "Tractive is not available. Internet connection is down?"
+                        " Sleeping %i seconds and retrying"
+                    ),
                     RECONNECT_INTERVAL.total_seconds(),
                 )
                 self._last_hw_time = 0
@@ -244,14 +275,21 @@ class TractiveClient:
             ATTR_BATTERY_LEVEL: event["hardware"]["battery_level"],
             ATTR_TRACKER_STATE: event["tracker_state"].lower(),
             ATTR_BATTERY_CHARGING: event["charging_state"] == "CHARGING",
-            ATTR_LIVE_TRACKING: event.get("live_tracking", {}).get("active"),
-            ATTR_BUZZER: event.get("buzzer_control", {}).get("active"),
-            ATTR_LED: event.get("led_control", {}).get("active"),
         }
         self._dispatch_tracker_event(
             TRACKER_HARDWARE_STATUS_UPDATED, event["tracker_id"], payload
         )
 
+    def _send_switch_update(self, event: dict[str, Any]) -> None:
+        # Sometimes the event contains data for all switches, sometimes only for one.
+        payload = {}
+        for switch, key in SWITCH_KEY_MAP.items():
+            if switch_data := event.get(key):
+                payload[switch] = switch_data["active"]
+        self._dispatch_tracker_event(
+            TRACKER_SWITCH_STATUS_UPDATED, event["tracker_id"], payload
+        )
+
     def _send_activity_update(self, event: dict[str, Any]) -> None:
         payload = {
             ATTR_MINUTES_ACTIVE: event["progress"]["achieved_minutes"],
@@ -261,6 +299,24 @@ class TractiveClient:
             TRACKER_ACTIVITY_STATUS_UPDATED, event["pet_id"], payload
         )
 
+    def _send_wellness_update(self, event: dict[str, Any]) -> None:
+        sleep_day = None
+        sleep_night = None
+        if isinstance(event["sleep"], dict):
+            sleep_day = event["sleep"]["minutes_day_sleep"]
+            sleep_night = event["sleep"]["minutes_night_sleep"]
+        payload = {
+            ATTR_ACTIVITY_LABEL: event["wellness"].get("activity_label"),
+            ATTR_CALORIES: event["activity"]["calories"],
+            ATTR_MINUTES_DAY_SLEEP: sleep_day,
+            ATTR_MINUTES_NIGHT_SLEEP: sleep_night,
+            ATTR_MINUTES_REST: event["activity"]["minutes_rest"],
+            ATTR_SLEEP_LABEL: event["wellness"].get("sleep_label"),
+        }
+        self._dispatch_tracker_event(
+            TRACKER_WELLNESS_STATUS_UPDATED, event["pet_id"], payload
+        )
+
     def _send_position_update(self, event: dict[str, Any]) -> None:
         payload = {
             "latitude": event["position"]["latlong"][0],
