@@ -1,13 +1,13 @@
 """Support for Tuya select."""
 from __future__ import annotations
 
-from tuya_iot import TuyaDevice, TuyaDeviceManager
+from tuya_sharing import CustomerDevice, Manager
 
 from homeassistant.components.select import SelectEntity, SelectEntityDescription
 from homeassistant.config_entries import ConfigEntry
+from homeassistant.const import EntityCategory
 from homeassistant.core import HomeAssistant, callback
 from homeassistant.helpers.dispatcher import async_dispatcher_connect
-from homeassistant.helpers.entity import EntityCategory
 from homeassistant.helpers.entity_platform import AddEntitiesCallback
 
 from . import HomeAssistantTuyaData
@@ -23,7 +23,7 @@ SELECTS: dict[str, tuple[SelectEntityDescription, ...]] = {
     "dgnbj": (
         SelectEntityDescription(
             key=DPCode.ALARM_VOLUME,
-            name="Volume",
+            translation_key="volume",
             entity_category=EntityCategory.CONFIG,
         ),
     ),
@@ -32,23 +32,23 @@ SELECTS: dict[str, tuple[SelectEntityDescription, ...]] = {
     "kfj": (
         SelectEntityDescription(
             key=DPCode.CUP_NUMBER,
-            name="Cups",
+            translation_key="cups",
             icon="mdi:numeric",
         ),
         SelectEntityDescription(
             key=DPCode.CONCENTRATION_SET,
-            name="Concentration",
+            translation_key="concentration",
             icon="mdi:altimeter",
             entity_category=EntityCategory.CONFIG,
         ),
         SelectEntityDescription(
             key=DPCode.MATERIAL,
-            name="Material",
+            translation_key="material",
             entity_category=EntityCategory.CONFIG,
         ),
         SelectEntityDescription(
             key=DPCode.MODE,
-            name="Mode",
+            translation_key="mode",
             icon="mdi:coffee",
         ),
     ),
@@ -57,13 +57,11 @@ SELECTS: dict[str, tuple[SelectEntityDescription, ...]] = {
     "kg": (
         SelectEntityDescription(
             key=DPCode.RELAY_STATUS,
-            name="Power on behavior",
             entity_category=EntityCategory.CONFIG,
             translation_key="relay_status",
         ),
         SelectEntityDescription(
             key=DPCode.LIGHT_MODE,
-            name="Indicator light mode",
             entity_category=EntityCategory.CONFIG,
             translation_key="light_mode",
         ),
@@ -73,21 +71,31 @@ SELECTS: dict[str, tuple[SelectEntityDescription, ...]] = {
     "qn": (
         SelectEntityDescription(
             key=DPCode.LEVEL,
-            name="Temperature level",
+            translation_key="temperature_level",
             icon="mdi:thermometer-lines",
         ),
     ),
+    # Smart Water Timer
+    "sfkzq": (
+        # Irrigation will not be run within this set delay period
+        SelectEntityDescription(
+            key=DPCode.WEATHER_DELAY,
+            translation_key="weather_delay",
+            icon="mdi:weather-cloudy-clock",
+            entity_category=EntityCategory.CONFIG,
+        ),
+    ),
     # Siren Alarm
     # https://developer.tuya.com/en/docs/iot/categorysgbj?id=Kaiuz37tlpbnu
     "sgbj": (
         SelectEntityDescription(
             key=DPCode.ALARM_VOLUME,
-            name="Volume",
+            translation_key="volume",
             entity_category=EntityCategory.CONFIG,
         ),
         SelectEntityDescription(
             key=DPCode.BRIGHT_STATE,
-            name="Brightness",
+            translation_key="brightness",
             entity_category=EntityCategory.CONFIG,
         ),
     ),
@@ -96,41 +104,35 @@ SELECTS: dict[str, tuple[SelectEntityDescription, ...]] = {
     "sp": (
         SelectEntityDescription(
             key=DPCode.IPC_WORK_MODE,
-            name="IPC mode",
             entity_category=EntityCategory.CONFIG,
             translation_key="ipc_work_mode",
         ),
         SelectEntityDescription(
             key=DPCode.DECIBEL_SENSITIVITY,
-            name="Sound detection densitivity",
             icon="mdi:volume-vibrate",
             entity_category=EntityCategory.CONFIG,
             translation_key="decibel_sensitivity",
         ),
         SelectEntityDescription(
             key=DPCode.RECORD_MODE,
-            name="Record mode",
             icon="mdi:record-rec",
             entity_category=EntityCategory.CONFIG,
             translation_key="record_mode",
         ),
         SelectEntityDescription(
             key=DPCode.BASIC_NIGHTVISION,
-            name="Night vision",
             icon="mdi:theme-light-dark",
             entity_category=EntityCategory.CONFIG,
             translation_key="basic_nightvision",
         ),
         SelectEntityDescription(
             key=DPCode.BASIC_ANTI_FLICKER,
-            name="Anti-flicker",
             icon="mdi:image-outline",
             entity_category=EntityCategory.CONFIG,
             translation_key="basic_anti_flicker",
         ),
         SelectEntityDescription(
             key=DPCode.MOTION_SENSITIVITY,
-            name="Motion detection sensitivity",
             icon="mdi:motion-sensor",
             entity_category=EntityCategory.CONFIG,
             translation_key="motion_sensitivity",
@@ -141,13 +143,11 @@ SELECTS: dict[str, tuple[SelectEntityDescription, ...]] = {
     "tdq": (
         SelectEntityDescription(
             key=DPCode.RELAY_STATUS,
-            name="Power on behavior",
             entity_category=EntityCategory.CONFIG,
             translation_key="relay_status",
         ),
         SelectEntityDescription(
             key=DPCode.LIGHT_MODE,
-            name="Indicator light mode",
             entity_category=EntityCategory.CONFIG,
             translation_key="light_mode",
         ),
@@ -157,33 +157,28 @@ SELECTS: dict[str, tuple[SelectEntityDescription, ...]] = {
     "tgkg": (
         SelectEntityDescription(
             key=DPCode.RELAY_STATUS,
-            name="Power on behavior",
             entity_category=EntityCategory.CONFIG,
             translation_key="relay_status",
         ),
         SelectEntityDescription(
             key=DPCode.LIGHT_MODE,
-            name="Indicator light mode",
             entity_category=EntityCategory.CONFIG,
             translation_key="light_mode",
         ),
         SelectEntityDescription(
             key=DPCode.LED_TYPE_1,
-            name="Light source type",
             entity_category=EntityCategory.CONFIG,
             translation_key="led_type",
         ),
         SelectEntityDescription(
             key=DPCode.LED_TYPE_2,
-            name="Light 2 source type",
             entity_category=EntityCategory.CONFIG,
-            translation_key="led_type",
+            translation_key="led_type_2",
         ),
         SelectEntityDescription(
             key=DPCode.LED_TYPE_3,
-            name="Light 3 source type",
             entity_category=EntityCategory.CONFIG,
-            translation_key="led_type",
+            translation_key="led_type_3",
         ),
     ),
     # Dimmer
@@ -191,22 +186,19 @@ SELECTS: dict[str, tuple[SelectEntityDescription, ...]] = {
     "tgq": (
         SelectEntityDescription(
             key=DPCode.LED_TYPE_1,
-            name="Light source type",
             entity_category=EntityCategory.CONFIG,
             translation_key="led_type",
         ),
         SelectEntityDescription(
             key=DPCode.LED_TYPE_2,
-            name="Light 2 source type",
             entity_category=EntityCategory.CONFIG,
-            translation_key="led_type",
+            translation_key="led_type_2",
         ),
     ),
     # Fingerbot
     "szjqr": (
         SelectEntityDescription(
             key=DPCode.MODE,
-            name="Mode",
             entity_category=EntityCategory.CONFIG,
             translation_key="fingerbot_mode",
         ),
@@ -216,21 +208,18 @@ SELECTS: dict[str, tuple[SelectEntityDescription, ...]] = {
     "sd": (
         SelectEntityDescription(
             key=DPCode.CISTERN,
-            name="Water tank adjustment",
             entity_category=EntityCategory.CONFIG,
             icon="mdi:water-opacity",
             translation_key="vacuum_cistern",
         ),
         SelectEntityDescription(
             key=DPCode.COLLECTION_MODE,
-            name="Dust collection mode",
             entity_category=EntityCategory.CONFIG,
             icon="mdi:air-filter",
             translation_key="vacuum_collection",
         ),
         SelectEntityDescription(
             key=DPCode.MODE,
-            name="Mode",
             entity_category=EntityCategory.CONFIG,
             icon="mdi:layers-outline",
             translation_key="vacuum_mode",
@@ -241,28 +230,24 @@ SELECTS: dict[str, tuple[SelectEntityDescription, ...]] = {
     "fs": (
         SelectEntityDescription(
             key=DPCode.FAN_VERTICAL,
-            name="Vertical swing flap angle",
             entity_category=EntityCategory.CONFIG,
             icon="mdi:format-vertical-align-center",
-            translation_key="fan_angle",
+            translation_key="vertical_fan_angle",
         ),
         SelectEntityDescription(
             key=DPCode.FAN_HORIZONTAL,
-            name="Horizontal swing flap angle",
             entity_category=EntityCategory.CONFIG,
             icon="mdi:format-horizontal-align-center",
-            translation_key="fan_angle",
+            translation_key="horizontal_fan_angle",
         ),
         SelectEntityDescription(
             key=DPCode.COUNTDOWN,
-            name="Countdown",
             entity_category=EntityCategory.CONFIG,
             icon="mdi:timer-cog-outline",
             translation_key="countdown",
         ),
         SelectEntityDescription(
             key=DPCode.COUNTDOWN_SET,
-            name="Countdown",
             entity_category=EntityCategory.CONFIG,
             icon="mdi:timer-cog-outline",
             translation_key="countdown",
@@ -273,14 +258,12 @@ SELECTS: dict[str, tuple[SelectEntityDescription, ...]] = {
     "cl": (
         SelectEntityDescription(
             key=DPCode.CONTROL_BACK_MODE,
-            name="Motor mode",
             entity_category=EntityCategory.CONFIG,
             icon="mdi:swap-horizontal",
             translation_key="curtain_motor_mode",
         ),
         SelectEntityDescription(
             key=DPCode.MODE,
-            name="Mode",
             entity_category=EntityCategory.CONFIG,
             translation_key="curtain_mode",
         ),
@@ -290,35 +273,30 @@ SELECTS: dict[str, tuple[SelectEntityDescription, ...]] = {
     "jsq": (
         SelectEntityDescription(
             key=DPCode.SPRAY_MODE,
-            name="Spray mode",
             entity_category=EntityCategory.CONFIG,
             icon="mdi:spray",
             translation_key="humidifier_spray_mode",
         ),
         SelectEntityDescription(
             key=DPCode.LEVEL,
-            name="Spraying level",
             entity_category=EntityCategory.CONFIG,
             icon="mdi:spray",
             translation_key="humidifier_level",
         ),
         SelectEntityDescription(
             key=DPCode.MOODLIGHTING,
-            name="Moodlighting",
             entity_category=EntityCategory.CONFIG,
             icon="mdi:lightbulb-multiple",
             translation_key="humidifier_moodlighting",
         ),
         SelectEntityDescription(
             key=DPCode.COUNTDOWN,
-            name="Countdown",
             entity_category=EntityCategory.CONFIG,
             icon="mdi:timer-cog-outline",
             translation_key="countdown",
         ),
         SelectEntityDescription(
             key=DPCode.COUNTDOWN_SET,
-            name="Countdown",
             entity_category=EntityCategory.CONFIG,
             icon="mdi:timer-cog-outline",
             translation_key="countdown",
@@ -329,19 +307,33 @@ SELECTS: dict[str, tuple[SelectEntityDescription, ...]] = {
     "kj": (
         SelectEntityDescription(
             key=DPCode.COUNTDOWN,
-            name="Countdown",
             entity_category=EntityCategory.CONFIG,
             icon="mdi:timer-cog-outline",
             translation_key="countdown",
         ),
         SelectEntityDescription(
             key=DPCode.COUNTDOWN_SET,
-            name="Countdown",
             entity_category=EntityCategory.CONFIG,
             icon="mdi:timer-cog-outline",
             translation_key="countdown",
         ),
     ),
+    # Dehumidifier
+    # https://developer.tuya.com/en/docs/iot/categorycs?id=Kaiuz1vcz4dha
+    "cs": (
+        SelectEntityDescription(
+            key=DPCode.COUNTDOWN_SET,
+            entity_category=EntityCategory.CONFIG,
+            icon="mdi:timer-cog-outline",
+            translation_key="countdown",
+        ),
+        SelectEntityDescription(
+            key=DPCode.DEHUMIDITY_SET_ENUM,
+            translation_key="target_humidity",
+            entity_category=EntityCategory.CONFIG,
+            icon="mdi:water-percent",
+        ),
+    ),
 }
 
 # Socket (duplicate of `kg`)
@@ -364,19 +356,17 @@ async def async_setup_entry(
         """Discover and add a discovered Tuya select."""
         entities: list[TuyaSelectEntity] = []
         for device_id in device_ids:
-            device = hass_data.device_manager.device_map[device_id]
+            device = hass_data.manager.device_map[device_id]
             if descriptions := SELECTS.get(device.category):
                 for description in descriptions:
                     if description.key in device.status:
                         entities.append(
-                            TuyaSelectEntity(
-                                device, hass_data.device_manager, description
-                            )
+                            TuyaSelectEntity(device, hass_data.manager, description)
                         )
 
         async_add_entities(entities)
 
-    async_discover_device([*hass_data.device_manager.device_map])
+    async_discover_device([*hass_data.manager.device_map])
 
     entry.async_on_unload(
         async_dispatcher_connect(hass, TUYA_DISCOVERY_NEW, async_discover_device)
@@ -388,8 +378,8 @@ class TuyaSelectEntity(TuyaEntity, SelectEntity):
 
     def __init__(
         self,
-        device: TuyaDevice,
-        device_manager: TuyaDeviceManager,
+        device: CustomerDevice,
+        device_manager: Manager,
         description: SelectEntityDescription,
     ) -> None:
         """Init Tuya sensor."""
