@@ -5,13 +5,14 @@ import asyncio
 from datetime import datetime, timedelta
 import ssl
 from types import MappingProxyType
-from typing import Any
+from typing import Any, Literal
 
+import aiohttp
 from aiohttp import CookieJar
 import aiounifi
 from aiounifi.interfaces.api_handlers import ItemEvent
-from aiounifi.websocket import WebsocketState
-import async_timeout
+from aiounifi.models.configuration import Configuration
+from aiounifi.models.device import DeviceSetPoePortModeRequest
 
 from homeassistant.config_entries import ConfigEntry
 from homeassistant.const import (
@@ -28,22 +29,26 @@ from homeassistant.helpers import (
     device_registry as dr,
     entity_registry as er,
 )
-from homeassistant.helpers.device_registry import CONNECTION_NETWORK_MAC
+from homeassistant.helpers.device_registry import (
+    DeviceEntry,
+    DeviceEntryType,
+    DeviceInfo,
+)
 from homeassistant.helpers.dispatcher import (
     async_dispatcher_connect,
     async_dispatcher_send,
 )
 from homeassistant.helpers.entity_platform import AddEntitiesCallback
 from homeassistant.helpers.entity_registry import async_entries_for_config_entry
-from homeassistant.helpers.event import async_track_time_interval
+from homeassistant.helpers.event import async_call_later, async_track_time_interval
 import homeassistant.util.dt as dt_util
 
 from .const import (
     ATTR_MANUFACTURER,
-    BLOCK_SWITCH,
     CONF_ALLOW_BANDWIDTH_SENSORS,
     CONF_ALLOW_UPTIME_SENSORS,
     CONF_BLOCK_CLIENT,
+    CONF_CLIENT_SOURCE,
     CONF_DETECTION_TIME,
     CONF_DPI_RESTRICTIONS,
     CONF_IGNORE_WIRED_BUG,
@@ -70,6 +75,7 @@ from .errors import AuthenticationRequired, CannotConnect
 
 RETRY_TIMER = 15
 CHECK_HEARTBEAT_INTERVAL = timedelta(seconds=1)
+CHECK_WEBSOCKET_INTERVAL = timedelta(minutes=1)
 
 
 class UniFiController:
@@ -83,14 +89,14 @@ class UniFiController:
         self.config_entry = config_entry
         self.api = api
 
-        api.ws_state_callback = self.async_unifi_ws_state_callback
+        self.ws_task: asyncio.Task | None = None
+        self._cancel_websocket_check: CALLBACK_TYPE | None = None
 
         self.available = True
         self.wireless_clients = hass.data[UNIFI_WIRELESS_CLIENTS]
 
-        self.site_id: str = ""
-        self._site_name: str | None = None
-        self._site_role: str | None = None
+        self.site = config_entry.data[CONF_SITE_ID]
+        self.is_admin = False
 
         self._cancel_heartbeat_check: CALLBACK_TYPE | None = None
         self._heartbeat_time: dict[str, datetime] = {}
@@ -100,10 +106,16 @@ class UniFiController:
         self.entities: dict[str, str] = {}
         self.known_objects: set[tuple[str, str]] = set()
 
+        self.poe_command_queue: dict[str, dict[int, str]] = {}
+        self._cancel_poe_command: CALLBACK_TYPE | None = None
+
     def load_config_entry_options(self) -> None:
         """Store attributes to avoid property call overhead since they are called frequently."""
         options = self.config_entry.options
 
+        # Allow creating entities from clients.
+        self.option_supported_clients: list[str] = options.get(CONF_CLIENT_SOURCE, [])
+
         # Device tracker options
 
         # Config entry option to not track clients.
@@ -132,7 +144,7 @@ class UniFiController:
         # Client control options
 
         # Config entry option with list of clients to control network access.
-        self.option_block_clients = options.get(CONF_BLOCK_CLIENT, [])
+        self.option_block_clients: list[str] = options.get(CONF_BLOCK_CLIENT, [])
         # Config entry option to control DPI restriction groups.
         self.option_dpi_restrictions: bool = options.get(
             CONF_DPI_RESTRICTIONS, DEFAULT_DPI_RESTRICTIONS
@@ -155,29 +167,23 @@ class UniFiController:
         host: str = self.config_entry.data[CONF_HOST]
         return host
 
-    @property
-    def site(self) -> str:
-        """Return the site of this config entry."""
-        site_id: str = self.config_entry.data[CONF_SITE_ID]
-        return site_id
-
-    @property
-    def site_name(self) -> str | None:
-        """Return the nice name of site."""
-        return self._site_name
-
-    @property
-    def site_role(self) -> str | None:
-        """Return the site user role of this controller."""
-        return self._site_role
-
-    @property
-    def mac(self) -> str | None:
-        """Return the mac address of this controller."""
-        for client in self.api.clients.values():
-            if self.host == client.ip:
-                return client.mac
-        return None
+    @callback
+    @staticmethod
+    def register_platform(
+        hass: HomeAssistant,
+        config_entry: ConfigEntry,
+        async_add_entities: AddEntitiesCallback,
+        entity_class: type[UnifiEntity],
+        descriptions: tuple[UnifiEntityDescription, ...],
+        requires_admin: bool = False,
+    ) -> None:
+        """Register platform for UniFi entity management."""
+        controller: UniFiController = hass.data[UNIFI_DOMAIN][config_entry.entry_id]
+        if requires_admin and not controller.is_admin:
+            return
+        controller.register_platform_add_entities(
+            entity_class, descriptions, async_add_entities
+        )
 
     @callback
     def register_platform_add_entities(
@@ -229,23 +235,6 @@ class UniFiController:
         for description in descriptions:
             async_load_entities(description)
 
-    @callback
-    def async_unifi_ws_state_callback(self, state: WebsocketState) -> None:
-        """Handle messages back from UniFi library."""
-        if state == WebsocketState.DISCONNECTED and self.available:
-            LOGGER.warning("Lost connection to UniFi Network")
-
-        if (state == WebsocketState.RUNNING and not self.available) or (
-            state == WebsocketState.DISCONNECTED and self.available
-        ):
-            self.available = state == WebsocketState.RUNNING
-            async_dispatcher_send(self.hass, self.signal_reachable)
-
-            if not self.available:
-                self.hass.loop.call_later(RETRY_TIMER, self.reconnect, True)
-            else:
-                LOGGER.info("Connected to UniFi Network")
-
     @property
     def signal_reachable(self) -> str:
         """Integration specific event to signal a change in connection status."""
@@ -265,40 +254,21 @@ class UniFiController:
         """Set up a UniFi Network instance."""
         await self.api.initialize()
 
-        sites = await self.api.sites()
-        for site in sites.values():
-            if self.site == site["name"]:
-                self.site_id = site["_id"]
-                self._site_name = site["desc"]
-                break
+        assert self.config_entry.unique_id is not None
+        self.is_admin = self.api.sites[self.config_entry.unique_id].role == "admin"
 
-        description = await self.api.site_description()
-        self._site_role = description[0]["site_role"]
-
-        # Restore clients that are not a part of active clients list.
+        # Restore device tracker clients that are not a part of active clients list.
+        macs: list[str] = []
         entity_registry = er.async_get(self.hass)
         for entry in async_entries_for_config_entry(
             entity_registry, self.config_entry.entry_id
         ):
-            if entry.domain == Platform.DEVICE_TRACKER:
-                mac = entry.unique_id.split("-", 1)[0]
-            elif entry.domain == Platform.SWITCH and entry.unique_id.startswith(
-                BLOCK_SWITCH
-            ):
-                mac = entry.unique_id.split("-", 1)[1]
-            else:
-                continue
+            if entry.domain == Platform.DEVICE_TRACKER and "-" in entry.unique_id:
+                macs.append(entry.unique_id.split("-", 1)[1])
 
-            if mac in self.api.clients or mac not in self.api.clients_all:
-                continue
-
-            client = self.api.clients_all[mac]
-            self.api.clients.process_raw([dict(client.raw)])
-            LOGGER.debug(
-                "Restore disconnected client %s (%s)",
-                entry.entity_id,
-                client.mac,
-            )
+        for mac in self.option_supported_clients + self.option_block_clients + macs:
+            if mac not in self.api.clients and mac in self.api.clients_all:
+                self.api.clients.process_raw([dict(self.api.clients_all[mac].raw)])
 
         self.wireless_clients.update_clients(set(self.api.clients.values()))
 
@@ -307,6 +277,9 @@ class UniFiController:
         self._cancel_heartbeat_check = async_track_time_interval(
             self.hass, self._async_check_for_stale, CHECK_HEARTBEAT_INTERVAL
         )
+        self._cancel_websocket_check = async_track_time_interval(
+            self.hass, self._async_watch_websocket, CHECK_WEBSOCKET_INTERVAL
+        )
 
     @callback
     def async_heartbeat(
@@ -336,20 +309,55 @@ class UniFiController:
         for unique_id in unique_ids_to_remove:
             del self._heartbeat_time[unique_id]
 
-    async def async_update_device_registry(self) -> None:
+    @callback
+    def async_queue_poe_port_command(
+        self, device_id: str, port_idx: int, poe_mode: str
+    ) -> None:
+        """Queue commands to execute them together per device."""
+        if self._cancel_poe_command:
+            self._cancel_poe_command()
+            self._cancel_poe_command = None
+
+        device_queue = self.poe_command_queue.setdefault(device_id, {})
+        device_queue[port_idx] = poe_mode
+
+        async def async_execute_command(now: datetime) -> None:
+            """Execute previously queued commands."""
+            queue = self.poe_command_queue.copy()
+            self.poe_command_queue.clear()
+            for device_id, device_commands in queue.items():
+                device = self.api.devices[device_id]
+                commands = list(device_commands.items())
+                await self.api.request(
+                    DeviceSetPoePortModeRequest.create(device, targets=commands)
+                )
+
+        self._cancel_poe_command = async_call_later(self.hass, 5, async_execute_command)
+
+    @property
+    def device_info(self) -> DeviceInfo:
+        """UniFi controller device info."""
+        assert self.config_entry.unique_id is not None
+
+        version: str | None = None
+        if sysinfo := next(iter(self.api.system_information.values()), None):
+            version = sysinfo.version
+
+        return DeviceInfo(
+            entry_type=DeviceEntryType.SERVICE,
+            identifiers={(UNIFI_DOMAIN, self.config_entry.unique_id)},
+            manufacturer=ATTR_MANUFACTURER,
+            model="UniFi Network Application",
+            name="UniFi Network",
+            sw_version=version,
+        )
+
+    @callback
+    def async_update_device_registry(self) -> DeviceEntry:
         """Update device registry."""
-        if self.mac is None:
-            return
-
         device_registry = dr.async_get(self.hass)
-
-        device_registry.async_get_or_create(
-            config_entry_id=self.config_entry.entry_id,
-            configuration_url=self.api.url,
-            connections={(CONNECTION_NETWORK_MAC, self.mac)},
-            default_manufacturer=ATTR_MANUFACTURER,
-            default_model="UniFi Network",
-            default_name="UniFi Network",
+        return device_registry.async_get_or_create(
+            config_entry_id=self.config_entry.entry_id, **self.device_info
         )
 
     @staticmethod
@@ -366,6 +374,22 @@ class UniFiController:
         controller.load_config_entry_options()
         async_dispatcher_send(hass, controller.signal_options_update)
 
+    @callback
+    def start_websocket(self) -> None:
+        """Start up connection to websocket."""
+
+        async def _websocket_runner() -> None:
+            """Start websocket."""
+            try:
+                await self.api.start_websocket()
+            except (aiohttp.ClientConnectorError, aiounifi.WebsocketError):
+                LOGGER.error("Websocket disconnected")
+            self.available = False
+            async_dispatcher_send(self.hass, self.signal_reachable)
+            self.hass.loop.call_later(RETRY_TIMER, self.reconnect, True)
+
+        self.ws_task = self.hass.loop.create_task(_websocket_runner())
+
     @callback
     def reconnect(self, log: bool = False) -> None:
         """Prepare to reconnect UniFi session."""
@@ -376,9 +400,13 @@ class UniFiController:
     async def async_reconnect(self) -> None:
         """Try to reconnect UniFi Network session."""
         try:
-            async with async_timeout.timeout(5):
+            async with asyncio.timeout(5):
                 await self.api.login()
-                self.api.start_websocket()
+                self.start_websocket()
+
+            if not self.available:
+                self.available = True
+                async_dispatcher_send(self.hass, self.signal_reachable)
 
         except (
             asyncio.TimeoutError,
@@ -388,13 +416,22 @@ class UniFiController:
         ):
             self.hass.loop.call_later(RETRY_TIMER, self.reconnect)
 
+    @callback
+    def _async_watch_websocket(self, now: datetime) -> None:
+        """Watch timestamp for last received websocket message."""
+        LOGGER.debug(
+            "Last received websocket timestamp: %s",
+            self.api.connectivity.ws_message_received,
+        )
+
     @callback
     def shutdown(self, event: Event) -> None:
         """Wrap the call to unifi.close.
 
         Used as an argument to EventBus.async_listen_once.
         """
-        self.api.stop_websocket()
+        if self.ws_task is not None:
+            self.ws_task.cancel()
 
     async def async_reset(self) -> bool:
         """Reset this controller to default state.
@@ -402,7 +439,18 @@ class UniFiController:
         Will cancel any scheduled setup retry and will unload
         the config entry.
         """
-        self.api.stop_websocket()
+        if self.ws_task is not None:
+            self.ws_task.cancel()
+
+            _, pending = await asyncio.wait([self.ws_task], timeout=10)
+
+            if pending:
+                LOGGER.warning(
+                    "Unloading %s (%s) config entry. Task %s did not complete in time",
+                    self.config_entry.title,
+                    self.config_entry.domain,
+                    self.ws_task,
+                )
 
         unload_ok = await self.hass.config_entries.async_unload_platforms(
             self.config_entry, PLATFORMS
@@ -415,6 +463,14 @@ class UniFiController:
             self._cancel_heartbeat_check()
             self._cancel_heartbeat_check = None
 
+        if self._cancel_websocket_check:
+            self._cancel_websocket_check()
+            self._cancel_websocket_check = None
+
+        if self._cancel_poe_command:
+            self._cancel_poe_command()
+            self._cancel_poe_command = None
+
         return True
 
 
@@ -423,7 +479,7 @@ async def get_unifi_controller(
     config: MappingProxyType[str, Any],
 ) -> aiounifi.Controller:
     """Create a controller object and verify authentication."""
-    ssl_context: ssl.SSLContext | bool = False
+    ssl_context: ssl.SSLContext | Literal[False] = False
 
     if verify_ssl := config.get(CONF_VERIFY_SSL):
         session = aiohttp_client.async_get_clientsession(hass)
@@ -435,18 +491,19 @@ async def get_unifi_controller(
         )
 
     controller = aiounifi.Controller(
-        host=config[CONF_HOST],
-        username=config[CONF_USERNAME],
-        password=config[CONF_PASSWORD],
-        port=config[CONF_PORT],
-        site=config[CONF_SITE_ID],
-        websession=session,
-        ssl_context=ssl_context,
+        Configuration(
+            session,
+            host=config[CONF_HOST],
+            username=config[CONF_USERNAME],
+            password=config[CONF_PASSWORD],
+            port=config[CONF_PORT],
+            site=config[CONF_SITE_ID],
+            ssl_context=ssl_context,
+        )
     )
 
     try:
-        async with async_timeout.timeout(10):
-            await controller.check_unifi_os()
+        async with asyncio.timeout(10):
             await controller.login()
         return controller
 
@@ -461,6 +518,7 @@ async def get_unifi_controller(
     except (
         asyncio.TimeoutError,
         aiounifi.BadGateway,
+        aiounifi.Forbidden,
         aiounifi.ServiceUnavailable,
         aiounifi.RequestError,
         aiounifi.ResponseError,
