@@ -2,6 +2,7 @@
 from __future__ import annotations
 
 from dataclasses import dataclass
+from datetime import datetime
 
 from homeassistant.components.sensor import (
     SensorDeviceClass,
@@ -10,9 +11,13 @@ from homeassistant.components.sensor import (
     SensorStateClass,
 )
 from homeassistant.config_entries import ConfigEntry
-from homeassistant.const import UnitOfDataRate, UnitOfInformation, UnitOfTime
+from homeassistant.const import (
+    EntityCategory,
+    UnitOfDataRate,
+    UnitOfInformation,
+    UnitOfTime,
+)
 from homeassistant.core import HomeAssistant
-from homeassistant.helpers.entity import EntityCategory
 from homeassistant.helpers.entity_platform import AddEntitiesCallback
 
 from .const import (
@@ -36,7 +41,7 @@ from .coordinator import UpnpDataUpdateCoordinator
 from .entity import UpnpEntity, UpnpEntityDescription
 
 
-@dataclass
+@dataclass(frozen=True)
 class UpnpSensorEntityDescription(UpnpEntityDescription, SensorEntityDescription):
     """A class that describes a sensor UPnP entities."""
 
@@ -44,107 +49,107 @@ class UpnpSensorEntityDescription(UpnpEntityDescription, SensorEntityDescription
 SENSOR_DESCRIPTIONS: tuple[UpnpSensorEntityDescription, ...] = (
     UpnpSensorEntityDescription(
         key=BYTES_RECEIVED,
-        name=f"{UnitOfInformation.BYTES} received",
+        translation_key="data_received",
         icon="mdi:server-network",
         device_class=SensorDeviceClass.DATA_SIZE,
         native_unit_of_measurement=UnitOfInformation.BYTES,
-        format="d",
         entity_registry_enabled_default=False,
         state_class=SensorStateClass.TOTAL_INCREASING,
+        suggested_display_precision=0,
     ),
     UpnpSensorEntityDescription(
         key=BYTES_SENT,
-        name=f"{UnitOfInformation.BYTES} sent",
+        translation_key="data_sent",
         icon="mdi:server-network",
         device_class=SensorDeviceClass.DATA_SIZE,
         native_unit_of_measurement=UnitOfInformation.BYTES,
-        format="d",
         entity_registry_enabled_default=False,
         state_class=SensorStateClass.TOTAL_INCREASING,
+        suggested_display_precision=0,
     ),
     UpnpSensorEntityDescription(
         key=PACKETS_RECEIVED,
-        name=f"{DATA_PACKETS} received",
+        translation_key="packets_received",
         icon="mdi:server-network",
         native_unit_of_measurement=DATA_PACKETS,
-        format="d",
         entity_registry_enabled_default=False,
         state_class=SensorStateClass.TOTAL_INCREASING,
+        suggested_display_precision=0,
     ),
     UpnpSensorEntityDescription(
         key=PACKETS_SENT,
-        name=f"{DATA_PACKETS} sent",
+        translation_key="packets_sent",
         icon="mdi:server-network",
         native_unit_of_measurement=DATA_PACKETS,
-        format="d",
         entity_registry_enabled_default=False,
         state_class=SensorStateClass.TOTAL_INCREASING,
+        suggested_display_precision=0,
     ),
     UpnpSensorEntityDescription(
         key=ROUTER_IP,
-        name="External IP",
+        translation_key="external_ip",
         icon="mdi:server-network",
         entity_category=EntityCategory.DIAGNOSTIC,
     ),
     UpnpSensorEntityDescription(
         key=ROUTER_UPTIME,
-        name="Uptime",
+        translation_key="uptime",
         icon="mdi:server-network",
         native_unit_of_measurement=UnitOfTime.SECONDS,
         entity_registry_enabled_default=False,
-        format="d",
         entity_category=EntityCategory.DIAGNOSTIC,
+        suggested_display_precision=0,
     ),
     UpnpSensorEntityDescription(
         key=WAN_STATUS,
-        name="wan status",
+        translation_key="wan_status",
         icon="mdi:server-network",
         entity_category=EntityCategory.DIAGNOSTIC,
         entity_registry_enabled_default=False,
     ),
     UpnpSensorEntityDescription(
         key=BYTES_RECEIVED,
+        translation_key="download_speed",
         value_key=KIBIBYTES_PER_SEC_RECEIVED,
         unique_id="KiB/sec_received",
-        name=f"{UnitOfDataRate.KIBIBYTES_PER_SECOND} received",
         icon="mdi:server-network",
         device_class=SensorDeviceClass.DATA_RATE,
         native_unit_of_measurement=UnitOfDataRate.KIBIBYTES_PER_SECOND,
-        format=".1f",
         state_class=SensorStateClass.MEASUREMENT,
+        suggested_display_precision=1,
     ),
     UpnpSensorEntityDescription(
         key=BYTES_SENT,
+        translation_key="upload_speed",
         value_key=KIBIBYTES_PER_SEC_SENT,
         unique_id="KiB/sec_sent",
-        name=f"{UnitOfDataRate.KIBIBYTES_PER_SECOND} sent",
         icon="mdi:server-network",
         device_class=SensorDeviceClass.DATA_RATE,
         native_unit_of_measurement=UnitOfDataRate.KIBIBYTES_PER_SECOND,
-        format=".1f",
         state_class=SensorStateClass.MEASUREMENT,
+        suggested_display_precision=1,
     ),
     UpnpSensorEntityDescription(
         key=PACKETS_RECEIVED,
+        translation_key="packet_download_speed",
         value_key=PACKETS_PER_SEC_RECEIVED,
         unique_id="packets/sec_received",
-        name=f"{DATA_RATE_PACKETS_PER_SECOND} received",
         icon="mdi:server-network",
         native_unit_of_measurement=DATA_RATE_PACKETS_PER_SECOND,
-        format=".1f",
         entity_registry_enabled_default=False,
         state_class=SensorStateClass.MEASUREMENT,
+        suggested_display_precision=1,
     ),
     UpnpSensorEntityDescription(
         key=PACKETS_SENT,
+        translation_key="packet_upload_speed",
         value_key=PACKETS_PER_SEC_SENT,
         unique_id="packets/sec_sent",
-        name=f"{DATA_RATE_PACKETS_PER_SECOND} sent",
         icon="mdi:server-network",
         native_unit_of_measurement=DATA_RATE_PACKETS_PER_SECOND,
-        format=".1f",
         entity_registry_enabled_default=False,
         state_class=SensorStateClass.MEASUREMENT,
+        suggested_display_precision=1,
     ),
 )
 
@@ -176,9 +181,8 @@ class UpnpSensor(UpnpEntity, SensorEntity):
     entity_description: UpnpSensorEntityDescription
 
     @property
-    def native_value(self) -> str | None:
+    def native_value(self) -> str | datetime | int | float | None:
         """Return the state of the device."""
-        value = self.coordinator.data[self.entity_description.value_key]
-        if value is None:
+        if (key := self.entity_description.value_key) is None:
             return None
-        return format(value, self.entity_description.format)
+        return self.coordinator.data[key]
