@@ -9,30 +9,29 @@ from homeassistant.components.binary_sensor import (
 from homeassistant.config_entries import ConfigEntry
 from homeassistant.core import HomeAssistant
 from homeassistant.helpers.entity_platform import AddEntitiesCallback
-from homeassistant.helpers.update_coordinator import DataUpdateCoordinator
 
 from .const import DOMAIN
+from .coordinator import UptimeRobotDataUpdateCoordinator
 from .entity import UptimeRobotEntity
 
 
 async def async_setup_entry(
-    hass: HomeAssistant, entry: ConfigEntry, async_add_entities: AddEntitiesCallback
+    hass: HomeAssistant,
+    entry: ConfigEntry,
+    async_add_entities: AddEntitiesCallback,
 ) -> None:
     """Set up the UptimeRobot binary_sensors."""
-    coordinator: DataUpdateCoordinator = hass.data[DOMAIN][entry.entry_id]
+    coordinator: UptimeRobotDataUpdateCoordinator = hass.data[DOMAIN][entry.entry_id]
     async_add_entities(
-        [
-            UptimeRobotBinarySensor(
-                coordinator,
-                BinarySensorEntityDescription(
-                    key=str(monitor.id),
-                    name=monitor.friendly_name,
-                    device_class=BinarySensorDeviceClass.CONNECTIVITY,
-                ),
-                monitor=monitor,
-            )
-            for monitor in coordinator.data
-        ],
+        UptimeRobotBinarySensor(
+            coordinator,
+            BinarySensorEntityDescription(
+                key=str(monitor.id),
+                device_class=BinarySensorDeviceClass.CONNECTIVITY,
+            ),
+            monitor=monitor,
+        )
+        for monitor in coordinator.data
     )
 
 
