@@ -10,7 +10,11 @@ from homeassistant.components.sensor import DOMAIN as SENSOR_DOMAIN
 from homeassistant.config_entries import ConfigEntry
 from homeassistant.const import ATTR_ENTITY_ID, CONF_NAME, CONF_UNIQUE_ID, Platform
 from homeassistant.core import HomeAssistant, split_entity_id
-from homeassistant.helpers import discovery, entity_registry as er
+from homeassistant.helpers import (
+    device_registry as dr,
+    discovery,
+    entity_registry as er,
+)
 import homeassistant.helpers.config_validation as cv
 from homeassistant.helpers.dispatcher import async_dispatcher_send
 from homeassistant.helpers.typing import ConfigType
@@ -21,7 +25,9 @@ from .const import (
     CONF_METER_DELTA_VALUES,
     CONF_METER_NET_CONSUMPTION,
     CONF_METER_OFFSET,
+    CONF_METER_PERIODICALLY_RESETTING,
     CONF_METER_TYPE,
+    CONF_SENSOR_ALWAYS_AVAILABLE,
     CONF_SOURCE_SENSOR,
     CONF_TARIFF,
     CONF_TARIFF_ENTITY,
@@ -62,10 +68,10 @@ def period_or_cron(config):
 
 
 def max_28_days(config):
-    """Check that time period does not include more then 28 days."""
+    """Check that time period does not include more than 28 days."""
     if config.days >= 28:
         raise vol.Invalid(
-            "Unsupported offset of more then 28 days, please use a cron pattern."
+            "Unsupported offset of more than 28 days, please use a cron pattern."
         )
 
     return config
@@ -83,10 +89,12 @@ METER_CONFIG_SCHEMA = vol.Schema(
             ),
             vol.Optional(CONF_METER_DELTA_VALUES, default=False): cv.boolean,
             vol.Optional(CONF_METER_NET_CONSUMPTION, default=False): cv.boolean,
+            vol.Optional(CONF_METER_PERIODICALLY_RESETTING, default=True): cv.boolean,
             vol.Optional(CONF_TARIFFS, default=[]): vol.All(
                 cv.ensure_list, vol.Unique(), [cv.string]
             ),
             vol.Optional(CONF_CRON_PATTERN): validate_cron_pattern,
+            vol.Optional(CONF_SENSOR_ALWAYS_AVAILABLE, default=False): cv.boolean,
         },
         period_or_cron,
     )
@@ -180,7 +188,9 @@ async def async_setup(hass: HomeAssistant, config: ConfigType) -> bool:
 async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
     """Set up Utility Meter from a config entry."""
     entity_registry = er.async_get(hass)
-    hass.data[DATA_UTILITY][entry.entry_id] = {}
+    hass.data[DATA_UTILITY][entry.entry_id] = {
+        "source": entry.options[CONF_SOURCE_SENSOR],
+    }
     hass.data[DATA_UTILITY][entry.entry_id][DATA_TARIFF_SENSORS] = []
 
     try:
@@ -216,18 +226,49 @@ async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
 
 async def config_entry_update_listener(hass: HomeAssistant, entry: ConfigEntry) -> None:
     """Update listener, called when the config entry options are changed."""
+    old_source = hass.data[DATA_UTILITY][entry.entry_id]["source"]
     await hass.config_entries.async_reload(entry.entry_id)
 
+    if old_source == entry.options[CONF_SOURCE_SENSOR]:
+        return
+
+    entity_registry = er.async_get(hass)
+    device_registry = dr.async_get(hass)
+
+    old_source_entity = entity_registry.async_get(old_source)
+    if not old_source_entity or not old_source_entity.device_id:
+        return
+
+    device_registry.async_update_device(
+        old_source_entity.device_id, remove_config_entry_id=entry.entry_id
+    )
+
 
 async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
     """Unload a config entry."""
+    platforms_to_unload = [Platform.SENSOR]
+    if entry.options.get(CONF_TARIFFS):
+        platforms_to_unload.append(Platform.SELECT)
+
     if unload_ok := await hass.config_entries.async_unload_platforms(
         entry,
-        (
-            Platform.SELECT,
-            Platform.SENSOR,
-        ),
+        platforms_to_unload,
     ):
         hass.data[DATA_UTILITY].pop(entry.entry_id)
 
     return unload_ok
+
+
+async def async_migrate_entry(hass: HomeAssistant, config_entry: ConfigEntry) -> bool:
+    """Migrate old entry."""
+    _LOGGER.debug("Migrating from version %s", config_entry.version)
+
+    if config_entry.version == 1:
+        new = {**config_entry.options}
+        new[CONF_METER_PERIODICALLY_RESETTING] = True
+        config_entry.version = 2
+        hass.config_entries.async_update_entry(config_entry, options=new)
+
+    _LOGGER.info("Migration to version %s successful", config_entry.version)
+
+    return True
