@@ -6,10 +6,13 @@ from homeassistant.components.binary_sensor import (
     BinarySensorEntity,
 )
 from homeassistant.config_entries import ConfigEntry
+from homeassistant.const import ATTR_LAST_TRIP_TIME, EntityCategory
 from homeassistant.core import HomeAssistant
-from homeassistant.helpers.entity import DeviceInfo, Entity, EntityCategory
+from homeassistant.helpers.device_registry import DeviceInfo
+from homeassistant.helpers.entity import Entity
 from homeassistant.helpers.entity_platform import AddEntitiesCallback
 from homeassistant.helpers.update_coordinator import CoordinatorEntity
+from homeassistant.util import dt as dt_util
 
 from .const import CONF_GIID, DOMAIN
 from .coordinator import VerisureDataUpdateCoordinator
@@ -55,7 +58,6 @@ class VerisureDoorWindowSensor(
         area = self.coordinator.data["door_window"][self.serial_number]["area"]
         return DeviceInfo(
             name=area,
-            suggested_area=area,
             manufacturer="Verisure",
             model="Shock Sensor Detector",
             identifiers={(DOMAIN, self.serial_number)},
@@ -78,6 +80,15 @@ class VerisureDoorWindowSensor(
             and self.serial_number in self.coordinator.data["door_window"]
         )
 
+    @property
+    def extra_state_attributes(self):
+        """Return the state attributes of the sensor."""
+        return {
+            ATTR_LAST_TRIP_TIME: dt_util.parse_datetime(
+                self.coordinator.data["door_window"][self.serial_number]["reportTime"]
+            )
+        }
+
 
 class VerisureEthernetStatus(
     CoordinatorEntity[VerisureDataUpdateCoordinator], BinarySensorEntity
@@ -87,7 +98,7 @@ class VerisureEthernetStatus(
     _attr_device_class = BinarySensorDeviceClass.CONNECTIVITY
     _attr_entity_category = EntityCategory.DIAGNOSTIC
     _attr_has_entity_name = True
-    _attr_name = "Ethernet status"
+    _attr_translation_key = "ethernet"
 
     @property
     def unique_id(self) -> str:
@@ -108,9 +119,9 @@ class VerisureEthernetStatus(
     @property
     def is_on(self) -> bool:
         """Return the state of the sensor."""
-        return self.coordinator.data["ethernet"]
+        return self.coordinator.data["broadband"]["isBroadbandConnected"]
 
     @property
     def available(self) -> bool:
         """Return True if entity is available."""
-        return super().available and self.coordinator.data["ethernet"] is not None
+        return super().available and self.coordinator.data["broadband"] is not None
