@@ -2,6 +2,7 @@
 from __future__ import annotations
 
 import asyncio
+from typing import Any
 
 from verisure import Error as VerisureError
 
@@ -9,7 +10,7 @@ from homeassistant.components.lock import LockEntity
 from homeassistant.config_entries import ConfigEntry
 from homeassistant.const import ATTR_CODE, STATE_LOCKED, STATE_UNLOCKED
 from homeassistant.core import HomeAssistant
-from homeassistant.helpers.entity import DeviceInfo
+from homeassistant.helpers.device_registry import DeviceInfo
 from homeassistant.helpers.entity_platform import (
     AddEntitiesCallback,
     async_get_current_platform,
@@ -19,7 +20,6 @@ from homeassistant.helpers.update_coordinator import CoordinatorEntity
 from .const import (
     CONF_GIID,
     CONF_LOCK_CODE_DIGITS,
-    CONF_LOCK_DEFAULT_CODE,
     DEFAULT_LOCK_CODE_DIGITS,
     DOMAIN,
     LOGGER,
@@ -55,33 +55,28 @@ async def async_setup_entry(
     )
 
 
-class VerisureDoorlock(CoordinatorEntity, LockEntity):
+class VerisureDoorlock(CoordinatorEntity[VerisureDataUpdateCoordinator], LockEntity):
     """Representation of a Verisure doorlock."""
 
-    coordinator: VerisureDataUpdateCoordinator
+    _attr_has_entity_name = True
+    _attr_name = None
 
     def __init__(
         self, coordinator: VerisureDataUpdateCoordinator, serial_number: str
     ) -> None:
         """Initialize the Verisure lock."""
         super().__init__(coordinator)
-
-        self._attr_name = coordinator.data["locks"][serial_number]["area"]
         self._attr_unique_id = serial_number
 
         self.serial_number = serial_number
         self._state: str | None = None
-        self._digits = coordinator.entry.options.get(
-            CONF_LOCK_CODE_DIGITS, DEFAULT_LOCK_CODE_DIGITS
-        )
 
     @property
     def device_info(self) -> DeviceInfo:
         """Return device information about this entity."""
-        area = self.coordinator.data["locks"][self.serial_number]["area"]
+        area = self.coordinator.data["locks"][self.serial_number]["device"]["area"]
         return DeviceInfo(
             name=area,
-            suggested_area=area,
             manufacturer="Verisure",
             model="Lockguard Smartlock",
             identifiers={(DOMAIN, self.serial_number)},
@@ -99,75 +94,93 @@ class VerisureDoorlock(CoordinatorEntity, LockEntity):
     @property
     def changed_by(self) -> str | None:
         """Last change triggered by."""
-        return self.coordinator.data["locks"][self.serial_number].get("userString")
+        return (
+            self.coordinator.data["locks"][self.serial_number]
+            .get("user", {})
+            .get("name")
+        )
+
+    @property
+    def changed_method(self) -> str:
+        """Last change method."""
+        return self.coordinator.data["locks"][self.serial_number]["lockMethod"]
 
     @property
     def code_format(self) -> str:
-        """Return the required six digit code."""
-        return "^\\d{%s}$" % self._digits
+        """Return the configured code format."""
+        digits = self.coordinator.entry.options.get(
+            CONF_LOCK_CODE_DIGITS, DEFAULT_LOCK_CODE_DIGITS
+        )
+        return "^\\d{%s}$" % digits
 
     @property
     def is_locked(self) -> bool:
         """Return true if lock is locked."""
         return (
-            self.coordinator.data["locks"][self.serial_number]["lockedState"]
-            == "LOCKED"
+            self.coordinator.data["locks"][self.serial_number]["lockStatus"] == "LOCKED"
         )
 
-    async def async_unlock(self, **kwargs) -> None:
+    @property
+    def extra_state_attributes(self) -> dict[str, str]:
+        """Return the state attributes."""
+        return {"method": self.changed_method}
+
+    async def async_unlock(self, **kwargs: Any) -> None:
         """Send unlock command."""
-        code = kwargs.get(
-            ATTR_CODE, self.coordinator.entry.options.get(CONF_LOCK_DEFAULT_CODE)
-        )
-        if code is None:
-            LOGGER.error("Code required but none provided")
-            return
+        code = kwargs.get(ATTR_CODE)
+        if code:
+            await self.async_set_lock_state(code, STATE_UNLOCKED)
 
-        await self.async_set_lock_state(code, STATE_UNLOCKED)
-
-    async def async_lock(self, **kwargs) -> None:
+    async def async_lock(self, **kwargs: Any) -> None:
         """Send lock command."""
-        code = kwargs.get(
-            ATTR_CODE, self.coordinator.entry.options.get(CONF_LOCK_DEFAULT_CODE)
-        )
-        if code is None:
-            LOGGER.error("Code required but none provided")
-            return
-
-        await self.async_set_lock_state(code, STATE_LOCKED)
+        code = kwargs.get(ATTR_CODE)
+        if code:
+            await self.async_set_lock_state(code, STATE_LOCKED)
 
     async def async_set_lock_state(self, code: str, state: str) -> None:
         """Send set lock state command."""
-        target_state = "lock" if state == STATE_LOCKED else "unlock"
-        lock_state = await self.hass.async_add_executor_job(
-            self.coordinator.verisure.set_lock_state,
-            code,
-            self.serial_number,
-            target_state,
+        command = (
+            self.coordinator.verisure.door_lock(self.serial_number, code)
+            if state == STATE_LOCKED
+            else self.coordinator.verisure.door_unlock(self.serial_number, code)
+        )
+        lock_request = await self.hass.async_add_executor_job(
+            self.coordinator.verisure.request,
+            command,
         )
-
         LOGGER.debug("Verisure doorlock %s", state)
-        transaction = {}
+        transaction_id = lock_request.get("data", {}).get(command["operationName"])
+        target_state = "LOCKED" if state == STATE_LOCKED else "UNLOCKED"
+        lock_status = None
         attempts = 0
-        while "result" not in transaction:
-            transaction = await self.hass.async_add_executor_job(
-                self.coordinator.verisure.get_lock_state_transaction,
-                lock_state["doorLockStateChangeTransactionId"],
-            )
-            attempts += 1
+        while lock_status != "OK":
             if attempts == 30:
                 break
             if attempts > 1:
                 await asyncio.sleep(0.5)
-        if transaction["result"] == "OK":
+            attempts += 1
+            poll_data = await self.hass.async_add_executor_job(
+                self.coordinator.verisure.request,
+                self.coordinator.verisure.poll_lock_state(
+                    transaction_id, self.serial_number, target_state
+                ),
+            )
+            lock_status = (
+                poll_data.get("data", {})
+                .get("installation", {})
+                .get("doorLockStateChangePollResult", {})
+                .get("result")
+            )
+        if lock_status == "OK":
             self._state = state
 
     def disable_autolock(self) -> None:
         """Disable autolock on a doorlock."""
         try:
-            self.coordinator.verisure.set_lock_config(
+            command = self.coordinator.verisure.set_autolock_enabled(
                 self.serial_number, auto_lock_enabled=False
             )
+            self.coordinator.verisure.request(command)
             LOGGER.debug("Disabling autolock on %s", self.serial_number)
         except VerisureError as ex:
             LOGGER.error("Could not disable autolock, %s", ex)
@@ -175,9 +188,10 @@ class VerisureDoorlock(CoordinatorEntity, LockEntity):
     def enable_autolock(self) -> None:
         """Enable autolock on a doorlock."""
         try:
-            self.coordinator.verisure.set_lock_config(
+            command = self.coordinator.verisure.set_autolock_enabled(
                 self.serial_number, auto_lock_enabled=True
             )
+            self.coordinator.verisure.request(command)
             LOGGER.debug("Enabling autolock on %s", self.serial_number)
         except VerisureError as ex:
             LOGGER.error("Could not enable autolock, %s", ex)
