@@ -7,7 +7,7 @@ from typing import Any
 from homeassistant.components.switch import SwitchEntity
 from homeassistant.config_entries import ConfigEntry
 from homeassistant.core import HomeAssistant
-from homeassistant.helpers.entity import DeviceInfo
+from homeassistant.helpers.device_registry import DeviceInfo
 from homeassistant.helpers.entity_platform import AddEntitiesCallback
 from homeassistant.helpers.update_coordinator import CoordinatorEntity
 
@@ -32,6 +32,7 @@ class VerisureSmartplug(CoordinatorEntity[VerisureDataUpdateCoordinator], Switch
     """Representation of a Verisure smartplug."""
 
     _attr_has_entity_name = True
+    _attr_name = None
 
     def __init__(
         self, coordinator: VerisureDataUpdateCoordinator, serial_number: str
@@ -47,10 +48,11 @@ class VerisureSmartplug(CoordinatorEntity[VerisureDataUpdateCoordinator], Switch
     @property
     def device_info(self) -> DeviceInfo:
         """Return device information about this entity."""
-        area = self.coordinator.data["smart_plugs"][self.serial_number]["area"]
+        area = self.coordinator.data["smart_plugs"][self.serial_number]["device"][
+            "area"
+        ]
         return DeviceInfo(
             name=area,
-            suggested_area=area,
             manufacturer="Verisure",
             model="SmartPlug",
             identifiers={(DOMAIN, self.serial_number)},
@@ -77,16 +79,23 @@ class VerisureSmartplug(CoordinatorEntity[VerisureDataUpdateCoordinator], Switch
             and self.serial_number in self.coordinator.data["smart_plugs"]
         )
 
-    def turn_on(self, **kwargs: Any) -> None:
-        """Set smartplug status on."""
-        self.coordinator.verisure.set_smartplug_state(self.serial_number, True)
-        self._state = True
-        self._change_timestamp = monotonic()
-        self.schedule_update_ha_state()
+    async def async_turn_on(self, **kwargs: Any) -> None:
+        """Turn the smartplug on."""
+        await self.async_set_plug_state(True)
 
-    def turn_off(self, **kwargs: Any) -> None:
-        """Set smartplug status off."""
-        self.coordinator.verisure.set_smartplug_state(self.serial_number, False)
-        self._state = False
+    async def async_turn_off(self, **kwargs: Any) -> None:
+        """Turn the smartplug off."""
+        await self.async_set_plug_state(False)
+
+    async def async_set_plug_state(self, state: bool) -> None:
+        """Set smartplug state."""
+        command: dict[
+            str, str | dict[str, str]
+        ] = self.coordinator.verisure.set_smartplug(self.serial_number, state)
+        await self.hass.async_add_executor_job(
+            self.coordinator.verisure.request,
+            command,
+        )
+        self._state = state
         self._change_timestamp = monotonic()
-        self.schedule_update_ha_state()
+        await self.coordinator.async_request_refresh()
