@@ -1,15 +1,14 @@
 """Sensor platform support for wiffi devices."""
-
 from homeassistant.components.sensor import (
-    DEVICE_CLASS_HUMIDITY,
-    DEVICE_CLASS_ILLUMINANCE,
-    DEVICE_CLASS_PRESSURE,
-    DEVICE_CLASS_TEMPERATURE,
+    SensorDeviceClass,
     SensorEntity,
+    SensorStateClass,
 )
-from homeassistant.const import DEGREE, PRESSURE_MBAR, TEMP_CELSIUS
-from homeassistant.core import callback
+from homeassistant.config_entries import ConfigEntry
+from homeassistant.const import DEGREE, LIGHT_LUX, UnitOfPressure, UnitOfTemperature
+from homeassistant.core import HomeAssistant, callback
 from homeassistant.helpers.dispatcher import async_dispatcher_connect
+from homeassistant.helpers.entity_platform import AddEntitiesCallback
 
 from . import WiffiEntity
 from .const import CREATE_ENTITY_SIGNAL
@@ -23,21 +22,26 @@ from .wiffi_strings import (
 
 # map to determine HA device class from wiffi's unit of measurement
 UOM_TO_DEVICE_CLASS_MAP = {
-    WIFFI_UOM_TEMP_CELSIUS: DEVICE_CLASS_TEMPERATURE,
-    WIFFI_UOM_PERCENT: DEVICE_CLASS_HUMIDITY,
-    WIFFI_UOM_MILLI_BAR: DEVICE_CLASS_PRESSURE,
-    WIFFI_UOM_LUX: DEVICE_CLASS_ILLUMINANCE,
+    WIFFI_UOM_TEMP_CELSIUS: SensorDeviceClass.TEMPERATURE,
+    WIFFI_UOM_PERCENT: SensorDeviceClass.HUMIDITY,
+    WIFFI_UOM_MILLI_BAR: SensorDeviceClass.PRESSURE,
+    WIFFI_UOM_LUX: SensorDeviceClass.ILLUMINANCE,
 }
 
 # map to convert wiffi unit of measurements to common HA uom's
 UOM_MAP = {
     WIFFI_UOM_DEGREE: DEGREE,
-    WIFFI_UOM_TEMP_CELSIUS: TEMP_CELSIUS,
-    WIFFI_UOM_MILLI_BAR: PRESSURE_MBAR,
+    WIFFI_UOM_TEMP_CELSIUS: UnitOfTemperature.CELSIUS,
+    WIFFI_UOM_MILLI_BAR: UnitOfPressure.MBAR,
+    WIFFI_UOM_LUX: LIGHT_LUX,
 }
 
 
-async def async_setup_entry(hass, config_entry, async_add_entities):
+async def async_setup_entry(
+    hass: HomeAssistant,
+    config_entry: ConfigEntry,
+    async_add_entities: AddEntitiesCallback,
+) -> None:
     """Set up platform for a new integration.
 
     Called by the HA framework after async_forward_entry_setup has been called
@@ -65,27 +69,25 @@ class NumberEntity(WiffiEntity, SensorEntity):
     def __init__(self, device, metric, options):
         """Initialize the entity."""
         super().__init__(device, metric, options)
-        self._device_class = UOM_TO_DEVICE_CLASS_MAP.get(metric.unit_of_measurement)
-        self._unit_of_measurement = UOM_MAP.get(
+        self._attr_device_class = UOM_TO_DEVICE_CLASS_MAP.get(
+            metric.unit_of_measurement
+        )
+        self._attr_native_unit_of_measurement = UOM_MAP.get(
             metric.unit_of_measurement, metric.unit_of_measurement
         )
-        self._value = metric.value
+        self._attr_native_value = metric.value
+
+        if self._is_measurement_entity():
+            self._attr_state_class = SensorStateClass.MEASUREMENT
+        elif self._is_metered_entity():
+            self._attr_state_class = SensorStateClass.TOTAL_INCREASING
+
         self.reset_expiration_date()
 
     @property
-    def device_class(self):
-        """Return the automatically determined device class."""
-        return self._device_class
-
-    @property
-    def native_unit_of_measurement(self):
-        """Return the unit of measurement of this entity."""
-        return self._unit_of_measurement
-
-    @property
-    def native_value(self):
-        """Return the value of the entity."""
-        return self._value
+    def available(self):
+        """Return true if value is valid."""
+        return self._attr_native_value is not None
 
     @callback
     def _update_value_callback(self, device, metric):
@@ -94,10 +96,12 @@ class NumberEntity(WiffiEntity, SensorEntity):
         Called if a new message has been received from the wiffi device.
         """
         self.reset_expiration_date()
-        self._unit_of_measurement = UOM_MAP.get(
+        self._attr_native_unit_of_measurement = UOM_MAP.get(
             metric.unit_of_measurement, metric.unit_of_measurement
         )
-        self._value = metric.value
+
+        self._attr_native_value = metric.value
+
         self.async_write_ha_state()
 
 
@@ -107,13 +111,13 @@ class StringEntity(WiffiEntity, SensorEntity):
     def __init__(self, device, metric, options):
         """Initialize the entity."""
         super().__init__(device, metric, options)
-        self._value = metric.value
+        self._attr_native_value = metric.value
         self.reset_expiration_date()
 
     @property
-    def native_value(self):
-        """Return the value of the entity."""
-        return self._value
+    def available(self):
+        """Return true if value is valid."""
+        return self._attr_native_value is not None
 
     @callback
     def _update_value_callback(self, device, metric):
@@ -122,5 +126,5 @@ class StringEntity(WiffiEntity, SensorEntity):
         Called if a new message has been received from the wiffi device.
         """
         self.reset_expiration_date()
-        self._value = metric.value
+        self._attr_native_value = metric.value
         self.async_write_ha_state()
