@@ -3,7 +3,7 @@ from __future__ import annotations
 
 from collections.abc import Callable, Coroutine
 from dataclasses import dataclass
-from typing import Optional, cast
+from typing import cast
 
 from pywizlight import wizlight
 
@@ -13,6 +13,7 @@ from homeassistant.components.number import (
     NumberMode,
 )
 from homeassistant.config_entries import ConfigEntry
+from homeassistant.const import EntityCategory
 from homeassistant.core import HomeAssistant, callback
 from homeassistant.helpers.entity_platform import AddEntitiesCallback
 
@@ -21,21 +22,14 @@ from .entity import WizEntity
 from .models import WizData
 
 
-@dataclass
-class WizNumberEntityDescriptionMixin:
-    """Mixin to describe a WiZ number entity."""
-
-    value_fn: Callable[[wizlight], int | None]
-    set_value_fn: Callable[[wizlight, int], Coroutine[None, None, None]]
-    required_feature: str
-
-
-@dataclass
-class WizNumberEntityDescription(
-    NumberEntityDescription, WizNumberEntityDescriptionMixin
-):
+@dataclass(frozen=True, kw_only=True)
+class WizNumberEntityDescription(NumberEntityDescription):
     """Class to describe a WiZ number entity."""
 
+    required_feature: str
+    set_value_fn: Callable[[wizlight, int], Coroutine[None, None, None]]
+    value_fn: Callable[[wizlight], int | None]
+
 
 async def _async_set_speed(device: wizlight, speed: int) -> None:
     await device.set_speed(speed)
@@ -48,25 +42,27 @@ async def _async_set_ratio(device: wizlight, ratio: int) -> None:
 NUMBERS: tuple[WizNumberEntityDescription, ...] = (
     WizNumberEntityDescription(
         key="effect_speed",
+        translation_key="effect_speed",
         native_min_value=10,
         native_max_value=200,
         native_step=1,
         icon="mdi:speedometer",
-        name="Effect speed",
-        value_fn=lambda device: cast(Optional[int], device.state.get_speed()),
+        value_fn=lambda device: cast(int | None, device.state.get_speed()),
         set_value_fn=_async_set_speed,
         required_feature="effect",
+        entity_category=EntityCategory.CONFIG,
     ),
     WizNumberEntityDescription(
         key="dual_head_ratio",
+        translation_key="dual_head_ratio",
         native_min_value=0,
         native_max_value=100,
         native_step=1,
         icon="mdi:floor-lamp-dual",
-        name="Dual head ratio",
-        value_fn=lambda device: cast(Optional[int], device.state.get_ratio()),
+        value_fn=lambda device: cast(int | None, device.state.get_ratio()),
         set_value_fn=_async_set_ratio,
         required_feature="dual_head",
+        entity_category=EntityCategory.CONFIG,
     ),
 )
 
