@@ -1,11 +1,14 @@
 """Support for Xiaomi Gateways."""
+import asyncio
 from datetime import timedelta
 import logging
+from typing import Any
 
 import voluptuous as vol
-from xiaomi_gateway import XiaomiGateway, XiaomiGatewayDiscovery
+from xiaomi_gateway import AsyncXiaomiGatewayMulticast, XiaomiGateway
 
-from homeassistant import config_entries, core
+from homeassistant.components import persistent_notification
+from homeassistant.config_entries import ConfigEntry, ConfigEntryState
 from homeassistant.const import (
     ATTR_BATTERY_LEVEL,
     ATTR_DEVICE_ID,
@@ -15,13 +18,15 @@ from homeassistant.const import (
     CONF_PORT,
     CONF_PROTOCOL,
     EVENT_HOMEASSISTANT_STOP,
+    Platform,
 )
-from homeassistant.core import callback
+from homeassistant.core import HomeAssistant, ServiceCall, callback
 from homeassistant.helpers import device_registry as dr
 import homeassistant.helpers.config_validation as cv
-from homeassistant.helpers.device_registry import format_mac
-from homeassistant.helpers.entity import DeviceInfo, Entity
+from homeassistant.helpers.device_registry import DeviceInfo, format_mac
+from homeassistant.helpers.entity import Entity
 from homeassistant.helpers.event import async_track_point_in_utc_time
+from homeassistant.helpers.typing import ConfigType
 from homeassistant.util.dt import utcnow
 
 from .const import (
@@ -31,13 +36,22 @@ from .const import (
     DEFAULT_DISCOVERY_RETRY,
     DOMAIN,
     GATEWAYS_KEY,
+    KEY_SETUP_LOCK,
+    KEY_UNSUB_STOP,
     LISTENER_KEY,
 )
 
 _LOGGER = logging.getLogger(__name__)
 
-GATEWAY_PLATFORMS = ["binary_sensor", "sensor", "switch", "light", "cover", "lock"]
-GATEWAY_PLATFORMS_NO_KEY = ["binary_sensor", "sensor"]
+GATEWAY_PLATFORMS = [
+    Platform.BINARY_SENSOR,
+    Platform.COVER,
+    Platform.LIGHT,
+    Platform.LOCK,
+    Platform.SENSOR,
+    Platform.SWITCH,
+]
+GATEWAY_PLATFORMS_NO_KEY = [Platform.BINARY_SENSOR, Platform.SENSOR]
 
 ATTR_GW_MAC = "gw_mac"
 ATTR_RINGTONE_ID = "ringtone_id"
@@ -65,14 +79,16 @@ SERVICE_SCHEMA_REMOVE_DEVICE = vol.Schema(
     {vol.Required(ATTR_DEVICE_ID): vol.All(cv.string, vol.Length(min=14, max=14))}
 )
 
+CONFIG_SCHEMA = cv.config_entry_only_config_schema(DOMAIN)
 
-def setup(hass, config):
+
+def setup(hass: HomeAssistant, config: ConfigType) -> bool:
     """Set up the Xiaomi component."""
 
-    def play_ringtone_service(call):
+    def play_ringtone_service(call: ServiceCall) -> None:
         """Service to play ringtone through Gateway."""
         ring_id = call.data.get(ATTR_RINGTONE_ID)
-        gateway = call.data.get(ATTR_GW_MAC)
+        gateway: XiaomiGateway = call.data[ATTR_GW_MAC]
 
         kwargs = {"mid": ring_id}
 
@@ -81,25 +97,28 @@ def setup(hass, config):
 
         gateway.write_to_hub(gateway.sid, **kwargs)
 
-    def stop_ringtone_service(call):
+    def stop_ringtone_service(call: ServiceCall) -> None:
         """Service to stop playing ringtone on Gateway."""
-        gateway = call.data.get(ATTR_GW_MAC)
+        gateway: XiaomiGateway = call.data[ATTR_GW_MAC]
         gateway.write_to_hub(gateway.sid, mid=10000)
 
-    def add_device_service(call):
+    def add_device_service(call: ServiceCall) -> None:
         """Service to add a new sub-device within the next 30 seconds."""
-        gateway = call.data.get(ATTR_GW_MAC)
+        gateway: XiaomiGateway = call.data[ATTR_GW_MAC]
         gateway.write_to_hub(gateway.sid, join_permission="yes")
-        hass.components.persistent_notification.async_create(
-            "Join permission enabled for 30 seconds! "
-            "Please press the pairing button of the new device once.",
+        persistent_notification.async_create(
+            hass,
+            (
+                "Join permission enabled for 30 seconds! "
+                "Please press the pairing button of the new device once."
+            ),
             title="Xiaomi Aqara Gateway",
         )
 
-    def remove_device_service(call):
+    def remove_device_service(call: ServiceCall) -> None:
         """Service to remove a sub-device from the gateway."""
         device_id = call.data.get(ATTR_DEVICE_ID)
-        gateway = call.data.get(ATTR_GW_MAC)
+        gateway: XiaomiGateway = call.data[ATTR_GW_MAC]
         gateway.write_to_hub(gateway.sid, remove_device=device_id)
 
     gateway_only_schema = _add_gateway_to_schema(hass, vol.Schema({}))
@@ -129,11 +148,10 @@ def setup(hass, config):
     return True
 
 
-async def async_setup_entry(
-    hass: core.HomeAssistant, entry: config_entries.ConfigEntry
-):
+async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
     """Set up the xiaomi aqara components from a config entry."""
     hass.data.setdefault(DOMAIN, {})
+    setup_lock = hass.data[DOMAIN].setdefault(KEY_SETUP_LOCK, asyncio.Lock())
     hass.data[DOMAIN].setdefault(GATEWAYS_KEY, {})
 
     # Connect to Xiaomi Aqara Gateway
@@ -149,30 +167,35 @@ async def async_setup_entry(
     )
     hass.data[DOMAIN][GATEWAYS_KEY][entry.entry_id] = xiaomi_gateway
 
-    gateway_discovery = hass.data[DOMAIN].setdefault(
-        LISTENER_KEY,
-        XiaomiGatewayDiscovery(hass.add_job, [], entry.data[CONF_INTERFACE]),
-    )
+    async with setup_lock:
+        if LISTENER_KEY not in hass.data[DOMAIN]:
+            multicast = AsyncXiaomiGatewayMulticast(
+                interface=entry.data[CONF_INTERFACE]
+            )
+            hass.data[DOMAIN][LISTENER_KEY] = multicast
 
-    if len(hass.data[DOMAIN][GATEWAYS_KEY]) == 1:
-        # start listining for local pushes (only once)
-        await hass.async_add_executor_job(gateway_discovery.listen)
+            # start listining for local pushes (only once)
+            await multicast.start_listen()
 
-        # register stop callback to shutdown listining for local pushes
-        def stop_xiaomi(event):
-            """Stop Xiaomi Socket."""
-            _LOGGER.debug("Shutting down Xiaomi Gateway Listener")
-            gateway_discovery.stop_listen()
+            # register stop callback to shutdown listining for local pushes
+            @callback
+            def stop_xiaomi(event):
+                """Stop Xiaomi Socket."""
+                _LOGGER.debug("Shutting down Xiaomi Gateway Listener")
+                multicast.stop_listen()
 
-        hass.bus.async_listen_once(EVENT_HOMEASSISTANT_STOP, stop_xiaomi)
+            unsub = hass.bus.async_listen_once(EVENT_HOMEASSISTANT_STOP, stop_xiaomi)
+            hass.data[DOMAIN][KEY_UNSUB_STOP] = unsub
 
-    gateway_discovery.gateways[entry.data[CONF_HOST]] = xiaomi_gateway
+    multicast = hass.data[DOMAIN][LISTENER_KEY]
+    multicast.register_gateway(entry.data[CONF_HOST], xiaomi_gateway.multicast_callback)
     _LOGGER.debug(
         "Gateway with host '%s' connected, listening for broadcasts",
         entry.data[CONF_HOST],
     )
 
-    device_registry = await dr.async_get_registry(hass)
+    assert entry.unique_id
+    device_registry = dr.async_get(hass)
     device_registry.async_get_or_create(
         config_entry_id=entry.entry_id,
         identifiers={(DOMAIN, entry.unique_id)},
@@ -186,30 +209,37 @@ async def async_setup_entry(
     else:
         platforms = GATEWAY_PLATFORMS_NO_KEY
 
-    hass.config_entries.async_setup_platforms(entry, platforms)
+    await hass.config_entries.async_forward_entry_setups(entry, platforms)
 
     return True
 
 
-async def async_unload_entry(
-    hass: core.HomeAssistant, entry: config_entries.ConfigEntry
-):
+async def async_unload_entry(hass: HomeAssistant, config_entry: ConfigEntry) -> bool:
     """Unload a config entry."""
-    if entry.data[CONF_KEY] is not None:
+    if config_entry.data[CONF_KEY] is not None:
         platforms = GATEWAY_PLATFORMS
     else:
         platforms = GATEWAY_PLATFORMS_NO_KEY
 
-    unload_ok = await hass.config_entries.async_unload_platforms(entry, platforms)
+    unload_ok = await hass.config_entries.async_unload_platforms(
+        config_entry, platforms
+    )
     if unload_ok:
-        hass.data[DOMAIN][GATEWAYS_KEY].pop(entry.entry_id)
+        hass.data[DOMAIN][GATEWAYS_KEY].pop(config_entry.entry_id)
 
-    if len(hass.data[DOMAIN][GATEWAYS_KEY]) == 0:
+    loaded_entries = [
+        entry
+        for entry in hass.config_entries.async_entries(DOMAIN)
+        if entry.state == ConfigEntryState.LOADED
+    ]
+    if len(loaded_entries) == 1:
         # No gateways left, stop Xiaomi socket
+        unsub_stop = hass.data[DOMAIN].pop(KEY_UNSUB_STOP)
+        unsub_stop()
         hass.data[DOMAIN].pop(GATEWAYS_KEY)
         _LOGGER.debug("Shutting down Xiaomi Gateway Listener")
-        gateway_discovery = hass.data[DOMAIN].pop(LISTENER_KEY)
-        await hass.async_add_executor_job(gateway_discovery.stop_listen)
+        multicast = hass.data[DOMAIN].pop(LISTENER_KEY)
+        multicast.stop_listen()
 
     return unload_ok
 
@@ -217,6 +247,8 @@ async def async_unload_entry(
 class XiaomiDevice(Entity):
     """Representation a base Xiaomi device."""
 
+    _attr_should_poll = False
+
     def __init__(self, device, device_type, xiaomi_hub, config_entry):
         """Initialize the Xiaomi device."""
         self._state = None
@@ -235,10 +267,8 @@ class XiaomiDevice(Entity):
         self.parse_data(device["data"], device["raw_data"])
         self.parse_voltage(device["data"])
 
-        if hasattr(self, "_data_key") and self._data_key:  # pylint: disable=no-member
-            self._unique_id = (
-                f"{self._data_key}{self._sid}"  # pylint: disable=no-member
-            )
+        if hasattr(self, "_data_key") and self._data_key:
+            self._unique_id = f"{self._data_key}{self._sid}"
         else:
             self._unique_id = f"{self._type}{self._sid}"
 
@@ -252,12 +282,9 @@ class XiaomiDevice(Entity):
             self._is_gateway = False
             self._device_id = self._sid
 
-    def _add_push_data_job(self, *args):
-        self.hass.add_job(self.push_data, *args)
-
     async def async_added_to_hass(self):
         """Start unavailability tracking."""
-        self._xiaomi_hub.callbacks[self._sid].append(self._add_push_data_job)
+        self._xiaomi_hub.callbacks[self._sid].append(self.push_data)
         self._async_track_unavailable()
 
     @property
@@ -301,11 +328,6 @@ class XiaomiDevice(Entity):
         """Return True if entity is available."""
         return self._is_available
 
-    @property
-    def should_poll(self):
-        """Return the polling state. No polling needed."""
-        return False
-
     @property
     def extra_state_attributes(self):
         """Return the state attributes."""
@@ -330,9 +352,13 @@ class XiaomiDevice(Entity):
             return True
         return False
 
+    def push_data(self, data: dict[str, Any], raw_data: dict[Any, Any]) -> None:
+        """Push from Hub running in another thread."""
+        self.hass.loop.call_soon_threadsafe(self.async_push_data, data, raw_data)
+
     @callback
-    def push_data(self, data, raw_data):
-        """Push from Hub."""
+    def async_push_data(self, data: dict[str, Any], raw_data: dict[Any, Any]) -> None:
+        """Push from Hub handled in the event loop."""
         _LOGGER.debug("PUSH >> %s: %s", self, data)
         was_unavailable = self._async_track_unavailable()
         is_data = self.parse_data(data, raw_data)
