@@ -1,17 +1,28 @@
 """Support for Xiaomi Mi Air Purifier and Xiaomi Mi Air Humidifier."""
+from __future__ import annotations
+
 from abc import abstractmethod
 import asyncio
 import logging
 import math
+from typing import Any
 
-from miio.airfresh import OperationMode as AirfreshOperationMode
-from miio.airfresh_t2017 import OperationMode as AirfreshOperationModeT2017
-from miio.airpurifier import OperationMode as AirpurifierOperationMode
-from miio.airpurifier_miot import OperationMode as AirpurifierMiotOperationMode
 from miio.fan_common import (
     MoveDirection as FanMoveDirection,
     OperationMode as FanOperationMode,
 )
+from miio.integrations.airpurifier.dmaker.airfresh_t2017 import (
+    OperationMode as AirfreshOperationModeT2017,
+)
+from miio.integrations.airpurifier.zhimi.airfresh import (
+    OperationMode as AirfreshOperationMode,
+)
+from miio.integrations.airpurifier.zhimi.airpurifier import (
+    OperationMode as AirpurifierOperationMode,
+)
+from miio.integrations.airpurifier.zhimi.airpurifier_miot import (
+    OperationMode as AirpurifierMiotOperationMode,
+)
 from miio.integrations.fan.zhimi.zhimi_miot import (
     OperationModeFanZA5 as FanZA5OperationMode,
 )
@@ -19,7 +30,7 @@ import voluptuous as vol
 
 from homeassistant.components.fan import FanEntity, FanEntityFeature
 from homeassistant.config_entries import ConfigEntry
-from homeassistant.const import ATTR_ENTITY_ID, CONF_MODEL
+from homeassistant.const import ATTR_ENTITY_ID, CONF_DEVICE, CONF_MODEL
 from homeassistant.core import HomeAssistant, ServiceCall, callback
 import homeassistant.helpers.config_validation as cv
 from homeassistant.helpers.entity_platform import AddEntitiesCallback
@@ -29,7 +40,6 @@ from homeassistant.util.percentage import (
 )
 
 from .const import (
-    CONF_DEVICE,
     CONF_FLOW_TYPE,
     DOMAIN,
     FEATURE_FLAGS_AIRFRESH,
@@ -37,11 +47,14 @@ from .const import (
     FEATURE_FLAGS_AIRFRESH_T2017,
     FEATURE_FLAGS_AIRPURIFIER_2S,
     FEATURE_FLAGS_AIRPURIFIER_3C,
+    FEATURE_FLAGS_AIRPURIFIER_4,
+    FEATURE_FLAGS_AIRPURIFIER_4_LITE,
     FEATURE_FLAGS_AIRPURIFIER_MIIO,
     FEATURE_FLAGS_AIRPURIFIER_MIOT,
     FEATURE_FLAGS_AIRPURIFIER_PRO,
     FEATURE_FLAGS_AIRPURIFIER_PRO_V7,
     FEATURE_FLAGS_AIRPURIFIER_V3,
+    FEATURE_FLAGS_AIRPURIFIER_ZA1,
     FEATURE_FLAGS_FAN,
     FEATURE_FLAGS_FAN_1C,
     FEATURE_FLAGS_FAN_P5,
@@ -57,9 +70,14 @@ from .const import (
     MODEL_AIRPURIFIER_2H,
     MODEL_AIRPURIFIER_2S,
     MODEL_AIRPURIFIER_3C,
+    MODEL_AIRPURIFIER_4,
+    MODEL_AIRPURIFIER_4_LITE_RMA1,
+    MODEL_AIRPURIFIER_4_LITE_RMB1,
+    MODEL_AIRPURIFIER_4_PRO,
     MODEL_AIRPURIFIER_PRO,
     MODEL_AIRPURIFIER_PRO_V7,
     MODEL_AIRPURIFIER_V3,
+    MODEL_AIRPURIFIER_ZA1,
     MODEL_FAN_1C,
     MODEL_FAN_P5,
     MODEL_FAN_P9,
@@ -137,11 +155,13 @@ AVAILABLE_ATTRIBUTES_AIRFRESH = {
 }
 
 PRESET_MODES_AIRPURIFIER = ["Auto", "Silent", "Favorite", "Idle"]
+PRESET_MODES_AIRPURIFIER_4_LITE = ["Auto", "Silent", "Favorite"]
 PRESET_MODES_AIRPURIFIER_MIOT = ["Auto", "Silent", "Favorite", "Fan"]
 PRESET_MODES_AIRPURIFIER_PRO = ["Auto", "Silent", "Favorite"]
 PRESET_MODES_AIRPURIFIER_PRO_V7 = PRESET_MODES_AIRPURIFIER_PRO
 PRESET_MODES_AIRPURIFIER_2S = ["Auto", "Silent", "Favorite"]
 PRESET_MODES_AIRPURIFIER_3C = ["Auto", "Silent", "Favorite"]
+PRESET_MODES_AIRPURIFIER_ZA1 = ["Auto", "Silent", "Favorite"]
 PRESET_MODES_AIRPURIFIER_V3 = [
     "Auto",
     "Silent",
@@ -180,14 +200,14 @@ async def async_setup_entry(
     async_add_entities: AddEntitiesCallback,
 ) -> None:
     """Set up the Fan from a config entry."""
-    entities = []
+    entities: list[FanEntity] = []
+    entity: FanEntity
 
-    if not config_entry.data[CONF_FLOW_TYPE] == CONF_DEVICE:
+    if config_entry.data[CONF_FLOW_TYPE] != CONF_DEVICE:
         return
 
     hass.data.setdefault(DATA_KEY, {})
 
-    name = config_entry.title
     model = config_entry.data[CONF_MODEL]
     unique_id = config_entry.unique_id
     coordinator = hass.data[DOMAIN][config_entry.entry_id][KEY_COORDINATOR]
@@ -195,7 +215,6 @@ async def async_setup_entry(
 
     if model == MODEL_AIRPURIFIER_3C:
         entity = XiaomiAirPurifierMB4(
-            name,
             device,
             config_entry,
             unique_id,
@@ -203,28 +222,29 @@ async def async_setup_entry(
         )
     elif model in MODELS_PURIFIER_MIOT:
         entity = XiaomiAirPurifierMiot(
-            name,
             device,
             config_entry,
             unique_id,
             coordinator,
         )
     elif model.startswith("zhimi.airpurifier."):
-        entity = XiaomiAirPurifier(name, device, config_entry, unique_id, coordinator)
+        entity = XiaomiAirPurifier(device, config_entry, unique_id, coordinator)
     elif model.startswith("zhimi.airfresh."):
-        entity = XiaomiAirFresh(name, device, config_entry, unique_id, coordinator)
+        entity = XiaomiAirFresh(device, config_entry, unique_id, coordinator)
     elif model == MODEL_AIRFRESH_A1:
-        entity = XiaomiAirFreshA1(name, device, config_entry, unique_id, coordinator)
+        entity = XiaomiAirFreshA1(device, config_entry, unique_id, coordinator)
     elif model == MODEL_AIRFRESH_T2017:
-        entity = XiaomiAirFreshT2017(name, device, config_entry, unique_id, coordinator)
+        entity = XiaomiAirFreshT2017(device, config_entry, unique_id, coordinator)
     elif model == MODEL_FAN_P5:
-        entity = XiaomiFanP5(name, device, config_entry, unique_id, coordinator)
+        entity = XiaomiFanP5(device, config_entry, unique_id, coordinator)
     elif model in MODELS_FAN_MIIO:
-        entity = XiaomiFan(name, device, config_entry, unique_id, coordinator)
+        entity = XiaomiFan(device, config_entry, unique_id, coordinator)
     elif model == MODEL_FAN_ZA5:
-        entity = XiaomiFanZA5(name, device, config_entry, unique_id, coordinator)
+        entity = XiaomiFanZA5(device, config_entry, unique_id, coordinator)
+    elif model == MODEL_FAN_1C:
+        entity = XiaomiFan1C(device, config_entry, unique_id, coordinator)
     elif model in MODELS_FAN_MIOT:
-        entity = XiaomiFanMiot(name, device, config_entry, unique_id, coordinator)
+        entity = XiaomiFanMiot(device, config_entry, unique_id, coordinator)
     else:
         return
 
@@ -254,9 +274,7 @@ async def async_setup_entry(
             if not entity_method:
                 continue
             await entity_method(**params)
-            update_tasks.append(
-                hass.async_create_task(entity.async_update_ha_state(True))
-            )
+            update_tasks.append(asyncio.create_task(entity.async_update_ha_state(True)))
 
         if update_tasks:
             await asyncio.wait(update_tasks)
@@ -273,9 +291,11 @@ async def async_setup_entry(
 class XiaomiGenericDevice(XiaomiCoordinatedMiioEntity, FanEntity):
     """Representation of a generic Xiaomi device."""
 
-    def __init__(self, name, device, entry, unique_id, coordinator):
+    _attr_name = None
+
+    def __init__(self, device, entry, unique_id, coordinator):
         """Initialize the generic Xiaomi device."""
-        super().__init__(name, device, entry, unique_id, coordinator)
+        super().__init__(device, entry, unique_id, coordinator)
 
         self._available_attributes = {}
         self._state = None
@@ -291,30 +311,30 @@ class XiaomiGenericDevice(XiaomiCoordinatedMiioEntity, FanEntity):
         """Hold operation mode class."""
 
     @property
-    def preset_modes(self) -> list:
+    def preset_modes(self) -> list[str]:
         """Get the list of available preset modes."""
         return self._preset_modes
 
     @property
-    def percentage(self):
+    def percentage(self) -> int | None:
         """Return the percentage based speed of the fan."""
         return None
 
     @property
-    def extra_state_attributes(self):
+    def extra_state_attributes(self) -> dict[str, Any]:
         """Return the state attributes of the device."""
         return self._state_attrs
 
     @property
-    def is_on(self):
+    def is_on(self) -> bool | None:
         """Return true if device is on."""
         return self._state
 
     async def async_turn_on(
         self,
-        percentage: int = None,
-        preset_mode: str = None,
-        **kwargs,
+        percentage: int | None = None,
+        preset_mode: str | None = None,
+        **kwargs: Any,
     ) -> None:
         """Turn the device on."""
         result = await self._try_command(
@@ -331,7 +351,7 @@ class XiaomiGenericDevice(XiaomiCoordinatedMiioEntity, FanEntity):
             self._state = True
             self.async_write_ha_state()
 
-    async def async_turn_off(self, **kwargs) -> None:
+    async def async_turn_off(self, **kwargs: Any) -> None:
         """Turn the device off."""
         result = await self._try_command(
             "Turning the miio device off failed.", self._device.off
@@ -345,19 +365,19 @@ class XiaomiGenericDevice(XiaomiCoordinatedMiioEntity, FanEntity):
 class XiaomiGenericAirPurifier(XiaomiGenericDevice):
     """Representation of a generic AirPurifier device."""
 
-    def __init__(self, name, device, entry, unique_id, coordinator):
+    def __init__(self, device, entry, unique_id, coordinator):
         """Initialize the generic AirPurifier device."""
-        super().__init__(name, device, entry, unique_id, coordinator)
+        super().__init__(device, entry, unique_id, coordinator)
 
         self._speed_count = 100
 
     @property
-    def speed_count(self):
+    def speed_count(self) -> int:
         """Return the number of speeds of the fan supported."""
         return self._speed_count
 
     @property
-    def preset_mode(self):
+    def preset_mode(self) -> str | None:
         """Get the active preset mode."""
         if self._state:
             preset_mode = self.operation_mode_class(self._mode).name
@@ -392,9 +412,9 @@ class XiaomiAirPurifier(XiaomiGenericAirPurifier):
 
     REVERSE_SPEED_MODE_MAPPING = {v: k for k, v in SPEED_MODE_MAPPING.items()}
 
-    def __init__(self, name, device, entry, unique_id, coordinator):
+    def __init__(self, device, entry, unique_id, coordinator):
         """Initialize the plug switch."""
-        super().__init__(name, device, entry, unique_id, coordinator)
+        super().__init__(device, entry, unique_id, coordinator)
 
         if self._model == MODEL_AIRPURIFIER_PRO:
             self._device_features = FEATURE_FLAGS_AIRPURIFIER_PRO
@@ -402,6 +422,23 @@ class XiaomiAirPurifier(XiaomiGenericAirPurifier):
             self._preset_modes = PRESET_MODES_AIRPURIFIER_PRO
             self._attr_supported_features = FanEntityFeature.PRESET_MODE
             self._speed_count = 1
+        elif self._model in [MODEL_AIRPURIFIER_4, MODEL_AIRPURIFIER_4_PRO]:
+            self._device_features = FEATURE_FLAGS_AIRPURIFIER_4
+            self._available_attributes = AVAILABLE_ATTRIBUTES_AIRPURIFIER_MIOT
+            self._preset_modes = PRESET_MODES_AIRPURIFIER_MIOT
+            self._attr_supported_features = (
+                FanEntityFeature.SET_SPEED | FanEntityFeature.PRESET_MODE
+            )
+            self._speed_count = 3
+        elif self._model in [
+            MODEL_AIRPURIFIER_4_LITE_RMA1,
+            MODEL_AIRPURIFIER_4_LITE_RMB1,
+        ]:
+            self._device_features = FEATURE_FLAGS_AIRPURIFIER_4_LITE
+            self._available_attributes = AVAILABLE_ATTRIBUTES_AIRPURIFIER_MIOT
+            self._preset_modes = PRESET_MODES_AIRPURIFIER_4_LITE
+            self._attr_supported_features = FanEntityFeature.PRESET_MODE
+            self._speed_count = 1
         elif self._model == MODEL_AIRPURIFIER_PRO_V7:
             self._device_features = FEATURE_FLAGS_AIRPURIFIER_PRO_V7
             self._available_attributes = AVAILABLE_ATTRIBUTES_AIRPURIFIER_PRO_V7
@@ -414,6 +451,12 @@ class XiaomiAirPurifier(XiaomiGenericAirPurifier):
             self._preset_modes = PRESET_MODES_AIRPURIFIER_2S
             self._attr_supported_features = FanEntityFeature.PRESET_MODE
             self._speed_count = 1
+        elif self._model == MODEL_AIRPURIFIER_ZA1:
+            self._device_features = FEATURE_FLAGS_AIRPURIFIER_ZA1
+            self._available_attributes = AVAILABLE_ATTRIBUTES_AIRPURIFIER_MIOT
+            self._preset_modes = PRESET_MODES_AIRPURIFIER_ZA1
+            self._attr_supported_features = FanEntityFeature.PRESET_MODE
+            self._speed_count = 1
         elif self._model in MODELS_PURIFIER_MIOT:
             self._device_features = FEATURE_FLAGS_AIRPURIFIER_MIOT
             self._available_attributes = AVAILABLE_ATTRIBUTES_AIRPURIFIER_MIOT
@@ -451,7 +494,7 @@ class XiaomiAirPurifier(XiaomiGenericAirPurifier):
         return AirpurifierOperationMode
 
     @property
-    def percentage(self):
+    def percentage(self) -> int | None:
         """Return the current percentage based speed."""
         if self._state:
             mode = self.operation_mode_class(self._mode)
@@ -486,9 +529,6 @@ class XiaomiAirPurifier(XiaomiGenericAirPurifier):
 
         This method is a coroutine.
         """
-        if preset_mode not in self.preset_modes:
-            _LOGGER.warning("'%s'is not a valid preset mode", preset_mode)
-            return
         if await self._try_command(
             "Setting operation mode of the miio device failed.",
             self._device.set_mode,
@@ -528,7 +568,7 @@ class XiaomiAirPurifierMiot(XiaomiAirPurifier):
         return AirpurifierMiotOperationMode
 
     @property
-    def percentage(self):
+    def percentage(self) -> int | None:
         """Return the current percentage based speed."""
         if self._fan_level is None:
             return None
@@ -561,9 +601,9 @@ class XiaomiAirPurifierMiot(XiaomiAirPurifier):
 class XiaomiAirPurifierMB4(XiaomiGenericAirPurifier):
     """Representation of a Xiaomi Air Purifier MB4."""
 
-    def __init__(self, name, device, entry, unique_id, coordinator):
+    def __init__(self, device, entry, unique_id, coordinator):
         """Initialize Air Purifier MB4."""
-        super().__init__(name, device, entry, unique_id, coordinator)
+        super().__init__(device, entry, unique_id, coordinator)
 
         self._device_features = FEATURE_FLAGS_AIRPURIFIER_3C
         self._preset_modes = PRESET_MODES_AIRPURIFIER_3C
@@ -579,9 +619,6 @@ class XiaomiAirPurifierMB4(XiaomiGenericAirPurifier):
 
     async def async_set_preset_mode(self, preset_mode: str) -> None:
         """Set the preset mode of the fan."""
-        if preset_mode not in self.preset_modes:
-            _LOGGER.warning("'%s'is not a valid preset mode", preset_mode)
-            return
         if await self._try_command(
             "Setting operation mode of the miio device failed.",
             self._device.set_mode,
@@ -615,9 +652,9 @@ class XiaomiAirFresh(XiaomiGenericAirPurifier):
         "Interval": AirfreshOperationMode.Interval,
     }
 
-    def __init__(self, name, device, entry, unique_id, coordinator):
+    def __init__(self, device, entry, unique_id, coordinator):
         """Initialize the miio device."""
-        super().__init__(name, device, entry, unique_id, coordinator)
+        super().__init__(device, entry, unique_id, coordinator)
 
         self._device_features = FEATURE_FLAGS_AIRFRESH
         self._available_attributes = AVAILABLE_ATTRIBUTES_AIRFRESH
@@ -642,7 +679,7 @@ class XiaomiAirFresh(XiaomiGenericAirPurifier):
         return AirfreshOperationMode
 
     @property
-    def percentage(self):
+    def percentage(self) -> int | None:
         """Return the current percentage based speed."""
         if self._state:
             mode = AirfreshOperationMode(self._mode)
@@ -677,9 +714,6 @@ class XiaomiAirFresh(XiaomiGenericAirPurifier):
 
         This method is a coroutine.
         """
-        if preset_mode not in self.preset_modes:
-            _LOGGER.warning("'%s'is not a valid preset mode", preset_mode)
-            return
         if await self._try_command(
             "Setting operation mode of the miio device failed.",
             self._device.set_mode,
@@ -713,9 +747,9 @@ class XiaomiAirFresh(XiaomiGenericAirPurifier):
 class XiaomiAirFreshA1(XiaomiGenericAirPurifier):
     """Representation of a Xiaomi Air Fresh A1."""
 
-    def __init__(self, name, device, entry, unique_id, coordinator):
+    def __init__(self, device, entry, unique_id, coordinator):
         """Initialize the miio device."""
-        super().__init__(name, device, entry, unique_id, coordinator)
+        super().__init__(device, entry, unique_id, coordinator)
         self._favorite_speed = None
         self._device_features = FEATURE_FLAGS_AIRFRESH_A1
         self._preset_modes = PRESET_MODES_AIRFRESH_A1
@@ -733,7 +767,7 @@ class XiaomiAirFreshA1(XiaomiGenericAirPurifier):
         return AirfreshOperationModeT2017
 
     @property
-    def percentage(self):
+    def percentage(self) -> int | None:
         """Return the current percentage based speed."""
         if self._favorite_speed is None:
             return None
@@ -765,9 +799,6 @@ class XiaomiAirFreshA1(XiaomiGenericAirPurifier):
 
     async def async_set_preset_mode(self, preset_mode: str) -> None:
         """Set the preset mode of the fan. This method is a coroutine."""
-        if preset_mode not in self.preset_modes:
-            _LOGGER.warning("'%s'is not a valid preset mode", preset_mode)
-            return
         if await self._try_command(
             "Setting operation mode of the miio device failed.",
             self._device.set_mode,
@@ -788,9 +819,9 @@ class XiaomiAirFreshA1(XiaomiGenericAirPurifier):
 class XiaomiAirFreshT2017(XiaomiAirFreshA1):
     """Representation of a Xiaomi Air Fresh T2017."""
 
-    def __init__(self, name, device, entry, unique_id, coordinator):
+    def __init__(self, device, entry, unique_id, coordinator):
         """Initialize the miio device."""
-        super().__init__(name, device, entry, unique_id, coordinator)
+        super().__init__(device, entry, unique_id, coordinator)
         self._device_features = FEATURE_FLAGS_AIRFRESH_T2017
         self._speed_range = (60, 300)
 
@@ -798,9 +829,9 @@ class XiaomiAirFreshT2017(XiaomiAirFreshA1):
 class XiaomiGenericFan(XiaomiGenericDevice):
     """Representation of a generic Xiaomi Fan."""
 
-    def __init__(self, name, device, entry, unique_id, coordinator):
+    def __init__(self, device, entry, unique_id, coordinator):
         """Initialize the fan."""
-        super().__init__(name, device, entry, unique_id, coordinator)
+        super().__init__(device, entry, unique_id, coordinator)
 
         if self._model == MODEL_FAN_P5:
             self._device_features = FEATURE_FLAGS_FAN_P5
@@ -826,17 +857,17 @@ class XiaomiGenericFan(XiaomiGenericDevice):
         self._percentage = None
 
     @property
-    def preset_mode(self):
+    def preset_mode(self) -> str | None:
         """Get the active preset mode."""
         return self._preset_mode
 
     @property
-    def preset_modes(self) -> list:
+    def preset_modes(self) -> list[str]:
         """Get the list of available preset modes."""
         return [mode.name for mode in self.operation_mode_class]
 
     @property
-    def percentage(self):
+    def percentage(self) -> int | None:
         """Return the current speed as a percentage."""
         if self._state:
             return self._percentage
@@ -844,7 +875,7 @@ class XiaomiGenericFan(XiaomiGenericDevice):
         return None
 
     @property
-    def oscillating(self):
+    def oscillating(self) -> bool | None:
         """Return whether or not the fan is currently oscillating."""
         return self._oscillating
 
@@ -873,9 +904,9 @@ class XiaomiGenericFan(XiaomiGenericDevice):
 class XiaomiFan(XiaomiGenericFan):
     """Representation of a Xiaomi Fan."""
 
-    def __init__(self, name, device, entry, unique_id, coordinator):
+    def __init__(self, device, entry, unique_id, coordinator):
         """Initialize the fan."""
-        super().__init__(name, device, entry, unique_id, coordinator)
+        super().__init__(device, entry, unique_id, coordinator)
 
         self._state = self.coordinator.data.is_on
         self._oscillating = self.coordinator.data.oscillate
@@ -890,12 +921,12 @@ class XiaomiFan(XiaomiGenericFan):
         """Hold operation mode class."""
 
     @property
-    def preset_mode(self):
+    def preset_mode(self) -> str:
         """Get the active preset mode."""
         return ATTR_MODE_NATURE if self._nature_mode else ATTR_MODE_NORMAL
 
     @property
-    def preset_modes(self) -> list:
+    def preset_modes(self) -> list[str]:
         """Get the list of available preset modes."""
         return [ATTR_MODE_NATURE, ATTR_MODE_NORMAL]
 
@@ -914,10 +945,6 @@ class XiaomiFan(XiaomiGenericFan):
 
     async def async_set_preset_mode(self, preset_mode: str) -> None:
         """Set the preset mode of the fan."""
-        if preset_mode not in self.preset_modes:
-            _LOGGER.warning("'%s'is not a valid preset mode", preset_mode)
-            return
-
         if preset_mode == ATTR_MODE_NATURE:
             await self._try_command(
                 "Setting natural fan speed percentage of the miio device failed.",
@@ -964,9 +991,9 @@ class XiaomiFan(XiaomiGenericFan):
 class XiaomiFanP5(XiaomiGenericFan):
     """Representation of a Xiaomi Fan P5."""
 
-    def __init__(self, name, device, entry, unique_id, coordinator):
+    def __init__(self, device, entry, unique_id, coordinator):
         """Initialize the fan."""
-        super().__init__(name, device, entry, unique_id, coordinator)
+        super().__init__(device, entry, unique_id, coordinator)
 
         self._state = self.coordinator.data.is_on
         self._preset_mode = self.coordinator.data.mode.name
@@ -990,9 +1017,6 @@ class XiaomiFanP5(XiaomiGenericFan):
 
     async def async_set_preset_mode(self, preset_mode: str) -> None:
         """Set the preset mode of the fan."""
-        if preset_mode not in self.preset_modes:
-            _LOGGER.warning("'%s'is not a valid preset mode", preset_mode)
-            return
         await self._try_command(
             "Setting operation mode of the miio device failed.",
             self._device.set_mode,
@@ -1030,7 +1054,7 @@ class XiaomiFanMiot(XiaomiGenericFan):
         return FanOperationMode
 
     @property
-    def preset_mode(self):
+    def preset_mode(self) -> str | None:
         """Get the active preset mode."""
         return self._preset_mode
 
@@ -1049,9 +1073,6 @@ class XiaomiFanMiot(XiaomiGenericFan):
 
     async def async_set_preset_mode(self, preset_mode: str) -> None:
         """Set the preset mode of the fan."""
-        if preset_mode not in self.preset_modes:
-            _LOGGER.warning("'%s'is not a valid preset mode", preset_mode)
-            return
         await self._try_command(
             "Setting operation mode of the miio device failed.",
             self._device.set_mode,
@@ -1067,16 +1088,17 @@ class XiaomiFanMiot(XiaomiGenericFan):
             await self.async_turn_off()
             return
 
-        await self._try_command(
+        result = await self._try_command(
             "Setting fan speed percentage of the miio device failed.",
             self._device.set_speed,
             percentage,
         )
-        self._percentage = percentage
+        if result:
+            self._percentage = percentage
 
         if not self.is_on:
             await self.async_turn_on()
-        else:
+        elif result:
             self.async_write_ha_state()
 
 
@@ -1087,3 +1109,52 @@ class XiaomiFanZA5(XiaomiFanMiot):
     def operation_mode_class(self):
         """Hold operation mode class."""
         return FanZA5OperationMode
+
+
+class XiaomiFan1C(XiaomiFanMiot):
+    """Representation of a Xiaomi Fan 1C (Standing Fan 2 Lite)."""
+
+    def __init__(self, device, entry, unique_id, coordinator):
+        """Initialize MIOT fan with speed count."""
+        super().__init__(device, entry, unique_id, coordinator)
+        self._speed_count = 3
+
+    @callback
+    def _handle_coordinator_update(self):
+        """Fetch state from the device."""
+        self._state = self.coordinator.data.is_on
+        self._preset_mode = self.coordinator.data.mode.name
+        self._oscillating = self.coordinator.data.oscillate
+        if self.coordinator.data.is_on:
+            self._percentage = ranged_value_to_percentage(
+                (1, self._speed_count), self.coordinator.data.speed
+            )
+        else:
+            self._percentage = 0
+
+        self.async_write_ha_state()
+
+    async def async_set_percentage(self, percentage: int) -> None:
+        """Set the percentage of the fan."""
+        if percentage == 0:
+            self._percentage = 0
+            await self.async_turn_off()
+            return
+
+        speed = math.ceil(
+            percentage_to_ranged_value((1, self._speed_count), percentage)
+        )
+
+        # if the fan is not on, we have to turn it on first
+        if not self.is_on:
+            await self.async_turn_on()
+
+        result = await self._try_command(
+            "Setting fan speed percentage of the miio device failed.",
+            self._device.set_speed,
+            speed,
+        )
+
+        if result:
+            self._percentage = ranged_value_to_percentage((1, self._speed_count), speed)
+            self.async_write_ha_state()
