@@ -1,23 +1,32 @@
 """Support for Xiaomi Mi Air Purifier and Xiaomi Mi Air Humidifier with humidifier entity."""
 import logging
 import math
+from typing import Any
 
-from miio.airhumidifier import OperationMode as AirhumidifierOperationMode
-from miio.airhumidifier_miot import OperationMode as AirhumidifierMiotOperationMode
-from miio.airhumidifier_mjjsq import OperationMode as AirhumidifierMjjsqOperationMode
+from miio.integrations.humidifier.deerma.airhumidifier_mjjsq import (
+    OperationMode as AirhumidifierMjjsqOperationMode,
+)
+from miio.integrations.humidifier.zhimi.airhumidifier import (
+    OperationMode as AirhumidifierOperationMode,
+)
+from miio.integrations.humidifier.zhimi.airhumidifier_miot import (
+    OperationMode as AirhumidifierMiotOperationMode,
+)
 
-from homeassistant.components.humidifier import HumidifierDeviceClass, HumidifierEntity
-from homeassistant.components.humidifier.const import SUPPORT_MODES
+from homeassistant.components.humidifier import (
+    ATTR_HUMIDITY,
+    HumidifierDeviceClass,
+    HumidifierEntity,
+    HumidifierEntityFeature,
+)
 from homeassistant.config_entries import ConfigEntry
-from homeassistant.const import ATTR_MODE
+from homeassistant.const import ATTR_MODE, CONF_DEVICE, CONF_MODEL
 from homeassistant.core import HomeAssistant, callback
 from homeassistant.helpers.entity_platform import AddEntitiesCallback
 from homeassistant.util.percentage import percentage_to_ranged_value
 
 from .const import (
-    CONF_DEVICE,
     CONF_FLOW_TYPE,
-    CONF_MODEL,
     DOMAIN,
     KEY_COORDINATOR,
     KEY_DEVICE,
@@ -37,6 +46,7 @@ ATTR_TARGET_HUMIDITY = "target_humidity"
 AVAILABLE_ATTRIBUTES = {
     ATTR_MODE: "mode",
     ATTR_TARGET_HUMIDITY: "target_humidity",
+    ATTR_HUMIDITY: "humidity",
 }
 
 AVAILABLE_MODES_CA1_CB1 = [
@@ -63,19 +73,18 @@ async def async_setup_entry(
     async_add_entities: AddEntitiesCallback,
 ) -> None:
     """Set up the Humidifier from a config entry."""
-    if not config_entry.data[CONF_FLOW_TYPE] == CONF_DEVICE:
+    if config_entry.data[CONF_FLOW_TYPE] != CONF_DEVICE:
         return
 
-    entities = []
+    entities: list[HumidifierEntity] = []
+    entity: HumidifierEntity
     model = config_entry.data[CONF_MODEL]
     unique_id = config_entry.unique_id
     coordinator = hass.data[DOMAIN][config_entry.entry_id][KEY_COORDINATOR]
-    name = config_entry.title
 
     if model in MODELS_HUMIDIFIER_MIOT:
         air_humidifier = hass.data[DOMAIN][config_entry.entry_id][KEY_DEVICE]
         entity = XiaomiAirHumidifierMiot(
-            name,
             air_humidifier,
             config_entry,
             unique_id,
@@ -84,7 +93,6 @@ async def async_setup_entry(
     elif model in MODELS_HUMIDIFIER_MJJSQ:
         air_humidifier = hass.data[DOMAIN][config_entry.entry_id][KEY_DEVICE]
         entity = XiaomiAirHumidifierMjjsq(
-            name,
             air_humidifier,
             config_entry,
             unique_id,
@@ -93,7 +101,6 @@ async def async_setup_entry(
     else:
         air_humidifier = hass.data[DOMAIN][config_entry.entry_id][KEY_DEVICE]
         entity = XiaomiAirHumidifier(
-            name,
             air_humidifier,
             config_entry,
             unique_id,
@@ -109,11 +116,12 @@ class XiaomiGenericHumidifier(XiaomiCoordinatedMiioEntity, HumidifierEntity):
     """Representation of a generic Xiaomi humidifier device."""
 
     _attr_device_class = HumidifierDeviceClass.HUMIDIFIER
-    _attr_supported_features = SUPPORT_MODES
+    _attr_supported_features = HumidifierEntityFeature.MODES
+    _attr_name = None
 
-    def __init__(self, name, device, entry, unique_id, coordinator):
+    def __init__(self, device, entry, unique_id, coordinator):
         """Initialize the generic Xiaomi device."""
-        super().__init__(name, device, entry, unique_id, coordinator=coordinator)
+        super().__init__(device, entry, unique_id, coordinator=coordinator)
 
         self._state = None
         self._attributes = {}
@@ -131,10 +139,7 @@ class XiaomiGenericHumidifier(XiaomiCoordinatedMiioEntity, HumidifierEntity):
         """Get the current mode."""
         return self._mode
 
-    async def async_turn_on(
-        self,
-        **kwargs,
-    ) -> None:
+    async def async_turn_on(self, **kwargs: Any) -> None:
         """Turn the device on."""
         result = await self._try_command(
             "Turning the miio device on failed.", self._device.on
@@ -143,7 +148,7 @@ class XiaomiGenericHumidifier(XiaomiCoordinatedMiioEntity, HumidifierEntity):
             self._state = True
             self.async_write_ha_state()
 
-    async def async_turn_off(self, **kwargs) -> None:
+    async def async_turn_off(self, **kwargs: Any) -> None:
         """Turn the device off."""
         result = await self._try_command(
             "Turning the miio device off failed.", self._device.off
@@ -167,9 +172,11 @@ class XiaomiGenericHumidifier(XiaomiCoordinatedMiioEntity, HumidifierEntity):
 class XiaomiAirHumidifier(XiaomiGenericHumidifier, HumidifierEntity):
     """Representation of a Xiaomi Air Humidifier."""
 
-    def __init__(self, name, device, entry, unique_id, coordinator):
+    available_modes: list[str]
+
+    def __init__(self, device, entry, unique_id, coordinator):
         """Initialize the plug switch."""
-        super().__init__(name, device, entry, unique_id, coordinator)
+        super().__init__(device, entry, unique_id, coordinator)
 
         self._attr_min_humidity = 30
         self._attr_max_humidity = 80
@@ -194,6 +201,7 @@ class XiaomiAirHumidifier(XiaomiGenericHumidifier, HumidifierEntity):
             }
         )
         self._target_humidity = self._attributes[ATTR_TARGET_HUMIDITY]
+        self._attr_current_humidity = self._attributes[ATTR_HUMIDITY]
         self._mode = self._attributes[ATTR_MODE]
 
     @property
@@ -212,6 +220,7 @@ class XiaomiAirHumidifier(XiaomiGenericHumidifier, HumidifierEntity):
             }
         )
         self._target_humidity = self._attributes[ATTR_TARGET_HUMIDITY]
+        self._attr_current_humidity = self._attributes[ATTR_HUMIDITY]
         self._mode = self._attributes[ATTR_MODE]
         self.async_write_ha_state()
 
@@ -244,7 +253,7 @@ class XiaomiAirHumidifier(XiaomiGenericHumidifier, HumidifierEntity):
         ):
             self._target_humidity = target_humidity
         if (
-            self.supported_features & SUPPORT_MODES == 0
+            self.supported_features & HumidifierEntityFeature.MODES == 0
             or AirhumidifierOperationMode(self._attributes[ATTR_MODE])
             == AirhumidifierOperationMode.Auto
             or AirhumidifierOperationMode.Auto.name not in self.available_modes
@@ -262,7 +271,7 @@ class XiaomiAirHumidifier(XiaomiGenericHumidifier, HumidifierEntity):
 
     async def async_set_mode(self, mode: str) -> None:
         """Set the mode of the humidifier."""
-        if self.supported_features & SUPPORT_MODES == 0 or not mode:
+        if self.supported_features & HumidifierEntityFeature.MODES == 0 or not mode:
             return
 
         if mode not in self.available_modes:
@@ -322,7 +331,7 @@ class XiaomiAirHumidifierMiot(XiaomiAirHumidifier):
         ):
             self._target_humidity = target_humidity
         if (
-            self.supported_features & SUPPORT_MODES == 0
+            self.supported_features & HumidifierEntityFeature.MODES == 0
             or AirhumidifierMiotOperationMode(self._attributes[ATTR_MODE])
             == AirhumidifierMiotOperationMode.Auto
         ):
@@ -339,7 +348,7 @@ class XiaomiAirHumidifierMiot(XiaomiAirHumidifier):
 
     async def async_set_mode(self, mode: str) -> None:
         """Set the mode of the fan."""
-        if self.supported_features & SUPPORT_MODES == 0 or not mode:
+        if self.supported_features & HumidifierEntityFeature.MODES == 0 or not mode:
             return
 
         if mode not in self.REVERSE_MODE_MAPPING:
@@ -397,7 +406,7 @@ class XiaomiAirHumidifierMjjsq(XiaomiAirHumidifier):
         ):
             self._target_humidity = target_humidity
         if (
-            self.supported_features & SUPPORT_MODES == 0
+            self.supported_features & HumidifierEntityFeature.MODES == 0
             or AirhumidifierMjjsqOperationMode(self._attributes[ATTR_MODE])
             == AirhumidifierMjjsqOperationMode.Humidity
         ):
