@@ -4,41 +4,39 @@ from __future__ import annotations
 import asyncio
 import collections
 from collections.abc import Callable
+from contextlib import suppress
 from datetime import timedelta
 from enum import Enum
 import itertools
 import logging
-import os
+import re
 import time
-import traceback
-from typing import TYPE_CHECKING, Any, NamedTuple, Union
+from typing import TYPE_CHECKING, Any, NamedTuple, Self, cast
 
-from serial import SerialException
 from zigpy.application import ControllerApplication
-from zigpy.config import CONF_DEVICE
+from zigpy.config import (
+    CONF_DATABASE,
+    CONF_DEVICE,
+    CONF_DEVICE_PATH,
+    CONF_NWK,
+    CONF_NWK_CHANNEL,
+    CONF_NWK_VALIDATE_SETTINGS,
+)
 import zigpy.device
 import zigpy.endpoint
 import zigpy.group
+from zigpy.state import State
 from zigpy.types.named import EUI64
 
+from homeassistant import __path__ as HOMEASSISTANT_PATH
 from homeassistant.components.system_log import LogEntry, _figure_out_source
 from homeassistant.config_entries import ConfigEntry
 from homeassistant.core import HomeAssistant, callback
-from homeassistant.exceptions import ConfigEntryNotReady
-from homeassistant.helpers.device_registry import (
-    CONNECTION_ZIGBEE,
-    DeviceRegistry,
-    async_get_registry as get_dev_reg,
-)
+from homeassistant.helpers import device_registry as dr, entity_registry as er
+from homeassistant.helpers.device_registry import DeviceInfo
 from homeassistant.helpers.dispatcher import async_dispatcher_send
-from homeassistant.helpers.entity import DeviceInfo
-from homeassistant.helpers.entity_registry import (
-    EntityRegistry,
-    async_entries_for_device,
-    async_get_registry as get_ent_reg,
-)
-from homeassistant.helpers.event import async_track_time_interval
 from homeassistant.helpers.typing import ConfigType
+from homeassistant.util.async_ import gather_with_limited_concurrency
 
 from . import discovery
 from .const import (
@@ -48,12 +46,10 @@ from .const import (
     ATTR_NWK,
     ATTR_SIGNATURE,
     ATTR_TYPE,
-    CONF_DATABASE,
     CONF_RADIO_TYPE,
+    CONF_USE_THREAD,
     CONF_ZIGPY,
     DATA_ZHA,
-    DATA_ZHA_BRIDGE_ID,
-    DATA_ZHA_GATEWAY,
     DEBUG_COMP_BELLOWS,
     DEBUG_COMP_ZHA,
     DEBUG_COMP_ZIGPY,
@@ -90,17 +86,16 @@ from .const import (
 )
 from .device import DeviceStatus, ZHADevice
 from .group import GroupMember, ZHAGroup
+from .helpers import get_zha_data
 from .registries import GROUP_ENTITY_DOMAINS
-from .store import async_get_registry
 
 if TYPE_CHECKING:
     from logging import Filter, LogRecord
 
     from ..entity import ZhaEntity
-    from .channels.base import ZigbeeChannel
-    from .store import ZhaStorage
+    from .cluster_handlers import ClusterHandler
 
-    _LogFilterType = Union[Filter, Callable[[LogRecord], int]]
+    _LogFilterType = Filter | Callable[[LogRecord], bool]
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -110,7 +105,7 @@ class EntityReference(NamedTuple):
 
     reference_id: str
     zha_device: ZHADevice
-    cluster_channels: dict[str, ZigbeeChannel]
+    cluster_handlers: dict[str, ClusterHandler]
     device_info: DeviceInfo
     remove_future: asyncio.Future[Any]
 
@@ -127,22 +122,16 @@ class DevicePairingStatus(Enum):
 class ZHAGateway:
     """Gateway that handles events that happen on the ZHA Zigbee network."""
 
-    # -- Set in async_initialize --
-    zha_storage: ZhaStorage
-    ha_device_registry: DeviceRegistry
-    ha_entity_registry: EntityRegistry
-    application_controller: ControllerApplication
-    radio_description: str
-
     def __init__(
         self, hass: HomeAssistant, config: ConfigType, config_entry: ConfigEntry
     ) -> None:
         """Initialize the gateway."""
-        self._hass = hass
+        self.hass = hass
         self._config = config
         self._devices: dict[EUI64, ZHADevice] = {}
         self._groups: dict[int, ZHAGroup] = {}
-        self.coordinator_zha_device: ZHADevice | None = None
+        self.application_controller: ControllerApplication = None
+        self.coordinator_zha_device: ZHADevice = None  # type: ignore[assignment]
         self._device_registry: collections.defaultdict[
             EUI64, list[EntityReference]
         ] = collections.defaultdict(list)
@@ -155,112 +144,204 @@ class ZHAGateway:
         self.config_entry = config_entry
         self._unsubs: list[Callable[[], None]] = []
 
-    async def async_initialize(self) -> None:
-        """Initialize controller and connect radio."""
-        discovery.PROBE.initialize(self._hass)
-        discovery.GROUP_PROBE.initialize(self._hass)
+        self.shutting_down = False
+        self._reload_task: asyncio.Task | None = None
 
-        self.zha_storage = await async_get_registry(self._hass)
-        self.ha_device_registry = await get_dev_reg(self._hass)
-        self.ha_entity_registry = await get_ent_reg(self._hass)
-
-        radio_type = self.config_entry.data[CONF_RADIO_TYPE]
-
-        app_controller_cls = RadioType[radio_type].controller
-        self.radio_description = RadioType[radio_type].description
+    def get_application_controller_data(self) -> tuple[ControllerApplication, dict]:
+        """Get an uninitialized instance of a zigpy `ControllerApplication`."""
+        radio_type = RadioType[self.config_entry.data[CONF_RADIO_TYPE]]
 
         app_config = self._config.get(CONF_ZIGPY, {})
         database = self._config.get(
             CONF_DATABASE,
-            os.path.join(self._hass.config.config_dir, DEFAULT_DATABASE_NAME),
+            self.hass.config.path(DEFAULT_DATABASE_NAME),
         )
         app_config[CONF_DATABASE] = database
         app_config[CONF_DEVICE] = self.config_entry.data[CONF_DEVICE]
 
-        app_config = app_controller_cls.SCHEMA(app_config)
+        if CONF_NWK_VALIDATE_SETTINGS not in app_config:
+            app_config[CONF_NWK_VALIDATE_SETTINGS] = True
+
+        # The bellows UART thread sometimes propagates a cancellation into the main Core
+        # event loop, when a connection to a TCP coordinator fails in a specific way
+        if (
+            CONF_USE_THREAD not in app_config
+            and radio_type is RadioType.ezsp
+            and app_config[CONF_DEVICE][CONF_DEVICE_PATH].startswith("socket://")
+        ):
+            app_config[CONF_USE_THREAD] = False
+
+        # Local import to avoid circular dependencies
+        # pylint: disable-next=import-outside-toplevel
+        from homeassistant.components.homeassistant_hardware.silabs_multiprotocol_addon import (
+            is_multiprotocol_url,
+        )
+
+        # Until we have a way to coordinate channels with the Thread half of multi-PAN,
+        # stick to the old zigpy default of channel 15 instead of dynamically scanning
+        if (
+            is_multiprotocol_url(app_config[CONF_DEVICE][CONF_DEVICE_PATH])
+            and app_config.get(CONF_NWK, {}).get(CONF_NWK_CHANNEL) is None
+        ):
+            app_config.setdefault(CONF_NWK, {})[CONF_NWK_CHANNEL] = 15
+
+        return radio_type.controller, radio_type.controller.SCHEMA(app_config)
+
+    @classmethod
+    async def async_from_config(
+        cls, hass: HomeAssistant, config: ConfigType, config_entry: ConfigEntry
+    ) -> Self:
+        """Create an instance of a gateway from config objects."""
+        instance = cls(hass, config, config_entry)
+        await instance.async_initialize()
+        return instance
+
+    async def async_initialize(self) -> None:
+        """Initialize controller and connect radio."""
+        discovery.PROBE.initialize(self.hass)
+        discovery.GROUP_PROBE.initialize(self.hass)
+
+        self.shutting_down = False
+
+        app_controller_cls, app_config = self.get_application_controller_data()
+        app = await app_controller_cls.new(
+            config=app_config,
+            auto_form=False,
+            start_radio=False,
+        )
+
         try:
-            self.application_controller = await app_controller_cls.new(
-                app_config, auto_form=True, start_radio=True
-            )
-        except (asyncio.TimeoutError, SerialException, OSError) as exception:
-            _LOGGER.error(
-                "Couldn't start %s coordinator",
-                self.radio_description,
-                exc_info=exception,
-            )
-            raise ConfigEntryNotReady from exception
+            await app.startup(auto_form=True)
+        except Exception:
+            # Explicitly shut down the controller application on failure
+            await app.shutdown()
+            raise
+
+        self.application_controller = app
+
+        zha_data = get_zha_data(self.hass)
+        zha_data.gateway = self
+
+        self.coordinator_zha_device = self._async_get_or_create_device(
+            self._find_coordinator_device()
+        )
+
+        self.async_load_devices()
+        self.async_load_groups()
 
         self.application_controller.add_listener(self)
         self.application_controller.groups.add_listener(self)
-        self._hass.data[DATA_ZHA][DATA_ZHA_GATEWAY] = self
-        self._hass.data[DATA_ZHA][DATA_ZHA_BRIDGE_ID] = str(
-            self.application_controller.ieee
+
+    def connection_lost(self, exc: Exception) -> None:
+        """Handle connection lost event."""
+        _LOGGER.debug("Connection to the radio was lost: %r", exc)
+
+        if self.shutting_down:
+            return
+
+        # Ensure we do not queue up multiple resets
+        if self._reload_task is not None:
+            _LOGGER.debug("Ignoring reset, one is already running")
+            return
+
+        self._reload_task = self.hass.async_create_task(
+            self.hass.config_entries.async_reload(self.config_entry.entry_id)
         )
-        self.async_load_devices()
-        self.async_load_groups()
+
+    def _find_coordinator_device(self) -> zigpy.device.Device:
+        zigpy_coordinator = self.application_controller.get_device(nwk=0x0000)
+
+        if last_backup := self.application_controller.backups.most_recent_backup():
+            with suppress(KeyError):
+                zigpy_coordinator = self.application_controller.get_device(
+                    ieee=last_backup.node_info.ieee
+                )
+
+        return zigpy_coordinator
 
     @callback
     def async_load_devices(self) -> None:
         """Restore ZHA devices from zigpy application state."""
+
         for zigpy_device in self.application_controller.devices.values():
-            zha_device = self._async_get_or_create_device(zigpy_device, restored=True)
-            if zha_device.ieee == self.application_controller.ieee:
-                self.coordinator_zha_device = zha_device
-            zha_dev_entry = self.zha_storage.devices.get(str(zigpy_device.ieee))
+            zha_device = self._async_get_or_create_device(zigpy_device)
             delta_msg = "not known"
-            if zha_dev_entry and zha_dev_entry.last_seen is not None:
-                delta = round(time.time() - zha_dev_entry.last_seen)
-                zha_device.available = delta < zha_device.consider_unavailable_time
+            if zha_device.last_seen is not None:
+                delta = round(time.time() - zha_device.last_seen)
                 delta_msg = f"{str(timedelta(seconds=delta))} ago"
             _LOGGER.debug(
-                "[%s](%s) restored as '%s', last seen: %s, consider_unavailable_time: %s seconds",
+                (
+                    "[%s](%s) restored as '%s', last seen: %s,"
+                    " consider_unavailable_time: %s seconds"
+                ),
                 zha_device.nwk,
                 zha_device.name,
                 "available" if zha_device.available else "unavailable",
                 delta_msg,
                 zha_device.consider_unavailable_time,
             )
-        # update the last seen time for devices every 10 minutes to avoid thrashing
-        # writes and shutdown issues where storage isn't updated
-        self._unsubs.append(
-            async_track_time_interval(
-                self._hass, self.async_update_device_storage, timedelta(minutes=10)
-            )
-        )
 
     @callback
     def async_load_groups(self) -> None:
         """Initialize ZHA groups."""
+
         for group_id in self.application_controller.groups:
             group = self.application_controller.groups[group_id]
             zha_group = self._async_get_or_create_group(group)
-            # we can do this here because the entities are in the entity registry tied to the devices
+            # we can do this here because the entities are in the
+            # entity registry tied to the devices
             discovery.GROUP_PROBE.discover_group_entities(zha_group)
 
+    @property
+    def radio_concurrency(self) -> int:
+        """Maximum configured radio concurrency."""
+        return self.application_controller._concurrent_requests_semaphore.max_value  # pylint: disable=protected-access
+
+    async def async_fetch_updated_state_mains(self) -> None:
+        """Fetch updated state for mains powered devices."""
+        _LOGGER.debug("Fetching current state for mains powered devices")
+
+        now = time.time()
+
+        # Only delay startup to poll mains-powered devices that are online
+        online_devices = [
+            dev
+            for dev in self.devices.values()
+            if dev.is_mains_powered
+            and dev.last_seen is not None
+            and (now - dev.last_seen) < dev.consider_unavailable_time
+        ]
+
+        # Prioritize devices that have recently been contacted
+        online_devices.sort(key=lambda dev: cast(float, dev.last_seen), reverse=True)
+
+        # Make sure that we always leave slots for non-startup requests
+        max_poll_concurrency = max(1, self.radio_concurrency - 4)
+
+        await gather_with_limited_concurrency(
+            max_poll_concurrency,
+            *(dev.async_initialize(from_cache=False) for dev in online_devices),
+        )
+
+        _LOGGER.debug("completed fetching current state for mains powered devices")
+
     async def async_initialize_devices_and_entities(self) -> None:
         """Initialize devices and load entities."""
-        semaphore = asyncio.Semaphore(2)
 
-        async def _throttle(zha_device: ZHADevice, cached: bool) -> None:
-            async with semaphore:
-                await zha_device.async_initialize(from_cache=cached)
-
-        _LOGGER.debug("Loading battery powered devices")
+        _LOGGER.debug("Initializing all devices from Zigpy cache")
         await asyncio.gather(
-            *(
-                _throttle(dev, cached=True)
-                for dev in self.devices.values()
-                if not dev.is_mains_powered
-            )
+            *(dev.async_initialize(from_cache=True) for dev in self.devices.values())
         )
 
-        _LOGGER.debug("Loading mains powered devices")
-        await asyncio.gather(
-            *(
-                _throttle(dev, cached=False)
-                for dev in self.devices.values()
-                if dev.is_mains_powered
-            )
+        async def fetch_updated_state() -> None:
+            """Fetch updated state for mains powered devices."""
+            await self.async_fetch_updated_state_mains()
+            _LOGGER.debug("Allowing polled requests")
+            self.hass.data[DATA_ZHA].allow_polling = True
+
+        # background the fetching of state for mains powered devices
+        self.config_entry.async_create_background_task(
+            self.hass, fetch_updated_state(), "zha.gateway-fetch_updated_state"
         )
 
     def device_joined(self, device: zigpy.device.Device) -> None:
@@ -270,7 +351,7 @@ class ZHAGateway:
         address
         """
         async_dispatcher_send(
-            self._hass,
+            self.hass,
             ZHA_GW_MSG,
             {
                 ATTR_TYPE: ZHA_GW_MSG_DEVICE_JOINED,
@@ -286,7 +367,7 @@ class ZHAGateway:
         """Handle a device initialization without quirks loaded."""
         manuf = device.manufacturer
         async_dispatcher_send(
-            self._hass,
+            self.hass,
             ZHA_GW_MSG,
             {
                 ATTR_TYPE: ZHA_GW_MSG_RAW_INIT,
@@ -303,7 +384,7 @@ class ZHAGateway:
 
     def device_initialized(self, device: zigpy.device.Device) -> None:
         """Handle device joined and basic information discovered."""
-        self._hass.async_create_task(self.async_device_initialized(device))
+        self.hass.async_create_task(self.async_device_initialized(device))
 
     def device_left(self, device: zigpy.device.Device) -> None:
         """Handle device leaving the network."""
@@ -318,7 +399,7 @@ class ZHAGateway:
         zha_group.info("group_member_removed - endpoint: %s", endpoint)
         self._send_group_gateway_message(zigpy_group, ZHA_GW_MSG_GROUP_MEMBER_REMOVED)
         async_dispatcher_send(
-            self._hass, f"{SIGNAL_GROUP_MEMBERSHIP_CHANGE}_0x{zigpy_group.group_id:04x}"
+            self.hass, f"{SIGNAL_GROUP_MEMBERSHIP_CHANGE}_0x{zigpy_group.group_id:04x}"
         )
 
     def group_member_added(
@@ -330,10 +411,11 @@ class ZHAGateway:
         zha_group.info("group_member_added - endpoint: %s", endpoint)
         self._send_group_gateway_message(zigpy_group, ZHA_GW_MSG_GROUP_MEMBER_ADDED)
         async_dispatcher_send(
-            self._hass, f"{SIGNAL_GROUP_MEMBERSHIP_CHANGE}_0x{zigpy_group.group_id:04x}"
+            self.hass, f"{SIGNAL_GROUP_MEMBERSHIP_CHANGE}_0x{zigpy_group.group_id:04x}"
         )
         if len(zha_group.members) == 2:
-            # we need to do this because there wasn't already a group entity to remove and re-add
+            # we need to do this because there wasn't already
+            # a group entity to remove and re-add
             discovery.GROUP_PROBE.discover_group_entities(zha_group)
 
     def group_added(self, zigpy_group: zigpy.group.Group) -> None:
@@ -346,7 +428,7 @@ class ZHAGateway:
     def group_removed(self, zigpy_group: zigpy.group.Group) -> None:
         """Handle zigpy group removed event."""
         self._send_group_gateway_message(zigpy_group, ZHA_GW_MSG_GROUP_REMOVED)
-        zha_group = self._groups.pop(zigpy_group.group_id, None)
+        zha_group = self._groups.pop(zigpy_group.group_id)
         zha_group.info("group_removed")
         self._cleanup_group_entity_registry_entries(zigpy_group)
 
@@ -357,7 +439,7 @@ class ZHAGateway:
         zha_group = self._groups.get(zigpy_group.group_id)
         if zha_group is not None:
             async_dispatcher_send(
-                self._hass,
+                self.hass,
                 ZHA_GW_MSG,
                 {
                     ATTR_TYPE: gateway_message_type,
@@ -374,9 +456,11 @@ class ZHAGateway:
                 remove_tasks.append(entity_ref.remove_future)
             if remove_tasks:
                 await asyncio.wait(remove_tasks)
-        reg_device = self.ha_device_registry.async_get(device.device_id)
+
+        device_registry = dr.async_get(self.hass)
+        reg_device = device_registry.async_get(device.device_id)
         if reg_device is not None:
-            self.ha_device_registry.async_remove_device(reg_device.id)
+            device_registry.async_remove_device(reg_device.id)
 
     def device_removed(self, device: zigpy.device.Device) -> None:
         """Handle device being removed from the network."""
@@ -385,11 +469,14 @@ class ZHAGateway:
         if zha_device is not None:
             device_info = zha_device.zha_device_info
             zha_device.async_cleanup_handles()
-            async_dispatcher_send(self._hass, f"{SIGNAL_REMOVE}_{str(zha_device.ieee)}")
-            asyncio.ensure_future(self._async_remove_device(zha_device, entity_refs))
+            async_dispatcher_send(self.hass, f"{SIGNAL_REMOVE}_{str(zha_device.ieee)}")
+            self.hass.async_create_task(
+                self._async_remove_device(zha_device, entity_refs),
+                "ZHAGateway._async_remove_device",
+            )
             if device_info is not None:
                 async_dispatcher_send(
-                    self._hass,
+                    self.hass,
                     ZHA_GW_MSG,
                     {
                         ATTR_TYPE: ZHA_GW_MSG_DEVICE_REMOVED,
@@ -427,7 +514,9 @@ class ZHAGateway:
         if entity.zha_device.ieee in self.device_registry:
             entity_refs = self.device_registry.get(entity.zha_device.ieee)
             self.device_registry[entity.zha_device.ieee] = [
-                e for e in entity_refs if e.reference_id != entity.entity_id  # type: ignore[union-attr]
+                e
+                for e in entity_refs  # type: ignore[union-attr]
+                if e.reference_id != entity.entity_id
             ]
 
     def _cleanup_group_entity_registry_entries(
@@ -441,13 +530,16 @@ class ZHAGateway:
         ]
 
         # then we get all group entity entries tied to the coordinator
-        all_group_entity_entries = async_entries_for_device(
-            self.ha_entity_registry,
+        entity_registry = er.async_get(self.hass)
+        assert self.coordinator_zha_device
+        all_group_entity_entries = er.async_entries_for_device(
+            entity_registry,
             self.coordinator_zha_device.device_id,
             include_disabled_entities=True,
         )
 
-        # then we get the entity entries for this specific group by getting the entries that match
+        # then we get the entity entries for this specific group
+        # by getting the entries that match
         entries_to_remove = [
             entry
             for entry in all_group_entity_entries
@@ -459,7 +551,12 @@ class ZHAGateway:
             _LOGGER.debug(
                 "cleaning up entity registry entry for entity: %s", entry.entity_id
             )
-            self.ha_entity_registry.async_remove(entry.entity_id)
+            entity_registry.async_remove(entry.entity_id)
+
+    @property
+    def state(self) -> State:
+        """Return the active coordinator's network state."""
+        return self.application_controller.state
 
     @property
     def devices(self) -> dict[EUI64, ZHADevice]:
@@ -481,7 +578,7 @@ class ZHAGateway:
         ieee: EUI64,
         reference_id: str,
         zha_device: ZHADevice,
-        cluster_channels: dict[str, ZigbeeChannel],
+        cluster_handlers: dict[str, ClusterHandler],
         device_info: DeviceInfo,
         remove_future: asyncio.Future[Any],
     ):
@@ -490,7 +587,7 @@ class ZHAGateway:
             EntityReference(
                 reference_id=reference_id,
                 zha_device=zha_device,
-                cluster_channels=cluster_channels,
+                cluster_handlers=cluster_handlers,
                 device_info=device_info,
                 remove_future=remove_future,
             )
@@ -524,23 +621,23 @@ class ZHAGateway:
 
     @callback
     def _async_get_or_create_device(
-        self, zigpy_device: zigpy.device.Device, restored: bool = False
+        self, zigpy_device: zigpy.device.Device
     ) -> ZHADevice:
         """Get or create a ZHA device."""
         if (zha_device := self._devices.get(zigpy_device.ieee)) is None:
-            zha_device = ZHADevice.new(self._hass, zigpy_device, self, restored)
+            zha_device = ZHADevice.new(self.hass, zigpy_device, self)
             self._devices[zigpy_device.ieee] = zha_device
-            device_registry_device = self.ha_device_registry.async_get_or_create(
+
+            device_registry = dr.async_get(self.hass)
+            device_registry_device = device_registry.async_get_or_create(
                 config_entry_id=self.config_entry.entry_id,
-                connections={(CONNECTION_ZIGBEE, str(zha_device.ieee))},
+                connections={(dr.CONNECTION_ZIGBEE, str(zha_device.ieee))},
                 identifiers={(DOMAIN, str(zha_device.ieee))},
                 name=zha_device.name,
                 manufacturer=zha_device.manufacturer,
                 model=zha_device.model,
             )
             zha_device.set_device_id(device_registry_device.id)
-        entry = self.zha_storage.async_get_or_create_device(zha_device)
-        zha_device.async_update_last_seen(entry.last_seen)
         return zha_device
 
     @callback
@@ -548,7 +645,7 @@ class ZHAGateway:
         """Get or create a ZHA group."""
         zha_group = self._groups.get(zigpy_group.group_id)
         if zha_group is None:
-            zha_group = ZHAGroup(self._hass, self, zigpy_group)
+            zha_group = ZHAGroup(self.hass, self, zigpy_group)
             self._groups[zigpy_group.group_id] = zha_group
         return zha_group
 
@@ -563,17 +660,9 @@ class ZHAGateway:
             if device.status is DeviceStatus.INITIALIZED:
                 device.update_available(available)
 
-    async def async_update_device_storage(self, *_: Any) -> None:
-        """Update the devices in the store."""
-        for device in self.devices.values():
-            self.zha_storage.async_update_device(device)
-
     async def async_device_initialized(self, device: zigpy.device.Device) -> None:
         """Handle device joined and basic information discovered (async)."""
         zha_device = self._async_get_or_create_device(device)
-        # This is an active device so set a last seen if it is none
-        if zha_device.last_seen is None:
-            zha_device.async_update_last_seen(time.time())
         _LOGGER.debug(
             "device - %s:%s entering async_device_initialized - is_new_join: %s",
             device.nwk,
@@ -601,7 +690,7 @@ class ZHAGateway:
         device_info = zha_device.zha_device_info
         device_info[DEVICE_PAIRING_STATUS] = DevicePairingStatus.INITIALIZED.name
         async_dispatcher_send(
-            self._hass,
+            self.hass,
             ZHA_GW_MSG,
             {
                 ATTR_TYPE: ZHA_GW_MSG_DEVICE_FULL_INIT,
@@ -615,7 +704,7 @@ class ZHAGateway:
         await zha_device.async_configure()
         device_info[DEVICE_PAIRING_STATUS] = DevicePairingStatus.CONFIGURED.name
         async_dispatcher_send(
-            self._hass,
+            self.hass,
             ZHA_GW_MSG,
             {
                 ATTR_TYPE: ZHA_GW_MSG_DEVICE_FULL_INIT,
@@ -623,7 +712,7 @@ class ZHAGateway:
             },
         )
         await zha_device.async_initialize(from_cache=False)
-        async_dispatcher_send(self._hass, SIGNAL_ADD_ENTITIES)
+        async_dispatcher_send(self.hass, SIGNAL_ADD_ENTITIES)
 
     async def _async_device_rejoined(self, zha_device: ZHADevice) -> None:
         _LOGGER.debug(
@@ -631,12 +720,13 @@ class ZHAGateway:
             zha_device.nwk,
             zha_device.ieee,
         )
-        # we don't have to do this on a nwk swap but we don't have a way to tell currently
+        # we don't have to do this on a nwk swap
+        # but we don't have a way to tell currently
         await zha_device.async_configure()
         device_info = zha_device.device_info
         device_info[DEVICE_PAIRING_STATUS] = DevicePairingStatus.CONFIGURED.name
         async_dispatcher_send(
-            self._hass,
+            self.hass,
             ZHA_GW_MSG,
             {
                 ATTR_TYPE: ZHA_GW_MSG_DEVICE_FULL_INIT,
@@ -654,6 +744,7 @@ class ZHAGateway:
         group_id: int | None = None,
     ) -> ZHAGroup | None:
         """Create a new Zigpy Zigbee group."""
+
         # we start with two to fill any gaps from a user removing existing groups
 
         if group_id is None:
@@ -668,7 +759,10 @@ class ZHAGateway:
                 tasks = []
                 for member in members:
                     _LOGGER.debug(
-                        "Adding member with IEEE: %s and endpoint ID: %s to group: %s:0x%04x",
+                        (
+                            "Adding member with IEEE: %s and endpoint ID: %s to group:"
+                            " %s:0x%04x"
+                        ),
                         member.ieee,
                         member.endpoint_id,
                         name,
@@ -685,7 +779,7 @@ class ZHAGateway:
     async def async_remove_zigpy_group(self, group_id: int) -> None:
         """Remove a Zigbee group from Zigpy."""
         if not (group := self.groups.get(group_id)):
-            _LOGGER.debug("Group: %s:0x%04x could not be found", group.name, group_id)
+            _LOGGER.debug("Group: 0x%04x could not be found", group_id)
             return
         if group.members:
             tasks = []
@@ -697,10 +791,18 @@ class ZHAGateway:
 
     async def shutdown(self) -> None:
         """Stop ZHA Controller Application."""
+        if self.shutting_down:
+            _LOGGER.debug("Ignoring duplicate shutdown event")
+            return
+
         _LOGGER.debug("Shutting down ZHA ControllerApplication")
+        self.shutting_down = True
+
         for unsubscribe in self._unsubs:
             unsubscribe()
-        await self.application_controller.pre_shutdown()
+        for device in self.devices.values():
+            device.async_cleanup_handles()
+        await self.application_controller.shutdown()
 
     def handle_message(
         self,
@@ -758,14 +860,20 @@ class LogRelayHandler(logging.Handler):
         super().__init__()
         self.hass = hass
         self.gateway = gateway
+        hass_path: str = HOMEASSISTANT_PATH[0]
+        config_dir = self.hass.config.config_dir
+        self.paths_re = re.compile(
+            r"(?:{})/(.*)".format(
+                "|".join([re.escape(x) for x in (hass_path, config_dir)])
+            )
+        )
 
     def emit(self, record: LogRecord) -> None:
         """Relay log message via dispatcher."""
-        stack = []
-        if record.levelno >= logging.WARN and not record.exc_info:
-            stack = [f for f, _, _, _ in traceback.extract_stack()]
-
-        entry = LogEntry(record, stack, _figure_out_source(record, stack, self.hass))
+        if record.levelno >= logging.WARN:
+            entry = LogEntry(record, _figure_out_source(record, self.paths_re))
+        else:
+            entry = LogEntry(record, (record.pathname, record.lineno))
         async_dispatcher_send(
             self.hass,
             ZHA_GW_MSG,
