@@ -1,6 +1,12 @@
 """Constants for the Z-Wave JS integration."""
+from __future__ import annotations
+
 import logging
 
+from zwave_js_server.const.command_class.window_covering import (
+    WindowCoveringPropertyKey,
+)
+
 from homeassistant.const import APPLICATION_NAME, __version__ as HA_VERSION
 
 USER_AGENT = {APPLICATION_NAME: HA_VERSION}
@@ -25,10 +31,15 @@ CONF_DATA_COLLECTION_OPTED_IN = "data_collection_opted_in"
 DOMAIN = "zwave_js"
 
 DATA_CLIENT = "client"
+DATA_OLD_SERVER_LOG_LEVEL = "old_server_log_level"
 
 EVENT_DEVICE_ADDED_TO_REGISTRY = f"{DOMAIN}_device_added_to_registry"
 
 LOGGER = logging.getLogger(__package__)
+LIB_LOGGER = logging.getLogger("zwave_js_server")
+
+# constants extra state attributes
+ATTR_RESERVED_VALUES = "reserved_values"  # ConfigurationValue number entities
 
 # constants for events
 ZWAVE_JS_VALUE_NOTIFICATION_EVENT = f"{DOMAIN}_value_notification"
@@ -61,6 +72,8 @@ ATTR_STATUS = "status"
 ATTR_ACKNOWLEDGED_FRAMES = "acknowledged_frames"
 ATTR_EVENT_TYPE_LABEL = "event_type_label"
 ATTR_DATA_TYPE_LABEL = "data_type_label"
+ATTR_NOTIFICATION_TYPE = "notification_type"
+ATTR_NOTIFICATION_EVENT = "notification_event"
 
 ATTR_NODE = "node"
 ATTR_ZWAVE_VALUE = "zwave_value"
@@ -81,10 +94,12 @@ SERVICE_CLEAR_LOCK_USERCODE = "clear_lock_usercode"
 SERVICE_INVOKE_CC_API = "invoke_cc_api"
 SERVICE_MULTICAST_SET_VALUE = "multicast_set_value"
 SERVICE_PING = "ping"
+SERVICE_REFRESH_NOTIFICATIONS = "refresh_notifications"
 SERVICE_REFRESH_VALUE = "refresh_value"
 SERVICE_RESET_METER = "reset_meter"
 SERVICE_SET_CONFIG_PARAMETER = "set_config_parameter"
 SERVICE_SET_LOCK_USERCODE = "set_lock_usercode"
+SERVICE_SET_LOCK_CONFIGURATION = "set_lock_configuration"
 SERVICE_SET_VALUE = "set_value"
 
 ATTR_NODES = "nodes"
@@ -92,6 +107,8 @@ ATTR_NODES = "nodes"
 ATTR_CONFIG_PARAMETER = "parameter"
 ATTR_CONFIG_PARAMETER_BITMASK = "bitmask"
 ATTR_CONFIG_VALUE = "value"
+ATTR_VALUE_SIZE = "value_size"
+ATTR_VALUE_FORMAT = "value_format"
 # refresh value
 ATTR_REFRESH_ALL_VALUES = "refresh_all_values"
 # multicast
@@ -102,6 +119,13 @@ ATTR_METER_TYPE_NAME = "meter_type_name"
 # invoke CC API
 ATTR_METHOD_NAME = "method_name"
 ATTR_PARAMETERS = "parameters"
+# lock set configuration
+ATTR_AUTO_RELOCK_TIME = "auto_relock_time"
+ATTR_BLOCK_TO_BLOCK = "block_to_block"
+ATTR_HOLD_AND_RELEASE_TIME = "hold_and_release_time"
+ATTR_LOCK_TIMEOUT = "lock_timeout"
+ATTR_OPERATION_TYPE = "operation_type"
+ATTR_TWIST_ASSIST = "twist_assist"
 
 ADDON_SLUG = "core_zwave_js"
 
@@ -121,11 +145,49 @@ ENTITY_DESC_KEY_PRESSURE = "pressure"
 ENTITY_DESC_KEY_SIGNAL_STRENGTH = "signal_strength"
 ENTITY_DESC_KEY_TEMPERATURE = "temperature"
 ENTITY_DESC_KEY_TARGET_TEMPERATURE = "target_temperature"
+ENTITY_DESC_KEY_UV_INDEX = "uv_index"
 ENTITY_DESC_KEY_MEASUREMENT = "measurement"
 ENTITY_DESC_KEY_TOTAL_INCREASING = "total_increasing"
 
+ENTITY_DESC_KEY_ENERGY_PRODUCTION_POWER = "energy_production_power"
+ENTITY_DESC_KEY_ENERGY_PRODUCTION_TIME = "energy_production_time"
+ENTITY_DESC_KEY_ENERGY_PRODUCTION_TOTAL = "energy_production_total"
+ENTITY_DESC_KEY_ENERGY_PRODUCTION_TODAY = "energy_production_today"
+
 # This API key is only for use with Home Assistant. Reach out to Z-Wave JS to apply for
 # your own (https://github.com/zwave-js/firmware-updates/).
 API_KEY_FIRMWARE_UPDATE_SERVICE = (
     "2e39d98fc56386389fbb35e5a98fa1b44b9fdd8f971460303587cff408430d4cfcde6134"
 )
+
+# Platform constants
+# cover
+COVER_POSITION_PROPERTY_KEYS: set[str | int | None] = {
+    WindowCoveringPropertyKey.INBOUND_BOTTOM,
+    WindowCoveringPropertyKey.INBOUND_BOTTOM_NO_POSITION,
+    WindowCoveringPropertyKey.INBOUND_LEFT,
+    WindowCoveringPropertyKey.INBOUND_LEFT_NO_POSITION,
+    WindowCoveringPropertyKey.INBOUND_LEFT_RIGHT,
+    WindowCoveringPropertyKey.INBOUND_LEFT_RIGHT_NO_POSITION,
+    WindowCoveringPropertyKey.INBOUND_RIGHT,
+    WindowCoveringPropertyKey.INBOUND_RIGHT_NO_POSITION,
+    WindowCoveringPropertyKey.INBOUND_TOP,
+    WindowCoveringPropertyKey.INBOUND_TOP_NO_POSITION,
+    WindowCoveringPropertyKey.INBOUND_TOP_BOTTOM,
+    WindowCoveringPropertyKey.INBOUND_TOP_BOTTOM_NO_POSITION,
+    WindowCoveringPropertyKey.OUTBOUND_BOTTOM,
+    WindowCoveringPropertyKey.OUTBOUND_BOTTOM_NO_POSITION,
+    WindowCoveringPropertyKey.OUTBOUND_LEFT,
+    WindowCoveringPropertyKey.OUTBOUND_LEFT_NO_POSITION,
+    WindowCoveringPropertyKey.OUTBOUND_RIGHT,
+    WindowCoveringPropertyKey.OUTBOUND_RIGHT_NO_POSITION,
+    WindowCoveringPropertyKey.OUTBOUND_TOP,
+    WindowCoveringPropertyKey.OUTBOUND_TOP_NO_POSITION,
+}
+
+COVER_TILT_PROPERTY_KEYS: set[str | int | None] = {
+    WindowCoveringPropertyKey.HORIZONTAL_SLATS_ANGLE,
+    WindowCoveringPropertyKey.HORIZONTAL_SLATS_ANGLE_NO_POSITION,
+    WindowCoveringPropertyKey.VERTICAL_SLATS_ANGLE,
+    WindowCoveringPropertyKey.VERTICAL_SLATS_ANGLE_NO_POSITION,
+}
