@@ -16,25 +16,26 @@ from homeassistant.exceptions import HomeAssistantError
 from homeassistant.helpers import condition, config_validation as cv
 from homeassistant.helpers.typing import ConfigType, TemplateVarsType
 
-from . import DOMAIN
+from .config_validation import VALUE_SCHEMA
 from .const import (
     ATTR_COMMAND_CLASS,
     ATTR_ENDPOINT,
     ATTR_PROPERTY,
     ATTR_PROPERTY_KEY,
     ATTR_VALUE,
-    VALUE_SCHEMA,
+    DOMAIN,
 )
 from .device_automation_helpers import (
     CONF_SUBTYPE,
     CONF_VALUE_ID,
     NODE_STATUSES,
-    get_config_parameter_value_schema,
+    async_bypass_dynamic_config_validation,
+    generate_config_parameter_subtype,
 )
 from .helpers import (
     async_get_node_from_device_id,
-    async_is_device_config_entry_not_loaded,
     check_type_schema_map,
+    get_value_state_schema,
     get_zwave_value_from_config,
     remove_keys_with_empty_values,
 )
@@ -99,7 +100,16 @@ async def async_validate_condition_config(
 
     # We return early if the config entry for this device is not ready because we can't
     # validate the value without knowing the state of the device
-    if async_is_device_config_entry_not_loaded(hass, config[CONF_DEVICE_ID]):
+    try:
+        bypass_dynamic_config_validation = async_bypass_dynamic_config_validation(
+            hass, config[CONF_DEVICE_ID]
+        )
+    except ValueError as err:
+        raise InvalidDeviceAutomationConfig(
+            f"Device {config[CONF_DEVICE_ID]} not found"
+        ) from err
+
+    if bypass_dynamic_config_validation:
         return config
 
     if config[CONF_TYPE] == VALUE_TYPE:
@@ -116,7 +126,7 @@ async def async_get_conditions(
     hass: HomeAssistant, device_id: str
 ) -> list[dict[str, str]]:
     """List device conditions for Z-Wave JS devices."""
-    conditions = []
+    conditions: list[dict] = []
     base_condition = {
         CONF_CONDITION: "device",
         CONF_DEVICE_ID: device_id,
@@ -124,6 +134,9 @@ async def async_get_conditions(
     }
     node = async_get_node_from_device_id(hass, device_id)
 
+    if node.client.driver and node.client.driver.controller.own_node == node:
+        return conditions
+
     # Any value's value condition
     conditions.append({**base_condition, CONF_TYPE: VALUE_TYPE})
 
@@ -137,7 +150,7 @@ async def async_get_conditions(
                 **base_condition,
                 CONF_VALUE_ID: config_value.value_id,
                 CONF_TYPE: CONFIG_PARAMETER_TYPE,
-                CONF_SUBTYPE: f"{config_value.value_id} ({config_value.property_name})",
+                CONF_SUBTYPE: generate_config_parameter_subtype(config_value),
             }
             for config_value in node.get_configuration_values().values()
         ]
@@ -148,12 +161,9 @@ async def async_get_conditions(
 
 @callback
 def async_condition_from_config(
-    config: ConfigType, config_validation: bool
+    hass: HomeAssistant, config: ConfigType
 ) -> condition.ConditionCheckerType:
     """Create a function to test a device condition."""
-    if config_validation:
-        config = CONDITION_SCHEMA(config)
-
     condition_type = config[CONF_TYPE]
     device_id = config[CONF_DEVICE_ID]
 
@@ -189,7 +199,6 @@ def async_condition_from_config(
     raise HomeAssistantError(f"Unhandled condition type {condition_type}")
 
 
-@callback
 async def async_get_condition_capabilities(
     hass: HomeAssistant, config: ConfigType
 ) -> dict[str, vol.Schema]:
@@ -200,7 +209,7 @@ async def async_get_condition_capabilities(
     # Add additional fields to the automation trigger UI
     if config[CONF_TYPE] == CONFIG_PARAMETER_TYPE:
         value_id = config[CONF_VALUE_ID]
-        value_schema = get_config_parameter_value_schema(node, value_id)
+        value_schema = get_value_state_schema(node.values[value_id])
         if value_schema is None:
             return {}
         return {"extra_fields": vol.Schema({vol.Required(ATTR_VALUE): value_schema})}
@@ -214,7 +223,9 @@ async def async_get_condition_capabilities(
                     vol.Required(ATTR_COMMAND_CLASS): vol.In(
                         {
                             CommandClass(cc.id).value: cc.name
-                            for cc in sorted(node.command_classes, key=lambda cc: cc.name)  # type: ignore[no-any-return]
+                            for cc in sorted(
+                                node.command_classes, key=lambda cc: cc.name
+                            )
                             if cc.id != CommandClass.CONFIGURATION
                         }
                     ),
