@@ -3,7 +3,8 @@ from __future__ import annotations
 
 from collections.abc import Generator
 from dataclasses import asdict, dataclass, field
-from typing import Any
+from enum import StrEnum
+from typing import TYPE_CHECKING, Any, cast
 
 from awesomeversion import AwesomeVersion
 from zwave_js_server.const import (
@@ -41,14 +42,17 @@ from zwave_js_server.const.command_class.thermostat import (
 from zwave_js_server.exceptions import UnknownValueData
 from zwave_js_server.model.device_class import DeviceClassItem
 from zwave_js_server.model.node import Node as ZwaveNode
-from zwave_js_server.model.value import Value as ZwaveValue
+from zwave_js_server.model.value import (
+    ConfigurationValue,
+    ConfigurationValueType,
+    Value as ZwaveValue,
+)
 
-from homeassistant.backports.enum import StrEnum
-from homeassistant.const import Platform
+from homeassistant.const import EntityCategory, Platform
 from homeassistant.core import callback
 from homeassistant.helpers.device_registry import DeviceEntry
 
-from .const import LOGGER
+from .const import COVER_POSITION_PROPERTY_KEYS, COVER_TILT_PROPERTY_KEYS, LOGGER
 from .discovery_data_template import (
     BaseDiscoverySchemaDataTemplate,
     ConfigurableFanValueMappingDataTemplate,
@@ -60,6 +64,9 @@ from .discovery_data_template import (
 )
 from .helpers import ZwaveValueID
 
+if TYPE_CHECKING:
+    from _typeshed import DataclassInstance
+
 
 class ValueType(StrEnum):
     """Enum with all value types."""
@@ -73,7 +80,7 @@ class ValueType(StrEnum):
 class DataclassMustHaveAtLeastOne:
     """A dataclass that must have at least one input parameter that is not None."""
 
-    def __post_init__(self) -> None:
+    def __post_init__(self: DataclassInstance) -> None:
         """Post dataclass initialization."""
         if all(val is None for val in asdict(self).values()):
             raise ValueError("At least one input parameter must not be None")
@@ -105,7 +112,8 @@ class ZwaveDiscoveryInfo:
     node: ZwaveNode
     # the value object itself for primary value
     primary_value: ZwaveValue
-    # bool to specify whether state is assumed and events should be fired on value update
+    # bool to specify whether state is assumed and events should be fired on value
+    # update
     assumed_state: bool
     # the home assistant platform for which an entity should be created
     platform: Platform
@@ -119,6 +127,8 @@ class ZwaveDiscoveryInfo:
     platform_data_template: BaseDiscoverySchemaDataTemplate | None = None
     # bool to specify whether entity should be enabled by default
     entity_registry_enabled_default: bool = True
+    # the entity category for the discovered entity
+    entity_category: EntityCategory | None = None
 
 
 @dataclass
@@ -127,7 +137,7 @@ class ZWaveValueDiscoverySchema(DataclassMustHaveAtLeastOne):
 
     The Z-Wave Value must match these conditions.
     Use the Z-Wave specifications to find out the values for these parameters:
-    https://github.com/zwave-js/node-zwave-js/tree/master/specs
+    https://github.com/zwave-js/specs/tree/master
     """
 
     # [optional] the value's command class must match ANY of these values
@@ -140,10 +150,20 @@ class ZWaveValueDiscoverySchema(DataclassMustHaveAtLeastOne):
     property_name: set[str] | None = None
     # [optional] the value's property key must match ANY of these values
     property_key: set[str | int | None] | None = None
-    # [optional] the value's property key name must match ANY of these values
-    property_key_name: set[str | None] | None = None
+    # [optional] the value's property key must NOT match ANY of these values
+    not_property_key: set[str | int | None] | None = None
     # [optional] the value's metadata_type must match ANY of these values
     type: set[str] | None = None
+    # [optional] the value's metadata_readable must match this value
+    readable: bool | None = None
+    # [optional] the value's metadata_writeable must match this value
+    writeable: bool | None = None
+    # [optional] the value's states map must include ANY of these key/value pairs
+    any_available_states: set[tuple[int, str]] | None = None
+    # [optional] the value's value must match this value
+    value: Any | None = None
+    # [optional] the value's metadata_stateful must match this value
+    stateful: bool | None = None
 
 
 @dataclass
@@ -152,7 +172,7 @@ class ZWaveDiscoverySchema:
 
     The Z-Wave node and it's (primary) value for an entity must match these conditions.
     Use the Z-Wave specifications to find out the values for these parameters:
-    https://github.com/zwave-js/node-zwave-js/tree/master/specs
+    https://github.com/zwave-js/specs/tree/master
     """
 
     # specify the hass platform for which this scheme applies (e.g. light, sensor)
@@ -171,53 +191,26 @@ class ZWaveDiscoverySchema:
     product_type: set[int] | None = None
     # [optional] the node's firmware_version must be within this range
     firmware_version_range: FirmwareVersionRange | None = None
-    # [optional] the node's firmware_version must match ANY of these values
-    firmware_version: set[str] | None = None
-    # [optional] the node's basic device class must match ANY of these values
-    device_class_basic: set[str | int] | None = None
     # [optional] the node's generic device class must match ANY of these values
-    device_class_generic: set[str | int] | None = None
+    device_class_generic: set[str] | None = None
     # [optional] the node's specific device class must match ANY of these values
-    device_class_specific: set[str | int] | None = None
-    # [optional] additional values that ALL need to be present on the node for this scheme to pass
+    device_class_specific: set[str] | None = None
+    # [optional] additional values that ALL need to be present
+    # on the node for this scheme to pass
     required_values: list[ZWaveValueDiscoverySchema] | None = None
-    # [optional] additional values that MAY NOT be present on the node for this scheme to pass
+    # [optional] additional values that MAY NOT be present
+    # on the node for this scheme to pass
     absent_values: list[ZWaveValueDiscoverySchema] | None = None
-    # [optional] bool to specify if this primary value may be discovered by multiple platforms
+    # [optional] bool to specify if this primary value may be discovered
+    # by multiple platforms
     allow_multi: bool = False
-    # [optional] bool to specify whether state is assumed and events should be fired on value update
+    # [optional] bool to specify whether state is assumed
+    # and events should be fired on value update
     assumed_state: bool = False
     # [optional] bool to specify whether entity should be enabled by default
     entity_registry_enabled_default: bool = True
-
-
-def get_config_parameter_discovery_schema(
-    property_: set[str | int] | None = None,
-    property_name: set[str] | None = None,
-    property_key: set[str | int | None] | None = None,
-    property_key_name: set[str | None] | None = None,
-    **kwargs: Any,
-) -> ZWaveDiscoverySchema:
-    """
-    Return a discovery schema for a config parameter.
-
-    Supports all keyword arguments to ZWaveValueDiscoverySchema except platform, hint,
-    and primary_value.
-    """
-    return ZWaveDiscoverySchema(
-        platform=Platform.SENSOR,
-        hint="config_parameter",
-        primary_value=ZWaveValueDiscoverySchema(
-            command_class={CommandClass.CONFIGURATION},
-            property=property_,
-            property_name=property_name,
-            property_key=property_key,
-            property_key_name=property_key_name,
-            type={ValueType.NUMBER},
-        ),
-        entity_registry_enabled_default=False,
-        **kwargs,
-    )
+    # [optional] the entity category for the discovered entity
+    entity_category: EntityCategory | None = None
 
 
 DOOR_LOCK_CURRENT_MODE_SCHEMA = ZWaveValueDiscoverySchema(
@@ -232,6 +225,12 @@ SWITCH_MULTILEVEL_CURRENT_VALUE_SCHEMA = ZWaveValueDiscoverySchema(
     type={ValueType.NUMBER},
 )
 
+SWITCH_MULTILEVEL_TARGET_VALUE_SCHEMA = ZWaveValueDiscoverySchema(
+    command_class={CommandClass.SWITCH_MULTILEVEL},
+    property={TARGET_VALUE_PROPERTY},
+    type={ValueType.NUMBER},
+)
+
 SWITCH_BINARY_CURRENT_VALUE_SCHEMA = ZWaveValueDiscoverySchema(
     command_class={CommandClass.SWITCH_BINARY}, property={CURRENT_VALUE_PROPERTY}
 )
@@ -242,6 +241,18 @@ SIREN_TONE_SCHEMA = ZWaveValueDiscoverySchema(
     type={ValueType.NUMBER},
 )
 
+WINDOW_COVERING_COVER_CURRENT_VALUE_SCHEMA = ZWaveValueDiscoverySchema(
+    command_class={CommandClass.WINDOW_COVERING},
+    property={CURRENT_VALUE_PROPERTY},
+    property_key=COVER_POSITION_PROPERTY_KEYS,
+)
+
+WINDOW_COVERING_SLAT_CURRENT_VALUE_SCHEMA = ZWaveValueDiscoverySchema(
+    command_class={CommandClass.WINDOW_COVERING},
+    property={CURRENT_VALUE_PROPERTY},
+    property_key=COVER_TILT_PROPERTY_KEYS,
+)
+
 # For device class mapping see:
 # https://github.com/zwave-js/node-zwave-js/blob/master/packages/config/config/deviceClasses.json
 DISCOVERY_SCHEMAS = [
@@ -249,10 +260,15 @@ DISCOVERY_SCHEMAS = [
     # Honeywell 39358 In-Wall Fan Control using switch multilevel CC
     ZWaveDiscoverySchema(
         platform=Platform.FAN,
+        hint="has_fan_value_mapping",
         manufacturer_id={0x0039},
         product_id={0x3131},
         product_type={0x4944},
         primary_value=SWITCH_MULTILEVEL_CURRENT_VALUE_SCHEMA,
+        required_values=[SWITCH_MULTILEVEL_TARGET_VALUE_SCHEMA],
+        data_template=FixedFanValueMappingDataTemplate(
+            FanValueMapping(speeds=[(1, 32), (33, 66), (67, 99)]),
+        ),
     ),
     # GE/Jasco - In-Wall Smart Fan Control - 12730 / ZW4002
     ZWaveDiscoverySchema(
@@ -266,12 +282,12 @@ DISCOVERY_SCHEMAS = [
             FanValueMapping(speeds=[(1, 33), (34, 67), (68, 99)]),
         ),
     ),
-    # GE/Jasco - In-Wall Smart Fan Control - 14287 / ZW4002
+    # GE/Jasco - In-Wall Smart Fan Control - 14287 / 55258 / ZW4002
     ZWaveDiscoverySchema(
         platform=Platform.FAN,
         hint="has_fan_value_mapping",
         manufacturer_id={0x0063},
-        product_id={0x3131},
+        product_id={0x3131, 0x3337},
         product_type={0x4944},
         primary_value=SWITCH_MULTILEVEL_CURRENT_VALUE_SCHEMA,
         data_template=FixedFanValueMappingDataTemplate(
@@ -339,18 +355,24 @@ DISCOVERY_SCHEMAS = [
     # Fibaro Shutter Fibaro FGR222
     ZWaveDiscoverySchema(
         platform=Platform.COVER,
-        hint="window_shutter_tilt",
+        hint="shutter_tilt",
         manufacturer_id={0x010F},
         product_id={0x1000, 0x1001},
         product_type={0x0301, 0x0302},
         primary_value=SWITCH_MULTILEVEL_CURRENT_VALUE_SCHEMA,
         data_template=CoverTiltDataTemplate(
-            tilt_value_id=ZwaveValueID(
+            current_tilt_value_id=ZwaveValueID(
                 property_="fibaro",
                 command_class=CommandClass.MANUFACTURER_PROPRIETARY,
                 endpoint=0,
                 property_key="venetianBlindsTilt",
-            )
+            ),
+            target_tilt_value_id=ZwaveValueID(
+                property_="fibaro",
+                command_class=CommandClass.MANUFACTURER_PROPRIETARY,
+                endpoint=0,
+                property_key="venetianBlindsTilt",
+            ),
         ),
         required_values=[
             ZWaveValueDiscoverySchema(
@@ -360,10 +382,75 @@ DISCOVERY_SCHEMAS = [
             )
         ],
     ),
+    # Fibaro Shutter Fibaro FGR223
+    # Combine both switch_multilevel endpoints into shutter_tilt
+    # if operating mode (151) is set to venetian blind (2)
+    ZWaveDiscoverySchema(
+        platform=Platform.COVER,
+        hint="shutter_tilt",
+        manufacturer_id={0x010F},
+        product_id={0x1000, 0x1001},
+        product_type={0x0303},
+        primary_value=ZWaveValueDiscoverySchema(
+            command_class={CommandClass.SWITCH_MULTILEVEL},
+            property={CURRENT_VALUE_PROPERTY},
+            endpoint={1},
+            type={ValueType.NUMBER},
+        ),
+        data_template=CoverTiltDataTemplate(
+            current_tilt_value_id=ZwaveValueID(
+                property_=CURRENT_VALUE_PROPERTY,
+                command_class=CommandClass.SWITCH_MULTILEVEL,
+                endpoint=2,
+            ),
+            target_tilt_value_id=ZwaveValueID(
+                property_=TARGET_VALUE_PROPERTY,
+                command_class=CommandClass.SWITCH_MULTILEVEL,
+                endpoint=2,
+            ),
+        ),
+        required_values=[
+            ZWaveValueDiscoverySchema(
+                command_class={CommandClass.CONFIGURATION},
+                property={151},
+                endpoint={0},
+                value={2},
+            )
+        ],
+    ),
+    # Fibaro Shutter Fibaro FGR223
+    # Disable endpoint 2 (slat),
+    # as these are either combined with endpoint one as shutter_tilt
+    # or it has no practical function.
+    # CC: Switch_Multilevel
+    ZWaveDiscoverySchema(
+        platform=Platform.COVER,
+        hint="shutter",
+        manufacturer_id={0x010F},
+        product_id={0x1000, 0x1001},
+        product_type={0x0303},
+        primary_value=ZWaveValueDiscoverySchema(
+            command_class={CommandClass.SWITCH_MULTILEVEL},
+            property={CURRENT_VALUE_PROPERTY},
+            endpoint={2},
+            type={ValueType.NUMBER},
+        ),
+        entity_registry_enabled_default=False,
+    ),
+    # Fibaro Nice BiDi-ZWave (IBT4ZWAVE)
+    ZWaveDiscoverySchema(
+        platform=Platform.COVER,
+        hint="gate",
+        manufacturer_id={0x0441},
+        product_id={0x1000},
+        product_type={0x2400},
+        primary_value=SWITCH_MULTILEVEL_CURRENT_VALUE_SCHEMA,
+        required_values=[SWITCH_MULTILEVEL_TARGET_VALUE_SCHEMA],
+    ),
     # Qubino flush shutter
     ZWaveDiscoverySchema(
         platform=Platform.COVER,
-        hint="window_shutter",
+        hint="shutter",
         manufacturer_id={0x0159},
         product_id={0x0052, 0x0053},
         product_type={0x0003},
@@ -372,7 +459,7 @@ DISCOVERY_SCHEMAS = [
     # Graber/Bali/Spring Fashion Covers
     ZWaveDiscoverySchema(
         platform=Platform.COVER,
-        hint="window_blind",
+        hint="blind",
         manufacturer_id={0x026E},
         product_id={0x5A31},
         product_type={0x4353},
@@ -381,12 +468,59 @@ DISCOVERY_SCHEMAS = [
     # iBlinds v2 window blind motor
     ZWaveDiscoverySchema(
         platform=Platform.COVER,
-        hint="window_blind",
+        hint="blind",
         manufacturer_id={0x0287},
         product_id={0x000D},
         product_type={0x0003},
         primary_value=SWITCH_MULTILEVEL_CURRENT_VALUE_SCHEMA,
     ),
+    # Merten 507801 Connect Roller Shutter
+    ZWaveDiscoverySchema(
+        platform=Platform.COVER,
+        hint="shutter",
+        manufacturer_id={0x007A},
+        product_id={0x0001},
+        product_type={0x8003},
+        primary_value=ZWaveValueDiscoverySchema(
+            command_class={CommandClass.SWITCH_MULTILEVEL},
+            property={CURRENT_VALUE_PROPERTY},
+            endpoint={0, 1},
+            type={ValueType.NUMBER},
+        ),
+        assumed_state=True,
+    ),
+    # Merten 507801 Connect Roller Shutter.
+    # Disable endpoint 2, as it has no practical function. CC: Switch_Multilevel
+    ZWaveDiscoverySchema(
+        platform=Platform.COVER,
+        hint="shutter",
+        manufacturer_id={0x007A},
+        product_id={0x0001},
+        product_type={0x8003},
+        primary_value=ZWaveValueDiscoverySchema(
+            command_class={CommandClass.SWITCH_MULTILEVEL},
+            property={CURRENT_VALUE_PROPERTY},
+            endpoint={2},
+            type={ValueType.NUMBER},
+        ),
+        assumed_state=True,
+        entity_registry_enabled_default=False,
+    ),
+    # Merten 507801 Connect Roller Shutter.
+    # Disable endpoint 2, as it has no practical function. CC: Protection
+    ZWaveDiscoverySchema(
+        platform=Platform.SELECT,
+        manufacturer_id={0x007A},
+        product_id={0x0001},
+        product_type={0x8003},
+        primary_value=ZWaveValueDiscoverySchema(
+            command_class={CommandClass.PROTECTION},
+            property={LOCAL_PROPERTY, RF_PROPERTY},
+            endpoint={2},
+            type={ValueType.NUMBER},
+        ),
+        entity_registry_enabled_default=False,
+    ),
     # Vision Security ZL7432 In Wall Dual Relay Switch
     ZWaveDiscoverySchema(
         platform=Platform.SWITCH,
@@ -396,6 +530,68 @@ DISCOVERY_SCHEMAS = [
         primary_value=SWITCH_BINARY_CURRENT_VALUE_SCHEMA,
         assumed_state=True,
     ),
+    # Heatit Z-TRM6
+    ZWaveDiscoverySchema(
+        platform=Platform.CLIMATE,
+        hint="dynamic_current_temp",
+        manufacturer_id={0x019B},
+        product_id={0x3001},
+        product_type={0x0030},
+        primary_value=ZWaveValueDiscoverySchema(
+            command_class={CommandClass.THERMOSTAT_MODE},
+            property={THERMOSTAT_MODE_PROPERTY},
+            type={ValueType.NUMBER},
+        ),
+        data_template=DynamicCurrentTempClimateDataTemplate(
+            lookup_table={
+                # Floor sensor
+                "Floor": ZwaveValueID(
+                    property_=THERMOSTAT_CURRENT_TEMP_PROPERTY,
+                    command_class=CommandClass.SENSOR_MULTILEVEL,
+                    endpoint=4,
+                ),
+                # Internal sensor
+                "Internal": ZwaveValueID(
+                    property_=THERMOSTAT_CURRENT_TEMP_PROPERTY,
+                    command_class=CommandClass.SENSOR_MULTILEVEL,
+                    endpoint=2,
+                ),
+                # Internal with limit by floor sensor
+                "Internal with floor limit": ZwaveValueID(
+                    property_=THERMOSTAT_CURRENT_TEMP_PROPERTY,
+                    command_class=CommandClass.SENSOR_MULTILEVEL,
+                    endpoint=2,
+                ),
+                # External sensor (connected to device)
+                "External": ZwaveValueID(
+                    property_=THERMOSTAT_CURRENT_TEMP_PROPERTY,
+                    command_class=CommandClass.SENSOR_MULTILEVEL,
+                    endpoint=3,
+                ),
+                # External sensor (connected to device) with limit by floor sensor (2x sensors)
+                "External with floor limit": ZwaveValueID(
+                    property_=THERMOSTAT_CURRENT_TEMP_PROPERTY,
+                    command_class=CommandClass.SENSOR_MULTILEVEL,
+                    endpoint=3,
+                ),
+                # PWER - Power regulator mode (no sensor used).
+                # This mode is not supported by the climate entity.
+                # Heating is set by adjusting parameter 25.
+                # P25: Set % of time the relay should be active when using PWER mode.
+                # (30-minute duty cycle)
+                # Use the air temperature as current temperature in the climate entity
+                # as we have nothing else.
+                "Power regulator": ZwaveValueID(
+                    property_=THERMOSTAT_CURRENT_TEMP_PROPERTY,
+                    command_class=CommandClass.SENSOR_MULTILEVEL,
+                    endpoint=2,
+                ),
+            },
+            dependent_value=ZwaveValueID(
+                property_=2, command_class=CommandClass.CONFIGURATION, endpoint=0
+            ),
+        ),
+    ),
     # Heatit Z-TRM3
     ZWaveDiscoverySchema(
         platform=Platform.CLIMATE,
@@ -513,18 +709,31 @@ DISCOVERY_SCHEMAS = [
         ),
         absent_values=[SWITCH_MULTILEVEL_CURRENT_VALUE_SCHEMA],
     ),
-    # ====== START OF CONFIG PARAMETER SPECIFIC MAPPING SCHEMAS =======
-    # Door lock mode config parameter. Functionality equivalent to Notification CC
-    # list sensors.
-    get_config_parameter_discovery_schema(
-        property_name={"Door lock mode"},
-        device_class_generic={"Entry Control"},
+    # Logic Group ZDB5100
+    ZWaveDiscoverySchema(
+        platform=Platform.LIGHT,
+        hint="black_is_off",
+        manufacturer_id={0x0234},
+        product_id={0x0121},
+        product_type={0x0003},
+        primary_value=ZWaveValueDiscoverySchema(
+            command_class={CommandClass.SWITCH_COLOR},
+            property={CURRENT_COLOR_PROPERTY},
+            property_key={None},
+        ),
     ),
     # ====== START OF GENERIC MAPPING SCHEMAS =======
     # locks
     # Door Lock CC
     ZWaveDiscoverySchema(
-        platform=Platform.LOCK, primary_value=DOOR_LOCK_CURRENT_MODE_SCHEMA
+        platform=Platform.LOCK,
+        primary_value=DOOR_LOCK_CURRENT_MODE_SCHEMA,
+        allow_multi=True,
+    ),
+    ZWaveDiscoverySchema(
+        platform=Platform.SELECT,
+        primary_value=DOOR_LOCK_CURRENT_MODE_SCHEMA,
+        hint="door_lock",
     ),
     # Only discover the Lock CC if the Door Lock CC isn't also present on the node
     ZWaveDiscoverySchema(
@@ -641,6 +850,18 @@ DISCOVERY_SCHEMAS = [
         ),
         allow_multi=True,
     ),
+    # binary sensor for Indicator CC
+    ZWaveDiscoverySchema(
+        platform=Platform.BINARY_SENSOR,
+        hint="boolean",
+        primary_value=ZWaveValueDiscoverySchema(
+            command_class={CommandClass.INDICATOR},
+            type={ValueType.BOOLEAN},
+            readable=True,
+            writeable=False,
+        ),
+        entity_category=EntityCategory.DIAGNOSTIC,
+    ),
     # generic text sensors
     ZWaveDiscoverySchema(
         platform=Platform.SENSOR,
@@ -650,24 +871,16 @@ DISCOVERY_SCHEMAS = [
             type={ValueType.STRING},
         ),
     ),
-    ZWaveDiscoverySchema(
-        platform=Platform.SENSOR,
-        hint="string_sensor",
-        primary_value=ZWaveValueDiscoverySchema(
-            command_class={CommandClass.INDICATOR},
-            type={ValueType.STRING},
-        ),
-        entity_registry_enabled_default=False,
-    ),
     # generic numeric sensors
     ZWaveDiscoverySchema(
         platform=Platform.SENSOR,
         hint="numeric_sensor",
         primary_value=ZWaveValueDiscoverySchema(
             command_class={
-                CommandClass.SENSOR_MULTILEVEL,
-                CommandClass.SENSOR_ALARM,
                 CommandClass.BATTERY,
+                CommandClass.ENERGY_PRODUCTION,
+                CommandClass.SENSOR_ALARM,
+                CommandClass.SENSOR_MULTILEVEL,
             },
             type={ValueType.NUMBER},
         ),
@@ -679,9 +892,11 @@ DISCOVERY_SCHEMAS = [
         primary_value=ZWaveValueDiscoverySchema(
             command_class={CommandClass.INDICATOR},
             type={ValueType.NUMBER},
+            readable=True,
+            writeable=False,
         ),
         data_template=NumericSensorDataTemplate(),
-        entity_registry_enabled_default=False,
+        entity_category=EntityCategory.DIAGNOSTIC,
     ),
     # Meter sensors for Meter CC
     ZWaveDiscoverySchema(
@@ -709,34 +924,46 @@ DISCOVERY_SCHEMAS = [
         allow_multi=True,
         entity_registry_enabled_default=False,
     ),
-    # number for Basic CC
+    # number for Indicator CC (exclude property keys 3-5)
     ZWaveDiscoverySchema(
         platform=Platform.NUMBER,
-        hint="Basic",
         primary_value=ZWaveValueDiscoverySchema(
-            command_class={
-                CommandClass.BASIC,
-            },
+            command_class={CommandClass.INDICATOR},
             type={ValueType.NUMBER},
-            property={CURRENT_VALUE_PROPERTY},
+            not_property_key={3, 4, 5},
+            readable=True,
+            writeable=True,
         ),
-        required_values=[
-            ZWaveValueDiscoverySchema(
-                command_class={
-                    CommandClass.BASIC,
-                },
-                type={ValueType.NUMBER},
-                property={TARGET_VALUE_PROPERTY},
-            )
-        ],
-        data_template=NumericSensorDataTemplate(),
-        entity_registry_enabled_default=False,
+        entity_category=EntityCategory.CONFIG,
+    ),
+    # button for Indicator CC
+    ZWaveDiscoverySchema(
+        platform=Platform.BUTTON,
+        primary_value=ZWaveValueDiscoverySchema(
+            command_class={CommandClass.INDICATOR},
+            type={ValueType.BOOLEAN},
+            readable=False,
+            writeable=True,
+        ),
+        entity_category=EntityCategory.CONFIG,
     ),
     # binary switches
     ZWaveDiscoverySchema(
         platform=Platform.SWITCH,
         primary_value=SWITCH_BINARY_CURRENT_VALUE_SCHEMA,
     ),
+    # switch for Indicator CC
+    ZWaveDiscoverySchema(
+        platform=Platform.SWITCH,
+        hint="indicator",
+        primary_value=ZWaveValueDiscoverySchema(
+            command_class={CommandClass.INDICATOR},
+            type={ValueType.BOOLEAN},
+            readable=True,
+            writeable=True,
+        ),
+        entity_category=EntityCategory.CONFIG,
+    ),
     # binary switch
     # barrier operator signaling states
     ZWaveDiscoverySchema(
@@ -752,7 +979,18 @@ DISCOVERY_SCHEMAS = [
     # window coverings
     ZWaveDiscoverySchema(
         platform=Platform.COVER,
-        hint="window_cover",
+        hint="window_covering",
+        primary_value=WINDOW_COVERING_COVER_CURRENT_VALUE_SCHEMA,
+    ),
+    ZWaveDiscoverySchema(
+        platform=Platform.COVER,
+        hint="window_covering",
+        primary_value=WINDOW_COVERING_SLAT_CURRENT_VALUE_SCHEMA,
+        absent_values=[WINDOW_COVERING_COVER_CURRENT_VALUE_SCHEMA],
+    ),
+    ZWaveDiscoverySchema(
+        platform=Platform.COVER,
+        hint="multilevel_switch",
         device_class_generic={"Multilevel Switch"},
         device_class_specific={
             "Motor Control Class A",
@@ -761,6 +999,10 @@ DISCOVERY_SCHEMAS = [
             "Multiposition Motor",
         },
         primary_value=SWITCH_MULTILEVEL_CURRENT_VALUE_SCHEMA,
+        absent_values=[
+            WINDOW_COVERING_COVER_CURRENT_VALUE_SCHEMA,
+            WINDOW_COVERING_SLAT_CURRENT_VALUE_SCHEMA,
+        ],
     ),
     # cover
     # motorized barriers
@@ -787,6 +1029,7 @@ DISCOVERY_SCHEMAS = [
         device_class_generic={"Multilevel Switch"},
         device_class_specific={"Fan Switch"},
         primary_value=SWITCH_MULTILEVEL_CURRENT_VALUE_SCHEMA,
+        required_values=[SWITCH_MULTILEVEL_TARGET_VALUE_SCHEMA],
     ),
     # number platform
     # valve control for thermostats
@@ -805,6 +1048,24 @@ DISCOVERY_SCHEMAS = [
         platform=Platform.LIGHT,
         primary_value=SWITCH_MULTILEVEL_CURRENT_VALUE_SCHEMA,
     ),
+    # light for Basic CC
+    ZWaveDiscoverySchema(
+        platform=Platform.LIGHT,
+        primary_value=ZWaveValueDiscoverySchema(
+            command_class={CommandClass.BASIC},
+            type={ValueType.NUMBER},
+            property={CURRENT_VALUE_PROPERTY},
+        ),
+        required_values=[
+            ZWaveValueDiscoverySchema(
+                command_class={
+                    CommandClass.BASIC,
+                },
+                type={ValueType.NUMBER},
+                property={TARGET_VALUE_PROPERTY},
+            )
+        ],
+    ),
     # sirens
     ZWaveDiscoverySchema(
         platform=Platform.SIREN,
@@ -844,6 +1105,26 @@ DISCOVERY_SCHEMAS = [
             type={ValueType.NUMBER},
         ),
     ),
+    # button
+    # Notification CC idle
+    ZWaveDiscoverySchema(
+        platform=Platform.BUTTON,
+        hint="notification idle",
+        primary_value=ZWaveValueDiscoverySchema(
+            command_class={CommandClass.NOTIFICATION},
+            type={ValueType.NUMBER},
+            any_available_states={(0, "idle")},
+        ),
+    ),
+    # event
+    # stateful = False
+    ZWaveDiscoverySchema(
+        platform=Platform.EVENT,
+        hint="stateless",
+        primary_value=ZWaveValueDiscoverySchema(
+            stateful=False,
+        ),
+    ),
 ]
 
 
@@ -854,9 +1135,8 @@ def async_discover_node_values(
     """Run discovery on ZWave node and return matching (primary) values."""
     for value in node.values.values():
         # We don't need to rediscover an already processed value_id
-        if value.value_id in discovered_value_ids[device.id]:
-            continue
-        yield from async_discover_single_value(value, device, discovered_value_ids)
+        if value.value_id not in discovered_value_ids[device.id]:
+            yield from async_discover_single_value(value, device, discovered_value_ids)
 
 
 @callback
@@ -866,24 +1146,20 @@ def async_discover_single_value(
     """Run discovery on a single ZWave value and return matching schema info."""
     discovered_value_ids[device.id].add(value.value_id)
     for schema in DISCOVERY_SCHEMAS:
-        # check manufacturer_id
+        # check manufacturer_id, product_id, product_type
         if (
-            schema.manufacturer_id is not None
-            and value.node.manufacturer_id not in schema.manufacturer_id
-        ):
-            continue
-
-        # check product_id
-        if (
-            schema.product_id is not None
-            and value.node.product_id not in schema.product_id
-        ):
-            continue
-
-        # check product_type
-        if (
-            schema.product_type is not None
-            and value.node.product_type not in schema.product_type
+            (
+                schema.manufacturer_id is not None
+                and value.node.manufacturer_id not in schema.manufacturer_id
+            )
+            or (
+                schema.product_id is not None
+                and value.node.product_id not in schema.product_id
+            )
+            or (
+                schema.product_type is not None
+                and value.node.product_type not in schema.product_type
+            )
         ):
             continue
 
@@ -902,27 +1178,14 @@ def async_discover_single_value(
         ):
             continue
 
-        # check firmware_version
-        if (
-            schema.firmware_version is not None
-            and value.node.firmware_version not in schema.firmware_version
-        ):
-            continue
-
-        # check device_class_basic
-        if not check_device_class(
-            value.node.device_class.basic, schema.device_class_basic
-        ):
-            continue
-
         # check device_class_generic
-        if not check_device_class(
+        if value.node.device_class and not check_device_class(
             value.node.device_class.generic, schema.device_class_generic
         ):
             continue
 
         # check device_class_specific
-        if not check_device_class(
+        if value.node.device_class and not check_device_class(
             value.node.device_class.specific, schema.device_class_specific
         ):
             continue
@@ -975,12 +1238,86 @@ def async_discover_single_value(
             platform_data=resolved_data,
             additional_value_ids_to_watch=additional_value_ids_to_watch,
             entity_registry_enabled_default=schema.entity_registry_enabled_default,
+            entity_category=schema.entity_category,
         )
 
         if not schema.allow_multi:
-            # return early since this value may not be discovered by other schemas/platforms
+            # return early since this value may not be discovered
+            # by other schemas/platforms
             return
 
+    if value.command_class == CommandClass.CONFIGURATION:
+        yield from async_discover_single_configuration_value(
+            cast(ConfigurationValue, value)
+        )
+
+
+@callback
+def async_discover_single_configuration_value(
+    value: ConfigurationValue,
+) -> Generator[ZwaveDiscoveryInfo, None, None]:
+    """Run discovery on single Z-Wave configuration value and return schema matches."""
+    if value.metadata.writeable and value.metadata.readable:
+        if value.configuration_value_type == ConfigurationValueType.ENUMERATED:
+            yield ZwaveDiscoveryInfo(
+                node=value.node,
+                primary_value=value,
+                assumed_state=False,
+                platform=Platform.SELECT,
+                platform_hint="config_parameter",
+                platform_data=None,
+                additional_value_ids_to_watch=set(),
+                entity_registry_enabled_default=False,
+            )
+        elif value.configuration_value_type in (
+            ConfigurationValueType.RANGE,
+            ConfigurationValueType.MANUAL_ENTRY,
+        ):
+            yield ZwaveDiscoveryInfo(
+                node=value.node,
+                primary_value=value,
+                assumed_state=False,
+                platform=Platform.NUMBER,
+                platform_hint="config_parameter",
+                platform_data=None,
+                additional_value_ids_to_watch=set(),
+                entity_registry_enabled_default=False,
+            )
+        elif value.configuration_value_type == ConfigurationValueType.BOOLEAN:
+            yield ZwaveDiscoveryInfo(
+                node=value.node,
+                primary_value=value,
+                assumed_state=False,
+                platform=Platform.SWITCH,
+                platform_hint="config_parameter",
+                platform_data=None,
+                additional_value_ids_to_watch=set(),
+                entity_registry_enabled_default=False,
+            )
+    elif not value.metadata.writeable and value.metadata.readable:
+        if value.configuration_value_type == ConfigurationValueType.BOOLEAN:
+            yield ZwaveDiscoveryInfo(
+                node=value.node,
+                primary_value=value,
+                assumed_state=False,
+                platform=Platform.BINARY_SENSOR,
+                platform_hint="config_parameter",
+                platform_data=None,
+                additional_value_ids_to_watch=set(),
+                entity_registry_enabled_default=False,
+            )
+        else:
+            yield ZwaveDiscoveryInfo(
+                node=value.node,
+                primary_value=value,
+                assumed_state=False,
+                platform=Platform.SENSOR,
+                platform_hint="config_parameter",
+                platform_data=None,
+                additional_value_ids_to_watch=set(),
+                entity_registry_enabled_default=False,
+            )
+
 
 @callback
 def check_value(value: ZwaveValue, schema: ZWaveValueDiscoverySchema) -> bool:
@@ -1009,28 +1346,47 @@ def check_value(value: ZwaveValue, schema: ZWaveValueDiscoverySchema) -> bool:
         and value.property_key not in schema.property_key
     ):
         return False
-    # check property_key_name
+    # check property_key against not_property_key set
     if (
-        schema.property_key_name is not None
-        and value.property_key_name not in schema.property_key_name
+        schema.not_property_key is not None
+        and value.property_key in schema.not_property_key
     ):
         return False
     # check metadata_type
     if schema.type is not None and value.metadata.type not in schema.type:
         return False
+    # check metadata_readable
+    if schema.readable is not None and value.metadata.readable != schema.readable:
+        return False
+    # check metadata_writeable
+    if schema.writeable is not None and value.metadata.writeable != schema.writeable:
+        return False
+    # check available states
+    if (
+        schema.any_available_states is not None
+        and value.metadata.states is not None
+        and not any(
+            str(key) in value.metadata.states and value.metadata.states[str(key)] == val
+            for key, val in schema.any_available_states
+        )
+    ):
+        return False
+    # check value
+    if schema.value is not None and value.value not in schema.value:
+        return False
+    # check metadata_stateful
+    if schema.stateful is not None and value.metadata.stateful != schema.stateful:
+        return False
     return True
 
 
 @callback
 def check_device_class(
-    device_class: DeviceClassItem, required_value: set[str | int] | None
+    device_class: DeviceClassItem, required_value: set[str] | None
 ) -> bool:
     """Check if device class id or label matches."""
     if required_value is None:
         return True
-    for val in required_value:
-        if isinstance(val, str) and device_class.label == val:
-            return True
-        if isinstance(val, int) and device_class.key == val:
-            return True
+    if any(device_class.label == val for val in required_value):
+        return True
     return False
