@@ -5,15 +5,14 @@ from typing import Any
 
 from zwave_js_server.client import Client as ZwaveClient
 from zwave_js_server.const.command_class.sound_switch import ToneID
+from zwave_js_server.model.driver import Driver
 
-from homeassistant.components.siren import DOMAIN as SIREN_DOMAIN, SirenEntity
-from homeassistant.components.siren.const import (
+from homeassistant.components.siren import (
     ATTR_TONE,
     ATTR_VOLUME_LEVEL,
-    SUPPORT_TONES,
-    SUPPORT_TURN_OFF,
-    SUPPORT_TURN_ON,
-    SUPPORT_VOLUME_SET,
+    DOMAIN as SIREN_DOMAIN,
+    SirenEntity,
+    SirenEntityFeature,
 )
 from homeassistant.config_entries import ConfigEntry
 from homeassistant.core import HomeAssistant, callback
@@ -24,6 +23,8 @@ from .const import DATA_CLIENT, DOMAIN
 from .discovery import ZwaveDiscoveryInfo
 from .entity import ZWaveBaseEntity
 
+PARALLEL_UPDATES = 0
+
 
 async def async_setup_entry(
     hass: HomeAssistant,
@@ -36,8 +37,10 @@ async def async_setup_entry(
     @callback
     def async_add_siren(info: ZwaveDiscoveryInfo) -> None:
         """Add Z-Wave siren entity."""
+        driver = client.driver
+        assert driver is not None  # Driver is ready before platforms are loaded.
         entities: list[ZWaveBaseEntity] = []
-        entities.append(ZwaveSirenEntity(config_entry, client, info))
+        entities.append(ZwaveSirenEntity(config_entry, driver, info))
         async_add_entities(entities)
 
     config_entry.async_on_unload(
@@ -53,19 +56,23 @@ class ZwaveSirenEntity(ZWaveBaseEntity, SirenEntity):
     """Representation of a Z-Wave siren entity."""
 
     def __init__(
-        self, config_entry: ConfigEntry, client: ZwaveClient, info: ZwaveDiscoveryInfo
+        self, config_entry: ConfigEntry, driver: Driver, info: ZwaveDiscoveryInfo
     ) -> None:
         """Initialize a ZwaveSirenEntity entity."""
-        super().__init__(config_entry, client, info)
+        super().__init__(config_entry, driver, info)
         # Entity class attributes
         self._attr_available_tones = {
             int(id): val for id, val in self.info.primary_value.metadata.states.items()
         }
         self._attr_supported_features = (
-            SUPPORT_TURN_ON | SUPPORT_TURN_OFF | SUPPORT_VOLUME_SET
+            SirenEntityFeature.TURN_ON
+            | SirenEntityFeature.TURN_OFF
+            | SirenEntityFeature.VOLUME_SET
         )
         if self._attr_available_tones:
-            self._attr_supported_features |= SUPPORT_TONES
+            self._attr_supported_features |= SirenEntityFeature.TONES
+
+        self._attr_name = self.generate_name(include_value_name=True)
 
     @property
     def is_on(self) -> bool | None:
@@ -74,14 +81,6 @@ class ZwaveSirenEntity(ZWaveBaseEntity, SirenEntity):
             return None
         return bool(self.info.primary_value.value)
 
-    async def async_set_value(
-        self, new_value: int, options: dict[str, Any] | None = None
-    ) -> None:
-        """Set a value on a siren node."""
-        await self.info.node.async_set_value(
-            self.info.primary_value, new_value, options=options
-        )
-
     async def async_turn_on(self, **kwargs: Any) -> None:
         """Turn the device on."""
         tone_id: int | None = kwargs.get(ATTR_TONE)
@@ -90,11 +89,13 @@ class ZwaveSirenEntity(ZWaveBaseEntity, SirenEntity):
             options["volume"] = round(volume * 100)
         # Play the default tone if a tone isn't provided
         if tone_id is None:
-            await self.async_set_value(ToneID.DEFAULT, options)
+            await self._async_set_value(
+                self.info.primary_value, ToneID.DEFAULT, options
+            )
             return
 
-        await self.async_set_value(tone_id, options)
+        await self._async_set_value(self.info.primary_value, tone_id, options)
 
     async def async_turn_off(self, **kwargs: Any) -> None:
         """Turn the device off."""
-        await self.async_set_value(ToneID.OFF)
+        await self._async_set_value(self.info.primary_value, ToneID.OFF)
