@@ -10,13 +10,14 @@ from __future__ import annotations
 from abc import ABC, ABCMeta, abstractmethod
 import asyncio
 from collections.abc import Awaitable, Callable
+from http import HTTPStatus
+from json import JSONDecodeError
 import logging
 import secrets
 import time
-from typing import Any, Dict, cast
+from typing import Any, cast
 
-from aiohttp import client, web
-import async_timeout
+from aiohttp import ClientError, ClientResponseError, client, web
 import jwt
 import voluptuous as vol
 from yarl import URL
@@ -25,21 +26,25 @@ from homeassistant import config_entries
 from homeassistant.components import http
 from homeassistant.core import HomeAssistant, callback
 from homeassistant.data_entry_flow import FlowResult
-from homeassistant.helpers.network import NoURLAvailableError
+from homeassistant.loader import async_get_application_credentials
 
 from .aiohttp_client import async_get_clientsession
+from .network import NoURLAvailableError
 
 _LOGGER = logging.getLogger(__name__)
 
 DATA_JWT_SECRET = "oauth2_jwt_secret"
-DATA_VIEW_REGISTERED = "oauth2_view_reg"
 DATA_IMPLEMENTATIONS = "oauth2_impl"
 DATA_PROVIDERS = "oauth2_providers"
 AUTH_CALLBACK_PATH = "/auth/external/callback"
 HEADER_FRONTEND_BASE = "HA-Frontend-Base"
+MY_AUTH_CALLBACK_PATH = "https://my.home-assistant.io/redirect/oauth"
 
 CLOCK_OUT_OF_SYNC_MAX_SEC = 20
 
+OAUTH_AUTHORIZE_URL_TIMEOUT_SEC = 30
+OAUTH_TOKEN_TIMEOUT_SEC = 30
+
 
 class AbstractOAuth2Implementation(ABC):
     """Base class to abstract OAuth2 authentication."""
@@ -69,7 +74,8 @@ class AbstractOAuth2Implementation(ABC):
         Pass external data in with:
 
         await hass.config_entries.flow.async_configure(
-            flow_id=flow_id, user_input={'code': 'abcd', 'state': { … }
+            flow_id=flow_id, user_input={'code': 'abcd', 'state': … }
+
         )
 
         """
@@ -129,6 +135,9 @@ class LocalOAuth2Implementation(AbstractOAuth2Implementation):
     @property
     def redirect_uri(self) -> str:
         """Return the redirect uri."""
+        if "my" in self.hass.config.components:
+            return MY_AUTH_CALLBACK_PATH
+
         if (req := http.current_request.get()) is None:
             raise RuntimeError("No current request in context")
 
@@ -190,13 +199,20 @@ class LocalOAuth2Implementation(AbstractOAuth2Implementation):
         if self.client_secret is not None:
             data["client_secret"] = self.client_secret
 
+        _LOGGER.debug("Sending token request to %s", self.token_url)
         resp = await session.post(self.token_url, data=data)
-        if resp.status >= 400 and _LOGGER.isEnabledFor(logging.DEBUG):
-            body = await resp.text()
-            _LOGGER.debug(
-                "Token request failed with status=%s, body=%s",
-                resp.status,
-                body,
+        if resp.status >= 400:
+            try:
+                error_response = await resp.json()
+            except (ClientError, JSONDecodeError):
+                error_response = {}
+            error_code = error_response.get("error", "unknown")
+            error_description = error_response.get("error_description", "unknown error")
+            _LOGGER.error(
+                "Token request for %s failed (%s): %s",
+                self.domain,
+                error_code,
+                error_description,
             )
         resp.raise_for_status()
         return cast(dict, await resp.json())
@@ -213,11 +229,12 @@ class AbstractOAuth2FlowHandler(config_entries.ConfigFlow, metaclass=ABCMeta):
         """Instantiate config flow."""
         if self.DOMAIN == "":
             raise TypeError(
-                f"Can't instantiate class {self.__class__.__name__} without DOMAIN being set"
+                f"Can't instantiate class {self.__class__.__name__} without DOMAIN"
+                " being set"
             )
 
         self.external_data: Any = None
-        self.flow_impl: AbstractOAuth2Implementation = None  # type: ignore
+        self.flow_impl: AbstractOAuth2Implementation = None  # type: ignore[assignment]
 
     @property
     @abstractmethod
@@ -229,6 +246,11 @@ class AbstractOAuth2FlowHandler(config_entries.ConfigFlow, metaclass=ABCMeta):
         """Extra data that needs to be appended to the authorize url."""
         return {}
 
+    async def async_generate_authorize_url(self) -> str:
+        """Generate a url for the user to authorize."""
+        url = await self.flow_impl.async_generate_authorize_url(self.flow_id)
+        return str(URL(url).update_query(self.extra_authorize_data))
+
     async def async_step_pick_implementation(
         self, user_input: dict | None = None
     ) -> FlowResult:
@@ -240,6 +262,8 @@ class AbstractOAuth2FlowHandler(config_entries.ConfigFlow, metaclass=ABCMeta):
             return await self.async_step_auth()
 
         if not implementations:
+            if self.DOMAIN in await async_get_application_credentials(self.hass):
+                return self.async_abort(reason="missing_credentials")
             return self.async_abort(reason="missing_configuration")
 
         req = http.current_request.get()
@@ -265,32 +289,55 @@ class AbstractOAuth2FlowHandler(config_entries.ConfigFlow, metaclass=ABCMeta):
     ) -> FlowResult:
         """Create an entry for auth."""
         # Flow has been triggered by external data
-        if user_input:
+        if user_input is not None:
             self.external_data = user_input
-            return self.async_external_step_done(next_step_id="creation")
+            next_step = "authorize_rejected" if "error" in user_input else "creation"
+            return self.async_external_step_done(next_step_id=next_step)
 
         try:
-            with async_timeout.timeout(10):
-                url = await self.flow_impl.async_generate_authorize_url(self.flow_id)
-        except asyncio.TimeoutError:
+            async with asyncio.timeout(OAUTH_AUTHORIZE_URL_TIMEOUT_SEC):
+                url = await self.async_generate_authorize_url()
+        except asyncio.TimeoutError as err:
+            _LOGGER.error("Timeout generating authorize url: %s", err)
             return self.async_abort(reason="authorize_url_timeout")
         except NoURLAvailableError:
             return self.async_abort(
                 reason="no_url_available",
                 description_placeholders={
-                    "docs_url": "https://www.home-assistant.io/more-info/no-url-available"
+                    "docs_url": (
+                        "https://www.home-assistant.io/more-info/no-url-available"
+                    )
                 },
             )
 
-        url = str(URL(url).update_query(self.extra_authorize_data))
-
         return self.async_external_step(step_id="auth", url=url)
 
     async def async_step_creation(
         self, user_input: dict[str, Any] | None = None
     ) -> FlowResult:
         """Create config entry from external data."""
-        token = await self.flow_impl.async_resolve_external_data(self.external_data)
+        _LOGGER.debug("Creating config entry from external data")
+
+        try:
+            async with asyncio.timeout(OAUTH_TOKEN_TIMEOUT_SEC):
+                token = await self.flow_impl.async_resolve_external_data(
+                    self.external_data
+                )
+        except asyncio.TimeoutError as err:
+            _LOGGER.error("Timeout resolving OAuth token: %s", err)
+            return self.async_abort(reason="oauth_timeout")
+        except (ClientResponseError, ClientError) as err:
+            if (
+                isinstance(err, ClientResponseError)
+                and err.status == HTTPStatus.UNAUTHORIZED
+            ):
+                return self.async_abort(reason="oauth_unauthorized")
+            return self.async_abort(reason="oauth_failed")
+
+        if "expires_in" not in token:
+            _LOGGER.warning("Invalid token: %s", token)
+            return self.async_abort(reason="oauth_error")
+
         # Force int for non-compliant oauth2 providers
         try:
             token["expires_in"] = int(token["expires_in"])
@@ -305,6 +352,13 @@ class AbstractOAuth2FlowHandler(config_entries.ConfigFlow, metaclass=ABCMeta):
             {"auth_implementation": self.flow_impl.domain, "token": token}
         )
 
+    async def async_step_authorize_rejected(self, data: None = None) -> FlowResult:
+        """Step to handle flow rejection."""
+        return self.async_abort(
+            reason="user_rejected_authorize",
+            description_placeholders={"error": self.external_data["error"]},
+        )
+
     async def async_oauth_create_entry(self, data: dict) -> FlowResult:
         """Create an entry for the flow.
 
@@ -331,12 +385,6 @@ def async_register_implementation(
     hass: HomeAssistant, domain: str, implementation: AbstractOAuth2Implementation
 ) -> None:
     """Register an OAuth2 flow implementation for an integration."""
-    if isinstance(implementation, LocalOAuth2Implementation) and not hass.data.get(
-        DATA_VIEW_REGISTERED, False
-    ):
-        hass.http.register_view(OAuth2AuthorizeCallbackView())
-        hass.data[DATA_VIEW_REGISTERED] = True
-
     implementations = hass.data.setdefault(DATA_IMPLEMENTATIONS, {})
     implementations.setdefault(domain, {})[implementation.domain] = implementation
 
@@ -346,7 +394,7 @@ async def async_get_implementations(
 ) -> dict[str, AbstractOAuth2Implementation]:
     """Return OAuth2 implementations for specified domain."""
     registered = cast(
-        Dict[str, AbstractOAuth2Implementation],
+        dict[str, AbstractOAuth2Implementation],
         hass.data.setdefault(DATA_IMPLEMENTATIONS, {}).get(domain, {}),
     )
 
@@ -354,11 +402,9 @@ async def async_get_implementations(
         return registered
 
     registered = dict(registered)
-
-    for provider_domain, get_impl in hass.data[DATA_PROVIDERS].items():
-        implementation = await get_impl(hass, domain)
-        if implementation is not None:
-            registered[provider_domain] = implementation
+    for get_impl in list(hass.data[DATA_PROVIDERS].values()):
+        for impl in await get_impl(hass, domain):
+            registered[impl.domain] = impl
 
     return registered
 
@@ -381,7 +427,7 @@ def async_add_implementation_provider(
     hass: HomeAssistant,
     provider_domain: str,
     async_provide_implementation: Callable[
-        [HomeAssistant, str], Awaitable[AbstractOAuth2Implementation | None]
+        [HomeAssistant, str], Awaitable[list[AbstractOAuth2Implementation]]
     ],
 ) -> None:
     """Add an implementation provider.
@@ -402,24 +448,35 @@ class OAuth2AuthorizeCallbackView(http.HomeAssistantView):
 
     async def get(self, request: web.Request) -> web.Response:
         """Receive authorization code."""
-        # pylint: disable=no-self-use
-        if "code" not in request.query or "state" not in request.query:
-            return web.Response(
-                text=f"Missing code or state parameter in {request.url}"
-            )
+        if "state" not in request.query:
+            return web.Response(text="Missing state parameter")
 
         hass = request.app["hass"]
 
         state = _decode_jwt(hass, request.query["state"])
 
         if state is None:
-            return web.Response(text="Invalid state")
+            return web.Response(
+                text=(
+                    "Invalid state. Is My Home Assistant configured "
+                    "to go to the right instance?"
+                ),
+                status=400,
+            )
+
+        user_input: dict[str, Any] = {"state": state}
+
+        if "code" in request.query:
+            user_input["code"] = request.query["code"]
+        elif "error" in request.query:
+            user_input["error"] = request.query["error"]
+        else:
+            return web.Response(text="Missing code or error parameter")
 
         await hass.config_entries.flow.async_configure(
-            flow_id=state["flow_id"],
-            user_input={"state": state, "code": request.query["code"]},
+            flow_id=state["flow_id"], user_input=user_input
         )
-
+        _LOGGER.debug("Resumed OAuth configuration flow")
         return web.Response(
             headers={"content-type": "text/html"},
             text="<script>window.close()</script>",
@@ -482,13 +539,13 @@ async def async_oauth2_request(
     This method will not refresh tokens. Use OAuth2 session for that.
     """
     session = async_get_clientsession(hass)
-
+    headers = kwargs.pop("headers", {})
     return await session.request(
         method,
         url,
         **kwargs,
         headers={
-            **(kwargs.get("headers") or {}),
+            **headers,
             "authorization": f"Bearer {token['access_token']}",
         },
     )
@@ -504,11 +561,14 @@ def _encode_jwt(hass: HomeAssistant, data: dict) -> str:
 
 
 @callback
-def _decode_jwt(hass: HomeAssistant, encoded: str) -> dict | None:
+def _decode_jwt(hass: HomeAssistant, encoded: str) -> dict[str, Any] | None:
     """JWT encode data."""
-    secret = cast(str, hass.data.get(DATA_JWT_SECRET))
+    secret: str | None = hass.data.get(DATA_JWT_SECRET)
+
+    if secret is None:
+        return None
 
     try:
-        return jwt.decode(encoded, secret, algorithms=["HS256"])
+        return jwt.decode(encoded, secret, algorithms=["HS256"])  # type: ignore[no-any-return]
     except jwt.InvalidTokenError:
         return None
