@@ -4,11 +4,13 @@ from __future__ import annotations
 import asyncio
 from collections.abc import Callable, Iterable
 from datetime import timedelta
+from functools import partial
 from itertools import chain
 import logging
 from types import ModuleType
-from typing import Any, Generic, TypeVar
+from typing import Any, Generic
 
+from typing_extensions import TypeVar
 import voluptuous as vol
 
 from homeassistant import config as conf_util
@@ -18,25 +20,33 @@ from homeassistant.const import (
     CONF_SCAN_INTERVAL,
     EVENT_HOMEASSISTANT_STOP,
 )
-from homeassistant.core import Event, HomeAssistant, ServiceCall, callback
+from homeassistant.core import (
+    Event,
+    HassJob,
+    HomeAssistant,
+    ServiceCall,
+    ServiceResponse,
+    SupportsResponse,
+    callback,
+)
 from homeassistant.exceptions import HomeAssistantError
 from homeassistant.loader import async_get_integration, bind_hass
 from homeassistant.setup import async_prepare_setup_platform
 
-from . import config_per_platform, config_validation as cv, discovery, entity, service
+from . import config_validation as cv, discovery, entity, service
 from .entity_platform import EntityPlatform
 from .typing import ConfigType, DiscoveryInfoType
 
 DEFAULT_SCAN_INTERVAL = timedelta(seconds=15)
 DATA_INSTANCES = "entity_components"
 
-_EntityT = TypeVar("_EntityT", bound=entity.Entity)
+_EntityT = TypeVar("_EntityT", bound=entity.Entity, default=entity.Entity)
 
 
 @bind_hass
 async def async_update_entity(hass: HomeAssistant, entity_id: str) -> None:
     """Trigger an update for an entity."""
-    domain = entity_id.split(".", 1)[0]
+    domain = entity_id.partition(".")[0]
     entity_comp: EntityComponent[entity.Entity] | None
     entity_comp = hass.data.get(DATA_INSTANCES, {}).get(domain)
 
@@ -80,36 +90,45 @@ class EntityComponent(Generic[_EntityT]):
 
         self.config: ConfigType | None = None
 
+        domain_platform = self._async_init_entity_platform(domain, None)
         self._platforms: dict[
             str | tuple[str, timedelta | None, str | None], EntityPlatform
-        ] = {domain: self._async_init_entity_platform(domain, None)}
-        self.async_add_entities = self._platforms[domain].async_add_entities
-        self.add_entities = self._platforms[domain].add_entities
-
+        ] = {domain: domain_platform}
+        self.async_add_entities = domain_platform.async_add_entities
+        self.add_entities = domain_platform.add_entities
+        self._entities: dict[str, entity.Entity] = domain_platform.domain_entities
         hass.data.setdefault(DATA_INSTANCES, {})[domain] = self
 
     @property
     def entities(self) -> Iterable[_EntityT]:
-        """Return an iterable that returns all entities."""
-        return chain.from_iterable(
-            platform.entities.values()  # type: ignore[misc]
-            for platform in self._platforms.values()
-        )
+        """Return an iterable that returns all entities.
+
+        As the underlying dicts may change when async context is lost,
+        callers that iterate over this asynchronously should make a copy
+        using list() before iterating.
+        """
+        return self._entities.values()  # type: ignore[return-value]
 
     def get_entity(self, entity_id: str) -> _EntityT | None:
         """Get an entity."""
-        for platform in self._platforms.values():
-            entity_obj = platform.entities.get(entity_id)
-            if entity_obj is not None:
-                return entity_obj  # type: ignore[return-value]
-        return None
+        return self._entities.get(entity_id)  # type: ignore[return-value]
+
+    def register_shutdown(self) -> None:
+        """Register shutdown on Home Assistant STOP event.
+
+        Note: this is only required if the integration never calls
+        `setup` or `async_setup`.
+        """
+        self.hass.bus.async_listen_once(EVENT_HOMEASSISTANT_STOP, self._async_shutdown)
 
     def setup(self, config: ConfigType) -> None:
         """Set up a full entity component.
 
         This doesn't block the executor to protect from deadlocks.
         """
-        self.hass.add_job(self.async_setup(config))
+        self.hass.create_task(
+            self.async_setup(config), f"EntityComponent setup {self.domain}"
+        )
 
     async def async_setup(self, config: ConfigType) -> None:
         """Set up a full entity component.
@@ -119,14 +138,17 @@ class EntityComponent(Generic[_EntityT]):
 
         This method must be run in the event loop.
         """
-        self.hass.bus.async_listen_once(EVENT_HOMEASSISTANT_STOP, self._async_shutdown)
+        self.register_shutdown()
 
         self.config = config
 
         # Look in config for Domain, Domain 2, Domain 3 etc and load them
-        for p_type, p_config in config_per_platform(config, self.domain):
+        for p_type, p_config in conf_util.config_per_platform(config, self.domain):
             if p_type is not None:
-                self.hass.async_create_task(self.async_setup_platform(p_type, p_config))
+                self.hass.async_create_task(
+                    self.async_setup_platform(p_type, p_config),
+                    f"EntityComponent setup platform {p_type} {self.domain}",
+                )
 
         # Generic discovery listener for loading platform dynamically
         # Refer to: homeassistant.helpers.discovery.async_load_platform()
@@ -191,25 +213,72 @@ class EntityComponent(Generic[_EntityT]):
             self.hass, self.entities, service_call, expand_group
         )
 
+    @callback
+    def async_register_legacy_entity_service(
+        self,
+        name: str,
+        schema: dict[str | vol.Marker, Any] | vol.Schema,
+        func: str | Callable[..., Any],
+        required_features: list[int] | None = None,
+        supports_response: SupportsResponse = SupportsResponse.NONE,
+    ) -> None:
+        """Register an entity service with a legacy response format."""
+        if isinstance(schema, dict):
+            schema = cv.make_entity_service_schema(schema)
+
+        service_func: str | HassJob[..., Any]
+        service_func = func if isinstance(func, str) else HassJob(func)
+
+        async def handle_service(
+            call: ServiceCall,
+        ) -> ServiceResponse:
+            """Handle the service."""
+
+            result = await service.entity_service_call(
+                self.hass, self._entities, service_func, call, required_features
+            )
+
+            if result:
+                if len(result) > 1:
+                    raise HomeAssistantError(
+                        "Deprecated service call matched more than one entity"
+                    )
+                return result.popitem()[1]
+            return None
+
+        self.hass.services.async_register(
+            self.domain, name, handle_service, schema, supports_response
+        )
+
     @callback
     def async_register_entity_service(
         self,
         name: str,
-        schema: dict[str, Any] | vol.Schema,
+        schema: dict[str | vol.Marker, Any] | vol.Schema,
         func: str | Callable[..., Any],
         required_features: list[int] | None = None,
+        supports_response: SupportsResponse = SupportsResponse.NONE,
     ) -> None:
         """Register an entity service."""
         if isinstance(schema, dict):
             schema = cv.make_entity_service_schema(schema)
 
-        async def handle_service(call: ServiceCall) -> None:
-            """Handle the service."""
-            await service.entity_service_call(
-                self.hass, self._platforms.values(), func, call, required_features
-            )
+        service_func: str | HassJob[..., Any]
+        service_func = func if isinstance(func, str) else HassJob(func)
 
-        self.hass.services.async_register(self.domain, name, handle_service, schema)
+        self.hass.services.async_register(
+            self.domain,
+            name,
+            partial(
+                service.entity_service_call,
+                self.hass,
+                self._entities,
+                service_func,
+                required_features=required_features,
+            ),
+            schema,
+            supports_response,
+        )
 
     async def async_setup_platform(
         self,
@@ -289,7 +358,7 @@ class EntityComponent(Generic[_EntityT]):
 
         integration = await async_get_integration(self.hass, self.domain)
 
-        processed_conf = await conf_util.async_process_component_config(
+        processed_conf = await conf_util.async_process_component_and_handle_errors(
             self.hass, conf, integration
         )
 
@@ -313,7 +382,7 @@ class EntityComponent(Generic[_EntityT]):
         if scan_interval is None:
             scan_interval = self.scan_interval
 
-        return EntityPlatform(
+        entity_platform = EntityPlatform(
             hass=self.hass,
             logger=self.logger,
             domain=self.domain,
@@ -322,6 +391,8 @@ class EntityComponent(Generic[_EntityT]):
             scan_interval=scan_interval,
             entity_namespace=entity_namespace,
         )
+        entity_platform.async_prepare()
+        return entity_platform
 
     async def _async_shutdown(self, event: Event) -> None:
         """Call when Home Assistant is stopping."""
