@@ -1,30 +1,28 @@
 """Translation string lookup helpers."""
-import asyncio
-from collections import ChainMap
-import logging
-from typing import Any, Dict, List, Optional, Set
+from __future__ import annotations
 
-from homeassistant.core import callback
+import asyncio
+from collections.abc import Iterable, Mapping
+import logging
+import string
+from typing import Any
+
+from homeassistant.core import HomeAssistant, callback
 from homeassistant.loader import (
-    MAX_LOAD_CONCURRENTLY,
     Integration,
     async_get_config_flows,
-    async_get_integration,
+    async_get_integrations,
     bind_hass,
 )
-from homeassistant.util.async_ import gather_with_concurrency
 from homeassistant.util.json import load_json
 
-from .typing import HomeAssistantType
-
 _LOGGER = logging.getLogger(__name__)
 
-TRANSLATION_LOAD_LOCK = "translation_load_lock"
 TRANSLATION_FLATTEN_CACHE = "translation_flatten_cache"
 LOCALE_EN = "en"
 
 
-def recursive_flatten(prefix: Any, data: Dict) -> Dict[str, Any]:
+def recursive_flatten(prefix: Any, data: dict[str, Any]) -> dict[str, Any]:
     """Return a flattened representation of dict data."""
     output = {}
     for key, value in data.items():
@@ -38,7 +36,7 @@ def recursive_flatten(prefix: Any, data: Dict) -> Dict[str, Any]:
 @callback
 def component_translation_path(
     component: str, language: str, integration: Integration
-) -> Optional[str]:
+) -> str | None:
     """Return the translation json file location for a component.
 
     For component:
@@ -50,7 +48,7 @@ def component_translation_path(
     If component is just a single file, will return None.
     """
     parts = component.split(".")
-    domain = parts[-1]
+    domain = parts[0]
     is_platform = len(parts) == 2
 
     # If it's a component that is just one file, we don't support translations
@@ -59,7 +57,7 @@ def component_translation_path(
         return None
 
     if is_platform:
-        filename = f"{parts[0]}.{language}.json"
+        filename = f"{parts[1]}.{language}.json"
     else:
         filename = f"{language}.json"
 
@@ -69,8 +67,8 @@ def component_translation_path(
 
 
 def load_translations_files(
-    translation_files: Dict[str, str]
-) -> Dict[str, Dict[str, Any]]:
+    translation_files: dict[str, str],
+) -> dict[str, dict[str, Any]]:
     """Load and parse translation.json files."""
     loaded = {}
     for component, translation_file in translation_files.items():
@@ -90,18 +88,15 @@ def load_translations_files(
 
 
 def _merge_resources(
-    translation_strings: Dict[str, Dict[str, Any]],
-    components: Set[str],
+    translation_strings: dict[str, dict[str, Any]],
+    components: set[str],
     category: str,
-) -> Dict[str, Dict[str, Any]]:
+) -> dict[str, dict[str, Any]]:
     """Build and merge the resources response for the given components and platforms."""
     # Build response
-    resources: Dict[str, Dict[str, Any]] = {}
+    resources: dict[str, dict[str, Any]] = {}
     for component in components:
-        if "." not in component:
-            domain = component
-        else:
-            domain = component.split(".", 1)[0]
+        domain = component.rpartition(".")[-1]
 
         domain_resources = resources.setdefault(domain, {})
 
@@ -122,7 +117,10 @@ def _merge_resources(
             domain_resources.update(new_value)
         else:
             _LOGGER.error(
-                "An integration providing translations for %s provided invalid data: %s",
+                (
+                    "An integration providing translations for %s provided invalid"
+                    " data: %s"
+                ),
                 domain,
                 new_value,
             )
@@ -130,11 +128,11 @@ def _merge_resources(
     return resources
 
 
-def _build_resources(
-    translation_strings: Dict[str, Dict[str, Any]],
-    components: Set[str],
+def build_resources(
+    translation_strings: dict[str, dict[str, Any]],
+    components: set[str],
     category: str,
-) -> Dict[str, Dict[str, Any]]:
+) -> dict[str, dict[str, Any] | str]:
     """Build the resources response for the given components."""
     # Build response
     return {
@@ -145,28 +143,18 @@ def _build_resources(
     }
 
 
-async def async_get_component_strings(
-    hass: HomeAssistantType, language: str, components: Set[str]
-) -> Dict[str, Any]:
+async def _async_get_component_strings(
+    hass: HomeAssistant,
+    language: str,
+    components: set[str],
+    integrations: dict[str, Integration],
+) -> dict[str, Any]:
     """Load translations."""
-    domains = list({loaded.split(".")[-1] for loaded in components})
-    integrations = dict(
-        zip(
-            domains,
-            await gather_with_concurrency(
-                MAX_LOAD_CONCURRENTLY,
-                *[async_get_integration(hass, domain) for domain in domains],
-            ),
-        )
-    )
-
-    translations: Dict[str, Any] = {}
-
+    translations: dict[str, Any] = {}
     # Determine paths of missing components/platforms
     files_to_load = {}
     for loaded in components:
-        parts = loaded.split(".")
-        domain = parts[-1]
+        domain = loaded.partition(".")[0]
         integration = integrations[domain]
 
         path = component_translation_path(loaded, language, integration)
@@ -202,29 +190,47 @@ async def async_get_component_strings(
 class _TranslationCache:
     """Cache for flattened translations."""
 
-    def __init__(self, hass: HomeAssistantType) -> None:
+    __slots__ = ("hass", "loaded", "cache", "lock")
+
+    def __init__(self, hass: HomeAssistant) -> None:
         """Initialize the cache."""
         self.hass = hass
-        self.loaded: Dict[str, Set[str]] = {}
-        self.cache: Dict[str, Dict[str, Dict[str, Any]]] = {}
+        self.loaded: dict[str, set[str]] = {}
+        self.cache: dict[str, dict[str, dict[str, dict[str, str]]]] = {}
+        self.lock = asyncio.Lock()
 
     async def async_fetch(
         self,
         language: str,
         category: str,
-        components: Set,
-    ) -> List[Dict[str, Dict[str, Any]]]:
+        components: set[str],
+    ) -> dict[str, str]:
         """Load resources into the cache."""
-        components_to_load = components - self.loaded.setdefault(language, set())
+        loaded = self.loaded.setdefault(language, set())
+        if components_to_load := components - loaded:
+            # Translations are never unloaded so if there are no components to load
+            # we can skip the lock which reduces contention when multiple different
+            # translations categories are being fetched at the same time which is
+            # common from the frontend.
+            async with self.lock:
+                # Check components to load again, as another task might have loaded
+                # them while we were waiting for the lock.
+                if components_to_load := components - loaded:
+                    await self._async_load(language, components_to_load)
 
-        if components_to_load:
-            await self._async_load(language, components_to_load)
+        category_cache = self.cache.get(language, {}).get(category, {})
+        # If only one component was requested, return it directly
+        # to avoid merging the dictionaries and keeping additional
+        # copies of the same data in memory.
+        if len(components) == 1 and (component := next(iter(components))):
+            return category_cache.get(component, {})
 
-        cached = self.cache.get(language, {})
+        result: dict[str, str] = {}
+        for component in components.intersection(category_cache):
+            result.update(category_cache[component])
+        return result
 
-        return [cached.get(component, {}).get(category, {}) for component in components]
-
-    async def _async_load(self, language: str, components: Set) -> None:
+    async def _async_load(self, language: str, components: set[str]) -> None:
         """Populate the cache for a given set of components."""
         _LOGGER.debug(
             "Cache miss for %s: %s",
@@ -233,81 +239,134 @@ class _TranslationCache:
         )
         # Fetch the English resources, as a fallback for missing keys
         languages = [LOCALE_EN] if language == LOCALE_EN else [LOCALE_EN, language]
+
+        integrations: dict[str, Integration] = {}
+        domains = list({loaded.partition(".")[0] for loaded in components})
+        ints_or_excs = await async_get_integrations(self.hass, domains)
+        for domain, int_or_exc in ints_or_excs.items():
+            if isinstance(int_or_exc, Exception):
+                raise int_or_exc
+            integrations[domain] = int_or_exc
+
         for translation_strings in await asyncio.gather(
-            *[
-                async_get_component_strings(self.hass, lang, components)
+            *(
+                _async_get_component_strings(self.hass, lang, components, integrations)
                 for lang in languages
-            ]
+            )
         ):
             self._build_category_cache(language, components, translation_strings)
 
         self.loaded[language].update(components)
 
+    def _validate_placeholders(
+        self,
+        language: str,
+        updated_resources: dict[str, Any],
+        cached_resources: dict[str, Any] | None = None,
+    ) -> dict[str, Any]:
+        """Validate if updated resources have same placeholders as cached resources."""
+        if cached_resources is None:
+            return updated_resources
+
+        mismatches: set[str] = set()
+
+        for key, value in updated_resources.items():
+            if key not in cached_resources:
+                continue
+            tuples = list(string.Formatter().parse(value))
+            updated_placeholders = {tup[1] for tup in tuples if tup[1] is not None}
+
+            tuples = list(string.Formatter().parse(cached_resources[key]))
+            cached_placeholders = {tup[1] for tup in tuples if tup[1] is not None}
+            if updated_placeholders != cached_placeholders:
+                _LOGGER.error(
+                    (
+                        "Validation of translation placeholders for localized (%s) string "
+                        "%s failed"
+                    ),
+                    language,
+                    key,
+                )
+                mismatches.add(key)
+
+        for mismatch in mismatches:
+            del updated_resources[mismatch]
+
+        return updated_resources
+
     @callback
     def _build_category_cache(
         self,
         language: str,
-        components: Set,
-        translation_strings: Dict[str, Dict[str, Any]],
+        components: set[str],
+        translation_strings: dict[str, dict[str, Any]],
     ) -> None:
         """Extract resources into the cache."""
+        resource: dict[str, Any] | str
         cached = self.cache.setdefault(language, {})
-        categories: Set[str] = set()
+        categories: set[str] = set()
         for resource in translation_strings.values():
             categories.update(resource)
 
         for category in categories:
-            resource_func = (
-                _merge_resources if category == "state" else _build_resources
-            )
-            new_resources = resource_func(translation_strings, components, category)
+            new_resources: Mapping[str, dict[str, Any] | str]
+
+            if category in ("state", "entity_component"):
+                new_resources = _merge_resources(
+                    translation_strings, components, category
+                )
+            else:
+                new_resources = build_resources(
+                    translation_strings, components, category
+                )
+
+            category_cache = cached.setdefault(category, {})
 
             for component, resource in new_resources.items():
-                category_cache: Dict[str, Any] = cached.setdefault(
-                    component, {}
-                ).setdefault(category, {})
+                component_cache = category_cache.setdefault(component, {})
 
                 if isinstance(resource, dict):
-                    category_cache.update(
-                        recursive_flatten(
-                            f"component.{component}.{category}.",
-                            resource,
-                        )
+                    resources_flatten = recursive_flatten(
+                        f"component.{component}.{category}.",
+                        resource,
                     )
+                    resources_flatten = self._validate_placeholders(
+                        language, resources_flatten, component_cache
+                    )
+                    component_cache.update(resources_flatten)
                 else:
-                    category_cache[f"component.{component}.{category}"] = resource
+                    component_cache[f"component.{component}.{category}"] = resource
 
 
 @bind_hass
 async def async_get_translations(
-    hass: HomeAssistantType,
+    hass: HomeAssistant,
     language: str,
     category: str,
-    integration: Optional[str] = None,
-    config_flow: Optional[bool] = None,
-) -> Dict[str, Any]:
+    integrations: Iterable[str] | None = None,
+    config_flow: bool | None = None,
+) -> dict[str, str]:
     """Return all backend translations.
 
     If integration specified, load it for that one.
-    Otherwise default to loaded intgrations combined with config flow
+    Otherwise default to loaded integrations combined with config flow
     integrations if config_flow is true.
     """
-    lock = hass.data.setdefault(TRANSLATION_LOAD_LOCK, asyncio.Lock())
-
-    if integration is not None:
-        components = {integration}
+    if integrations is not None:
+        components = set(integrations)
     elif config_flow:
         components = (await async_get_config_flows(hass)) - hass.config.components
-    elif category == "state":
-        components = set(hass.config.components)
+    elif category in ("state", "entity_component", "services"):
+        components = hass.config.components
     else:
         # Only 'state' supports merging, so remove platforms from selection
         components = {
             component for component in hass.config.components if "." not in component
         }
 
-    async with lock:
-        cache = hass.data.setdefault(TRANSLATION_FLATTEN_CACHE, _TranslationCache(hass))
-        cached = await cache.async_fetch(language, category, components)
+    if TRANSLATION_FLATTEN_CACHE in hass.data:
+        cache: _TranslationCache = hass.data[TRANSLATION_FLATTEN_CACHE]
+    else:
+        cache = hass.data[TRANSLATION_FLATTEN_CACHE] = _TranslationCache(hass)
 
-    return dict(ChainMap(*cached))
+    return await cache.async_fetch(language, category, components)
