@@ -7,7 +7,7 @@ import logging
 import os
 from typing import Any, cast
 
-import pkg_resources
+from packaging.requirements import Requirement
 
 from .core import HomeAssistant, callback
 from .exceptions import HomeAssistantError
@@ -15,7 +15,8 @@ from .helpers.typing import UNDEFINED, UndefinedType
 from .loader import Integration, IntegrationNotFound, async_get_integration
 from .util import package as pkg_util
 
-PIP_TIMEOUT = 60  # The default is too low when the internet connection is satellite or high latency
+# The default is too low when the internet connection is satellite or high latency
+PIP_TIMEOUT = 60
 MAX_INSTALL_FAILURES = 3
 DATA_REQUIREMENTS_MANAGER = "requirements_manager"
 CONSTRAINT_FILE = "package_constraints.txt"
@@ -62,6 +63,13 @@ async def async_process_requirements(
     await _async_get_manager(hass).async_process_requirements(name, requirements)
 
 
+async def async_load_installed_versions(
+    hass: HomeAssistant, requirements: set[str]
+) -> None:
+    """Load the installed version of requirements."""
+    await _async_get_manager(hass).async_load_installed_versions(requirements)
+
+
 @callback
 def _async_get_manager(hass: HomeAssistant) -> RequirementsManager:
     """Get the requirements manager."""
@@ -84,11 +92,8 @@ def pip_kwargs(config_dir: str | None) -> dict[str, Any]:
     is_docker = pkg_util.is_docker_env()
     kwargs = {
         "constraints": os.path.join(os.path.dirname(__file__), CONSTRAINT_FILE),
-        "no_cache_dir": is_docker,
         "timeout": PIP_TIMEOUT,
     }
-    if "WHEELS_LINKS" in os.environ:
-        kwargs["find_links"] = os.environ["WHEELS_LINKS"]
     if not (config_dir is None or pkg_util.is_virtual_env()) and not is_docker:
         kwargs["target"] = os.path.join(config_dir, "deps")
     return kwargs
@@ -132,7 +137,7 @@ class RequirementsManager:
     async def async_get_integration_with_requirements(
         self, domain: str, done: set[str] | None = None
     ) -> Integration:
-        """Get an integration with all requirements installed, including the dependencies.
+        """Get an integration with all requirements installed, including dependencies.
 
         This can raise IntegrationNotFound if manifest or integration
         is invalid, RequirementNotFound if there was some type of
@@ -231,8 +236,7 @@ class RequirementsManager:
             skipped_requirements = [
                 req
                 for req in requirements
-                if pkg_resources.Requirement.parse(req).project_name
-                in self.hass.config.skip_pip_packages
+                if Requirement(req).name in self.hass.config.skip_pip_packages
             ]
 
             for req in skipped_requirements:
@@ -257,11 +261,18 @@ class RequirementsManager:
     def _raise_for_failed_requirements(
         self, integration: str, missing: list[str]
     ) -> None:
-        """Raise RequirementsNotFound so we do not keep trying requirements that have already failed."""
+        """Raise for failed installing integration requirements.
+
+        Raise RequirementsNotFound so we do not keep trying requirements
+        that have already failed.
+        """
         for req in missing:
             if req in self.install_failure_history:
                 _LOGGER.info(
-                    "Multiple attempts to install %s failed, install will be retried after next configuration check or restart",
+                    (
+                        "Multiple attempts to install %s failed, install will be"
+                        " retried after next configuration check or restart"
+                    ),
                     req,
                 )
                 raise RequirementsNotFound(integration, [req])
@@ -280,3 +291,15 @@ class RequirementsManager:
         self.install_failure_history |= failures
         if failures:
             raise RequirementsNotFound(name, list(failures))
+
+    async def async_load_installed_versions(
+        self,
+        requirements: set[str],
+    ) -> None:
+        """Load the installed version of requirements."""
+        if not (requirements_to_check := requirements - self.is_installed_cache):
+            return
+
+        self.is_installed_cache |= await self.hass.async_add_executor_job(
+            pkg_util.get_installed_versions, requirements_to_check
+        )
