@@ -1,12 +1,16 @@
 """Utilities to help with aiohttp."""
+from __future__ import annotations
+
+from http import HTTPStatus
 import io
-import json
-from typing import Any, Dict, Optional
+from typing import Any
 from urllib.parse import parse_qsl
 
+from aiohttp import payload, web
+from aiohttp.typedefs import JSONDecoder
 from multidict import CIMultiDict, MultiDict
 
-from homeassistant.const import HTTP_OK
+from .json import json_loads
 
 
 class MockStreamReader:
@@ -26,16 +30,16 @@ class MockStreamReader:
 class MockRequest:
     """Mock an aiohttp request."""
 
-    mock_source: Optional[str] = None
+    mock_source: str | None = None
 
     def __init__(
         self,
         content: bytes,
         mock_source: str,
         method: str = "GET",
-        status: int = HTTP_OK,
-        headers: Optional[Dict[str, str]] = None,
-        query_string: Optional[str] = None,
+        status: int = HTTPStatus.OK,
+        headers: dict[str, str] | None = None,
+        query_string: str | None = None,
         url: str = "",
     ) -> None:
         """Initialize a request."""
@@ -62,9 +66,14 @@ class MockRequest:
         """Return the body as text."""
         return MockStreamReader(self._content)
 
-    async def json(self) -> Any:
+    @property
+    def body_exists(self) -> bool:
+        """Return True if request has HTTP BODY, False otherwise."""
+        return bool(self._text)
+
+    async def json(self, loads: JSONDecoder = json_loads) -> Any:
         """Return the body as JSON."""
-        return json.loads(self._text)
+        return loads(self._text)
 
     async def post(self) -> MultiDict[str]:
         """Return POST parameters."""
@@ -73,3 +82,22 @@ class MockRequest:
     async def text(self) -> str:
         """Return the body as text."""
         return self._text
+
+
+def serialize_response(response: web.Response) -> dict[str, Any]:
+    """Serialize an aiohttp response to a dictionary."""
+    if (body := response.body) is None:
+        body_decoded = None
+    elif isinstance(body, payload.StringPayload):
+        # pylint: disable-next=protected-access
+        body_decoded = body._value.decode(body.encoding)
+    elif isinstance(body, bytes):
+        body_decoded = body.decode(response.charset or "utf-8")
+    else:
+        raise TypeError("Unknown payload encoding")
+
+    return {
+        "status": response.status,
+        "body": body_decoded,
+        "headers": dict(response.headers),
+    }
