@@ -1,15 +1,18 @@
 """Asyncio utilities."""
 from __future__ import annotations
 
-from asyncio import Semaphore, coroutines, ensure_future, gather, get_running_loop
+from asyncio import Future, Semaphore, gather, get_running_loop
 from asyncio.events import AbstractEventLoop
-from collections.abc import Awaitable, Callable, Coroutine
+from collections.abc import Awaitable, Callable
 import concurrent.futures
+from contextlib import suppress
 import functools
 import logging
 import threading
 from traceback import extract_stack
-from typing import Any, ParamSpec, TypeVar
+from typing import Any, ParamSpec, TypeVar, TypeVarTuple
+
+from homeassistant.exceptions import HomeAssistantError
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -18,33 +21,16 @@ _SHUTDOWN_RUN_CALLBACK_THREADSAFE = "_shutdown_run_callback_threadsafe"
 _T = TypeVar("_T")
 _R = TypeVar("_R")
 _P = ParamSpec("_P")
+_Ts = TypeVarTuple("_Ts")
 
 
-def fire_coroutine_threadsafe(
-    coro: Coroutine[Any, Any, Any], loop: AbstractEventLoop
-) -> None:
-    """Submit a coroutine object to a given event loop.
-
-    This method does not provide a way to retrieve the result and
-    is intended for fire-and-forget use. This reduces the
-    work involved to fire the function on the loop.
-    """
-    ident = loop.__dict__.get("_thread_ident")
-    if ident is not None and ident == threading.get_ident():
-        raise RuntimeError("Cannot be called from within the event loop")
-
-    if not coroutines.iscoroutine(coro):
-        raise TypeError(f"A coroutine object is required: {coro}")
-
-    def callback() -> None:
-        """Handle the firing of a coroutine."""
-        ensure_future(coro, loop=loop)
-
-    loop.call_soon_threadsafe(callback)
+def cancelling(task: Future[Any]) -> bool:
+    """Return True if task is cancelling."""
+    return bool((cancelling_ := getattr(task, "cancelling", None)) and cancelling_())
 
 
 def run_callback_threadsafe(
-    loop: AbstractEventLoop, callback: Callable[..., _T], *args: Any
+    loop: AbstractEventLoop, callback: Callable[[*_Ts], _T], *args: *_Ts
 ) -> concurrent.futures.Future[_T]:
     """Submit a callback object to a given event loop.
 
@@ -100,6 +86,14 @@ def check_loop(
     The default advisory message is 'Use `await hass.async_add_executor_job()'
     Set `advise_msg` to an alternate message if the solution differs.
     """
+    # pylint: disable=import-outside-toplevel
+    from homeassistant.core import HomeAssistant, async_get_hass
+    from homeassistant.helpers.frame import (
+        MissingIntegrationFrame,
+        get_integration_frame,
+    )
+    from homeassistant.loader import async_suggest_report_issue
+
     try:
         get_running_loop()
         in_loop = True
@@ -122,54 +116,47 @@ def check_loop(
         # stack[-1] is us, stack[-2] is protected_loop_func, stack[-3] is the offender
         return
 
-    for frame in reversed(stack):
-        for path in ("custom_components/", "homeassistant/components/"):
-            try:
-                index = frame.filename.index(path)
-                found_frame = frame
-                break
-            except ValueError:
-                continue
+    try:
+        integration_frame = get_integration_frame()
+    except MissingIntegrationFrame:
+        # Did not source from integration? Hard error.
+        if found_frame is None:
+            raise RuntimeError(  # noqa: TRY200
+                f"Detected blocking call to {func.__name__} inside the event loop. "
+                f"{advise_msg or 'Use `await hass.async_add_executor_job()`'}; "
+                "This is causing stability issues. Please create a bug report at "
+                f"https://github.com/home-assistant/core/issues?q=is%3Aopen+is%3Aissue"
+            )
 
-        if found_frame is not None:
-            break
-
-    # Did not source from integration? Hard error.
-    if found_frame is None:
-        raise RuntimeError(
-            f"Detected blocking call to {func.__name__} inside the event loop. "
-            f"{advise_msg or 'Use `await hass.async_add_executor_job()`'}; "
-            "This is causing stability issues. Please report issue"
-        )
-
-    start = index + len(path)
-    end = found_frame.filename.index("/", start)
-
-    integration = found_frame.filename[start:end]
-
-    if path == "custom_components/":
-        extra = " to the custom integration author"
-    else:
-        extra = ""
+    hass: HomeAssistant | None = None
+    with suppress(HomeAssistantError):
+        hass = async_get_hass()
+    report_issue = async_suggest_report_issue(
+        hass,
+        integration_domain=integration_frame.integration,
+        module=integration_frame.module,
+    )
 
+    found_frame = integration_frame.frame
     _LOGGER.warning(
         (
-            "Detected blocking call to %s inside the event loop. This is causing"
-            " stability issues. Please report issue%s for %s doing blocking calls at"
-            " %s, line %s: %s"
+            "Detected blocking call to %s inside the event loop by %sintegration '%s' "
+            "at %s, line %s: %s, please %s"
         ),
         func.__name__,
-        extra,
-        integration,
-        found_frame.filename[index:],
+        "custom " if integration_frame.custom_integration else "",
+        integration_frame.integration,
+        integration_frame.relative_filename,
         found_frame.lineno,
         (found_frame.line or "?").strip(),
+        report_issue,
     )
+
     if strict:
         raise RuntimeError(
             "Blocking calls must be done in the executor or a separate thread;"
             f" {advise_msg or 'Use `await hass.async_add_executor_job()`'}; at"
-            f" {found_frame.filename[index:]}, line {found_frame.lineno}:"
+            f" {integration_frame.relative_filename}, line {found_frame.lineno}:"
             f" {(found_frame.line or '?').strip()}"
         )
 
@@ -185,7 +172,7 @@ def protect_loop(func: Callable[_P, _R], strict: bool = True) -> Callable[_P, _R
     return protected_loop_func
 
 
-async def gather_with_concurrency(
+async def gather_with_limited_concurrency(
     limit: int, *tasks: Any, return_exceptions: bool = False
 ) -> Any:
     """Wrap asyncio.gather to limit the number of concurrent tasks.
