@@ -4,7 +4,7 @@ from typing import Any
 
 import yaml
 
-from .objects import Input, NodeListClass
+from .objects import Input, NodeDictClass, NodeListClass, NodeStrClass
 
 # mypy: allow-untyped-calls, no-warn-return-any
 
@@ -12,10 +12,12 @@ from .objects import Input, NodeListClass
 try:
     from yaml import CSafeDumper as FastestAvailableSafeDumper
 except ImportError:
-    from yaml import SafeDumper as FastestAvailableSafeDumper  # type: ignore[misc]
+    from yaml import (  # type: ignore[assignment]
+        SafeDumper as FastestAvailableSafeDumper,
+    )
 
 
-def dump(_dict: dict) -> str:
+def dump(_dict: dict | list) -> str:
     """Dump YAML to a string and remove null."""
     return yaml.dump(
         _dict,
@@ -72,11 +74,21 @@ add_representer(
     lambda dumper, value: represent_odict(dumper, "tag:yaml.org,2002:map", value),
 )
 
+add_representer(
+    NodeDictClass,
+    lambda dumper, value: represent_odict(dumper, "tag:yaml.org,2002:map", value),
+)
+
 add_representer(
     NodeListClass,
     lambda dumper, value: dumper.represent_sequence("tag:yaml.org,2002:seq", value),
 )
 
+add_representer(
+    NodeStrClass,
+    lambda dumper, value: dumper.represent_scalar("tag:yaml.org,2002:str", str(value)),
+)
+
 add_representer(
     Input,
     lambda dumper, value: dumper.represent_scalar("!input", value.name),
